package MainLibrary;

import java.util.List;
import com.threed.jpct.SimpleVector;


public abstract class AbstractCollisionDomainStrategy {
	
	protected AbstractDomain domain;
	
	public abstract void checkExternalCollisionBound(AbstractDomain domainToCheck, NanoObj objsToCheck);
	public abstract void checkInternalCollisionBound(List<NanoObj> objsToCheck);
	
	//FIXME Under test... Higher Performance for the CheckCollision?
	public abstract void checkInternalCollisionBound(NanoObj objsToCheck);
	
	public abstract void relocateInnerObjs(NanoObj toRelocate, AbstractDomain domain);
	public abstract void relocateOuterObjs(NanoObj toRelocate, AbstractDomain domain);
	public abstract double getExternalCollisionTime(NanoObj collisionObj, AbstractDomain d);
	public abstract double getInternalCollisionTime(NanoObj collisionObj, AbstractDomain d);	
	public abstract boolean isAnInternalCollisionBound(NanoObj n);	
	public abstract boolean isCollidingFromOutside(AbstractDomain domainToCheck, NanoObj objToCheck);
	
	public int type;
	public float inelasticCoefficient=1;
	public float inelasticCoefficientLongit=1;
	
	
	
	public AbstractCollisionDomainStrategy(int type){
		this.type=type;		
		inelasticCoefficient=Configurator2.m.domainStrategyConfig.get(type+"-e");
		inelasticCoefficientLongit=Configurator2.m.domainStrategyConfig.get(type+"-elongit");
	}	
	
	public void setDomain(AbstractDomain d){
		domain=d;		
	}
	
	public float getAngle(SimpleVector v1, SimpleVector v2){
		return v1.calcAngle(v2);
	}	
	
	public void switchDomain(NanoObj toRelocate, AbstractDomain domainToRelocate){
		toRelocate.switchDomain(domainToRelocate);
		toRelocate.activateOnDomain(domainToRelocate);
		//addToExceptionList(toRelocate);  // FIXME not necessary. Now the exceptionList is in the AbstractDomain
		domainToRelocate.addToExceptionList(toRelocate);
		toRelocate.switchMotionStrategy(LinearMotion.getInstance());
	}
	
	public float[] checkNextStepCollision(NanoObj n){
		// ===================================================
		// Assuming a Linear Motion for this transitory state!		
		// ===================================================
		
		float[] nanoCenter=n.getCenter();			
		float[] component=n.getVelocityComponent();
		float[] nextPosition=new float[3];
		
		nextPosition[0]=(nanoCenter[0]+component[0]);
		nextPosition[1]=(nanoCenter[1]+component[1]);
		nextPosition[2]=(nanoCenter[2]+component[2]);

		return nextPosition;		
	}	
	
	// 17/03/2014
	public float[] getDistanceOnAxis(AbstractDomain domainToCheck, NanoObj objToCheck){				
		
		SimpleVector nanoObjRelativeCenter=getRelativeCenter(domainToCheck, objToCheck);
		
		SimpleVector d=domainToCheck.getDirectionP();
		SimpleVector n=domainToCheck.getDirectionN();
		SimpleVector o=domainToCheck.getDirectionO();			
		
		float distanceN=nanoObjRelativeCenter.calcDot(n);
		float distanceO=nanoObjRelativeCenter.calcDot(o);		
		float distanceP=nanoObjRelativeCenter.calcDot(d);			
		
		distanceN=Math.abs(distanceN);
		distanceO=Math.abs(distanceO);
		distanceP=Math.abs(distanceP);
		
		
		return new float[]{distanceP, distanceN, distanceO};
	}
	
	// 30/04/2015 - for SHELL CYLINDRIC DOMAIN!
	public float[] getSignedDistanceOnAxis(AbstractDomain domainToCheck, NanoObj objToCheck){				
		
		SimpleVector nanoObjRelativeCenter=getRelativeCenter(domainToCheck, objToCheck);
		
		SimpleVector d=domainToCheck.getDirectionP();
		SimpleVector n=domainToCheck.getDirectionN();
		SimpleVector o=domainToCheck.getDirectionO();			
		
		float distanceN=nanoObjRelativeCenter.calcDot(n);
		float distanceO=nanoObjRelativeCenter.calcDot(o);		
		float distanceP=nanoObjRelativeCenter.calcDot(d);			
		
		/*
		distanceN=Math.abs(distanceN);
		distanceO=Math.abs(distanceO);
		distanceP=Math.abs(distanceP);
		*/
		
		return new float[]{distanceP, distanceN, distanceO};
	}	
	
	
	// For GRID
	public float[] getDistanceOnAxis(AbstractDomain referenceDomain, AbstractDomain domainToCheck){
		
		//FIXME Under test. Useful only for the CylindricDomain case?
		
		SimpleVector nanoObjRelativeCenter=getRelativeCenter(referenceDomain, domainToCheck);
		float nanoObjRelativeModule=nanoObjRelativeCenter.length();
		
		SimpleVector d=referenceDomain.getDirectionP();

		double angleP=getAngle(d,nanoObjRelativeCenter);				

		float distanceP= (float) Math.abs((nanoObjRelativeModule)*Math.cos(angleP)); //era ok
		
		double angleN=(Math.PI/2)-angleP;

		float distanceN= (float) Math.abs((nanoObjRelativeModule)*Math.cos(angleN)); //era ok
	
		return new float[]{distanceP,distanceN};		
	}		
	
	public SimpleVector getRelativeCenter(AbstractDomain domainToCheck, NanoObj objToCheck){
		
		float[] domainCenter=domainToCheck.getAbsCenter();		
		float[] nanoCenter=objToCheck.getCenter();				
	
		float x=nanoCenter[0]-domainCenter[0];
		float y=nanoCenter[1]-domainCenter[1];
		float z=nanoCenter[2]-domainCenter[2];

		SimpleVector nanoObjRelativeCenter=new SimpleVector(x,y,z);
		return nanoObjRelativeCenter;
	}
	
	public SimpleVector getRelativeCenter(AbstractDomain domainToCheck1, AbstractDomain domainToCheck2){
		
		float[] domainCenter=domainToCheck1.getAbsCenter();		
		float[] nanoCenter=domainToCheck2.getAbsCenter();				
	
		float x=nanoCenter[0]-domainCenter[0];
		float y=nanoCenter[1]-domainCenter[1];
		float z=nanoCenter[2]-domainCenter[2];

		SimpleVector nanoObjRelativeCenter=new SimpleVector(x,y,z);
		return nanoObjRelativeCenter;
	}	
}
