package MainLibrary;

import java.awt.Color;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.Manager;
import MainLibrary.NanoObj;
import MainLibrary.NodeObj;
import Octree.OctreeNode;

import com.threed.jpct.Object3D;
import com.threed.jpct.SimpleVector;

/** TODO IMPORTANT!!! 
 * Every time you add a new AbstractDomain specialization, you have to define the appropriate
 * addWrapper3DDomain() method inside Engine3D, that instruct the engine 3d to draw the object3D for that specific domain
 */

public abstract class AbstractEngine3D implements Runnable{

	public Manager m;
	public float[] cameraCenter=new float[3];
	public SimpleVector cameraLookAt=new SimpleVector();
	public int[] lights=new int[3];	
	public int[] frameSize=new int[]{320,200};
	public String title="3D View";
	

	public abstract void run();  
	
	public abstract void setManager(Manager m);
	
	public abstract void addWrapper3D(NanoObj p);

	public abstract void addWrapper3D(NodeObj p);

	public abstract void addWrapper3D(CarrierObj p);
	
	public abstract void removeWrapper3D (NanoObj obj);

	public abstract void changeWrapper3D(NanoObj obj);

	public abstract void changeWrapper3D(NodeObj obj);

	public abstract void changeWrapper3D(CarrierObj obj);
	
	public abstract void removeWrapper3D(AbstractDomain d);

	public abstract void addWrapper3DDomain(AbstractDomain d);
	
	public abstract void addWrapper3DRootCubicDomain(float[] center, float halfSide);
	
	public abstract void changeWrapper3DDomain(AbstractDomain d);

	public abstract void changeObjColor(NanoObj obj ,Color color);
	
	public abstract void changeDomainColor(AbstractDomain obj, Color color);
	
	public abstract void createPoint(float x,float y, float z, Color c);
	
	public abstract Object3D getWrapperDomain3D(AbstractDomain d);
	
	
	public AbstractEngine3D(){
		
	}

	public AbstractEngine3D(Manager m){
		this.m=m;

		cameraCenter=m.configWrapper.camera;
		cameraLookAt=m.configWrapper.lookAt;
		lights=m.configWrapper.lights;			
		frameSize=m.configWrapper.window;
	}

	public abstract void addWrapper3DOctree(OctreeNode octreeNode);
	
}

