package MainLibrary;

import java.awt.Color;
import Octree.Octree;


public abstract class CarrierObj extends NanoObj{

	private NodeObj parent=null;	
	public abstract LigandObserver getMyObserver();
	public abstract void specializedDetach();
	
	public int packetType=0;		// TPCLike Simulations
	

	public CarrierObj(AbstractDomain domain, NodeObj n,float x, float y, float z, float mass, float r,int TTL,float zoom, int numberType) {	
		super(domain,x, y, z, mass, r,TTL, numberType);
		LigandObserver o=getMyObserver();
		attachObs(o);
		o.obsLigand(this);
		// The Generic Carrier Observer is always included!
		attachObs(GenericLigandObserver.getInstance());
		GenericLigandObserver.getInstance().obsLigand(this);
		setParent(n);
		getParent().getInnerDomain().attachNanoObj(this);
		zoom3D=zoom;
		color=Color.orange;
	}
	
	// This constructor allows the free carriers creation outside the Nodes!
	public CarrierObj(AbstractDomain domain, float x, float y, float z, float mass, float r, int TTL, float zoom, int numberType) {	
		super(domain, x, y, z, mass, r, TTL, numberType);
		LigandObserver o=getMyObserver();
		attachObs(o);
		o.obsLigand(this);
		// The Generic Carrier Observer is always included!
		attachObs(GenericLigandObserver.getInstance());
		GenericLigandObserver.getInstance().obsLigand(this);		
		
		zoom3D=zoom;
		color=Color.orange;	
		
		//FIXME TO BE TESTED!		
		domain.attachNanoObj(this);		
		/*
		if(getDomain().isInside(this) && domain.isFreeSpace(this)){
			disposeOnMedium();
		}
		*/
		
		// FIXME  isFreeSpace check is very important! You can avoid it only if you do not perform the CheckCollision between NanoObj-NanoObj in order to increase the performance
		disposeOnMedium();
		/*
		if (domain.isFreeSpace(this)){
			disposeOnMedium();
		}else{
			kill();			
		}
		*/		
	}
	
	
	//FIXME Ca++  -  Added for the Calcium scenario!!!
	public CarrierObj(AbstractDomain domain, float x, float y, float z, float mass, float r, int TTL, float zoom, int numberType, boolean isDisposed) {	
		super(domain, x, y, z, mass, r, TTL, numberType);
		LigandObserver o=getMyObserver();
		attachObs(o);
		o.obsLigand(this);
		// The Generic Carrier Observer is always included!
		attachObs(GenericLigandObserver.getInstance());
		GenericLigandObserver.getInstance().obsLigand(this);		
		
		zoom3D=zoom;
		color=Color.orange;	
		
		
		setDomain(domain);			
		setLifeManager(domain.getLivingObjObserver());
		notifyToLifeManager();
		
		initializeWrapper3d();	
		
		synchronized(domain.carrierList){
			domain.carrierList.add(this);
		}
	}
	
	public void attachOnDomain(){
		getDomain().attachNanoObj(this);
	}
	
	public void notifyToLifeManager(){
		getLifeManager().addCarrier(this);
	}
	
	public void specializedFinalize() {
		try {
			if(getDomain()!=null) {
				getDomain().detachNanoObj(this); // FIXME To be TESTED with more attention! The remove from domain is well done??
			}
			//getMyObserver().removeLigand(this);
			for (int i=0;i<obsList.size();i++){
				obsList.get(i).removeLigand(this);
			}
		} catch (Exception e) {
			System.out.println("WARNING - CARRIER SPECIALIZED FINALIZE ERROR!");			
		}
	}
	
	public void setParent(NodeObj n){
		parent=n;
	}

	public NodeObj getParent(){
		return parent;
	}	
		
	public void specializedDisposeOnDomain(){
		getDomain().activatePropagationObj(this);
		initializeWrapper3d();	
	}
	
	// It is called if the carrier pass-through the node's membrane!
	public void switchDomain(AbstractDomain newDomain){
		
		if(newDomain.ID.matches("SHELL SPHERICAL DOMAIN")){
			
			setColor(Color.pink);

		}else{			
			setColor(Color.black);
		}
		
		removeFromActualDomain();
		try {						
			newDomain.attachNanoObj(this);
		} catch (NullPointerException e) {
			// Handles the passage of the nanoobj from two distinct ROOT CUBE DOMAIN
		}
	}
	
	public void removeFromActualDomain(){
		getDomain().detachNanoObj(this);	
	}

	public void estabilishContact(NanoObj n){
		n.handle(this);
	}
	
	@Override
	public void estabilishContact(AbstractDomain d) {
		d.handleBoundCollision(this);		
	}
	
	public void handle(CarrierObj c){
		getDomain().getMobilityModel().relocateObjs(c,this);		
	}
	
	public void handle(NodeObj n){
		n.checkIncomingCarrier(this);
	}
	
	public void handle(NanoObj n){
		getDomain().getMobilityModel().relocateObjs(n,this);	
		
		// FIXME ONLY FOR DEBUG!!
		//getManager().carriersCollision++;		
	}	
	
	public void detach(Receptor r){
		r.detachCarrier();
		specializedDetach();
	}
	
	public void exec() {
		
	}
	
	public void standardBehavior() {
		
	}	
	
	public void activateOnDomain(AbstractDomain domain){
		domain.activatePropagationObj(this);
	}
	
	public void initializeWrapper3d(){
		if(getManager().is3DEnable==true)
			getManager().engine.addWrapper3D(this);
	}
	
	public void reassignLigandObserver(){
		getMyObserver().obsLigand(this);		
	}
	
	// Octree
	public void attachOnNodeTree(Octree tree){
		tree.firstInsertOnNode(this);
	}
}
