package MainLibrary;

import java.awt.Color;
import java.util.List;

public class InnerSphericalStrategy extends AbstractCollisionDomainStrategy{
	
	public InnerSphericalStrategy(int type){
		super(type);
	}
	
	public void checkInternalCollisionBound(List<NanoObj> objsToCheck) {		
		boolean isFinish=false;
		for (int i=0;i<objsToCheck.size() && !isFinish;i++){			
			try{

				/** FIXME have to control also the collisions with the other domains that share the same parentDomain! 
				 * So it is needed also the collision control on the external shells. 
				 * In what follows it considers only the collisions with inner shell. */  
				InnerSphericalDomain domainToCheck=(InnerSphericalDomain)domain;

				// IDEA: You can evaluate the bounding sphere to perform a preliminary check collision
				if(objsToCheck.get(i).getTimeToLive()>0){
					
					if((objsToCheck.get(i).getAbsDistancePlusRadius()>=domainToCheck.getBoundingRadius())){													
						checkRealDomainBounds(objsToCheck.get(i));						
					} else {						
						isFinish=true;
					}					
				}
			}catch (Exception e){
				e.printStackTrace();
			}
		}		
	}	

	public void checkRealDomainBounds(NanoObj n) {
		float[] domainCenter=domain.getAbsCenter();		
		float domainRadius=((InnerSphericalDomain)domain).getRadius();
		float[] nanoCenter=n.getCenter();
		float nanoRadius=n.getRadius();		
		float[] velocity1=n.getVelocityComponent();
		
		float dx = (domainCenter[0]-nanoCenter[0]), dy = (domainCenter[1]-nanoCenter[1]), dz= (domainCenter[2]-nanoCenter[2]);
		float distance = (float) Math.sqrt(dx*dx+dy*dy+dz*dz);
				
		if (distance>=((domainRadius-nanoRadius))){
			
			// Unit vector in the direction of the collision
			float ax=(dx/distance), ay=(dy/distance), az=(dz/distance);
			
			// Projection of the velocities in these axes
			float va1=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az); 
			float va2=(0*ax+0*ay+0*az);
			
			// the collision should have occurred at t-dt (Actually this is also an approximation).
			float dt=1;
			
			try {
				dt =(float) ((distance-(domainRadius-nanoRadius))/Math.abs(va1-va2)); // Warning! catch the div-by-zero!!!
			} catch (Exception e) {
				System.out.println(e.getMessage());	
			}			
			addToCollisionsList(n,dt);
		}
	}	
	
	public void relocateInnerObjs(NanoObj toRelocate, AbstractDomain domain) {
		
		InnerSphericalDomain owner=(InnerSphericalDomain)domain;
		
		float[] center1=toRelocate.getCenter();
		float[] center2=owner.getAbsCenter();
		float[] velocity1=toRelocate.getVelocityComponent();
		float r1=toRelocate.getRadius();
		float r2=owner.getRadius();
		float modVelocity1=toRelocate.getVelocityModule();
		float modVelocity2=0;
		double dt=0;			
		
		if (modVelocity1==0 && modVelocity2==0){
		
			// If both velocities are zero, you may want to handle the relocation in some way...
			
		} else{
					
			float dx21 = (center2[0]-center1[0]);
			float dy21 = (center2[1]-center1[1]);
			float dz21 = (center2[2]-center1[2]);		
			
			float distance = (float) Math.sqrt(dx21*dx21+dy21*dy21+dz21*dz21);
				
			
			if (distance>((r2-r1))){	
				float modDiffVelocity=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
				dt = ((distance-(r2-r1))/modDiffVelocity);			
				if (dt>1.1) {
					dt=1;					
				}
				
				center1[0]+=-velocity1[0]*dt;
				center1[1]+=-velocity1[1]*dt;
				center1[2]+=-velocity1[2]*dt;
				toRelocate.setCenter(center1[0], center1[1], center1[2]);
				
				float fi1=toRelocate.getVelocityAngles(-velocity1[0],-velocity1[1], -velocity1[2])[0];
				float theta1=toRelocate.getVelocityAngles(-velocity1[0], -velocity1[1], -velocity1[2])[1];
				float nextVelocityModule1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
				toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);			
				
				velocity1=toRelocate.getVelocityComponent();

				center1[0]+=velocity1[0]*dt;
				center1[1]+=velocity1[1]*dt;
				center1[2]+=velocity1[2]*dt;
				toRelocate.setCenter(center1[0], center1[1], center1[2]);	
			}
		}
	}
	
	public synchronized void addToCollisionsList(NanoObj p1,float dt){
		AbstractCollision c=new DomainCollisionInternal(p1,p1.getDomain(),dt);
		
		InnerSphericalDomain sDomain=(InnerSphericalDomain)p1.getDomain();		
		
		domain.getCollisionsList().add(c);
		
		if (domain.getManager().is3DEnable) {
			p1.setColor(Color.red);			
			if (sDomain.getOwner()!=null) sDomain.getOwner().setColor(Color.red);
		}
		p1.increaseCollisionCount();
		if (sDomain.getOwner()!=null) sDomain.getOwner().increaseCollisionCount();		
	}

	@Override
	public void relocateOuterObjs(NanoObj toRelocate, AbstractDomain domain) {
		// TODO Auto-generated method stub		
	}

	@Override
	public void checkExternalCollisionBound(AbstractDomain domainToCheck, NanoObj objsToCheck) {
		// TODO Auto-generated method stub		
	}

	@Override
	public double getExternalCollisionTime(NanoObj collisionObj, AbstractDomain d) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public double getInternalCollisionTime(NanoObj collisionObj, AbstractDomain d) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public boolean isAnInternalCollisionBound(NanoObj n) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isCollidingFromOutside(AbstractDomain domainToCheck,
			NanoObj objToCheck) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void checkInternalCollisionBound(NanoObj objsToCheck) {
		// TODO Auto-generated method stub
		
	}
}
