package MainLibrary;
import java.util.*;


public class LivingObjObserver {

	public List<NanoObj> activeGenericObjList=new ArrayList<NanoObj>();
	public List<NodeObj> activeNodeList=new ArrayList<NodeObj>();
	public List<CarrierObj> activeCarrierList=new ArrayList<CarrierObj>();

	private AbstractDomain domain;
	private static int objCount=0;	
	
	private static long globalObjCount=0;	// FIXES
	

	public LivingObjObserver(AbstractDomain d) {
		setDomain(d);
	}

	public void killDeadObjs(){		

		for (int i=0;i<activeNodeList.size();i++){			
			if(activeNodeList.get(i).getTimeToLive()<=0){		    	
				NodeObj o = activeNodeList.get(i);		
				//activeNodeList.remove(i);
				o.destroy();	
				i--;
			}
		}
		
		for (int i=0;i<activeCarrierList.size();i++){			
			if(activeCarrierList.get(i).getTimeToLive()<=0){		    	
				CarrierObj o = activeCarrierList.get(i);		
				//activeNodeList.remove(i);
				o.destroy();	
				i--;
			}
		}		

		for (int i=0;i<activeGenericObjList.size();i++){			
			if(activeGenericObjList.get(i).getTimeToLive()<=0){		    	
				NanoObj o = activeGenericObjList.get(i);		
				//activeNodeList.remove(i);
				o.destroy();	
				i--;
			}
		}
	}

	public boolean isAlive(NanoObj n){
		boolean isAlive=false;
		if(n.getTimeToLive()>0) isAlive=true;
		return isAlive;
	}
	
	public synchronized void addNode(NodeObj n){
		activeNodeList.add(n);		
		increaseObjCount();		
		//n.setID(getObjCount());
		n.setID(getGlobalObjCount());
		
	}

	public synchronized void addCarrier(CarrierObj c){
		activeCarrierList.add(c);		
		increaseObjCount();
		//c.setID(getObjCount());
		c.setID(getGlobalObjCount());
		
	}	
	
	public synchronized void addGenericObj(NanoObj n){
		activeGenericObjList.add(n);		
		increaseObjCount();
		//n.setID(getObjCount());
		n.setID(getGlobalObjCount());
		
	}
	
	public synchronized void removeNode(NodeObj n){
		activeNodeList.remove(n);
		n.setLifeManager(null);		
		decreaseObjCount();		
	}

	public synchronized void removeCarrier(CarrierObj c){
		activeCarrierList.remove(c);	// FIXME MANCAVA!!! 23/04/2015
		c.setLifeManager(null);
		decreaseObjCount();
	}	
	
	public synchronized void removeGenericObj(NanoObj n){
		activeGenericObjList.remove(n);
		n.setLifeManager(null);
		decreaseObjCount();
	}	
	
	public void trigger() {	

		try{
			for(NodeObj n : activeNodeList){
				n.trigger();
			}
			
		}catch (Exception e){
			System.out.println("-- Error on LivingObjObs.trigger() - NODE! "+e.toString());		
			e.printStackTrace();
		}
		
		try{
			for(CarrierObj c : activeCarrierList){
				c.trigger();
			}
			
		}catch (Exception e){
			System.out.println("-- Error on LivingObjObs.trigger() - CARRIER! "+e.toString());
		}
		
		try{
			for(NanoObj n : activeGenericObjList){
				n.trigger();
			}
			
		}catch (Exception e){
			System.out.println("-- Error on LivingObjObs.trigger() - GENERIC! "+e.toString());
		}	

		if (domain.carrierList.isEmpty()) domain.carrierList=new ArrayList<CarrierObj>();
		if (domain.undisposedCarrierList.isEmpty()) domain.undisposedCarrierList=new ArrayList<CarrierObj>();
		if (domain.nodeList.isEmpty()) domain.nodeList=new ArrayList<NodeObj>();
		if (domain.undisposedNodeList.isEmpty()) domain.undisposedNodeList=new ArrayList<NodeObj>();
		if (domain.genericObjList.isEmpty()) domain.genericObjList=new ArrayList<NanoObj>();
		if (domain.undisposedGenericObjList.isEmpty()) domain.undisposedGenericObjList=new ArrayList<NanoObj>();
	}

	public synchronized List<NodeObj> getLivingNodes(){
		return activeNodeList;		
	}

	public synchronized List<CarrierObj> getLivingCarriers(){
		return activeCarrierList;		
	}

	public synchronized List<NanoObj> getLivingGenericObj(){
		return activeGenericObjList;		
	}

	public List<NanoObj> getAllLivingObjs(){
		List<NanoObj> allLivingObjs=new ArrayList<NanoObj>();
		if (activeNodeList!=null) allLivingObjs.addAll(activeNodeList);
		if (activeCarrierList!=null) allLivingObjs.addAll(activeCarrierList);
		if (activeGenericObjList!=null) allLivingObjs.addAll(activeGenericObjList);
		return allLivingObjs;
	}
	
	public void setDomain(AbstractDomain domain) {
		this.domain = domain;
	}

	public AbstractDomain getDomain() {
		return domain;
	}
	/**/
	public synchronized static int getObjCount() {
		return objCount;
	}
	
	public synchronized static long getGlobalObjCount() {
		return globalObjCount;
	}

	public synchronized static void increaseObjCount() {
		LivingObjObserver.objCount++;	
		LivingObjObserver.globalObjCount++;	
	}

	public synchronized static void decreaseObjCount() {
		LivingObjObserver.objCount--;
	}
	
	public void destroy(){
		Iterator<NanoObj> itND = getAllLivingObjs().iterator();
		while (itND.hasNext()) {
			NanoObj n = itND.next();
			n.destroy();
			itND.remove();
		}

		activeGenericObjList=null;
		activeNodeList=null;
		activeCarrierList=null;		
		domain=null;
		
		try {
			super.finalize();
		} catch (Throwable e) {		
			e.printStackTrace();
		}		
	}
}
