package MainLibrary;

import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import com.threed.jpct.SimpleVector;


public abstract class Manager implements Runnable{
	
	public boolean hasSaved=false;		// Store ALL Simulator Objs into a DB
	public int core = Runtime.getRuntime().availableProcessors();	
	public List<OutputStrategy> outputObjList=new ArrayList<OutputStrategy>();	
	public ArrayList<AbstractDomain> exportDomain=new ArrayList<AbstractDomain>();
	public ArrayList<AbstractDomain> activeDomain=new ArrayList<AbstractDomain>();
	public RootDomain rootDomain;	
	public ConfigWrapper configWrapper;
	//public Configurator configurator=new Configurator(this);	 // TODO If you comment this line, you have to comment createNetwork() on Manager Constructor...
	public Configurator2 configurator=Configurator2.getConfigurator(this);		
	public AbstractEngine3D engine;
	public Thread engineThreaded;		
	protected long currentTime=0;
	public boolean is3DEnable=false;	
	public boolean hasStopped=false;	
	public int totalObj=0;
	public int totalLivingObj=0;	
	public double elapsedTime=0;		
	public static long start=System.currentTimeMillis();
	public static long stop=0;	
	public static long triggerDomainTime=0;
	public static long propagationTime=0;	
	public static boolean hasToSaveSimulationStatus=false;
	public static String dbName="nano_backup.db4o";
	public static float saveTime=2.5f;	
	
	private final int maxThreads = Runtime.getRuntime().availableProcessors();				
	private ExecutorService executorThreads = Executors.newFixedThreadPool(maxThreads);	
	
	public long carriersCollision=0;
	
	//public float ed=1;
	
	public abstract void specializedExport();
	public abstract void printCurrentStep();	
	
	public Manager(MobilityModel m, OutputStrategy o, AbstractEngine3D engine, boolean useOctree){		
		initialize();
		RootDomain rootDomain=new RootDomain(this, m, new NoCollisionDomainStrategy(0), false, useOctree);
		setRootDomain(rootDomain);
		setMobilityModel(m);
		o.setManager(this);
		
		if(engine!=null){
			this.is3DEnable = true;
			this.engine = engine;
			engine.setManager(this);			
			
			engine.cameraCenter[0]=configurator.readCustomConfigurations("config.xml", "camera", "x");
			engine.cameraCenter[1]=configurator.readCustomConfigurations("config.xml", "camera", "y");
			engine.cameraCenter[2]=configurator.readCustomConfigurations("config.xml", "camera", "z");
			
			engine.cameraLookAt=new SimpleVector(configurator.readCustomConfigurations("config.xml", "camera", "lookX"),
					configurator.readCustomConfigurations("config.xml", "camera", "lookY"),
					configurator.readCustomConfigurations("config.xml", "camera", "lookZ"));
			engine.lights[0]=(int) configurator.readCustomConfigurations("config.xml", "light", "r");
			engine.lights[1]=(int) configurator.readCustomConfigurations("config.xml", "light", "g");
			engine.lights[2]=(int) configurator.readCustomConfigurations("config.xml", "light", "b");			
			
			engine.frameSize[0]=(int) configurator.readCustomConfigurations("config.xml", "window", "w");
			engine.frameSize[1]=(int) configurator.readCustomConfigurations("config.xml", "window", "h");			
			
			Thread t = new Thread(engine);
			t.start();
		}	
		
		createNetwork();  //Comment it for GRID!!! 
		System.out.print("INITIALIZING OUTPUT STRATEGY...");
		setOutputStrategy(o);		
		System.out.println("DONE");	
		
		
		float resto=configWrapper.timeUnit%get_dt();
		if(resto>0 || (configWrapper.timeUnit<get_dt())){
			System.out.println("");
			System.out.println("");
			System.out.println("===============================================================================");
			System.out.println(">>>  WARNING!!  (config.xml )  *timeUnit* must be an integer multiple of *_dt* ");
			System.out.println("===============================================================================");
			System.out.println("");
			System.out.println("[Simulation SHUTDOWN...]");
			
			
			System.exit(0);
		}
		
		
	}
	
	public Manager(){
		
	}
	
	public void setRootDomain(RootDomain domain){
		rootDomain=domain;
	}
	
	public RootDomain getRootDomain(){
		return rootDomain;
	}	
	
	public void initialize() {		
		configurator.readConfigurations("config.xml");		
	}	
	
	public void dropNanoObj(NanoObj o){
		rootDomain.detachNanoObj(o);
	}
	
	public void dropNanoObj(NodeObj o){
		rootDomain.detachNanoObj(o);
	}
	
	public void dropNanoObj(CarrierObj o){
		rootDomain.detachNanoObj(o);
	}
			
	public long getCurrentTime(){
		return currentTime;
	}
		
	public void setMobilityModel(MobilityModel m){
		rootDomain.setMobilityModel(m);	
		m.setRootDomain(rootDomain);
	}
	
	public MobilityModel getMobilityModel(){
		return rootDomain.getMobilityModel();
	}

	public void run(){		
		try {
		
		start=System.currentTimeMillis();		
		System.out.println("Starting Simulation...");
		
		for (int i=0;i<getSimulationTimeLength() && !hasStopped;i++){	
			nextSimulationStep();
		}
		
		specializedExport();
		
		} catch (Exception e){
			System.out.println("Error on Manager. "+e.toString());
			e.printStackTrace();
		} finally{			
			System.out.println();
			System.out.println("---- END OF SIMULATION ---- Total time: "+(stop-start)+" ms");
		}
	}
	
	public void setSimulationTimeLength(long t){
		configWrapper.simulationTimeLength=t;
	}
		
	public long getSimulationTimeLength(){		
		return configWrapper.simulationTimeLength;
	}
	
	public void setExportTimeLength(long t){
		configWrapper.exportTimeLength=t;
	}

	public void nextSimulationStep(){	

		currentTime++;
		configWrapper.ddt=configWrapper.ddt-get_dt();	
		activeDomain.clear();
		
		try{
			
			fetchingDomains(rootDomain);			
			
			//System.out.println("FETCH");
			
		} catch(Exception e){
			System.out.println("-0- Error on Manager.NextSimulationStep.Domain.fetching: "+e.toString());
		}	

		try{
			
			nextSimulationStepForAnyDomains();  
			
			//System.out.println("DOMAINS");
			
		} catch(Exception e){
			System.out.println("-0- Error on Manager.NextSimulationStep.Domain.trigger: "+e.toString());
		}		
		
		if (configWrapper.ddt<1) {
			configWrapper.ddt=(float) (configWrapper.timeUnit);
		}		
		
		try{
			export();			
			
			//System.out.println("EXPORT");
			
		} catch(Exception e){
			System.out.println("-3- Error on Manager.NextSimulationStep.export: "+e.toString());
		}
		
		try{
			printCurrentStep();	
			
			//System.out.println("PRINT");
			
			
		} catch(Exception e){
			System.out.println("-5- Error on Manager.NextSimulationStep.printCurrentStep: "+e.toString());
			e.printStackTrace();
		}		
	}

	public void setCurrentTime(int t){
		currentTime=t;
	}

	private void nextSimulationStepForAnyDomains(){
		
		CountDownLatch latch=new CountDownLatch(1);
		
		int div=maxThreads;

		if(activeDomain.size()>div){

			int intPart= activeDomain.size()/div;

			latch=new CountDownLatch(div);	

			for(int i=0;i<div-1;i++){
				Thread tt=new Thread_NextSimulationStep(activeDomain.subList(intPart*i, intPart*(i+1)), latch);
				executorThreads.execute(tt);												
			}
			
			Thread tt=new Thread_NextSimulationStep(activeDomain.subList(intPart*(div-1), activeDomain.size()), latch);
			executorThreads.execute(tt);

		} else {

			Thread tt=new Thread_NextSimulationStep(activeDomain, latch);
			executorThreads.execute(tt);			
		}
		
		try {
			latch.await();
			
		} catch (InterruptedException e) {
			e.printStackTrace();
		}				
	}	
	
	public void fetchingDomains(AbstractDomain domain){
		List<AbstractDomain> childDomainList=domain.getAllChildDomains();
		for(int i=0;i<childDomainList.size();i++){
			fetchingDomains(childDomainList.get(i));
		}		
		if(!activeDomain.contains(domain)) activeDomain.add(domain);
	}
	
	public void createNetwork(){
		getRootDomain().getMobilityModel().ed=configWrapper.ed;  //0.5
		getRootDomain().getMobilityModel().networkCreation();		
	}
	
	public void export(){
		if ((getCurrentTime() % (configWrapper.exportTimeLength))==0){
			try{
				/*
				long start=System.nanoTime();
				*/
				specializedExport();
				/*
				long stop=System.nanoTime();
				System.out.println(""+(stop-start));
				*/
			} catch (Exception e) {			
				System.out.println("Error on Manager.specializedExport");
			}
		}		
	}	
	
	public void setOutputStrategy(OutputStrategy o){
		if (!outputObjList.contains(o)){
			outputObjList.add(o);
			o.initialize(this);
		}
	}
	
	public List<OutputStrategy> getOutputStrategy(){
		return outputObjList;
	}	
	
	public ArrayList<LivingObjObserver> getAllLivingObjObserver() {
		activeDomain.clear();
		try{
			fetchingDomains(rootDomain);
		} catch(Exception e){
			System.out.println("-0- Error on Manager.getAllLivingObjObserver(): "+e.toString());
		}
		
		int size=activeDomain.size();
		ArrayList<LivingObjObserver> obsList=new ArrayList<LivingObjObserver>();
		 
		for(int i=0;i<size;i++){
			obsList.add(activeDomain.get(i).getLivingObjObserver());
		}
		return obsList;
	}
	
	public void set_dt(float _dt) {
		configWrapper._dt = _dt;
	}
	
	public float get_dt() {
		return configWrapper._dt;
	}
	
	public void addDomainForExport(AbstractDomain d){
		if (!exportDomain.contains(d)) exportDomain.add(d);
	}
	
	public void removeDomainForExport(AbstractDomain d){
		if (exportDomain.contains(d)) exportDomain.remove(d);
	}
	
	public ArrayList<AbstractDomain> getDomainsToExport(){
		return exportDomain;
	}
	
	public void setMobilityThreads(int t){
		configWrapper.threads=t;
	}
	
	public int getMobilityThreads(){
		return configWrapper.threads;
	}
}

