package MainLibrary;

import java.util.*;

public abstract class MobilityModel {
			
	public double ed=0.9;  //0.5
	
	public static long moveTime=0;
	public static long checkTime=0;
	public static long avoidTime=0;	
	
	public String extraInfo="";

	protected RootDomain rootDomain = null;
	protected double dt;	
	public Random rand=new Random();	
	
	// XORShiftRandom
	int val=31;
	double div=Math.pow(2, val);
	XORShiftRandom random=new XORShiftRandom();	
	// End XORShiftRandom
	
	public abstract void algorithm(AbstractDomain domain);
	public abstract void networkCreation();			// FIXME Commented to configure the Octree structure!
	public abstract void checkCollision(List<NanoObj> objsToCheck);	
	public abstract AbstractThreadingStrategy getThreadingStrategy();
	public abstract void detectCollision(NanoObj n1, NanoObj n2);	
	public abstract void move(List<NanoObj> nanoObjList);
	public abstract void avoidCollision(AbstractDomain domain);
	public abstract void relocateObjs(NanoObj p1, NanoObj p2);	
	public abstract float getDiffusionCoefficient(NanoObj p);	
	
	//Method used in Grid for fixing the creation of carriers inside cylinders
	public abstract boolean checkCoords(float x, float y, float z);
	
	public MobilityModel(){
		
	}
	
	public MobilityModel(RootDomain d){
		rootDomain=d;
	}	
	
	public Manager getManager(){
		return rootDomain.getManager();
	}	

	public void setRootDomain(RootDomain domain){
		this.rootDomain=domain;
	}	
	
	public RootDomain getRootDomain(){
		return this.rootDomain;
	}
		
	public void setdt(double dt){
		this.dt=dt;
	}
	public double getdt(){
		return this.dt;
	}	
	
	//synchronized
	public  boolean reorderCollision_byHeap(AbstractDomain domain){
				
		ArrayList<AbstractCollision> collisionsListTemp=new ArrayList< AbstractCollision>();
		ArrayList<AbstractCollision> collisionsListTemp2=new ArrayList< AbstractCollision>();		
	
		int size=domain.getCollisionsList().size();	
		
		boolean isDone=false;
		try{
					
		
		for (int i=0;i<size;i++){			
						
			if (domain.getCollisionsList().get(i) instanceof DomainCollisionExternal){				
				collisionsListTemp.add( domain.getCollisionsList().get(i));			
			}else {				
				collisionsListTemp2.add( domain.getCollisionsList().get(i));				
			}
		}

		
		// Reorder first!!
		HeapSorter_Collisions h=new HeapSorter_Collisions();
		h.sort(collisionsListTemp);
					
		/*		
		size=collisionsListTemp.size();		

		for (int i=0;i<size-1;i++){			
			for (int j=i+1;j<size;j++){				
				if(collisionsListTemp.get(j).getDt() > collisionsListTemp.get(i).getDt()){
					tempCollision=collisionsListTemp.get(i);				
					
					collisionsListTemp.add(i, collisionsListTemp.get(j));
					collisionsListTemp.add(j, tempCollision);						
				}
			}
		}
		*/
		
		// Reorder first!!
		//HeapSorter_Collisions h=new HeapSorter_Collisions();
		h.sort(collisionsListTemp2);
		
		
		/*
		size=collisionsListTemp2.size();
		for (int i=0;i<size-1;i++){			
			for (int j=i+1;j<size;j++){				
				try {
					collisionsListTemp2.get(j);
					collisionsListTemp2.get(i);
				
					if(collisionsListTemp2.get(j).getDt() > collisionsListTemp2.get(i).getDt() ){
						tempCollision=collisionsListTemp2.get(i);
						
						collisionsListTemp2.add(i, collisionsListTemp2.get(j));
						collisionsListTemp2.add(j, tempCollision);							
					}
				} catch (Exception e) {					
					System.out.println("-- Exception on MobilityModel.reorderCollision! - size: "+size+", i: "+i+", j: "+j);
					e.printStackTrace();
				}
			}
		}
		*/
		
		collisionsListTemp.addAll(collisionsListTemp2);		
		domain.setCollisionsList(collisionsListTemp);
		
		}catch (Exception e){
			
			e.printStackTrace();
			isDone=false;
		}		
		
		return isDone;
	}	

	public synchronized boolean reorderCollision(AbstractDomain d){
		
		AbstractDomain domain=d;
		
		ArrayList<AbstractCollision> collisionsListTemp=new ArrayList< AbstractCollision>();
		ArrayList<AbstractCollision> collisionsListTemp2=new ArrayList< AbstractCollision>();		
	
		int size=domain.getCollisionsList().size();	
		
		boolean isDone=false;
		try{
			AbstractCollision tempCollision;			
		
		for (int i=0;i<size;i++){			
						
			if (domain.getCollisionsList().get(i) instanceof DomainCollisionExternal){				
				collisionsListTemp.add( domain.getCollisionsList().get(i));			
			}else {				
				collisionsListTemp2.add( domain.getCollisionsList().get(i));				
			}
		}

		size=collisionsListTemp.size();		

		for (int i=0;i<size-1;i++){			
			for (int j=i+1;j<size;j++){				
				if(collisionsListTemp.get(j).getDt() > collisionsListTemp.get(i).getDt()){
					tempCollision=collisionsListTemp.get(i);				
					
					collisionsListTemp.add(i, collisionsListTemp.get(j));
					collisionsListTemp.add(j, tempCollision);						
				}
			}
		}
		
		size=collisionsListTemp2.size();
		for (int i=0;i<size-1;i++){			
			for (int j=i+1;j<size;j++){				
				try {
					//collisionsListTemp2.get(j);
					//collisionsListTemp2.get(i);
				
					if(collisionsListTemp2.get(j).getDt() > collisionsListTemp2.get(i).getDt() ){
						tempCollision=collisionsListTemp2.get(i);
						
						collisionsListTemp2.add(i, collisionsListTemp2.get(j));
						collisionsListTemp2.add(j, tempCollision);							
					}
				} catch (Exception e) {					
					System.out.println("-- Exception on MobilityModel.reorderCollision! - size: "+size+", i: "+i+", j: "+j);
					e.printStackTrace();
				}
			}
		}
		//isDone=false;		
		
		//size=collisionsListTemp.size();			
		
		collisionsListTemp.addAll(collisionsListTemp2);		
		domain.setCollisionsList(collisionsListTemp);
		
		}catch (Exception e){
			
			e.printStackTrace();
			isDone=false;
		}
		return isDone;
	}			
	
	public List<AbstractDomain> reorderDomain_byHeap(AbstractDomain parent){
		
		List<AbstractDomain> domainList=parent.getAllChildDomains();
		List<AbstractDomain> reorderedList=domainList;
		
		try{			
			
			HeapSorter_Domains h1=new HeapSorter_Domains();	
			h1.sort(domainList);	
			reorderedList=domainList; 			
			
		}catch (Exception e){
			//System.out.println("-- Exception in reorderDomain! TimeStep: "+getManager().getCurrentTime()+" - size: "+size+" ActualSize: "+actualSize+" - i: "+indice1+", j: "+indice2);
			System.out.println("-- Exception in reorderDomain_byHeap! TimeStep: "+getManager().getCurrentTime());
			e.printStackTrace();			
		}				
		return reorderedList;
	}
	
	public List<AbstractDomain> reorderDomain(AbstractDomain parent){
		
		List<AbstractDomain> domainList=parent.getAllChildDomains();
		List<AbstractDomain> reorderedList=domainList; 		
		
		int size=domainList.size();
		int actualSize=-1;
		int indice1=-1;
		int indice2=-1;		
		
		try{
			AbstractDomain tempDomain;	
		
			for (int i=0;i<domainList.size()-1;i++){
				indice1=i;
				for (int j=i+1;j<domainList.size();j++){
					indice2=j;
					actualSize=domainList.size();					
					
					if((domainList.get(j).getBoundingRadius()+domainList.get(j).getDistance()) > (domainList.get(i).getBoundingRadius()+domainList.get(i).getDistance())){
						tempDomain=domainList.get(i);					
						reorderedList.set(i, domainList.get(j));
						reorderedList.set(j, tempDomain);					
					}							
				}
			}
		
		}catch (Exception e){
			System.out.println("-- Exception in reorderDomain! TimeStep: "+getManager().getCurrentTime()+" - size: "+size+" ActualSize: "+actualSize+" - i: "+indice1+", j: "+indice2);
			e.printStackTrace();			
		}			
		
		return reorderedList;
	}

	// Random angles in spherical coordinates
	public float[] getRandomAngles(){
		float[] anglesThetaFi=new float[2];
		anglesThetaFi[1]=(float) ((2*Math.PI*(rand.nextFloat()))*0.9999); //fi
		anglesThetaFi[0]=(float) ((Math.acos((2*rand.nextFloat())-1))*0.9999); //theta
		return anglesThetaFi;
	}	
	
	// A better implementation of Gaussian Random Number
	public float getGauss(float sigma){
		float rai=(float) Math.sqrt(-2*sigma*sigma*Math.log(1-(random.next(val)/div)));
		float teta=(float) (2*Math.PI*(random.next(val)/div));
		return (float) (rai*Math.cos(teta));
	}
}
