package MainLibrary;

import java.awt.Color;
import java.util.*;
import Octree.Octree;

public abstract class NanoObj {
	
	protected List<LigandObserver> obsList=new ArrayList<LigandObserver>();
	protected Map<Object, Object> exportString;	
	private LivingObjObserver lifeManager;	
	
	public Color color=Color.cyan;
	public Random rand=new Random();
	protected MotionStrategy motionStrategy;
	private AbstractDomain outerDomain;
	public String IDDomain="";
	public int numberType=-1;
	
	protected long TTL;	
	private float[] center=new float[3];
	protected float radius;	
	
	// Global Velocity
	private float velocityModule;
	private float fi;
	private float theta;
	
	// Holds the new value of Velocity due to MobilityModel Algorithm. Not Mandatory for the motion phase. 
	private float velocityModuleMM;
	private float fiMM;
	private float thetaMM;
	
	//  Holds the new value of Velocity due to MotionStrategy Algorithm. Not Mandatory for the motion phase. 
	private float velocityModuleMS;
	private float fiMS;
	private float thetaMS;
	
	public int collisionCount=0;
	public int informationCount=0;
	private float absDistance=0;
	public float zoom3D=1;	

	protected float mass;
	public long ID;	
	public String name;
	
	// Check value for maximum velocity in Blood Flow!
	public float check=0;
	
	public boolean exportThis=false;	
	
	public abstract void exec();
	protected abstract Map<Object, Object> specializedExport();	
	public abstract void specializedFinalize();
	public abstract void handle(NodeObj n);
	public abstract void handle(CarrierObj c);
	public abstract void handle(NanoObj n);
	public abstract void standardBehavior();	
	public abstract void estabilishContact(NanoObj n);
	public abstract void estabilishContact(AbstractDomain d);
	public abstract void specializedDisposeOnDomain();
	public abstract void setMotionStrategy();
	public abstract void switchDomain(AbstractDomain newDomain);
	public abstract void activateOnDomain(AbstractDomain domain);
	public abstract void removeFromActualDomain(); //FIXME Not to be abstract to handles the generic NanoObj...
	public abstract void attachOnDomain();
	
	
	public abstract String assignName();
	
	public String getTypeNanoObj(){
		return this.name+this.ID;
	}

	@SuppressWarnings("unchecked")
	public NanoObj(AbstractDomain d,float x,float y,float z,float mass,float r,int TTL, int numberType){	
						
		if(getTimeToLive()>-1){
			center[0]=x;
			center[1]=y;
			center[2]=z;
			setDomain(d);
			refreshAbsDistance();	
			
			this.numberType=numberType;

			setVelocityModule(0);
			setFi(0); 
			setTheta(0);

			setVelocityModuleMM(0);
			setFiMM(0); 
			setThetaMM(0);

			setVelocityModuleMS(0);
			setFiMS(0); 
			setThetaMS(0);

			setRadius(r);
			setMass(mass);
			setTimeToLive(TTL);
			
			setMotionStrategy();
			
			name=assignName();
			
			if(!(this instanceof CarrierObj)){ 
				if(d.isInside(this)){
					setDomain(d);
					attachOnDomain();				
					
				
					// FIXME To increase performance, the following condition has been commented
					// but you have to do this control after the network creation and every time you create a NanoObj 
					if (d.getRootTree()!=null)  {						
						
						Octree leaf=d.getRootTree();
						boolean done=false;		
						
						while(!done){
							leaf=leaf.locateChildFor(center);	
							if (leaf.getChildList().size()==0){
								done=true;
							}
						}
						ArrayList<NanoObj> disposedObj=leaf.getAllNanoObjs();
						
						if(d.isFreeSpace(disposedObj, this)){
							disposeOnMedium();	
							attachOnNodeTree(d.getRootTree());
						}else{
							setDomain(d);
							setTimeToLive(-1);							
						}
						
					}else if(d.isFreeSpace((ArrayList)getDomain().getAllDisposedObj(),this)){
						disposeOnMedium();						
					}else{
						setDomain(d);
						setTimeToLive(-1);
					}
				}else{
					setDomain(d);
					setTimeToLive(-1);					
				}
			}	
					
		}
	}

	// If you want that its creation always happens even if it collides with any other NanoObj (the colliding obj will be destroyed!!)
	public boolean mandatoryCreation(){
		return false;
	}
	
	public void initializeWrapper3d(){
		if(getManager().is3DEnable==true)
			getManager().engine.addWrapper3D(this);
	}	

	public void changeDomain(AbstractDomain d){
		getDomain().detachNanoObj(this);
		d.attachNanoObj(this);
	}

	public int getCollisionCount(){
		return collisionCount;
	}
	
	public void increaseCollisionCount(){
		collisionCount++;
	}
	
	public float getAbsDistance(){
		return this.absDistance;
	}
	
	public void refreshAbsDistance(){
		//It is refereed to its Domain!
		try{
			
			AbstractDomain dddd=getDomain();
			if(dddd==null){
				System.out.println();
				System.out.println("**** NULL DOMAIN!!!!!!!!!!! "+getTimeToLive());	
				System.out.println();
			}
			float[] domainCenter=getDomain().getAbsCenter();			
			float dx,dy,dz;
			dx=-domainCenter[0]+center[0];
			dy=-domainCenter[1]+center[1];
			dz=-domainCenter[2]+center[2];			
			this.absDistance=(float) Math.sqrt(dx*dx+dy*dy+dz*dz);
		} catch (Exception e){
			
			System.out.println("Exception - NanoObj.refreshAbsDistance  "+this.toString());			
			e.printStackTrace();
		}
	}
	
	public float getAbsDistancePlusRadius(){
		return this.absDistance+this.radius;
	}
	
	public float getAbsDistanceMinusRadius(){
		return this.absDistance-this.radius;
	}

	public void setLifeManager(LivingObjObserver lifeManager){
		this.lifeManager=lifeManager;
	}
	
	public LivingObjObserver getLifeManager(){
		return this.lifeManager;
	}

	public void notifyToLifeManager(){			
		getLifeManager().addGenericObj(this);
	}
	
	public void kill(){
		setTimeToLive(-1);
	}
	
	public void destroy(){		
		try {
			specializedFinalize();
			
			if ((getDomain()!=null)){
				if (getDomain().contains(this))	{	
					if (this instanceof NodeObj){
						getDomain().detachNanoObj((NodeObj)this);
					}else if (this instanceof CarrierObj){
						getDomain().detachNanoObj((CarrierObj)this);
					}else{
						getDomain().detachNanoObj(this);
					}					
				}
			}
			
			kill();
			
			try {
				super.finalize();
			} catch (Throwable e) {				
				e.printStackTrace();
			}
		} catch (Exception e) {			
			System.out.println("-- Exception on NanoObj.finalize()");
			e.printStackTrace();
		}		
	}	

	public void attachObs(LigandObserver o){
		if (!obsList.contains(o)) obsList.add(o);
	}
	
	public List<LigandObserver> getObsList(){
		return obsList;
	}
	
	public void setTimeToLive(long t){
		TTL=t;
	}
	
	public long getTimeToLive(){
		return TTL;
	}

	public Manager getManager(){
		if(outerDomain!=null){
			return outerDomain.getManager();
		}else {
			return null;
		}
	}		
	
	public void trigger(){
		TTL--;
		exec();		
	}
		
	public void setCenter(float x, float y, float z){
		//Absolute coordinates!!!		
		this.center[0]=x;
		this.center[1]=y;
		this.center[2]=z;
		refreshAbsDistance();
	}
	
	public void setRadius(float r){
		this.radius=r;			
	}
	
	public void setMass(float mass){
		this.mass=mass;
	}
	
	public void disposeOnMedium(){
		specializedDisposeOnDomain();
	}
		
	public float getRadius(){
		return radius;
	}
	
	public float[] getCenter(){
		return center;
	}
	
	public float getMass(){
		return mass;
	}
	
	public void setID(long ID){
		this.ID=ID;
	}
	
	public long getID(){
		return this.ID;
	}
	
	public Map<Object, Object> export(){		
		exportString=new HashMap<Object, Object>();		
		exportString.put("ID", getID());		
		exportString.put("x", center[0]);
		exportString.put("y", center[1]);
		exportString.put("z", center[2]);
		exportString.put("Time To Live", TTL);
		exportString.put("Collision Count", collisionCount);
		exportString.put("Number of Information object", informationCount);		
		Map<Object, Object> specializedString = specializedExport();	
		exportString.putAll(specializedString);		
		return exportString;
	}	
	
	public void switchMotionStrategy(MotionStrategy m){
		this.motionStrategy=m;
	}
	
	public MotionStrategy getMotionStrategy() {
		return motionStrategy;
	}
	
	public void setDomain(AbstractDomain domain) {
		if (domain!=null) {
			this.outerDomain = domain;	
			setDomainID(domain.ID);	
						
		}else{				
			this.outerDomain = null;	
			//setDomainID(null); FIXME commented to grant the objects passing on GRID
		}
	}
	
	public void setDomainID(String s){
		this.IDDomain=s;
	}
	
	public AbstractDomain getDomain() {
		return this.outerDomain;
	}
	
	public synchronized void setColor(Color color){
		this.color=color;	
		
		if(getManager().is3DEnable)
			getManager().engine.changeObjColor(this, color);
	}	
	
	public void setMotionStrategy(MotionStrategy m) {
		this.motionStrategy=m;
	}
	
	public void setVelocityModule(float v){
		this.velocityModule=v;
	}
	
	public float getVelocityModule(){
		return velocityModule;
	}
	
	public void setFi(float fi) {
		this.fi = fi;
	}

	public float getFi() {
		return fi;
	}

	public void setTheta(float theta) {
		this.theta = theta;
	}

	public float getTheta() {
		return theta;
	}	
	
	public float getVelocityModuleMM() {
		return velocityModuleMM;
	}
	
	private void setVelocityModuleMM(float velocityModuleMM) {
		this.velocityModuleMM = velocityModuleMM;
	}
	
	public float getFiMM() {
		return fiMM;
	}
	
	private void setFiMM(float fiMM) {
		this.fiMM = fiMM;
	}
	
	public float getThetaMM() {
		return thetaMM;
	}
	
	private void setThetaMM(float thetaMM) {
		this.thetaMM = thetaMM;
	}
	
	public float getVelocityModuleMS() {
		return velocityModuleMS;
	}
	
	private void setVelocityModuleMS(float velocityModuleMS) {
		this.velocityModuleMS = velocityModuleMS;
	}
	
	public float getFiMS() {
		return fiMS;
	}
	
	private void setFiMS(float fiMS) {
		this.fiMS = fiMS;
	}
	
	public float getThetaMS() {
		return thetaMS;
	}
	
	private void setThetaMS(float thetaMS) {
		this.thetaMS = thetaMS;
	}
	
	public void setVelocityComponentMM(float v, float fi, float theta){
		setVelocityModuleMM(v);
		this.setFiMM(fi);
		this.setThetaMM(theta);
	}
	
	public float[] getVelocityComponentMM(){
		float[] v=new float[3];
		
		v[0]=(float) (velocityModuleMM*Math.sin(getThetaMM())*Math.cos(getFiMM()));
		v[1]=(float) (velocityModuleMM*Math.sin(getThetaMM())*Math.sin(getFiMM()));
		v[2]=(float) (velocityModuleMM*Math.cos(getThetaMM()));			
		
		return v;
	}
	
	public void setVelocityComponentMS(float v, float fi, float theta){
		setVelocityModuleMS(v);
		this.setFiMS(fi);
		this.setThetaMS(theta);
	}
	
	public float[] getVelocityComponentMS(){
		float[] v=new float[3];
		v[0]=(float) (velocityModuleMS*Math.sin(getThetaMS())*Math.cos(getFiMS()));
		v[1]=(float) (velocityModuleMS*Math.sin(getThetaMS())*Math.sin(getFiMS()));
		v[2]=(float) (velocityModuleMS*Math.cos(getThetaMS()));		
		return v;
	}
	
	public void setVelocityComponent(float v, float fi, float theta){			
		setVelocityModule(v);
		this.setFi(fi);
		this.setTheta(theta);
	}

	public float[] getVelocityComponent(float velocity,double fi,double theta){
		float[] v=new float[3];
		v[0]=(float) (velocity*Math.sin(theta)*Math.cos(fi));
		v[1]=(float) (velocity*Math.sin(theta)*Math.sin(fi));
		v[2]=(float) (velocity*Math.cos(theta));		
		return v;
	}
	
	public float[] getVelocityComponent(){
		float[] v=new float[3];
		v[0]=(float) (velocityModule*Math.sin(getTheta())*Math.cos(getFi()));
		v[1]=(float) (velocityModule*Math.sin(getTheta())*Math.sin(getFi()));
		v[2]=(float) (velocityModule*Math.cos(getTheta()));				
		
		return v;
	}
	
	public float[] getVelocityAngles(float vx,float vy,float vz){
		float[] angles=new float[2];		
		float module=(float) Math.sqrt(vx*vx+vy*vy+vz*vz);		
		//angles[0]=0;
		//angles[1]=0;
		
		// FIXME AGGIUNTO PER DEBUG XEON
		angles[0]=(float) ((rand.nextFloat()-0.5)*Math.PI);
		angles[1]=(float) ((rand.nextFloat())*Math.PI);
		
		if (module!=0){
			float thetaArg=vz/module;
			if(thetaArg>1) {
				thetaArg=1;
			}else if(thetaArg<-1){
				thetaArg=-1;
			}
			
			angles[1]=(float) Math.acos(thetaArg);
		}
		if (Double.isNaN(vy)||Double.isNaN(vx)){
			//angles[0]=0;		
			
			// FIXME AGGIUNTO PER DEBUG XEON
			angles[0]=(float) ((rand.nextFloat()-0.5)*Math.PI);	
			
		}else{
			angles[0]=(float) Math.atan2(vy, vx);
		}		
		
		return angles;		
	}
	
	public float[] convertSphericalAngles(float rrx,float rry,float rrz, float distance){

		float[] angles=new float[2];		
			
		angles[0]=0;
		angles[1]=0;		
		if (distance!=0){
			float thetaArg=rrz/distance;
			if(thetaArg>1) {
				thetaArg=1;
			}else if(thetaArg<-1){
				thetaArg=-1;
			}

			angles[1]=(float) Math.acos(thetaArg);
		}
		if (Double.isNaN(rry)||Double.isNaN(rrx)){
			angles[0]=0;
		}else{
			angles[0]=(float) Math.atan2(rry, rrx);
		}
		
		return angles;	
	}	

	// Octree
	public void attachOnNodeTree(Octree tree){
		tree.firstInsertOnNode(this);
	}
}
