package MainLibrary;

import java.awt.Color;
import java.util.*;
import Octree.Octree;

public abstract class NodeObj extends NanoObj {

	private NodeObjState currentState;	
	public List<Receptor> receptorList=new ArrayList<Receptor>();
	public List<Receptor> freeReceptorList=new ArrayList<Receptor>();	
	public EventHandlerForReceptors eventHandler=new EventHandlerForReceptors();	
	protected InnerSphericalDomain innerDomain;	
	public int receptorNumberType=0;
	protected int stateType=0;
	public int burstEmission=0;

	protected ReceptorLocalizationService receptorService;


	public abstract void searchNeighbor();
	public abstract void idle();
	public abstract void encode();
	public abstract void transmit();
	public abstract void decode();
	public abstract void receptorSetup(Receptor r);
	public abstract void createReceptor(int n);
	public abstract void createSingleReceptor(float[] center, float nodeRadius, short receptorRadiusint, int receptorType);
	public abstract void processInfo();

	public NodeObj(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){			
		super(domain,x,y,z,
				domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-mass"), 
				domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-radius").floatValue(), 
				domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-ttl").intValue(), nodeNumberType); 

		/** === Creating a new Inner Domain === */
		/* FIXME The coordinates of the InnerDomain should be relative to parentDomain.
		 * 		 They are now related to the real position of the node!	 */
		float[] dCenter=new float[3];
		dCenter[0]=x-domain.getAbsCenter()[0];
		dCenter[1]=y-domain.getAbsCenter()[1];
		dCenter[2]=z-domain.getAbsCenter()[2];

		InnerSphericalDomain inner=new InnerSphericalDomain(domain, this, new NoMobilityModel(getManager().rootDomain), new InnerSphericalStrategy(0), false, false, dCenter);
		setInnerDomain(inner);
		innerDomain.setOwner(this);
		/** === END creation of Inner Domain === */	

		if(getTimeToLive()>-1){		

			int receptor=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-receptor").intValue();
			this.receptorNumberType=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-receptorType").intValue();
			zoom3D=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-zoom3D");
			color=Color.white;
			stateType=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-stateType").intValue();
			setState(new nIdle(this));		
			burstEmission=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-burstEmission").intValue();

			//if((!domain.getAllUndisposedObj().contains(this)) && domain.getAllDisposedObj().contains(this)){
			if(observer.size()>0){
				for(int i=0; i<observer.size();i++){
					attachObs(observer.get(i));
				}
			}else{
				attachObs(GenericLigandObserver.getInstance());
			}

			receptorService=new ReceptorLocalizationService(this, radius);

			createReceptor(receptor);				

			//checkOverlappingReceptors();

			// NEW CHECK FOR OVERLAPPING RECEPTORS!
			for(short q=0;q<8;q++){					
				checkOverlappingReceptors(receptorService.getQuadrantList(q));
			}
		}
	}	

	public void attachOnDomain() {
		getDomain().attachNanoObj(this);		
	}	

	public InnerSphericalDomain getInnerDomain(){
		return this.innerDomain;
	}	

	public void setInnerDomain(InnerSphericalDomain d){
		this.innerDomain=d;
	}

	public void switchObserverType(LigandObserver currentObs, LigandObserver newObs){
		if (obsList.contains(currentObs)){
			obsList.remove(currentObs);
			obsList.add(newObs);
			for(int i=0;i<receptorList.size();i++){
				if(receptorList.get(i).getObsList().equals(currentObs)){
					receptorList.get(i).attachObs(newObs);
				}
			}
		}
	}	

	public void notifyToLifeManager(){
		getLifeManager().addNode(this);
	}

	public void removeReceptor(Receptor r){
		receptorList.remove(r);
		r.finalize();
	}

	//public void checkOverlappingReceptors(){
	public void checkOverlappingReceptors(ArrayList<UniversalReceptor> rList){

		//FIXME New feature
		List<UniversalReceptor> receptorList=rList;


		float dim1, dim2;		
		float distance;
		//int deathCount=0;
		double lat1;
		double lat2;
		double lon1;
		double lon2;
		double dLat;
		double dLon;  
		double a2;
		double c;
		List<Receptor> receptorToRemove=new ArrayList<Receptor>();

		for (int i=0;i<receptorList.size()-1;i++){
			Receptor r1=(Receptor) receptorList.get(i);			
			dim1=r1.radius;
			lat1=r1.fi;
			lon1=r1.theta;

			for (int j=i+1;j<receptorList.size();j++){				
				Receptor r2=(Receptor) receptorList.get(j);		

				dim2=r2.radius;

				// Geodetic				
				lat2=r2.fi;				
				lon2=r2.theta;

				dLat = (lat2-lat1);  
				dLon = (lon2-lon1);  
				a2 = Math.sin(dLat/2) * Math.sin(dLat/2) + 
						Math.cos(lat1) * Math.cos(lat2) *  
						Math.sin(dLon/2) * Math.sin(dLon/2);  
				c = 2 * Math.atan2(Math.sqrt(a2), Math.sqrt(1-a2));  
				distance = (float) (getRadius() * c); 
				// END Geodetic
				if (distance<((dim1+dim2))){			
					if(!(receptorToRemove.contains(j))){
						receptorToRemove.add(r2);
					}					
					//deathCount++;
				}
			}

			for(int k=0;k<receptorToRemove.size();k++){				
				receptorList.remove(receptorToRemove.get(k));
				receptorService.removeReceptor(receptorToRemove.get(k));				
			}			
			receptorToRemove.clear();			
		}		
	}	

	// ORIGINAL, before ReceptorLocalizationService
	public void checkOverlappingReceptors(){		

		float dim1, dim2;		
		float distance;
		//int deathCount=0;
		double lat1;
		double lat2;
		double lon1;
		double lon2;
		double dLat;
		double dLon;  
		double a2;
		double c;
		List<Receptor> receptorToRemove=new ArrayList<Receptor>();

		for (int i=0;i<receptorList.size()-1;i++){
			dim1=receptorList.get(i).radius;
			lat1=receptorList.get(i).fi;
			lon1=receptorList.get(i).theta;
			for (int j=i+1;j<receptorList.size();j++){				

				dim2=receptorList.get(j).radius;

				// Geodetic				
				lat2=receptorList.get(j).fi;				
				lon2=receptorList.get(j).theta;

				dLat = (lat2-lat1);  
				dLon = (lon2-lon1);  
				a2 = Math.sin(dLat/2) * Math.sin(dLat/2) + 
						Math.cos(lat1) * Math.cos(lat2) *  
						Math.sin(dLon/2) * Math.sin(dLon/2);  
				c = 2 * Math.atan2(Math.sqrt(a2), Math.sqrt(1-a2));  
				distance = (float) (getRadius() * c); 
				// END Geodetic
				if (distance<((dim1+dim2))){			
					if(!(receptorToRemove.contains(j))){
						receptorToRemove.add(receptorList.get(j));
					}					
					//deathCount++;
				}
			}
			for(int k=0;k<receptorToRemove.size();k++){				
				receptorList.remove(receptorToRemove.get(k));
			}			
			receptorToRemove.clear();
		}		
	}	


	public double getCollisionTime(CarrierObj p2){
		float[] center1=getCenter();
		float[] center2=p2.getCenter();
		float[] velocity1=getVelocityComponent();
		float[] velocity2=p2.getVelocityComponent();
		float r1=getRadius();
		float r2=p2.getRadius();	
		double dt=0;	

		float dx21 = (center2[0]-center1[0]);
		float dy21 = (center2[1]-center1[1]);
		float dz21 = (center2[2]-center1[2]);		

		float distance = (float) Math.sqrt(dx21*dx21+dy21*dy21+dz21*dz21);

		// Unit vector in the direction of the collision
		float ax=((dx21/distance)), ay=((dy21/distance)), az=((dz21/distance));

		float va1t=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az);
		float va2t=(velocity2[0]*(ax)+velocity2[1]*(ay)+velocity2[2]*(az));

		dt =((r1+r2-distance)/Math.abs(va1t-va2t)); 			
		
		if (dt>1.1) {
			
			dt =1; 						
		}
		return dt;
	}

	// Verifies the reception of the ligand near receptors
	public void checkIncomingCarrier(CarrierObj c){		
		boolean catched=false;
		float x,  y,  z,  r;
		
		float[] coords=null;	

		// FIXME Relocates the carrierObj on the surface of the nanoObj
		float dt=(float) getCollisionTime(c);
		float[] velocity=c.getVelocityComponent();
		// Carrier Coords on the Node External Surface
		x=c.getCenter()[0]-velocity[0]*dt;
		y=c.getCenter()[1]-velocity[1]*dt;
		z=c.getCenter()[2]-velocity[2]*dt;	
		r=(float) c.getRadius();
		// END FIXME
		

		if (c.getDomain()==getInnerDomain()){		
			// Internal Domain!			
			for (int i=0;i<receptorList.size() && !catched;i++){				
				if((receptorList.get(i).isFree() && !(receptorList.get(i).canDownload()))){
					coords=receptorList.get(i).getAbsoluteCoords();
					float dim=receptorList.get(i).radius;							
					float dx = (coords[0]-x), dy = (coords[1]-y), dz= (coords[2]-z);
					float distance =(float) Math.sqrt(dx*dx+dy*dy+dz*dz);	

					if (distance<=((dim+r)) && (receptorList.get(i).checkCarrierCompatibility(c))){			
						catched=true;						
						receptorList.get(i).configureForUpload(c);	
						//System.out.println("NodeObj.checkIncomingCarrier() - CARRIER DOWNLOAD!");
					}
				}
			}

			if(!catched){	
				getInnerDomain().getCollisionStrategy().relocateInnerObjs(c,getInnerDomain());					
			}

		} else {		
			// External Domain!
			for (int i=0;i<receptorList.size() && !catched;i++){
				if((receptorList.get(i).isFree() && receptorList.get(i).canDownload())){					
					coords=receptorList.get(i).getAbsoluteCoords();
					float dim=receptorList.get(i).radius;								
					float dx = (coords[0]-x), dy = (coords[1]-y), dz= (coords[2]-z);
					float distance =(float) Math.sqrt(dx*dx+dy*dy+dz*dz);									
					if (distance<((dim+r)) && (receptorList.get(i).checkCarrierCompatibility(c))){							
						catched=true;
						receptorList.get(i).configureForDownload(c);							
					}
				}
			}

			if(!catched){											
				getDomain().getMobilityModel().relocateObjs(c,this);
			}			
		} 
	}

	public NodeObjState getState(){
		return currentState;
	}

	public void setState(NodeObjState s){
		currentState=s;
	}
	public int getStateType(){
		return stateType;
	}
	public void setStateType(int s){
		stateType=s;
	} 		

	public void exec(){		

		freeReceptorList.clear();

		this.eventHandler.trigger();
		for (int i=0;i<receptorList.size();i++){			
			if(receptorList.get(i).isFree()){			
				freeReceptorList.add(receptorList.get(i));				
			}
		}		

		if(currentState!=null){
			try{
				currentState.performAction();

			}catch (Exception e){
				System.out.println("-- Error on NodeObj.exec() "+e.toString());
			}
		}else{
			// IF something wrong, set Idle!
			setState(new nIdle(this));
		}		
	}

	public void specializedFinalize(){
		if (receptorList!=null){
			Iterator<Receptor> itR = receptorList.iterator();
			while (itR.hasNext()) {
				Receptor r = itR.next();
				r.finalize();
				itR.remove();
			}
		}

		getInnerDomain().destroy();		
	}	

	public void specializedDisposeOnDomain(){

		getDomain().activatePropagationObj(this);
		initializeWrapper3d();
	}

	public void estabilishContact(NanoObj n){		
		n.handle(this);
	}

	public void estabilishContact(AbstractDomain d){
		try {
			d.handleBoundCollision(this);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			
		
			System.out.println("--- Error on estabilishContact(AbstractDomain d).  "+e.toString());
			e.printStackTrace();
		
			
		}
	}

	public void removeFromActualDomain(){
		AbstractDomain dd=getDomain();
		AbstractDomain dd1=getInnerDomain();
		AbstractDomain dd2=getInnerDomain().getParentDomain();
		
		
		try {
			getDomain().detachNanoObj(this);
			getInnerDomain().getParentDomain().detachChildDomain(getInnerDomain());  //FIXME Does not works! This line is to be removed into setDomain(null)...
		} catch (Exception e) {
			// TODO Auto-generated catch block
			System.out.println(dd.ID+" - "+dd1.ID+" - "+dd2.ID);
			e.printStackTrace();
		}

		for(UniversalReceptor rr:receptorList){
			rr.setOuterDomain(null);
		}
	}

	public void handle(CarrierObj c){
		try{
			checkIncomingCarrier(c);
			
		}catch(Exception e){
			System.out.println("-- EXCEPTION! NodeObj.Handle(CARRIER) by "+this.toString()+"   -   "+e.toString());			
			e.printStackTrace();
		}
	}

	public void handle(NodeObj n){		
		try{
			if(getInnerDomain()==n.getDomain()){
				getInnerDomain().getCollisionStrategy().relocateInnerObjs(n, getInnerDomain());
			} else {
				getDomain().getMobilityModel().relocateObjs(n,this);
			}
		}catch(Exception e){
			System.out.println("-- EXCEPTION! NodeObj.Handle(NODE) "+e.toString());			
		}
	}

	public void handle(NanoObj n){		
		try{
			if(getInnerDomain()==n.getDomain()){
				getInnerDomain().getCollisionStrategy().relocateInnerObjs(n, getInnerDomain());
			} else {
				getDomain().getMobilityModel().relocateObjs(n,this);
			}
		}catch(Exception e){
			System.out.println("-- EXCEPTION! NodeObj.Handle(NANO-GENERIC) "+e.toString());			
		}
	}

	public Receptor getOutgoingReceptor(CarrierObj c) {		
		Random rand=new Random();
		Receptor receptorChoosen=null;
		List<Receptor> freeCandidates=new ArrayList<Receptor>(); 

		for (int i=0;i<freeReceptorList.size();i++){
			if(freeReceptorList.get(i).isFree()){
				if(freeReceptorList.get(i).getObsList().equals(c.getMyObserver()) ||
						freeReceptorList.get(i).getObsList().equals(GenericLigandObserver.getInstance())){
					freeCandidates.add(freeReceptorList.get(i));
				}
			} 
		}
		if (freeCandidates.size()!=0){
			int index=rand.nextInt(freeCandidates.size());
			receptorChoosen=freeCandidates.get(index);
			freeReceptorList.remove(receptorChoosen);
		}
		return receptorChoosen;
	}	

	public void standardBehavior() {		
		// SAFE Version	

		int cSize=getInnerDomain().undisposedCarrierList.size();

		if (cSize>0){
			for(int i=cSize-1;i>=0;i--){
				Receptor r=getOutgoingReceptor(getInnerDomain().undisposedCarrierList.get(i)); 

				if (r!=null){					
					r.configureForUpload(getInnerDomain().undisposedCarrierList.get(i));
				} 
			}
		}
		// END SAFE Version	
	}

	public void downloadInfoFrom(CarrierObj c, Receptor r){		
		informationCount++;			
		c.destroy();
	}

	public void adjChildsCenter(AbstractDomain domain, float[] displacement){		
		List<NanoObj> allDisposedObj=domain.getAllDisposedObj();

		for (int j=0;j<allDisposedObj.size();j++){
			float[] oldCenter=allDisposedObj.get(j).getCenter();
			allDisposedObj.get(j).setCenter(oldCenter[0]+displacement[0], oldCenter[1]+displacement[1], oldCenter[2]+displacement[2]);
		}
	}

	public void setCenter(float x, float y, float z){
		float[] oldCenter=getCenter();
		float[] displacement=new float[3];

		displacement[0]=x-oldCenter[0];
		displacement[1]=y-oldCenter[1];
		displacement[2]=z-oldCenter[2];
		if ((displacement[0]!=0) || (displacement[1]!=0) || (displacement[2]!=0)){
			adjChildsCenter(getInnerDomain(),displacement);
		}		
		super.setCenter(x, y, z);
	}

	// It is called if the carrier pass-through the node's membrane!
	public void switchDomain(AbstractDomain newDomain){
		removeFromActualDomain();
		try {						
			newDomain.attachNanoObj(this);
		} catch (NullPointerException e) {
			// Handles the passage of the nanoobj from two distinct ROOT CUBE DOMAIN
		}
	}

	public void activateOnDomain(AbstractDomain domain){
		domain.activatePropagationObj(this);
	}

	public void initializeWrapper3d(){
		if(getManager().is3DEnable==true)
			getManager().engine.addWrapper3D(this);
	}

	public void setDomain(AbstractDomain domain) {
		if (domain!=null) {				
			super.setDomain(domain);
			setDomainID(domain.ID);
			if (getInnerDomain()!=null) {
				/* FIXME To be checked if works correctly.
				  		 Everytime a NodeObj changes Domain, even its inner domain has to refresh its reference on the parent domain!  */						
				getInnerDomain().changeParentDomain(domain); 	 
			}			
		}else{
			/* FIXME To be checked if works correctly.
	  		 		 Everytime a NodeObj changes Domain, even its inner domain has to refresh its reference on the parent domain!  */			
			super.setDomain(null);
			setInnerDomain(null);	//FIXME 23/04/2015 - May cause problems! try the following line:
			//getInnerDomain().getParentDomain().detachChildDomain(getInnerDomain());
		}
	}

	// FIXME Octree
	public void attachOnNodeTree(Octree tree){
		tree.firstInsertOnNode(this);
	}	
}
