package MainLibrary;

import java.util.ArrayList;

import com.threed.jpct.SimpleVector;

public class ReceptorLocalizationService {
	
	private ArrayList<ArrayList<UniversalReceptor>> container=new ArrayList<ArrayList<UniversalReceptor>>();
	private float half_pi=(float) (Math.PI/2);
	private float side=0;
	private NodeObj node=null;
	private AbstractDomain domain=null;
	
	
	public ReceptorLocalizationService(NodeObj owner, float side){	
		this.side=side;
		this.node=owner;
		for(int i=0;i<8;i++){
			container.add(new ArrayList<UniversalReceptor>());			
		}		
	}
	
	public ReceptorLocalizationService(AbstractDomain owner, float side){
		this.side=side;
		this.domain=owner;
		for(int i=0;i<4;i++){
			container.add(new ArrayList<UniversalReceptor>());			
		}		
	}
	
	public short getQuadrant(float fi, float theta){			
		
		short q1=(short) (Math.floor(theta/half_pi));		
		short q2=(short) (Math.floor(fi/half_pi));			
		
		short quadrant=(short) (q1*4+q2);		
		
		if(quadrant>=8 || quadrant <0){
			System.out.println("ERROR ON getQuadrant()!! "+quadrant+"  -  theta: "+theta+"  -  fi: "+fi);
		}
		return quadrant;		
	}
	
	public short getQuadrant(float x, float y, float z){	
			
		short quad=-1;
		
		//FIXME Use DirectionO and DirectionN to obtain quadrant
		SimpleVector coords=new SimpleVector(x,y,z);
		SimpleVector dN=domain.getDirectionN();
		SimpleVector dO=domain.getDirectionO();
		
		float rdN=coords.calcDot(dN);
		float rdO=coords.calcDot(dO);		
		
		if(rdN<0){
			if(rdO<0){
				quad=2;
			}else{
				quad=3;
			}
		}else {
			if(rdO<0){
				quad=1;
			}else{
				quad=0;
			}			
		}
		
		return quad;		
	}
	
	public short getUniversalReceptorQuadrant(UniversalReceptor r){
		short quadrant=-1;
		
		if(r instanceof Receptor){
			Receptor rr=(Receptor)r;
			quadrant=getQuadrant(rr.fi,rr.theta);
		}else{
			
			float[] center=r.getRelativeCoords();			
			quadrant=getQuadrant(center[0],center[1],center[2]);
		}	
		return quadrant;		
	}
	
	
	public float[] getAnglesForCarrier(NodeObj n, CarrierObj c){
		float[] cCenter=c.getCenter();
		float[] nCenter=n.getCenter();
		
		float[] relativeCenter={cCenter[0]-nCenter[0], cCenter[1]-nCenter[1], cCenter[2]-nCenter[2]};
		
		float[] angles=new float[2];		// [0] = fi, [1] = theta

		angles[0]=(float) (Math.atan2(relativeCenter[1], relativeCenter[0]));
		if(angles[0]<0){
			angles[0]=(float) (angles[0]+(2*Math.PI));
		}
		angles[1]=(float) Math.acos(relativeCenter[2]/side);				
				
		return angles;		
	}
		
	public short getQuadrantForCarrier(NodeObj n, CarrierObj c){
		float[] angles=getAnglesForCarrier(n, c);		
		return getQuadrant(angles[0],angles[1]);
	}
	
	public short getQuadrantForCarrier(AbstractDomain d, CarrierObj c){				
		float[] ccenter=new float[3];
		ccenter[0]=c.getCenter()[0]-d.getAbsCenter()[0];	
		ccenter[1]=c.getCenter()[1]-d.getAbsCenter()[1];	
		ccenter[2]=c.getCenter()[2]-d.getAbsCenter()[2];	
		
		return getQuadrant(ccenter[0],ccenter[1],ccenter[2]);
	}
	
	public short addReceptor(UniversalReceptor r){			
		short quadrant=getUniversalReceptorQuadrant(r);			
		container.get(quadrant).add(r);			
		return quadrant;				
	}
	
	public void removeReceptor(UniversalReceptor r){		
		short quadrant=getUniversalReceptorQuadrant(r);			
		container.get(quadrant).remove(r);			
	}
	
	public ArrayList<UniversalReceptor> getQuadrantList(short quadrant){		
		return container.get(quadrant);		
	}	
}
