package MainLibrary;

import java.util.HashMap;
import java.util.Map;

public class RootDomain extends AbstractDomain {		

	private Manager manager;
	
	public RootDomain(Manager manager, MobilityModel model, AbstractCollisionDomainStrategy strategy, boolean export, boolean useOctree) {		
		super(manager, model, export, useOctree);
		setManager(manager);
		parentDomain=null;
		setCenter(0,0,0);
		model.setRootDomain(this);
		setCollisionStrategy(strategy);		
		ID="ROOT DOMAIN";
		
		//if(useOctree) initializeOctree(getManager().configurator);
		
		if (export) getManager().addDomainForExport(this);
	}
		
	public RootDomain(){
		super();
		parentDomain=null;
		setCenter(0,0,0);	
		ID="ROOT DOMAIN";
		
		//initializeOctree();
	}
	
	public RootDomain(Manager manager, MobilityModel model,	boolean export, boolean useOctree) {		
		super(manager, model, export, useOctree);
		setManager(manager);
		parentDomain=null;
		setCenter(0,0,0);
		ID="ROOT DOMAIN";
		
		//if(useOctree) initializeOctree(getManager().configurator);
		
		if (export) getManager().addDomainForExport(this);
	}
	
	public float getRadius(){
		return Float.POSITIVE_INFINITY;
	}
	
	public void handleBoundCollision(NanoObj n){		
		n.destroy();
	}
	
	public void handleBoundCollision(NodeObj n){		
		n.destroy();
	}
	
	public void handleBoundCollision(CarrierObj c){		
		c.destroy();
	}	

		
	public String getType(){
		return "Root Domain";		
	}

	@Override
	public void setBoundingRadius() {
		boundingRadius=Float.POSITIVE_INFINITY;		
	}
	
	public float[] getAbsCenter(){
		return getRelativeCenter();		
	}

	@Override
	public void trigger() {
		
	}

	@Override
	public Map<Object, Object> export() {
		exportString=new HashMap<Object, Object>();		
		exportString.put("ID", ID);	
		return exportString;
	}

	@Override
	public boolean isInside(NanoObj objToCheck) {
		// If the Obj's center is inside, then the whole object is considered inside!	
		// This is the world's domain, so every living obj have to be inside that!
		return true;
	}		
	
	public Manager getManager(){
		return manager;
	}
	
	public void setManager(Manager manager) {
		this.manager=manager;
	}
	
	public void attachOnGrid(NanoObj n){}	
	public void attachOnGrid(NodeObj n){}
	public void attachOnGrid(CarrierObj c){}

	@Override
	public void initializeWrapper3D(Manager m) {
		// TODO Auto-generated method stub		
	}

	@Override
	public void initializeDirectionAxes() {
		// TODO Auto-generated method stub		
	}
}