package MainLibrary;

import java.util.ArrayList;
import java.util.Map;

import Octree.Octree;

public abstract class UniversalReceptor {
	
	public int ID=0;
	private static int globalCountID=0;
	public static int deleted=0;
	public static int created=0;	
	
	public LigandObserver obsList;	
	public CarrierObj currentCarrier=null;
	public ReceptorState internalState;
	public AbstractDomain outerDomain;
	public AbstractDomain innerDomain;	
	public EventHandlerForReceptors eventHandler=null;		
	
	public float cx,cy,cz;					//refers to its domain center!	
	
	public short radius;	
	public boolean canDownload=true;
	public int receptorType;
	public float relativeDistance=0;
	
	public String name="UniversalReceptor";
	

	public abstract void downloadCarrier(CarrierObj c);	
	public abstract float[] getRelativeCoords();
	public abstract float[] getAbsoluteCoords();	
	public abstract Map<Object,Object> export();	
	public abstract LigandObserver getMyObserver();
	
	
	public UniversalReceptor(AbstractDomain parent, AbstractDomain outer, boolean canDownload, float[] center, short r, int type, EventHandlerForReceptors e){
			
		this.cx=center[0];
		this.cy=center[1];
		this.cz=center[2];
				
		this.relativeDistance=(float) Math.sqrt(center[0]*center[0]+center[1]*center[1]+center[2]*center[2]);
		
		this.radius=r;
		this.outerDomain=outer;
		this.innerDomain=parent;
		setReceptorType(type);
		this.eventHandler=e;
		setInternalState(new rActive(this));
		setCanDownload(canDownload);
		this.ID=increaseGlobalCountID();
		created++;		
		
		name=assignName();
	}
	
	public abstract String assignName();
	
	public float getRelativeDistancePlusRadius(){
		return this.relativeDistance+this.radius;
	}
	
	public float getRelativeDistanceMinusRadius(){
		return this.relativeDistance-this.radius;
	}
	
	public int getID(){
		return ID;
	}
	
	public int getGlobalCountID(){
		return globalCountID;
	}
	
	public int increaseGlobalCountID(){
		globalCountID=globalCountID+1;
		return globalCountID;
	}
	
	public void setReceptorType(int type){
		this.receptorType=type;
	}
	
	public int getReceptorType(){
		return receptorType;
	}
	
	public boolean canDownload(){
		return canDownload;
	}

	public void setCanDownload(boolean canDownload){
		this.canDownload=canDownload;
	}

	public AbstractDomain getOuterDomain(){
		return outerDomain;
	}

	public void setOuterDomain(AbstractDomain domain){
		this.outerDomain=domain;
	}

	public AbstractDomain getInnerDomain(){
		return innerDomain;
	}

	public AbstractDomain getParent(){
		return innerDomain;
	}

	public void attachObs(LigandObserver o){
		obsList=o;
	}

	public LigandObserver getObsList(){
		return obsList;
	}

	public void attachCarrier(CarrierObj c){
		currentCarrier=c;
		float[] absoluteCoords=getAbsoluteCoords();
		c.setCenter(absoluteCoords[0], absoluteCoords[1], absoluteCoords[2]);
		c.getDomain().detachNanoObj(c);				
	}

	public void detachCarrier(){
		currentCarrier=null;
		internalState.remainingTime=0;
		setInternalState(new rActive(this));
	}

	public CarrierObj getCurrentCarrier(){
		return currentCarrier;
	}

	public ReceptorState getInternalState(){
		return internalState;
	}

	public void setInternalState(ReceptorState s){
		this.internalState=s;
	}

	public boolean isFree(){
		return getInternalState().isFree();
	}

	public short getRadius(){
		return radius;
	}

	
	public void configureForDownload(CarrierObj c, boolean isRandomDownload){		
				
		if(isRandomDownload){
			attachCarrier(c);		
			setInternalState(new rRandomDownloading(this));	
		}else{
			configureForDownload(c);
		}
	}
	
	public void configureForDownload(CarrierObj c){
		attachCarrier(c);		
		setInternalState(new rDownloading(this));		
	}

	public void configureForUpload(CarrierObj c){		
		attachCarrier(c);
		setInternalState(new rUploading(this));		
	}

	public void upload(){	
		CarrierObj carrier=getCurrentCarrier();	
		
		// FIXME setCarrierMobility could cause an EXception, because carrier.getDomain()=null in this time instant		
		carrier.setDomain(getOuterDomain());  // This should FIX the problem...
		setCarrierMobility(carrier,this);
		
		
		/** NEW for Octree */
		// FIXME To increase performance, the following condition has been commented
		// but you have to do this control after the network creation and every time you create a NanoObj 
		AbstractDomain d=getOuterDomain();
		if (d.getRootTree()!=null)  {						
			
			Octree leaf=d.getRootTree();
			boolean done=false;		
			
			while(!done){				
				
				float[] center=new float[3];
				center[0]=cx;
				center[1]=cy;
				center[2]=cz;
				
				leaf=leaf.locateChildFor(center);	
				if (leaf.getChildList().size()==0){
					done=true;
				}
			}
			ArrayList<NanoObj> disposedObj=leaf.getAllNanoObjs();
			
			if(getOuterDomain().isFreeSpace(disposedObj, carrier)){
				getOuterDomain().attachNanoObj(carrier);
				carrier.disposeOnMedium();
				detachCarrier();
			}			
			
		}else if(getOuterDomain().isFreeSpace(carrier)){
			getOuterDomain().attachNanoObj(carrier);
			carrier.disposeOnMedium();
			detachCarrier();						
		}		
		/** END Octree */
		
		
		/** Before Octree */
		/*
		if (getOuterDomain().isFreeSpace(carrier)){			
			getOuterDomain().attachNanoObj(carrier);
			carrier.disposeOnMedium();
			detachCarrier();	

		}*//*else{			
			System.out.println(">>> Delayed Upload: "+getProcessingTime());
		}*/
	}		
	
	public void download(){		
		CarrierObj carrier=getCurrentCarrier();
		
		/* It assumes the CarrierObj was detached from its previous domain at the same time that it is attached to a Receptor! */	
		getInnerDomain().attachNanoObj(carrier);
		downloadCarrier(carrier);
		detachCarrier();	
	}
	
	public void setProcessingTime(int t){
		internalState.setProcessingTime(t);
		internalState.initRemainingTime();
	}

	public int getProcessingTime(){
		return internalState.getProcessingTime();
	}

	public void trigger(){		
		if (currentCarrier==null || !(getInnerDomain().getLivingObjObserver().isAlive(currentCarrier)
				|| getOuterDomain().getLivingObjObserver().isAlive(currentCarrier))){
			currentCarrier=null;
			setInternalState(new rActive(this));
		}		
		internalState.performAction();		
	}	

	public boolean checkCarrierCompatibility(CarrierObj c){
		boolean found=false;
		LigandObserver myObsList=getObsList();				

		if(myObsList.equals(GenericLigandObserver.getInstance())){
			found=true;
		}
		if (myObsList.ligandList.contains(c)){
			found=true;	
		}
		return found;
	}

	public void finalize(){		
		deleted++;
		if(getCurrentCarrier()!=null){
			getCurrentCarrier().destroy();
		}
		try {
			super.finalize();
		} catch (Throwable e) {			
			e.printStackTrace();
		}
	}
	
	private static float approx(double d, int precision){
		return (float) (Math.round( d * Math.pow( 10, precision ) )/Math.pow( 10, precision ));
	}
	
	public void setCarrierMobility(CarrierObj c, UniversalReceptor r){		
		
		float[] receptorCenter=r.getRelativeCoords();
		float[] nodeCenter=innerDomain.getAbsCenter();
		float[] nodeCenterRel=innerDomain.getRelativeCenter();
		nodeCenter[0]=nodeCenter[0]-nodeCenterRel[0];
		nodeCenter[1]=nodeCenter[1]-nodeCenterRel[1];
		nodeCenter[2]=nodeCenter[2]-nodeCenterRel[2];
		
		short receptorRadius=r.getRadius();

		float dx = approx(receptorCenter[0],6);
		float dy = approx(receptorCenter[1],6);
		float dz = approx(receptorCenter[2],6);
		
		float module=(float) Math.sqrt(dx*dx+dy*dy+dz*dz);
		float fi=0;
		float theta=0;
		
		if (module!=0){
			theta=(float) Math.acos(dz/module);
		}
		
		if (Double.isNaN(dy)||Double.isNaN(dx)){
			fi=0;
		}else{
			fi=(float) Math.atan2(dy, dx);
		}
		
		// set the carrier's position on the receptor's edge+carrier's radius
		module=module+receptorRadius+c.getRadius();  
		float x=approx(nodeCenter[0]+module*Math.sin(theta)*Math.cos(fi),6);
		float y=approx(nodeCenter[1]+module*Math.sin(theta)*Math.sin(fi),6);
		float z=approx(nodeCenter[2]+module*Math.cos(theta),6);

		c.setCenter(x, y, z);
		c.setVelocityComponent(c.getVelocityModule(),fi, theta);
	}
	
	public void reassignLigandObserver(){
		this.attachObs(getMyObserver());
	}	
}


