/** 
 * 		 ___________
 * 		|			|
 * 		|			|
 * 		|	  .		| L
 * 		|  (x,y,z)	|
 * 		|___________|
 * 			  L
 */

package Octree;

import java.util.ArrayList;
import java.util.Iterator;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.NanoObj;
import MainLibrary.NodeObj;

public abstract class Octree {
	
	public int maxLevel=0;
	
	/**	  CHILDS HIERARCHY	
	 * 	  0=NWtop, 1=NEtop, 2=SWtop, 3=SEtop, 4= NWbottom, 5=NEbottom, 6=SWbottom, 7=SEbottom	 */
	protected ArrayList<OctreeNode> childList=new ArrayList<OctreeNode>();	
	
	private ArrayList<NanoObj> genericObjs=new ArrayList<NanoObj>();
	private ArrayList<NodeObj> nodeObjs=new ArrayList<NodeObj>();
	private ArrayList<CarrierObj> carrierObjs=new ArrayList<CarrierObj>();
	
	public float x,y,z;
	public float side;				// this is the side L of the cube 
	public int maxObjects=1000;		// max quantity of nano objs for this Octree Node
	public int minSide=1000;		// min side for this Octree Node
	public String type="";
	public int level=0;	   
	private AbstractDomain domain=null;
		
	public abstract OctreeRoot getRoot(); 
	public abstract void firstInsertOnNode(NanoObj b);
	public abstract void firstInsertOnNode(NodeObj b);
	public abstract void firstInsertOnNode(CarrierObj b);
	
	public Octree(float x, float y, float z, float side, int maxObjects, int minSide){
		this.x = x;
        this.y = y;
        this.z = z;
        this.maxObjects=maxObjects;
        this.minSide=minSide;
        this.side=side; 
	}
	

	public boolean hasToSplit(){
		boolean toSplit=false;
		if (getSizeAllNanoObjs()>maxObjects && side>minSide*2) {	
			toSplit=true;		
		}
	
		return toSplit;		
	}

	public void insert(NodeObj b) {	
		if (childList.size()>0){
			locateChildFor(b).insert(b);
		}else{
			if(getSizeAllNanoObjs()>=maxObjects){
				//synchronized(this){
					//if (!(childList.size()>0)){
						//SPLITS!!!
						createChilds();							
						locateChildFor(b).insert(b);				

						// All Objs have to be relocated, not only this type!
						relocateAllNanoObjs();
					/*}else{
						locateChildFor(b).insert(b);
					}
				}*/
			}else{
				nodeObjs.add(b);
			}
		}	    	
	}
	
	public void insert(CarrierObj b) {		
		if (childList.size()>0){
			locateChildFor(b).insert(b);
		}else{
			if(getSizeAllNanoObjs()>=maxObjects){
				//synchronized(this){
					//if (!(childList.size()>0)){
						//SPLITS!!!
						createChilds();				
						locateChildFor(b).insert(b);

						// All Objs have to be relocated, not only this type!
						relocateAllNanoObjs();	
					/*}else{
						locateChildFor(b).insert(b);
					}
				}*/
			}else{
				synchronized(carrierObjs){
					carrierObjs.add(b);
				}
			}
		}	    	
	}	
	
	public void insertOLD(CarrierObj b) {		
		if (childList.size()>0){
			locateChildFor(b).insert(b);
		}else{
			if(getSizeAllNanoObjs()>=maxObjects){
				//synchronized(this){
					//if (!(childList.size()>0)){
						//SPLITS!!!
						//if (getRoot().domain!=null) System.out.println("DOMAIN: "+domain.ID);
						createChilds();				
						locateChildFor(b).insert(b);				

						// All Objs have to be relocated, not only this type!
						try {
							//synchronized(this){
								//relocateAllNanoObjs();
							//}
						} catch (Exception e) {
							// TODO Auto-generated catch block
							System.out.println();
							System.out.println("RELOCATE");
							System.out.println();
							e.printStackTrace();
							
						}
						//System.out.println("LEVEL: "+level+"  -  size: "+getSizeAllNanoObjs());
						//System.out.println("");
					/*}else{
						locateChildFor(b).insert(b);
					}
				}*/
			}else{
				//synchronized(this){
					carrierObjs.add(b);
				//}
			}
		}	    	
	}

	public void insert(NanoObj b) {		
		if (childList.size()>0){
			locateChildFor(b).insert(b);
		}else{
			if(getSizeAllNanoObjs()>=maxObjects){
				//synchronized(this){
					//if (!(childList.size()>0)){
						//SPLITS!!!
						createChilds();				
						locateChildFor(b).insert(b);

						// All Objs have to be relocated, not only this type!
						relocateAllNanoObjs();	
					/*}else{
						locateChildFor(b).insert(b);
					}
				}*/
			}else{
				genericObjs.add(b);
			}
		}	    	
	}


	public OctreeNode locateChildFor(NanoObj b){
		OctreeNode child = null;

			try {
				float[] center=b.getCenter();		
				float bx=center[0];
				float by=center[1];
				float bz=center[2];						
				
				if(bx<this.x) { // WEST SIDE
					if(by<this.y){	// SUD WEST SIDE
						if(bz<this.z){	// BOTTOM SUD WEST SIDE
							child=childList.get(6);
							
						}else{	// TOP SUD WEST SIDE
							child=childList.get(2);
							
						}

					}else{	// NORD WEST SIDE
						if(bz<this.z){	// BOTTOM NORD WEST SIDE
							child=childList.get(4);
							
						}else{	// TOP NORD WEST SIDE
							child=childList.get(0);
							
						}
					}
				}else{	// EST SIDE
					if(by<this.y){	// SUD EST SIDE
						if(bz<this.z){	// BOTTOM SUD EST SIDE
							child=childList.get(7);
							
						}else{	// TOP SUD EST SIDE
							child=childList.get(3);
							
						}

					}else{	// NORD EST SIDE
						if(bz<this.z){	// BOTTOM NORD EST SIDE
							child=childList.get(5);
							
						}else{	// TOP NORD EST SIDE
							child=childList.get(1);
							
						}
					}
				} 
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				System.out.println(" LOCATE ");
			}
		return child;
	}
	
	public OctreeNode locateChildFor(float[] center){
		OctreeNode child = null;

		try {
					
				float bx=center[0];
				float by=center[1];
				float bz=center[2];						
				
				if(bx<this.x) { // WEST SIDE
					if(by<this.y){	// SUD WEST SIDE
						if(bz<this.z){	// BOTTOM SUD WEST SIDE
							child=childList.get(6);
							
						}else{	// TOP SUD WEST SIDE
							child=childList.get(2);
							
						}

					}else{	// NORD WEST SIDE
						if(bz<this.z){	// BOTTOM NORD WEST SIDE
							child=childList.get(4);
							
						}else{	// TOP NORD WEST SIDE
							child=childList.get(0);
							
						}
					}
				}else{	// EST SIDE
					if(by<this.y){	// SUD EST SIDE
						if(bz<this.z){	// BOTTOM SUD EST SIDE
							child=childList.get(7);
							
						}else{	// TOP SUD EST SIDE
							child=childList.get(3);
							
						}

					}else{	// NORD EST SIDE
						if(bz<this.z){	// BOTTOM NORD EST SIDE
							child=childList.get(5);
							
						}else{	// TOP NORD EST SIDE
							child=childList.get(1);
							
						}
					}
				} 
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();				
			}
		return child;
	}

	
	public Octree locateLeaf(NanoObj b){
		
		float[] center=b.getCenter();		
		float bx=center[0];
		float by=center[1];
		float bz=center[2];
		
		long size=getChildList().size();
		
		float nx;
		float ny;
		float nz;
		
		
		Octree node=this;
		
		while(size>0){					
			
			nx=node.x;
			ny=node.y;
			nz=node.z;
			
			if(bx<nx) { // WEST SIDE
				if(by<ny){	// SUD WEST SIDE
					if(bz<nz){	// BOTTOM SUD WEST SIDE
						node=childList.get(6);
					}else{	// TOP SUD WEST SIDE
						node=childList.get(2);
					}
				}else{	// NORD WEST SIDE
					if(bz<nz){	// BOTTOM NORD WEST SIDE
						node=node.childList.get(4);
					}else{	// TOP NORD WEST SIDE
						node=node.childList.get(0);
					}
				}
			}else{	// EST SIDE
				if(by<ny){	// SUD EST SIDE
					if(bz<nz){	// BOTTOM SUD EST SIDE
						node=node.childList.get(7);
					}else{	// TOP SUD EST SIDE
						node=node.childList.get(3);
					}

				}else{	// NORD EST SIDE
					if(bz<nz){	// BOTTOM NORD EST SIDE
						node=node.childList.get(5);
					}else{	// TOP NORD EST SIDE
						node=node.childList.get(1);
					}
				}
			}
			
			
			size=node.getChildList().size();
		}	// END WHILE
		
		return node;
	}
	
	
	public boolean createChilds(){
		boolean created=false;
		
		if (childList.size()==0){		
			
			
			float sideFull=this.side/2;
			float side=this.side/4;
			
			OctreeRoot root=getRoot();			
			
			//  0=NWtop, 1=NEtop, 2=SWtop, 3=SEtop, 4= NWbottom, 5=NEbottom, 6=SWbottom, 7=SEbottom
			
			OctreeNode n0=new OctreeNode(root, this, "NW-TOP" , this.x-side, this.y+side, this.z+side, sideFull, maxObjects, minSide);
			OctreeNode n1=new OctreeNode(root, this, "NE-TOP" , this.x+side, this.y+side, this.z+side, sideFull, maxObjects, minSide);
			OctreeNode n2=new OctreeNode(root, this, "SW-TOP" , this.x-side, this.y-side, this.z+side, sideFull, maxObjects, minSide);
			OctreeNode n3=new OctreeNode(root, this, "SE-TOP" , this.x+side, this.y-side, this.z+side, sideFull, maxObjects, minSide);
			OctreeNode n4=new OctreeNode(root, this, "NW-BOTTOM" , this.x-side, this.y+side, this.z-side, sideFull, maxObjects, minSide);
			OctreeNode n5=new OctreeNode(root, this, "NE-BOTTOM" , this.x+side, this.y+side, this.z-side, sideFull, maxObjects, minSide);
			OctreeNode n6=new OctreeNode(root, this, "SW-BOTTOM" , this.x-side, this.y-side, this.z-side, sideFull, maxObjects, minSide);
			OctreeNode n7=new OctreeNode(root, this, "SE-BOTTOM" , this.x+side, this.y-side, this.z-side, sideFull, maxObjects, minSide);


			childList.add(0, n0); //NWtop
			childList.add(1, n1); //NEtop
			childList.add(2, n2); //SWtop
			childList.add(3, n3); //SEtop
			childList.add(4, n4); //NWbottom
			childList.add(5, n5); //NEbottom
			childList.add(6, n6); //SWbottom
			childList.add(7, n7); //SEbottom			

			ArrayList<OctreeNode> tempLeaf=root.getLeafList();
			synchronized(tempLeaf){
				
				root.getLeafList().remove(this);				

				root.getLeafList().add(n0);
				root.getLeafList().add(n1);
				root.getLeafList().add(n2);
				root.getLeafList().add(n3);
				root.getLeafList().add(n4);
				root.getLeafList().add(n5);
				root.getLeafList().add(n6);
				root.getLeafList().add(n7);					
			}			
			created=true;
		}		
		return created;
	}

	
	public OctreeNode getChild(int i){		
		return childList.get(i);		
	}
	
	public OctreeNode getChild(String id){		
		OctreeNode child=null;
		
		if (id=="NWtop") child=childList.get(0);
		if (id=="NEtop") child=childList.get(1);
		if (id=="SWtop") child=childList.get(2);
		if (id=="SEtop") child=childList.get(3);
		if (id=="NWbottom") child=childList.get(4);
		if (id=="NEbottom") child=childList.get(5);
		if (id=="SWbottom") child=childList.get(6);
		if (id=="SEbottom") child=childList.get(7);	
		
		return child;		
	}

	public ArrayList<OctreeNode> getChildList(){
		return childList;
	}	
	
	public ArrayList<Octree> getLeafs(Octree startNode){
		ArrayList<Octree> leafList=new ArrayList<Octree>();
		
		if(startNode.getChildList().size()>0) {				
			for(Octree n:startNode.getChildList()) {
				leafList.addAll(getLeafs(n));
			}			
		}else{
			leafList.add(startNode);			
		}	
		
		return leafList;		
	}
	
	public String getInfo(){	
		ArrayList<NanoObj> list=getAllNanoObjs();
		String info="["+x+", "+y+", "+z+"]  -  Child: "+childList.size()+" - Ball: "+list.size();	
		info=info+"\n";
		if(list.size()>0){
			for(NanoObj b:list){
				info=info+" |__ "+b.getCenter()[0]+", "+b.getCenter()[1]+", "+b.getCenter()[2]+"\n";
			}
		}
		return info;		
	}
	
	
	public String getInfoLight(){	
		ArrayList<NanoObj> list=getAllNanoObjs();
		ArrayList<NodeObj> nodes=nodeObjs;
		String info="L: "+level+"   - "+type+" -   ["+x+", "+y+", "+z+"]  - side: "+side+" -  Child: "+childList.size()+" - Nano: "+list.size()+" - Nodes: "+nodes.size();	
		info=info+"\n";
		
		
		return info;		
	}
	
	
	
	public String getInfoMatlab(){	
		ArrayList<NanoObj> list=getAllNanoObjs();
		ArrayList<NodeObj> nodes=nodeObjs;
		String info=level+", "+side+", "+x+", "+y+", "+z;	
		//info=info+"\n";
		
		
		return info;		
	}
	
	public void printTree(Octree node){	

		String blank=" ";
		
		for(int i=0;i<node.level;i++){
			blank=blank.concat("  ");
		}
		System.out.println(blank+" |__ "+node.type+"  -  L: "+node.level+"  -  Objs: "+getSizeAllNanoObjs());
		
		if(node.getChildList().size()>0) {	
			
			for(Octree n:node.getChildList()) {
				
				printTree(n);
			}			
		}				
	}
	
	public boolean remove(NanoObj n){		
		boolean hasDeleted=false;
		hasDeleted=genericObjs.remove(n);	
		
		
		if(!hasDeleted && getChildList().size()>0){
			ArrayList<Octree> leafsList = getLeafs(this);
			int size=leafsList.size();
			for(int i=0;i<size && !hasDeleted;i++){
				hasDeleted=leafsList.get(i).remove(n);
			}
		}		
		return hasDeleted;		
	}	
	
	public boolean remove(NodeObj n){		
		boolean hasDeleted=false;		
		hasDeleted=nodeObjs.remove(n);			
		
		if(!hasDeleted && getChildList().size()>0){
			ArrayList<Octree> leafsList = getLeafs(this);
			int size=leafsList.size();
			for(int i=0;i<size && !hasDeleted;i++){
				hasDeleted=leafsList.get(i).remove(n);
			}
		}		
		return hasDeleted;		
	}	
	
	public boolean remove(CarrierObj n){		
		boolean hasDeleted=false;	
		hasDeleted=carrierObjs.remove(n);			
		
		if(!hasDeleted && getChildList().size()>0){
			ArrayList<Octree> leafsList = getLeafs(this);
			int size=leafsList.size();
			for(int i=0;i<size && !hasDeleted;i++){
				hasDeleted=leafsList.get(i).remove(n);
			}
		}		
		return hasDeleted;		
	}	
	
	
	public ArrayList<NanoObj> getAllNanoObjs(){
		ArrayList<NanoObj> returnedList=new ArrayList<NanoObj>();
		
		returnedList.addAll(carrierObjs);
		returnedList.addAll(nodeObjs);
		returnedList.addAll(genericObjs);				
		
		return returnedList;	
	}
	
	public ArrayList<NanoObj> getGenericObjs(){
		return genericObjs;	
	}
	
	public ArrayList<NodeObj> getNodeObjs(){
		return nodeObjs;	
	}

	public ArrayList<CarrierObj> getCarrierObjs(){
		return carrierObjs;	
	}

	public void addToNodeList(NodeObj n){
		synchronized(nodeObjs){	
			nodeObjs.add(n);			
		}
	}

	public void addToGenericList(NanoObj n){
		synchronized(genericObjs){
			genericObjs.add(n);	
		}
	}

	public void addToCarrierList(CarrierObj c){	
		synchronized(carrierObjs){
			carrierObjs.add(c);				
		}	
	}

	public long getSizeAllNanoObjs(){	
		return carrierObjs.size()+nodeObjs.size()+genericObjs.size();	
	}

		
	public void relocateAllNanoObjs(){	
		
			
		Iterator<CarrierObj> itC=carrierObjs.iterator();
		while(itC.hasNext()){
			CarrierObj cc=itC.next();
			OctreeNode nn=locateChildFor(cc);			
			nn.addToCarrierList(cc);			
		}
		carrierObjs.clear();	
		
		Iterator<NanoObj> itG=genericObjs.iterator();
		while(itG.hasNext()){
			NanoObj cc=itG.next();
			OctreeNode nn=locateChildFor(cc);			
			nn.addToGenericList(cc);			
		}
		genericObjs.clear();	
		
		Iterator<NodeObj> itN=nodeObjs.iterator();
		while(itN.hasNext()){
			NodeObj cc=itN.next();
			OctreeNode nn=locateChildFor(cc);			
			nn.addToNodeList(cc);			
		}
		nodeObjs.clear();			
	}		

	public void relocateAllToLeafs(){	

		try {
			for(int i=0;i<carrierObjs.size();i++){
				Octree nn=locateChildFor(carrierObjs.get(i));
				nn.addToCarrierList(carrierObjs.get(i));			
			}		
			carrierObjs.clear();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}


		for(int i=0;i<genericObjs.size();i++){			
			Octree nn=locateChildFor(genericObjs.get(i));
			nn.addToGenericList(genericObjs.get(i));			
		}			
		genericObjs.clear();
		
		
		for(int i=0;i<nodeObjs.size();i++){			
			Octree nn=locateChildFor(nodeObjs.get(i));
			nn.addToNodeList(nodeObjs.get(i));				
		}			
		nodeObjs.clear();			
	}		

	public void clearNanoObjsLists(){
		carrierObjs.clear();		
		genericObjs.clear();
		nodeObjs.clear();		
	}
		
	public AbstractDomain getDomain() {
		return domain;
	}
	
	public void setDomain(AbstractDomain domain) {
		this.domain = domain;
		for(Octree oo: childList){
			oo.setDomain(domain);
		}
	}	
}
