package Octree;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import MainLibrary.CarrierObj;
import MainLibrary.NanoObj;
import MainLibrary.NodeObj;

public class OctreeRoot extends Octree {

	private ArrayList<OctreeNode> leafList=new ArrayList<OctreeNode>();
	
	private final int maxThreads = Runtime.getRuntime().availableProcessors();				
	private ExecutorService executorThreads = Executors.newFixedThreadPool(maxThreads);		
	
	public OctreeRoot(float x, float y, float z, float side, int maxObjects, int minSide) {
		super(x, y, z, side, maxObjects, minSide);
		this.type="ROOT";
		this.level=0;
		
		if (!(getChildList().size()>0)){			
			createChilds();
		}
		
		/*
		printTree(this);
		System.out.println();
		for(int i=0;i<getChildList().size();i++){			
			System.out.println("i: "+i+"  -  "+getChildList().get(i).getInfo());
		}
		*/			
	}
	
	public OctreeRoot getRoot(){
		return this;		
	}		
	
	/* MULTITHREAD */
	public void firstInsertOnNode(CarrierObj b) {			
	
		Octree leaf=this;
		boolean done=false;		
		
		while(!done){
			leaf=leaf.locateChildFor(b);	
			if (leaf.getChildList().size()==0){
				done=true;
			}
		}
		leaf.addToCarrierList(b);
	}	
	
	@Override
	public void firstInsertOnNode(NanoObj b) {		
		//locateChildFor(b).addToGenericList(b);	// era ok
		//locateLeaf(b).addToGenericList(b);		//ok per altra strategia -  da debuggare
		Octree leaf=this;
		boolean done=false;
		
		while(!done){
			leaf=leaf.locateChildFor(b);	// era ok
			if (leaf.getChildList().size()==0){
				done=true;
			}
		}
		leaf.addToGenericList(b);
	}

	@Override
	public void firstInsertOnNode(NodeObj b) {
		//locateChildFor(b).addToNodeList(b);	// was ok
		//locateLeaf(b).addToNodeList(b);		//was ok for the other strategy - carefully debug this!
		Octree leaf=this;
		boolean done=false;
		
		while(!done){
			leaf=leaf.locateChildFor(b);	// ok
			if (leaf.getChildList().size()==0){
				done=true;
			}
		}
		leaf.addToNodeList(b);
	}		
	
	public void fillUp8Child_OK(List<NanoObj> nanoObjList) {		
		// fillup root tree
		/*  PARALLEL FILL UP */

		CountDownLatch latchP=new CountDownLatch(1);

		int div=maxThreads;
		int size=nanoObjList.size();

		if(size>div){
			int intPart=size/div;	

			if (intPart>0) {
				latchP=new CountDownLatch(div);
			}		

			for(int i=0;i<div-1;i++){			
				executorThreads.execute(new Thread_ParallelFillUp3(this,nanoObjList.subList(intPart*i, intPart*(i+1)),latchP));
			}
			executorThreads.execute(new Thread_ParallelFillUp3(this,nanoObjList.subList(intPart*(div-1), size),latchP));
			
		}else if(size>0){
			executorThreads.execute(new Thread_ParallelFillUp3(this,nanoObjList,latchP));
		}		

		try {
			latchP.await();
			
		} catch (InterruptedException e) {
			e.printStackTrace();
		}		
			
		
		//With Multithread	
		ArrayList<OctreeNode> childs=new ArrayList<OctreeNode>();

		childs.addAll(getChildList());		// 1 Level	

		size=childs.size();

		latchP=new CountDownLatch(size);			

		for(OctreeNode n: childs){						
			executorThreads.execute(new Thread_ParallelFillUp2(n,latchP));
		}

		try {			
			latchP.await();

		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	
	public void fillUp_Down_To_Leafs(List<NanoObj> nanoObjList) {		
		// fillup root tree
		/*  PARALLEL FILL UP */
		CountDownLatch latchP=new CountDownLatch(1);

		int div=maxThreads;
		int size=nanoObjList.size();		

		if(size>div){
			int intPart=size/div;	

			if (intPart>0) {
				latchP=new CountDownLatch(div);
			}		

			for(int i=0;i<div-1;i++){			
				executorThreads.execute(new Thread_ParallelFillUp_Down_To_Leaf(this,nanoObjList.subList(intPart*i, intPart*(i+1)),latchP));
			}
			executorThreads.execute(new Thread_ParallelFillUp_Down_To_Leaf(this,nanoObjList.subList(intPart*(div-1), size),latchP));

			
		}else if(size>0){
			executorThreads.execute(new Thread_ParallelFillUp_Down_To_Leaf(this,nanoObjList,latchP));
		}		

		try {
			latchP.await();
			
		} catch (InterruptedException e) {
			e.printStackTrace();
		}		


		// SPLIT & RELOCATE		
		ArrayList<OctreeNode> tempLeafList=new ArrayList<OctreeNode>();	
		tempLeafList.addAll(leafList);

		size=tempLeafList.size();
		latchP=new CountDownLatch(size);			

		for(int i=0;i<size;i++){		
			//for(OctreeNode n: tempLeafList){
			OctreeNode n=tempLeafList.get(i);
			//long totalObjs=n.getSizeAllNanoObjs();

			//if(totalObjs>n.maxObjects){		// OLD
			if(n.hasToSplit()){					// NEW

				executorThreads.execute(new Thread_Parallel_Split(n,latchP));

			}else{
				latchP.countDown();
			}
		}		

		try {			
			latchP.await();

		} catch (InterruptedException e) {
			e.printStackTrace();
		}				
	}
	
	public void newFillUp(List<NanoObj> nanoObjList) {		
		// fillup root tree
		/*  PARALLEL FILL UP */
		CountDownLatch latchP=new CountDownLatch(1);

		int div=maxThreads;
		int size=nanoObjList.size();


		if(size>div){
			int intPart=size/div;	

			if (intPart>0) {
				latchP=new CountDownLatch(div);
			}		

			for(int i=0;i<div-1;i++){			
				executorThreads.execute(new Thread_ParallelFillUp3(this,nanoObjList.subList(intPart*i, intPart*(i+1)),latchP));
			}
			executorThreads.execute(new Thread_ParallelFillUp3(this,nanoObjList.subList(intPart*(div-1), size),latchP));

			
		}else if(size>0){
			executorThreads.execute(new Thread_ParallelFillUp3(this,nanoObjList,latchP));
		}		

		try {
			latchP.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		//=========================================================================================
		// SPLIT LEAFS
		//=========================================================================================
		
		ArrayList<OctreeNode> leafs=getLeafList();
		
		int lSize=leafs.size();		

		latchP=new CountDownLatch(1);	
		
		ArrayList<WrapperLeafs> subListPool=new ArrayList<WrapperLeafs>();
		ArrayList<OctreeNode> activeLeafs=new ArrayList<OctreeNode>();
		
		for(int k=0;k<lSize;k++){
			if(leafs.get(k).getAllNanoObjs().size()>0) activeLeafs.add(leafs.get(k));
		}
		
		lSize=activeLeafs.size();
		div=div/2;
		
		
		if(lSize>div){
			int intPart=lSize/div;	

			if (intPart>0) {
				latchP=new CountDownLatch(div);
			}			
			
			for(int i=0;i<div-1;i++){
				subListPool.add(new WrapperLeafs(activeLeafs.subList(intPart*i, intPart*(i+1))));
			}
			subListPool.add(new WrapperLeafs(activeLeafs.subList(intPart*(div-1), lSize)));
			
		}else if(lSize>0){
			subListPool.add(new WrapperLeafs(activeLeafs));
			
		}
		
		for(int i=0;i<subListPool.size();i++){
			executorThreads.execute(new Thread_SplitLeafs(this,subListPool.get(i),latchP));
		}
		
		try {
			latchP.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	
	public void fillUp(List<NanoObj> nanoObjList) {		
		// fillup root tree
		/*  OK SINGLE THREAD */
		for(NanoObj n:nanoObjList){
			n.attachOnNodeTree(this);
		}			
		
		//With Multithread	
		ArrayList<OctreeNode> childs=new ArrayList<OctreeNode>();
		childs.addAll(getChildList());
		int size=childs.size();		
		
		CountDownLatch latchP=new CountDownLatch(size);			
		
		for(OctreeNode n: childs){		
			executorThreads.execute(new Thread_ParallelFillUp2(n,latchP));
		}
					
		try {
			latchP.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}		
		//END Multithread
	}	
		
	public void fillUp_OldINSERT(List<NanoObj> nanoObjList) {		
		// fillup root tree
		/*  PARALLEL FILL UP */
		CountDownLatch latchP=new CountDownLatch(1);

		int div=maxThreads;
		int size=nanoObjList.size();

		if(size>div){
			int intPart=size/div;	

			if (intPart>0) {
				latchP=new CountDownLatch(div);
			}		

			for(int i=0;i<div-1;i++){			
				executorThreads.execute(new Thread_ParallelFillUp(this,nanoObjList.subList(intPart*i, intPart*(i+1)),latchP));
			}
			executorThreads.execute(new Thread_ParallelFillUp(this,nanoObjList.subList(intPart*(div-1), size),latchP));

		}else if(size>0){
			executorThreads.execute(new Thread_ParallelFillUp(this,nanoObjList,latchP));
		}		

		try {
			latchP.await();

		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}	

	
	public void parallelFillUp(List<NanoObj> nanoObjList) {		
		
		int childSize=getChildList().size();	
		CountDownLatch latch=new CountDownLatch(childSize);	
		
		
		for(int i=0;i<childSize;i++){			
			Thread tt = new Thread_FillUp(getChildList().get(i), latch);
			executorThreads.execute(tt);
		}		
		
		try {
			latch.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}			
	}
	
	public void cleanRootLeafs(){
		ArrayList<OctreeNode> leafsList=getLeafList();
		//synchronized(getLeafs(this)){
		int size=leafsList.size();		

		if (size>0){
			for(int i=0;i<size;i++){	
				if(leafsList.get(i)!=null){
					leafsList.get(i).clearNanoObjsLists();
				}
			}
		}
		//}			
	}
	
	
	public ArrayList<OctreeNode> getLeafList() {
		return leafList;
	}

	public void setLeafList(ArrayList<OctreeNode> futureLeafs) {
		this.leafList=futureLeafs;
	}
	
	public void prune(List<NanoObj> nanoObjList){
	
		ArrayList<OctreeNode> childList=this.childList;		// Level 1
	
		System.out.println("  CHILD:  "+childList.size());		
		
		for(OctreeNode n: childList){

			n.getChildList().clear();
		}		
		
		leafList.clear();
		leafList.addAll(childList);
		
		// REFILL the Octree!!!
		fillUp_Down_To_Leafs(nanoObjList);		
	}
}
