package Octree;


/** OcTree based  */

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import MainLibrary.AbstractCollisionDomainStrategy;
import MainLibrary.AbstractDomain;
import MainLibrary.HeapSorter;
import MainLibrary.MobilityModel;
import MainLibrary.NanoObj;


public class Thread_CheckCollision extends Thread{

	public Octree node;
	private CountDownLatch latch;
	private MobilityModel model;
	private AbstractDomain domain;
	private List<AbstractDomain> domainToCheck;
	private short ObjFor1Check, ObjFor2Check;
	private boolean hasToSkipCarriers=false;
	

	public Thread_CheckCollision(AbstractDomain domain, MobilityModel model, Octree node, List<AbstractDomain> domainToCheck, short ObjFor1Check, short ObjFor2Check, CountDownLatch latch){
		this.node=node;
		this.latch=latch;
		this.model=model;
		this.domain=domain;
		this.domainToCheck=domainToCheck;
		this.ObjFor1Check=ObjFor1Check;
		this.ObjFor2Check=ObjFor2Check;
		this.hasToSkipCarriers=false;
	}	
	
	public void run() {		
		ArrayList<NanoObj> objsToCheck1=new ArrayList<NanoObj>();		

		int nodeSizeTemp=0;
		
		switch (ObjFor1Check){
		case 0: 
			objsToCheck1.addAll(node.getAllNanoObjs());

			break;
		case 1: 
			objsToCheck1.addAll(node.getNodeObjs());

			break;
		case 2: 
			objsToCheck1.addAll(node.getCarrierObjs());

			break;
		case 3: 
			objsToCheck1.addAll(node.getGenericObjs());
			
			break;
			
		case 10:		
			// BE AWARE!! - SPECIFIC CASE: 1 single NodeObj and a multitude of CarrierObjs
			// The checkCollision will be performed in a not conventional way! 
			// DON'T USE FOR STANDARD SIMULATIONS!!!
			
			//objsToCheck1.addAll(node.getAllNanoObjs());			
			
			nodeSizeTemp=node.getNodeObjs().size();
			if(nodeSizeTemp>1){
				System.out.println(">>> THIS KIND OF SIMULATION CANNOT BE EXECUTED WITHOUT CHECK COLLISION ON CARRIERS!   ->   Exiting...");
				System.exit(0);
			}
			objsToCheck1.addAll(node.getCarrierObjs());
			hasToSkipCarriers=true;

			break;
		
		default: 
			objsToCheck1.addAll(node.getAllNanoObjs());

			break;
		}

		//objsToCheck1.addAll(node.getAllNanoObjs());			


		//Check Collision!!
		boolean isFinish=false;
		NanoObj n=null;
		int size1=objsToCheck1.size()+nodeSizeTemp;
		AbstractCollisionDomainStrategy strategy=domain.getCollisionStrategy();	
		

		if (size1>0){
			
			if(hasToSkipCarriers){
				// EXPERIMENTAL - SKIP CARRIER RECIPROCAL COLLISION CONTROL
				
				if(nodeSizeTemp>0){
					// WARNING! Only ONE NodeObj for THIS specific scenario!!!
					n=node.getNodeObjs().get(0);
					
					try {
						strategy.checkInternalCollisionBound(n);
					} catch (Exception e) {
						// TODO Auto-generated catch block
						System.out.println("NodeObj: "+n.toString());
						e.printStackTrace();
					}
				}
				
				int carrierListSize=size1-nodeSizeTemp;
				
				for (int j=0;j<carrierListSize;j++){	
						
					NanoObj cc=objsToCheck1.get(j);
					try {	
						if(nodeSizeTemp>0 && cc.getTimeToLive()>=0){						
						
							model.detectCollision(cc,n);
						}
					} catch (Exception e) {
						e.printStackTrace();						
					}
					
					try {
						strategy.checkInternalCollisionBound(cc);
					} catch (Exception e) {
						System.out.println("Carrier: "+n.toString());
						e.printStackTrace();
					}
				}				
				// END EXPERIMENTAL
							
				
			}else{

				// ORIGINAL - CHECK COLLISION BETWEEN ALL NANOOBJS!				
				
				// Reorder first!!		
				HeapSorter h=new HeapSorter();
				try {
					h.sort(objsToCheck1);
				} catch (Exception e) {
					e.printStackTrace();
					System.out.println("");
					System.out.println(">>>  "+objsToCheck1.size());
					System.out.println("");
				}		
				
				// Check now!
				for (int i=0;i<size1;i++){
					n=objsToCheck1.get(i);	


					// FIXME Ca++ - DEBUG DISABLED CARRIER COLLISION CHECKING...
					// Some comparisons could be skipped... Don't use it!!
					// if (!(n instanceof CarrierObj) ){				

					if(n.getTimeToLive()<0){
						isFinish=true;
					}else{
						isFinish=false;
					}
					//isFinish=false;	


					// ORIGINAL
					for (int j=i+1;j<size1 && !isFinish;j++){	
						//for (int j=i+1;j<size && !isFinish;j++){		
						try {	
							if(objsToCheck1.get(j).getTimeToLive()<0){
								
							}else if(objsToCheck1.get(j).getAbsDistancePlusRadius()>=n.getAbsDistanceMinusRadius()){
								model.detectCollision(objsToCheck1.get(j),n);					

							} else{
								isFinish=true;
							}
						} catch (Exception e) {
							e.printStackTrace();
							System.out.println("");
							System.out.println(">>> SIZE: "+size1+",  i: "+i+", j: "+j+"  -  "+n.toString());
							System.out.println("");
						}
					}
					

					//}
					try {
						strategy.checkInternalCollisionBound(n);
					} catch (Exception e) {
						System.out.println("Carrier: "+n.toString());
						e.printStackTrace();
					}
				}
				

			}	// END ELSE hasToSkipCarriers
		}


		/** ===================  */
		/** === SECOND STEP ===  */
		/** ===================  */

		ArrayList<NanoObj> objsToCheck2=new ArrayList<NanoObj>();


		switch (ObjFor2Check){
		case 0: objsToCheck2.addAll(node.getAllNanoObjs());

		break;
		case 1: objsToCheck2.addAll(node.getNodeObjs());

		break;
		case 2: objsToCheck2.addAll(node.getCarrierObjs());

		break;
		case 3: objsToCheck2.addAll(node.getGenericObjs());

		break;
		default: objsToCheck2.addAll(node.getAllNanoObjs());

		break;
		}

		int size2=objsToCheck2.size();

		if (size2>0){
			// Reorder first!!		
			HeapSorter h=new HeapSorter();
			try {
				h.sort(objsToCheck2);
			} catch (Exception e) {
				e.printStackTrace();
				System.out.println("");
				System.out.println(">>>  "+objsToCheck2.size());
				System.out.println("");
			}

			//Check Collision!!
			isFinish=false;


			// Check the collision with other domains!					
			try {			

				// FIXME - The next command is moved on AbstractDomain.nextSimulationStep, before the calls to Thread_CheckCollision(...)
				//List<AbstractDomain> domainToCheck=model.reorderDomain(domain);					

				for (int i=0;i<domainToCheck.size();i++){					
					isFinish=false;
					for (int j=0;j<size2 && !isFinish;j++){ 
						//if(objsToCheck.get(j) instanceof CarrierObj && objsToCheck.get(j).getDomain()==domain){		// FIXME Introduced the check that works only for the carriers!					
						//NanoObj n=objsToCheck2.get(j);
						n=objsToCheck2.get(j);
						if(n.getDomain()==domain){
							float Dplus=domainToCheck.get(i).getDistance()+domainToCheck.get(i).getBoundingRadius();
							float Dminus=domainToCheck.get(i).getDistance()-domainToCheck.get(i).getBoundingRadius();
							float Nplus=n.getAbsDistancePlusRadius();
							float Nminus=n.getAbsDistanceMinusRadius();

							if(Dminus>Nplus) { 
								// Do not overlap! (neither with the other following domains)
								isFinish=true;						

							} else if((Dminus<=Nminus && Nminus<=Dplus) || (Dminus>=Nminus && Nplus>=Dminus) || (Dplus<=Nplus && Nminus<=Dplus)) { 
								// Probable collision							
								try{
									domainToCheck.get(i).getCollisionStrategy().checkExternalCollisionBound(domainToCheck.get(i), n);

								}catch (Exception e){									
									e.printStackTrace();						
								}
							} 
						}
					}	
				}		

			} catch (Exception e1) {					
				e1.printStackTrace();
			}			
		}	
		synchronized(latch){
			latch.countDown();	
		}
	}


	// NOT USED
	public void run_ORIGINAL() {		
		ArrayList<NanoObj> objsToCheck=new ArrayList<NanoObj>();
		objsToCheck.addAll(node.getAllNanoObjs());	
		//objsToCheck=(node.getAllNanoObjs());	

		int size=objsToCheck.size();

		if (size>0){
			// Reorder first!!		
			HeapSorter h=new HeapSorter();
			try {
				h.sort(objsToCheck);
			} catch (Exception e) {
				e.printStackTrace();
				System.out.println("");
				System.out.println(">>>  "+objsToCheck.size());
				System.out.println("");
			}			

			AbstractCollisionDomainStrategy strategy=domain.getCollisionStrategy();		

			//Check Collision!!
			boolean isFinish=false;
			NanoObj n=null;

			for (int i=0;i<size;i++){
				n=objsToCheck.get(i);	

				// FIXME Ca++ - DEBUG DISABLED CARRIER COLLISION CHECKING...
				//if (!(n instanceof CarrierObj) ){

				if(n.getTimeToLive()<0){
					isFinish=true;
				}else{
					isFinish=false;
				}

				for (int j=i+1;j<size && !isFinish;j++){	
					//for (int j=i+1;j<size && !isFinish;j++){		
					try {						
						// FIXME BUGFIX for timeToLive=-1. No checks needed for killed objs!
						if(objsToCheck.get(j).getTimeToLive()<0){

						}else if(objsToCheck.get(j).getAbsDistancePlusRadius()>=n.getAbsDistanceMinusRadius()){
							model.detectCollision(objsToCheck.get(j),n);					

						} else{
							isFinish=true;
						}
					} catch (Exception e) {
						e.printStackTrace();
						System.out.println("");
						System.out.println(">>> SIZE: "+size+",  i: "+i+", j: "+j+"  -  "+n.toString());
						System.out.println("");
					}
				}
				//}
				strategy.checkInternalCollisionBound(n);
			}			

			// Check the collision with other domains!					
			try {			

				// FIXME - The next command is moved on AbstractDomain.nextSimulationStep, before the calls to Thread_CheckCollision(...)
				//List<AbstractDomain> domainToCheck=model.reorderDomain(domain);

				for (int i=0;i<domainToCheck.size();i++){					
					isFinish=false;
					for (int j=0;j<size && !isFinish;j++){ 
						//if(objsToCheck.get(j) instanceof CarrierObj && objsToCheck.get(j).getDomain()==domain){		// FIXME AGGIUNTO CONTROLLO SOLO PER LE CARRIER					
						if(objsToCheck.get(j).getDomain()==domain){
							float Dplus=domainToCheck.get(i).getDistance()+domainToCheck.get(i).getBoundingRadius();
							float Dminus=domainToCheck.get(i).getDistance()-domainToCheck.get(i).getBoundingRadius();
							float Nplus=objsToCheck.get(j).getAbsDistancePlusRadius();
							float Nminus=objsToCheck.get(j).getAbsDistanceMinusRadius();

							if(Dminus>Nplus) { 
								// Do not overlap! (neither with the other following domains)
								isFinish=true;						

							} else if((Dminus<=Nminus && Nminus<=Dplus) || (Dminus>=Nminus && Nplus>=Dminus) || (Dplus<=Nplus && Nminus<=Dplus)) { 
								// Probable collision							
								try{
									domainToCheck.get(i).getCollisionStrategy().checkExternalCollisionBound(domainToCheck.get(i), objsToCheck.get(j));

								}catch (Exception e){									
									e.printStackTrace();						
								}
							} 
						}
					}	
				}				

			} catch (Exception e1) {					
				e1.printStackTrace();
			}				
		}	
		synchronized(latch){					
			latch.countDown();	
		}
	}
}
