package engine;

import java.awt.Color;
import java.util.*;

import javax.swing.JFrame;

import simulator.CalciumSphericalDomain;
import simulator.PlateletVessel;
import simulator.RedBloodCell;
import simulator.Platelet;
import simulator.Macrophage;
import simulator.CubicDomain;
import simulator.CylindricDomain;
import simulator.CylindricPoreDomain;
import simulator.CylindricReceptorDomain;
import simulator.CalciumCell_TX;
import simulator.CalciumCell_RX;
import simulator.ShellCylindricDomain;
import simulator.ShellSphericalDomain;
import simulator.TCPLike_TEST_RX;

import MainLibrary.AbstractDomain;
import MainLibrary.AbstractEngine3D;
import MainLibrary.CarrierObj;
import MainLibrary.Manager;
import MainLibrary.NanoObj;
import MainLibrary.NodeObj;
import MainLibrary.InnerSphericalDomain;
import Octree.OctreeNode;

import com.threed.jpct.Camera;
import com.threed.jpct.Config;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.Matrix;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureManager;
import com.threed.jpct.World;


/** TODO IMPORTANT! Following the modification of the classes for the insertion of the GRID is necessary, 
 * each time it is added to the library a new type of AbstractDomain, 
 * add to  the Engine3D class a new method addWrapper3DDomain that instructs the 3D Engine 
 * how to create the object3d that represents the object in the simulator. 
 * In addition, you have to create the right method call inside the method addWrapper3DDomain!!
 */

public class Engine3D extends AbstractEngine3D implements Runnable{

	private List<Wrapper3D> wrappers = new ArrayList<Wrapper3D>();
	private List<WrapperDomain3D> domainWrappers = new ArrayList<WrapperDomain3D>();


	//For the drawSmallPoint method! Not required, but useful to remove older points!
	public List<Object3D> smallPoint = new ArrayList<Object3D>();

	public boolean followNano=false;
	public World world;
	public FrameBuffer buffer;
	public JFrame frame;
	public Matrix rot;

	public int frameWidth=800;
	public int frameHeight=600;	

	//Manages all cameras
	public ArrayList<Camera> cams=new ArrayList<Camera>();
	public SimpleVector posCam1 =new SimpleVector(868125,868125,878125);
	public SimpleVector lookCam1 =new SimpleVector(878125,878125,878125);
	public SimpleVector posCam2 =new SimpleVector(878125,888125,878125);
	public SimpleVector lookCam2 =new SimpleVector(878125,878125,878125);
	
	//public SimpleVector posCam3 =new SimpleVector(97615.19,1137679.0,112421.31);
	public SimpleVector posCam3 =new SimpleVector(888125,878125,878125);
	public SimpleVector lookCam3 =new SimpleVector(878125,878125,878125);	
	
	public SimpleVector posCam4 =new SimpleVector(878125,878125,888125);
	public SimpleVector lookCam4 =new SimpleVector(878125,878125,878125);
	
	public SimpleVector posCam5 =new SimpleVector(104123,-944250,98601);
	public SimpleVector lookCam5 =new SimpleVector(0,100000,0);	
	public SimpleVector posCam6 =new SimpleVector(104123,102515,98601);
	public SimpleVector lookCam6 =new SimpleVector(0,1000000,0);	
	public SimpleVector posCam7 =new SimpleVector(2000000,0,100000);
	public SimpleVector lookCam7 =new SimpleVector(100000,100000,100000);
	public SimpleVector posCam8 =new SimpleVector(100000,-1450000,100000);
	public SimpleVector lookCam8 =new SimpleVector(100000,100000,100000);
	
	public SimpleVector posCam9 =new SimpleVector(100000,-2450000,100000);
	public SimpleVector lookCam9 =new SimpleVector(100000,100000,100000);

	public Engine3D(){
		int numberOfProcs = Runtime.getRuntime().availableProcessors();
		Config.useMultipleThreads = numberOfProcs > 1;
		Config.useMultiThreadedBlitting = numberOfProcs > 1;
		Config.loadBalancingStrategy = 0;  // 1 = enable
		Config.maxNumberOfCores = numberOfProcs;
		Config.lightMul = 1;
		Config.mtDebug = true;	
		Config.glAvoidTextureCopies = true;
		Config.saveMemory=true;
		Config.maxPolysVisible = 512000; //512000;  
		Config.useMultipleThreads=true;
		Config.fadeoutLight=false;	

		world = new World();
		world.setClippingPlanes(1, 9000000); //500000		
		cams.add(world.getCamera());

		Camera tempCam=new Camera();			
		tempCam.setPosition(posCam1);
		tempCam.lookAt(lookCam1);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam2);
		tempCam.lookAt(lookCam2);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam3);			
		tempCam.lookAt(lookCam3);
		cams.add(tempCam);

		tempCam=new Camera();			
		tempCam.setPosition(posCam4);
		tempCam.lookAt(lookCam4);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam5);
		tempCam.lookAt(lookCam5);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam6);			
		tempCam.lookAt(lookCam6);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam7);			
		tempCam.lookAt(lookCam7);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam8);			
		tempCam.lookAt(lookCam8);
		cams.add(tempCam);
		
		tempCam=new Camera();
		tempCam.setPosition(posCam9);			
		tempCam.lookAt(lookCam9);
		cams.add(tempCam);

		createAxis(100000,100000,100000,(float)Math.PI/2,0,0,Color.green,20000); // Z axis
		createAxis(100000,100000,100000,0,(float)Math.PI/2,0,Color.ORANGE,20000); // Y axis
		createAxis(100000,100000,100000,0,0,(float)Math.PI/2,Color.BLACK,20000); // X	axis			
	}

	public Engine3D(Manager m){
		super(m);
		int numberOfProcs = Runtime.getRuntime().availableProcessors();
		Config.useMultipleThreads = numberOfProcs > 1;
		Config.useMultiThreadedBlitting = numberOfProcs > 1;
		Config.loadBalancingStrategy = 0;  // 1 = enable
		Config.maxNumberOfCores = numberOfProcs;
		Config.lightMul = 1;
		Config.mtDebug = true;		
		Config.glAvoidTextureCopies = true;
		Config.saveMemory=true;
		Config.maxPolysVisible = 512000; //512000;  
		Config.useMultipleThreads=true;
		Config.fadeoutLight=false;	

		world = new World();
		world.setClippingPlanes(1, 9000000); //500000

		cams.add(world.getCamera());
		Camera tempCam=new Camera();			
		tempCam.setPosition(posCam1);
		tempCam.lookAt(lookCam1);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam2);
		tempCam.lookAt(lookCam2);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam3);			
		tempCam.lookAt(lookCam3);
		cams.add(tempCam);

		tempCam=new Camera();			
		tempCam.setPosition(posCam4);
		tempCam.lookAt(lookCam4);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam5);
		tempCam.lookAt(lookCam5);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam6);			
		tempCam.lookAt(lookCam6);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam7);			
		tempCam.lookAt(lookCam7);
		cams.add(tempCam);

		tempCam=new Camera();
		tempCam.setPosition(posCam8);			
		tempCam.lookAt(lookCam8);
		cams.add(tempCam);
		
		tempCam=new Camera();
		tempCam.setPosition(posCam9);			
		tempCam.lookAt(lookCam9);
		cams.add(tempCam);

		createAxis(100000,100000,100000,(float)Math.PI/2,0,0,Color.green,20000); // Z axis
		createAxis(100000,100000,100000,0,(float)Math.PI/2,0,Color.ORANGE,20000); // Y axis
		createAxis(100000,100000,100000,0,0,(float)Math.PI/2,Color.BLACK,20000); // X axis

	}

	public void run() {   
		frameWidth=frameSize[0];
		frameHeight=frameSize[1];

		getMyCam(0).setPosition(cameraCenter[0],cameraCenter[1],cameraCenter[2]);
		getMyCam(0).lookAt(cameraLookAt);
		world.setAmbientLight(lights[0],lights[1],lights[2]);

		frame=new JFrame("3D View");
		frame.setSize(frameWidth, frameHeight);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		frame.addMouseWheelListener(new MyMouseListener(this));
		frame.addMouseMotionListener(new MyMouseListener(this));
		frame.addMouseListener(new MyMouseListener(this));
		frame.addWindowListener(new MyWindowListener(this));
		frame.addKeyListener(new MyKeyListener(this));

		newFrameBuffer(frame.getWidth(), frame.getHeight());

		while (true) {
			frame.setTitle(title);
			if(buffer.getOutputHeight()!=frame.getHeight() || buffer.getOutputWidth()!=frame.getWidth()){
				newFrameBuffer(frame.getWidth(),frame.getHeight());
			}
			try {				
				for(Wrapper3D w3d : wrappers){					
					w3d.draw3D();					
				}
				for(WrapperDomain3D w3d : domainWrappers)
					w3d.draw3D();	
				

				buffer.clear(java.awt.Color.blue);
				world.renderScene(buffer);
				world.draw(buffer);
				buffer.update();
				// LWJGL
				//buffer.displayGLOnly();				
				// END LWJGL
				buffer.display(frame.getGraphics());
				Thread.sleep(35);

			} catch (InterruptedException e) {

			}
			catch (Exception e) {

			}
		}
	}
	
	public synchronized void addWrapper3D(NanoObj p){
		float xf=(float)p.getCenter()[0];
		float yf=(float)p.getCenter()[1];
		float zf=(float)p.getCenter()[2];

		SimpleVector v0 =new SimpleVector(xf,yf,zf);
		Object3D obj3D = Primitives.getSphere((float)p.getRadius()*p.zoom3D);			
		obj3D.setCulling(false);
		obj3D.setTexture("cell");		
		obj3D.setAdditionalColor(p.color);		
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);	
		obj3D.build();
		
		obj3D.translate(v0);
		world.addObject(obj3D);
		obj3D.compileAndStrip();

		//Create new Wrapper3D and add to the list
		wrappers.add(new Wrapper3D(p, obj3D, this));
	}

	public synchronized void addWrapper3D(NodeObj p){
		float xf=(float)p.getCenter()[0];
		float yf=(float)p.getCenter()[1];
		float zf=(float)p.getCenter()[2];

		SimpleVector v0 =new SimpleVector(xf,yf,zf);
		Object3D obj3D = Primitives.getSphere((float)p.getRadius()*p.zoom3D);			
		obj3D.setCulling(false);

		// Preload of textures!
		if(!TextureManager.getInstance().containsTexture("platelet")) TextureManager.getInstance().addTexture("platelet", new Texture(System.getProperty("user.home")+"/texture/piatrinaROT.png"));
		if(!TextureManager.getInstance().containsTexture("redBloodCell")) TextureManager.getInstance().addTexture("redBloodCell", new Texture(System.getProperty("user.home")+"/texture/RBC.png"));
		if(!TextureManager.getInstance().containsTexture("macrophage")) TextureManager.getInstance().addTexture("macrophage", new Texture(System.getProperty("user.home")+"/texture/macrophage.png"));
		if(!TextureManager.getInstance().containsTexture("generic")) TextureManager.getInstance().addTexture("generic", new Texture(System.getProperty("user.home")+"/texture/textureNode.jpg"));

		obj3D.calcTextureWrapSpherical();
		
		if (p instanceof RedBloodCell){
			obj3D.setTexture("redBloodCell");
		}
		if (p instanceof Macrophage){
			obj3D.setTexture("macrophage");
		}
		if (p instanceof Platelet || p instanceof PlateletVessel){
			obj3D.setTexture("platelet");
		}
		if (p instanceof CalciumCell_TX){
			obj3D.setTexture("platelet");
		}
		
		if (p instanceof CalciumCell_RX){
			obj3D.setTexture("generic");
		}
		
		if (p instanceof TCPLike_TEST_RX){
			obj3D.setTexture("macrophage");
		}		
		

		obj3D.setAdditionalColor(p.color);		
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);	
		obj3D.build();		
		obj3D.translate(v0);
		world.addObject(obj3D);
		obj3D.compileAndStrip();

		//Create new Wrapper3D and add to the list
		wrappers.add(new Wrapper3D(p, obj3D, this));
	}

	public synchronized void addWrapper3D(CarrierObj p){
		SimpleVector v0 =new SimpleVector(p.getCenter()[0],p.getCenter()[1],p.getCenter()[2]);
		Object3D obj3D = Primitives.getBox((float)p.getRadius()*p.zoom3D, 1);
		
		obj3D.setAdditionalColor(p.color);
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);
		obj3D.build();		
		obj3D.translate(v0);	
		world.addObject(obj3D);
		obj3D.compileAndStrip();

		//Create new Wrapper3D and add to the list
		wrappers.add(new Wrapper3D(p, obj3D, this));
	}
	
	public synchronized void changeWrapper3D(NanoObj obj){
		for(Wrapper3D wrapper: wrappers){
			if(wrapper.getObj() == obj){
				world.removeObject(wrapper.getObject3d());
				wrappers.remove(wrapper);
				addWrapper3D(obj);
			}
		}	   
	}

	public synchronized void changeWrapper3D(NodeObj obj){
		for(Wrapper3D wrapper: wrappers){
			if(wrapper.getObj() == obj){
				world.removeObject(wrapper.getObject3d());
				wrappers.remove(wrapper);
				addWrapper3D(obj);
			}
		}	   
	}

	public synchronized void changeWrapper3D(CarrierObj obj){
		for(Wrapper3D wrapper: wrappers){
			if(wrapper.getObj() == obj){
				world.removeObject(wrapper.getObject3d());
				wrappers.remove(wrapper);
				addWrapper3D(obj);
			}
		}	   
	}

	public synchronized void changeObjColor(NanoObj obj ,Color color){
		for(Wrapper3D wrapper: wrappers){
			if(wrapper.getObj() == obj){			   
				wrapper.changeObj3DColor(color);
			}
		}	   
	}

	public synchronized void addWrapper3DDomain(CylindricDomain c){

		float xf=(float)c.getAbsCenter()[0];
		float yf=(float)c.getAbsCenter()[1];
		float zf=(float)c.getAbsCenter()[2];

		SimpleVector v0 =new SimpleVector(xf,yf,zf);
		Object3D obj3D = Primitives.getCylinder(90,(float)c.getRadius(),(float)((c.getHeight()/2)/c.getRadius()));				
		obj3D.setCulling(false);		
		obj3D.setAdditionalColor(Color.WHITE);		
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);

		if(!TextureManager.getInstance().containsTexture("cylindric")) TextureManager.getInstance().addTexture("cylindric", new Texture(System.getProperty("user.home")+"/texture/textureVetro5.jpg"));
		obj3D.calcTextureWrapSpherical();
		obj3D.invert();	   
		obj3D.setTexture("cylindric");
		
		//obj3D.setTransparency(3);
		
		obj3D.translate(v0);	
		world.addObject(obj3D);	
		obj3D.compileAndStrip();
		obj3D.build();
		domainWrappers.add(new WrapperDomain3D(c,obj3D,this));
	}

	public synchronized void addWrapper3DDomain(CylindricPoreDomain c){

		float xf=(float)c.getAbsCenter()[0];
		float yf=(float)c.getAbsCenter()[1];
		float zf=(float)c.getAbsCenter()[2];

		SimpleVector v0 =new SimpleVector(xf,yf,zf);
		Object3D obj3D = Primitives.getCylinder(90,(float)c.getRadius(),(float)((c.getHeight()/2)/c.getRadius()));				
		obj3D.setCulling(false);		
		obj3D.setAdditionalColor(Color.WHITE);		
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);

		if(!TextureManager.getInstance().containsTexture("cylindric")) TextureManager.getInstance().addTexture("cylindric", new Texture(System.getProperty("user.home")+"/texture/textureBlood1.jpg"));
		obj3D.invert();	   
		obj3D.setTexture("cylindric");		
		obj3D.translate(v0);	
		world.addObject(obj3D);	
		obj3D.compileAndStrip();
		obj3D.build();
		domainWrappers.add(new WrapperDomain3D(c,obj3D,this));
	}

	public synchronized void addWrapper3DDomain(CylindricReceptorDomain c){

		float xf=(float)c.getAbsCenter()[0];
		float yf=(float)c.getAbsCenter()[1];
		float zf=(float)c.getAbsCenter()[2];

		SimpleVector v0 =new SimpleVector(xf,yf,zf);
		Object3D obj3D = Primitives.getCylinder(90,(float)c.getRadius(),(float)((c.getHeight()/2)/c.getRadius()));				
		obj3D.setCulling(false);		
		obj3D.setAdditionalColor(Color.BLUE);		
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);

		if(!TextureManager.getInstance().containsTexture("cylindric")) TextureManager.getInstance().addTexture("cylindric", new Texture(System.getProperty("user.home")+"/texture/textureBlood1.jpg"));
		
		obj3D.invert();	   		
		obj3D.translate(v0);	
		world.addObject(obj3D);	
		obj3D.compileAndStrip();
		obj3D.build();
		domainWrappers.add(new WrapperDomain3D(c,obj3D,this));
	}
	
	public synchronized void addWrapper3DDomain(ShellCylindricDomain c){

		float xf=(float)c.getAbsCenter()[0];
		float yf=(float)c.getAbsCenter()[1];
		float zf=(float)c.getAbsCenter()[2];

		SimpleVector v0 =new SimpleVector(xf,yf,zf);
		Object3D obj3D = Primitives.getCylinder(90,(float)c.getRadius(),(float)((c.getHeight()/2)/c.getRadius()));				
		obj3D.setCulling(false);		
		obj3D.setAdditionalColor(Color.WHITE);		
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);
		obj3D.rotateX((float) (Math.PI/2));
		obj3D.rotateY((float) (Math.PI/2));
		
		if(!TextureManager.getInstance().containsTexture("cylindric")) TextureManager.getInstance().addTexture("cylindric", new Texture(System.getProperty("user.home")+"/texture/textureVetro5.jpg"));
		obj3D.calcTextureWrapSpherical();
		obj3D.invert();	   
		obj3D.setTexture("cylindric");
		
		//obj3D.setTransparency(3);
		
		obj3D.translate(v0);	
		world.addObject(obj3D);	
		obj3D.compileAndStrip();
		obj3D.build();
		domainWrappers.add(new WrapperDomain3D(c,obj3D,this));
	}


	public synchronized void addWrapper3DDomain(InnerSphericalDomain c){

		float xf=(float)c.getAbsCenter()[0];
		float yf=(float)c.getAbsCenter()[1];
		float zf=(float)c.getAbsCenter()[2];

		SimpleVector v0 =new SimpleVector(xf,yf,zf);
		Object3D obj3D = Primitives.getSphere(c.getBoundingRadius());				
		obj3D.setCulling(false);
		obj3D.setAdditionalColor(Color.red);		
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);	
		obj3D.build();		
		obj3D.translate(v0);
		world.addObject(obj3D);
		obj3D.compileAndStrip();	

		domainWrappers.add(new WrapperDomain3D(c,obj3D,this));
	}
	
	public synchronized void addWrapper3DDomain(ShellSphericalDomain c){

		float xf=(float)c.getAbsCenter()[0];
		float yf=(float)c.getAbsCenter()[1];
		float zf=(float)c.getAbsCenter()[2];

		SimpleVector v0 =new SimpleVector(xf,yf,zf);
		Object3D obj3D = Primitives.getSphere(c.getBoundingRadius());				
		obj3D.setCulling(false);
		obj3D.setAdditionalColor(Color.red);
		obj3D.setTransparency(1);
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);	
		obj3D.build();		
		obj3D.translate(v0);
		world.addObject(obj3D);
		obj3D.compileAndStrip();	
		
		/*
		SimpleVector d=c.getParabolicVersor();		
		SimpleVector xx=new SimpleVector(1,0,0);
		SimpleVector yy=new SimpleVector(0,1,0);
		SimpleVector zz=new SimpleVector(0,0,1);		
		float ax=d.calcAngle(xx);
		float ay=d.calcAngle(yy);
		float az=d.calcAngle(zz);		
		createAxis(xf,yf,zf,ax,ay,az,Color.green,20000); // Z axis
		*/
		
		
		
		domainWrappers.add(new WrapperDomain3D(c,obj3D,this));
	}
	
	public synchronized void addWrapper3DDomain(CalciumSphericalDomain c){

		float xf=(float)c.getAbsCenter()[0];
		float yf=(float)c.getAbsCenter()[1];
		float zf=(float)c.getAbsCenter()[2];

		//float bb=c.getBoundingRadius();		
		
		SimpleVector v0 =new SimpleVector(xf,yf,zf);
		Object3D obj3D = Primitives.getSphere(c.getBoundingRadius());				
		obj3D.setCulling(false);
		obj3D.setAdditionalColor(Color.cyan);
		obj3D.setTransparency(1);
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);	
		
		if(!TextureManager.getInstance().containsTexture("sphericalCell")) TextureManager.getInstance().addTexture("sphericalCell", new Texture(System.getProperty("user.home")+"/texture/macrophage-BW.png"));
		obj3D.setTexture("sphericalCell");
		
		obj3D.build();		
		obj3D.translate(v0);
		world.addObject(obj3D);
		obj3D.compileAndStrip();

		domainWrappers.add(new WrapperDomain3D(c,obj3D,this));
	}

	public synchronized void addWrapper3DRootCubicDomain(float[] center, float halfSide){

		float xf=center[0];
		float yf=center[1];
		float zf=center[2];

		SimpleVector v0 =new SimpleVector(xf,yf,zf);		
		Object3D obj3D = Primitives.getCube(halfSide);
		obj3D.setCulling(false);
		obj3D.setAdditionalColor(Color.yellow);		
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);

		if(!TextureManager.getInstance().containsTexture("cube")) TextureManager.getInstance().addTexture("cube", new Texture(System.getProperty("user.home")+"/texture/textureVetro4.jpg"));
		obj3D.setTexture("cube");
		float yr=(float) (Math.PI/4);
		obj3D.rotateY(yr);
		obj3D.build();
		obj3D.translate(v0);	
		world.addObject(obj3D);	
		obj3D.compileAndStrip();
	}

	public synchronized void addWrapper3DDomain(CubicDomain c){

		float xf=(float)c.getAbsCenter()[0];
		float yf=(float)c.getAbsCenter()[1];
		float zf=(float)c.getAbsCenter()[2];

		SimpleVector v0 =new SimpleVector(xf,yf,zf);
		Object3D obj3D = Primitives.getCube(c.getHalfSide());
		obj3D.setCulling(false);		
		obj3D.setAdditionalColor(Color.white);		
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);
		if(!TextureManager.getInstance().containsTexture("cube")) TextureManager.getInstance().addTexture("cube", new Texture(System.getProperty("user.home")+"/texture/huvec.jpg"));

		obj3D.setTexture("cube");
		obj3D.setTransparency(1);
		
		SimpleVector yy =new SimpleVector(0,1,0);			
		float yr=(float) (yy.calcAngle(c.getDirectionP())+Math.PI/4);		
		obj3D.rotateY(yr);
		obj3D.build();		
		obj3D.translate(v0);	
		world.addObject(obj3D);				

		obj3D.compileAndStrip();
		domainWrappers.add(new WrapperDomain3D(c,obj3D,this));
				
		/*
		// =======================================
		// ==== SHOW BOUNDING SPHERE =============
		// =======================================		
		Object3D boundingObj3D=null;

		float xfb=(float)c.getAbsCenter()[0];
		float yfb=(float)c.getAbsCenter()[1];
		float zfb=(float)c.getAbsCenter()[2];

		SimpleVector v0b =new SimpleVector(xfb,yfb,zfb);
		boundingObj3D = Primitives.getSphere(c.getBoundingRadius());				
		boundingObj3D.setCulling(false);
		boundingObj3D.setTexture("cube");
		boundingObj3D.setAdditionalColor(Color.YELLOW);	
		boundingObj3D.setTransparency(4);
		boundingObj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);
		boundingObj3D.build();
		boundingObj3D.translate(v0b);	
		boundingObj3D.compileAndStrip();
		world.addObject(boundingObj3D);		
		// =======================================
		*/
	}
	
	public synchronized void addWrapper3DOctree(OctreeNode c){

		float xf=(float)c.x;
		float yf=(float)c.y;
		float zf=(float)c.z;

		SimpleVector v0 =new SimpleVector(xf,yf,zf);
		Object3D obj3D = Primitives.getCube(c.side);
		obj3D.setCulling(false);		
		obj3D.setAdditionalColor(Color.white);		
		obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);
	
		obj3D.setTransparency(1);		
		
		obj3D.build();		
		obj3D.translate(v0);	
		world.addObject(obj3D);				

		obj3D.compileAndStrip();
		
		/*
		// =======================================
		// ==== SHOW BOUNDING SPHERE =============
		// =======================================		
		Object3D boundingObj3D=null;

		float xfb=(float)c.getAbsCenter()[0];
		float yfb=(float)c.getAbsCenter()[1];
		float zfb=(float)c.getAbsCenter()[2];

		SimpleVector v0b =new SimpleVector(xfb,yfb,zfb);
		boundingObj3D = Primitives.getSphere(c.getBoundingRadius());				
		boundingObj3D.setCulling(false);
		boundingObj3D.setTexture("cube");
		boundingObj3D.setAdditionalColor(Color.YELLOW);	
		boundingObj3D.setTransparency(4);
		boundingObj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);
		boundingObj3D.build();
		boundingObj3D.translate(v0b);	
		boundingObj3D.compileAndStrip();
		world.addObject(boundingObj3D);		
		// =======================================
		*/
	}

	@Override
	public void addWrapper3DDomain(AbstractDomain d) {
		if(d instanceof ShellCylindricDomain){
			addWrapper3DDomain((ShellCylindricDomain)d);
		}else if(d instanceof CylindricDomain){
			addWrapper3DDomain((CylindricDomain)d);
		}else if(d instanceof InnerSphericalDomain){
			addWrapper3DDomain((InnerSphericalDomain)d);
		}else if(d instanceof CubicDomain){
			addWrapper3DDomain((CubicDomain)d);
		}else if(d instanceof CylindricPoreDomain){
			addWrapper3DDomain((CylindricDomain)d);
		}else if(d instanceof CylindricReceptorDomain){
			addWrapper3DDomain((CylindricDomain)d);
		}else if(d instanceof CalciumSphericalDomain){
			addWrapper3DDomain((CalciumSphericalDomain)d);
		}else if(d instanceof ShellSphericalDomain){
			addWrapper3DDomain((ShellSphericalDomain)d);
		}
	}

	@Override
	public void changeWrapper3DDomain(AbstractDomain d) {	   
		int size=domainWrappers.size();
		boolean isFinished=false;		
		for (int i=0;i<size && !isFinished;i++){
			WrapperDomain3D wrapper=domainWrappers.get(i);
			if(wrapper.getDomain() == d){
				world.removeObject(wrapper.getObject3d());				
				domainWrappers.remove(wrapper);
				addWrapper3DDomain(d);
				isFinished=true;
			}
		}	   
	}

	@Override
	public void changeDomainColor(AbstractDomain d, Color color) {
		for(WrapperDomain3D wrapper: domainWrappers){
			if(wrapper.getDomain() == d){			   
				wrapper.changeObj3DColor(color);
			}
		}	
	}  

	public synchronized void removeWrapper3d(Wrapper3D w){
		wrappers.remove(w);
	}

	public void removeWrapperDomain3d(WrapperDomain3D w){
		domainWrappers.remove(w);
	}

	public void setManager(Manager m){
		this.m=m;
	}


	public Camera getMyCam(int i){
		return cams.get(i);
	}

	public void createPlane(float x,float y, float z, float rx, float ry, float rz, Color c,float s){
		Object3D plane;
		SimpleVector v0 =new SimpleVector(x,y,z);
		plane=Primitives.getPlane(10,s);
		plane.setCulling(false);
		plane.setAdditionalColor(c);		
		plane.setEnvmapped(Object3D.ENVMAP_ENABLED);	
		plane.build();
		plane.setCenter(v0);
		plane.setOrigin(v0);
		plane.rotateX(rx);
		plane.rotateY(ry);
		plane.rotateZ(rz);
		world.addObject(plane);
		plane.compileAndStrip();	
		plane.setTransparency(1);
	}
	

	public void customObj3D(float[] coord,float[] uvs, int[] indices, int texture ){
		Object3D oo=new Object3D(coord,uvs, indices, texture);
		oo.setEnvmapped(Object3D.ENVMAP_ENABLED);	
		oo.build();
		world.addObject(oo);
		oo.compileAndStrip();	
	}

	public void createAxis(float x,float y, float z, float rx, float ry, float rz, Color c,float s){
		Object3D xAxis;
		SimpleVector v0 =new SimpleVector(x,y,z);
		xAxis=Primitives.getBox(5,s);  
		xAxis.setCulling(false);
		xAxis.setAdditionalColor(c);		
		xAxis.setEnvmapped(Object3D.ENVMAP_ENABLED);	
		xAxis.build();
		xAxis.setCenter(v0);
		xAxis.setOrigin(v0);
		xAxis.rotateX(rx);
		xAxis.rotateY(ry);
		xAxis.rotateZ(rz);
		world.addObject(xAxis);
		xAxis.compileAndStrip();
	}

	public void createBox(float x,float y, float z, float rx, float ry, float rz, Color c,float s){
		Object3D xAxis;
		SimpleVector v0 =new SimpleVector(x,y,z);
		xAxis=Primitives.getBox(s,1);
		xAxis.setCulling(false);
		xAxis.setAdditionalColor(c);		
		xAxis.setEnvmapped(Object3D.ENVMAP_ENABLED);	
		xAxis.build();
		xAxis.setCenter(v0);
		xAxis.setOrigin(v0);
		xAxis.rotateX(rx);
		xAxis.rotateY(ry);
		xAxis.rotateZ(rz);
		world.addObject(xAxis);
		xAxis.compileAndStrip();
		xAxis.setTransparency(1);
	}

	public void createCylinder(float x,float y, float z, Color c,float s){
		Object3D xAxis;
		SimpleVector v0 =new SimpleVector(x,y,z);
		xAxis=Primitives.getCylinder(s);
		xAxis.setCulling(false);
		xAxis.setAdditionalColor(c);		
		xAxis.setEnvmapped(Object3D.ENVMAP_ENABLED);	
		xAxis.build();
		xAxis.setCenter(v0);
		xAxis.setOrigin(v0);		
		world.addObject(xAxis);
		xAxis.compileAndStrip();
		xAxis.setTransparency(1);
	}
	
	public void createPoint(float x,float y, float z, Color c){
		Object3D xAxis;
		SimpleVector v0 =new SimpleVector(x,y,z);
		xAxis=Primitives.getBox(100, 1);
		xAxis.setCulling(false);
		xAxis.setAdditionalColor(c);		
		xAxis.setEnvmapped(Object3D.ENVMAP_ENABLED);	
		xAxis.build();
		xAxis.setCenter(v0);
		xAxis.setOrigin(v0);
		world.addObject(xAxis);
		xAxis.compileAndStrip();
	}

	public void newFrameBuffer(float w, float h){		
		buffer = new FrameBuffer((int)w,(int)h, FrameBuffer.SAMPLINGMODE_GL_AA_4X ); //.SAMPLINGMODE_NORMAL
		/*
		buffer = new FrameBuffer((int)w,(int)h, FrameBuffer.SAMPLINGMODE_NORMAL ); //.SAMPLINGMODE_NORMAL		
		buffer.disableRenderer(IRenderer.RENDERER_SOFTWARE);
		buffer.enableRenderer(IRenderer.RENDERER_OPENGL);
		*/
	}

	public void setCameraLookAt(float x,float y,float z){
		SimpleVector vc =new SimpleVector(x,y,z);
		world.getCamera().lookAt(vc);
	}

	public void rotateCameraX(float dx){
		Camera cam = world.getCamera();
		cam.rotateY(-0.01f*dx);
	}

	public void rotateCameraY(float dy){		
		Camera cam = world.getCamera();		
		cam.rotateX(0.01f*dy);
	}	

	public void moveCameraLeft(){
		Camera cam = world.getCamera();
		cam.moveCamera(Camera.CAMERA_MOVELEFT, 1);
	}

	public void moveCameraRight(){
		Camera cam = world.getCamera();
		cam.moveCamera(Camera.CAMERA_MOVERIGHT, 1);
	}
	public void moveCameraUp(){
		Camera cam = world.getCamera();
		cam.moveCamera(Camera.CAMERA_MOVEUP, 1);
	}
	public void moveCameraDown(){
		Camera cam = world.getCamera();
		cam.moveCamera(Camera.CAMERA_MOVEDOWN, 1);
	}
	public void moveCameraIn(){
		Camera cam = world.getCamera();
		cam.moveCamera(Camera.CAMERA_MOVEIN, 1);
	}
	public void moveCameraOut(){
		Camera cam = world.getCamera();
		cam.moveCamera(Camera.CAMERA_MOVEOUT, 1);
	}	

	public Object3D getWrapperDomain3D(AbstractDomain d){
		int size=domainWrappers.size();
		boolean isFinished=false;
		WrapperDomain3D wrapper=null;
		for (int i=0;i<size && !isFinished;i++){
			wrapper=domainWrappers.get(i);
			if(wrapper.getDomain() == d){			  
				isFinished=true;
			}
		}
		return wrapper.getObject3d();	 	
	}
	
	@Override
	public synchronized void removeWrapper3D(NanoObj obj) {	
		int size=wrappers.size();
		boolean isFinished=false;		 
		for (int i=0;i<size && !isFinished;i++){
			Wrapper3D wrapper=wrappers.get(i);
			if(wrapper.getObj() == obj){
				world.removeObject(wrapper.getObject3d());				
				wrappers.remove(wrapper);
				isFinished=true;
			}
		}		
	}

	@Override
	public void removeWrapper3D(AbstractDomain d) {
		int size=domainWrappers.size();
		boolean isFinished=false;		
		for (int i=0;i<size && !isFinished;i++){
			WrapperDomain3D wrapper=domainWrappers.get(i);
			if(wrapper.getDomain() == d){
				world.removeObject(wrapper.getObject3d());				
				domainWrappers.remove(wrapper);
				isFinished=true;
			}
		}	
	}	
}