package simulator;

import java.awt.Color;
//import java.math.BigDecimal;
import java.lang.reflect.Constructor;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import MainLibrary.AbstractDomain;
import MainLibrary.AbstractThreadingStrategy;
import MainLibrary.CarrierObj;
import MainLibrary.LigandObserver;
import MainLibrary.LinearMotion;
import MainLibrary.MobilityModel;
import MainLibrary.NanoObj;
import MainLibrary.NoMotion;
import MainLibrary.NodeObj;
import MainLibrary.RootDomain;

import com.threed.jpct.Matrix;
import com.threed.jpct.Object3D;
import com.threed.jpct.SimpleVector;
import flanagan.complex.Complex;
import flanagan.math.Polynomial;


public class BloodFlowModel extends MobilityModel{

	private final static double k=1.381*Math.pow(10, -23);

	// FIXME This attributes bypass the global configuration. Fix them!! 
	//public float plasmaDensity = 1025;               	// Density of plasma, [kg/m^3]
	public float plasmaViscosity = (float) 0.00130;  	// Viscosity of plasma at 37 Celsius degree, [Pa*s]

	public int plateletLevel=0;

	public float meanFlowVel = -1;
	public int ECReceptor=2000;
	// END attributes

	public int nodeCount=0;
	public int HUVECCount=0;
	public boolean isTheFirst=true;	

	/** =========  Emission Cells for Blood Vessels ========= */	
	//public int totalDesideredCells=1051;	// For radius = 30 um
	public int totalDesideredCells=4204;	// For radius = 60 um ?
	public long createdCells=0;
	public long notCreatedCells=0;
	public int step=0;
	public int cycle=0;
	public boolean hasCreatedTxNode=false;	
	/** =========  End Emission Parameters  ========= */


	/** === Diffusion Coefficient === */
	float powMinusU=0;	
	float constantTerm1=0;		
	float constantTerm2=0;
	float constantTerm3=0;
	boolean hasInitialized=false;
	/** === END Diffusion Coefficient === */


	/** DEBUG ONLY */
	//public float previuosVelocityRegime=0;
	public long nanoCollision=0;

	private boolean hasToMonitor=false;	

	protected int domainStrategyType=1;

	private transient AbstractThreadingStrategy threadingStrategy = new NewFlowThreadingStrategy(this);		

	private final int maxThreads = Runtime.getRuntime().availableProcessors();				
	private ExecutorService executorThreads = Executors.newFixedThreadPool(maxThreads);		


	public BloodFlowModel(){

	}

	public BloodFlowModel(RootDomain d){
		super(d);
	}

	/*
	// FIXME The OLD implementation
	public float getDiffusionCoefficient_OLD_OK(NanoObj p){
		float eta=p.getManager().configWrapper.viscosity;		
		float T=p.getManager().configWrapper.T;
		float D=(float) (k*T/(6*Math.PI*eta*p.getRadius()*Math.pow(10, (p.getManager().configWrapper.u))));
		float sigma2=(float) (6*D/3*p.getManager().get_dt()*Math.pow(10, (p.getManager().configWrapper.u)));
		return sigma2=(float) ((float) Math.sqrt(sigma2)*Math.pow(10, (-p.getManager().configWrapper.u)));
	}
	 */


	//FIXME The New implementation
	public void initDiffusionCoeff(float nano_dt){
		float eta=getManager().configWrapper.viscosity;		
		float T=getManager().configWrapper.T;	
		float u=getManager().configWrapper.u;
		powMinusU=(float) Math.pow(10, (-u));

		constantTerm1=(float) (k*T/(6*Math.PI*eta*Math.pow(10, u)));		
		//constantTerm2=(float) (2*getManager().get_dt()*Math.pow(10, u));		// OLD BEFORE THE 2 TIMESTEPS!!!  -  31/07/2015		
		constantTerm2=(float) (2*nano_dt*Math.pow(10, u));		// OLD BEFORE THE 2 TIMESTEPS!!!  -  31/07/2015
		
		constantTerm3=constantTerm1*constantTerm2;
		constantTerm3=(float) (Math.sqrt(constantTerm3)*powMinusU);

	}

	public float getDiffusionCoefficient(NanoObj p){		

		if(!hasInitialized){
			
			float nano_dt=1;
			
			if(p.getRadius()<100){
				nano_dt=getManager().get_dt();
			}else{
				nano_dt=getManager().configWrapper.timeUnit;
			}
			
			initDiffusionCoeff(nano_dt);
			hasInitialized=true;
		}			

		//float sigma2NEW=(float) (Math.sqrt(constantTerm3/p.getRadius())*powMinusU);
		float sigma2NEW=(float) (constantTerm3/Math.sqrt(p.getRadius()));
		return sigma2NEW;		
	}


	public void networkCreationForEndothelium_LongVessel(boolean hasToRestore) {

		if (!hasToRestore){

			// ==============================================
			//              NEW NETWORK CREATION
			// ==============================================

			System.out.println("-----------------------------------");
			System.out.println("Auto-Create Network... [ENDOTHELIUM]");
			System.out.println("-----------------------------------");
			System.out.println();
			System.out.println(">>>  Node TX release WITHOUT RECEPTORS!!!");
			System.out.println();			

			boolean useOctree=false;

			ArrayList<LigandObserver> observerForDomain=new ArrayList<LigandObserver>();
			observerForDomain.add(CD40LObserver.getInstance());

			//float height3=4000000;	// ORIGINAL
			float height3=(float) 8E6;		// BUG FIX??
			//float height3=2500000+88500+12500;  // vessel lenght
			float rad3=30000;  // Venule diameter 60um	ORIGINAL SIMULATION
			//float rad3=60000;  // Venule diameter 120um		REALI SIMULATION

			float[] cdCenter3=new float[3];
			cdCenter3[0]=100000;
			cdCenter3[1]=100000;
			cdCenter3[2]=100000;	

			CylindricStrategyBloodVessel cs=new CylindricStrategyBloodVessel(domainStrategyType);			
			CylindricBloodVesselDomain mainCylinder=new CylindricBloodVesselDomain(getManager().rootDomain, this, cs, true, false, height3, rad3,cdCenter3, useOctree);
			mainCylinder.ID="CylindricBloodVesselDomain";			

			// FIXME Only for debug!!!
			getManager().addDomainForExport(mainCylinder);

			SimpleVector directionN=new SimpleVector(1,0,0);
			mainCylinder.setDirectionN(directionN);			



			// Creation of cubes representing endothelial cells
			float huvecDiameter = 15000;  // 15 um
			float cylinderCircumference = (float) (2*Math.PI*rad3);
			int huvecNumber = Math.round(cylinderCircumference/huvecDiameter);
			float huvecSide = cylinderCircumference/huvecNumber;
			float apothem = (float) (rad3*Math.cos(Math.PI/huvecNumber));
			float huvecCenter = apothem + (huvecSide/2);

			SimpleVector dVersor = mainCylinder.getDirectionP().normalize();
			SimpleVector tempVector = null;
			if (dVersor.x <= dVersor.y && dVersor.x <= dVersor.z)
				tempVector = new SimpleVector(dVersor.x+0.5, dVersor.y, dVersor.z);
			else if (dVersor.y <= dVersor.x && dVersor.y <= dVersor.z)
				tempVector = new SimpleVector(dVersor.x, dVersor.y+0.5, dVersor.z);
			else if (dVersor.z <= dVersor.x && dVersor.z <= dVersor.y)
				tempVector = new SimpleVector(dVersor.x, dVersor.y, dVersor.z+0.5);

			SimpleVector huvecCenterVector = dVersor.calcCross(tempVector).normalize();
			huvecCenterVector.scalarMul(huvecCenter);

			float[] domainCenter = mainCylinder.getAbsCenter();
			SimpleVector domainCenterVector = new SimpleVector();
			domainCenterVector.x = domainCenter[0];
			domainCenterVector.y = domainCenter[1];
			domainCenterVector.z = domainCenter[2];	


			//float displacement=(float) ((1400000));		// ORIGINAL
			float displacement=-600000;						// BUG FIX??

			float[] layerCenter = mainCylinder.getAbsCenter();
			SimpleVector layerCenterVector = new SimpleVector();
			layerCenterVector.x = layerCenter[0];
			layerCenterVector.y = layerCenter[1]-displacement;
			layerCenterVector.z = layerCenter[2];


			int totalEC=0;
			//j=236 for 3.4 mm covered
			//j=128 Covers the relevant part of the venule under study
			//j=134 long
			//j=95 short venule		
			for (int j=1; j<236; j++) {  // from 1 to 236
				for (int i=0; i<huvecNumber; i++){
					huvecCenterVector.add(layerCenterVector);
					System.out.println("Layer Center - x: "+layerCenterVector.x+", y: "+layerCenterVector.y+", z: "+layerCenterVector.z+"  -  HUVEC Size: "+huvecSide);
					SimpleVector relativeHuvecCenterVector = new SimpleVector();
					relativeHuvecCenterVector.x = huvecCenterVector.x - domainCenter[0];
					relativeHuvecCenterVector.y = huvecCenterVector.y - domainCenter[1];
					relativeHuvecCenterVector.z = huvecCenterVector.z - domainCenter[2];
					float[] cellCenter = {relativeHuvecCenterVector.x, relativeHuvecCenterVector.y, relativeHuvecCenterVector.z};
					CubicReceptorStrategy cubic = new CubicReceptorStrategy(domainStrategyType);
					CubicReceptorDomain cube = new CubicReceptorDomain(mainCylinder, this, cubic, false, true, huvecSide/2, cellCenter, 2, ECReceptor, observerForDomain, false);
					huvecCenterVector.sub(layerCenterVector);
					System.out.println("relative Center - x: "+relativeHuvecCenterVector.x+", y: "+relativeHuvecCenterVector.y+", z: "+relativeHuvecCenterVector.z);

					SimpleVector cubeDirection = new SimpleVector(huvecCenterVector).normalize();
					cube.setDirectionP(cubeDirection);

					//FIXME Imposto gli altri assi per il receptorService sulla base del directionP del cubo e del cilindro!
					cube.setDirectionN(dVersor);


					Matrix rotation = new Matrix();
					float angle = (float) ((2*Math.PI)/huvecNumber);
					rotation.rotateAxis(dVersor, angle);
					huvecCenterVector.rotate(rotation);

					/*  DRAW CUBE RECEPTOR DOMAIN IN THE 3D SPACE!! */
					/*
					if (rootDomain.getManager().is3DEnable && i > 0) {
						Object3D o = rootDomain.getManager().engine.getWrapperDomain3D(cube);
						o.rotateAxis(dVersor, angle*i);
					}*/

					cube.rotationAngleP=angle*i;
					long start = System.nanoTime();
					cube.createReceptorsForHUVEC(cube.receptorQuantity, huvecNumber, i, layerCenterVector, dVersor);	
					long stop = System.nanoTime();
					System.out.println("RECEPTOR CREATED IN: "+(stop-start)/1000);					

					totalEC++;
				}

				dVersor.scalarMul(huvecSide*j);
				domainCenterVector.add(dVersor);
				layerCenterVector = new SimpleVector(domainCenterVector);
				layerCenterVector.y=layerCenterVector.y-displacement;

				dVersor = dVersor.normalize();
				domainCenter = mainCylinder.getAbsCenter();
				domainCenterVector = new SimpleVector(domainCenter);
			}
			System.out.println(">>> ENDOTHELIAL CELLS DISPOSED!");	
			System.out.println();		



			extraInfo="[Vessel Radius: "+rad3+" nm - Vessel Length: "+height3+" nm - Domain Type: "+domainStrategyType+
					" - Plasma Viscosity: "+plasmaViscosity+" - Mean Flow Velocity: "+meanFlowVel+" m/s - Endothelial cells: "+totalEC+" - Endothelial side: "+huvecSide+
					" - Endothelial Receptors: "+ECReceptor+" nm - Inelastic coeff. (walls): "+cs.inelasticCoefficient+" - Inelastic coeff. (walls longit): "+cs.inelasticCoefficientLongit+
					" - Inelastic coeff. (particles): "+ed+"] ";


			System.out.println();
			System.out.println("===========  GLOBAL PARAMETERS  ==========");
			System.out.println(">>>  Vessel Radius: "+rad3+" nm");
			System.out.println(">>>  Vessel Length: "+height3+" nm");
			System.out.println(">>>  Domain Type: "+domainStrategyType);
			System.out.println(">>>  Plasma Viscosity: "+plasmaViscosity);
			System.out.println(">>>  Mean Flow Velocity: "+meanFlowVel+" m/s");
			System.out.println(">>>  Endothelial cells: "+totalEC);
			System.out.println(">>>  Endothelial side: "+huvecSide+" nm");
			System.out.println(">>>  Endothelial Receptors: "+ECReceptor);
			System.out.println(">>>  Inelastic coeff. (walls): "+cs.inelasticCoefficient);
			System.out.println(">>>  Inelastic coeff. longit. (walls): "+cs.inelasticCoefficientLongit);
			System.out.println(">>>  Inelastic coeff. (particles): "+ed);
			System.out.println("===========================================");
			System.out.println();

			if(meanFlowVel<=0){
				try {
					throw new Exception();
				} catch (Exception e) {

					System.out.println(">>> ERROR - Simulation STOPPED!");
					System.out.println(">>> Set the  *Mean Flow Velocity [m/s]*  by command line. (eg:  -vmean0.0005)");					
					System.exit(0);
				}
			}

		}
		/*
		else{
			// ==============================================
			//           RESTORE SIMULATION (MySQL)
			// ==============================================

			System.out.println("-----------------------------------");
			System.out.println("Restore from backup... [ENDOTHELIUM]");
			System.out.println("-----------------------------------");

			// FIXME Only for DEBUG, then they have to be parametric!
			boolean useOctree=true;			
			int configWrapper=1;
			// END DEBUG, poi dovr essere parametrico!



			// CYLINDRIC DOMAIN
			Map<String,String> cylindricDomains=MySQLBackup_BloodVessel.getInstance().getQuery_CylindricDomain(configWrapper);
			AbstractDomain mainCylinder=null;	
			String mainCylinder_toString=null;

			float huvecDiameter = 0;  // 15 um
			float cylinderCircumference = 0;
			int huvecNumber = 0;
			float huvecSide = 0;
			float apothem = 0;
			float huvecCenter = 0;

			float radiusCyl=0;


			Iterator it0=cylindricDomains.keySet().iterator();			
			if(it0.hasNext()){	
				String element = (String) it0.next();
				String[] parsed=element.split("-",2);

				float[] relativeCenter=new float[3];			

				String mainDomain=cylindricDomains.get(parsed[0]+"-mainDomain");
				relativeCenter[0]=Float.valueOf(cylindricDomains.get(parsed[0]+"-x"));	
				relativeCenter[1]=Float.valueOf(cylindricDomains.get(parsed[0]+"-y"));
				relativeCenter[2]=Float.valueOf(cylindricDomains.get(parsed[0]+"-z"));	
				float height=Float.valueOf(cylindricDomains.get(parsed[0]+"-height"));	
				float radius=Float.valueOf(cylindricDomains.get(parsed[0]+"-radius"));	
				String myClass=cylindricDomains.get(parsed[0]+"-class");
				String type=cylindricDomains.get(parsed[0]+"-type");
				String toString=cylindricDomains.get(parsed[0]+"-toString");

				mainCylinder_toString=toString;

				radiusCyl=radius;

				String dP=cylindricDomains.get(parsed[0]+"-directionP");
				String dN=cylindricDomains.get(parsed[0]+"-directionN");
				String dO=cylindricDomains.get(parsed[0]+"-directionO");								


				boolean visible=false;
				if(Short.parseShort(cylindricDomains.get(parsed[0]+"-visible"))==1) visible=true;

				boolean export=false;
				if(Short.parseShort(cylindricDomains.get(parsed[0]+"-export"))==1) export=true;

				boolean octree=false;
				if(Short.parseShort(cylindricDomains.get(parsed[0]+"-octree"))==1) octree=true;

				int strategy=Integer.parseInt(cylindricDomains.get(parsed[0]+"-strategy"));


				CylindricStrategyBloodVessel cs=new CylindricStrategyBloodVessel(strategy);
				BloodFlowModel mainFlow=new BloodFlowModel(getManager().rootDomain);		



				String thisClass=toString.substring(0,toString.lastIndexOf("@"));
				String className=toString.substring(toString.lastIndexOf("@")+1,toString.length());

				try{

					Class domainClass = Class.forName(thisClass);		
					Class[] parameterTypes=new Class[]{AbstractDomain.class, MobilityModel.class, AbstractCollisionDomainStrategy.class, boolean.class, boolean.class, float.class, float.class, float[].class,  boolean.class};
					Constructor domainContructor=domainClass.getConstructor(parameterTypes);

					mainCylinder=(AbstractDomain) domainContructor.newInstance(getManager().rootDomain, mainFlow, cs, visible, export, height, radius, relativeCenter, octree);
					mainCylinder.ID=toString;


					// Sets the direction axes!
					float dpx=Float.valueOf(dP.substring(1,dP.indexOf(",")));
					float dpy=Float.valueOf(dP.substring(dP.indexOf(",")+1,dP.lastIndexOf(",")));
					float dpz=Float.valueOf(dP.substring(dP.lastIndexOf(",")+1,dP.lastIndexOf(")")));					
					mainCylinder.directionP=new SimpleVector(dpx,dpy,dpz);

					float dnx=Float.valueOf(dN.substring(1,dN.indexOf(",")));
					float dny=Float.valueOf(dN.substring(dN.indexOf(",")+1,dN.lastIndexOf(",")));
					float dnz=Float.valueOf(dN.substring(dN.lastIndexOf(",")+1,dN.lastIndexOf(")")));					
					mainCylinder.directionN=new SimpleVector(dnx,dny,dnz);

					float dox=Float.valueOf(dO.substring(1,dO.indexOf(",")));
					float doy=Float.valueOf(dO.substring(dO.indexOf(",")+1,dO.lastIndexOf(",")));
					float doz=Float.valueOf(dO.substring(dO.lastIndexOf(",")+1,dO.lastIndexOf(")")));					
					mainCylinder.directionO=new SimpleVector(dox,doy,doz);					
					// End of direction axes

					cylinderCircumference = (float) (2*Math.PI*radius);

				}catch (Exception e){
					e.printStackTrace();
				}		

				//CylindricBloodVesselDomain mainCylinder=new CylindricBloodVesselDomain(getManager().rootDomain, mainFlow,cs, visible, export, height, radius, relativeCenter, octree);
				//mainCylinder.ID="CylindricBloodVesselDomain";	
			}


			if (mainCylinder==null){				
				try{										
					throw new Exception();					
				}catch (Exception e){
					System.out.println(" ==========================================================");
					System.out.println(" ERROR - It cannot be possible to rebuild the Blood Vessel!");
					System.out.println(" ==========================================================");
					System.exit(0);
				}
			}


			// LIGAND OBSERVER
			ArrayList<LigandObserver> observerForDomain=new ArrayList<LigandObserver>();			
			MySQLBackup_BloodVessel.getInstance().ligandObserver.containsValue("CubicReceptorDomain");
			observerForDomain.add(CD40LObserver.getInstance());

			// CUBIC RECEPTOR DOMAIN

			Map<String,String> cubicReceptorDomains=MySQLBackup_BloodVessel.getInstance().getQuery_CubicReceptorDomain(configWrapper);

			Iterator it00=cubicReceptorDomains.keySet().iterator();		
			int i=0;

			Map<String,String> mapIter=new HashMap<String,String>();
			while(it00.hasNext()){
				String element = (String) it00.next();
				String[] parsed=element.split("-",2);
				mapIter.put(parsed[0], "ok");
			}

			Iterator it=mapIter.keySet().iterator();

			while(it.hasNext()){	

				String element = (String) it.next();
				String[] parsed=element.split("-",2);

				float[] relativeCenter=new float[3];


				String mainDomain=cubicReceptorDomains.get(parsed[0]+"-mainDomain");
				relativeCenter[0]=Float.valueOf(cubicReceptorDomains.get(parsed[0]+"-x"));	
				relativeCenter[1]=Float.valueOf(cubicReceptorDomains.get(parsed[0]+"-y"));
				relativeCenter[2]=Float.valueOf(cubicReceptorDomains.get(parsed[0]+"-z"));	
				float side=Float.valueOf(cubicReceptorDomains.get(parsed[0]+"-side"));	
				String myClass=cubicReceptorDomains.get(parsed[0]+"-class");
				String type=cubicReceptorDomains.get(parsed[0]+"-type");
				String toString=cubicReceptorDomains.get(parsed[0]+"-toString");
				int receptorQuantity=Integer.parseInt(cubicReceptorDomains.get(parsed[0]+"-receptorQuantity"));
				int receptorType=Integer.parseInt(cubicReceptorDomains.get(parsed[0]+"-receptorType"));
				int strategy=Integer.parseInt(cubicReceptorDomains.get(parsed[0]+"-strategy"));


				String dP=cubicReceptorDomains.get(parsed[0]+"-directionP");
				String dN=cubicReceptorDomains.get(parsed[0]+"-directionN");
				String dO=cubicReceptorDomains.get(parsed[0]+"-directionO");		
				float rotationAngle=Float.valueOf(cubicReceptorDomains.get(parsed[0]+"-rotationAngle"));

				boolean visible=false;
				if(Short.parseShort(cubicReceptorDomains.get(parsed[0]+"-visible"))==1) visible=true;

				boolean export=false;
				if(Short.parseShort(cubicReceptorDomains.get(parsed[0]+"-export"))==1) export=true;

				boolean octree=false;
				if(Short.parseShort(cubicReceptorDomains.get(parsed[0]+"-octree"))==1) octree=true;


				CubicReceptorStrategy cubic = new CubicReceptorStrategy(strategy);
				//CubicReceptorDomain cube = new CubicReceptorDomain(mainCylinder, this, cubic, visible, export, side, relativeCenter, receptorType, receptorQuantity, observerForDomain, octree);



				String thisClass=toString.substring(0,toString.lastIndexOf("@"));
				String className=toString.substring(toString.lastIndexOf("@")+1,toString.length());

				try{

					Class domainClass = Class.forName(thisClass);		
					Class[] parameterTypes=new Class[]{AbstractDomain.class, MobilityModel.class, AbstractCollisionDomainStrategy.class,
							boolean.class, boolean.class, float.class, float[].class, int.class, int.class, ArrayList.class,  boolean.class};
					Constructor domainContructor=domainClass.getConstructor(parameterTypes);

					AbstractDomain cube=(AbstractDomain) domainContructor.newInstance(mainCylinder, this, cubic, visible, export, side, relativeCenter, receptorType, receptorQuantity, observerForDomain, octree);
					cube.ID=toString;



					// Sets the direction axes!
					float dpx=Float.valueOf(dP.substring(1,dP.indexOf(",")));
					float dpy=Float.valueOf(dP.substring(dP.indexOf(",")+1,dP.lastIndexOf(",")));
					float dpz=Float.valueOf(dP.substring(dP.lastIndexOf(",")+1,dP.lastIndexOf(")")));					
					cube.directionP=new SimpleVector(dpx,dpy,dpz);

					float dnx=Float.valueOf(dN.substring(1,dN.indexOf(",")));
					float dny=Float.valueOf(dN.substring(dN.indexOf(",")+1,dN.lastIndexOf(",")));
					float dnz=Float.valueOf(dN.substring(dN.lastIndexOf(",")+1,dN.lastIndexOf(")")));					
					cube.directionN=new SimpleVector(dnx,dny,dnz);

					float dox=Float.valueOf(dO.substring(1,dO.indexOf(",")));
					float doy=Float.valueOf(dO.substring(dO.indexOf(",")+1,dO.lastIndexOf(",")));
					float doz=Float.valueOf(dO.substring(dO.lastIndexOf(",")+1,dO.lastIndexOf(")")));					
					cube.directionO=new SimpleVector(dox,doy,doz);					
					// End of direction axes



					huvecNumber = Math.round(cylinderCircumference/(2*side));

					//							
					huvecSide = 2*side;
					apothem = (float) (radiusCyl*Math.cos(Math.PI/huvecNumber));
					huvecCenter = apothem + (huvecSide/2);
					//da ripulire? non servono pi??
					//

					SimpleVector dVersor = mainCylinder.getDirectionP().normalize();



					//  DRAW CUBE RECEPTOR DOMAIN IN THE 3D SPACE!! 

					if (rootDomain.getManager().is3DEnable ) {
						Object3D o = rootDomain.getManager().engine.getWrapperDomain3D(cube);
						o.rotateAxis(dVersor, rotationAngle);

					}
					//cube.createReceptorsForHUVEC(1000, huvecNumber, i, layerCenterVector, dVersor);	



					// RELOAD EACH RECEPTOR...
					Map<String,String> receptors=MySQLBackup_BloodVessel.getInstance().getReceptorsForCubicDomain(toString, configWrapper);

					Iterator itR=receptors.keySet().iterator();		
					int iR=0;

					Map<String,String> mapIterReceptor=new HashMap<String,String>();
					while(itR.hasNext()){
						String elementR = (String) itR.next();
						String[] parsedR=elementR.split("-",2);
						mapIterReceptor.put(parsedR[0], "ok");
					}

					Iterator itR1=mapIterReceptor.keySet().iterator();

					while(itR1.hasNext()){	

						String elementR2 = (String) itR1.next();
						String[] parsedR2=elementR2.split("-",2);

						float[] center=new float[3];

						center[0]=Float.valueOf(receptors.get(parsedR2[0]+"-x"));	
						center[1]=Float.valueOf(receptors.get(parsedR2[0]+"-y"));	
						center[2]=Float.valueOf(receptors.get(parsedR2[0]+"-z"));	

						((CubicReceptorDomain)cube).createSingleReceptorForHUVEC(center);

					}
					((CubicReceptorDomain)cube).removeOverlappedReceptors(((CubicReceptorDomain)cube).receptorList);
					System.out.println("RECEPTOR SURVIVED: "+((CubicReceptorDomain)cube).receptorList.size());


				}catch (Exception e){
					e.printStackTrace();
				}	
			}	
			restoreBloodCells(mainCylinder_toString, configWrapper);
		}*/
	}

	public void networkCreationForEndothelium() {
		System.out.println("-----------------------------------");
		System.out.println("Auto-Create Network... [ENDOTHELIUM]");
		System.out.println("-----------------------------------");

		boolean useOctree=true;

		ArrayList<LigandObserver> observerForDomain=new ArrayList<LigandObserver>();
		observerForDomain.add(CD40LObserver.getInstance());

		//float height3=3000000;	
		float height3=2500000+88500+12500;  // medium lenght
		float rad3=30000;  // Venule diameter 60um	

		float[] cdCenter3=new float[3];
		cdCenter3[0]=100000;
		cdCenter3[1]=100000;
		cdCenter3[2]=100000;	

		CylindricStrategyBloodVessel cs=new CylindricStrategyBloodVessel(1);
		BloodFlowModel mainFlow=new BloodFlowModel(getManager().rootDomain);		

		CylindricBloodVesselDomain mainCylinder=new CylindricBloodVesselDomain(getManager().rootDomain, mainFlow,cs, true, false, height3, rad3,cdCenter3, useOctree);
		mainCylinder.ID="CylindricBloodVesselDomain";

		// Creation of cubes representing endothelial cells
		float huvecDiameter = 15000;  // 15 um
		float cylinderCircumference = (float) (2*Math.PI*rad3);
		int huvecNumber = Math.round(cylinderCircumference/huvecDiameter);
		float huvecSide = cylinderCircumference/huvecNumber;
		float apothem = (float) (rad3*Math.cos(Math.PI/huvecNumber));
		float huvecCenter = apothem + (huvecSide/2);

		SimpleVector dVersor = mainCylinder.getDirectionP().normalize();
		SimpleVector tempVector = null;
		if (dVersor.x <= dVersor.y && dVersor.x <= dVersor.z)
			tempVector = new SimpleVector(dVersor.x+0.5, dVersor.y, dVersor.z);
		else if (dVersor.y <= dVersor.x && dVersor.y <= dVersor.z)
			tempVector = new SimpleVector(dVersor.x, dVersor.y+0.5, dVersor.z);
		else if (dVersor.z <= dVersor.x && dVersor.z <= dVersor.y)
			tempVector = new SimpleVector(dVersor.x, dVersor.y, dVersor.z+0.5);

		SimpleVector huvecCenterVector = dVersor.calcCross(tempVector).normalize();
		huvecCenterVector.scalarMul(huvecCenter);

		float[] domainCenter = mainCylinder.getAbsCenter();
		SimpleVector domainCenterVector = new SimpleVector();
		domainCenterVector.x = domainCenter[0];
		domainCenterVector.y = domainCenter[1];
		domainCenterVector.z = domainCenter[2];		

		float displacement=(float) (((mainCylinder.getHeight()-101000-500000)/2)-((mainCylinder.getHeight()-101000-250000)*0.1))-(huvecDiameter-500)/2;
		displacement=(float) (((mainCylinder.getHeight()-101000-500000)/2)-((mainCylinder.getHeight()-101000-250000)*0.1));

		float[] layerCenter = mainCylinder.getAbsCenter();
		SimpleVector layerCenterVector = new SimpleVector();
		layerCenterVector.x = layerCenter[0];
		layerCenterVector.y = layerCenter[1]-displacement;
		layerCenterVector.z = layerCenter[2];

		//j=128 Covers the relevant part of the venule under study
		//j=134 long
		//j=95 short venule		
		for (int j=1; j<3; j++) {
			for (int i=0; i<huvecNumber; i++){
				huvecCenterVector.add(layerCenterVector);
				System.out.println("Layer Center - x: "+layerCenterVector.x+", y: "+layerCenterVector.y+", z: "+layerCenterVector.z+"  -  HUVEC Size: "+huvecDiameter);
				SimpleVector relativeHuvecCenterVector = new SimpleVector();
				relativeHuvecCenterVector.x = huvecCenterVector.x - domainCenter[0];
				relativeHuvecCenterVector.y = huvecCenterVector.y - domainCenter[1];
				relativeHuvecCenterVector.z = huvecCenterVector.z - domainCenter[2];
				float[] cellCenter = {relativeHuvecCenterVector.x, relativeHuvecCenterVector.y, relativeHuvecCenterVector.z};
				CubicReceptorStrategy cubic = new CubicReceptorStrategy(1);
				CubicReceptorDomain cube = new CubicReceptorDomain(mainCylinder, this, cubic, true, true, huvecSide/2, cellCenter, 2, 0, observerForDomain, false);
				huvecCenterVector.sub(layerCenterVector);
				System.out.println("relative Center - x: "+relativeHuvecCenterVector.x+", y: "+relativeHuvecCenterVector.y+", z: "+relativeHuvecCenterVector.z);

				SimpleVector cubeDirection = new SimpleVector(huvecCenterVector).normalize();
				cube.setDirectionP(cubeDirection);

				Matrix rotation = new Matrix();
				float angle = (float) ((2*Math.PI)/huvecNumber);
				rotation.rotateAxis(dVersor, angle);
				huvecCenterVector.rotate(rotation);

				/*  DRAW CUBE RECEPTOR DOMAIN IN THE 3D SPACE!! */			
				if (rootDomain.getManager().is3DEnable && i > 0) {
					Object3D o = rootDomain.getManager().engine.getWrapperDomain3D(cube);
					o.rotateAxis(dVersor, angle*i);
				}			
				cube.createReceptorsForHUVEC(1000, huvecNumber, i, layerCenterVector, dVersor);				
			}

			dVersor.scalarMul(huvecSide*j);
			domainCenterVector.add(dVersor);
			layerCenterVector = new SimpleVector(domainCenterVector);
			layerCenterVector.y=layerCenterVector.y-displacement;

			dVersor = dVersor.normalize();
			domainCenter = mainCylinder.getAbsCenter();
			domainCenterVector = new SimpleVector(domainCenter);
		}
		System.out.println(">>> ENDOTHELIAL CELLS DISPOSED!");	
		System.out.println();
	}

	public void emissionProcessInVessel(CylindricBloodVesselDomain mainCylinder){
		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		float bound=mainCylinder.getRadius();
		float depth=88500;
		float tempDepth=-(2500000)/2+250000;  	

		float[] initialDeploy={mainCylinder.getAbsCenter()[0],mainCylinder.getAbsCenter()[1]+tempDepth,mainCylinder.getAbsCenter()[2]};
		initialDeploy[1]=-775000+100000-7500-44250;

		if (createFixedPlatelet(mainCylinder,0)){
			mainCylinder.hasToCreateTXNode=false;			
			float xx=0;
			float yy=0;
			float zz=0;
			int wastedLimit=1000;
			int wastedAttempt=0;
			boolean hasFinished=false;
			int i=0;
			float cylQuadraticRadius=mainCylinder.getRadius()*mainCylinder.getRadius();

			int cellCount=0;  //1	
			float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(15+"-radius").floatValue();
			bound=bound-radius;

			while(!hasFinished && cellCount>0){
				xx=(float)((rand.nextFloat()*2*bound)-bound);
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)((rand.nextFloat()*2*bound)-bound);

				if (xx*xx+zz*zz<cylQuadraticRadius){
					/*  BUILDS THE MACROPHAGE CELL */
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					Macrophage macrophage=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);

					checkIsFullInside(macrophage);

					if (macrophage.getTimeToLive()>0) {
						nodeCount++;			
						wastedAttempt=0;
						i++;
					}else{
						wastedAttempt++;
					}

					if(wastedAttempt>wastedLimit || i>=cellCount) hasFinished=true;
					macrophage.burstEmission=0;
				}
			}
			System.out.println(">>> MACROPHAGE WASTE: "+wastedAttempt+"  -  Created: "+i);

			bound=mainCylinder.getRadius();
			cellCount=0;	//1000
			radius=mainCylinder.getManager().configWrapper.nodeConfig.get(14+"-radius").floatValue();
			bound=bound-radius;
			xx=0;
			yy=0;
			zz=0;

			i=0;
			wastedAttempt=0;
			hasFinished=false;

			while(!hasFinished && cellCount>0){
				xx=(float)((rand.nextFloat()*2*bound)-bound);
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)((rand.nextFloat()*2*bound)-bound);

				if (xx*xx+zz*zz<cylQuadraticRadius){
					/*  BUILD THE RED BLOOD CELL */
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					RedBloodCell redBloodCell=new RedBloodCell(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,14,observer);

					checkIsFullInside(redBloodCell);

					if (redBloodCell.getTimeToLive()>0){
						nodeCount++;			
						wastedAttempt=0;
						i++;
					}else{
						wastedAttempt++;
					}

					if(wastedAttempt>wastedLimit || i>=cellCount) hasFinished=true;
					redBloodCell.burstEmission=0;
				}
			}

			System.out.println(">>> RED CELL WASTE: "+wastedAttempt+"  -  Created: "+i);

			cellCount=0;//50
			bound=mainCylinder.getRadius();
			radius=mainCylinder.getManager().configWrapper.nodeConfig.get(13+"-radius").floatValue();
			bound=bound-radius;
			xx=0;
			yy=0;
			zz=0;

			i=0;
			wastedAttempt=0;
			hasFinished=false;	

			while(!hasFinished && cellCount>0){				
				xx=(float)((rand.nextFloat()*2*bound)-bound);
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)((rand.nextFloat()*2*bound)-bound);

				if ((xx*xx+zz*zz)<cylQuadraticRadius){
					/* BUILDS THE PLATELET CELL */
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					PlateletVessel platelet1=new PlateletVessel(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,13,observer);

					checkIsFullInside(platelet1);
					platelet1.burstEmission=0;

					if (platelet1.getTimeToLive()>0) {
						nodeCount++;	
						wastedAttempt=0;
						i++;
					}else{
						wastedAttempt++;
					}				
					if(wastedAttempt>wastedLimit || i>=cellCount) hasFinished=true;
				}
			}
			System.out.println(">>> PLATELET WASTE: "+wastedAttempt+"  -  Created: "+i);

		} else {			
			mainCylinder.hasToCreateTXNode=false;  
			System.out.println(">>> DELAYED RELEASE");
		}		
	}

	public double[] getParabolicCoordinates(float cylRadius, float nanoRadius){
		/**  SOLVING POLYNOMIAL EQ  */
		Random rr=new Random();		

		Complex[] routh = Polynomial.cubic(2*rr.nextDouble(),-3,0,1);	
		double sol=0;

		if(routh[2].getReal()<=1 && routh[2].getReal()>=0){
			sol=routh[2].getReal();
		}else if(routh[1].getReal()<=1 && routh[1].getReal()>=0){
			sol=routh[1].getReal();
		}else if(routh[0].getReal()<=1 && routh[0].getReal()>=0){
			sol=routh[0].getReal();
		}		

		double ro=(sol)*(cylRadius-nanoRadius);
		//double ro=rr.nextDouble()*(cylRadius-nanoRadius);
		double theta=rr.nextDouble()*Math.PI*2;

		double x=ro*Math.cos(theta);
		double z=ro*Math.sin(theta);

		double[] results={x,z};
		return results;
		/**  END SOLVING POLYNOMIAL EQ  */
	}

	public double[] getFlatCoordinates(float cylRadius, float nanoRadius){
		/**  SOLVING POLYNOMIAL EQ  */
		Random rr=new Random();		

		double ro=(rr.nextDouble())*(cylRadius-nanoRadius);
		//double ro=rr.nextDouble()*(cylRadius-nanoRadius);
		double theta=rr.nextDouble()*Math.PI*2;

		double x=ro*Math.cos(theta);
		double z=ro*Math.sin(theta);

		double[] results={x,z};
		return results;
		/**  END SOLVING POLYNOMIAL EQ  */
	}	


	public void restoreBloodCells(String mainCylinder, int configWrapper){

		Map<String,String> nanoObjs=MySQLBackup_BloodVessel.getInstance().getQuery_NanoObj(mainCylinder, configWrapper);

		Iterator it00=nanoObjs.keySet().iterator();		
		int i=0;

		Map<String,String> mapIter=new HashMap<String,String>();
		while(it00.hasNext()){
			String element = (String) it00.next();
			String[] parsed=element.split("-",2);
			mapIter.put(parsed[0], "ok");
		}

		Iterator it=mapIter.keySet().iterator();

		while(it.hasNext()){	

			String element = (String) it.next();
			String[] parsed=element.split("-",2);


			String ID=parsed[0];

			float[] relativeCenter=new float[3];


			String mainDomain=nanoObjs.get(parsed[0]+"-domain");
			float x=Float.valueOf(nanoObjs.get(parsed[0]+"-x"));	
			float y=Float.valueOf(nanoObjs.get(parsed[0]+"-y"));
			float z=Float.valueOf(nanoObjs.get(parsed[0]+"-z"));	
			float radius=Float.valueOf(nanoObjs.get(parsed[0]+"-radius"));	
			float mass=Float.valueOf(nanoObjs.get(parsed[0]+"-mass"));	

			float velocity_module=Float.valueOf(nanoObjs.get(parsed[0]+"-velocity_module"));	
			float velocity_fi=Float.valueOf(nanoObjs.get(parsed[0]+"-velocity_fi"));	
			float velocity_theta=Float.valueOf(nanoObjs.get(parsed[0]+"-velocity_theta"));	

			float velocityMM_module=Float.valueOf(nanoObjs.get(parsed[0]+"-velocityMM_module"));	
			float velocityMM_fi=Float.valueOf(nanoObjs.get(parsed[0]+"-velocityMM_fi"));	
			float velocityMM_theta=Float.valueOf(nanoObjs.get(parsed[0]+"-velocityMM_theta"));	

			float velocityMS_module=Float.valueOf(nanoObjs.get(parsed[0]+"-velocityMS_module"));	
			float velocityMS_fi=Float.valueOf(nanoObjs.get(parsed[0]+"-velocityMS_fi"));	
			float velocityMS_theta=Float.valueOf(nanoObjs.get(parsed[0]+"-velocityMS_theta"));	

			String myClass=nanoObjs.get(parsed[0]+"-class");	
			String type=nanoObjs.get(parsed[0]+"-type");	
			long ttl=Long.valueOf(nanoObjs.get(parsed[0]+"-ttl"));	
			float absDistance=Float.valueOf(nanoObjs.get(parsed[0]+"-absDistance"));	
			long informationCount=Long.valueOf(nanoObjs.get(parsed[0]+"-informationCount"));	

			long collisionCount=Long.valueOf(nanoObjs.get(parsed[0]+"-collisionCount"));	
			String toString=nanoObjs.get(parsed[0]+"-toString");	
			byte disposed=Byte.valueOf(nanoObjs.get(parsed[0]+"-disposed"));			
			int configType=Integer.valueOf(nanoObjs.get(parsed[0]+"-configType"));	

			String thisClass=toString.substring(0,toString.lastIndexOf("@"));
			String className=toString.substring(toString.lastIndexOf("@")+1,toString.length());

			try{

				Class nanoClass = Class.forName(thisClass);		
				Class[] parameterTypes=new Class[]{AbstractDomain.class, float.class, float.class, float.class,  int.class, List.class};
				Constructor nanoConstructor=nanoClass.getConstructor(parameterTypes);


				List<LigandObserver> observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());				

				// The type is taken from the config_wrapper_node? Should save on the nanoobj tablethe N and the type, then, on the basis of that type of Nanoobj, it is selected
				// the correct table in order to obtain the desidered informations				
				//NanoObj nano=(NanoObj) nanoConstructor.newInstance(mainCylinder, x,y,z, configType, observer);
				NodeObj nano=(NodeObj) nanoConstructor.newInstance(mainCylinder, x,y,z, configType,  observer);


				// Restore the internal state of the nanoobj
				nano.setVelocityComponent(velocity_module, velocity_fi, velocity_theta);
				nano.setVelocityComponentMM(velocityMM_module, velocityMM_fi, velocityMM_theta);
				nano.setVelocityComponentMM(velocityMS_module, velocityMS_fi, velocityMS_theta);
				nano.setTimeToLive(ttl);
				nano.informationCount=(int) informationCount;
				nano.collisionCount=(int) collisionCount;

				// from config_wrapper_node
				nano.setMass(mass);				
				if(type.matches("Platelet") || type.matches("Macrophage")){
					Map<String,String> nodeConfig=MySQLBackup_BloodVessel.getInstance().getQuery_config_wrapper_node(configWrapper, configType); 

					NodeObj node=((NodeObj)nano);
					node.burstEmission=Integer.valueOf(nodeConfig.get("burstEmission"));
					node.receptorNumberType=Integer.valueOf(nodeConfig.get("receptorType"));
					node.zoom3D=Integer.valueOf(nodeConfig.get("zoom3D"));
					node.setStateType(Integer.valueOf(nodeConfig.get("burstEmission")));
					node.receptorList.clear();

					// RELOAD ALL RECEPTOR FOR THIS NODE...
					Map<String,String> receptors=MySQLBackup_BloodVessel.getInstance().getReceptorsForNode(toString,configWrapper); 

					Iterator itR=receptors.keySet().iterator();		
					int iR=0;

					Map<String,String> mapIterReceptor=new HashMap<String,String>();
					while(itR.hasNext()){
						String elementR = (String) itR.next();
						String[] parsedR=elementR.split("-",2);
						mapIterReceptor.put(parsedR[0], "ok");
					}

					Iterator itR1=mapIterReceptor.keySet().iterator();

					while(itR1.hasNext()){	

						String elementR2 = (String) itR1.next();
						String[] parsedR2=elementR2.split("-",2);

						float[] center=new float[3];

						center[0]=Float.valueOf(receptors.get(parsedR2[0]+"-x"));	
						center[1]=Float.valueOf(receptors.get(parsedR2[0]+"-y"));	
						center[2]=Float.valueOf(receptors.get(parsedR2[0]+"-z"));	

						short rRadius=Short.valueOf(receptors.get(parsedR2[0]+"-radius"));

						node.createSingleReceptor(center, radius, rRadius, node.receptorNumberType);
					}
					System.out.println("NodeObj - "+toString+"  >>> RECEPTORS COUNT: "+node.receptorList.size());
				}
			}catch (Exception e){
				e.printStackTrace();
			}
		}

		// DISPOSED???

		// Config_WrapperNode
		// Config_wrapper_node_state
	}


	public void emissionProcessInVesselDeterministic(CylindricBloodVesselDomain mainCylinder){

		/**  CREATION EVERY 33 step (if timeStep=1000 ns), 3.3 step (if timeStep=10000)  */
		List<LigandObserver> observer=new ArrayList<LigandObserver>();

		float bound=mainCylinder.getRadius();
		float depth=88500;
		float tempDepth=-(2500000)/2+250000;  	
		float[] initialDeploy={mainCylinder.getAbsCenter()[0],mainCylinder.getAbsCenter()[1]+tempDepth,mainCylinder.getAbsCenter()[2]};
		initialDeploy[1]=-775000+100000-7500-depth/2;		

		// FIXME If you comment the following two lines, you get an instantaneous emission!
		//long timeThreshold=(long) (2.5*Math.pow(10, 9)/getManager().configWrapper.timeUnit);
		long timeThreshold=(long) (0.2*Math.pow(10, 9)/getManager().configWrapper.timeUnit);
		if (getManager().getCurrentTime()>timeThreshold && !hasCreatedTxNode){				
			//int plateletLevel=((iManager) getManager()).plateletLevel;		
			int plateletLevel=this.plateletLevel;		
			hasCreatedTxNode=createFixedPlatelet(mainCylinder,plateletLevel);
		}		

		float xx=0;
		float yy=0;
		float zz=0;

		int wastedLimit=100;
		int wastedAttempt=0;
		boolean hasFinished=false;
		float cylQuadraticRadius=mainCylinder.getRadius()*mainCylinder.getRadius();

		step=-1;  // FIXME IF SET, NEVER CREATES OTHER CELLS!		
		if (step==0 ){

			// MACROPHAGE			
			float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(15+"-radius").floatValue();
			bound=bound-radius;			
			hasFinished=false;
			while(!hasFinished){
				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);

				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];				

				if (xx*xx+zz*zz<cylQuadraticRadius){
					/*  BUILDS THE MACROPHAGE CELL */
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					Macrophage macrophage=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
					macrophage.burstEmission=0;

					if (macrophage.getTimeToLive()>0) {
						nodeCount++;			
						wastedAttempt=0;						
						createdCells++;
						hasFinished=true;						
						setInitializationVelocity(macrophage);					
					}else{
						wastedAttempt++;					
					}

					if(wastedAttempt>wastedLimit) {
						notCreatedCells++;
						hasFinished=true;
					}
				}
			}	


			//step++; // FIXME If you comment this line, you do not create the other cells! You will have only the Macrophages!
			//step=-1;

		}else if (step>0 && step<=20){

			// RED BLOOD CELLS			
			bound=mainCylinder.getRadius();
			float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(14+"-radius").floatValue();
			bound=bound-radius;
			xx=0;
			yy=0;
			zz=0;

			wastedAttempt=0;
			hasFinished=false;

			while(!hasFinished){

				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);				

				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];				

				if (xx*xx+zz*zz<cylQuadraticRadius){
					/*  BUILDS THE RED BLOOD CELL */
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					RedBloodCell redBloodCell=new RedBloodCell(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,14,observer);
					redBloodCell.burstEmission=0;

					if (redBloodCell.getTimeToLive()>0){
						nodeCount++;			
						wastedAttempt=0;						
						createdCells++;
						hasFinished=true;						
						setInitializationVelocity(redBloodCell);						
					}else{
						wastedAttempt++;						
					}
					if(wastedAttempt>wastedLimit) {
						notCreatedCells++;
						hasFinished=true;
					}
				}
			}			
			step++;


		}else if (step==21){

			// PLATELETS			
			bound=mainCylinder.getRadius();
			float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(13+"-radius").floatValue();
			bound=bound-radius;

			xx=0;
			yy=0;
			zz=0;
			wastedAttempt=0;
			hasFinished=false;				

			while(!hasFinished){

				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);				

				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];				

				if ((xx*xx+zz*zz)<cylQuadraticRadius){
					/*  BUILDS THE PLATELET CELL */
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					PlateletVessel platelet1=new PlateletVessel(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,13,observer);
					platelet1.burstEmission=0;

					if (platelet1.getTimeToLive()>0) {
						nodeCount++;	
						wastedAttempt=0;						
						createdCells++;
						hasFinished=true;						
						setInitializationVelocity(platelet1);
					}else{
						wastedAttempt++;						
					}

					if(wastedAttempt>wastedLimit) {
						notCreatedCells++;
						hasFinished=true;
					}
				}
			}		

			if(cycle<50){
				cycle++;
				step=1;

			}else{
				System.out.println("| Step | "+step+"  | Cycle | "+cycle+" | Created | "+createdCells+" | Not Created | "+notCreatedCells);
				cycle=0;
				step=0;						
			}
		}
	}


	public void emissionProcessInVesselDeterministic_LongVessel(CylindricBloodVesselDomain mainCylinder){

		ArrayList<NanoObj> createdObjs=new ArrayList<NanoObj>();		

		/**  CREATION EVERY 33 step (if timeStep=1000 ns), 3.3 step (if timeStep=10000)  */
		List<LigandObserver> observer=new ArrayList<LigandObserver>();

		float bound=mainCylinder.getRadius();
		float depth=88500;
		//float tempDepth=-(2500000)/2+250000;  	
		float displacement=-1400000-depth/2;	
		float[] initialDeploy={mainCylinder.getAbsCenter()[0],mainCylinder.getAbsCenter()[1]+displacement,mainCylinder.getAbsCenter()[2]};


		// FIXME If you comment the following two lines, you get an instantaneous emission!
		//long timeThreshold=(long) (2.5*Math.pow(10, 9)/getManager().configWrapper.timeUnit);
		long timeThreshold=(long) (0.2*Math.pow(10, 9)/getManager().configWrapper.timeUnit);		
		if (getManager().getCurrentTime()>timeThreshold && !hasCreatedTxNode){				
			//int plateletLevel=((iManager) getManager()).plateletLevel;
			int plateletLevel=this.plateletLevel;		
			hasCreatedTxNode=createFixedPlatelet_LongVessel(mainCylinder,plateletLevel);

		}		

		float xx=0;
		float yy=0;
		float zz=0;

		int wastedLimit=100;
		int wastedAttempt=0;
		boolean hasFinished=false;
		float cylQuadraticRadius=mainCylinder.getRadius()*mainCylinder.getRadius();

		//step=-1;  // FIXME IF SET, NEVER CREATES OTHER CELLS!		
		if (step==0 ){

			//System.out.println(ttimettemp);
			// MACROPHAGE			
			float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(15+"-radius").floatValue();
			bound=bound-radius;			
			hasFinished=false;
			while(!hasFinished){
				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);

				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];				

				if (xx*xx+zz*zz<cylQuadraticRadius){
					/*  BUILDS THE MACROPHAGE CELL */
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					Macrophage macrophage=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
					macrophage.burstEmission=0;

					if (macrophage.getTimeToLive()>0) {
						nodeCount++;			
						wastedAttempt=0;						
						createdCells++;
						hasFinished=true;						
						setInitializationVelocity(macrophage);	
						createdObjs.add(macrophage);
					}else{
						wastedAttempt++;					
					}

					if(wastedAttempt>wastedLimit) {
						notCreatedCells++;
						hasFinished=true;
					}
				}
			}	

			step++; // FIXME If you comment this line, you do not create the other cells! You will have only the Macrophages!
			//step=-1;

		}else if (step>0 && step<=20){

			// RED BLOOD CELLS			
			bound=mainCylinder.getRadius();
			float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(14+"-radius").floatValue();
			bound=bound-radius;
			xx=0;
			yy=0;
			zz=0;

			wastedAttempt=0;
			hasFinished=false;

			while(!hasFinished){

				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);				

				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];				

				if (xx*xx+zz*zz<cylQuadraticRadius){
					//  BUILDS THE RED BLOOD CELL 
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					RedBloodCell redBloodCell=new RedBloodCell(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,14,observer);
					redBloodCell.burstEmission=0;

					if (redBloodCell.getTimeToLive()>0){
						nodeCount++;			
						wastedAttempt=0;						
						createdCells++;
						hasFinished=true;						
						setInitializationVelocity(redBloodCell);		
						createdObjs.add(redBloodCell);
					}else{
						wastedAttempt++;						
					}
					if(wastedAttempt>wastedLimit) {
						notCreatedCells++;
						hasFinished=true;
					}
				}
			}	

			step++;


		}else if (step==21){

			// PLATELETS			
			bound=mainCylinder.getRadius();
			float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(13+"-radius").floatValue();
			bound=bound-radius;

			xx=0;
			yy=0;
			zz=0;
			wastedAttempt=0;
			hasFinished=false;				

			while(!hasFinished){

				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);				

				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];				

				if ((xx*xx+zz*zz)<cylQuadraticRadius){
					//  BUILDS THE PLATELET CELL 
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					PlateletVessel platelet1=new PlateletVessel(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,13,observer);
					platelet1.burstEmission=0;

					if (platelet1.getTimeToLive()>0) {
						nodeCount++;	
						wastedAttempt=0;						
						createdCells++;
						hasFinished=true;						
						setInitializationVelocity(platelet1);
						createdObjs.add(platelet1);
					}else{
						wastedAttempt++;						
					}

					if(wastedAttempt>wastedLimit) {
						notCreatedCells++;
						hasFinished=true;
					}
				}
			}	

			if(cycle<50){	// 50 ok!
				cycle++;
				step=1;

			}else{
				System.out.println("| Step | "+step+"  | Cycle | "+cycle+" | Created | "+createdCells+" | Not Created | "+notCreatedCells);
				cycle=0;
				step=0;						
			}
		}
	}


	public void emissionProcessInVesselDeterministic_LongVessel_NEW_Casson(CylindricBloodVesselDomain mainCylinder){

		ArrayList<NanoObj> createdObjs=new ArrayList<NanoObj>();

		/**  CREATION EVERY 33 step (if timeStep=1000 ns), 3.3 step (if timeStep=10000)  */
		List<LigandObserver> observer=new ArrayList<LigandObserver>();

		float bound=mainCylinder.getRadius();
		float depth=88500;	
		//float displacement=-1400000-depth/2;		// ORIGINAL
		float displacement=600000-depth/2;		// SHIFTED

		float[] initialDeploy={mainCylinder.getAbsCenter()[0],mainCylinder.getAbsCenter()[1]+displacement,mainCylinder.getAbsCenter()[2]};


		// FIXME If you comment the following two lines, you get an instantaneous emission!
		long timeThreshold=(long) (0.2*Math.pow(10, 9)/getManager().configWrapper.timeUnit);			// was 0.2
		if (getManager().getCurrentTime()>timeThreshold && !hasCreatedTxNode){				
			//int plateletLevel=((iManager) getManager()).plateletLevel;	
			int plateletLevel=this.plateletLevel;		
			hasCreatedTxNode=createFixedPlatelet_LongVessel(mainCylinder,plateletLevel);		
		}	


		// ==============================
		// CHECK THE GLOBAL CONCENTRATION
		// ==============================
		long[] originalCount=((LivingBloodObjObserver)mainCylinder.getLivingObjObserver()).getOriginalCount();	
		long[] cellCount=((LivingBloodObjObserver)mainCylinder.getLivingObjObserver()).getCellCount();

		long miss_red=originalCount[0]-cellCount[0];
		long miss_mac=originalCount[1]-cellCount[1];
		long miss_plat=originalCount[2]-cellCount[2];	
		// ==============================
		// END CHECK
		// ==============================


		float xx=0;
		float yy=0;
		float zz=0;

		int wastedLimit=100;
		int wastedAttempt=0;	
		boolean hasFinished=false;
		float cylQuadraticRadius=mainCylinder.getRadius()*mainCylinder.getRadius();


		/*
		boolean debug=true;
		if(debug){

		}else{
		 */

		if(miss_mac>0){	

			// MACROPHAGE			
			float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(15+"-radius").floatValue();
			bound=bound-radius;			
			hasFinished=false;

			while(!hasFinished){

				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);

				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];				

				if (xx*xx+zz*zz<cylQuadraticRadius){
					/*  BUILDS THE MACROPHAGE CELL */
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					Macrophage macrophage=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
					macrophage.burstEmission=0;

					if (macrophage.getTimeToLive()>0) {
						nodeCount++;			
						wastedAttempt=0;						
						createdCells++;
						miss_mac--;
						//hasFinished=true;						
						setInitializationVelocity(macrophage);	
						createdObjs.add(macrophage);
					}else{
						wastedAttempt++;					
					}

					if(wastedAttempt>wastedLimit || miss_mac<=0) {
						notCreatedCells++;
						hasFinished=true;
					}
				}
			}
		}


		if(miss_red>0){			

			// RED BLOOD CELLS			
			bound=mainCylinder.getRadius();
			float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(14+"-radius").floatValue();
			bound=bound-radius;
			xx=0;
			yy=0;
			zz=0;

			wastedAttempt=0;
			hasFinished=false;

			while(!hasFinished){

				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);				

				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];				

				if (xx*xx+zz*zz<cylQuadraticRadius){
					//  BUILDS THE RED BLOOD CELL 
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					RedBloodCell redBloodCell=new RedBloodCell(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,14,observer);
					redBloodCell.burstEmission=0;

					if (redBloodCell.getTimeToLive()>0){
						nodeCount++;			
						wastedAttempt=0;						
						createdCells++;
						miss_red--;
						if(hasToMonitor){
							redBloodCell.exportThis=true;
							hasToMonitor=false;
						}

						//hasFinished=true;						
						setInitializationVelocity(redBloodCell);		
						createdObjs.add(redBloodCell);
					}else{
						wastedAttempt++;						
					}
					if(wastedAttempt>wastedLimit || miss_red<=0) {
						notCreatedCells++;
						hasFinished=true;
					}
				}
			}	
		}

		if(miss_plat>0){	

			// PLATELETS			
			bound=mainCylinder.getRadius();
			float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(13+"-radius").floatValue();
			bound=bound-radius;

			xx=0;
			yy=0;
			zz=0;
			wastedAttempt=0;
			hasFinished=false;				

			while(!hasFinished){

				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);				

				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];				

				if ((xx*xx+zz*zz)<cylQuadraticRadius){
					//  BUILDS THE PLATELET CELL 
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					PlateletVessel platelet1=new PlateletVessel(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,13,observer);
					platelet1.burstEmission=0;

					if (platelet1.getTimeToLive()>0) {
						nodeCount++;	
						wastedAttempt=0;						
						createdCells++;
						miss_plat--;
						//hasFinished=true;						
						setInitializationVelocity(platelet1);
						createdObjs.add(platelet1);
					}else{
						wastedAttempt++;						
					}

					if(wastedAttempt>wastedLimit || miss_plat<=0) {
						notCreatedCells++;
						hasFinished=true;
					}
				}
			}			
		}	
		//}  // DEBUG
	}

	public void setInitializationVelocity(NanoObj n){		
		/** FIXME IMPORTANT!! ONLY for Cylinder AXIS parallel to Y-Axis!! */		
		CylindricBloodVesselDomain d = (CylindricBloodVesselDomain) n.getDomain();

		float dx=0;
		float dy=getPoiseuilleVelocity(d, n);
		float dz=0;			

		float[] angles = n.getVelocityAngles(dx, dy, dz);	
		n.setVelocityComponentMM(dy, angles[0], angles[1]);		
		n.setVelocityComponent(dy, angles[0], angles[1]);	
	}

	public void emissionProcessInVesselDeterministicINITIALIZE(CylindricBloodVesselDomain mainCylinder){

		ArrayList<Macrophage> firstMacrophage=new ArrayList<Macrophage>();
		ArrayList<RedBloodCell> firstRedBloodCell=new ArrayList<RedBloodCell>();
		ArrayList<PlateletVessel> firstPlatelet=new ArrayList<PlateletVessel>();

		/**  CREATION EVERY 33 step (if timeStep=1000 ns), 3.3 step (if timeStep=1000)  */
		List<LigandObserver> observer=new ArrayList<LigandObserver>();

		float bound=mainCylinder.getRadius();
		float depth=88500;
		float tempDepth=-(2500000)/2+250000;  	// ok

		float[] initialDeploy={mainCylinder.getAbsCenter()[0],mainCylinder.getAbsCenter()[1]+tempDepth,mainCylinder.getAbsCenter()[2]};
		initialDeploy[1]=-775000+100000-7500-depth/2;
		float xx=0;
		float yy=0;
		float zz=0;
		int wastedLimit=2000;
		int wastedAttempt=0;
		boolean hasFinished=false;
		float cylQuadraticRadius=mainCylinder.getRadius()*mainCylinder.getRadius();

		// MACROPHAGE
		float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(15+"-radius").floatValue();
		bound=bound-radius;

		hasFinished=false;
		@SuppressWarnings("unused")
		boolean isLogged=false;

		while(!hasFinished){

			double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);

			xx=(float)results[0];
			yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
			zz=(float)results[1];

			if (xx*xx+zz*zz<cylQuadraticRadius){
				/*  BUILDS THE MACROPHAGE CELL */
				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage.burstEmission=0;

				if (macrophage.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage);					
					setInitializationVelocity(macrophage);
				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}
			}
		}			

		for(int gg=0;gg<50;gg++){	
			for(int hh=0;hh<20;hh++){

				// RED BLOOD CELLS
				bound=mainCylinder.getRadius();
				radius=mainCylinder.getManager().configWrapper.nodeConfig.get(14+"-radius").floatValue();
				bound=bound-radius;
				xx=0;
				yy=0;
				zz=0;
				wastedAttempt=0;
				hasFinished=false;

				while(!hasFinished){

					double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);

					xx=(float)results[0];
					yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
					zz=(float)results[1];

					if (xx*xx+zz*zz<cylQuadraticRadius){
						/*  BUILDS THE RED BLOOD CELL */
						observer=new ArrayList<LigandObserver>();
						observer.add(CD40LObserver.getInstance());
						RedBloodCell redBloodCell=new RedBloodCell(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,14,observer);
						redBloodCell.burstEmission=0;

						if (redBloodCell.getTimeToLive()>0){
							nodeCount++;			
							wastedAttempt=0;
							createdCells++;
							hasFinished=true;
							firstRedBloodCell.add(redBloodCell);							
							setInitializationVelocity(redBloodCell);
						}else{
							wastedAttempt++;
						}
						if(wastedAttempt>wastedLimit) {
							notCreatedCells++;
							hasFinished=true;
						}
					}
				}
			}

			// PLATELETS
			bound=mainCylinder.getRadius();
			radius=mainCylinder.getManager().configWrapper.nodeConfig.get(13+"-radius").floatValue();
			bound=bound-radius;
			xx=0;
			yy=0;
			zz=0;
			wastedAttempt=0;
			hasFinished=false;	
			while(!hasFinished){				
				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);
				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];

				if ((xx*xx+zz*zz)<cylQuadraticRadius){
					/*  BUILDS THE PLATELET CELL */
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					PlateletVessel platelet1=new PlateletVessel(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,13,observer);
					platelet1.burstEmission=0;

					if (platelet1.getTimeToLive()>0) {
						nodeCount++;	
						wastedAttempt=0;
						createdCells++;
						hasFinished=true;
						firstPlatelet.add(platelet1);						
						setInitializationVelocity(platelet1);
					}else{
						wastedAttempt++;
					}
					if(wastedAttempt>wastedLimit) {
						notCreatedCells++;
						hasFinished=true;
					}
				}
			}		
		}

		Random rr=new Random();
		SimpleVector dVersor=new SimpleVector(0,1,0);	
		isLogged=false;

		//aa=24 complete venule
		//aa=17 short venule
		for(int aa=1;aa<17;aa++){
			int size=firstMacrophage.size();
			System.out.println("=========================");
			System.out.println("Macrophage - SIZE: "+size+" - aa: "+aa);
			System.out.println("Blood Cell - SIZE: "+firstRedBloodCell.size()+" - aa: "+aa);
			System.out.println("Platelet Cell - SIZE: "+firstPlatelet.size()+" - aa: "+aa);
			System.out.println("=========================");

			float angle = (float) ((2*Math.PI)*rr.nextDouble());

			for(int i=0;i<size;i++){
				float[] center=new float[3];				
				center[0]=firstMacrophage.get(i).getCenter()[0];
				center[1]=firstMacrophage.get(i).getCenter()[1];
				center[2]=firstMacrophage.get(i).getCenter()[2];

				center[0]=center[0]-100000;
				center[1]=center[1]-100000+(88500+5000)*aa;
				center[2]=center[2]-100000;

				SimpleVector centerVector=new SimpleVector(center);		

				Matrix rotation = new Matrix();				
				rotation.rotateAxis(dVersor, angle);
				centerVector.rotate(rotation);

				centerVector.x=centerVector.x+100000;
				centerVector.y=centerVector.y+100000;
				centerVector.z=centerVector.z+100000;

				/*  BUILDS THE MACROPHAGE CELL */
				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage=new Macrophage(mainCylinder,centerVector.x,centerVector.y,centerVector.z,15,observer);				
				macrophage.burstEmission=0;

				if (macrophage.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;					
					setInitializationVelocity(macrophage);					
				}else{
					wastedAttempt++;
				}
			}

			int size2=firstRedBloodCell.size();			

			for(int i=0;i<size2;i++){				
				float[] center=new float[3];

				center[0]=firstRedBloodCell.get(i).getCenter()[0];
				center[1]=firstRedBloodCell.get(i).getCenter()[1];
				center[2]=firstRedBloodCell.get(i).getCenter()[2];

				center[0]=center[0]-100000;
				center[1]=center[1]-100000+(88500+5000)*aa;
				center[2]=center[2]-100000;

				SimpleVector centerVector=new SimpleVector(center);				

				Matrix rotation = new Matrix();				
				rotation.rotateAxis(dVersor, angle);
				centerVector.rotate(rotation);

				centerVector.x=centerVector.x+100000;
				centerVector.y=centerVector.y+100000;
				centerVector.z=centerVector.z+100000;

				/*  BUILDS THE RED BLOOD CELL */
				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				RedBloodCell redBloodCell=new RedBloodCell(mainCylinder,centerVector.x,centerVector.y,centerVector.z,14,observer);
				redBloodCell.burstEmission=0;

				if (redBloodCell.getTimeToLive()>0){
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					setInitializationVelocity(redBloodCell);
				}else{
					wastedAttempt++;
				}
			}

			int size3=firstPlatelet.size();

			for(int i=0;i<size3;i++){				
				float[] center=new float[3];

				center[0]=firstPlatelet.get(i).getCenter()[0];
				center[1]=firstPlatelet.get(i).getCenter()[1];
				center[2]=firstPlatelet.get(i).getCenter()[2];

				center[0]=center[0]-100000;
				center[1]=center[1]-100000+(88500+5000)*aa;
				center[2]=center[2]-100000;

				SimpleVector centerVector=new SimpleVector(center);						

				Matrix rotation = new Matrix();			
				rotation.rotateAxis(dVersor, angle);
				centerVector.rotate(rotation);

				centerVector.x=centerVector.x+100000;
				centerVector.y=centerVector.y+100000;
				centerVector.z=centerVector.z+100000;			

				/*  BUILDS THE PLATELET CELL */
				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				PlateletVessel platelet1=new PlateletVessel(mainCylinder,centerVector.x,centerVector.y,centerVector.z,13,observer);
				platelet1.burstEmission=0;

				if (platelet1.getTimeToLive()>0) {
					nodeCount++;	
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;					
					setInitializationVelocity(platelet1);
				}else{
					wastedAttempt++;
				}
			}
		}		
	}

	public void emissionProcessInVesselDeterministicINITIALIZE_LongVessel(CylindricBloodVesselDomain mainCylinder){

		ArrayList<Macrophage> firstMacrophage=new ArrayList<Macrophage>();
		ArrayList<RedBloodCell> firstRedBloodCell=new ArrayList<RedBloodCell>();
		ArrayList<PlateletVessel> firstPlatelet=new ArrayList<PlateletVessel>();

		/**  CREATION EVERY 33 step (if timeStep=1000 ns), 3.3 step (if timeStep=1000)  */
		List<LigandObserver> observer=new ArrayList<LigandObserver>();

		float bound=mainCylinder.getRadius();
		float depth=88500;
		//float tempDepth=-(2500000)/2+250000;  	// ok

		//float displacement=-1400000+depth/2;		// ORIGINAL
		float displacement=600000+depth/2;		// BUG FIX??

		float[] initialDeploy={mainCylinder.getAbsCenter()[0],mainCylinder.getAbsCenter()[1]+displacement,mainCylinder.getAbsCenter()[2]};
		//initialDeploy[1]=-775000+100000-7500-depth/2;
		float xx=0;
		float yy=0;
		float zz=0;
		int wastedLimit=2000;		// 2000 ok!
		int wastedAttempt=0;
		boolean hasFinished=false;
		float cylQuadraticRadius=mainCylinder.getRadius()*mainCylinder.getRadius();


		//WrapperBloodCells wrapper=WrapperBloodCells.getInstance();


		// MACROPHAGE
		float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(15+"-radius").floatValue();
		bound=bound-radius;

		try {
			//for(int times=0;times<4;times++){	// 4 - For vessel radius = 60 um
			hasFinished=false;
			while(!hasFinished){

				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);

				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];

				if (xx*xx+zz*zz<cylQuadraticRadius){
					/*  BUILDS THE MACROPHAGE CELL */
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					Macrophage macrophage=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
					macrophage.burstEmission=0;

					if (macrophage.getTimeToLive()>0) {
						nodeCount++;			
						wastedAttempt=0;					
						createdCells++;
						hasFinished=true;
						firstMacrophage.add(macrophage);
						//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
						setInitializationVelocity(macrophage);


					}else{
						wastedAttempt++;
					}
					if(wastedAttempt>wastedLimit) {
						notCreatedCells++;
						hasFinished=true;
					}					
				}
			}
			//}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}			


		//for(int times=0;times<0;times++){	// 4 - For vessel radius = 60 um
		System.out.println();
		for(int gg=0;gg<50;gg++){	// 50 - for vessel radius = 30 um
			System.out.print(" | "+gg);
			for(int hh=0;hh<20;hh++){	// 20 - for vessel radius = 30 um

				// RED BLOOD CELLS
				bound=mainCylinder.getRadius();
				radius=mainCylinder.getManager().configWrapper.nodeConfig.get(14+"-radius").floatValue();
				bound=bound-radius;
				xx=0;
				yy=0;
				zz=0;
				wastedAttempt=0;
				hasFinished=false;

				while(!hasFinished){

					double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);

					xx=(float)results[0];
					yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
					zz=(float)results[1];

					if (xx*xx+zz*zz<cylQuadraticRadius){
						/*  BUILDS THE RED BLOOD CELL */
						observer=new ArrayList<LigandObserver>();
						observer.add(CD40LObserver.getInstance());
						RedBloodCell redBloodCell=new RedBloodCell(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,14,observer);
						redBloodCell.burstEmission=0;

						if (redBloodCell.getTimeToLive()>0){
							nodeCount++;			
							wastedAttempt=0;
							createdCells++;
							hasFinished=true;
							firstRedBloodCell.add(redBloodCell);	
							//wrapper.addRedBloodCells(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,14,observer));
							setInitializationVelocity(redBloodCell);
						}else{
							wastedAttempt++;
							redBloodCell.destroy();
						}
						if(wastedAttempt>wastedLimit) {
							notCreatedCells++;
							hasFinished=true;
						}
					}
				}
			}

			// PLATELETS
			bound=mainCylinder.getRadius();
			radius=mainCylinder.getManager().configWrapper.nodeConfig.get(13+"-radius").floatValue();
			bound=bound-radius;
			xx=0;
			yy=0;
			zz=0;
			wastedAttempt=0;
			hasFinished=false;	
			while(!hasFinished){				
				double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);
				xx=(float)results[0];
				yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)results[1];

				if ((xx*xx+zz*zz)<cylQuadraticRadius){
					/*  BUILDS THE PLATELET CELL */
					observer=new ArrayList<LigandObserver>();
					observer.add(CD40LObserver.getInstance());
					PlateletVessel platelet1=new PlateletVessel(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,13,observer);
					platelet1.burstEmission=0;

					if (platelet1.getTimeToLive()>0) {
						nodeCount++;	
						wastedAttempt=0;
						createdCells++;
						hasFinished=true;
						firstPlatelet.add(platelet1);		
						//wrapper.addPlateletList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,13,observer));
						setInitializationVelocity(platelet1);
					}else{
						wastedAttempt++;
						platelet1.destroy();
					}
					if(wastedAttempt>wastedLimit) {
						notCreatedCells++;
						hasFinished=true;
					}
				}
			}		
		}
		//}


		System.out.println();		
		System.out.println("=========================");
		System.out.println("Macrophage - SIZE: "+firstMacrophage.size());
		System.out.println("Blood Cell - SIZE: "+firstRedBloodCell.size());
		System.out.println("Platelet Cell - SIZE: "+firstPlatelet.size());
		System.out.println("Created: "+createdCells+"  -  NOT Created: "+notCreatedCells);
		System.out.println("=========================");

		Random rr=new Random();
		//SimpleVector dVersor=new SimpleVector(0,1,0);		
		SimpleVector dVersor=mainCylinder.getDirectionP();		


		//aa=38 for long vessel
		//aa=24 complete venule
		//aa=17 short venule
		for(int aa=1;aa<38;aa++){ // da 1 a 38  (LONG VESSEL!)
			int size=firstMacrophage.size();
			System.out.println("=========================");
			System.out.println("Macrophage - SIZE: "+size+" - aa: "+aa);
			System.out.println("Blood Cell - SIZE: "+firstRedBloodCell.size()+" - aa: "+aa);
			System.out.println("Platelet Cell - SIZE: "+firstPlatelet.size()+" - aa: "+aa);
			System.out.println("=========================");


			float angle = (float) ((2*Math.PI)*rr.nextDouble());

			for(int i=0;i<size;i++){
				float[] center=new float[3];				
				center[0]=firstMacrophage.get(i).getCenter()[0];
				center[1]=firstMacrophage.get(i).getCenter()[1];
				center[2]=firstMacrophage.get(i).getCenter()[2];

				center[0]=center[0]-100000;
				center[1]=center[1]-100000+(88500+5000)*aa;
				center[2]=center[2]-100000;

				SimpleVector centerVector=new SimpleVector(center);		

				Matrix rotation = new Matrix();				
				rotation.rotateAxis(dVersor, angle);
				centerVector.rotate(rotation);

				centerVector.x=centerVector.x+100000;
				centerVector.y=centerVector.y+100000;
				centerVector.z=centerVector.z+100000;

				/*  BUILDS THE MACROPHAGE CELL */
				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage=new Macrophage(mainCylinder,centerVector.x,centerVector.y,centerVector.z,15,observer);				
				macrophage.burstEmission=0;

				if (macrophage.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;					
					setInitializationVelocity(macrophage);	
				}else{
					wastedAttempt++;
				}
			}

			int size2=firstRedBloodCell.size();			

			for(int i=0;i<size2;i++){				
				float[] center=new float[3];

				center[0]=firstRedBloodCell.get(i).getCenter()[0];
				center[1]=firstRedBloodCell.get(i).getCenter()[1];
				center[2]=firstRedBloodCell.get(i).getCenter()[2];

				center[0]=center[0]-100000;
				center[1]=center[1]-100000+(88500+5000)*aa;
				center[2]=center[2]-100000;

				SimpleVector centerVector=new SimpleVector(center);				

				Matrix rotation = new Matrix();				
				rotation.rotateAxis(dVersor, angle);
				centerVector.rotate(rotation);

				centerVector.x=centerVector.x+100000;
				centerVector.y=centerVector.y+100000;
				centerVector.z=centerVector.z+100000;

				/*  BUILDS THE RED BLOOD CELL */
				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				RedBloodCell redBloodCell=new RedBloodCell(mainCylinder,centerVector.x,centerVector.y,centerVector.z,14,observer);
				redBloodCell.burstEmission=0;

				if (redBloodCell.getTimeToLive()>0){
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					setInitializationVelocity(redBloodCell);

				}else{
					wastedAttempt++;
					redBloodCell.destroy();
				}
			}

			int size3=firstPlatelet.size();

			for(int i=0;i<size3;i++){				
				float[] center=new float[3];

				center[0]=firstPlatelet.get(i).getCenter()[0];
				center[1]=firstPlatelet.get(i).getCenter()[1];
				center[2]=firstPlatelet.get(i).getCenter()[2];

				center[0]=center[0]-100000;
				center[1]=center[1]-100000+(88500+5000)*aa;
				center[2]=center[2]-100000;

				SimpleVector centerVector=new SimpleVector(center);						

				Matrix rotation = new Matrix();			
				rotation.rotateAxis(dVersor, angle);
				centerVector.rotate(rotation);

				centerVector.x=centerVector.x+100000;
				centerVector.y=centerVector.y+100000;
				centerVector.z=centerVector.z+100000;			

				/*  BUILDS THE PLATELET CELL */
				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				PlateletVessel platelet1=new PlateletVessel(mainCylinder,centerVector.x,centerVector.y,centerVector.z,13,observer);
				platelet1.burstEmission=0;

				if (platelet1.getTimeToLive()>0) {
					nodeCount++;	
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;					
					setInitializationVelocity(platelet1);

				}else{
					wastedAttempt++;
					platelet1.destroy();
				}
			}
		}		
	}


	public void emissionProcessInVesselDeterministicINITIALIZE_LongVessel_DEBUG_FLUSSO(CylindricBloodVesselDomain mainCylinder){

		ArrayList<Macrophage> firstMacrophage=new ArrayList<Macrophage>();
		ArrayList<RedBloodCell> firstRedBloodCell=new ArrayList<RedBloodCell>();
		ArrayList<PlateletVessel> firstPlatelet=new ArrayList<PlateletVessel>();

		/**  CREATION EVERY 33 step (if timeStep=1000 ns), 3.3 step (if timeStep=1000)  */
		List<LigandObserver> observer=new ArrayList<LigandObserver>();

		float bound=mainCylinder.getRadius();
		float depth=88500;
		//float tempDepth=-(2500000)/2+250000;  	// ok
		float displacement=-1400000+depth/2;
		//float displacement=600000+depth/2;

		float[] initialDeploy={mainCylinder.getAbsCenter()[0],mainCylinder.getAbsCenter()[1]+displacement,mainCylinder.getAbsCenter()[2]};
		//initialDeploy[1]=-775000+100000-7500-depth/2;
		float xx=0;
		float yy=0;
		float zz=0;
		int wastedLimit=2000;		// 2000 ok!
		int wastedAttempt=0;
		boolean hasFinished=false;
		float cylQuadraticRadius=mainCylinder.getRadius()*mainCylinder.getRadius();


		//WrapperBloodCells wrapper=WrapperBloodCells.getInstance();


		// MACROPHAGE
		float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(15+"-radius").floatValue();
		bound=bound-radius;

		try {
			double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);

			xx=(float)results[0];
			yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
			zz=(float)results[1];

			int iter=2;
			for (int i=1;i<iter;i=i+2){

				int ds=19000;
				//int ds=10000;

				xx=(float)5000;
				yy=(float)(-depth/2)+(i-1)*ds;
				zz=(float)5000;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage31=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage31.burstEmission=0;
				macrophage31.setColor(Color.gray);
				macrophage31.debugName="gray";

				if (macrophage31.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage31);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage31);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}
				/*
				xx=(float)5000;
				yy=(float)(-depth/2)+(i-1)*ds;
				zz=(float)-5000;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage32=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage32.burstEmission=0;
				macrophage32.setColor(Color.black);
				macrophage32.debugName="black";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage32.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage32);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage32);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}



				xx=(float)-5000;
				yy=(float)(-depth/2)+(i-1)*ds;
				zz=(float)-5000;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage33=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage33.burstEmission=0;
				macrophage33.setColor(Color.blue);
				macrophage33.debugName="blue";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage33.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage33);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage33);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}

				xx=(float)-5000;
				yy=(float)(-depth/2)+(i-1)*ds;
				zz=(float)5000;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage34=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage34.burstEmission=0;
				macrophage34.setColor(Color.cyan);
				macrophage34.debugName="cyan";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage34.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage34);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage34);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}


				xx=(float)0;
				yy=(float)(-depth/2)+(i)*ds;
				zz=(float)7500;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage35=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage35.burstEmission=0;
				macrophage35.setColor(Color.green);
				macrophage35.debugName="green";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage35.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage35);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage35);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}

				xx=(float)7500;
				yy=(float)(-depth/2)+(i)*ds;
				zz=(float)0;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage36=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage36.burstEmission=0;
				macrophage36.setColor(Color.lightGray);
				macrophage36.debugName="lightGray";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage36.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage36);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage36);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}

				xx=(float)-7500;
				yy=(float)(-depth/2)+(i)*ds;
				zz=(float)0;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage37=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage37.burstEmission=0;
				macrophage37.setColor(Color.magenta);
				macrophage37.debugName="magenta";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage37.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage37);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage37);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}

				xx=(float)0;
				yy=(float)(-depth/2)+(i)*ds;
				zz=(float)-7500;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage38=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage38.burstEmission=0;
				macrophage38.setColor(Color.orange);
				macrophage38.debugName="orange";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage38.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage38);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage38);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}


				xx=(float)0;
				yy=(float)(-depth/2);
				zz=(float)0;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage39=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage39.burstEmission=0;
				macrophage39.setColor(Color.pink);
				macrophage39.debugName="pink";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage39.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage39);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage39);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}


				xx=(float)0;
				yy=(float)(-depth/2)-10000;
				zz=(float)7500;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage35a=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage35a.burstEmission=0;
				macrophage35a.setColor(Color.green);
				macrophage35a.debugName="green1";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage35a.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage35a);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage35a);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}

				xx=(float)7500;
				yy=(float)(-depth/2)-10000;
				zz=(float)0;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage36a=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage36a.burstEmission=0;
				macrophage36a.setColor(Color.lightGray);
				macrophage36a.debugName="lightGray1";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage36a.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage36a);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage36a);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}

				xx=(float)-7500;
				yy=(float)(-depth/2)-10000;
				zz=(float)0;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage37a=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage37a.burstEmission=0;
				macrophage37a.setColor(Color.magenta);
				macrophage37a.debugName="magenta1";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage37a.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage37a);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage37a);


				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}

				xx=(float)0;
				yy=(float)(-depth/2)-10000;
				zz=(float)-7500;

				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());
				Macrophage macrophage38a=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
				macrophage38a.burstEmission=0;
				macrophage38a.setColor(Color.orange);
				macrophage38a.debugName="orange1";
				//macrophage3.setMass((float) (macrophage3.getMass()*0.01));

				if (macrophage38a.getTimeToLive()>0) {
					nodeCount++;			
					wastedAttempt=0;					
					createdCells++;
					hasFinished=true;
					firstMacrophage.add(macrophage38a);
					//wrapper.addMacrophageList(new WrapperNanoObj(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer));
					setInitializationVelocity(macrophage38a);

				}else{
					wastedAttempt++;
				}
				if(wastedAttempt>wastedLimit) {
					notCreatedCells++;
					hasFinished=true;
				}*/
			}	

		} catch (Exception e) {
			e.printStackTrace();
		}		


		System.out.println();		
		System.out.println("========  DEBUG  ========");
		System.out.println("Macrophage - SIZE: "+firstMacrophage.size());
		System.out.println("Blood Cell - SIZE: "+firstRedBloodCell.size());
		System.out.println("Platelet Cell - SIZE: "+firstPlatelet.size());
		System.out.println("Created: "+createdCells+"  -  NOT Created: "+notCreatedCells);
		System.out.println("=========================");	

	}

	public void emissionProcess_DEBUG_RECETTORI(CylindricBloodVesselDomain mainCylinder){

		ArrayList<Macrophage> firstMacrophage=new ArrayList<Macrophage>();		

		List<LigandObserver> observer=new ArrayList<LigandObserver>();

		float bound=mainCylinder.getRadius();
		float depth=88500;
		float displacement=-1400000+depth/2;

		float[] initialDeploy={mainCylinder.getAbsCenter()[0],mainCylinder.getAbsCenter()[1]+displacement,mainCylinder.getAbsCenter()[2]};

		float xx=0;
		float yy=0;
		float zz=0;		


		// MACROPHAGE
		float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(15+"-radius").floatValue();
		bound=bound-radius;

		try {

			double[] results=getFlatCoordinates(mainCylinder.getRadius(),radius);

			xx=(float)results[0];
			yy=(float)((rand.nextFloat()*2*depth/2)-depth/2);
			zz=(float)results[1];


			//FIXME Only for DEBUG!

			xx=(float)10000;
			//yy=(float)-depth/2;
			zz=(float)-10000;

			//  BUILDS THE MACROPHAGE CELL 
			observer=new ArrayList<LigandObserver>();
			observer.add(CD40LObserver.getInstance());
			Macrophage macrophage=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
			macrophage.burstEmission=0;
			macrophage.setColor(Color.white);
			macrophage.debugName="white";

			if (macrophage.getTimeToLive()>0) {
				nodeCount++;									
				createdCells++;					
				firstMacrophage.add(macrophage);					
				setInitializationVelocity(macrophage);
				macrophage.setMotionStrategy(NoMotion.getInstance());				
			}	


			for(int o=0;o<10000;o++){

				double[] coords=getFlatCoordinates(mainCylinder.getRadius(),2);


				xx=(float)coords[0];
				//yy=(float)(float)((rand.nextFloat()*2*depth/2)-depth/2);
				zz=(float)coords[1];

				//  BUILDS THE CARRIER 
				observer=new ArrayList<LigandObserver>();
				observer.add(CD40LObserver.getInstance());

				float mod=10;				

				float fi=0;
				//float theta=(float) (+1.98*Math.PI/3);
				float theta=(float) (0.2*Math.PI);

				//CD40LCarrier cc=new CD40LCarrier(mainCylinder, initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,1);			
				CD40LCarrier cc=new CD40LCarrier(mainCylinder, initialDeploy[0]+xx,yy,initialDeploy[2]+zz,1);			

				cc.setMotionStrategy(LinearMotion.getInstance());
				cc.setVelocityComponent(mod, fi, theta);
				cc.setColor(Color.black);
			}

			//FIXME Only for DEBUG!
			xx=(float)20200;
			yy=(float)1300000;
			zz=(float)-20200;

			//  BUILDS THE MACROPHAGE CELL 
			observer=new ArrayList<LigandObserver>();
			observer.add(CD40LObserver.getInstance());
			Macrophage macrophage1=new Macrophage(mainCylinder,initialDeploy[0]+xx,initialDeploy[1]+yy,initialDeploy[2]+zz,15,observer);
			macrophage1.burstEmission=0;
			macrophage1.setColor(Color.white);
			macrophage1.debugName="white";

			if (macrophage1.getTimeToLive()>0) {
				nodeCount++;									
				createdCells++;					
				firstMacrophage.add(macrophage1);					
				setInitializationVelocity(macrophage1);
				macrophage1.setMotionStrategy(NoMotion.getInstance());				
			}	

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public boolean createFixedPlatelet(CylindricBloodVesselDomain mainCylinder, int level){

		float depth=88500;		
		float tempDepth=-(2500000)/2+250000;  	
		float[] pos={mainCylinder.getAbsCenter()[0],mainCylinder.getAbsCenter()[1]+tempDepth,mainCylinder.getAbsCenter()[2]};				
		boolean created=false;
		float bound=mainCylinder.getRadius();
		float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(12+"-radius").floatValue();
		bound=(float) (bound*Math.cos(Math.PI/4)-(2*radius));
		float levelSize=bound/5;

		bound=bound-levelSize*level;		

		float xx=bound;
		float yy=depth/2;
		float zz=bound;

		yy=-775000+100000-7500+400000;

		/*  BUILDS PLATELET CELL */
		ArrayList<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer.add(CD40LObserver.getInstance());		
		PlateletVessel platelet1=new PlateletVessel(mainCylinder,pos[0]+xx,yy,pos[2]+zz,12,observer);		
		platelet1.setColor(Color.RED);		
		checkIsFullInside(platelet1);

		if (platelet1.getTimeToLive()>0) {
			nodeCount++;	
			created=true;
			createdCells++;			
			setInitializationVelocity(platelet1);
		}else{
			created=false;
			notCreatedCells++;
		}		
		return created;
	}

	public boolean createFixedPlatelet_LongVessel(CylindricBloodVesselDomain mainCylinder, int level){

		//float depth=88500;		
		//float tempDepth=-(2500000)/2+250000;  	
		float[] pos={mainCylinder.getAbsCenter()[0],mainCylinder.getAbsCenter()[1],mainCylinder.getAbsCenter()[2]};				
		boolean created=false;
		float bound=mainCylinder.getRadius();
		float radius=mainCylinder.getManager().configWrapper.nodeConfig.get(12+"-radius").floatValue();
		bound=(float) (bound*Math.cos(Math.PI/4)-(2*radius));
		float levelSize=bound/5;

		bound=bound-levelSize*level;		

		float xx=bound;
		float yy=-1350000;	// era 1000000		// ORIGINAL -1.35E6
		float zz=bound;
		yy=650000;								// BUGFIX




		/*  BUILDS PLATELET CELL */
		/**/
		ArrayList<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer.add(CD40LObserver.getInstance());		
		PlateletVesselTX platelet1=new PlateletVesselTX(mainCylinder,pos[0]+xx,pos[1]+yy,pos[2]+zz,12,observer);		
		platelet1.setColor(Color.RED);		
		/**/
		//checkIsFullInside(platelet1);	// Non serve e l'implementazione  incompleta



		/*  BUILDS FIXED STATIC PLATELET CELL */	
		/*
		ArrayList<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer.add(CD40LObserver.getInstance());		
		PlateletVesselTX_STATIC platelet1=new PlateletVesselTX_STATIC(mainCylinder,pos[0]+xx,pos[1]+yy,pos[2]+zz,12,observer);		
		platelet1.setColor(Color.RED);			
		 */

		if (platelet1.getTimeToLive()>0) {
			nodeCount++;	
			created=true;
			createdCells++;			
			setInitializationVelocity(platelet1);
			System.out.println(">>> TX NODE DISPOSED!!! (Platelet)");
			//System.out.println(">>> TX NODE DISPOSED!!! (Platelet STATIC)");
		}else{
			created=false;
			notCreatedCells++;
		}	


		/*
		yy=yy-11000;

		// DEBUG
		//  BUILDS PLATELET CELL 
		ArrayList<LigandObserver> observer1=new ArrayList<LigandObserver>();
		observer1.add(CD40LObserver.getInstance());		
		Macrophage nn=new Macrophage(mainCylinder,pos[0]+xx,pos[1]+yy,pos[2]+zz,15,observer1);		
		nn.setColor(Color.WHITE);	

		if (nn.getTimeToLive()>0) {
			nodeCount++;	
			created=true;
			createdCells++;			
			setInitializationVelocity(nn);
			System.out.println(">>> NODE DISPOSED!!! (DEBUG)");
		}else{
			created=false;
			notCreatedCells++;
		}		
		 */


		return created;
	}

	public void checkIsFullInside(NanoObj n){
		/** FIXME EXPERIMENTAL */	
		/*
		CylindricBloodVesselDomain domain=(CylindricBloodVesselDomain) n.getDomain();				
		float[] distances=domain.getCollisionStrategy().getDistanceOnAxis(domain,n);	
		float distanceP= distances[0]+n.getRadius();		
		float distanceN= distances[1]+n.getRadius();			

		if ((Math.abs(distanceP))>=domain.getHeight()/2){	
			n.setTimeToLive(-1);		

		} else if(Math.abs(distanceN)>=domain.getRadius()){
			System.out.println("");
			if (distanceN+n.getRadius()>30000){				
			}
			n.setTimeToLive(-1);	
		}
		 */
		/** END EXPERIMENTAL */
	}

	public void networkCreation(){	
		plasmaViscosity=getManager().configWrapper.viscosity;

		//networkCreationForEndothelium();		// short vessel
		networkCreationForEndothelium_LongVessel(false);
	}

	public void algorithm(AbstractDomain d){
		AbstractDomain domain=d;
		List<NanoObj> nanoObjList=d.getAllDisposedObj();
		try{
			long start0=System.currentTimeMillis();
			move(nanoObjList);
			long stop0=System.currentTimeMillis();	
			moveTime+=(stop0-start0);				
		}catch (Exception e){
			System.out.println("-- Error on BloodFlowModel.Algorithm.Move: "+e.toString());
		}
		try{
			long start0=System.currentTimeMillis();			
			if (nanoObjList.size()>0) checkCollision(nanoObjList);   			
			long stop0=System.currentTimeMillis();		
			checkTime+=(stop0-start0);
		}catch (Exception e){
			System.out.println("-- Error on BloodFlowModel.Algorithm.Checkcollision: "+e.toString());
		}
		try{
			long start0=System.currentTimeMillis();		
			while(domain.getCollisionsList().size()>0){					
				avoidCollision(domain);	
			}
			long stop0=System.currentTimeMillis();		
			avoidTime+=(stop0-start0);
		}catch (Exception e){
			System.out.println("-- Error on BloodFlowModel.Algorithm.AvoidCollision: "+e.toString());
		}
	}		

	public void detectCollision(NanoObj p1,NanoObj p2){

		try {
			float[] center1=p1.getCenter();
			float[] center2=p2.getCenter();
			float[] velocity1=p1.getVelocityComponent();
			float[] velocity2=p2.getVelocityComponent();
			double r1=p1.getRadius();
			double r2=p2.getRadius();

			float dx = (center2[0]-center1[0]), dy = (center2[1]-center1[1]), dz= (center2[2]-center1[2]);

			// where x1,y1,z1 are center of ball1, and x2,y2,z2 are center of ball2
			float distance = (float) Math.sqrt(dx*dx+dy*dy+dz*dz);

			if (distance<((r1+r2))){

				// Unit vector in the direction of the collision
				float ax=(dx/distance), ay=(dy/distance), az=(dz/distance);

				// Projection of the velocities in these axes
				float va1=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az); 
				float va2=(velocity2[0]*ax+velocity2[1]*ay+velocity2[2]*az);


				// the collision should have occurred at t-dt (Actually this is also an approximation).
				//float dt1=1;
				float dt=1;
				try {
					dt =(float) ((r1+r2-distance)/Math.abs(va1-va2)); // Warning! catch the div-by-zero!!!
					//dt =(float) ((r1+r2-distance)/Math.abs(va1+va2)); // Warning! catch the div-by-zero!!!
				} catch (Exception e) {
					System.out.println(e.getMessage());					

					dt=1;
				}

				p1.getDomain().addToCollisionsList(p1,p2,dt);				
			}
		} catch (Exception e) {
			System.out.println("-- BloodFlowModel.Algorithm.detectCollision-INSIDE!: "+e.toString());
			e.printStackTrace();
		}		
	}		

	public void move(List<NanoObj> nanoList){
		if (nanoList.size()>0){
			try{						
				threadingStrategy.move(nanoList);
			}catch (Exception e){
				e.printStackTrace();
			}			
		}
	}

	public void checkCollision(List<NanoObj> objsToCheck){		
		if (objsToCheck.size()>0){
			try{					
				threadingStrategy.checkCollision(objsToCheck);
			}catch (Exception e){
				e.printStackTrace();
			}			
		}		
	}

	// invoked by DECUZZI
	public float getPouseuilleCoefficient(CylindricBloodVesselDomain d, NanoObj p){		
		float velocityRegime = (float) (5*(Math.pow(10, -4)));
		float R = (float) (p.getRadius()/Math.pow(10,-p.getManager().configWrapper.u));   // NanoObj RADIUS [m]	
		float T=p.getManager().configWrapper.T;  // Temperature in Kelvin degrees
		float vesselRadius=(float) (d.getRadius()*Math.pow(10,p.getManager().configWrapper.u));
		float convectiveForSmallParticles=(float) ((vesselRadius*vesselRadius*velocityRegime*velocityRegime*Math.PI*plasmaViscosity*R)/(8*k*T));		
		return convectiveForSmallParticles;
	}	

	public synchronized void avoidCollision(AbstractDomain d){		
		boolean isDone=false;		
		try {			

			isDone=reorderCollision_byHeap(d);

		}catch (Exception e) {
			System.out.println("-- Error on avoidCollision. Collisionlist: "+d.getCollisionsList().size()+" - "+e.toString());
			e.printStackTrace();
		}

		try{			
			int size=d.getCollisionsList().size(); 	

			for (int i=0;i<size;i++){				
				d.getCollisionsList().get(i).handleCollision();						
			}			

		}catch (Exception e) {
			System.out.println("-- Error on avoidCollision. "+d.getCollisionsList().size()+" - step:"+step+" - "+e.toString()+" - isDone: "+isDone);
			e.printStackTrace();
		}		
		d.getCollisionsList().clear();			
	}


	// FIXME RelocateObs by SimpleVector startegy...  TO DEBUG...
	public void relocateObjs_SIMPLEVECTOR(NanoObj p1, NanoObj p2) {

		float[] center1=p1.getCenter();
		float[] center2=p2.getCenter();
		float[] velocity1=p1.getVelocityComponent();
		float[] velocity2=p2.getVelocityComponent();
		float r1=p1.getRadius();
		float r2=p2.getRadius();
		float modVelocity1=p1.getVelocityModule();
		float modVelocity2=p2.getVelocityModule();
		float mass1=p1.getMass();
		float mass2=p2.getMass();
		double dt=0;

		// The momentum has to be <= before and after the collision!
		double momentum1=modVelocity1*mass1;
		double momentum2=modVelocity2*mass2;
		double momentumBefore=momentum1+momentum2;



		// FIXME DEBUG ONLY
		float xxx1=center1[0];
		float yyy1=center1[1];
		float zzz1=center1[2];

		float xxx2=center2[0];
		float yyy2=center2[1];
		float zzz2=center2[2];

		float debug_v1x=p1.getVelocityComponent()[0];
		float debug_v1y=p1.getVelocityComponent()[1];
		float debug_v1z=p1.getVelocityComponent()[2];

		float debug_v2x=p2.getVelocityComponent()[0];
		float debug_v2y=p2.getVelocityComponent()[1];
		float debug_v2z=p2.getVelocityComponent()[2];

		float initialV1=modVelocity1;
		float initialV2=modVelocity2;


		float fkfkf=getManager().get_dt();

		velocity1[0]=velocity1[0]/getManager().get_dt();
		velocity1[1]=velocity1[1]/getManager().get_dt();
		velocity1[2]=velocity1[2]/getManager().get_dt();

		velocity2[0]=velocity2[0]/getManager().get_dt();
		velocity2[1]=velocity2[1]/getManager().get_dt();
		velocity2[2]=velocity2[2]/getManager().get_dt();



		if (modVelocity1==0 && modVelocity2==0){

		} else{

			//FIXME DEBUG
			nanoCollision++;

			float dx21 = (center2[0]-center1[0]);
			float dy21 = (center2[1]-center1[1]);
			float dz21 = (center2[2]-center1[2]);

			float dx12 = (center1[0]-center2[0]);
			float dy12 = (center1[1]-center2[1]);
			float dz12 = (center1[2]-center2[2]);

			float distance = (float) Math.sqrt(dx21*dx21+dy21*dy21+dz21*dz21);

			if (distance<((r1+r2))){


				SimpleVector v1=new SimpleVector(velocity1);
				SimpleVector v2=new SimpleVector(velocity2);

				SimpleVector v_dist1=new SimpleVector(dx21,dy21,dz21);
				SimpleVector v_dist2=new SimpleVector(dx12,dy12,dz12);


				//v1.scalarMul(1/getManager().get_dt());
				//v2.scalarMul(1/getManager().get_dt());

				// Unit vector in the direction of the collision
				//float ax=((dx21/distance)), ay=((dy21/distance)), az=((dz21/distance));

				/*
				float modVa1=(float) (Math.abs(modVelocity1)*Math.cos(alfa));
				float vax1=(float) (Math.abs(modVa1)*Math.cos(alfaX1));
				float vay1=(float) (Math.abs(modVa1)*Math.cos(alfaY1));
				float vaz1=(float) (Math.abs(modVa1)*Math.cos(alfaZ1));
				 */

				float modVa1=Math.abs(v1.calcDot(v_dist1.normalize()));

				// Velocity component on collision's direction
				SimpleVector va1=new SimpleVector(v_dist1.normalize());
				va1.scalarMul(modVa1);

				// Velocity component on normal direction
				SimpleVector vb1=v1.calcSub(va1);

				float vax1=va1.x;
				float vay1=va1.y;
				float vaz1=va1.z;

				float vbx1=vb1.x;
				float vby1=vb1.y;
				float vbz1=vb1.z;

				/*
				float vbx1=velocity1[0]-vax1;
				float vby1=velocity1[1]-vay1;
				float vbz1=velocity1[2]-vaz1;
				 */


				float modVa2=Math.abs(v2.calcDot(v_dist2.normalize()));

				// Velocity component on collision's direction
				SimpleVector va2=new SimpleVector(v_dist2.normalize());
				va2.scalarMul(modVa2);

				// Velocity component on normal direction
				SimpleVector vb2=v2.calcSub(va2);

				float vax2=va2.x;
				float vay2=va2.y;
				float vaz2=va2.z;

				float vbx2=vb2.x;
				float vby2=vb2.y;
				float vbz2=vb2.z;


				/*
				// Velocity component on collision's direction
				float modVa2= (float) (Math.abs(modVelocity2)*Math.cos(beta));
				float vax2=(float) (Math.abs(modVa2)*Math.cos(alfaX2));
				float vay2=(float) (Math.abs(modVa2)*Math.cos(alfaY2));
				float vaz2=(float) (Math.abs(modVa2)*Math.cos(alfaZ2));

				float vbx2=velocity2[0]-vax2;
				float vby2=velocity2[1]-vay2;
				float vbz2=velocity2[2]-vaz2;
				 */

				/*
				float va1t=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az);
				float va2t=(velocity2[0]*(ax)+velocity2[1]*(ay)+velocity2[2]*(az));
				 */

				float va1t=modVa1;
				float va2t=modVa2;


				dt =((r1+r2-distance)/Math.abs(va1t-va2t)); 			

				double checkNum=(r1+r2-distance);
				double checkDen=(va1t-va2t);

				double bak_dt=dt;
				/**/	
				if (dt>1.1) {
					//System.out.println("dt: "+dt);
					dt=1;
					p1.setColor(Color.black);
					p2.setColor(Color.white);					
				}			

				/*
				center1[0]-=velocity1[0]*dt;
				center1[1]-=velocity1[1]*dt;
				center1[2]-=velocity1[2]*dt;
				 */

				center1[0]-=v1.x*dt;
				center1[1]-=v1.y*dt;
				center1[2]-=v1.z*dt;


				p1.setCenter(center1[0], center1[1], center1[2]);

				center2[0]-=v2.x*dt;
				center2[1]-=v2.y*dt;
				center2[2]-=v2.z*dt;

				p2.setCenter(center2[0], center2[1], center2[2]);



				float[] vaP1=new float[3];
				vaP1[0]=(float) (vax1 + (1+ed)*(vax2-vax1)/(1+mass1/mass2));
				vaP1[1]=(float) (vay1 + (1+ed)*(vay2-vay1)/(1+mass1/mass2));
				vaP1[2]=(float) (vaz1 + (1+ed)*(vaz2-vaz1)/(1+mass1/mass2));

				float[] vaP2=new float[3];
				vaP2[0]=(float) (vax2 + (1+ed)*(vax1-vax2)/(1+mass2/mass1));
				vaP2[1]=(float) (vay2 + (1+ed)*(vay1-vay2)/(1+mass2/mass1));
				vaP2[2]=(float) (vaz2 + (1+ed)*(vaz1-vaz2)/(1+mass2/mass1));

				float vxRound = ((vaP1[0]+vbx1));
				float vyRound = ((vaP1[1]+vby1));
				float vzRound = ((vaP1[2]+vbz1));
				velocity1[0]= vxRound;
				velocity1[1]= vyRound;
				velocity1[2]= vzRound;

				float fi1=p1.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[0];
				float theta1=p1.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[1];
				float nextVelocityModule1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
				p1.setVelocityComponent(nextVelocityModule1, fi1, theta1);

				vxRound = ((vaP2[0]+vbx2));
				vyRound = ((vaP2[1]+vby2));
				vzRound = ((vaP2[2]+vbz2));
				velocity2[0]= vxRound;
				velocity2[1]= vyRound;
				velocity2[2]= vzRound;

				float fi2=p2.getVelocityAngles(velocity2[0], velocity2[1], velocity2[2])[0];
				float theta2=p2.getVelocityAngles(velocity2[0], velocity2[1], velocity2[2])[1];
				float nextVelocityModule2=(float) Math.sqrt(velocity2[0]*velocity2[0]+velocity2[1]*velocity2[1]+velocity2[2]*velocity2[2]);
				p2.setVelocityComponent(nextVelocityModule2, fi2, theta2);

				/*
				float oldV1=p1.old_velocityModule;
				float old_fi1=p1.old_fi;
				float old_theta1=p1.old_theta;

				float oldV2=p2.old_velocityModule;
				float old_fi2=p2.old_fi;
				float old_theta2=p2.old_theta;

				 */	

				/*
				// Evaluates the Momentum!
				momentum1=nextVelocityModule1*mass1;				
				momentum2=nextVelocityModule2*mass2;				
				double momentumAfter=momentum1+momentum2;


				if((momentumBefore*1.1)<momentumAfter){

					double error=momentumBefore/momentumAfter;
					nextVelocityModule1=(float) (nextVelocityModule1*error);
					nextVelocityModule2=(float) (nextVelocityModule2*error);

					p1.setVelocityModule(nextVelocityModule1);
					p2.setVelocityModule(nextVelocityModule2);

					velocity1=p1.getVelocityComponent();
					velocity2=p2.getVelocityComponent();

				}*/

				center1[0]+=velocity1[0]*dt;
				center1[1]+=velocity1[1]*dt;
				center1[2]+=velocity1[2]*dt;			

				p1.setCenter(center1[0], center1[1], center1[2]);


				center2[0]+=velocity2[0]*dt;
				center2[1]+=velocity2[1]*dt;
				center2[2]+=velocity2[2]*dt;	

				p2.setCenter(center2[0], center2[1], center2[2]);

				//modVelocity1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
				//modVelocity2=(float) Math.sqrt(velocity2[0]*velocity2[0]+velocity2[1]*velocity2[1]+velocity2[2]*velocity2[2]);


				/**/
				//FIXME DEBUG
				float ddy1=yyy1-center1[1];
				float ddy2=yyy2-center2[1];

				float perc1=nextVelocityModule1/initialV1;
				float perc2=nextVelocityModule2/initialV2;


				if(nextVelocityModule1==0 || nextVelocityModule2==0){
					int ok=0;
				}
				System.out.println(nextVelocityModule1+"/"+initialV1+"     -     "+nextVelocityModule2+"/"+initialV2);
				System.out.println(perc1+" - "+perc2);

				if(center1[1]-p1.getDomain().getAbsCenter()[1]<10000 && center1[1]-p1.getDomain().getAbsCenter()[1]>-10000){

					if(ddy1==0){
						p1.setColor(Color.black);						
					}
					if(ddy2==0){
						p2.setColor(Color.white);						
					}					
					//System.out.println("[v.x: ]"+p1.getVelocityComponent()[0]+"    [v.y: ]"+p1.getVelocityComponent()[1]+"    [v.z: ]"+p1.getVelocityComponent()[2]);
				}

			}
		}
	}



	// FIXME It is the same method of the Diffusion Model (that is newer)
	public void relocateObjs(NanoObj p1, NanoObj p2) {


		float[] center1t=p1.getCenter();
		float[] center2t=p2.getCenter();
		float[] center1={center1t[0],center1t[1],center1t[2]};
		float[] center2={center2t[0],center2t[1],center2t[2]};

		float[] velocity1=p1.getVelocityComponent();
		float[] velocity2=p2.getVelocityComponent();
		float r1=p1.getRadius();
		float r2=p2.getRadius();
		float modVelocity1=p1.getVelocityModule();
		float modVelocity2=p2.getVelocityModule();
		float mass1=p1.getMass();
		float mass2=p2.getMass();
		double dt=0;

		// The momentum has to be <= before and after the collision!
		double momentum1=modVelocity1*mass1;
		double momentum2=modVelocity2*mass2;
		double momentumBefore=momentum1+momentum2;

		/*
		if(p1 instanceof PlateletVesselTX_STATIC || p2 instanceof PlateletVesselTX_STATIC){
			int ok=0;
			ok++;

		}*/

		if (modVelocity1==0 && modVelocity2==0){
			System.out.println("VELOCITA' NULLE!");
		} else{

			//FIXME DEBUG
			nanoCollision++;			


			float dx21 = (center2[0]-center1[0]);
			float dy21 = (center2[1]-center1[1]);
			float dz21 = (center2[2]-center1[2]);

			float dx12 = (center1[0]-center2[0]);
			float dy12 = (center1[1]-center2[1]);
			float dz12 = (center1[2]-center2[2]);

			float distance = (float) Math.sqrt(dx21*dx21+dy21*dy21+dz21*dz21);

			if (distance<((r1+r2))){

				// Unit vector in the direction of the collision
				float ax=((dx21/distance)), ay=((dy21/distance)), az=((dz21/distance));


				float alfa=0;
				if (modVelocity1*distance!=0){
					try{
						/*
							BigDecimal bg = new BigDecimal((velocity1[0]*dx21+velocity1[1]*dy21+velocity1[2]*dz21)/(modVelocity1*distance)); 
							bg = bg.setScale(3, BigDecimal.ROUND_HALF_UP); 
							alfa=(float) Math.acos(bg.doubleValue()); // angle between v and d
						 */

						float args=(velocity1[0]*dx21+velocity1[1]*dy21+velocity1[2]*dz21)/(modVelocity1*distance);

						if(args>1) {
							args=1;
						}else if(args<-1) {
							args=-1;
						}
						alfa=(float) Math.acos(args); // angle between v and d

					} catch (NumberFormatException e){
						e.printStackTrace();
					}
				} 

				if(ax>1 || ax<-1 || ay>1 || ay<-1 || az>1 || az<-1){
					System.out.println("==========================================================================");
					System.out.println(" RelocateNANOOBJ - gli ax ay e az eccedono i limiti dell'arccos() "+distance);
					System.out.println("==========================================================================");
				}

				float alfaX1=(float) Math.acos((dx21)/(distance));
				float alfaY1=(float) Math.acos((dy21)/(distance));
				float alfaZ1=(float) Math.acos((dz21)/(distance));

				float modVa1=(float) (Math.abs(modVelocity1)*Math.cos(alfa));
				float vax1=(float) (Math.abs(modVa1)*Math.cos(alfaX1));
				float vay1=(float) (Math.abs(modVa1)*Math.cos(alfaY1));
				float vaz1=(float) (Math.abs(modVa1)*Math.cos(alfaZ1));

				float vbx1=velocity1[0]-vax1;
				float vby1=velocity1[1]-vay1;
				float vbz1=velocity1[2]-vaz1;

				float beta=0;
				if (modVelocity2*distance!=0){	
					try{
						/*
							BigDecimal bg = new BigDecimal((velocity2[0]*dx12+velocity2[1]*dy12+velocity2[2]*dz12)/(modVelocity2*distance)); 
							bg = bg.setScale(3, BigDecimal.ROUND_HALF_UP); 
							beta=(float) Math.acos(bg.doubleValue()); // angle between v and d
						 */


						float args=(velocity2[0]*dx12+velocity2[1]*dy12+velocity2[2]*dz12)/(modVelocity2*distance);

						if(args>1) {
							args=1;
						}else if(args<-1) {
							args=-1;
						}

						beta=(float) Math.acos(args); // angle between v and d					


					} catch (NumberFormatException e){
						e.printStackTrace();
					}				
				}

				float alfaX2=(float) Math.acos((dx12)/(distance));
				float alfaY2=(float) Math.acos((dy12)/(distance));
				float alfaZ2=(float) Math.acos((dz12)/(distance));

				// Velocity component on collision's direction
				float modVa2= (float) (Math.abs(modVelocity2)*Math.cos(beta));
				float vax2=(float) (Math.abs(modVa2)*Math.cos(alfaX2));
				float vay2=(float) (Math.abs(modVa2)*Math.cos(alfaY2));
				float vaz2=(float) (Math.abs(modVa2)*Math.cos(alfaZ2));

				float vbx2=velocity2[0]-vax2;
				float vby2=velocity2[1]-vay2;
				float vbz2=velocity2[2]-vaz2;

				float va1t=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az);
				float va2t=(velocity2[0]*(ax)+velocity2[1]*(ay)+velocity2[2]*(az));


				dt =((r1+r2-distance)/Math.abs(va1t-va2t)); 	


				//double checkNum=(r1+r2-distance);
				//double checkDen=(va1t-va2t);


				if (dt>1.1) {					

					dt=1;
					//p1.setColor(Color.white);
					//p2.setColor(Color.white);
				}	

				dt=dt*2;

				center1[0]-=velocity1[0]*dt;
				center1[1]-=velocity1[1]*dt;
				center1[2]-=velocity1[2]*dt;


				p1.setCenter(center1[0], center1[1], center1[2]);

				center2[0]-=velocity2[0]*dt;
				center2[1]-=velocity2[1]*dt;
				center2[2]-=velocity2[2]*dt;


				p2.setCenter(center2[0], center2[1], center2[2]);

				float[] vaP1=new float[3];
				vaP1[0]=(float) (vax1 + (1+ed)*(vax2-vax1)/(1+mass1/mass2));
				vaP1[1]=(float) (vay1 + (1+ed)*(vay2-vay1)/(1+mass1/mass2));
				vaP1[2]=(float) (vaz1 + (1+ed)*(vaz2-vaz1)/(1+mass1/mass2));

				float[] vaP2=new float[3];
				vaP2[0]=(float) (vax2 + (1+ed)*(vax1-vax2)/(1+mass2/mass1));
				vaP2[1]=(float) (vay2 + (1+ed)*(vay1-vay2)/(1+mass2/mass1));
				vaP2[2]=(float) (vaz2 + (1+ed)*(vaz1-vaz2)/(1+mass2/mass1));

				float vxRound = ((vaP1[0]+vbx1));
				float vyRound = ((vaP1[1]+vby1));
				float vzRound = ((vaP1[2]+vbz1));
				velocity1[0]= vxRound;
				velocity1[1]= vyRound;
				velocity1[2]= vzRound;

				float fi1=p1.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[0];
				float theta1=p1.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[1];
				float nextVelocityModule1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
				p1.setVelocityComponent(nextVelocityModule1, fi1, theta1);

				vxRound = ((vaP2[0]+vbx2));
				vyRound = ((vaP2[1]+vby2));
				vzRound = ((vaP2[2]+vbz2));
				velocity2[0]= vxRound;
				velocity2[1]= vyRound;
				velocity2[2]= vzRound;

				float fi2=p2.getVelocityAngles(velocity2[0], velocity2[1], velocity2[2])[0];
				float theta2=p2.getVelocityAngles(velocity2[0], velocity2[1], velocity2[2])[1];
				float nextVelocityModule2=(float) Math.sqrt(velocity2[0]*velocity2[0]+velocity2[1]*velocity2[1]+velocity2[2]*velocity2[2]);
				p2.setVelocityComponent(nextVelocityModule2, fi2, theta2);

				// Evaluates the Momentum!
				momentum1=nextVelocityModule1*mass1;				
				momentum2=nextVelocityModule2*mass2;				
				double momentumAfter=momentum1+momentum2;

				if((momentumBefore*1.1)<momentumAfter){

					double error=momentumBefore/momentumAfter;
					nextVelocityModule1=(float) (nextVelocityModule1*error);
					nextVelocityModule2=(float) (nextVelocityModule2*error);

					p1.setVelocityModule(nextVelocityModule1);
					p2.setVelocityModule(nextVelocityModule2);

					velocity1=p1.getVelocityComponent();
					velocity2=p2.getVelocityComponent();

				}

				center1[0]+=velocity1[0]*dt;
				center1[1]+=velocity1[1]*dt;
				center1[2]+=velocity1[2]*dt;

				p1.setCenter(center1[0], center1[1], center1[2]);

				center2[0]+=velocity2[0]*dt;
				center2[1]+=velocity2[1]*dt;
				center2[2]+=velocity2[2]*dt;

				p2.setCenter(center2[0], center2[1], center2[2]);

				modVelocity1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
				modVelocity2=(float) Math.sqrt(velocity2[0]*velocity2[0]+velocity2[1]*velocity2[1]+velocity2[2]*velocity2[2]);

			}
		}	
	}

	// FIXME It is the same method of the Diffusion Model (that is newer)
	public void relocateObjs_FOR_STATIC_PLATELET(NanoObj p1, NanoObj p2) {



		// p2 is the STATIC PLATELET!!

		float[] center1t=p1.getCenter();
		float[] center2t=p2.getCenter();
		float[] center1={center1t[0],center1t[1],center1t[2]};
		float[] center2={center2t[0],center2t[1],center2t[2]};

		float[] velocity1=p1.getVelocityComponent();
		float[] velocity2=p2.getVelocityComponent();
		float r1=p1.getRadius();
		float r2=p2.getRadius();
		float modVelocity1=p1.getVelocityModule();
		float modVelocity2=p2.getVelocityModule();
		float mass1=p1.getMass();
		float mass2=p2.getMass();
		double dt=0;

		// The momentum has to be <= before and after the collision!
		double momentum1=modVelocity1*mass1;
		double momentum2=modVelocity2*mass2;
		double momentumBefore=momentum1+momentum2;

		/*
			if(p1 instanceof PlateletVesselTX_STATIC || p2 instanceof PlateletVesselTX_STATIC && (p1 instanceof Macrophage || p2 instanceof Macrophage)){
				int ok=0;
				ok++;
			}*/

		if (modVelocity1==0 && modVelocity2==0){
			System.out.println("VELOCITA' NULLE!");
		} else{

			//FIXME DEBUG
			nanoCollision++;			


			float dx21 = (center2[0]-center1[0]);
			float dy21 = (center2[1]-center1[1]);
			float dz21 = (center2[2]-center1[2]);

			float dx12 = (center1[0]-center2[0]);
			float dy12 = (center1[1]-center2[1]);
			float dz12 = (center1[2]-center2[2]);

			float distance = (float) Math.sqrt(dx21*dx21+dy21*dy21+dz21*dz21);

			if (distance<((r1+r2))){

				// Unit vector in the direction of the collision
				float ax=((dx21/distance)), ay=((dy21/distance)), az=((dz21/distance));


				float alfa=0;
				if (modVelocity1*distance!=0){
					try{
						/*
								BigDecimal bg = new BigDecimal((velocity1[0]*dx21+velocity1[1]*dy21+velocity1[2]*dz21)/(modVelocity1*distance)); 
								bg = bg.setScale(3, BigDecimal.ROUND_HALF_UP); 
								alfa=(float) Math.acos(bg.doubleValue()); // angle between v and d
						 */

						float args=(velocity1[0]*dx21+velocity1[1]*dy21+velocity1[2]*dz21)/(modVelocity1*distance);

						if(args>1) {
							args=1;
						}else if(args<-1) {
							args=-1;
						}
						alfa=(float) Math.acos(args); // angle between v and d

					} catch (NumberFormatException e){
						e.printStackTrace();
					}
				} 

				if(ax>1 || ax<-1 || ay>1 || ay<-1 || az>1 || az<-1){
					System.out.println("==========================================================================");
					System.out.println(" RelocateNANOOBJ - gli ax ay e az eccedono i limiti dell'arccos() "+distance);
					System.out.println("==========================================================================");
				}

				float alfaX1=(float) Math.acos((dx21)/(distance));
				float alfaY1=(float) Math.acos((dy21)/(distance));
				float alfaZ1=(float) Math.acos((dz21)/(distance));

				float modVa1=(float) (Math.abs(modVelocity1)*Math.cos(alfa));
				float vax1=(float) (Math.abs(modVa1)*Math.cos(alfaX1));
				float vay1=(float) (Math.abs(modVa1)*Math.cos(alfaY1));
				float vaz1=(float) (Math.abs(modVa1)*Math.cos(alfaZ1));

				float vbx1=velocity1[0]-vax1;
				float vby1=velocity1[1]-vay1;
				float vbz1=velocity1[2]-vaz1;

				float beta=0;
				if (modVelocity2*distance!=0){	
					try{
						/*
								BigDecimal bg = new BigDecimal((velocity2[0]*dx12+velocity2[1]*dy12+velocity2[2]*dz12)/(modVelocity2*distance)); 
								bg = bg.setScale(3, BigDecimal.ROUND_HALF_UP); 
								beta=(float) Math.acos(bg.doubleValue()); // angle between v and d
						 */


						float args=(velocity2[0]*dx12+velocity2[1]*dy12+velocity2[2]*dz12)/(modVelocity2*distance);

						if(args>1) {
							args=1;
						}else if(args<-1) {
							args=-1;
						}

						beta=(float) Math.acos(args); // angle between v and d					


					} catch (NumberFormatException e){
						e.printStackTrace();
					}				
				}

				float alfaX2=(float) Math.acos((dx12)/(distance));
				float alfaY2=(float) Math.acos((dy12)/(distance));
				float alfaZ2=(float) Math.acos((dz12)/(distance));

				// Velocity component on collision's direction
				float modVa2= (float) (Math.abs(modVelocity2)*Math.cos(beta));
				float vax2=(float) (Math.abs(modVa2)*Math.cos(alfaX2));
				float vay2=(float) (Math.abs(modVa2)*Math.cos(alfaY2));
				float vaz2=(float) (Math.abs(modVa2)*Math.cos(alfaZ2));

				float vbx2=velocity2[0]-vax2;
				float vby2=velocity2[1]-vay2;
				float vbz2=velocity2[2]-vaz2;

				float va1t=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az);
				float va2t=(velocity2[0]*(ax)+velocity2[1]*(ay)+velocity2[2]*(az));


				dt =((r1+r2-distance)/Math.abs(va1t-va2t)); 	


				//double checkNum=(r1+r2-distance);
				//double checkDen=(va1t-va2t);


				if (dt>1.1) {					

					dt=1;
					//p1.setColor(Color.white);
					//p2.setColor(Color.white);
				}	

				dt=dt*2;

				center1[0]-=velocity1[0]*dt;
				center1[1]-=velocity1[1]*dt;
				center1[2]-=velocity1[2]*dt;					

				p1.setCenter(center1[0], center1[1], center1[2]);


				center2[0]-=velocity2[0]*dt;
				center2[1]-=velocity2[1]*dt;
				center2[2]-=velocity2[2]*dt;					

				p2.setCenter(center2[0], center2[1], center2[2]);



				float[] vaP1=new float[3];
				vaP1[0]=(float) (vax1 + (1+ed)*(vax2-vax1)/(1+mass1/mass2));
				vaP1[1]=(float) (vay1 + (1+ed)*(vay2-vay1)/(1+mass1/mass2));
				vaP1[2]=(float) (vaz1 + (1+ed)*(vaz2-vaz1)/(1+mass1/mass2));

				float[] vaP2=new float[3];
				vaP2[0]=(float) (vax2 + (1+ed)*(vax1-vax2)/(1+mass2/mass1));
				vaP2[1]=(float) (vay2 + (1+ed)*(vay1-vay2)/(1+mass2/mass1));
				vaP2[2]=(float) (vaz2 + (1+ed)*(vaz1-vaz2)/(1+mass2/mass1));

				float vxRound = ((vaP1[0]+vbx1));
				float vyRound = ((vaP1[1]+vby1));
				float vzRound = ((vaP1[2]+vbz1));
				velocity1[0]= vxRound;
				velocity1[1]= vyRound;
				velocity1[2]= vzRound;

				float fi1=p1.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[0];
				float theta1=p1.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[1];
				float nextVelocityModule1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
				p1.setVelocityComponent(nextVelocityModule1, fi1, theta1);

				vxRound = ((vaP2[0]+vbx2));
				vyRound = ((vaP2[1]+vby2));
				vzRound = ((vaP2[2]+vbz2));
				velocity2[0]= vxRound;
				velocity2[1]= vyRound;
				velocity2[2]= vzRound;


				float fi2=p2.getVelocityAngles(velocity2[0], velocity2[1], velocity2[2])[0];
				float theta2=p2.getVelocityAngles(velocity2[0], velocity2[1], velocity2[2])[1];
				float nextVelocityModule2=(float) Math.sqrt(velocity2[0]*velocity2[0]+velocity2[1]*velocity2[1]+velocity2[2]*velocity2[2]);
				p2.setVelocityComponent(nextVelocityModule2, fi2, theta2);


				// Evaluates the Momentum!
				momentum1=nextVelocityModule1*mass1;				
				momentum2=nextVelocityModule2*mass2;				
				double momentumAfter=momentum1+momentum2;

				if((momentumBefore*1.1)<momentumAfter){

					double error=momentumBefore/momentumAfter;
					nextVelocityModule1=(float) (nextVelocityModule1*error);
					nextVelocityModule2=(float) (nextVelocityModule2*error);

					p1.setVelocityModule(nextVelocityModule1);
					p2.setVelocityModule(nextVelocityModule2);

					velocity1=p1.getVelocityComponent();
					velocity2=p2.getVelocityComponent();

				}

				center1[0]+=velocity1[0]*dt;
				center1[1]+=velocity1[1]*dt;
				center1[2]+=velocity1[2]*dt;

				p1.setCenter(center1[0], center1[1], center1[2]);

				center2[0]+=velocity2[0]*dt;
				center2[1]+=velocity2[1]*dt;
				center2[2]+=velocity2[2]*dt;

				p2.setCenter(center2[0], center2[1], center2[2]);					


				modVelocity1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
				modVelocity2=(float) Math.sqrt(velocity2[0]*velocity2[0]+velocity2[1]*velocity2[1]+velocity2[2]*velocity2[2]);

			}
		}	
	}

	@Deprecated
	// FIXME  To be controlled! Consider the NEW implementation of DiffusionModel.relocateObjs()!!!
	public void relocateObjs_OLDBLOODFLOW(NanoObj p1, NanoObj p2) {
		float[] center1=p1.getCenter();
		float[] center2=p2.getCenter();
		float[] velocity1=p1.getVelocityComponent();
		float[] velocity2=p2.getVelocityComponent();
		float r1=p1.getRadius();
		float r2=p2.getRadius();
		float modVelocity1=p1.getVelocityModule();
		float modVelocity2=p2.getVelocityModule();
		double dt=0;

		if (!p1.equals(p2)) {

			if (modVelocity1==0 && modVelocity2==0){

			} else{

				float dx21 = (center2[0]-center1[0]);
				float dy21 = (center2[1]-center1[1]);
				float dz21 = (center2[2]-center1[2]);

				float dx12 = (center1[0]-center2[0]);
				float dy12 = (center1[1]-center2[1]);
				float dz12 = (center1[2]-center2[2]);

				float distance = (float) Math.sqrt(dx21*dx21+dy21*dy21+dz21*dz21);

				if (distance<((r1+r2))){

					// Unit vector in the direction of the collision
					float ax=((dx21/distance)), ay=((dy21/distance)), az=((dz21/distance));

					float alfa=0;
					if (modVelocity1*distance!=0){
						try{
							// FIXME Trying to discard the BigDecimal class...
							/*
						BigDecimal bg = new BigDecimal((velocity1[0]*dx21+velocity1[1]*dy21+velocity1[2]*dz21)/(modVelocity1*distance)); 
						bg = bg.setScale(3, BigDecimal.ROUND_HALF_UP); 
						alfa=(float) Math.acos(bg.doubleValue()); // angle between v and d
							 */
							float args=(velocity1[0]*dx21+velocity1[1]*dy21+velocity1[2]*dz21)/(modVelocity1*distance);
							alfa=(float) Math.acos(args); // angle between v and d
						} catch (NumberFormatException e){
							e.printStackTrace();
						}
					} 

					float alfaX1=(float) Math.acos((dx21)/(distance));
					float alfaY1=(float) Math.acos((dy21)/(distance));
					float alfaZ1=(float) Math.acos((dz21)/(distance));

					float modVa1=(float) (Math.abs(modVelocity1)*Math.cos(alfa));
					float vax1=(float) (Math.abs(modVa1)*Math.cos(alfaX1));
					float vay1=(float) (Math.abs(modVa1)*Math.cos(alfaY1));
					float vaz1=(float) (Math.abs(modVa1)*Math.cos(alfaZ1));

					float vbx1=velocity1[0]-vax1;
					float vby1=velocity1[1]-vay1;
					float vbz1=velocity1[2]-vaz1;

					float beta=0;
					if (modVelocity2*distance!=0){	
						try{
							// FIXME Trying to discard the BigDecimal class...
							/*
						BigDecimal bg = new BigDecimal((velocity2[0]*dx12+velocity2[1]*dy12+velocity2[2]*dz12)/(modVelocity2*distance)); 
						bg = bg.setScale(3, BigDecimal.ROUND_HALF_UP); 
						beta=(float) Math.acos(bg.doubleValue()); // angle between v and d
							 */
							float args=(velocity2[0]*dx12+velocity2[1]*dy12+velocity2[2]*dz12)/(modVelocity2*distance);
							beta=(float) Math.acos(args); // angle between v and d
						} catch (NumberFormatException e){
							e.printStackTrace();
						}				
					}

					float alfaX2=(float) Math.acos((dx12)/(distance));
					float alfaY2=(float) Math.acos((dy12)/(distance));
					float alfaZ2=(float) Math.acos((dz12)/(distance));

					// Velocity component on collision's direction
					float modVa2= (float) (Math.abs(modVelocity2)*Math.cos(beta));
					float vax2=(float) (Math.abs(modVa2)*Math.cos(alfaX2));
					float vay2=(float) (Math.abs(modVa2)*Math.cos(alfaY2));
					float vaz2=(float) (Math.abs(modVa2)*Math.cos(alfaZ2));

					float vbx2=velocity2[0]-vax2;
					float vby2=velocity2[1]-vay2;
					float vbz2=velocity2[2]-vaz2;

					float va1t=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az);
					float va2t=(velocity2[0]*(ax)+velocity2[1]*(ay)+velocity2[2]*(az));

					dt =((r1+r2-distance)/Math.abs(va1t-va2t)); 					

					if (dt>1.1) {					
						dt=1;
					}				 

					center1[0]-=velocity1[0]*dt;
					center1[1]-=velocity1[1]*dt;
					center1[2]-=velocity1[2]*dt;
					p1.setCenter(center1[0], center1[1], center1[2]);

					center2[0]-=velocity2[0]*dt;
					center2[1]-=velocity2[1]*dt;
					center2[2]-=velocity2[2]*dt;
					p2.setCenter(center2[0], center2[1], center2[2]);	

					float mass1=p1.getMass();
					float mass2=p2.getMass();

					float[] vaP1=new float[3];
					vaP1[0]=(float) (vax1 + (1+ed)*(vax2-vax1)/(1+mass1/mass2));
					vaP1[1]=(float) (vay1 + (1+ed)*(vay2-vay1)/(1+mass1/mass2));
					vaP1[2]=(float) (vaz1 + (1+ed)*(vaz2-vaz1)/(1+mass1/mass2));

					float[] vaP2=new float[3];
					vaP2[0]=(float) (vax2 + (1+ed)*(vax1-vax2)/(1+mass2/mass1));
					vaP2[1]=(float) (vay2 + (1+ed)*(vay1-vay2)/(1+mass2/mass1));
					vaP2[2]=(float) (vaz2 + (1+ed)*(vaz1-vaz2)/(1+mass2/mass1));

					float vxRound = ((vaP1[0]+vbx1));
					float vyRound = ((vaP1[1]+vby1));
					float vzRound = ((vaP1[2]+vbz1));
					velocity1[0]= vxRound;
					velocity1[1]= vyRound;
					velocity1[2]= vzRound;

					float fi1=p1.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[0];
					float theta1=p1.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[1];
					float nextVelocityModule1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
					p1.setVelocityComponent(nextVelocityModule1, fi1, theta1);				

					vxRound = ((vaP2[0]+vbx2));
					vyRound = ((vaP2[1]+vby2));
					vzRound = ((vaP2[2]+vbz2));
					velocity2[0]= vxRound;
					velocity2[1]= vyRound;
					velocity2[2]= vzRound;

					float fi2=p2.getVelocityAngles(velocity2[0], velocity2[1], velocity2[2])[0];
					float theta2=p2.getVelocityAngles(velocity2[0], velocity2[1], velocity2[2])[1];
					float nextVelocityModule2=(float) Math.sqrt(velocity2[0]*velocity2[0]+velocity2[1]*velocity2[1]+velocity2[2]*velocity2[2]);
					p2.setVelocityComponent(nextVelocityModule2, fi2, theta2);

					center1[0]+=velocity1[0]*dt;
					center1[1]+=velocity1[1]*dt;
					center1[2]+=velocity1[2]*dt;
					p1.setCenter(center1[0], center1[1], center1[2]);

					center2[0]+=velocity2[0]*dt;
					center2[1]+=velocity2[1]*dt;
					center2[2]+=velocity2[2]*dt;
					p2.setCenter(center2[0], center2[1], center2[2]);

					modVelocity1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
					modVelocity2=(float) Math.sqrt(velocity2[0]*velocity2[0]+velocity2[1]*velocity2[1]+velocity2[2]*velocity2[2]);				
				}
			}
		}
	}	

	public float getPoiseuilleVelocity(CylindricBloodVesselDomain c, NanoObj p){

		// === Pipe parameters ===
		float pipeLength = (float) ((c.getHeight())/Math.pow(10,-p.getManager().configWrapper.u)); // vessel length [m]
		float pipeRadius = (float) ((c.getRadius())/Math.pow(10,-p.getManager().configWrapper.u)); // vessel radius (4 micrometers for a capillary) [m]
		float sectionArea = (float) (Math.PI*Math.pow(pipeRadius, 2));  // cross area of the vessel [m^2]				
		// CAPILLARY
		//float meanFlowVel = (float) (2*(Math.pow(10, -4)));  // mean velocity of the bloodflow [m/s] (for a capillary is 0.02 cm/s)		
		// VENULE
		//float meanFlowVel = (float) (2*(Math.pow(10, -3)));  // mean velocity of the bloodflow [m/s] (for a venule is < 0.3 cm/s)		

		// vvvvvvvvvvvvvvvvvvvvvvvvvvv
		// FIXME Both following lines are commented, because the meanFlowVel is set on simulation start (command line)
		//float meanFlowVel = (float) (5*(Math.pow(10, -4)));	//OK BLOOD FLOW, radius 30um	// TODO  You have to get the mean velocity for each vessel size!
		//float meanFlowVel = (float) (2*(Math.pow(10, -3)));	// TRY FOR radius = 60um	// TODO  You have to get the mean velocity for each vessel size!
		// === End pipe parameters ===


		// === PessureDifference ===
		float flowRate = sectionArea*meanFlowVel;   // [m^3/s]
		float pressureDifference = (float) ((8*plasmaViscosity*pipeLength*flowRate)/(Math.PI*Math.pow(pipeRadius, 4)));   // Hagen-Poiseuille formula [Pa]
		// === End of pressureDifference ===


		/** HAGEN-POISEUILLE EQUATIONS */
		// === Calculates the distance of the nano-object from the cylinder longitudinal axis ===
		/*
		float[] domainCenter = c.getAbsCenter();
		float[] nanoCenter = p.getCenter();
		float x = nanoCenter[0] - domainCenter[0];
		float y = nanoCenter[1] - domainCenter[1];
		float z = nanoCenter[2] - domainCenter[2];


		SimpleVector nanoObjRelativeCenter = new SimpleVector(x,y,z);	// FIXME ORIGINAL
		double nanoObjRelativeModule = Math.sqrt(x*x+y*y+z*z);		// FIXME ORIGINAL


		double angleP = c.getDirectionP().calcAngle(nanoObjRelativeCenter);		
		// FIXME Trying to use directionN...
		double angleN = (Math.PI/2) - angleP;		
		 */


		// NEW		
		float[] distances1=c.getCollisionStrategy().getDistanceOnAxis(c,p);	

		SimpleVector vectorN=new SimpleVector(c.getDirectionN());
		vectorN.scalarMul(distances1[1]);

		SimpleVector vectorO=new SimpleVector(c.getDirectionO());
		vectorO.scalarMul(distances1[2]);

		vectorN.add(vectorO);			

		double distanceN = (vectorN.length())*Math.pow(10,p.getManager().configWrapper.u);  // multiplied by 10^-9 because it is needed the distanceN in m!!

		// ORIGINAL
		/*
		//double angleN=vectorN.calcAngle(nanoObjRelativeCenter);		
		angleN=Math.round(angleN*1E7)/1E7;			
		double cosN=Math.cos(angleN);		
		cosN=Math.round(cosN*1E7)/1E7;		
		double distanceN = (nanoObjRelativeModule*cosN)*Math.pow(10,p.getManager().configWrapper.u);  // multiplied by 10^-9 because it is needed the distanceN in m!!
		 */
		// === End calculations	===		

		// flow velocity as a function of the distance from the longitudinal axis [m/s]
		float velocityRegime = (float) ((1/(4*plasmaViscosity))*(pressureDifference/pipeLength)*(Math.pow(pipeRadius, 2)-Math.pow(distanceN, 2)));			
		velocityRegime=(float) (Math.round(velocityRegime*1E7)/1E7);
		return velocityRegime;
	}	


	// ORIGINAL BEFORE DEBUG AND FIXING 22/01/2014
	public void laminarFlowMovement_OLD_OK(List<NanoObj> nanoObjList) {

		CylindricBloodVesselDomain c = (CylindricBloodVesselDomain) nanoObjList.get(0).getDomain();
		NanoObj p = null;

		for (int i=0; i<nanoObjList.size(); i++){
			p = nanoObjList.get(i);

			float[] center = p.getCenter();

			/** ================================================= computation of MM and M2 component =============================================*/
			// Get previous step velocity vector 
			SimpleVector flusso=c.getDirectionP().normalize();
			SimpleVector mm=new SimpleVector(p.getVelocityComponentMM());		
			SimpleVector ms=new SimpleVector(p.getVelocityComponentMS());

			SimpleVector oldv=new SimpleVector();		
			oldv.add(mm);
			oldv.add(ms);			

			SimpleVector oldvVERSOR=oldv.normalize();				
			float mmproj=(mm.x*flusso.x+mm.y*flusso.y+mm.z*flusso.z);			

			/** Projection of MM component on old velocity vector */
			float mm_su_oldv=(oldvVERSOR.x*mm.x+oldvVERSOR.y*mm.y+oldvVERSOR.z*mm.z);					
			SimpleVector mm_lungo_oldv=new SimpleVector(oldvVERSOR);
			mm_lungo_oldv.scalarMul(mm_su_oldv);
			/** ============================= */

			/** Projection of [mm_su_oldv] along the direction of the flow */
			//float mm_lungo_oldv_proj=(mm_lungo_oldv.x*flusso.x+mm_lungo_oldv.y*flusso.y+mm_lungo_oldv.z*flusso.z);
			/** ============================= */

			float k_mm_su_oldv=0;
			if(oldv.length()!=0){	
				k_mm_su_oldv=mm_su_oldv/oldv.length();	// Ratio between the projection of MM velocity component on old velocity vector and old velocity vector
				if(k_mm_su_oldv<0){
					k_mm_su_oldv=Math.abs(k_mm_su_oldv);	//FIXME Introduced to fix anomalous behaviour
				}

			}

			float k_mm_su_mmold=1;
			if(mm.length()!=0){
				// Angle between MM component and old velocity vector
				k_mm_su_mmold=mm_su_oldv/mm.length();		// Ratio between the projection of MM velocity component on old velocity vector and the original MM component
			}
			if (k_mm_su_mmold>1) k_mm_su_mmold=1;
			if (k_mm_su_mmold<-1) k_mm_su_mmold=-1;


			/** NEW VELOCITY */
			SimpleVector newv=new SimpleVector(p.getVelocityComponent());				
			SimpleVector newvversor=newv.normalize();		

			float mm_su_newV=k_mm_su_oldv*newv.length();

			//float anglebeta=mm.calcAngle(oldv);
			float beta=(float) Math.acos(k_mm_su_mmold);
			float gamma_old=(oldv.calcAngle(flusso));
			float sigma_old=gamma_old-beta;

			float sigma=0;
			float gamma=newv.calcAngle(flusso);

			if (gamma<=Math.PI/2) {
				sigma=gamma-beta;					
			}else if (gamma>Math.PI/2 && gamma<(1.5*Math.PI)){
				sigma=gamma+beta;
			}else{
				sigma=(float) (+gamma+beta);
			}

			if (sigma<0) sigma=beta-gamma;

			float mm2=mm_su_newV/k_mm_su_mmold;
			float mm2_su_flusso=(float) (mm2*Math.cos(sigma));

			SimpleVector mm_flusso=new SimpleVector(flusso);
			mm_flusso.scalarMul(mmproj);

			SimpleVector mm_trasverso=new SimpleVector(oldvVERSOR);
			mm_trasverso.scalarMul(mm_su_oldv);
			mm_trasverso=mm_trasverso.calcSub(mm_flusso);


			/** The cross product is used to get the orthogonal vector to flow! */
			SimpleVector mm_su_oldvVector=new SimpleVector(oldvVERSOR);
			mm_su_oldvVector.scalarMul(mm_su_oldv);
			SimpleVector asseRotazione0=mm_su_oldvVector.calcCross(flusso);

			float ro0=(float) (Math.PI/2-sigma_old);
			SimpleVector mm_trasverso2=new SimpleVector(flusso.normalize());

			float mm_traverso_module=(float) (mm.length()*Math.sin(sigma_old));
			if (sigma_old>Math.PI) mm_traverso_module=mm_traverso_module*-1;

			mm_trasverso2.scalarMul(mm_traverso_module);

			ro0=(float) Math.PI/2;
			Matrix rotation0 = new Matrix();
			rotation0.rotateAxis(asseRotazione0, ro0);
			mm_trasverso2.rotate(rotation0);

			SimpleVector mm_verifica=new SimpleVector();
			mm_verifica.add(mm_trasverso2);
			mm_verifica.add(mm_flusso);
			/** END of computation of the orthogonal vector */

			SimpleVector mm2_flusso=new SimpleVector(flusso);
			mm2_flusso.scalarMul(mm2_su_flusso);  	//TODO  This is the flow projection along MM2 direction. Needed in Poiseuille! 


			/** In fact, does not take into account the transvert component to the flow when the new velocity is parallel to the flow itself! */
			SimpleVector mm2_trasverso=new SimpleVector(newvversor);
			mm2_trasverso.scalarMul(mm_su_newV);
			mm2_trasverso=mm2_trasverso.calcSub(mm2_flusso);	//TODO  This is the orthogonal component to the flow projection along MM2 direction. Needed in Poiseuille!

			//SimpleVector cross=newvversor.calcCross(mm2_flusso);


			/** Uses the cross product to get the orthogonal vector to the flow direction */
			SimpleVector mm2_su_newV=new SimpleVector(newvversor);
			mm2_su_newV.scalarMul(mm_su_newV);
			SimpleVector asseRotazione=mm2_su_newV.calcCross(flusso);

			float ro=(float) (Math.PI/2-sigma);
			ro=(float) Math.PI/2;

			float[] nullcoords={0,0,0};		
			SimpleVector mm2_trasverso2=new SimpleVector(nullcoords);

			if (asseRotazione.length()==0){
				SimpleVector tempVector=new SimpleVector(mm2_su_newV);
				tempVector.x=(float) (tempVector.x+(0.001));
				tempVector.y=(float) (tempVector.y+(0.001));
				tempVector.z=(float) (tempVector.z+(0.001));
				asseRotazione=tempVector.calcCross(flusso);
			}

			mm2_trasverso2=new SimpleVector(flusso.normalize());

			float mm2_traverso2_module=(float) (mm2*Math.sin(sigma));
			if (sigma>Math.PI) mm2_traverso2_module=mm2_traverso2_module*-1;

			mm2_trasverso2.scalarMul(mm2_traverso2_module);
			Matrix rotation = new Matrix();
			rotation.rotateAxis(asseRotazione, ro);
			mm2_trasverso2.rotate(rotation);

			mm2_trasverso=mm2_trasverso2;

			SimpleVector mm_verifica2=new SimpleVector();
			mm_verifica2.add(mm2_trasverso2);
			mm_verifica2.add(mm2_flusso);
			/** END of transverse projection */				
			/** =================================================  END computation of MM and M2 component  =============================================*/


			/** HAGEN-POISEUILLE EQUATIONS */
			float timeUnit = p.getManager().configWrapper.timeUnit;
			float interval = (float) (timeUnit/Math.pow(10,-p.getManager().configWrapper.u));  // Time interval for the computation [s]

			// FIXME - DECUZZI: The carriers takes into account even the convective contribution
			if(p.getManager().configWrapper.ddt<1 && !(p instanceof CarrierObj)) {
				//if(p.getManager().configWrapper.ddt<1) {
				CylindricBloodVesselDomain d = (CylindricBloodVesselDomain) p.getDomain();
				float m = (float) (p.getMass()/Math.pow(10,-p.getManager().configWrapper.u));     // Mass of the nano-object [g]
				float R = (float) (p.getRadius()/Math.pow(10,-p.getManager().configWrapper.u));   // Radius of the nano-object [m]				

				float velocityRegime=getPoiseuilleVelocity(d, p);
				SimpleVector flowDir=new SimpleVector(d.getDirectionP());
				flowDir=flowDir.normalize();
				flowDir.scalarMul(velocityRegime);

				/** Devo dividere tutto il vettore non solo la componente lungo Y */		
				SimpleVector oldVelTemp=new SimpleVector(mm2_flusso);				
				oldVelTemp.scalarMul(1/interval);	// velocityOnFlux	 // oldVelTemp is a displacement, so it computes the velocity...        
				oldVelTemp.scalarMul(1/(float) Math.pow(10,-p.getManager().configWrapper.u));		// scales the result in te units of [m/s]						
				SimpleVector oldVelocity=oldVelTemp;

				float rx=flowDir.x-oldVelocity.x;
				float ry=flowDir.y-oldVelocity.y;
				float rz=flowDir.z-oldVelocity.z;
				SimpleVector relVel=new SimpleVector(rx,ry,rz);

				relVel.scalarMul((float) (6*Math.PI*plasmaViscosity*R));  // Stokes Law, [m/s]
				relVel.scalarMul(1/m);	// [m/s^2]
				relVel.scalarMul(interval/2);        // The Stokes Law gives the velocity [m/s]

				oldVelocity.add(relVel);	
				SimpleVector newVelocity=new SimpleVector(oldVelocity);
				newVelocity.scalarMul((float) (interval*Math.pow(10,-p.getManager().configWrapper.u)));

				// The new velocity along the flux direction have to be added to the old velocity, in order to keep track the old direction
				SimpleVector aaV1=new SimpleVector(newVelocity);  // Defines a new component along vMM. Up to now it is given only by the component along the flow direction



				/** ======================================================================= 
				 *  Computes the friction along the velocity component not parallel to the flow 
				 *  ======================================================================= */				
				/*
					SimpleVector attritoTemp=new SimpleVector(mm2_trasverso);		//TODO  Uses the MM2 component (transverse to the flow) obtained before by testVettori				
					attritoTemp.scalarMul(1/interval);				
					attritoTemp.scalarMul(1/(float) Math.pow(10,-p.getManager().configWrapper.u));	

					float rxx=-attritoTemp.x;
					float ryy=-attritoTemp.y;
					float rzz=-attritoTemp.z;
					SimpleVector relVelAttrito=new SimpleVector(rxx,ryy,rzz);

					relVelAttrito.scalarMul((float) (6*Math.PI*plasmaViscosity*R));  // Stokes Law [m/s]
					relVelAttrito.scalarMul(1/m);	// [m/s^2]
					relVelAttrito.scalarMul(interval/2);        // Velocity given by the Stokes Law [m/s]

					attritoTemp.add(relVelAttrito);	
					SimpleVector newVelocityTang=new SimpleVector(attritoTemp);
					newVelocityTang.scalarMul((float) (interval*Math.pow(10,-p.getManager().configWrapper.u)));
					aaV1.add(newVelocityTang);
				 */
				/** ----------- END FRICTION -------------- */

				float dx=aaV1.x;
				float dy=aaV1.y;
				float dz=aaV1.z;


				float newVelocityModule = aaV1.length();					
				float[] angles = p.getVelocityAngles(dx, dy, dz);
				p.setVelocityComponentMM(newVelocityModule, angles[0], angles[1]);				

				center[0]=center[0]+aaV1.x; 
				center[1]=center[1]+aaV1.y;  
				center[2]=center[2]+aaV1.z;
				p.setCenter(center[0],center[1],center[2]);

			}else{

				/** Carriers are handled only with the Special Brownian Diffusion (diffusion+convective) */				
				CylindricBloodVesselDomain d = (CylindricBloodVesselDomain) p.getDomain();
				float velocityRegime=getPoiseuilleVelocity(d, p);
				velocityRegime=(float) (velocityRegime*interval*Math.pow(10, 9));		

				float newVelocityModule = velocityRegime;
				float[] angles = p.getVelocityAngles(0, velocityRegime, 0);
				p.setVelocityComponentMM(newVelocityModule, angles[0], angles[1]);

				center[0]=center[0]; 
				center[1]=center[1]+velocityRegime;  
				center[2]=center[2];					
				p.setCenter(center[0],center[1],center[2]);
			}		
		}	
		//}
	}	

	// ====================================================================================================================
	// 30/07/2015
	// FIXME The original is below this method. Here it is defined double timestep for nodes and carriers velocity profile!
	// ====================================================================================================================
	// NEW AND SIMPLER IMPLEMENTATION  - Without the MM and MS component!!!  TO DEBUG
	public void laminarFlowMovement(List<NanoObj> nanoObjList) {
		
		CylindricBloodVesselDomain c = (CylindricBloodVesselDomain) nanoObjList.get(0).getDomain();
		NanoObj p = null;		
		
		
		/** HAGEN-POISEUILLE EQUATIONS */
		float timeUnit = getManager().configWrapper.timeUnit;
		float interval = (float) (timeUnit/Math.pow(10,-getManager().configWrapper.u));  		// Time interval for the computation [s]
		
		float timeCarrier = getManager().get_dt();
		float intervalCarriers=(float) (timeCarrier/Math.pow(10,-getManager().configWrapper.u));  	// Time interval for the computation [s]
		
		
		float ddt_check=getManager().configWrapper.ddt;			
		//int ddt_step=(int) (getManager().configWrapper.timeUnit/getManager().configWrapper._dt);
		
		// Get previous step velocity vector 
		SimpleVector flusso=c.getDirectionP().normalize();
		
		
		
		for (int i=0; i<nanoObjList.size(); i++){
			p = nanoObjList.get(i);

			//float[] nano_center = p.getCenter();
			//float[] center = {nano_center[0],nano_center[1],nano_center[2]};

			/** ================================================= computation of MM and M2 component =============================================*/
			/*
			// Get previous step velocity vector 
			SimpleVector flusso=c.getDirectionP().normalize();
			*/
			
			
			/** NEW VELOCITY */			
			SimpleVector newv=new SimpleVector(p.getVelocityComponent());	// This is the velocity after collisions!			

			float vmm_flux_module=newv.calcDot(flusso);	// dot product. V component on flux
			SimpleVector vmm_flux=new SimpleVector(flusso);
			vmm_flux.scalarMul(vmm_flux_module);

			/** =================================================  END computation of MM and M2 component  =============================================*/


			/** HAGEN-POISEUILLE EQUATIONS */
			/*
			float timeUnit = p.getManager().configWrapper.timeUnit;
			float interval = (float) (timeUnit/Math.pow(10,-p.getManager().configWrapper.u));  // Time interval for the computation [s]
			*/


			// FIXME - DECUZZI: The carriers takes into account even the convective contribution
			//if(p.getManager().configWrapper.ddt<1 && !(p instanceof CarrierObj)) {

			/*
			float ddt_check=p.getManager().configWrapper.ddt;			
			int ddt_step=(int) (p.getManager().configWrapper.timeUnit/p.getManager().configWrapper._dt);					
			*/

			if(!(p instanceof CarrierObj)) {

				if(ddt_check<1){

					// INITIALIZATION PHASE
					
					//float interval=timeInterval/ddt_step;
					
					CylindricBloodVesselDomain d = (CylindricBloodVesselDomain) p.getDomain();
					float m = (float) (p.getMass()/Math.pow(10,-p.getManager().configWrapper.u));     // Mass of the nano-object [g]
					float R = (float) (p.getRadius()/Math.pow(10,-p.getManager().configWrapper.u));   // Radius of the nano-object [m]				

					float velocityRegime=getPoiseuilleVelocity(d, p);
					SimpleVector flowDir=new SimpleVector(d.getDirectionP());
					flowDir=flowDir.normalize();
					flowDir.scalarMul(velocityRegime);

					/** Devo dividere tutto il vettore non solo la componente lungo Y */		
					SimpleVector oldVelTemp=new SimpleVector(vmm_flux);				
					oldVelTemp.scalarMul(1/interval);	// velocityOnFlux	 // oldVelTemp is a displacement, so it computes the velocity...        
					oldVelTemp.scalarMul(1/(float) Math.pow(10,-p.getManager().configWrapper.u));		// scales the result in the units of [m/s]						
					SimpleVector oldVelocity=oldVelTemp;

					float rx=flowDir.x-oldVelocity.x;
					float ry=flowDir.y-oldVelocity.y;
					float rz=flowDir.z-oldVelocity.z;
					SimpleVector relVel=new SimpleVector(rx,ry,rz);

					relVel.scalarMul((float) (6*Math.PI*plasmaViscosity*R));  // Stokes Law, [m/s]
					relVel.scalarMul(1/m);	// Acceleration [m/s^2]
					relVel.scalarMul(interval/2);        // Integral - The Stokes Law gives the velocity [m/s]

					oldVelocity.add(relVel);	
					SimpleVector newVelocity=new SimpleVector(oldVelocity);
					newVelocity.scalarMul((float) (interval*Math.pow(10,-p.getManager().configWrapper.u)));


					// The new velocity along the flux direction have to be added to the old velocity, in order to keep track the old direction
					SimpleVector aaV1=new SimpleVector(newVelocity);  // Defines a new component along vMM. Up to now it is given only by the component along the flow direction
					
					float dx=aaV1.x;
					float dy=aaV1.y;
					float dz=aaV1.z;
					float newVelocityModule = aaV1.length();	

					// Check the velocity angles!! 
					float[] angles = p.getVelocityAngles(dx, dy, dz);
					p.setVelocityComponentMM(newVelocityModule, angles[0], angles[1]);	  // perch MM e non la velocity ufficiale?

					// OK, ha appena calcolato il nuovo vettore velocit e lo memorizza nel NanoObj. 
					// Ora per aggiornare la posizione deve dividerlo N volte e applicare lo spostamento "infinitesimo"...
					
					
					
					/* COMMENTO PERCHE' lo metto fuori dall'IF, in modo da fare gli spostamenti infinitesimi 
					// FIXME Introduced the handling for the ddt.		

					center[0]=center[0]+aaV1.x; 
					center[1]=center[1]+aaV1.y;  
					center[2]=center[2]+aaV1.z;
					p.setCenter(center[0],center[1],center[2]);
					*/
					
				}	
				
			}else{

				/** Carriers are handled only with the Special Brownian Diffusion (diffusion+convective) */
				
				// FIXME ATTENZIONE!!!!
				// interval deve essere pari a _dt e non timeUnit!!! perch le carrier vengono spostate ohni timestep!
				
				CylindricBloodVesselDomain d = (CylindricBloodVesselDomain) p.getDomain();
				float velocityRegime=getPoiseuilleVelocity(d, p);
				velocityRegime=(float) (velocityRegime*intervalCarriers*Math.pow(10, 9));		// was ok

				float newVelocityModule = velocityRegime;
				float[] angles = p.getVelocityAngles(0, velocityRegime, 0);
				p.setVelocityComponentMM(newVelocityModule, angles[0], angles[1]);

				// FIXME Introduced the handling for the ddt. Refreshing outside the IF structure	

				/*
				center[0]=center[0]; 
				center[1]=center[1]+velocityRegime;  
				center[2]=center[2];					
				p.setCenter(center[0],center[1],center[2]);
				*/

			}					
		}			
	}	
	
	
	// 30/07/2015
	// FIXME Without the double timestep for nodes and carriers!
	// NEW AND SIMPLER IMPLEMENTATION  - Without the MM and MS component!!!  TO DEBUG
	public void laminarFlowMovement_ORIGINAL(List<NanoObj> nanoObjList) {
	
			CylindricBloodVesselDomain c = (CylindricBloodVesselDomain) nanoObjList.get(0).getDomain();
			NanoObj p = null;

			for (int i=0; i<nanoObjList.size(); i++){
				p = nanoObjList.get(i);

				//float[] nano_center = p.getCenter();
				//float[] center = {nano_center[0],nano_center[1],nano_center[2]};
				
				float[] center = p.getCenter();

				/** ================================================= computation of MM and M2 component =============================================*/
				// Get previous step velocity vector 
				SimpleVector flusso=c.getDirectionP().normalize();


				/** NEW VELOCITY */

				SimpleVector newv=new SimpleVector(p.getVelocityComponent());	// This is the velocity after collisions!					
				//SimpleVector newvversor=newv.normalize();	

				float vmm_flux_module=newv.calcDot(flusso);	// dot product. V component on flux
				SimpleVector vmm_flux=new SimpleVector(flusso);
				vmm_flux.scalarMul(vmm_flux_module);


				//SimpleVector vmm_trasverso=newv.calcSub(vmm_flux);
				//SimpleVector trasv_versor=vmm_trasverso.normalize();


				/** =================================================  END computation of MM and M2 component  =============================================*/


				/** HAGEN-POISEUILLE EQUATIONS */
				float timeUnit = p.getManager().configWrapper.timeUnit;
				float interval = (float) (timeUnit/Math.pow(10,-p.getManager().configWrapper.u));  // Time interval for the computation [s]

				// FIXME - DECUZZI: The carriers takes into account even the convective contribution
				//if(p.getManager().configWrapper.ddt<1 && !(p instanceof CarrierObj)) {


				float ddt_check=p.getManager().configWrapper.ddt;
				//float _dt=p.getManager().configWrapper._dt;

				if(ddt_check<1){
					if(!(p instanceof CarrierObj)) {

						//if(p.getManager().configWrapper.ddt<1) {
						CylindricBloodVesselDomain d = (CylindricBloodVesselDomain) p.getDomain();
						float m = (float) (p.getMass()/Math.pow(10,-p.getManager().configWrapper.u));     // Mass of the nano-object [g]
						float R = (float) (p.getRadius()/Math.pow(10,-p.getManager().configWrapper.u));   // Radius of the nano-object [m]				

						float velocityRegime=getPoiseuilleVelocity(d, p);
						SimpleVector flowDir=new SimpleVector(d.getDirectionP());
						flowDir=flowDir.normalize();
						flowDir.scalarMul(velocityRegime);

						/** Devo dividere tutto il vettore non solo la componente lungo Y */		
						SimpleVector oldVelTemp=new SimpleVector(vmm_flux);				
						oldVelTemp.scalarMul(1/interval);	// velocityOnFlux	 // oldVelTemp is a displacement, so it computes the velocity...        
						oldVelTemp.scalarMul(1/(float) Math.pow(10,-p.getManager().configWrapper.u));		// scales the result in te units of [m/s]						
						SimpleVector oldVelocity=oldVelTemp;

						float rx=flowDir.x-oldVelocity.x;
						float ry=flowDir.y-oldVelocity.y;
						float rz=flowDir.z-oldVelocity.z;
						SimpleVector relVel=new SimpleVector(rx,ry,rz);

						relVel.scalarMul((float) (6*Math.PI*plasmaViscosity*R));  // Stokes Law, [m/s]
						relVel.scalarMul(1/m);	// Acceleration [m/s^2]
						relVel.scalarMul(interval/2);        // Integral - The Stokes Law gives the velocity [m/s]

						oldVelocity.add(relVel);	
						SimpleVector newVelocity=new SimpleVector(oldVelocity);
						newVelocity.scalarMul((float) (interval*Math.pow(10,-p.getManager().configWrapper.u)));


						// The new velocity along the flux direction have to be added to the old velocity, in order to keep track the old direction
						SimpleVector aaV1=new SimpleVector(newVelocity);  // Defines a new component along vMM. Up to now it is given only by the component along the flow direction


						/** ======================================================================= 
						 *  Computes the friction along the velocity component not parallel to the flow 
						 *  ======================================================================= */				
						/*
						SimpleVector attritoTemp=new SimpleVector(vmm_trasverso);		//TODO  Uses the MM2 component (transverse to the flow) obtained before by testVettori				
						attritoTemp.scalarMul(1/interval);				
						attritoTemp.scalarMul(1/(float) Math.pow(10,-p.getManager().configWrapper.u));	

						float rxx=-attritoTemp.x;
						float ryy=-attritoTemp.y;
						float rzz=-attritoTemp.z;
						SimpleVector relVelAttrito=new SimpleVector(rxx,ryy,rzz);

						relVelAttrito.scalarMul((float) (6*Math.PI*plasmaViscosity*R));  // Stokes Law [m/s]
						relVelAttrito.scalarMul(1/m);	// [m/s^2]
						relVelAttrito.scalarMul(interval/2);        // Velocity given by the Stokes Law [m/s]

						attritoTemp.add(relVelAttrito);	
						SimpleVector newVelocityTang=new SimpleVector(attritoTemp);
						newVelocityTang.scalarMul((float) (interval*Math.pow(10,-p.getManager().configWrapper.u)));
						aaV1.add(newVelocityTang);
						 */
						/** ----------- END FRICTION -------------- */

						float dx=aaV1.x;
						float dy=aaV1.y;
						float dz=aaV1.z;



						float newVelocityModule = aaV1.length();	
						
						/*
						if(aaV1.y<0){
							System.out.println(aaV1.y);
							float tt=p.getTheta();
							float ff=p.getFi();
							
							float ttmm=p.getThetaMM();
							float ffmm=p.getFiMM();
							
							float ttms=p.getThetaMS();
							float ffms=p.getFiMS();
							
							int ok=0;
						}*/
						
						

						// Check the velocity angles!! 
						float[] angles = p.getVelocityAngles(dx, dy, dz);
						p.setVelocityComponentMM(newVelocityModule, angles[0], angles[1]);
						//p.setVelocityComponent(newVelocityModule, angles[0], angles[1]);


						/** Check for maximum velocity */
						/*
						float timeInterval=(float) (interval*Math.pow(10,-p.getManager().configWrapper.u));
						float updateCheck=newVelocityModule/timeInterval;
						float alfa=0.5f;
						float delta=updateCheck-velocityRegime;

						if(p.check!=0){

							float newCheck=alfa*p.check+(1-alfa)*delta;
							float old=p.check;
							p.check=newCheck;
							float velocityThreshold=(float) 1E-3;

							if(newCheck>=velocityThreshold){
								newVelocityModule=velocityRegime*timeInterval;
								p.setVelocityComponentMM(newVelocityModule, angles[0], angles[1]);	
								//p.setVelocityComponent(newVelocityModule, angles[0], angles[1]);
								p.check=alfa*old;
							}
						}else{
							p.check=(1-alfa)*delta;
						}*/


						// FIXME Introduced the handling for the ddt. Refreshing outside the IF structure		

						center[0]=center[0]+aaV1.x; 
						center[1]=center[1]+aaV1.y;  
						center[2]=center[2]+aaV1.z;
						p.setCenter(center[0],center[1],center[2]);
						
						
						
						
						/*
						refresh_x=aaV1.x/_dt;
						refresh_y=aaV1.y/_dt;
						refresh_z=aaV1.z/_dt;
						*/

					}else{

						/** Carriers are handled only with the Special Brownian Diffusion (diffusion+convective) */
						/**/
						CylindricBloodVesselDomain d = (CylindricBloodVesselDomain) p.getDomain();
						float velocityRegime=getPoiseuilleVelocity(d, p);
						velocityRegime=(float) (velocityRegime*interval*Math.pow(10, 9));		// was ok

						float newVelocityModule = velocityRegime;
						float[] angles = p.getVelocityAngles(0, velocityRegime, 0);
						p.setVelocityComponentMM(newVelocityModule, angles[0], angles[1]);

						// FIXME Introduced the handling for the ddt. Refreshing outside the IF structure	

						center[0]=center[0]; 
						center[1]=center[1]+velocityRegime;  
						center[2]=center[2];					
						p.setCenter(center[0],center[1],center[2]);

					}
				}			
			}	
			//}
		}	


	public void laminarFlowMovement_DEBUG_fino_al_31_01_2014(List<NanoObj> nanoObjList) {
		CylindricBloodVesselDomain c = (CylindricBloodVesselDomain) nanoObjList.get(0).getDomain();
		NanoObj p = null;

		for (int i=0; i<nanoObjList.size(); i++){
			p = nanoObjList.get(i);


			float[] center = p.getCenter();

			/** ================================================= computation of MM and M2 component =============================================*/
			// Get previous step velocity vector 
			SimpleVector flusso=c.getDirectionP().normalize();
			SimpleVector mm=new SimpleVector(p.getVelocityComponentMM());		
			SimpleVector ms=new SimpleVector(p.getVelocityComponentMS());

			SimpleVector oldv=new SimpleVector();		
			oldv.add(mm);
			oldv.add(ms);		


			SimpleVector oldvVERSOR=oldv.normalize();				
			float mmproj=(mm.x*flusso.x+mm.y*flusso.y+mm.z*flusso.z);			

			/** Projection of MM component on old velocity vector */
			float mm_su_oldv=(oldvVERSOR.x*mm.x+oldvVERSOR.y*mm.y+oldvVERSOR.z*mm.z);					
			//SimpleVector mm_lungo_oldv=new SimpleVector(oldvVERSOR);
			//mm_lungo_oldv.scalarMul(mm_su_oldv);
			/** ============================= */

			/** Projection of [mm_su_oldv] along the direction of the flow */
			//float mm_lungo_oldv_proj=(mm_lungo_oldv.x*flusso.x+mm_lungo_oldv.y*flusso.y+mm_lungo_oldv.z*flusso.z);
			/** ============================= */

			float k_mm_su_oldv=0;
			if(oldv.length()!=0){	
				k_mm_su_oldv=mm_su_oldv/oldv.length();	// Ratio between the projection of MM velocity component on old velocity vector and old velocity vector

				if(k_mm_su_oldv<0){
					k_mm_su_oldv=Math.abs(k_mm_su_oldv);	//FIXME Introduced to FIX anomalous behaviour
				}

			}

			float k_mm_su_mmold=1;
			if(mm.length()!=0){
				// Angle between MM component and old velocity vector (it is the cosine!)
				k_mm_su_mmold=mm_su_oldv/mm.length();		// Ratio between the projection of MM velocity component on old velocity vector and the original MM component	
			}
			if (k_mm_su_mmold>1) {
				k_mm_su_mmold=1;
			}else if (k_mm_su_mmold<-1) {
				k_mm_su_mmold=-1;
			}


			/** NEW VELOCITY */
			SimpleVector newv=new SimpleVector(p.getVelocityComponent());	// This is the velocity after collisions!			
			SimpleVector newvversor=newv.normalize();		

			float mm_su_newV=k_mm_su_oldv*newv.length();

			float anglebeta=mm.calcAngle(oldv);
			float beta=(float) Math.acos(k_mm_su_mmold);		// ANGLE BETWEEN MM AND OLDV
			float gamma_old=(oldv.calcAngle(flusso));			// ANGLE BETWEEN OLDV AND FLOW DIRECTION
			//float sigma_old_bak=gamma_old-beta;				// ANGLE BETWEEN MM AND FLOW DIRECTION??
			float sigma_old=mm.calcAngle(flusso);				// ANGLE BETWEEN MM AND FLOW DIRECTION??


			float sigma=0;
			float gamma=newv.calcAngle(flusso);

			// FIXME TEST DEBUG
			float diffAngle=gamma-gamma_old;
			sigma=beta+diffAngle;
			//int ops=0;


			// round all angles				
			anglebeta=(float) (Math.round(anglebeta*1E7)/1E7);
			beta=(float) (Math.round(beta*1E7)/1E7);					// ANGLE BETWEEN MM AND OLDV
			gamma_old=(float) (Math.round(gamma_old*1E7)/1E7);			// ANGLE BETWEEN OLDV AND FLOW DIRECTION
			//float sigma_old_bak=gamma_old-beta;						// ANGLE BETWEEN MM AND FLOW DIRECTION??
			sigma_old=(float) (Math.round(sigma_old*1E7)/1E7);			// ANGLE BETWEEN MM AND FLOW DIRECTION??
			gamma=(float) (Math.round(gamma*1E7)/1E7);
			sigma=(float) (Math.round(sigma*1E7)/1E7);
			// =========

			/*
				int quale=0;
				if (gamma<=Math.PI/2) {
					sigma=gamma-beta;	
					quale=1;
				}else if (gamma>Math.PI/2 && gamma<(1.5*Math.PI)){
					sigma=gamma+beta;
					quale=2;
				}else{
					sigma=(float) (+gamma+beta);
					quale=3;
				}

				if (sigma<0) sigma=beta-gamma;
			 */

			float mm2=mm_su_newV/k_mm_su_mmold;			
			//mm2=Math.abs(mm2);



			float mm2_su_flusso=(float) (mm2*Math.cos(sigma));

			SimpleVector mm_flusso=new SimpleVector(flusso);
			mm_flusso.scalarMul(mmproj);
			/*
				SimpleVector mm_trasverso=new SimpleVector(oldvVERSOR);
				mm_trasverso.scalarMul(mm_su_oldv);
				mm_trasverso=mm_trasverso.calcSub(mm_flusso);
			 */

			/** The cross product is used to get the orthogonal vector to flow! */
			SimpleVector mm_su_oldvVector=new SimpleVector(oldvVERSOR);
			mm_su_oldvVector.scalarMul(mm_su_oldv);
			SimpleVector asseRotazione0=mm_su_oldvVector.calcCross(flusso);

			//float ro0=(float) (Math.PI/2-sigma_old);
			SimpleVector mm_trasverso2=new SimpleVector(flusso.normalize());

			float mm_traverso_module=(float) (mm.length()*Math.sin(sigma_old));
			if (sigma_old>Math.PI) mm_traverso_module=mm_traverso_module*-1;

			mm_trasverso2.scalarMul(mm_traverso_module);

			float ro0=(float) Math.PI/2;
			Matrix rotation0 = new Matrix();
			rotation0.rotateAxis(asseRotazione0, ro0);
			mm_trasverso2.rotate(rotation0);

			/** END of computation of the orthogonal vector */

			SimpleVector mm2_flusso=new SimpleVector(flusso);
			mm2_flusso.scalarMul(mm2_su_flusso);  	//TODO  This is the flow projection along MM2 direction. Needed in Poiseuille! 


			/** In fact, does not take into account the transvert component to the flow when the new velocity is parallel to the flow itself! */
			SimpleVector mm2_trasverso=new SimpleVector(newvversor);
			mm2_trasverso.scalarMul(mm_su_newV);
			mm2_trasverso=mm2_trasverso.calcSub(mm2_flusso);	//TODO  This is the orthogonal component to the flow projection along MM2 direction. Needed in Poiseuille!

			//SimpleVector cross=newvversor.calcCross(mm2_flusso);


			/** Uses the cross product to get the orthogonal vector to the flow direction */
			SimpleVector mm2_su_newV=new SimpleVector(newvversor);
			mm2_su_newV.scalarMul(mm_su_newV);
			SimpleVector asseRotazione=mm2_su_newV.calcCross(flusso);

			float ro=(float) (Math.PI/2-sigma);
			ro=(float) Math.PI/2;

			float[] nullcoords={0,0,0};		
			SimpleVector mm2_trasverso2=new SimpleVector(nullcoords);

			if (asseRotazione.length()==0){
				SimpleVector tempVector=new SimpleVector(mm2_su_newV);
				tempVector.x=(float) (tempVector.x+(0.0000001));
				tempVector.y=(float) (tempVector.y+(0.0000001));
				tempVector.z=(float) (tempVector.z+(0.0000001));
				asseRotazione=tempVector.calcCross(flusso);
			}

			mm2_trasverso2=new SimpleVector(flusso.normalize());

			float mm2_traverso2_module=(float) (mm2*Math.sin(sigma));
			if (sigma>Math.PI) mm2_traverso2_module=mm2_traverso2_module*-1;

			mm2_trasverso2.scalarMul(mm2_traverso2_module);
			Matrix rotation = new Matrix();
			rotation.rotateAxis(asseRotazione, ro);
			mm2_trasverso2.rotate(rotation);

			mm2_trasverso=mm2_trasverso2;

			/** END of transverse projection */				
			/** =================================================  END computation of MM and M2 component  =============================================*/


			/** HAGEN-POISEUILLE EQUATIONS */
			float timeUnit = p.getManager().configWrapper.timeUnit;
			float interval = (float) (timeUnit/Math.pow(10,-p.getManager().configWrapper.u));  // Time interval for the computation [s]

			// FIXME - DECUZZI: The carriers takes into account even the convective contribution
			if(p.getManager().configWrapper.ddt<1 && !(p instanceof CarrierObj)) {
				//if(p.getManager().configWrapper.ddt<1) {
				CylindricBloodVesselDomain d = (CylindricBloodVesselDomain) p.getDomain();
				float m = (float) (p.getMass()/Math.pow(10,-p.getManager().configWrapper.u));     // Mass of the nano-object [g]
				float R = (float) (p.getRadius()/Math.pow(10,-p.getManager().configWrapper.u));   // Radius of the nano-object [m]				

				float velocityRegime=getPoiseuilleVelocity(d, p);
				SimpleVector flowDir=new SimpleVector(d.getDirectionP());
				flowDir=flowDir.normalize();
				flowDir.scalarMul(velocityRegime);

				/** Devo dividere tutto il vettore non solo la componente lungo Y */		
				SimpleVector oldVelTemp=new SimpleVector(mm2_flusso);				
				oldVelTemp.scalarMul(1/interval);	// velocityOnFlux	 // oldVelTemp is a displacement, so it computes the velocity...        
				oldVelTemp.scalarMul(1/(float) Math.pow(10,-p.getManager().configWrapper.u));		// scales the result in te units of [m/s]						
				SimpleVector oldVelocity=oldVelTemp;

				float rx=flowDir.x-oldVelocity.x;
				float ry=flowDir.y-oldVelocity.y;
				float rz=flowDir.z-oldVelocity.z;
				SimpleVector relVel=new SimpleVector(rx,ry,rz);

				relVel.scalarMul((float) (6*Math.PI*plasmaViscosity*R));  // Stokes Law, [m/s]
				relVel.scalarMul(1/m);	// Acceleration [m/s^2]
				relVel.scalarMul(interval/2);        // Integral - The Stokes Law gives the velocity [m/s]

				oldVelocity.add(relVel);	
				SimpleVector newVelocity=new SimpleVector(oldVelocity);
				newVelocity.scalarMul((float) (interval*Math.pow(10,-p.getManager().configWrapper.u)));

				// The new velocity along the flux direction has to be added to the OLD velocity in order to keep track of the old direction
				SimpleVector aaV1=new SimpleVector(newVelocity);  // Defines a new component along vMM. Up to now it is given only by the component along the flow direction


				/** ======================================================================= 
				 *  Computes the friction along the velocity component not parallel to the flow 
				 *  ======================================================================= */				

				SimpleVector attritoTemp=new SimpleVector(mm2_trasverso);		//TODO  Uses the MM2 component (transverse to the flow) obtained before by testVettori				
				attritoTemp.scalarMul(1/interval);				
				attritoTemp.scalarMul(1/(float) Math.pow(10,-p.getManager().configWrapper.u));	

				float rxx=-attritoTemp.x;
				float ryy=-attritoTemp.y;
				float rzz=-attritoTemp.z;
				SimpleVector relVelAttrito=new SimpleVector(rxx,ryy,rzz);

				relVelAttrito.scalarMul((float) (6*Math.PI*plasmaViscosity*R));  // Stokes Law [m/s]
				relVelAttrito.scalarMul(1/m);	// [m/s^2]
				relVelAttrito.scalarMul(interval/2);        // Velocity given by the Stokes Law [m/s]

				attritoTemp.add(relVelAttrito);	
				SimpleVector newVelocityTang=new SimpleVector(attritoTemp);
				newVelocityTang.scalarMul((float) (interval*Math.pow(10,-p.getManager().configWrapper.u)));
				aaV1.add(newVelocityTang);
				/** ----------- END FRICTION -------------- */

				float dx=aaV1.x;
				float dy=aaV1.y;
				float dz=aaV1.z;				

				float newVelocityModule = aaV1.length();					
				float[] angles = p.getVelocityAngles(dx, dy, dz);
				p.setVelocityComponentMM(newVelocityModule, angles[0], angles[1]);


				/** Check for maximum velocity */
				float timeInterval=(float) (interval*Math.pow(10,-p.getManager().configWrapper.u));
				float updateCheck=newVelocityModule/timeInterval;
				float alfa=0.5f;
				float delta=updateCheck-velocityRegime;

				if(p.check!=0){

					float newCheck=alfa*p.check+(1-alfa)*delta;
					float old=p.check;
					p.check=newCheck;
					float velocityThreshold=(float) 1E-3;

					if(newCheck>=velocityThreshold){
						newVelocityModule=velocityRegime*timeInterval;
						p.setVelocityComponentMM(newVelocityModule, angles[0], angles[1]);							
						p.check=alfa*old;
					}
				}else{
					p.check=(1-alfa)*delta;
				}

				center[0]=center[0]+aaV1.x; 
				center[1]=center[1]+aaV1.y;  
				center[2]=center[2]+aaV1.z;
				p.setCenter(center[0],center[1],center[2]);

			}else{

				/** Carriers are handled only with the Special Brownian Diffusion (diffusion+convective) */
				/**/
				CylindricBloodVesselDomain d = (CylindricBloodVesselDomain) p.getDomain();
				float velocityRegime=getPoiseuilleVelocity(d, p);
				velocityRegime=(float) (velocityRegime*interval*Math.pow(10, 9));		// era ok

				float newVelocityModule = velocityRegime;
				float[] angles = p.getVelocityAngles(0, velocityRegime, 0);
				p.setVelocityComponentMM(newVelocityModule, angles[0], angles[1]);

				center[0]=center[0]; 
				center[1]=center[1]+velocityRegime;  
				center[2]=center[2];					
				p.setCenter(center[0],center[1],center[2]);
			}		
		}	
		//}
	}	


	@Override
	public boolean checkCoords(float x, float y, float z) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public AbstractThreadingStrategy getThreadingStrategy() {
		// FIXME To be defined...
		return null;
	}
}