package simulator;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.concurrent.ThreadLocalRandom;

import MainLibrary.MobilityModel;
import MainLibrary.MotionStrategy;
import MainLibrary.NanoObj;


public class BrownianMotion extends MotionStrategy{

	private static BrownianMotion instance=null;	
	private FileOutputStream file;
	private PrintStream Output;
	
	private BrownianMotion(){
		
	}
	
	public static BrownianMotion getInstance(){
		if (instance==null) instance=new BrownianMotion();
		return instance;
	}	
	
	@Override
	public void move(NanoObj p) {
		float[] center=p.getCenter();
		float newVelocityModule=p.getVelocityModule();
		MobilityModel model=p.getManager().getMobilityModel();
			
		float sigma2=model.getDiffusionCoefficient(p);		
		float[] component=p.getVelocityComponent();		
		
		/*
		//float silly=0;
		float dx=component[0];
		float dy=component[1];
		float dz=component[2];
		*/
		
		float dx=component[0];
		float dy=component[1];
		float dz=component[2];
				
		if(p.getManager().configWrapper.ddt<1){				
			/*		
			dx=model.getGauss(sigma2);
			dy=model.getGauss(sigma2);
			dz=model.getGauss(sigma2);
			*/
			
						
			dx=(float) (ThreadLocalRandom.current().nextGaussian()*sigma2);
			dy=(float) (ThreadLocalRandom.current().nextGaussian()*sigma2);
			dz=(float) (ThreadLocalRandom.current().nextGaussian()*sigma2);	
			
			newVelocityModule=(float) Math.sqrt((dx*dx)+(dy*dy)+(dz*dz));
			float[] angles=p.getVelocityAngles(dx,dy,dz);
			p.setVelocityComponent(newVelocityModule,angles[0], angles[1]);		
			
			
		}
		
		center[0]=center[0]+dx;
		center[1]=center[1]+dy;
		center[2]=center[2]+dz;
		p.setCenter(center[0],center[1],center[2]);	
	}
	
	public void exportBrownian(float dx, float dy, float dz){
		
		try{		
			
			file = new FileOutputStream("Gaussian.txt",true);
			Output = new PrintStream(file);
			Output.println(dx+" "+dy+" "+dz);			
			 
		} catch (IOException e) {
			System.out.println("Output Error: " + e);
			
		} finally {
		    // Close resource
		    if (file!= null) try { 
		        file.close();
		    } catch (IOException logOrIgnore) {
		    	System.out.println("Output Error [closing file]: " + logOrIgnore);
		    }
		}
	}
}