package simulator;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import MainLibrary.MotionStrategy;
import MainLibrary.NanoObj;
import MainLibrary.CarrierObj;
import MainLibrary.NodeObj;


public class BrownianMotionBloodStream extends MotionStrategy{

	private static BrownianMotionBloodStream instance=null;	
	private FileOutputStream file;
	private PrintStream Output;
	
	private BrownianMotionBloodStream(){
		
	}
	
	public static BrownianMotionBloodStream getInstance(){
		if (instance==null) instance=new BrownianMotionBloodStream();
		return instance;
	}	
	
	
	
	// INTRODUCTION OF 2 TIMESTEPS STRATEGY  -  31/07/2015
	
	public void move(NanoObj p) {
		try {
			//float[] center=p.getCenter();			
			
			if((p instanceof CarrierObj) || (p.getManager().configWrapper.ddt<1)){	
				
				float newVelocityModule=p.getVelocityModule();
				BloodFlowModel model=(BloodFlowModel) p.getManager().getMobilityModel();		
				float sigma2=model.getDiffusionCoefficient(p);	
				
				float dx=(float) ((model.getGauss(sigma2)));				
				float dy=(float) ((model.getGauss(sigma2)));
				float dz=(float) ((model.getGauss(sigma2)));	

				newVelocityModule=(float) Math.sqrt((dx*dx)+(dy*dy)+(dz*dz));
				float[] angles=p.getVelocityAngles(dx,dy,dz);			
				p.setVelocityComponentMS(newVelocityModule,angles[0], angles[1]);	

			}

			// NOW THE FOLLOWING setCenter() is commented, because the updates are executed by FlowModelMoveThread2Timesteps()
			/*
			center[0]=center[0]+dx;
			center[1]=center[1]+dy;
			center[2]=center[2]+dz;
			p.setCenter(center[0],center[1],center[2]);
			*/

		} catch (Exception e) {
			// TODO Auto-generated catch block
			System.out.println("EXCEPTION on BrownianMotionBloodVessel.move()! ");
			e.printStackTrace();
		}			
	}
	
	
	
	// ORIGINAL BEFORE 2 TIMESTEPS FEATURE!  -  31/07/2015
	public void move_ORIGINAL(NanoObj p) {
		try {
			float[] center=p.getCenter();
			float newVelocityModule=p.getVelocityModule();
			BloodFlowModel model=(BloodFlowModel) p.getManager().getMobilityModel();		
			float sigma2=model.getDiffusionCoefficient(p);	
			float[] component=p.getVelocityComponentMS();
			
			
			float dx=component[0];
			float dy=component[1];
			float dz=component[2];			
			
			
			if(p.getManager().configWrapper.ddt<1){	
				CylindricBloodVesselDomain d=(CylindricBloodVesselDomain) p.getDomain();
				
				// FIXME All comments are relative to Decuzzi approach
				//if (p instanceof CarrierObj){				
					/*float brown=(float) (sigma2*sigma2*Math.pow(10, (2*p.getManager().configWrapper.u)));				
					brown=(float) (brown/(2*p.getManager().get_dt()*Math.pow(10, (p.getManager().configWrapper.u))));				
					float convective=model.getPouseuilleCoefficient(d,p);				
					float sigmaEff=brown+convective;				
					sigmaEff=(float) (6*sigmaEff/3*p.getManager().get_dt()*Math.pow(10, (p.getManager().configWrapper.u)));
					sigmaEff=(float) ((float) Math.sqrt(sigmaEff)*Math.pow(10, (-p.getManager().configWrapper.u)));		
						
					dx=(float) ((model.getGauss(sigma2))*1);				
					dy=(float) ((model.getGauss(sigmaEff))*1);
					dz=(float) ((model.getGauss(sigma2))*1);	
					*/	
					dx=(float) ((model.getGauss(sigma2))*1);				
					dy=(float) ((model.getGauss(sigma2))*1);
					dz=(float) ((model.getGauss(sigma2))*1);					
					
				/*
				}else{
					
					dx=(float) ((model.getGauss(sigma2))*1);
					dy=0;
					dz=(float) ((model.getGauss(sigma2))*1);
				}
				*/
					
				newVelocityModule=(float) Math.sqrt((dx*dx)+(dy*dy)+(dz*dz));
				float[] angles=p.getVelocityAngles(dx,dy,dz);			
				p.setVelocityComponentMS(newVelocityModule,angles[0], angles[1]);	
				
			}
			
			center[0]=center[0]+dx;
			center[1]=center[1]+dy;
			center[2]=center[2]+dz;
			p.setCenter(center[0],center[1],center[2]);
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			System.out.println("EXCEPTION on BrownianMotionBloodVessel.move()! ");
			e.printStackTrace();
		}			
	}
	
	
	public void moveDECUZZI(NanoObj p) {
		float[] center=p.getCenter();
		float newVelocityModule=p.getVelocityModule();
		BloodFlowModel model=(BloodFlowModel) p.getManager().getMobilityModel();		
		float sigma2=model.getDiffusionCoefficient(p);	
		float[] component=p.getVelocityComponentMS();
		
		float dx=component[0];
		float dy=component[1];
		float dz=component[2];
		
		if(p.getManager().configWrapper.ddt<1){	
			CylindricBloodVesselDomain d=(CylindricBloodVesselDomain) p.getDomain();
			
			if (p instanceof CarrierObj){
				
				float brown=(float) (sigma2*sigma2*Math.pow(10, (2*p.getManager().configWrapper.u)));				
				brown=(float) (brown/(2*p.getManager().get_dt()*Math.pow(10, (p.getManager().configWrapper.u))));				
				float convective=model.getPouseuilleCoefficient(d,p);				
				float sigmaEff=brown+convective;				
				sigmaEff=(float) (6*sigmaEff/3*p.getManager().get_dt()*Math.pow(10, (p.getManager().configWrapper.u)));
				sigmaEff=(float) ((float) Math.sqrt(sigmaEff)*Math.pow(10, (-p.getManager().configWrapper.u)));		
					
				dx=(float) ((model.getGauss(sigma2))*1);				
				dy=(float) ((model.getGauss(sigmaEff))*1);
				dz=(float) ((model.getGauss(sigma2))*1);
				
			}else{				
				dx=(float) ((model.getGauss(sigma2))*1);
				dy=0;
				dz=(float) ((model.getGauss(sigma2))*1);
			}

			newVelocityModule=(float) Math.sqrt((dx*dx)+(dy*dy)+(dz*dz));
			float[] angles=p.getVelocityAngles(dx,dy,dz);			
			p.setVelocityComponentMS(newVelocityModule,angles[0], angles[1]);	
		}
		
		center[0]=center[0]+dx;
		center[1]=center[1]+dy;
		center[2]=center[2]+dz;
		p.setCenter(center[0],center[1],center[2]);			
	}
	
	public void exportBrownian(float dx, float dy, float dz){
		
		try{	
			
			file = new FileOutputStream("Gaussian.txt",true);
			Output = new PrintStream(file);
			Output.println(dx+" "+dy+" "+dz);			
			 
		} catch (IOException e) {
			System.out.println("Output Error: " + e);
			
		} finally {
		    // Close resource
		    if (file!= null) try { 
		        file.close();
		    } catch (IOException logOrIgnore) {
		    	System.out.println("Output Error [closing file]: " + logOrIgnore);
		    }
		}
	}
}