package simulator;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.LigandObserver;
import MainLibrary.NoMotion;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.UniversalReceptor;
import MainLibrary.nIdle;


public class CalciumCell_RX extends NodeObj{
	
	private long firstContact=-1;
	
	public long assimilatedIons=0;
	public long assimilatedBG=0;
	
	
	
	public CalciumCell_RX(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);
				
		// You can set the internal state here!
		setState(new nIdle(this));
		getState().setExecutionTime(1000);	
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
		setColor(Color.white);	
		
		informationCount=0;
		assimilatedIons=0;
		assimilatedBG=0;
	}
	
	@Override
	public void idle() {
		processInfo();
	}
	
	@Override
	public void decode() {
		setState(new nIdle(this));
	}
	
	@Override
	public void encode() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(1000);	
	}

	@Override
	public void searchNeighbor() {
		
	}

	@Override
	public void transmit() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(1000);		
	}
	
	public void createCarriers(int n){
		while (n>0){				
			CD40LCarrier cc=new CD40LCarrier(getInnerDomain(), this, getCenter()[0], getCenter()[1], getCenter()[2],1);			
			cc.setMotionStrategy(BrownianMotion.getInstance());
			n--;
		}			
	}
		
	@Override
	public Map<Object, Object> specializedExport() {

		Map<Object, Object> m=new HashMap<Object,Object>();
		if(assimilatedIons>0 || assimilatedBG>0){
			m.put("Typev", "CalciumCellRX");

			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		

			m.put("IDv", "CalciumCellRX-"+ID+"-Ca++");	
			m.put("Xv", center[0]-domainCenter[0]);
			m.put("Yv", center[1]-domainCenter[1]);
			m.put("Zv", center[2]-domainCenter[2]);
			m.put("TimeStepv", ""+getManager().getCurrentTime()+"");	
			m.put("Assimilationsv", assimilatedIons);	
			m.put("Actionv", "assimilation");
			
			m.put("IDv2", "CalciumCellRX-"+ID+"-BG");	
			m.put("Xv2", center[0]-domainCenter[0]);
			m.put("Yv2", center[1]-domainCenter[1]);
			m.put("Zv2", center[2]-domainCenter[2]);
			m.put("TimeStepv2", ""+getManager().getCurrentTime()+"");	
			m.put("Assimilationsv2", assimilatedBG);	
			m.put("Actionv2", "assimilationBG");
			
			
			m.putAll(getState().export());
		}
		return m;
	}	
	
	@Override
	public void createSingleReceptor(float[] cdCenter, float radius, short receptorRadius, int receptorType) {
		float[] angles=convertSphericalAngles(cdCenter[0],cdCenter[1],cdCenter[2], radius);
		float theta=angles[1];
		float fi=angles[0];

		
		Receptor r=new CD40Receptor(this, true, cdCenter, theta, fi, receptorRadius, receptorType, this.eventHandler);
		receptorList.add(r);
		receptorSetup(r); 

		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		*/		
	}
	
	public void createReceptor(int n){
		Receptor r = null;
		
		float theta=0;
		float fi=0;				
		
		for (int i=0;i<n;i++){		
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];		
			
			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new CD40Receptor(this, true, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			receptorSetup(r); 		
		}
	}	
	
	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	}
	
	@Override
	public void setMotionStrategy() {		
		//this.motionStrategy=BrownianMotion.getInstance();
		this.motionStrategy=NoMotion.getInstance();
	}
	
	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}
		
	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	@Override
	public void processInfo() {
		if(getCollisionCount()>0 && firstContact<0){
			firstContact=getManager().getCurrentTime();				
		}		
	}
	
	public String assignName(){		
		return "Ca++ Cell RX";		
	}

	// Overrides the original one of the NodeObj, because in this case each collision is an assimilation!
	public void checkIncomingCarrier(CarrierObj c){	
		//c.setColor(Color.red);
		//System.out.println("CARRIER CENTER: "+c.getCenter()[0]+", "+c.getCenter()[1]+", "+c.getCenter()[2]);
		//getDomain().detachNanoObj(c);	
		if(c.getTimeToLive()>0)	downloadCarrier(c);
	}
	

	public void downloadCarrier(CarrierObj c) {			
		
		//if(carrierDebug.contains(c.toString())){
		/*
		if(((CalciumDiffusionModel)getDomain().getMobilityModel()).carrierDebug.contains(c)){
			System.out.println(">>> Ca++ ALREADY DOWNLOADED!!! "+c.toString());
			
			int kk=0;
			while(((CalciumDiffusionModel)getDomain().getMobilityModel()).carrierDebug2.contains(c)){
				boolean tt=((CalciumDiffusionModel)getDomain().getMobilityModel()).carrierDebug2.remove(c);
				kk++;
			}
			System.out.println(kk);
			
		}else{
			((CalciumDiffusionModel)getDomain().getMobilityModel()).carrierDebug.add(c);
			assimilatedIons++;
		}
		*/
		
		if(c instanceof CalciumCarrier){
			assimilatedIons++;
		}else if(c instanceof CalciumCarrierBG){
			assimilatedBG++;
		}
		
		c.kill(); 	
		
	}
}