
package simulator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import MainLibrary.AbstractDomain;
import MainLibrary.LigandObserver;
import MainLibrary.NoMotion;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.nEncode;
import MainLibrary.nIdle;
import MainLibrary.nTransmit;


public class CalciumCell_TX extends NodeObj{

	public int carrierCreated=0;	
	private long[] releaseTimeLog=new long[10];
	private int[] releaseCarrierLog=new int[10];
	private int indexReleaseLog=0;
	private int indexExportedLog=0;
	private ArrayList<float[]> releasePositions=new ArrayList<float[]>();
	
	public int internalReservoirOfCarrier=0;
	
	
	//private float releaseRateSeconds=(float) 2.09E8;	// extracellular calcium scenario
	//private float releaseRateSeconds=(float) 4.26E8;	// intracellular calcium scenario - cell 5um
	private float releaseRateSeconds=(float) 1.74E9;	// intracellular calcium scenario - cell 8um
	
	private int releaseRateTimeStep=1;

	/*
	private int tempReleaseWait=0;
	private int counterForReleaseWait=0;
    */

	public CalciumCell_TX(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);

		// You can set the internal state here!
		/*
		setState(new nIdle(this));
		getState().setExecutionTime(1);
		*/	
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
		
		
		//private float releaseRateSeconds=(float) 2.09E8;
		releaseRateTimeStep=(int) (releaseRateSeconds*domain.getManager().get_dt()*1E-9);
		
		//internalReservoirOfCarrier=(int) 3.15E7;  // cell 5um
		
		internalReservoirOfCarrier=(int) 1.29E8;	// cell 8um
		
		setState(new nTransmit(this));
		getState().setExecutionTime(0);		
	}
	
	public void enlargeReleaseTimeLog(){
		long[] temp=releaseTimeLog;
		int[] tempCarrier=releaseCarrierLog;
		int size=temp.length;
		size=size+size/2;
		
		releaseTimeLog=new long[size];
		releaseCarrierLog=new int[size];
		
		for(int i=0;i<temp.length;i++){
			releaseTimeLog[i]=temp[i];
			releaseCarrierLog[i]=tempCarrier[i];
		}		
	}
	
	@Override
	public boolean mandatoryCreation(){
		return true;
	}

	public void standardBehavior() {		
		super.standardBehavior();
		/*
		if (indexPattern<emissionPattern.length()){
			countdownForEmission--;
			if(countdownForEmission<=0){


				if (firstRelease<0) firstRelease=getManager().getCurrentTime();
				if(emissionPattern.charAt(indexPattern)=='1'){
					System.out.println(">>> Releasing CARRIERS!!! (Platelet)");
					createCarriers(burstEmission);
				}
								
				indexPattern++;
				countdownForEmission=timeForEmission;
			}
		}
		*/

	}
	
	@Override
	public void idle() {	
		
		/* OLD BEFORE PATTERN - SEE standardBehaviour()!!!
		if (carrierCreated==0 && burstEmission>0){
			activateCell();
		} */
		getState().setExecutionTime(10);	
		
	}

	@Override
	public void decode() {
		setState(new nIdle(this));

	}

	@Override
	public void encode() {
		/* OLD BEFORE PATTERN - SEE standardBehaviour()!!!
		createCarriers(burstEmission);
		setState(new nTransmit(this));
		*/
		setState(new nIdle(this));
	}

	@Override
	public void searchNeighbor() {

	}

	@Override
	public void transmit() {			

		if(carrierCreated<internalReservoirOfCarrier){
			createCarriers(releaseRateTimeStep);		
		}

	}

	
	public void transmit_OLD() {			
		
		//if (releaseRateTimeStep>0){
			//long timeTemp=getManager().getCurrentTime();
			if(getManager().getCurrentTime()%releaseRateTimeStep==0 /*&& timeTemp<3000*/){		
				createCarriers(burstEmission);		
			}
	    /* 
	    }else{
			
			tempReleaseWait=(int) (1/(releaseRateSeconds*getDomain().getManager().get_dt()*1E-9));			
			
			if(counterForReleaseWait==0){
				createCarriers(burstEmission);
				
			}else{
				counterForReleaseWait--;
			}
		}*/
	}

	
	public int createCarriers(int n){
		int createdNow=0;
		while (n>0){				
			CalciumCarrier cc=new CalciumCarrier(getInnerDomain(), this, getCenter()[0], getCenter()[1], getCenter()[2],2);			
			//cc.setMotionStrategy(BrownianMotion.getInstance());
			n--;
			createdNow++;			
		}
		carrierCreated=carrierCreated+createdNow;
		return createdNow;
	}

	@Override
	public Map<Object, Object> specializedExport() {
		
		Map<Object, Object> m=new HashMap<Object,Object>();
		if(carrierCreated>0 ){
			m.put("Typev", "CalciumCellTX");

			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		

			m.put("IDv", "CalciumCellTX-"+ID);	
			m.put("Xv", center[0]-domainCenter[0]);
			m.put("Yv", center[1]-domainCenter[1]);
			m.put("Zv", center[2]-domainCenter[2]);
			m.put("TimeStepv", ""+getManager().getCurrentTime()+"");	
			m.put("Emissionv", carrierCreated);	
			m.put("Actionv", "emitted");	
			
			m.putAll(getState().export());
		}
		return m;
	}

	
	@Override
	public void createSingleReceptor(float[] cdCenter, float radius, short receptorRadius, int receptorType) {
		float[] angles=convertSphericalAngles(cdCenter[0],cdCenter[1],cdCenter[2], radius);
		float theta=angles[1];
		float fi=angles[0];

		
		Receptor r=new CD154Receptor(this, true, cdCenter, theta, fi, receptorRadius, receptorType, this.eventHandler);
		receptorList.add(r);
		receptorSetup(r); 

		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		*/		
	}
	
	public void createReceptor(int n){		

		CalciumReceptor r = null;		
		float theta=0;
		float fi=0;			

		for (int i=0;i<n;i++){
						
			fi=(float) (2*Math.PI*(rand.nextFloat())); //fi
			theta=(float) ((Math.acos((2*rand.nextFloat())-1))/52.33); //theta

			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			

			r=new CalciumReceptor(this, false, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			receptorSetup(r);	
			
			//r.drawSmallPoint(getDomain());
		}
	}	
	
	public void createReceptor_Calotta(int n){		

		CalciumReceptor r = null;		
		float theta=0;
		float fi=0;			

		for (int i=0;i<n;i++){
						
			fi=(float) (2*Math.PI*(rand.nextFloat())); //fi
			theta=(float) ((Math.acos((2*rand.nextFloat())-1))/2.55); //theta

			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			

			r=new CalciumReceptor(this, false, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			receptorSetup(r);	
			
			r.drawSmallPoint(getDomain());
		}
	}	
	
	

	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	}

	@Override
	public void setMotionStrategy() {		
		this.motionStrategy=NoMotion.getInstance();
	}

	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}

	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	public void activateCell() {
		/*
		float check=(float) (((((CylindricDomain) getDomain()).getHeight()-101000-500000)/2)-(((CylindricDomain) getDomain()).getHeight()-101000-250000)*0.1);
		check=-775000+100000-7500+400000;
		*/
		float check=-900000;		

		if ((getCenter()[1])>=check){ 
			//releaseTime=getManager().getCurrentTime();
			setState(new nEncode(this));
			getState().setExecutionTime(1);	
		}

	}

	public String assignName(){		
		return "Calcium Cell TX";		
	}

	@Override
	public void processInfo() {
		// TODO Auto-generated method stub		
	}
}