package simulator;


import java.util.*;
import MainLibrary.AbstractDomain;
import MainLibrary.AbstractThreadingStrategy;
import MainLibrary.LigandObserver;
import MainLibrary.NanoObj;
import MainLibrary.RootDomain;

public class CalciumDiffusionModel extends DiffusionModel{
	//private final double ed=0.9;  //0.5
	private final static double k=1.381*Math.pow(10, -23);	
	public int nodeCount=0;	
	public boolean isTheFirst=true;
	//private transient AbstractThreadingStrategy threadingStrategy = new CalciumDiffusionThreadingStrategy(this);	
	
	public CalciumDiffusionModel(){

		threadingStrategy = new CalciumDiffusionThreadingStrategy(this);
	}

	public CalciumDiffusionModel(RootDomain d){
		super(d);
		threadingStrategy = new CalciumDiffusionThreadingStrategy(this);
	}

	

	public void networkCreationCalciumExternal(){	

		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [CALCIUM]");
		System.out.println("-------------------------------------");			
	
		
		// LOAD DATA FROM CONFIGURATION FILE
		/*
		Manager m=getManager();
		
		float height=m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "JarHeight");
		float rad=m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "JarRadius");
		float carriersConcentration=m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "carriersConcentration");
		float minutes=m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "minutesForCompleteRelease");
		float releaseLevelDisplacement=m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "releaseLevel_Displacement");
		int receptorsForCell=(int) m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "receptorsForCell");
		int receptorsType=(int) m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "receptorsType");
		 */
		// =========== EXTERNAL BIG CYLINDER DOMAIN ===========
		float[] cdCenter=new float[3];
		cdCenter[0]=100000;
		cdCenter[1]=100000;
		cdCenter[2]=100000;		

		float halfSide=50000;
		
		CubicStrategy cs=new CubicStrategy(0);		
		CalciumDiffusionModel mainDiffusion=new CalciumDiffusionModel(getManager().rootDomain);
		
		CubicCalciumDomain mainCube=new CubicCalciumDomain(getManager().rootDomain, mainDiffusion, cs, true, false, halfSide, cdCenter, false);
		mainCube.ID="EXTERNAL CALCIUM CUBE";
		
		//==============================
		
		float[] cdCenter2=new float[3];
		cdCenter2[0]=0;
		cdCenter2[1]=0;
		cdCenter2[2]=0;	
		
		float sRadius=(float) (5000+6.5+3.5);
		
		CalciumSphericalStrategy ss=new CalciumSphericalStrategy(0);		
		CalciumDiffusionModel innerDiffusion=new CalciumDiffusionModel(getManager().rootDomain);
		
		CalciumSphericalDomain sphericalBound=new CalciumSphericalDomain(mainCube, innerDiffusion, ss, true, false, sRadius, cdCenter2, false);
				
		// =================		
		
		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		
		observer.add(CalciumObserver.getInstance());
		CalciumCell_RX cell=new CalciumCell_RX(sphericalBound,100000,100000,100000,20,observer);
		
		
		observer=new ArrayList<LigandObserver>();
		observer.add(CalciumObserver.getInstance());
		CalciumCell_TX cell1=new CalciumCell_TX(mainCube,100000,100000,(float)89993.5,21,observer);
				
		System.out.println(">>> SIMULATION DOMAINS DISPOSED! ");			
		System.out.println();	
	}

	
	public void networkCreationCalciumInternal(){	

		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [CALCIUM INTRACELLULAR]");
		System.out.println("-------------------------------------");			

			
		
		float[] cdCenter2=new float[3];
		cdCenter2[0]=100000;
		cdCenter2[1]=100000;
		cdCenter2[2]=100000;	
		
		float sRadius=(float) (8000);
		
		CalciumSphericalStrategy ss=new CalciumSphericalStrategy(0);		
		CalciumDiffusionModel innerDiffusion=new CalciumDiffusionModel(getManager().rootDomain);
		
		CalciumSphericalDomain sphericalCell=new CalciumSphericalDomain(getManager().rootDomain, innerDiffusion, ss, true, false, sRadius, cdCenter2, false);
		
		sphericalCell.disposeBGCalcium();

		List<NanoObj> ions=sphericalCell.getAllDisposedObj();
		
		System.out.println(">>> 1 - BG Ca++ "+sphericalCell.carrierList.size());	
				
		for (int i=0;i<ions.size() ;i++){			
			try{

				/** FIXME have to control also the collisions with the other domains that share the same parentDomain! 
				 * So it is needed also the collision control on the external shells. 
				 * In what follows it considers only the collisions with inner shell. */  
				

				// TIP: You can evaluate the boundig sphere to perform a preliminary check collision
				if(ions.get(i).getTimeToLive()>0){
					
					if((ions.get(i).getAbsDistancePlusRadius()>=sphericalCell.getBoundingRadius())){													
						ions.get(i).destroy();					
					} 				
				}
			}catch (Exception e){
				e.printStackTrace();
			}
		}	
		
		System.out.println(">>> 2 - BG Ca++ "+sphericalCell.carrierList.size());	
		
		// =================
		
		
		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		
		observer.add(CalciumObserver.getInstance());
		CalciumCell_RX cell=new CalciumCell_RX(sphericalCell,100000,100000,107450,25,observer); // z: 101745  // z: 104600
				
		observer=new ArrayList<LigandObserver>();
		observer.add(CalciumObserver.getInstance());
		CalciumCell_TX cell1=new CalciumCell_TX(sphericalCell,100000,100000,(float)97260,24,observer);  // z: 98500
				
		sphericalCell.BGAssimilationCount=0;
		System.out.println(">>> 3 - BG Ca++ "+sphericalCell.carrierList.size());	
		System.out.println(">>> SIMULATION DOMAINS DISPOSED! ");			
		System.out.println();	
	}
	

	public void move(List<NanoObj> nanoList){		
			
		if (nanoList.size()>0){
			try{
				threadingStrategy.move(nanoList);				
			}catch (Exception e){
				e.printStackTrace();
			}			
		}
	}

	
	public synchronized void avoidCollision(AbstractDomain d){		
		boolean isDone=false;
		try {	
			isDone=reorderCollision_byHeap(d);
		}catch (Exception e) {
			System.out.println("-1- Error on avoidCollision. Collisionlist: "+d.getCollisionsList().size()+" - "+e.toString());
			e.printStackTrace();
		}		
		try{
			int size=d.getCollisionsList().size(); 				
			for (int i=0;i<size;i++){
				
				d.getCollisionsList().get(i).handleCollision();					
			}
		}catch (Exception e) {
			System.out.println("-2- Error on avoidCollision. "+d.getCollisionsList().size()+" - "+e.toString()+" - isDone: "+isDone);
			e.printStackTrace();
		}
		d.getCollisionsList().clear();	
	}
	

	public void networkCreation(){		
		//networkCreationCalciumExternal();		
		networkCreationCalciumInternal();		
	}	
	
	@Override
	public AbstractThreadingStrategy getThreadingStrategy(){
		return this.threadingStrategy;
	}
}
