package simulator;

import java.util.Random;
import java.util.concurrent.CountDownLatch;
import MainLibrary.AbstractDomain;


public class CalciumDisposeThread extends Thread {
	AbstractDomain domain;
	//DiffusionModel d;
	long quantity;
	float bound;
	float[] origin=new float[3];
	int type;
	float depth=1;
	
	private CountDownLatch latch;

	//public CalciumDisposeThread(AbstractDomain domain,DiffusionModel d, int size, float bound, float[] origin, int type, CountDownLatch latch) {
	public CalciumDisposeThread(AbstractDomain domain, long quantity, float bound, float depth, float[] origin, int type, CountDownLatch latch) {
		this.domain=domain;
		
		this.quantity=quantity;
		this.bound=bound;
	
		this.type=type;
		this.latch=latch;
		this.depth=depth;
		
		this.origin[0]=domain.getAbsCenter()[0]-origin[0];
		this.origin[1]=domain.getAbsCenter()[1]-origin[1];
		this.origin[2]=domain.getAbsCenter()[2]-origin[2];
	}

	public void run(){

		fillOnCube();
		//fillOnRing();	
		//fillOnSphere();
		//fillOnFixedPosition();		
		
		latch.countDown();	
	}
	
	public void fillOnCube(){
		float x,y,z;
		Random rand=new Random();	

		bound=bound*2;
		

		for(long i=0;i<quantity;i++){		

			x= (float) (((rand.nextDouble()-0.5)*bound)+origin[0]);
			y= (float) (((rand.nextDouble()-0.5)*bound)+origin[1]);
			z= (float) (((rand.nextDouble()-0.5)*bound)+origin[2]);	

			new CalciumCarrierBG(domain,x,y,z,type);	
		}
	}
	
	public void fillOnRing(){
		float x,y,z;
		Random rand=new Random();	
		
		float theta=0;
		float fi=0;			

		for (long i=0;i<quantity;i++){
			theta=domain.getMobilityModel().getRandomAngles()[0];
			fi=domain.getMobilityModel().getRandomAngles()[1];		

			double rr=rand.nextDouble();
			double rr1=rr*depth;
			double rad=bound+(rr1)+1;
			
			
			x=(float) (rad*Math.sin((double)theta)*Math.cos((double)fi))+origin[0];
			y=(float) (rad*Math.sin((double)theta)*Math.sin((double)fi))+origin[1];
			z=(float) (rad*Math.cos((double)theta))+origin[2];			

			new CalciumCarrierBG(domain,x,y,z,type);
		}
	}
	
	public void fillOnSphere(){
		float x,y,z;
		Random rand=new Random();	
		
		float theta=0;
		float fi=0;			

		for (long i=0;i<quantity;i++){
			theta=domain.getMobilityModel().getRandomAngles()[0];
			fi=domain.getMobilityModel().getRandomAngles()[1];		

			double rr=rand.nextDouble();
			
			double rad=bound*(rr)-1;	
			
			
			x=(float) (rad*Math.sin((double)theta)*Math.cos((double)fi))+origin[0];
			y=(float) (rad*Math.sin((double)theta)*Math.sin((double)fi))+origin[1];
			z=(float) (rad*Math.cos((double)theta))+origin[2];			

			new CalciumCarrierBG(domain,x,y,z,type);
		}
	}
	
	public void fillOnFixedPosition(){
		float x,y,z;
		Random rand=new Random();	
		
		float theta=(float) Math.PI;
		float fi=0;			

		for (long i=0;i<quantity;i++){
			
			double rad=bound+(rand.nextDouble()*depth)+1;			
			
			x=(float) (rad*Math.sin((double)theta)*Math.cos((double)fi))+origin[0];
			y=(float) (rad*Math.sin((double)theta)*Math.sin((double)fi))+origin[1];
			z=(float) (rad*Math.cos((double)theta))+origin[2];			

			new CalciumCarrierBG(domain,x,y,z,type);
		}
	}
}
