package simulator;

import java.awt.Color;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.threed.jpct.SimpleVector;

import MainLibrary.AbstractCollisionDomainStrategy;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.Manager;
import MainLibrary.MobilityModel;
import MainLibrary.NanoObj;
import MainLibrary.NodeObj;

public class CalciumSphericalDomain extends AbstractDomain{	

	private float radius=0;
	public long BGAssimilationCount=0;

	private final int maxThreads = Runtime.getRuntime().availableProcessors();
	private ExecutorService executorThreads = Executors.newFixedThreadPool(maxThreads);	 
	private CountDownLatch latch=new CountDownLatch(0);


	// ======     BG CALCIUM CONFIGURATION  EXTRACELLULAR  ======
	/*
	float depth=6.5f;
	float BGRadius=5000;	//RX CELL RADIUS
	long BGCalcium=18900;
	float[] BGCalciumCenter={0,0,0};
	float[] BGCenter=new float[3];
	 */
	// ====   END OF BG CALCIUM CONFIGURATION   ====


	// ======     BG CALCIUM CONFIGURATION  INTRACELLULAR  ======

	float depth=0f;
	float BGRadius=8000;	//RX CELL RADIUS
	//long BGCalcium=(long) (18900*2.38);   //1.59 - cell 5um radius
	long BGCalcium=(long) (77490*2.39);   // cell 10um radius

	float[] BGCalciumCenter={0,0,0};
	float[] BGCenter=new float[3];
	// ====   END OF BG CALCIUM CONFIGURATION   ====



	public CalciumSphericalDomain(AbstractDomain parentDomain, MobilityModel model, AbstractCollisionDomainStrategy strategy, boolean visible, boolean export, float radius, float[] dCenter, boolean useOctree) {
		super(parentDomain, model, strategy, visible, export, dCenter, useOctree);			
		setRadius(radius);
		setBoundingRadius();		
		ID="CALCIUM SPHERICAL DOMAIN";

		if(useOctree) initializeOctree(getManager().configurator);

		if (isVisible()) getManager().engine.addWrapper3DDomain(this);	
	}


	public void setRadius(float radius) {
		this.radius = radius;
	}	


	public void handleBoundCollision(NanoObj n){		
		if(n.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(n,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(n,this);
		}
	}

	public void handleBoundCollision(NodeObj n){
		if(n.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(n,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(n,this);
		}
	}

	public void handleBoundCollision(CarrierObj c){			
		if(c.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(c,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(c,this);
		}
	}

	public String getType(){
		return "CALCIUM SPHERICAL DOMAIN";
	}	

	public float getRadius() {
		return radius;
	}

	@Override
	public void setBoundingRadius() {
		boundingRadius=getRadius();		
	}	

	@Override
	public void trigger() {

		try {
			/*   MULTITHREAD???  */
			if(BGAssimilationCount>0){		

				long tempBGAssimilatedCount=BGAssimilationCount;
				int recreated=0;
				//preloadOfCalciumThread(this, BGAssimilationCount, depth, radius, center, 2);

				float x,y,z;
				Random rand=new Random();	

				float theta=0;
				float fi=0;			

				float[] dAbsCenter=getAbsCenter();

				BGCenter[0]=dAbsCenter[0]-BGCalciumCenter[0]; 
				BGCenter[1]=dAbsCenter[1]-BGCalciumCenter[1]; 
				BGCenter[2]=dAbsCenter[2]-BGCalciumCenter[2]; 



				for (long i=0;i<tempBGAssimilatedCount;i++){
					boolean done=false;
					while(!done){
						theta=getMobilityModel().getRandomAngles()[0];
						fi=getMobilityModel().getRandomAngles()[1];		

						double rad=BGRadius*2;			

						x= (float) (((rand.nextDouble()-0.5)*rad)+BGCenter[0]);
						y= (float) (((rand.nextDouble()-0.5)*rad)+BGCenter[1]);
						z= (float) (((rand.nextDouble()-0.5)*rad)+BGCenter[2]);	

						CalciumCarrierBG cc=new CalciumCarrierBG(this,x,y,z,2);
						cc.setColor(Color.green);

						if(cc.getTimeToLive()>0){

							if((cc.getAbsDistancePlusRadius()>=getBoundingRadius())){													
								cc.destroy();					
							} else{


								boolean endSearch=false;		

								float[] coords=cc.getCenter();
								float radius=cc.getRadius();

								for (int k=0;k<nodeList.size()& (!endSearch);k++){



									float[] tempCoords=nodeList.get(k).getCenter();
									float tempRadius=nodeList.get(k).getRadius();
									//check=i;
									float distance=getDistance(coords,tempCoords);


									if (distance<(radius+tempRadius)){
										cc.destroy();
										endSearch=true;
									}
								}

								if(!endSearch){
									done=true;
									recreated++;
								}

							}
						}
					}

				}
				/*
				System.out.println(">>>>>> BG ASSIMILATED: "+BGAssimilationCount);
				System.out.println(">>>>>> BG ASSIMILATED TEMP: "+tempBGAssimilatedCount);
				System.out.println(">>>>>> BG CREATED: "+recreated);
				 */
				BGAssimilationCount=0;
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("- Exception - CubicCalciumDomain.trigger ");
		}		


		/*  INITIALIZE  */
		//triggerForExternalCell();
	}

	public void disposeBGCalcium(){
		((iManager)getManager()).spherical=this;

		//preloadOfCalcium(mainCube, mainDiffusion, 1000000, halfSide, cdCenter, 2);
		long start=System.currentTimeMillis();
		//preloadOfCalciumThread(this, getMobilityModel(), 10000, 5000, center, 2);

		preloadOfCalciumThread(this, BGCalcium, BGRadius, depth, BGCalciumCenter, 2);
		long stop=System.currentTimeMillis();
		System.out.println("Calcium disposed in: "+(stop-start));

	}

	public void triggerForInternalCell(){
		try {

			if(getManager().getCurrentTime()==1){
				disposeBGCalcium();
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("- Exception - CubicCalciumDomain.trigger - INITIALIZATION... ");
		}		


		try {
			/*   MULTITHREAD???  */
			if(BGAssimilationCount>0){		

				//preloadOfCalciumThread(this, BGAssimilationCount, depth, radius, center, 2);

				float x,y,z;
				Random rand=new Random();	

				float theta=0;
				float fi=0;			

				float[] dAbsCenter=getAbsCenter();

				BGCenter[0]=dAbsCenter[0]-BGCalciumCenter[0]; 
				BGCenter[1]=dAbsCenter[1]-BGCalciumCenter[1]; 
				BGCenter[2]=dAbsCenter[2]-BGCalciumCenter[2]; 

				for (long i=0;i<BGAssimilationCount;i++){
					theta=getMobilityModel().getRandomAngles()[0];
					fi=getMobilityModel().getRandomAngles()[1];		

					double rad=BGRadius+(rand.nextDouble()*depth)+1;			

					x=(float) (rad*Math.sin((double)theta)*Math.cos((double)fi))+BGCenter[0];
					y=(float) (rad*Math.sin((double)theta)*Math.sin((double)fi))+BGCenter[1];
					z=(float) (rad*Math.cos((double)theta))+BGCenter[2];			

					CalciumCarrierBG cc=new CalciumCarrierBG(this,x,y,z,2);

					cc.setColor(Color.green);
				}
				//System.out.println(">>>>>> BG ASSIMILATED: "+BGAssimilationCount);
				BGAssimilationCount=0;
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("- Exception - CubicCalciumDomain.trigger ");
		}	
	}

	public void triggerForExternalCell(){
		try {

			if(getManager().getCurrentTime()==1){

				((iManager)getManager()).spherical=this;
				/**/
				//preloadOfCalcium(mainCube, mainDiffusion, 1000000, halfSide, cdCenter, 2);
				long start=System.currentTimeMillis();
				//preloadOfCalciumThread(this, getMobilityModel(), 10000, 5000, center, 2);

				preloadOfCalciumThread(this, BGCalcium, BGRadius, depth, BGCalciumCenter, 2);
				long stop=System.currentTimeMillis();
				System.out.println("Calcium disposed in: "+(stop-start));

			
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("- Exception - CubicCalciumDomain.trigger - INITIALIZATION... ");
		}		


		try {
			/*   MULTITHREAD???  */
			if(BGAssimilationCount>0){		

				//preloadOfCalciumThread(this, BGAssimilationCount, depth, radius, center, 2);

				float x,y,z;
				Random rand=new Random();	

				float theta=0;
				float fi=0;			

				float[] dAbsCenter=getAbsCenter();

				BGCenter[0]=dAbsCenter[0]-BGCalciumCenter[0]; 
				BGCenter[1]=dAbsCenter[1]-BGCalciumCenter[1]; 
				BGCenter[2]=dAbsCenter[2]-BGCalciumCenter[2]; 

				for (long i=0;i<BGAssimilationCount;i++){
					theta=getMobilityModel().getRandomAngles()[0];
					fi=getMobilityModel().getRandomAngles()[1];		

					double rad=BGRadius+(rand.nextDouble()*depth)+1;			

					x=(float) (rad*Math.sin((double)theta)*Math.cos((double)fi))+BGCenter[0];
					y=(float) (rad*Math.sin((double)theta)*Math.sin((double)fi))+BGCenter[1];
					z=(float) (rad*Math.cos((double)theta))+BGCenter[2];			

					CalciumCarrierBG cc=new CalciumCarrierBG(this,x,y,z,2);

					cc.setColor(Color.green);
				}
				//System.out.println(">>>>>> BG ASSIMILATED: "+BGAssimilationCount);
				BGAssimilationCount=0;
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("- Exception - CubicCalciumDomain.trigger ");
		}	
	}

	@Override
	public Map<Object, Object> export() {
		exportString=new HashMap<Object, Object>();		
		exportString.put("ID", ID);	
		return exportString;
	}

	@Override
	public boolean isInside(NanoObj objToCheck) {
		//FIXME To be deeper tested...


		// If the Obj's center is inside, then the whole object is considered inside!		
		boolean isInside=false;
		float[] distances=getCollisionStrategy().getDistanceOnAxis(this,objToCheck);			
		float distanceP= distances[0]-objToCheck.getRadius();		
		float distanceN= distances[1]-objToCheck.getRadius();			
		float distanceO= distances[2]-objToCheck.getRadius();	

		if (((Math.abs(distanceP))<=getRadius()) && (Math.abs(distanceN)<=getRadius()) && (Math.abs(distanceO)<=getRadius())){	
			isInside=true;
		} 
		return isInside;
	}

	@Override
	public void initializeWrapper3D(Manager m){
		if(m.is3DEnable) getManager().engine.addWrapper3DDomain(this);	
	}


	@Override
	public void initializeDirectionAxes() {
		// Sets the three direction axes!

		float[] dCenter1=new float[3];
		dCenter1[0]=0;
		dCenter1[1]=1;
		dCenter1[2]=0;		
		setDirectionP(new SimpleVector(dCenter1));				

		float[] dCenter2=new float[3];
		dCenter2[0]=1;
		dCenter2[1]=0;
		dCenter2[2]=0;
		setDirectionN(new SimpleVector(dCenter2));		
	}

	//public void preloadOfCalciumThread(AbstractDomain domain,DiffusionModel d, int size, float bound, float[] origin, int type){	
	public void preloadOfCalciumThread(AbstractDomain domain, long quantity, float bound, float depth, float[] origin, int type){

		int div=maxThreads*1;

		if(quantity>div){
			long intPart=quantity/div;	

			if (div>0) {
				latch=new CountDownLatch(div);
			}				

			for(int i=0;i<div-1;i++){
				//executorThreads.execute(new CalciumDisposeThread(domain, d, intPart, bound, origin, type, latch));	
				executorThreads.execute(new CalciumDisposeThread(domain, intPart, bound, depth, origin, type, latch));
			}
			//executorThreads.execute(new CalciumDisposeThread(domain, d, intPart, bound, origin, type, latch));
			executorThreads.execute(new CalciumDisposeThread(domain, intPart, bound, depth, origin, type, latch));

		}else if(quantity>0){
			//executorThreads.execute(new CalciumDisposeThread(domain, d, size, bound, origin, type, latch));
			executorThreads.execute(new CalciumDisposeThread(domain, quantity, bound, depth, origin, type, latch));

		}		

		try {
			latch.await();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}

	public void detachNanoObj(CarrierObj c){

		if(c instanceof CalciumCarrierBG){
			//System.out.println("CARRIER BG!!");
			BGAssimilationCount++;
			//c.setColor(Color.red);
		}		

		detachNanoObjOriginal(c);
	}

	public void detachNanoObjOriginal(CarrierObj c){
		undisposedCarrierList.remove(c);
		carrierList.remove(c);
		getLivingObjObserver().removeCarrier(c);		
		//c.setDomain(null);

		try {
			// Octree
			//getRootTree().remove(c);
			removeFromOctree(c);

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("Eccezione AbstractDomain.detachCarrier -> OCTREE - "+ID);
		}
	}	


	public boolean isFreeSpace(NanoObj obj){
		float[] coords=obj.getCenter();
		float radius=obj.getRadius();
		boolean isFree=true;
		float[] tempCoords=new float[3];
		double tempRadius=0;
		boolean endSearch=false;

		List<AbstractDomain> childDomainList=getAllChildDomains();		
		for(int i=0;i<childDomainList.size() && isFree;i++){
			if(childDomainList.get(i).isInside(obj)) isFree=false;
		}			

		if(isFree){
			List<NanoObj> allNanoObjList=getAllDisposedObj();			

			for (int i=0;i<allNanoObjList.size()& (!endSearch);i++){



				tempCoords=allNanoObjList.get(i).getCenter();
				tempRadius=allNanoObjList.get(i).getRadius();
				//check=i;
				float distance=getDistance(coords,tempCoords);


				if (distance<(radius+tempRadius)){
					if(allNanoObjList.get(i) instanceof CalciumCarrierBG){

						allNanoObjList.get(i).destroy();					

					}else{
						isFree=false;
						endSearch=true;
					}

				}
			}	
		}	
		return isFree;
	}	
}
