package simulator;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import MainLibrary.AbstractCollision;
import MainLibrary.AbstractCollisionDomainStrategy;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.Manager;
import MainLibrary.MobilityModel;
import MainLibrary.NanoCollision;
import MainLibrary.NanoObj;
import MainLibrary.NodeObj;

import com.threed.jpct.SimpleVector;

public class CubicDomain extends AbstractDomain{
	
	private float halfSide=0;
	public Random rand=new Random();
	
	
	public CubicDomain(AbstractDomain parentDomain, MobilityModel model, AbstractCollisionDomainStrategy strategy, boolean visible, boolean export, float halfSide, float[] center,  boolean useOctree) {
		super(parentDomain, model, strategy, visible, export, center, useOctree);		
		setHalfSide(halfSide);	
		setBoundingRadius();		
		
		if(useOctree) initializeOctree(getManager().configurator);
		
		if (isVisible()) getManager().engine.addWrapper3DDomain(this);			
	}		
	
	public void handleBoundCollision(NanoObj n){		
		if(n.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(n,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(n,this);
		}
	}
	
	public void handleBoundCollision(NodeObj n){
		if(n.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(n,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(n,this);
		}
	}
	
	public void handleBoundCollision(CarrierObj c){			
		if(c.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(c,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(c,this);
		}
	}
	
	public String getType(){
		return "Cube Domain";
	}
	
	public void setBoundingRadius(){
		boundingRadius=(float) (getHalfSide()*Math.sqrt(3));		
	}

	public float getHalfSide() {
		return this.halfSide;
	}

	public void setHalfSide(float halfSide) {
		this.halfSide = halfSide;
	}	

	@Override
	public void trigger() {
		
	}

	@Override
	public Map<Object, Object> export() {
		exportString=new HashMap<Object, Object>();		
		//exportString.put("ID", ID);	
		
		// FIXME DEBUG
		boolean hasToExport=false;
		long time=getManager().getCurrentTime();
		
		//if(time%10000==0){
		if(time%localExportTime==0 || time==1000){
			hasToExport=true;
		}
		/*
		if(hasToExport){
			exportString.put("Type", "CarrierCollision");
				
			DiffusionModel mod=(DiffusionModel)getMobilityModel();
			exportString.put("ID", "CubicDomain-"+ID);	
			exportString.put("cCollision", ""+mod.cCollision);
			
			exportString.put("timestep", time);
		}*/
		// END DEBUG
		
		
		
		return exportString;
	}

	@Override
	public boolean isInside(NanoObj objToCheck) {
		
		// If the Obj's center is inside, then the whole object is considered inside!		
		boolean isInside=false;
		float[] distances=getCollisionStrategy().getDistanceOnAxis(this,objToCheck);			
		float distanceP= distances[0]-objToCheck.getRadius();		
		float distanceN= distances[1]-objToCheck.getRadius();			
		float distanceO= distances[2]-objToCheck.getRadius();	

		if (((Math.abs(distanceP))<=getHalfSide()) && (Math.abs(distanceN)<=getHalfSide()) && (Math.abs(distanceO)<=getHalfSide())){	
			isInside=true;
		} 
		return isInside;
	}

	@Override
	public void initializeWrapper3D(Manager m){
		if(m.is3DEnable) getManager().engine.addWrapper3DDomain(this);	
	}

	@Override
	public void initializeDirectionAxes() {
		// Sets the three direction axes!
		
		float[] dCenter1=new float[3];
		dCenter1[0]=0;
		dCenter1[1]=1;
		dCenter1[2]=0;		
		setDirectionP(new SimpleVector(dCenter1));				
		
		float[] dCenter2=new float[3];
		dCenter2[0]=1;
		dCenter2[1]=0;
		dCenter2[2]=0;
		setDirectionN(new SimpleVector(dCenter2));		
	}
	
	public void addToCollisionsList(NanoObj p1,NanoObj p2,float dt){		
		AbstractCollision c=new NanoCollision(p1,p2,dt);
		
		//setCollisionsList(c);
		setToFreeCollisionsList3(c);				
		
		p1.increaseCollisionCount();
		p2.increaseCollisionCount();
	}
}
