package simulator;


/** 
 * The RECEPTOR CREATION has to be invoked outside of its constructor! 
 * Take into account the two cases: FLATTED surface or CURVED surface (in this case: createReceptorForHUVEC)
 */

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;


import Wrappers.OutputWrapper;

import MainLibrary.AbstractCollisionDomainStrategy;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.EventHandlerForReceptors;
import MainLibrary.HeapSorterReceptors;
import MainLibrary.LigandObserver;
import MainLibrary.Manager;
import MainLibrary.MobilityModel;
import MainLibrary.ReceptorLocalizationService;
import MainLibrary.UniversalReceptor;

import com.threed.jpct.Matrix;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;

import engine.Engine3D;


public class CubicReceptorDomain extends CubicDomain{

	protected int receptorType;
	protected ArrayList<UniversalReceptor> receptorList=new ArrayList<UniversalReceptor>();
	protected ArrayList<LigandObserver> obsList=new ArrayList<LigandObserver>();
	protected Random rand=new Random();
	protected Map<Object,Object> oldExport=new HashMap<Object,Object>();	
	private int totalAssimilations = 0;
	private OutputWrapper output;	
	public EventHandlerForReceptors eventHandler=new EventHandlerForReceptors();
	public float rotationAngleP=0;
	public int receptorQuantity = 0;

	protected ReceptorLocalizationService receptorService;


	public CubicReceptorDomain(AbstractDomain parentDomain, MobilityModel model, AbstractCollisionDomainStrategy strategy,
			boolean visible, boolean export, float halfSide, float[] dCenter, int receptorType, int receptorQuantity, ArrayList<LigandObserver> ligandObserver,  boolean useOctree) {
		super(parentDomain, model, strategy, visible, export, halfSide, dCenter, useOctree);
		this.receptorType=receptorType;
		this.obsList=ligandObserver;
		this.receptorQuantity=receptorQuantity;

		receptorService=new ReceptorLocalizationService(this, 2*halfSide);

		if(visibleOn3D && getManager().is3DEnable) getManager().engine.changeDomainColor(this, Color.green);

		//createReceptors(receptorQuantity);			// Used for normal Cubic Receptor Domain, NOT Endothelial cells (curved surface)
		//createReceptorsForHUVEC(receptorQuantity);  	// Used for the Endothelial cells
		ID="CubeReceptor["+getRelativeCenter()[0]+","+getRelativeCenter()[1]+","+getRelativeCenter()[2]+"]";


		/** Draw each Receptor */	
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		} */
	}

	public void handleBoundCollision(CarrierObj c){

		if(c.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(c,this);
		}else{		
			checkIncomingCarrier(c);
		}
	}

	public void checkIncomingCarrier(CarrierObj c){		
		boolean catched=false;
		float x,  y,  z,  r;
		float dt=(float) getCollisionStrategy().getExternalCollisionTime(c, this);
		float[] velocity=c.getVelocityComponent();

		// Carrier Coords on the Cylindric External Surface
		x=c.getCenter()[0]-velocity[0]*dt;
		y=c.getCenter()[1]-velocity[1]*dt;
		z=c.getCenter()[2]-velocity[2]*dt;		

		r=(float) c.getRadius();
		float[] coords=null;			


		// Gets the receptorList for this impact point!
		//short qqq=receptorService.getQuadrantForCarrier(this,c);
		//System.out.println("quadrant: "+qqq);
		ArrayList<UniversalReceptor> receptorList=receptorService.getQuadrantList(receptorService.getQuadrantForCarrier(this,c));
		//---------------------------------------------


		for (int i=0;i<receptorList.size() && !catched;i++){			
			if((receptorList.get(i).isFree() && receptorList.get(i).canDownload())){				
				coords=receptorList.get(i).getAbsoluteCoords();
				float dim=receptorList.get(i).radius;								
				float dx = (coords[0]-x), dy = (coords[1]-y), dz= (coords[2]-z);
				float distance =(float) Math.sqrt(dx*dx+dy*dy+dz*dz);	
				if (distance<=((dim+r)) && (receptorList.get(i).checkCarrierCompatibility(c))){	
					System.out.println("== Cube Receptor Domain - DOWNLOAD!");
					this.totalAssimilations++;
					catched=true;
					receptorList.get(i).configureForDownload(c);
				}
			}
		}
		if(!catched){			
			/** In this special case, do not relocate Objs! Because they have to collide with
			 * 	the inner side of the Cylinder Domain (i.e. the blood vessel). 
			 * */
			//getCollisionStrategy().relocateOuterObjs(c,this);
		}
	} 

	public ArrayList<UniversalReceptor> getReceptors(){
		return receptorList;
	}

	public void createReceptors(int n){	
		float theta=0;
		float fi=0;		
		float[] cdCenter1=new float[3];		
		int created=0;

		while(created<n){
			UniversalReceptor r = null;	

			float xx=(float)((rand.nextFloat()*2*getHalfSide())-getHalfSide());
			float zz=(float)((rand.nextFloat()*2*getHalfSide())-getHalfSide());

			// Relative Coords!
			cdCenter1[0]=xx;
			cdCenter1[1]=-getHalfSide();
			cdCenter1[2]=zz;	

			r=new CD40UniversalReceptor(this, parentDomain, true, cdCenter1, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorType+"-radius").shortValue(),receptorType, this.eventHandler);
			receptorList.add(r);
			receptorSetup(r); 
			created++;
		}		

		System.out.println();
		System.out.println(this.toString());
		removeOverlappedReceptors(receptorList);
		/*// Write Receptors Table
		WriteReceptorsFile w=WriteReceptorsFile.getInstance();
		w.initialize();
		w.export(receptorList);*/
		System.out.println("RECEPTOR SURVIVED: "+receptorList.size());
	}


	public void createReceptorsForHUVEC(int n, int huvecNumber, int index, SimpleVector layer, SimpleVector dVersor){	
		float theta=0;
		float fi=0;	
		float[] cdCenter=new float[3];		
		int created=0;
		float halfSide = (float)getHalfSide();	
		CylindricDomain parent=(CylindricDomain) this.parentDomain;

		while(created<n){

			//for(int ii=-1;ii<=1;ii=ii+2){
			//for(int jj=-1;jj<=1;jj=jj+2){

			UniversalReceptor r = null;	

			// Relative Coords!
			/*
			cdCenter[0]=xx;
			cdCenter[1]=-halfSide;
			cdCenter[2]=zz;*/

			//float xx=(float) (ii*0.5*halfSide);
			//float zz=(float) (jj*0.5*halfSide);


			// Receptors placement on the internal side of the cube
			float xx=(float)((rand.nextFloat()*2*halfSide)-halfSide);
			float zz=(float)((rand.nextFloat()*2*halfSide)-halfSide);

			if(xx > -halfSide && xx < halfSide && zz > -halfSide && zz < halfSide) {
				SimpleVector receptorVector = new SimpleVector();
				receptorVector.x = xx;
				receptorVector.y = 0;
				receptorVector.z = zz;

				SimpleVector planeVersor = new SimpleVector(0, 1, 0);
				SimpleVector cubeDirection = new SimpleVector(getDirectionP());

				SimpleVector rotationVersor = planeVersor.calcCross(cubeDirection);
				float angle = planeVersor.calcAngle(cubeDirection);

				receptorVector.y = receptorVector.y + halfSide;

				Matrix rotation = new Matrix();
				rotation.rotateAxis(rotationVersor, angle);
				receptorVector.rotate(rotation);

				Matrix rotation2 = new Matrix();
				rotation2.rotateAxis(cubeDirection, (float) ((2*Math.PI/huvecNumber))*(huvecNumber-index));
				receptorVector.rotate(rotation2);

				cdCenter[0] = receptorVector.x;
				cdCenter[1] = receptorVector.y;
				cdCenter[2] = receptorVector.z;


				/** ========================  Compute the curvature ======================= */
				float[] curveCoords=new float[3];

				curveCoords[0]=cdCenter[0]+getAbsCenter()[0]-layer.x;
				curveCoords[1]=cdCenter[1]+getAbsCenter()[1]-layer.y;
				curveCoords[2]=cdCenter[2]+getAbsCenter()[2]-layer.z;

				SimpleVector curveVector=new SimpleVector(curveCoords);				
				SimpleVector curveVectorTemp=new SimpleVector(curveCoords);

				float angleDVersor=(float) (Math.PI/2-curveVectorTemp.calcAngle(dVersor));	
				float module=(float) (parent.getRadius()/Math.cos(angleDVersor));

				curveVector=curveVector.normalize();
				curveVector.scalarMul(module);

				curveVector.x=curveVector.x-getAbsCenter()[0]+layer.x;
				curveVector.y=curveVector.y-getAbsCenter()[1]+layer.y;
				curveVector.z=curveVector.z-getAbsCenter()[2]+layer.z;

				cdCenter[0] = curveVector.x;
				cdCenter[1] = curveVector.y;
				cdCenter[2] = curveVector.z;

				float cylindricProjection=(float) (module*Math.sin(angleDVersor));
				/** ============================ END curvature =========================== */

				if(cylindricProjection<halfSide && cylindricProjection>-halfSide){
					r=new CD40UniversalReceptor(this, parentDomain, true, cdCenter, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorType+"-radius").shortValue(),receptorType, this.eventHandler);
					receptorList.add(r);
					receptorSetup(r); 
					created++;

					receptorService.addReceptor(r);

					//drawSmallPoint(this,r);
				}				
			}
		}		

		//}}
		//removeOverlappedReceptors(receptorList);		// ORIGINAL - Before receptorService

		/**/
		long start = System.nanoTime();
		// NEW CHECK FOR OVERLAPPING RECEPTORS!
		for(short q=0;q<4;q++){					
			removeOverlappedReceptors(receptorService.getQuadrantList(q));
		}
		long stop = System.nanoTime();

		System.out.println("Remove OVERLAPPING in: "+(stop-start)/1000);



		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		 */

		// Write Receptors Table
		/*
		WriteReceptorsFile w=WriteReceptorsFile.getInstance();
		w.initialize();
		w.export(receptorList);
		 */
		System.out.println("RECEPTOR SURVIVED: "+receptorList.size());
	}

	public void createReceptorsForHUVEC_ORIGINAL_NO_RECEPTORSERVICE(int n, int huvecNumber, int index, SimpleVector layer, SimpleVector dVersor){	
		float theta=0;
		float fi=0;	
		float[] cdCenter=new float[3];		
		int created=0;
		float halfSide = (float)getHalfSide();	
		CylindricDomain parent=(CylindricDomain) this.parentDomain;

		while(created<n){
			UniversalReceptor r = null;	

			// Relative Coords!
			/*
			cdCenter[0]=xx;
			cdCenter[1]=-halfSide;
			cdCenter[2]=zz;*/

			// Receptors placement on the internal side of the cube
			float xx=(float)((rand.nextFloat()*2*halfSide)-halfSide);
			float zz=(float)((rand.nextFloat()*2*halfSide)-halfSide);

			if(xx > -halfSide && xx < halfSide && zz > -halfSide && zz < halfSide) {
				SimpleVector receptorVector = new SimpleVector();
				receptorVector.x = xx;
				receptorVector.y = 0;
				receptorVector.z = zz;

				SimpleVector planeVersor = new SimpleVector(0, 1, 0);
				SimpleVector cubeDirection = new SimpleVector(getDirectionP());

				SimpleVector rotationVersor = planeVersor.calcCross(cubeDirection);
				float angle = planeVersor.calcAngle(cubeDirection);

				receptorVector.y = receptorVector.y + halfSide;

				Matrix rotation = new Matrix();
				rotation.rotateAxis(rotationVersor, angle);
				receptorVector.rotate(rotation);

				Matrix rotation2 = new Matrix();
				rotation2.rotateAxis(cubeDirection, (float) ((2*Math.PI/huvecNumber))*(huvecNumber-index));
				receptorVector.rotate(rotation2);

				cdCenter[0] = receptorVector.x;
				cdCenter[1] = receptorVector.y;
				cdCenter[2] = receptorVector.z;


				/** ========================  Compute the curvature ======================= */
				float[] curveCoords=new float[3];

				curveCoords[0]=cdCenter[0]+getAbsCenter()[0]-layer.x;
				curveCoords[1]=cdCenter[1]+getAbsCenter()[1]-layer.y;
				curveCoords[2]=cdCenter[2]+getAbsCenter()[2]-layer.z;

				SimpleVector curveVector=new SimpleVector(curveCoords);				
				SimpleVector curveVectorTemp=new SimpleVector(curveCoords);

				float angleDVersor=(float) (Math.PI/2-curveVectorTemp.calcAngle(dVersor));	
				float module=(float) (parent.getRadius()/Math.cos(angleDVersor));

				curveVector=curveVector.normalize();
				curveVector.scalarMul(module);

				curveVector.x=curveVector.x-getAbsCenter()[0]+layer.x;
				curveVector.y=curveVector.y-getAbsCenter()[1]+layer.y;
				curveVector.z=curveVector.z-getAbsCenter()[2]+layer.z;

				cdCenter[0] = curveVector.x;
				cdCenter[1] = curveVector.y;
				cdCenter[2] = curveVector.z;

				float cylindricProjection=(float) (module*Math.sin(angleDVersor));
				/** ============================ END curvature =========================== */

				if(cylindricProjection<halfSide && cylindricProjection>-halfSide){
					r=new CD40UniversalReceptor(this, parentDomain, true, cdCenter, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorType+"-radius").shortValue(),receptorType, this.eventHandler);
					receptorList.add(r);
					receptorSetup(r); 
					created++;
				}				
			}
		}		

		removeOverlappedReceptors(receptorList);			


		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		 */

		// Write Receptors Table
		/*
		WriteReceptorsFile w=WriteReceptorsFile.getInstance();
		w.initialize();
		w.export(receptorList);
		*/
		System.out.println("RECEPTOR SURVIVED: "+receptorList.size());
	}

	public void createSingleReceptorForHUVEC(float[] cdCenter){	
		float theta=0;
		float fi=0;	

		UniversalReceptor r = null;	

		r=new CD40UniversalReceptor(this, parentDomain, true, cdCenter, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorType+"-radius").shortValue(),receptorType, this.eventHandler);
		receptorList.add(r);
		receptorSetup(r); 

		//removeOverlappedReceptors(receptorList);


		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		 */

		// Write Receptors Table
		/*
		WriteReceptorsFile w=WriteReceptorsFile.getInstance();
		w.initialize();
		w.export(receptorList);
		 */
		//System.out.println("RECEPTOR SURVIVED: "+receptorList.size());
	}

	public void removeOverlappedReceptors(ArrayList<UniversalReceptor> list){
		System.out.println();
		System.out.println("TOTAL RECEPTORS: "+list.size());	

		float[] origin=new float[3];
		origin[0]=getAbsCenter()[0];
		origin[1]=getAbsCenter()[1];
		origin[2]=getAbsCenter()[2];

		SimpleVector coords1=new SimpleVector();
		double radius1=0;
		SimpleVector coords2=new SimpleVector();
		double radius2=0;
		int deathCount=0;

		List<UniversalReceptor> receptorToRemove=new ArrayList<UniversalReceptor>();
		deathCount=0;
		boolean isFinished=false;

		// Reorder first!!
		long start=System.currentTimeMillis();			
		HeapSorterReceptors h=new HeapSorterReceptors();
		h.sort(list);
		long stop=System.currentTimeMillis();
		System.out.println("| Sorted in: "+(stop-start)+" ms |");	
		// Reorder done...

		start=System.currentTimeMillis();
		for (int i=0;i<list.size()-1;i++){
			if(receptorToRemove.contains(list.get(i))){
				isFinished=true;
			}else{
				isFinished=false;
			}

			for (int j=i+1;j<list.size() && !isFinished;j++){		

				if(list.get(j).getRelativeDistancePlusRadius()>=list.get(i).getRelativeDistanceMinusRadius()){					
					coords1=new SimpleVector(list.get(i).getAbsoluteCoords());
					radius1=list.get(i).getRadius();
					coords2=new SimpleVector(list.get(j).getAbsoluteCoords());
					radius2=list.get(j).getRadius();

					float dx=coords1.x-coords2.x;
					float dy=coords1.y-coords2.y;
					float dz=coords1.z-coords2.z;
					float distance=(float) Math.sqrt(dx*dx+dy*dy+dz*dz);

					if (distance<(radius1+radius2)){

						if(!(receptorToRemove.contains(list.get(j)))){
							receptorToRemove.add(list.get(j));
							deathCount++;							
						}						
					}
				}else{
					isFinished=true;
				}			
			}
		}
		System.out.println("RECEPTORS -> DEATH COUNT: "+deathCount);

		list.removeAll(receptorToRemove);
		receptorList.removeAll(receptorToRemove);	// FIXME Added for receptorService...


		for(int k=0;k<receptorToRemove.size();k++){				
			receptorToRemove.get(k).finalize();				
		}	

		receptorToRemove.clear();
		receptorToRemove=null;		
	}	

	public void receptorSetup(UniversalReceptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	}

	public List<LigandObserver> getObsList(){
		return obsList;
	}

	public Map<Object, Object> exportBAK() {
		exportString=new HashMap<Object, Object>();		
		int currentID=0;
		int assimilations=0;

		for(int i=0;i<receptorList.size();i++){			
			currentID=((CD40UniversalReceptor) receptorList.get(i)).getID();
			assimilations=((CD40UniversalReceptor) receptorList.get(i)).assimilatedInfo;
			exportString.put("Receptor-"+i, currentID);	
			exportString.put("Assimilations-"+i, assimilations);	
		}

		if(oldExport.equals(exportString)){
			exportString=null;
		}else{
			oldExport=exportString;
		}
		return exportString;
	}

	public Map<Object, Object> export() {
		exportString=new HashMap<Object, Object>();		
		int assimilations=0;

		for(int i=0;i<receptorList.size();i++){			
			assimilations+=((CD40UniversalReceptor) receptorList.get(i)).assimilatedInfo;
		}

		if(assimilations>0){
		
			float[] center=getRelativeCenter();
			exportString.put("Type", "CubeReceptor");	
			exportString.put("ID", ID);	
			exportString.put("X", center[0]);
			exportString.put("Y", center[1]);
			exportString.put("Z", center[2]);
			exportString.put("TimeStep", getManager().getCurrentTime());
			exportString.put("Assimilations", assimilations);
			exportString.put("Action", "assimilation");
			
		}
		return exportString;
	}

	@Override
	public void trigger() {	
		// FIXME TO BE TESTED...
		eventHandler.trigger();

		// FIXME Can be removed if the eventHandler.trigger() works well!
		/*
		for (int i=0;i<receptorList.size();i++){			
			receptorList.get(i).trigger();			
		}	
		 */			
	}

	public void initializeWrapper3D(Manager m){
		super.initializeWrapper3D(m);

		for(int i=0; i<receptorList.size();i++){
			CD40UniversalReceptor r = (CD40UniversalReceptor)receptorList.get(i);
			r.drawSmallPoint(r.getParent());
		}
	}	

	public int getTotalAssimilations() {
		return totalAssimilations;
	}

	public void setTotalAssimilations(int totalAssimilations) {
		this.totalAssimilations = totalAssimilations;
	}

	public OutputWrapper getOutput() {
		return output;
	}

	public void setOutput(OutputWrapper output) {
		this.output = output;
	}

	// Draw the receptor in the 3D window
	public void drawSmallPoint(AbstractDomain parent, UniversalReceptor r){

		if(getManager().is3DEnable==true){
			float[] rcenter = new float[3];

			//FIXME per non usare float[]		
			/*
				rcenter[0]=r.center[0]+getAbsCenter()[0];
				rcenter[1]=r.center[1]+getAbsCenter()[1];
				rcenter[2]=r.center[2]+getAbsCenter()[2];
			 */
			rcenter[0]=r.cx+getAbsCenter()[0];
			rcenter[1]=r.cy+getAbsCenter()[1];
			rcenter[2]=r.cz+getAbsCenter()[2];


			SimpleVector v0=new SimpleVector(rcenter);
			Object3D obj3D=null;
			obj3D = Primitives.getBox(100, 1);	
			//obj3D = Primitives.getBox(getRadius(), 1);	
			obj3D.setCulling(false);		
			obj3D.setAdditionalColor(Color.WHITE);		
			obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);		
			obj3D.build();
			obj3D.translate(v0);
			Engine3D engine3d=(Engine3D)parent.getManager().engine;
			engine3d.world.addObject(obj3D);
			obj3D.compileAndStrip();	



			//short quad=receptorService.getQuadrant(r.center[0], r.center[1], r.center[2]);
			float[] relcenter=r.getRelativeCoords();
			short quad=receptorService.getQuadrant(relcenter[0], relcenter[1], relcenter[2]);

			switch (quad){
			case 0:
				obj3D.setAdditionalColor(Color.WHITE);
				break;
			case 1:
				obj3D.setAdditionalColor(Color.black);
				break;
			case 2:
				obj3D.setAdditionalColor(Color.cyan);
				break;
			case 3:
				obj3D.setAdditionalColor(Color.green);
				break;
			}
		}
	}
}
