package simulator;

import java.util.List;
import MainLibrary.*;
import com.threed.jpct.SimpleVector;


public class CubicStrategy extends AbstractCollisionDomainStrategy{	

	public CubicStrategy(int type){
		super(type);
	}

	@Override
	public void checkInternalCollisionBound(List<NanoObj> objsToCheck) {

		for (int i=0;i<objsToCheck.size();i++){			
			try{

				NanoObj objToCheck=objsToCheck.get(i);

				if(objToCheck.getTimeToLive()>0){						
					CubicDomain domain= (CubicDomain) objToCheck.getDomain();						


					SimpleVector dParallel=domain.getDirectionP();		
					SimpleVector nanoVector=new SimpleVector(getRelativeCenter(domain, objToCheck));
					float angle0=nanoVector.calcAngle(dParallel);
					float nanoModule=nanoVector.length();
					float dPOrto=(float) (nanoModule*Math.cos(angle0));

					SimpleVector dOrto=domain.getDirectionN();			
					float angle=nanoVector.calcAngle(dOrto);		
					float dNOrto=(float) (nanoModule*Math.cos(angle));

					SimpleVector dOrto2=domain.getDirectionO();		
					float angle2=nanoVector.calcAngle(dOrto2);
					float dNOrto2=(float) (nanoModule*Math.cos(angle2));

					float distanceP=Math.abs(dPOrto)+objToCheck.getRadius();
					float distanceN=Math.abs(dNOrto)+objToCheck.getRadius();
					float dN2=Math.abs(dNOrto2)+objToCheck.getRadius();

					double dsP=(Math.abs(distanceP))-(domain.getHalfSide());
					double dsN=(Math.abs(distanceN))-(domain.getHalfSide());
					double dsN2=(Math.abs(dN2))-(domain.getHalfSide());				


					float[] velocity1=objToCheck.getVelocityComponent();

					// the collision should have occurred at t-dt (Actually this is also an approximation).
					float dt=1;
					try {	
						if ((Math.abs(distanceP))>=domain.getHalfSide() || (Math.abs(distanceN))>=domain.getHalfSide() || (Math.abs(dN2))>=domain.getHalfSide()  ){	
							if (dsP<=dsN && dsN2<=dsN){								

								SimpleVector velocityVector=new SimpleVector(velocity1);
								float angleV=velocityVector.calcAngle(domain.getDirectionN());
								float dsPV=(float) ((dsN)*Math.cos(angleV));
								dt =(float) Math.abs((dsPV))/Math.abs(objToCheck.getVelocityModule()); // Warning! catch the div-by-zero!!!								

							}else if (dsP<=dsN2 && dsN<=dsN2){
								SimpleVector velocityVector=new SimpleVector(velocity1);
								float angleV=velocityVector.calcAngle(domain.getDirectionO());
								float dsNV=(float) ((dsN2)*Math.cos(angleV));
								dt =(float) Math.abs((dsNV))/Math.abs(objToCheck.getVelocityModule()); 							

							} else if (dsP>=dsN2 && dsN<=dsP){

								SimpleVector velocityVector=new SimpleVector(velocity1);
								float angleV=velocityVector.calcAngle(domain.getDirectionP());
								float dsNV=(float) ((dsP)*Math.cos(angleV));
								dt =(float) Math.abs((dsNV))/Math.abs(objToCheck.getVelocityModule()); 				

							}
							domain.addToCollisionsListInternal(domain,objToCheck,dt);	
						}
					} catch (Exception e) {
						System.out.println(e.getMessage());	
					}
				}
			} catch (Exception e) {
				System.out.println(e.getMessage());	
			}
		}
	}
	
	@Override
	public void checkInternalCollisionBound(NanoObj objToCheck) {
	
		if(objToCheck.getTimeToLive()>0){			
			CubicDomain domain = (CubicDomain) objToCheck.getDomain();
						
			SimpleVector dParallel=domain.getDirectionP();		
			SimpleVector nanoVector=new SimpleVector(getRelativeCenter(domain, objToCheck));
			float angle0=nanoVector.calcAngle(dParallel);
			float nanoModule=nanoVector.length();
			float dPOrto=(float) (nanoModule*Math.cos(angle0));

			SimpleVector dOrto=domain.getDirectionN();			
			float angle=nanoVector.calcAngle(dOrto);		
			float dNOrto=(float) (nanoModule*Math.cos(angle));

			SimpleVector dOrto2=domain.getDirectionO();		
			float angle2=nanoVector.calcAngle(dOrto2);
			float dNOrto2=(float) (nanoModule*Math.cos(angle2));

			float distanceP=Math.abs(dPOrto)+objToCheck.getRadius();
			float distanceN=Math.abs(dNOrto)+objToCheck.getRadius();
			float dN2=Math.abs(dNOrto2)+objToCheck.getRadius();

			double dsP=(Math.abs(distanceP))-(domain.getHalfSide());
			double dsN=(Math.abs(distanceN))-(domain.getHalfSide());
			double dsN2=(Math.abs(dN2))-(domain.getHalfSide());
			

			float[] velocity1=objToCheck.getVelocityComponent();

			// the collision should have occurred at t-dt (Actually this is also an approximation).
			float dt=1;
			try {	
				if ((Math.abs(distanceP))>=domain.getHalfSide() || (Math.abs(distanceN))>=domain.getHalfSide() || (Math.abs(dN2))>=domain.getHalfSide()  ){	
					if (dsP<=dsN && dsN2<=dsN){								

						SimpleVector velocityVector=new SimpleVector(velocity1);
						float angleV=velocityVector.calcAngle(domain.getDirectionN());
						float dsPV=(float) ((dsN)*Math.cos(angleV));
						dt =(float) Math.abs((dsPV))/Math.abs(objToCheck.getVelocityModule()); // Warning! catch the div-by-zero!!!


					}else if (dsP<=dsN2 && dsN<=dsN2){
						SimpleVector velocityVector=new SimpleVector(velocity1);
						float angleV=velocityVector.calcAngle(domain.getDirectionO());
						float dsNV=(float) ((dsN2)*Math.cos(angleV));
						dt =(float) Math.abs((dsNV))/Math.abs(objToCheck.getVelocityModule()); 

					} else if (dsP>=dsN2 && dsN<=dsP){

						SimpleVector velocityVector=new SimpleVector(velocity1);
						float angleV=velocityVector.calcAngle(domain.getDirectionP());
						float dsNV=(float) ((dsP)*Math.cos(angleV));						
						dt =(float) Math.abs((dsNV))/Math.abs(objToCheck.getVelocityModule()); 
					}
					domain.addToCollisionsListInternal(domain,objToCheck,dt);	
				}
			} catch (Exception e) {
				System.out.println(e.getMessage());	
			}
		}
	}

	public void relocateInnerObjs(NanoObj toRelocate, AbstractDomain domain) {

		if(toRelocate.getDomain()==domain){		
			CubicDomain domainToRelocate=(CubicDomain)domain;
			float modVelocity1=toRelocate.getVelocityModule();
			float modVelocity2=0;					

			if (modVelocity1==0 && modVelocity2==0){
				// FIXME Even if both velocities are null, it should handle this collision in some way!
			} else{

				SimpleVector dd=domainToRelocate.getDirectionP();		
				SimpleVector nanoVector=new SimpleVector(getRelativeCenter(domainToRelocate, toRelocate));
				float angle0=nanoVector.calcAngle(dd);
				float nanoModule=nanoVector.length();
				float dNOrto0=(float) (Math.abs((nanoModule*Math.cos(angle0))));
				float distanceP=dNOrto0-toRelocate.getRadius();

				SimpleVector dOrto=domainToRelocate.getDirectionN();				
				float angle=nanoVector.calcAngle(dOrto);

				float dNOrto=(float) (Math.abs((nanoModule*Math.cos(angle))));
				float distanceN= dNOrto-toRelocate.getRadius();	

				SimpleVector dOrto2=domainToRelocate.getDirectionO();		
				float angle2=nanoVector.calcAngle(dOrto2);
				float dNOrto2=(float) (Math.abs((nanoModule*Math.cos(angle2))));

				float dN2=Math.abs(dNOrto2)-toRelocate.getRadius();
				

				if ((distanceP>=domainToRelocate.getHalfSide()) || (distanceN>=domainToRelocate.getHalfSide()) || (dN2>=domainToRelocate.getHalfSide())){	

					double dsP=(Math.abs(distanceP))-(domainToRelocate.getHalfSide());
					double dsN=(Math.abs(distanceN))-(domainToRelocate.getHalfSide());
					double dsN2=(Math.abs(dN2))-(domainToRelocate.getHalfSide());

					if (dsP<=dsN && dsN2<=dsN){						
						SimpleVector[] P_N_N2_dt=relocateFromThisSide(dsN, toRelocate, domainToRelocate, domainToRelocate.getDirectionN());
						SimpleVector d1=P_N_N2_dt[0];
						SimpleVector n1=P_N_N2_dt[1];
						SimpleVector o1=P_N_N2_dt[2];	
						SimpleVector dt1FakeVector=P_N_N2_dt[3];

						//InelasticCollision
						n1.scalarMul(inelasticCoefficient);						
						// END InelasticCollision

						d1.add(o1);
						SimpleVector v2=d1.calcSub(n1);	

						
						//float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
						//float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];						
						float[] angles=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z);		
						float fi1=angles[0];
						float theta1=angles[1];
						float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
						toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);	
						
						float[] center1=toRelocate.getCenter();
						float[] velocity1=toRelocate.getVelocityComponent();
						float dt1=dt1FakeVector.length();

						center1[0]+=velocity1[0]*dt1;
						center1[1]+=velocity1[1]*dt1;
						center1[2]+=velocity1[2]*dt1;
						toRelocate.setCenter(center1[0], center1[1], center1[2]);	
						
						
					}else if (dsP<=dsN2 && dsN<=dsN2){					
						SimpleVector[] P_N_N2_dt=relocateFromThisSide(dsN2, toRelocate, domainToRelocate, domainToRelocate.getDirectionO());
						SimpleVector d1=P_N_N2_dt[0];
						SimpleVector n1=P_N_N2_dt[1];
						SimpleVector o1=P_N_N2_dt[2];		
						SimpleVector dt1FakeVector=P_N_N2_dt[3];

						//InelasticCollision
						o1.scalarMul(inelasticCoefficient);						
						// END InelasticCollision

						n1.add(d1);
						SimpleVector v2=n1.calcSub(o1);		

						//float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
						//float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];						
						float[] angles=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z);		
						float fi1=angles[0];
						float theta1=angles[1];
						float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
						toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);
						
						float[] center1=toRelocate.getCenter();
						float[] velocity1=toRelocate.getVelocityComponent();
						float dt1=dt1FakeVector.length();

						center1[0]+=velocity1[0]*dt1;
						center1[1]+=velocity1[1]*dt1;
						center1[2]+=velocity1[2]*dt1;
						toRelocate.setCenter(center1[0], center1[1], center1[2]);	
						
						
					}else if (dsP>=dsN2 && dsN<=dsP){
						
						SimpleVector[] P_N_N2_dt=relocateFromThisSide(dsP, toRelocate, domainToRelocate, domainToRelocate.getDirectionP());
						SimpleVector d1=P_N_N2_dt[0];
						SimpleVector n1=P_N_N2_dt[1];
						SimpleVector o1=P_N_N2_dt[2];	
						SimpleVector dt1FakeVector=P_N_N2_dt[3];

						//InelasticCollision
						d1.scalarMul(inelasticCoefficient);						
						// END InelasticCollision

						n1.add(o1);
						SimpleVector v2=n1.calcSub(d1);	

						//float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
						//float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];
						float[] angles=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z);		
						float fi1=angles[0];
						float theta1=angles[1];
						float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
						toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);	
						
						float[] center1=toRelocate.getCenter();
						float[] velocity1=toRelocate.getVelocityComponent();
						float dt1=dt1FakeVector.length();

						center1[0]+=velocity1[0]*dt1;
						center1[1]+=velocity1[1]*dt1;
						center1[2]+=velocity1[2]*dt1;
						toRelocate.setCenter(center1[0], center1[1], center1[2]);	
					}
				}
			}	
		}
	}

	@Override
	public void relocateOuterObjs(NanoObj toRelocate, AbstractDomain domainToRelocate) {
		if(toRelocate.getDomain()!=domain){			
			CubicDomain domain=(CubicDomain)domainToRelocate;			
			float modVelocity1=toRelocate.getVelocityModule();
			float modVelocity2=0;
		
			if (modVelocity1==0 && modVelocity2==0){
				// FIXME Even if both velocities are null, it should handle this collision in some way!
			} else{			
				
				SimpleVector dd=domain.getDirectionP();		
				SimpleVector nanoVector=new SimpleVector(getRelativeCenter(domainToRelocate, toRelocate));
				float angle0=nanoVector.calcAngle(dd);
				float nanoModule=nanoVector.length();
				float dNOrto0=(float) (Math.abs((nanoModule*Math.cos(angle0))));

				float distanceP=dNOrto0-toRelocate.getRadius();
				SimpleVector dOrto=domain.getDirectionN();					
				float angle=nanoVector.calcAngle(dOrto);				
				float dNOrto=(float) (Math.abs((nanoModule*Math.cos(angle))));
				float distanceN= dNOrto-toRelocate.getRadius();	

				SimpleVector dOrto2=domain.getDirectionO();		
				float angle2=nanoVector.calcAngle(dOrto2);
				float dNOrto2=(float) (Math.abs((nanoModule*Math.cos(angle2))));
				float dN2=Math.abs(dNOrto2)-toRelocate.getRadius();			

				if ((distanceP<=domain.getHalfSide()) && (distanceN<=domain.getHalfSide()) && (dN2<=domain.getHalfSide())){						

					double dsP=Math.abs((Math.abs(distanceP))-(domain.getHalfSide()));
					double dsN=Math.abs((Math.abs(distanceN))-(domain.getHalfSide()));
					double dsN2=Math.abs((Math.abs(dN2))-(domain.getHalfSide()));

					if (dsP>=dsN && dsN2>=dsN){		
						SimpleVector[] P_N_N2_dt=relocateFromThisSide(dsN, toRelocate, domain, domain.getDirectionN());
						SimpleVector d1=P_N_N2_dt[0];
						SimpleVector n1=P_N_N2_dt[1];
						SimpleVector o1=P_N_N2_dt[2];	
						SimpleVector dt1FakeVector=P_N_N2_dt[3];

						//InelasticCollision
						n1.scalarMul(inelasticCoefficient);						
						// END InelasticCollision

						d1.add(o1);
						SimpleVector v2=d1.calcSub(n1);	

						//float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
						//float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];						
						float[] angles=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z);		
						float fi1=angles[0];
						float theta1=angles[1];
						
						float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
						toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);	
						
						float[] center1=toRelocate.getCenter();
						float[] velocity1=toRelocate.getVelocityComponent();
						float dt1=dt1FakeVector.length();

						center1[0]+=velocity1[0]*dt1;
						center1[1]+=velocity1[1]*dt1;
						center1[2]+=velocity1[2]*dt1;
						toRelocate.setCenter(center1[0], center1[1], center1[2]);	
						

					}else if (dsP>=dsN2 && dsN>=dsN2){
					
						SimpleVector[] P_N_N2_dt=relocateFromThisSide(dsN2, toRelocate, domain, domain.getDirectionO());
						SimpleVector d1=P_N_N2_dt[0];
						SimpleVector n1=P_N_N2_dt[1];
						SimpleVector o1=P_N_N2_dt[2];
						SimpleVector dt1FakeVector=P_N_N2_dt[3];
						
						//InelasticCollision
						o1.scalarMul(inelasticCoefficient);						
						// END InelasticCollision

						n1.add(d1);
						SimpleVector v2=n1.calcSub(o1);		

						//float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
						//float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];
						
						float[] angles=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z);		
						float fi1=angles[0];
						float theta1=angles[1];
						
						float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
						toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);	
						
						float[] center1=toRelocate.getCenter();
						float[] velocity1=toRelocate.getVelocityComponent();
						float dt1=dt1FakeVector.length();

						center1[0]+=velocity1[0]*dt1;
						center1[1]+=velocity1[1]*dt1;
						center1[2]+=velocity1[2]*dt1;
						toRelocate.setCenter(center1[0], center1[1], center1[2]);	
							

					} else if (dsP<=dsN2 && dsN>=dsP){

						SimpleVector[] P_N_N2_dt=relocateFromThisSide(dsP, toRelocate, domain, domain.getDirectionP());
						SimpleVector d1=P_N_N2_dt[0];
						SimpleVector n1=P_N_N2_dt[1];
						SimpleVector o1=P_N_N2_dt[2];
						SimpleVector dt1FakeVector=P_N_N2_dt[3];

						//InelasticCollision
						d1.scalarMul(inelasticCoefficient);						
						// END InelasticCollision

						n1.add(o1);
						SimpleVector v2=n1.calcSub(d1);	

						//float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
						//float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];
						
						float[] angles=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z);		
						float fi1=angles[0];
						float theta1=angles[1];
						
						float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
						toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);	

						float[] center1=toRelocate.getCenter();
						float[] velocity1=toRelocate.getVelocityComponent();
						float dt1=dt1FakeVector.length();

						center1[0]+=velocity1[0]*dt1;
						center1[1]+=velocity1[1]*dt1;
						center1[2]+=velocity1[2]*dt1;
						toRelocate.setCenter(center1[0], center1[1], center1[2]);		
					}				
				} 				
			}				
		}		
	}

	@Override
	public void checkExternalCollisionBound(AbstractDomain domainToCheck, NanoObj objToCheck) {

		if(isCollidingFromOutside(domainToCheck, objToCheck)){
			CubicDomain domain=(CubicDomain)domainToCheck;

			SimpleVector dParallel=domain.getDirectionP();		
			SimpleVector nanoVector=new SimpleVector(getRelativeCenter(domainToCheck, objToCheck));
			float angle0=nanoVector.calcAngle(dParallel);
			float nanoModule=nanoVector.length();
			float dPOrto=(float) (nanoModule*Math.cos(angle0));

			SimpleVector dOrto=domain.getDirectionN();			
			float angle=nanoVector.calcAngle(dOrto);		
			float dNOrto=(float) (nanoModule*Math.cos(angle));

			SimpleVector dOrto2=domain.getDirectionO();		
			float angle2=nanoVector.calcAngle(dOrto2);
			float dNOrto2=(float) (nanoModule*Math.cos(angle2));

			float dP=Math.abs(dPOrto)-objToCheck.getRadius();
			float dN=Math.abs(dNOrto)-objToCheck.getRadius();
			float dN2=Math.abs(dNOrto2)-objToCheck.getRadius();
			float distanceP=dP;
			float distanceN=dN;

			double dsP=Math.abs((Math.abs(distanceP))-(domain.getHalfSide()));
			double dsN=Math.abs((Math.abs(distanceN))-(domain.getHalfSide()));
			double dsN2=Math.abs((Math.abs(dN2))-(domain.getHalfSide()));
			
			float[] velocity1=objToCheck.getVelocityComponent();			
			float dt=1;
			try {			
				// Computes the collision along the parallel component 
				if (dsP>=dsN && dsN2>=dsN){	
					SimpleVector velocityVector=new SimpleVector(velocity1);
					float angleV=velocityVector.calcAngle(domain.getDirectionN());
					float dsPV=(float) ((dsN)*Math.cos(angleV));

					dt =(float) Math.abs((dsPV))/Math.abs(objToCheck.getVelocityModule()); // Warning! catch the div-by-zero!!!
					dt=dt+0;

				}else if (dsP>=dsN2 && dsN>=dsN2){
					SimpleVector velocityVector=new SimpleVector(velocity1);
					float angleV=velocityVector.calcAngle(domain.getDirectionO());
					float dsNV=(float) ((dsN2)*Math.cos(angleV));
					dt =(float) Math.abs((dsNV))/Math.abs(objToCheck.getVelocityModule()); 

				} else if (dsP<=dsN2 && dsN>=dsP){
					SimpleVector velocityVector=new SimpleVector(velocity1);
					float angleV=velocityVector.calcAngle(domain.getDirectionP());
					float dsNV=(float) ((dsP)*Math.cos(angleV));
					dt =(float) Math.abs((dsNV))/Math.abs(objToCheck.getVelocityModule()); 
				}
			} catch (Exception e) {
				System.out.println(e.getMessage());	
			}
			domain.getParentDomain().addToCollisionsListExternal(domain,objToCheck,dt);			
		} 
	}

	public double getInternalCollisionTime(NanoObj collisionObj, AbstractDomain d){
		return 1;
	}

	public double getExternalCollisionTime(NanoObj collisionObj, AbstractDomain d){
		CubicDomain domain=(CubicDomain) d;
		float[] velocity1=collisionObj.getVelocityComponent();
		SimpleVector velocityVector=new SimpleVector(velocity1);
		
		SimpleVector dd=domain.getDirectionP();		
		SimpleVector nanoVector=new SimpleVector(getRelativeCenter(domain, collisionObj));
		float angle0=nanoVector.calcAngle(dd);
		float nanoModule=nanoVector.length();
		float dNOrto0=(float) (Math.abs((nanoModule*Math.cos(angle0))));
		float distanceP=dNOrto0-collisionObj.getRadius();

		SimpleVector dOrto=domain.getDirectionN();		
		float angle=nanoVector.calcAngle(dOrto);
		float dNOrto=(float) (Math.abs((nanoModule*Math.cos(angle))));
		float distanceN= dNOrto-collisionObj.getRadius();	

		SimpleVector dOrto2=domain.getDirectionO();		
		float angle2=nanoVector.calcAngle(dOrto2);
		float dNOrto2=(float) (Math.abs((nanoModule*Math.cos(angle2))));
		float dN2=Math.abs(dNOrto2)-collisionObj.getRadius();

		double dsP=Math.abs((Math.abs(distanceP))-(domain.getHalfSide()));
		double dsN=Math.abs((Math.abs(distanceN))-(domain.getHalfSide()));
		double dsN2=Math.abs((Math.abs(dN2))-(domain.getHalfSide()));	

		SimpleVector axes=null;
		float ds=0;

		if (dsP>=dsN && dsN2>=dsN){
			ds=(float) dsN;
			axes=domain.getDirectionN();

		} else if (dsP>=dsN2 && dsN>=dsN2){
			ds=(float) dsN2;
			axes=domain.getDirectionO();

		} else if (dsP<=dsN2 && dsN>=dsP){
			ds=(float) dsP;
			axes=domain.getDirectionP();
		}

		float angleV=velocityVector.calcAngle(axes);
		float dsV=(float) ((ds)*Math.cos(angleV));
		float dt =(float) Math.abs((dsV))/Math.abs(collisionObj.getVelocityModule()); // Warning! catch the div-by-zero!!!

		if (dt>1.1) {
			dt=1;					
		}
		return dt;
	}

	@Override
	public boolean isAnInternalCollisionBound(NanoObj n) {
		return false;	
	}
	
	@Override
	public boolean isCollidingFromOutside(AbstractDomain domainToCheck, NanoObj objToCheck) {

		CubicDomain domain=(CubicDomain)domainToCheck;
		float[] distancesOnAxis=getDistanceOnAxis(domainToCheck,objToCheck);
		boolean result=false;

		float dP=distancesOnAxis[0]-objToCheck.getRadius();
		float dN=distancesOnAxis[1]-objToCheck.getRadius();
		float dN2=distancesOnAxis[2]-objToCheck.getRadius();
		
		if(dP<=domain.getHalfSide() && dN<=domain.getHalfSide() && dN2<=domain.getHalfSide()){
			float x0=domain.getHalfSide()-objToCheck.getRadius();
			float y0=domain.getHalfSide()-objToCheck.getRadius();

			float alfa1=(float) Math.atan((domain.getHalfSide())/y0);
			float alfa2=(float) ((Math.PI/2) - Math.atan((domain.getHalfSide())/x0));

			SimpleVector nanoObjRelativeCenter=getRelativeCenter(domain, objToCheck);
			SimpleVector d=domain.getDirectionP();
			double angleP=getAngle(d,nanoObjRelativeCenter);		

			if(angleP>alfa2 && angleP<alfa1){
				float circle=(dP-x0)*(dP-x0)+(dN-y0)*(dN-y0);
				if(circle<objToCheck.getRadius()*objToCheck.getRadius()){
					result=true;
				}	else{				
					result=false;
				}
			} else {
				result=true;						
			}
		}
		return result;
	}

	private SimpleVector[] relocateFromThisSide(double ds, NanoObj toRelocate, CubicDomain domain, SimpleVector axes){

		float[] velocity1=toRelocate.getVelocityComponent();
		SimpleVector velocityVector=new SimpleVector(velocity1);

		float[] nanoCenter=toRelocate.getCenter();	
		float angleV=velocityVector.calcAngle(axes);
		float dsV=(float) ((ds)*Math.cos(angleV));
		float dt =(float) Math.abs((dsV))/Math.abs(toRelocate.getVelocityModule()); // Warning! catch the div-by-zero!!!

		if (dt>1.1) {
			dt=1;					
		}
		
		float spostamento0 = velocity1[0]*dt;
		float spostamento1 = velocity1[1]*dt;
		float spostamento2 = velocity1[2]*dt;

		nanoCenter[0]+=-spostamento0;
		nanoCenter[1]+=-spostamento1;
		nanoCenter[2]+=-spostamento2;
		toRelocate.setCenter(nanoCenter[0], nanoCenter[1], nanoCenter[2]);

		SimpleVector v1=new SimpleVector(toRelocate.getVelocityComponent());
		SimpleVector d=domain.getDirectionP();
		double dModule=Math.sqrt(d.x*d.x+d.y*d.y+d.z*d.z);
		float[] dVersorComponent=new float[3];
		dVersorComponent[0]=(float) (d.x/dModule);
		dVersorComponent[1]=(float) (d.y/dModule);
		dVersorComponent[2]=(float) (d.z/dModule);

		SimpleVector dVersor=new SimpleVector(dVersorComponent);
		SimpleVector oTempVersor=domain.getDirectionO();
		SimpleVector nTempVersor=domain.getDirectionN();

		float alfa=nTempVersor.calcAngle(v1);		
		double n1Module=toRelocate.getVelocityModule()*Math.cos(alfa);
		float[] n1components=new float[3];
		n1components[0]=(float) (n1Module*nTempVersor.x);
		n1components[1]=(float) (n1Module*nTempVersor.y);
		n1components[2]=(float) (n1Module*nTempVersor.z);				
		SimpleVector n1=new SimpleVector(n1components);

		float gamma=dVersor.calcAngle(v1);				
		double d1Module=toRelocate.getVelocityModule()*Math.cos(gamma);
		float[] d1components=new float[3];
		d1components[0]=(float) (d1Module*dVersor.x);
		d1components[1]=(float) (d1Module*dVersor.y);
		d1components[2]=(float) (d1Module*dVersor.z);				
		SimpleVector d1=new SimpleVector(d1components);

		float ro=oTempVersor.calcAngle(v1);				
		double o1Module=toRelocate.getVelocityModule()*Math.cos(ro);
		float[] o1components=new float[3];
		o1components[0]=(float) (o1Module*oTempVersor.x);
		o1components[1]=(float) (o1Module*oTempVersor.y);
		o1components[2]=(float) (o1Module*oTempVersor.z);				
		SimpleVector o1=new SimpleVector(o1components);
		
		SimpleVector dtFakeVector=new SimpleVector(1,0,0);
		dtFakeVector.scalarMul(dt);

		SimpleVector[] results_P_N_N2_dt = {d1,n1,o1,dtFakeVector};
		return results_P_N_N2_dt;	
	}
}