package simulator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import MainLibrary.*;

public class CylindricBloodVesselDomain extends CylindricDomain{
	
	public boolean hasToCreateTXNode=false;
	public Random rand=new Random();
	
	/* For the EmissionProcess In Vessel*/
	public int totalCells=0;	
	public int threshold=0;
	public int thresholdEmission=0;
	/* END EmissionProcess In Vessel*/
	
	private boolean hasBackuppedDomain=false;
	
	// PER DEBUG
	public long destroyedCell=0;
	public HashMap deathCount=new HashMap<String, String>();		
	
	public CylindricBloodVesselDomain(AbstractDomain parentDomain, MobilityModel model, AbstractCollisionDomainStrategy strategy, boolean visible, boolean export, float h, float r,float[] dCenter,  boolean useOctree) {
		super(parentDomain, model, strategy, visible, export, h, r, dCenter, useOctree);
		
		setLivingObjObserver(new LivingBloodObjObserver(this));
		
		/* For the EmissionProcess in Vessel*/
		totalCells=((BloodFlowModel)getMobilityModel()).totalDesideredCells;	
		threshold=(int) (35000000/getManager().configWrapper.timeUnit);
		thresholdEmission=(int) ((threshold/totalCells)/2);
		
		/* END the EmissionProcess in Vessel*/		
		
		/* NEW THRESHOLD FOR LONG VESSEL */
		//float timeU=(float) (getManager().configWrapper.timeUnit*1E-9);
		//thresholdEmission=(int) (((88500*1E-9)/(0.5*1E-3))/(timeU));
		/* END NEW EMISSION */
		
		/* NEW THRESHOLD FOR LONG VESSEL 2 */
		float timeU=(float) (getManager().configWrapper.timeUnit*1E-9);
		//thresholdEmission=(int) (((88500*1E-9)/(0.5*1E-3))/(timeU)/totalCells);		// OK for radius 30 um
		thresholdEmission=(int) (((88500*1E-9)/(2*1E-3))/(timeU)/totalCells);			// OK for radius 60 um
		/* END NEW EMISSION */
		
		
		if(thresholdEmission<=0) {
			thresholdEmission=1;
		}
		
		
		if (isVisible()) getManager().engine.addWrapper3DDomain(this);
		
		
		
		/** ===================================================================== */
		/** FIXME VERY IMPORTANT FOR BLOOD VESSELS (Improves the performance!)    */
		/** ===================================================================== */
		// Decides what kind of NanoObjs wants to use for the OCTREE.checkCollision phase!
		ObjFor1Check=0;
		ObjFor2Check=2;
		
	}				
	
	public void initializeWrapper3D(Manager m){
		if(m.is3DEnable) m.engine.addWrapper3DDomain(this);	
	}
	
	public void handleBoundCollision(NanoObj n){		
		if(n.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(n,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(n,this);
		}
	}
	
	public void handleBoundCollision(NodeObj n){		
		if(n.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(n,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(n,this);
		}
	}
	
	public void handleBoundCollision(CarrierObj c){		
		if(c.getDomain()==this){					
			getCollisionStrategy().relocateInnerObjs(c,this);				
		}else{
			getCollisionStrategy().relocateOuterObjs(c,this);
		}
	}
	
	public String getType(){
		return "Blood Vessel Domain";	
	}
	
	// FIXME Uses OCtree to optimize the checking AND do not controls the overlapping with the Cube Receptor Domains (its child domains)
	@Override
	public boolean isFreeSpace(ArrayList<NanoObj> objList, NanoObj obj){
		float[] coords=obj.getCenter();
		float radius=obj.getRadius();
		boolean isFree=true;
		float[] tempCoords=new float[3];
		double tempRadius=0;
		boolean endSearch=false;
		
		/*
		List<AbstractDomain> childDomainList=getAllChildDomains();		
		for(int i=0;i<childDomainList.size() && isFree;i++){
			if(childDomainList.get(i).isInside(obj)) isFree=false;
		}		
		*/	

		if(isFree){
			List<NanoObj> allNanoObjList=objList;			

			for (int i=0;i<allNanoObjList.size()& (!endSearch);i++){
				tempCoords=allNanoObjList.get(i).getCenter();
				tempRadius=allNanoObjList.get(i).getRadius();
				//check=i;
				float distance=getDistance(coords,tempCoords);

				if (distance<(radius+tempRadius)){
					if(obj.mandatoryCreation()){
						allNanoObjList.get(i).kill();
					}else{
						isFree=false;
						endSearch=true;
					}
				}
			}	
		}	
		return isFree;
	}	


	public void resetDeadCounter(){			
		//deathCount.put(getManager().getCurrentTime(), destroyedCell);
		destroyedCell=0;
	}
	
	@Override
	public void trigger() {	
		/*
		// Sets Backup!
		try {
			//FIXME DEBUG, TRUE only when RESTORE!!!
			
			hasBackuppedDomain=true;
			
			if(!hasBackuppedDomain){
				MySQLBackup_BloodVessel.getInstance().collectDataFrom(this);
				hasBackuppedDomain=true;
				
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block			
			e.printStackTrace();
			System.out.println("- 1 - CylindricBloodVesselDOMAIN - trigger ");
		}*/
		
		
		/* OK for VESSEL INITIALIZE*/
		try {
			if(getManager().getCurrentTime()==1){
				//((BloodFlowModel)getMobilityModel()).emissionProcessInVesselDeterministicINITIALIZE(this);		
				
				//========================  SIMULAZIONI OK ===================================
				((BloodFlowModel)getMobilityModel()).emissionProcessInVesselDeterministicINITIALIZE_LongVessel(this);		// OK
				((LivingBloodObjObserver)getLivingObjObserver()).refreshBloodCellsCounter();								// OK
				//========================  END SIMULAZIONI OK ===================================
				
				
				
				//========================  DEBUG ===================================
				//((BloodFlowModel)getMobilityModel()).emissionProcessInVesselDeterministicINITIALIZE_LongVessel_DEBUG_FLUSSO(this);	// solo DEBUG, usare quello sopra x simulazioni
				//((BloodFlowModel)getMobilityModel()).emissionProcess_DEBUG_RECETTORI(this);	// solo DEBUG, usare quello sopra x simulazioni
				//========================  END DEBUG ===================================
				
				
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("- 2 - CylindricBloodVesselDOMAIN - trigger ");
		}		
		
		/* OK for VESSEL */		
		try {
			if(getManager().getCurrentTime()%thresholdEmission==0){	
				
				//((BloodFlowModel)getMobilityModel()).emissionProcessInVesselDeterministic(this);			
				
				
				//========================  SIMULAZIONI OK ===================================
				// FIXME COMMENTATO SOLO PER FARE DEI TEST SENZA LA CREAZIONE DURANTE LA SIMULAZIONE
				//((BloodFlowModel)getMobilityModel()).emissionProcessInVesselDeterministic_LongVessel(this);					// OK. Old implementation (some errors on the creation of the total amount of Nanoobj, too many platelets!)				
				((BloodFlowModel)getMobilityModel()).emissionProcessInVesselDeterministic_LongVessel_NEW_Casson(this);			// OK. NEW CASSON	
				//========================  END SIMULAZIONI OK ===================================
				
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("- 3 - CylindricBloodVesselDOMAIN - trigger ");
		}		
	}	
	
	@Override
	public Map<Object, Object> export() {
		exportString=new HashMap<Object, Object>();		
		exportString.put("Typev","DeathCounter");	
		exportString.put("DeathTime",getManager().getCurrentTime());	
		exportString.put("DeathValue",destroyedCell);	
		
		/*
		int livingObjCount=nodeList.size()+carrierList.size()+genericObjList.size();		
		
		int mac=0;
		int red=0;
		int plat=0;
		
		int sss=nodeList.size();
		for(int i=0;i<sss;i++){
			if(nodeList.get(i).assignName().equals("Red Blood Cell")){
				red++;
			}else if(nodeList.get(i).assignName().equals("Macrophage")){
				mac++;
			}else if(nodeList.get(i).assignName().equals("Platelet")){
				plat++;
			}
		}		
		
		System.out.println("RED: "+red+"  -  WHITE: "+mac+"  -  PLAT: "+plat);
		
		exportString.put("Typew","LivingCounter");	
		exportString.put("LiveTime",getManager().getCurrentTime());	
		exportString.put("LiveValue",livingObjCount);	
		exportString.put("RED-LIVE",red);	
		exportString.put("WHITE-LIVE",mac);	
		exportString.put("PLAT-LIVE",plat);			
		exportString.put("nano-collision",((BloodFlowModel)getMobilityModel()).nanoCollision);			
		exportString.put("domain-collision",((CylindricStrategyBloodVessel)getCollisionStrategy()).domainCollision);	
		//resetDeadCounter();
		*/
		return exportString;
	}	
}