package simulator;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import MainLibrary.*;
import com.threed.jpct.SimpleVector;


public class CylindricDomain extends AbstractDomain{
	
	protected float height=0;
	protected float r=0;
	public Random rand=new Random();
		
	public CylindricDomain(AbstractDomain parentDomain, MobilityModel model, AbstractCollisionDomainStrategy strategy, boolean visible, boolean export, float h, float r,float[] dCenter,  boolean useOctree) {
		super(parentDomain, model, strategy, visible, export, dCenter, useOctree);		
		setHeight(h);
		setRadius(r);
		setBoundingRadius();		
	
		if(useOctree) initializeOctree(getManager().configurator);
		
		if (isVisible()) getManager().engine.addWrapper3DDomain(this);				
	}				
	
	public void initializeWrapper3D(Manager m){
		if(m.is3DEnable) m.engine.addWrapper3DDomain(this);	
	}	

	public void handleBoundCollision(NanoObj n){	
		if(n.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(n,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(n,this);
		}
	}
	
	public void handleBoundCollision(NodeObj n){
		if(n.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(n,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(n,this);
		}
	}
	
	public void handleBoundCollision(CarrierObj c){		
		if(c.getDomain()==this){					
			getCollisionStrategy().relocateInnerObjs(c,this);				
		}else{
			getCollisionStrategy().relocateOuterObjs(c,this);
		}
	}
	
	public String getType(){
		return "Cylindric Domain";		
	}
	
	public void setBoundingRadius(){
		/* OLD 
		if(getHeight()>getRadius()) boundingRadius=(float)getHeight();
		else boundingRadius=(float)getRadius();
		*/
		
		// FIXME Correct implementation
		boundingRadius=(float) Math.sqrt(Math.pow(getHeight()/2, 2)+Math.pow(getRadius(), 2));
		int d=0;
	}

	public float getHeight() {
		return height;
	}

	public void setHeight(float height) {
		this.height = height;
	}

	public float getRadius() {
		return r;
	}

	public void setRadius(float r) {
		this.r = r;
	}

	@Override
	public void trigger() {
		
	}

	@Override
	public Map<Object, Object> export() {
		exportString=new HashMap<Object, Object>();		
		exportString.put("ID", ID);	
		return exportString;
	}


	@Override
	public boolean isInside(NanoObj objToCheck) {
		// If the Obj's center is inside, then the whole object is considered inside!		
		boolean isInside=false;		
		float[] distances=getCollisionStrategy().getDistanceOnAxis(this,objToCheck);			
		float distanceP= distances[0]-objToCheck.getRadius();		
		//float distanceN= distances[1]-objToCheck.getRadius();	
		float distanceN= (float) ((Math.sqrt(distances[1]*distances[1]+distances[2]*distances[2]))-objToCheck.getRadius());	
		
		if (((Math.abs(distanceP))<=getHeight()/2) && (Math.abs(distanceN)<=getRadius())){	
			isInside=true;
		} 
		return isInside;
	}

	@Override
	public void initializeDirectionAxes() {
		float[] dCenter1=new float[3];
		dCenter1[0]=0;
		dCenter1[1]=1;
		dCenter1[2]=0;		
		setDirectionP(new SimpleVector(dCenter1));	
		
		float[] dCenter2=new float[3];
		dCenter2[0]=1;
		dCenter2[1]=0;
		dCenter2[2]=0;
		setDirectionN(new SimpleVector(dCenter2));		
	}	
	
	public double getAngleN(SimpleVector nanoObjRelativeCenter, float dN, float dO){
		SimpleVector vectorN=new SimpleVector(getDirectionN());
		vectorN.scalarMul(dN);

		SimpleVector vectorO=new SimpleVector(getDirectionO());
		vectorO.scalarMul(dO);

		vectorN.add(vectorO);		
		
		double angleN=getAngle(vectorN,nanoObjRelativeCenter);
		
		return angleN;
	}	
}
