package simulator;

import java.util.Random;
import MainLibrary.AbstractCollisionDomainStrategy;
import MainLibrary.AbstractDomain;
import MainLibrary.MobilityModel;
import MainLibrary.NoMotion;

public class CylindricJarDomain extends CylindricDomain{	
	
	private double volume=0;
	private long timeUnit=0;
	private long totalCarriers=0;
	private int releaseInterval=0;
	private int counter=0;
	private float releaseLevelDisplacement=0;
	boolean create=true;	
	

	public CylindricJarDomain(AbstractDomain parentDomain, MobilityModel model,
			AbstractCollisionDomainStrategy strategy, boolean visible,
			boolean export, float h, float r, float[] dCenter, float releaseLevelDisplacement, float carriersConcentration, float minutes,  boolean useOctree) {
		super(parentDomain, model, strategy, visible, export, h, r, dCenter, useOctree);
		
		this.volume=r*r*Math.PI*(h-releaseLevelDisplacement);
		this.timeUnit=model.getManager().configWrapper.timeUnit;
		this.totalCarriers=(long) ((carriersConcentration*volume)*Math.pow(10, -21));
		float creationTime=(float) (minutes*60/(timeUnit*Math.pow(10, -9)));
		this.releaseInterval=(int) (creationTime/totalCarriers);
		this.releaseLevelDisplacement=releaseLevelDisplacement;
		
		if(releaseInterval<1) {
			System.out.println();
			System.out.println("===================   WARNING   ===================");
			System.out.println(" The Release Interval is < 1 time step! ");
			System.out.println(" Consider to increase the concentration of carriers ");
			System.out.println(" or to reduce the Time Step value");
			System.out.println("====================================================");
			System.out.println();
			System.out.println(" >>> Simulation shutdown...");
			System.exit(0);		
		}else{
			System.out.println();
			System.out.println("==========   CARRIERS GENERATOR CYLINDER   =========");
			System.out.println(" Concentration: "+carriersConcentration+" [carrier/cm^3]");
			System.out.println(" Minutes: "+minutes);
			System.out.println(" Volume: "+(volume*Math.pow(10, -21))+" [cm^3]");
			System.out.println(" Time Unit: "+timeUnit+" [ns]");
			System.out.println(" Total Carriers: "+totalCarriers);
			System.out.println(" Total Time Steps: "+creationTime);
			System.out.println(" Release Interval: "+releaseInterval+" [time step]");
			System.out.println(" Release Level Displacement: "+releaseLevelDisplacement+" [nm]");
			System.out.println("====================================================");
			System.out.println();
		}
	}
	
	@Override
	public void trigger() {
		boolean debug=false;
		

		if (debug){
			
			releaseCarrierForDebug(create);
			create=false;
		}else{

			if(counter>1){
				counter--;			
			}else{
				counter=releaseInterval;

				// === CARRIERS GENERATION === 
				float x,y,z;
				Random r=new Random();
				int depth=200;		

				float rad=getRadius();
				x= (float) (center[0]+((r.nextDouble()*2*rad)-rad));
				y= (float) ((center[1]-height/2+releaseLevelDisplacement+2*depth)+((r.nextDouble()*2*depth)-(depth)));
				z= (float) (center[2]+((r.nextDouble()*2*rad)-rad));

				double distance=Math.sqrt((x-center[0])*(x-center[0])+(z-center[2])*(z-center[2]));
				if(distance>=(rad-500) || (Math.sqrt((y-center[1])*(y-center[1]))>=(height/2-500))){

				}else{

					CD40LCarrier cc=new CD40LCarrier(this,x,y,z,1);	

					cc.setVelocityComponent(cc.getVelocityModule(), r.nextFloat(), r.nextFloat());			
					cc.setCenter(x, y+(2*cc.getRadius()), z);		

					cc.setMotionStrategy(BrownianMotion.getInstance());	
				}				
			}	
		}		
	}
	
	// FIXME Release carrier only for debug!!
	public void releaseCarrierForDebug(boolean create){

		if (create){

			//for (int i=0;i<5;i++){

				// === CARRIERS GENERATION === 
				float x,y,z;
				Random r=new Random();
				int depth=200;		

				float rad=getRadius();
				x= (float) (150000);
				y= (float) (-529000);
				z= (float) (150000);

				double distance=Math.sqrt((x-center[0])*(x-center[0])+(z-center[2])*(z-center[2]));
				if(distance>=(rad-500) || (Math.sqrt((y-center[1])*(y-center[1]))>=(height/2-500))){

				}else{

					CD40LCarrier cc=new CD40LCarrier(this,x,y,z,1);	

					cc.setVelocityComponent(cc.getVelocityModule(), r.nextFloat(), r.nextFloat());			
					cc.setCenter(x, y+(2*cc.getRadius()), z);		

					cc.setMotionStrategy(NoMotion.getInstance());	
				}		



				rad=getRadius();
				x= (float) (70000);
				y= (float) (-529000);
				z= (float) (150000);

				distance=Math.sqrt((x-center[0])*(x-center[0])+(z-center[2])*(z-center[2]));
				if(distance>=(rad-500) || (Math.sqrt((y-center[1])*(y-center[1]))>=(height/2-500))){

				}else{

					CD40LCarrier cc=new CD40LCarrier(this,x,y,z,1);	

					cc.setVelocityComponent(cc.getVelocityModule(), r.nextFloat(), r.nextFloat());			
					cc.setCenter(x, y+(2*cc.getRadius()), z);		

					cc.setMotionStrategy(NoMotion.getInstance());	
				}		


				rad=getRadius();
				x= (float) (150000);
				y= (float) (-529000);
				z= (float) (70000);

				distance=Math.sqrt((x-center[0])*(x-center[0])+(z-center[2])*(z-center[2]));
				if(distance>=(rad-500) || (Math.sqrt((y-center[1])*(y-center[1]))>=(height/2-500))){

				}else{

					CD40LCarrier cc=new CD40LCarrier(this,x,y,z,1);	

					cc.setVelocityComponent(cc.getVelocityModule(), r.nextFloat(), r.nextFloat());			
					cc.setCenter(x, y+(2*cc.getRadius()), z);		

					cc.setMotionStrategy(NoMotion.getInstance());	
				}		


				rad=getRadius();
				x= (float) (70000);
				y= (float) (-529000);
				z= (float) (70000);

				distance=Math.sqrt((x-center[0])*(x-center[0])+(z-center[2])*(z-center[2]));
				if(distance>=(rad-500) || (Math.sqrt((y-center[1])*(y-center[1]))>=(height/2-500))){

				}else{

					CD40LCarrier cc=new CD40LCarrier(this,x,y,z,1);	

					cc.setVelocityComponent(cc.getVelocityModule(), r.nextFloat(), r.nextFloat());			
					cc.setCenter(x, y+(2*cc.getRadius()), z);		

					cc.setMotionStrategy(NoMotion.getInstance());	
				}		


				rad=getRadius();
				x= (float) (130000);
				y= (float) (-529000);
				z= (float) (90000);

				distance=Math.sqrt((x-center[0])*(x-center[0])+(z-center[2])*(z-center[2]));
				if(distance>=(rad-500) || (Math.sqrt((y-center[1])*(y-center[1]))>=(height/2-500))){

				}else{

					CD40LCarrier cc=new CD40LCarrier(this,x,y,z,1);	

					cc.setVelocityComponent(cc.getVelocityModule(), r.nextFloat(), r.nextFloat());			
					cc.setCenter(x, y+(2*cc.getRadius()), z);		

					cc.setMotionStrategy(NoMotion.getInstance());	
				}		


				rad=getRadius();
				x= (float) (50000);
				y= (float) (-529000);
				z= (float) (50000);

				distance=Math.sqrt((x-center[0])*(x-center[0])+(z-center[2])*(z-center[2]));
				if(distance>=(rad-500) || (Math.sqrt((y-center[1])*(y-center[1]))>=(height/2-500))){

				}else{

					CD40LCarrier cc=new CD40LCarrier(this,x,y,z,1);	

					cc.setVelocityComponent(cc.getVelocityModule(), r.nextFloat(), r.nextFloat());			
					cc.setCenter(x, y+(2*cc.getRadius()), z);		

					cc.setMotionStrategy(NoMotion.getInstance());	
				}		

			//}
		}
	}	
}
