package simulator;

import java.util.Random;
import MainLibrary.AbstractCollisionDomainStrategy;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.MobilityModel;

public class CylindricPoreDomain extends CylindricDomain{

	protected AbstractDomain secondDomain;
	protected float partialDt=0;
	protected long t;
	protected float releaseTime=0;
	protected int carrierType;
	protected int createdCarrier=0;
	protected int maxReleasedCarrier=0;
	
	
	public CylindricPoreDomain(AbstractDomain parentDomain,AbstractDomain secondDomain, MobilityModel model, AbstractCollisionDomainStrategy strategy,
			boolean visible, boolean export, float h, float r, float[] dCenter, float releaseTime, int maxReleasedCarrier, int carrierType,  boolean useOctree) {
		super(parentDomain, model, strategy, visible, export, h, r, dCenter, useOctree);
		
		this.t=getManager().configWrapper.timeUnit;
		this.secondDomain=secondDomain;
		this.releaseTime=releaseTime;
		this.partialDt=releaseTime;		
		this.carrierType=carrierType;
		this.maxReleasedCarrier=maxReleasedCarrier;		
	}
	
	public void trigger(){		
		if(createdCarrier!=0){
			try {
				releaseCarriers();
			} catch (Exception e) {				
				System.out.println("-ERROR- CylindricPoreDomain.Trigger()");
				e.printStackTrace();
			}
			
		}else{
			preloadOfCD40L(carrierType,maxReleasedCarrier);
		}		
	}
	
	public void handleBoundCollision(CarrierObj c){	
		if(c.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(c,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(c,this);
		}
	}
	
	public void releaseCarriers(){
		if(partialDt<=0){
			if(!undisposedCarrierList.isEmpty()){					
				if (isFreeSpace(undisposedCarrierList.get(0))){
					undisposedCarrierList.get(0).disposeOnMedium();
				}				
			}
			partialDt=releaseTime;
		}else{
			partialDt=partialDt-t;
		}			
	}
	
	public void preloadOfCD40L(int type, int quantity){	
		float x,y,z;
		Random r=new Random();
		
		for(int i=0;i<quantity;i++){

			x= (float) (getAbsCenter()[0]);
			y= (float) (getAbsCenter()[1]-getHeight()/2);
			z= (float) (getAbsCenter()[2]);	

			CD40LCarrier cc=new CD40LCarrier(this,x,y,z,type);	
			cc.setVelocityComponent(cc.getVelocityModule(), r.nextFloat(), r.nextFloat());			
			cc.setCenter(x, y+(2*cc.getRadius()), z);
			
			attachNanoObj(cc);
			createdCarrier++;			
			
			/** Along the direction of positive y axis */
			cc.setFi((float) (Math.PI/2));		
			cc.setTheta((float) (Math.PI/2));
		}
	}
}
