package simulator;

import java.util.List;
import MainLibrary.AbstractDomain;
import MainLibrary.NanoObj;
import com.threed.jpct.SimpleVector;

/** WARNING! This class calculates the distanceP and distanceN through the getAbsDistancePlusRadius() and does not evaluates 
 * the Direction Axes!!!
 */

public class CylindricPoreStrategy extends CylindricStrategy{	

	private static CylindricPoreStrategy instance=null;

	public static CylindricPoreStrategy getInstance(){
		if (instance==null) instance=new CylindricPoreStrategy(1);
		return instance;
	}

	private CylindricPoreStrategy(int type){
		super(type);
	}

	@Override
	public void checkInternalCollisionBound(List<NanoObj> objsToCheck) {

		for (int i=0;i<objsToCheck.size();i++){			
			try{
				if(objsToCheck.get(i).getTimeToLive()>0){				
					CylindricDomain domain=(CylindricDomain) objsToCheck.get(i).getDomain();

					float[] domainCenter=domain.getAbsCenter();					
					float[] nanoCenter=objsToCheck.get(i).getCenter();

					float x=nanoCenter[0]-domainCenter[0];
					float y=nanoCenter[1]-domainCenter[1];
					float z=nanoCenter[2]-domainCenter[2];

					SimpleVector nanoObjRelativeCenter=new SimpleVector(x,y,z);
					double angleP=getAngle(domain.getDirectionP(),nanoObjRelativeCenter);
					double distanceP= (objsToCheck.get(i).getAbsDistancePlusRadius()*Math.cos(angleP));

					double angleN=(Math.PI/2)-angleP;
					double distanceN= (objsToCheck.get(i).getAbsDistancePlusRadius()*Math.cos(angleN));
					
					if ((Math.abs(distanceP))>=domain.getHeight()/2){	
						if (!(objsToCheck.get(i).getDomain().isToBeExcepted(objsToCheck.get(i)))){										

							// the collision should have occurred at t-dt (Actually this is also an approximation).
							float dt=1;
							try {							
								// FIXME Computes the collision along the parallel direction.
								// Computes the penetration distance (ds) along this direction.
								// Relocate the ds along the direction of collision and divide for the velocity in order to get the time spent for the penetration...

								double ds=(Math.abs(distanceP)-(domain.getHeight()/2));								
								dt =(float) Math.abs((ds/Math.cos(angleP)))/Math.abs(objsToCheck.get(i).getVelocityModule()); // Warning! catch the div-by-zero!!!

							} catch (Exception e) {
								System.out.println(e.getMessage());	
							}						
							domain.addToCollisionsListInternal(objsToCheck.get(i).getDomain(),objsToCheck.get(i),dt);							
						}						

					}else if(Math.abs(distanceN)>=domain.getRadius()){
						if (!(objsToCheck.get(i).getDomain().isToBeExcepted(objsToCheck.get(i)))){											

							// the collision should have occurred at t-dt (Actually this is also an approximation).
							float dt=1;
							try {							
								// FIXME Computes the collision along the parallel direction.
								// Computes the penetration distance (ds) along this direction.
								// Relocate the ds along the direction of collision and divide for the velocity in order to get the time spent for the penetration...

								double ds=(Math.abs(distanceN)-(domain.getRadius()));								
								dt =(float) Math.abs((ds/Math.cos(angleN)))/Math.abs(objsToCheck.get(i).getVelocityModule()); // Warning! catch the div-by-zero!!!

							} catch (Exception e) {
								System.out.println(e.getMessage());	
							}							
							domain.addToCollisionsListInternal(domain,objsToCheck.get(i),dt);								
						}
					}else if (objsToCheck.get(i).getDomain().isToBeExcepted(objsToCheck.get(i))){
						objsToCheck.get(i).getDomain().removeFromExceptionList(objsToCheck.get(i));
					}
				}
			}catch (Exception e){
				e.printStackTrace();
			}
		}			
	}

	public void checkExternalCollisionBound(AbstractDomain domainToCheck, NanoObj objToCheck) {
		try{			
			if(objToCheck.getTimeToLive()>0){
				CylindricDomain domain=(CylindricDomain)domainToCheck;

				float[] domainCenter=domain.getAbsCenter();
				float[] nanoCenter=objToCheck.getCenter();		
				
				float x=nanoCenter[0]-domainCenter[0];
				float y=nanoCenter[1]-domainCenter[1];
				float z=nanoCenter[2]-domainCenter[2];

				SimpleVector nanoObjRelativeCenter=new SimpleVector(x,y,z);
				double nanoObjRelativeModule=Math.sqrt(x*x+y*y+z*z);

				double angleP=getAngle(domain.getDirectionP(),nanoObjRelativeCenter);	
				double distanceP=((nanoObjRelativeModule-objToCheck.getRadius())*Math.cos(angleP)); 

				double angleN=(Math.PI/2)-angleP;
				double distanceN= ((nanoObjRelativeModule-objToCheck.getRadius())*Math.cos(angleN)); 

				if (((Math.abs(distanceP))<=domain.getHeight()/2) && (Math.abs(distanceN)<=domain.getRadius())){
					if (!(objToCheck.getDomain().isToBeExcepted(objToCheck))){

						float[] velocity1=objToCheck.getVelocityComponent();					

						// the collision should have occurred at t-dt (Actually this is also an approximation).
						float dt=1;
						try {		
							double dsP=(Math.abs(distanceP)-(domain.getHeight()/2));
							double dsN=(Math.abs(distanceN)-(domain.getRadius()));
						
							if (dsP>=dsN){
								SimpleVector velocityVector=new SimpleVector(velocity1);
								float angleV=velocityVector.calcAngle(domain.getDirectionP());
								float dsPV=(float) ((dsP)*Math.cos(angleV));
								dt =(float) Math.abs((dsPV))/Math.abs(objToCheck.getVelocityModule()); // Warning! catch the div-by-zero!!!
								
							}else{
								SimpleVector velocityVector=new SimpleVector(velocity1);
								float angleV=velocityVector.calcAngle(domain.getDirectionP());
								float dsNV=(float) ((dsN)*Math.sin(angleV));
								dt =(float) Math.abs((dsNV))/Math.abs(objToCheck.getVelocityModule()); 
							}
						} catch (Exception e) {
							System.out.println(e.getMessage());	
						}
						domain.addToCollisionsListExternal(domain,objToCheck,dt);
						
					}else{						
						// ===================================================================================
						// Checks if the next step of the nanoobj may bring it deeper into the collided domain	
						// ===================================================================================

						float[] nextPosition=checkNextStepCollision(objToCheck);					

						x=(nextPosition[0])-domainCenter[0];
						y=(nextPosition[1])-domainCenter[1];
						z=(nextPosition[2])-domainCenter[2];

						SimpleVector nextNanoObjRelativeCenter=new SimpleVector(x,y,z);
						double nextNanoObjRelativeModule=Math.sqrt(x*x+y*y+z*z);
						double nextAngleP=getAngle(domain.getDirectionP(),nextNanoObjRelativeCenter);
						double nextDistanceP= ((nextNanoObjRelativeModule-objToCheck.getRadius())*Math.cos(nextAngleP)); 
						double nextAngleN=(Math.PI/2)-angleP;	
						double nextDistanceN= ((nextNanoObjRelativeModule-objToCheck.getRadius())*Math.cos(nextAngleN)); 		

						if (nextDistanceP>distanceP && nextDistanceN>distanceN){
							// FIXME The Nano Objects move away. 
							// Restores the nano-object strategy?
						}else{
							// FIXME the objects are approaching to each other.
							// Reverse the direction?
						}
					}
					
				}else if (objToCheck.getDomain().isToBeExcepted(objToCheck)){
					if(objToCheck.getDomain().removeFromExceptionList(objToCheck)){
						objToCheck.setMotionStrategy();
					}
				}
			}
		}catch (Exception e){
			e.printStackTrace();
		}
	}

	public void relocateInnerObjs(NanoObj toRelocate, AbstractDomain domain) {
		if(toRelocate.getDomain()==domain){			

			CylindricDomain owner=(CylindricDomain)domain;

			float[] center1=toRelocate.getCenter();
			float[] center2=owner.getAbsCenter();
			float[] velocity1=toRelocate.getVelocityComponent();
			float modVelocity1=toRelocate.getVelocityModule();
			float modVelocity2=0;
			double dt=0;			

			if (modVelocity1==0 && modVelocity2==0){
				// FIXME Even if the velocity are null, it should handles the collision in some way!
			} else{

				float dx21 = (center2[0]-center1[0]);
				float dy21 = (center2[1]-center1[1]);
				float dz21 = (center2[2]-center1[2]);		

				SimpleVector nanoObjRelativeCenter=new SimpleVector(dx21,dy21,dz21);
				double angleP=getAngle(domain.getDirectionP(),nanoObjRelativeCenter);
				double distanceP= (toRelocate.getAbsDistancePlusRadius()*Math.cos(angleP));
				double angleN=(Math.PI/2)-angleP;
				double distanceN= (toRelocate.getAbsDistancePlusRadius()*Math.cos(angleN));
					
				if ((Math.abs(distanceP))>=owner.getHeight()/2){		
					if(distanceP>0){
						double ds=(Math.abs(distanceN)-(owner.getRadius()));
						dt =(float) Math.abs((ds/Math.cos(angleN)))/Math.abs(toRelocate.getVelocityModule()); // Warning! catch the div-by-zero!!!
						
						if (dt>1.1) {
							dt=1;					
						}

						center1[0]+=-velocity1[0]*dt;
						center1[1]+=-velocity1[1]*dt;
						center1[2]+=-velocity1[2]*dt;
						toRelocate.setCenter(center1[0], center1[1], center1[2]);
						
						SimpleVector v1=new SimpleVector(toRelocate.getVelocityComponent());
						SimpleVector d=domain.getDirectionP();
						double dModule=Math.sqrt(d.x*d.x+d.y*d.y+d.z*d.z);
						float[] dVersorComponent=new float[3];
						dVersorComponent[0]=(float) (d.x/dModule);
						dVersorComponent[1]=(float) (d.y/dModule);
						dVersorComponent[2]=(float) (d.z/dModule);

						SimpleVector dVersor=new SimpleVector(dVersorComponent);

						float[] domainCenter=domain.getAbsCenter();						
						float[] nanoCenter=toRelocate.getCenter();
						float x=nanoCenter[0]-domainCenter[0];
						float y=nanoCenter[1]-domainCenter[1];
						float z=nanoCenter[2]-domainCenter[2];

						float relativeCenterModule=(float) Math.sqrt(x*x+y*y+z*z);
						SimpleVector nanoCenterVersor=new SimpleVector(x/relativeCenterModule,y/relativeCenterModule,z/relativeCenterModule);
						SimpleVector oTempVersor=nanoCenterVersor.calcCross(dVersor);
						oTempVersor=new SimpleVector(oTempVersor.x/oTempVersor.length(),oTempVersor.y/oTempVersor.length(),oTempVersor.z/oTempVersor.length());
						SimpleVector nTempVersor=oTempVersor.calcCross(dVersor);

						float alfa=nTempVersor.calcAngle(v1);	
						double n1Module=toRelocate.getVelocityModule()*Math.cos(alfa);
						float[] n1components=new float[3];
						n1components[0]=(float) (n1Module*nTempVersor.x);
						n1components[1]=(float) (n1Module*nTempVersor.y);
						n1components[2]=(float) (n1Module*nTempVersor.z);				
						SimpleVector n1=new SimpleVector(n1components);

						float gamma=dVersor.calcAngle(v1);				
						double d1Module=toRelocate.getVelocityModule()*Math.cos(gamma);
						float[] d1components=new float[3];
						d1components[0]=(float) (d1Module*dVersor.x);
						d1components[1]=(float) (d1Module*dVersor.y);
						d1components[2]=(float) (d1Module*dVersor.z);				
						SimpleVector d1=new SimpleVector(d1components);

						float ro=oTempVersor.calcAngle(v1);				
						double o1Module=toRelocate.getVelocityModule()*Math.cos(ro);
						float[] o1components=new float[3];
						o1components[0]=(float) (o1Module*oTempVersor.x);
						o1components[1]=(float) (o1Module*oTempVersor.y);
						o1components[2]=(float) (o1Module*oTempVersor.z);				
						SimpleVector o1=new SimpleVector(o1components);						

						n1.add(o1);
						SimpleVector v2=n1.calcSub(d1);

						float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
						float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];
						float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
						toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);	
						
						velocity1=toRelocate.getVelocityComponent();

						center1[0]+=velocity1[0]*dt;
						center1[1]+=velocity1[1]*dt;
						center1[2]+=velocity1[2]*dt;
						toRelocate.setCenter(center1[0], center1[1], center1[2]);	
						
					}else{
						CylindricPoreDomain cpd=(CylindricPoreDomain) toRelocate.getDomain();
						switchDomain(toRelocate,cpd.secondDomain);						
					}
				} else if ((Math.abs(distanceN))>=owner.getRadius()){		

					double ds=(Math.abs(distanceN)-(owner.getRadius()));
					dt =(float) Math.abs((ds/Math.cos(angleN)))/Math.abs(toRelocate.getVelocityModule()); // Warning! catch the div-by-zero!!!
					
					if (dt>1.1) {
						dt=1;					
					}

					center1[0]+=-velocity1[0]*dt;
					center1[1]+=-velocity1[1]*dt;
					center1[2]+=-velocity1[2]*dt;
					toRelocate.setCenter(center1[0], center1[1], center1[2]);					

					SimpleVector v1=new SimpleVector(toRelocate.getVelocityComponent());
					SimpleVector d=domain.getDirectionP();
					double dModule=Math.sqrt(d.x*d.x+d.y*d.y+d.z*d.z);
					float[] dVersorComponent=new float[3];
					dVersorComponent[0]=(float) (d.x/dModule);
					dVersorComponent[1]=(float) (d.y/dModule);
					dVersorComponent[2]=(float) (d.z/dModule);

					SimpleVector dVersor=new SimpleVector(dVersorComponent);
					float[] domainCenter=domain.getAbsCenter();
					float[] nanoCenter=toRelocate.getCenter();

					float x=nanoCenter[0]-domainCenter[0];
					float y=nanoCenter[1]-domainCenter[1];
					float z=nanoCenter[2]-domainCenter[2];

					float relativeCenterModule=(float) Math.sqrt(x*x+y*y+z*z);
					SimpleVector nanoCenterVersor=new SimpleVector(x/relativeCenterModule,y/relativeCenterModule,z/relativeCenterModule);
					SimpleVector oTempVersor=nanoCenterVersor.calcCross(dVersor);
					oTempVersor=new SimpleVector(oTempVersor.x/oTempVersor.length(),oTempVersor.y/oTempVersor.length(),oTempVersor.z/oTempVersor.length());
					SimpleVector nTempVersor=oTempVersor.calcCross(dVersor);					

					float alfa=nTempVersor.calcAngle(v1);	
					double n1Module=toRelocate.getVelocityModule()*Math.cos(alfa);
					float[] n1components=new float[3];
					n1components[0]=(float) (n1Module*nTempVersor.x);
					n1components[1]=(float) (n1Module*nTempVersor.y);
					n1components[2]=(float) (n1Module*nTempVersor.z);				
					SimpleVector n1=new SimpleVector(n1components);

					float gamma=dVersor.calcAngle(v1);				
					double d1Module=toRelocate.getVelocityModule()*Math.cos(gamma);
					float[] d1components=new float[3];
					d1components[0]=(float) (d1Module*dVersor.x);
					d1components[1]=(float) (d1Module*dVersor.y);
					d1components[2]=(float) (d1Module*dVersor.z);				
					SimpleVector d1=new SimpleVector(d1components);

					float ro=oTempVersor.calcAngle(v1);				
					double o1Module=toRelocate.getVelocityModule()*Math.cos(ro);
					float[] o1components=new float[3];
					o1components[0]=(float) (o1Module*oTempVersor.x);
					o1components[1]=(float) (o1Module*oTempVersor.y);
					o1components[2]=(float) (o1Module*oTempVersor.z);				
					SimpleVector o1=new SimpleVector(o1components);

					d1.add(o1);
					SimpleVector v2=d1.calcSub(n1);	

					float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
					float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];
					float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
					toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);		
					
					velocity1=toRelocate.getVelocityComponent();

					center1[0]+=velocity1[0]*dt;
					center1[1]+=velocity1[1]*dt;
					center1[2]+=velocity1[2]*dt;
					toRelocate.setCenter(center1[0], center1[1], center1[2]);
				}
			}
		}
	}
}
