package simulator;

import java.awt.Color;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import MainLibrary.AbstractDomain;
import MainLibrary.AbstractThreadingStrategy;
import MainLibrary.LigandObserver;
import MainLibrary.Manager;
import MainLibrary.MobilityModel;
import MainLibrary.NanoObj;
import MainLibrary.RootDomain;
import Wrappers.OutputWrapper;


public class DiffusionModel extends MobilityModel{
	//private final double ed=0.9;  //0.5
	private final static double k=1.381*Math.pow(10, -23);	
	public int nodeCount=0;
	public int HUVECCount=0;
	public boolean isTheFirst=true;
	//protected transient AbstractThreadingStrategy threadingStrategy = new NewDiffusionThreadingStrategy(this);	
	protected  AbstractThreadingStrategy threadingStrategy = new NewDiffusionThreadingStrategy(this);
	
	float powMinusU=0;	
	float constantTerm1=0;		
	float constantTerm2=0;
	float constantTerm3=0;
	
	boolean hasInitialized=false;
	
	//int cCollision=0;
	
	
	public DiffusionModel(){
	
	}

	public DiffusionModel(RootDomain d){
		super(d);		
	}
	
	public void initDiffusionCoeff(){
		float eta=getManager().configWrapper.viscosity;		
		float T=getManager().configWrapper.T;	
		float u=getManager().configWrapper.u;
		powMinusU=(float) Math.pow(10, (-u));
		
		constantTerm1=(float) (k*T/(6*Math.PI*eta*Math.pow(10, u)));		
		constantTerm2=(float) (2*getManager().get_dt()*Math.pow(10, u));
		constantTerm3=constantTerm1*constantTerm2;
	}

	public float getDiffusionCoefficient(NanoObj p){		
		
		if(!hasInitialized){
			initDiffusionCoeff();
			hasInitialized=true;
		}	
		
		/* THE ORIGINAL ONE! 
		float eta=p.getManager().configWrapper.viscosity;		
		float T=p.getManager().configWrapper.T;
		float D=(float) (k*T/(6*Math.PI*eta*p.getRadius()*Math.pow(10, (p.getManager().configWrapper.u))));		
		float sigma2=(float) (6*D/3*p.getManager().get_dt()*Math.pow(10, (p.getManager().configWrapper.u)));
		sigma2=(float) ((float) Math.sqrt(sigma2)*Math.pow(10, (-p.getManager().configWrapper.u)));
		return sigma2;
		*/
		
		
		//float DNEW=(float) (constantTerm1/p.getRadius());		
		//float sigma2NEW=(float) (constantTerm2*(constantTerm1/p.getRadius()));
		//float sigma2NEW=(float) (Math.sqrt((constantTerm2*(constantTerm1/p.getRadius())))*powMinusU);	
		
		float sigma2NEW=(float) (Math.sqrt(constantTerm3/p.getRadius())*powMinusU);
		return sigma2NEW;		
	}		

	public void networkCreationVasetti(){	

		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [VASETTI v2.0]");
		System.out.println("-------------------------------------");			

		// LOAD DATA FROM CONFIGURATION FILE
		Manager m=getManager();
		float height=m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "JarHeight");
		float rad=m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "JarRadius");
		float carriersConcentration=m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "carriersConcentration");
		float minutes=m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "minutesForCompleteRelease");
		float releaseLevelDisplacement=m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "releaseLevel_Displacement");
		int receptorsForCell=(int) m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "receptorsForCell");
		int receptorsType=(int) m.configurator.readCustomConfigurations("network.xml", "HUVEC2.0", "receptorsType");

		// =========== EXTERNAL BIG CYLINDER DOMAIN ===========
		float[] cdCenter=new float[3];
		cdCenter[0]=100000;
		cdCenter[1]=100000;
		cdCenter[2]=100000;		

		CylindricStrategy cs=new CylindricStrategy(0);
		DiffusionModel mainDiffusion=new DiffusionModel(getManager().rootDomain);

		CylindricJarDomain mainCylinder=new CylindricJarDomain(getManager().rootDomain, mainDiffusion, cs, true, false, height, rad, cdCenter, releaseLevelDisplacement, carriersConcentration, minutes, true);
		mainCylinder.ID="EXTERNAL CYLINDER JAR";
		//==============================


		// =========== THICK CYLINDRIC  DOMAIN  =============
		float[] cdCenter1=new float[3];	
		cdCenter1[0]=0;
		cdCenter1[1]=-height/2+releaseLevelDisplacement/4;
		cdCenter1[2]=0;		

		CylindricStrategy cs1=new CylindricStrategy(0);
		DiffusionModel mainDiffusion1=new DiffusionModel(getManager().rootDomain);

		CylindricDomain thickCylinder=new CylindricDomain(mainCylinder, mainDiffusion1, cs1, true, false, releaseLevelDisplacement/2, rad, cdCenter1, false);
		thickCylinder.ID="THICK CYLINDER";
		//==============================	


		// ===========  INCEPTION CUBIC RECEPTOR DOMAIN  =============		
		float halfSide = 7000;
		float diameter = (2*mainCylinder.getRadius());
		int cubesForSide =  Math.round(diameter/(2*halfSide));
		int numCubes = cubesForSide*cubesForSide;
		float[] centerX = createCenterX(numCubes, cubesForSide, halfSide);
		float[] centerY = createCenterY(numCubes, 1, halfSide);
		float[] centerZ = createCenterZ(numCubes, cubesForSide, 1, cubesForSide, halfSide);

		for(int i=0; i< numCubes; i++){
			float[] cubeCenter = new float[]{centerX[i],centerY[i]+(mainCylinder.getHeight()/2)+(halfSide - halfSide/100),centerZ[i]};			
			ArrayList<LigandObserver> observerForDomain=new ArrayList<LigandObserver>();
			observerForDomain.add(CD40LObserver.getInstance());

			CubicReceptorStrategy cubicReceptorStrategy = new CubicReceptorStrategy(1);
			CubicReceptorDomain cubeReceptor = new CubicReceptorDomain(mainCylinder, this, cubicReceptorStrategy, true, true, halfSide, cubeCenter, receptorsType, receptorsForCell, observerForDomain, false);
			cubeReceptor.ID="CUBE RECEPTOR "+i+1;
			cubeReceptor.setOutput(new OutputWrapper(cubeReceptor.ID, cubeReceptor.getAbsCenter(), 0, 0));
		}
		// ===========  END INCEPTION CUBIC RECEPTOR DOMAIN  =============		

		System.out.println(">>> TOTAL ENDOTHELIAL CELLS: "+numCubes);			
		System.out.println();	
	}

	public void removeOverlappedDomains(CylindricDomain d){

		double radius=d.getRadius();
		float[] origin=new float[3];
		origin[0]=0;
		origin[1]=0;
		origin[2]=0;

		float[] coords1=new float[3];
		double radius1=0;
		float[] coords2=new float[3];
		double radius2=0;
		int deathCount=0;

		List<AbstractDomain> childDomains=d.getAllChildDomains();
		List<AbstractDomain> domainsToRemove=new ArrayList<AbstractDomain>();

		for (int i=0;i<childDomains.size();i++){

			float[] tempCoords1=new float[3];
			childDomains.get(i).getRelativeCenter();
			tempCoords1[0]=childDomains.get(i).getRelativeCenter()[0];
			tempCoords1[1]=0;
			tempCoords1[2]=childDomains.get(i).getRelativeCenter()[2];

			radius1=((CylindricDomain)childDomains.get(i)).getRadius();

			float distance=AbstractDomain.getDistance(tempCoords1, origin);

			if (distance+radius1>radius){
				if(!(domainsToRemove.contains(childDomains.get(i)))){
					domainsToRemove.add(childDomains.get(i));
				}			
				deathCount++;		
			}
		}
		
		for(int k=0;k<domainsToRemove.size();k++){				
			domainsToRemove.get(k).destroy();
		}			
		domainsToRemove.clear();

		for (int i=0;i<childDomains.size()-1;i++){
			coords1=((CylindricDomain)childDomains.get(i)).getRelativeCenter();
			radius1=((CylindricDomain)childDomains.get(i)).getRadius();
			for (int j=i+1;j<childDomains.size();j++){						
				coords2=((CylindricDomain)childDomains.get(j)).getRelativeCenter();
				radius2=((CylindricDomain)childDomains.get(j)).getRadius();	

				float distance=AbstractDomain.getDistance(coords1,coords2);
				if (distance<(radius1+radius2)){
					if(!(domainsToRemove.contains(childDomains.get(j)))){
						domainsToRemove.add(childDomains.get(j));
					}					
					deathCount++;
				}
			}
			
			for(int k=0;k<domainsToRemove.size();k++){				
				domainsToRemove.get(k).destroy();
			}			
			domainsToRemove.clear();
		}
		System.out.println(">>> OVERLAPPED DOMAINS REMOVED: "+deathCount);
	}

	public void preloadOfCD40L(AbstractDomain domain,DiffusionModel d,int size,int bound,int[] origin,int scaleFactor, int type){	

		int x,y,z;
		Random rand=new Random();	

		for(int i=0;i<size;i++){		

			// Read from nodePositions.xml
			x=(int) (((rand.nextDouble()-0.5)*bound*scaleFactor)+origin[0]);
			y=(int) (((rand.nextDouble()-0.5)*bound*scaleFactor)+origin[1]);
			z=(int) (((rand.nextDouble()-0.5)*bound*scaleFactor)+origin[2]);	

			new CD40LCarrier(domain,x,y,z,type);	
		}
	}

	public void createHUVECNode(int[] center, AbstractDomain d){
		int x ,y ,z;
		x=center[0];
		y=center[1];
		z=center[2];		

		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer=new ArrayList<LigandObserver>();
		observer.add(CD40LObserver.getInstance());			
		HUVEC huvecNode=new HUVEC(d,x,y,z,(int)4,observer);
		if (huvecNode.getTimeToLive()>0) { 
			HUVECCount++;
			if (isTheFirst){
				huvecNode.setColor(Color.red);
				isTheFirst=false;
			}
		}
	}

	public int[] moveOnXAxis(int i,int[] origin, int radius,AbstractDomain d){
		int x = origin[0];
		int[] newOrigin=new int[3];
		short sign;
		int residue=i%2;
		sign=1;
		if (residue<1) sign=-1;

		for(int k=1;k<=i;k++){
			x=x+(2*radius*sign);
			newOrigin[0]=x;
			newOrigin[1]=origin[1];
			newOrigin[2]=origin[2];

			createHUVECNode(newOrigin,d);
		}
		return newOrigin;
	}

	public int[] moveOnYAxis(int i,int[] origin, int radius,AbstractDomain d){
		int y = origin[1];
		int[] newOrigin=new int[3];
		short sign;
		int residue=i%2;
		sign=1;
		if (residue<1) sign=-1;

		for(int k=1;k<=i;k++){
			y=y+(2*radius*sign);
			newOrigin[0]=origin[0];
			newOrigin[1]=y;
			newOrigin[2]=origin[2];

			createHUVECNode(newOrigin,d);
		}
		return newOrigin;
	}

	public int[] moveOnZAxis(int i, int[] origin, int radius,AbstractDomain d){
		int z = origin[2];
		int[] newOrigin=new int[3];
		short sign;
		int residue=i%2;
		sign=-1;
		if (residue<1) sign=+1;

		for(int k=1;k<=i;k++){
			z=z+(2*radius*sign);
			newOrigin[0]=origin[0];
			newOrigin[1]=origin[1];
			newOrigin[2]=z;

			createHUVECNode(newOrigin,d);
		}
		return newOrigin;
	}

	public void networkCreation(){		
		networkCreationVasetti();  
	}
	
	public void algorithm(AbstractDomain d){
		AbstractDomain domain=d;
		List<NanoObj> nanoObjList=d.getAllDisposedObj();
		try{

			//long start=System.nanoTime();
			
			move(nanoObjList);
			/*
			long stop=System.nanoTime();
			
			System.out.println("Move: "+(stop-start));
			*/
			

		}catch (Exception e){		
			System.out.println("-- Error on DiffusionModel.Algorithm.Move: "+e.toString());
			e.printStackTrace();
		}		
		try{
			
			//long start=System.nanoTime();
			
			if (nanoObjList.size()>0) checkCollision(nanoObjList);   

			/*
			long stop=System.nanoTime();
			
			System.out.println("Check: "+(stop-start));
			*/
			
		}catch (Exception e){
			System.out.println("-- Error on DiffusionModel.Algorithm.Checkcollision: "+e.toString());
		}
		try{
			
			while(domain.getCollisionsList().size()>0){					
				avoidCollision(domain);	
			}
		}catch (Exception e){
			System.out.println("-- Error on DiffusionModel.Algorithm.AvoidCollision: "+e.toString());
		}
	}	
	
	public void detectCollision(NanoObj p1,NanoObj p2){
		try {
			float[] center1=p1.getCenter();
			float[] center2=p2.getCenter();
			float[] velocity1=p1.getVelocityComponent();
			float[] velocity2=p2.getVelocityComponent();
			double r1=p1.getRadius();
			double r2=p2.getRadius();
			float dx = (center2[0]-center1[0]), dy = (center2[1]-center1[1]), dz= (center2[2]-center1[2]);

			// where x1,y1,z1 are center of ball1, and x2,y2,z2 are center of ball2
			float distance = (float) Math.sqrt(dx*dx+dy*dy+dz*dz);

			if (distance<((r1+r2))){

				// Unit vector in the direction of the collision
				float ax=(dx/distance), ay=(dy/distance), az=(dz/distance);

				// Projection of the velocities in these axes
				float va1=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az); 
				float va2=(velocity2[0]*ax+velocity2[1]*ay+velocity2[2]*az);

				// the collision should have occurred at t-dt (Actually this is also an approximation).
				float dt=1;
				try {
					dt =(float) ((r1+r2-distance)/Math.abs(va1-va2)); // Warning! catch the div-by-zero!!!
				} catch (Exception e) {
					System.out.println(e.getMessage());	
				}				
				p1.getDomain().addToCollisionsList(p1,p2,dt);				
			}
		} catch (Exception e) {
			System.out.println("-- DiffusionModel.Algorithm.detectCollision-INSIDE!: "+e.toString());
			e.printStackTrace();
		}
	}	

	public void move(List<NanoObj> nanoList){

		if (nanoList.size()>0){
			try{
				threadingStrategy.move(nanoList);				
			}catch (Exception e){
				e.printStackTrace();
			}			
		}
	}

	public void checkCollision(List<NanoObj> objsToCheck){
		
		if (objsToCheck.size()>0){
			try{						
				threadingStrategy.checkCollision(objsToCheck);		// ORIGINAL
			}catch (Exception e){
				e.printStackTrace();
			}			
		}	
	}

	public synchronized void avoidCollision(AbstractDomain d){		
		boolean isDone=false;
		try {	
			isDone=reorderCollision_byHeap(d);
		}catch (Exception e) {
			System.out.println("-1- Error on avoidCollision. Collisionlist: "+d.getCollisionsList().size()+" - "+e.toString());
			e.printStackTrace();
		}		
		try{
			int size=d.getCollisionsList().size(); 				
			for (int i=0;i<size;i++){
				d.getCollisionsList().get(i).handleCollision();					
			}
		}catch (Exception e) {
			System.out.println("-2- Error on avoidCollision. "+d.getCollisionsList().size()+" - "+e.toString()+" - isDone: "+isDone);
			e.printStackTrace();
		}
		d.getCollisionsList().clear();	
	}
	
	public void relocateObjs(NanoObj p1, NanoObj p2) {
		
		/*
		// FIXME DEBUG
		synchronized (this) {
			cCollision++;
		}*/
		
		float[] center1=p1.getCenter();
		float[] center2=p2.getCenter();
		float[] velocity1=p1.getVelocityComponent();
		float[] velocity2=p2.getVelocityComponent();
		float r1=p1.getRadius();
		float r2=p2.getRadius();
		float modVelocity1=p1.getVelocityModule();
		float modVelocity2=p2.getVelocityModule();
		float mass1=p1.getMass();
		float mass2=p2.getMass();
		double dt=0;

		// The momentum has to be <= before and after the collision!
		float momentum1=modVelocity1*mass1;
		float momentum2=modVelocity2*mass2;
		float momentumBefore=momentum1+momentum2;
		
		
		if(p1.getTimeToLive()<0 || p2.getTimeToLive()<0){
			System.out.println(" *******  KILLED!!!   "+p1.toString()+": "+p1.getTimeToLive()+",  "+p2.toString()+": "+p2.getTimeToLive());
			System.out.println();
		}
		
		if (modVelocity1==0 && modVelocity2==0){

		} else{

			float dx21 = (center2[0]-center1[0]);
			float dy21 = (center2[1]-center1[1]);
			float dz21 = (center2[2]-center1[2]);

			float dx12 = (center1[0]-center2[0]);
			float dy12 = (center1[1]-center2[1]);
			float dz12 = (center1[2]-center2[2]);

			float distance = (float) Math.sqrt(dx21*dx21+dy21*dy21+dz21*dz21);

			if (distance<((r1+r2))){

				// Unit vector in the direction of the collision
				float ax=((dx21/distance)), ay=((dy21/distance)), az=((dz21/distance));

				float alfa=0;
				if (modVelocity1*distance!=0){
					try{
						BigDecimal bg = new BigDecimal((velocity1[0]*dx21+velocity1[1]*dy21+velocity1[2]*dz21)/(modVelocity1*distance)); 
						bg = bg.setScale(3, BigDecimal.ROUND_HALF_UP); 
						alfa=(float) Math.acos(bg.doubleValue()); // angle between v and d
					} catch (NumberFormatException e){
						e.printStackTrace();
					}
				} 

				if(ax>1 || ax<-1 || ay>1 || ay<-1 || az>1 || az<-1){
					System.out.println("==========================================================================");
					System.out.println(" RelocateNANOOBJ - gli ax ay e az eccedono i limiti dell'arccos() "+distance);
					System.out.println("==========================================================================");
				}

				float alfaX1=(float) Math.acos((dx21)/(distance));
				float alfaY1=(float) Math.acos((dy21)/(distance));
				float alfaZ1=(float) Math.acos((dz21)/(distance));

				float modVa1=(float) (Math.abs(modVelocity1)*Math.cos(alfa));
				float vax1=(float) (Math.abs(modVa1)*Math.cos(alfaX1));
				float vay1=(float) (Math.abs(modVa1)*Math.cos(alfaY1));
				float vaz1=(float) (Math.abs(modVa1)*Math.cos(alfaZ1));

				float vbx1=velocity1[0]-vax1;
				float vby1=velocity1[1]-vay1;
				float vbz1=velocity1[2]-vaz1;

				float beta=0;
				if (modVelocity2*distance!=0){	
					try{
						BigDecimal bg = new BigDecimal((velocity2[0]*dx12+velocity2[1]*dy12+velocity2[2]*dz12)/(modVelocity2*distance)); 
						bg = bg.setScale(3, BigDecimal.ROUND_HALF_UP); 
						beta=(float) Math.acos(bg.doubleValue()); // angle between v and d
					} catch (NumberFormatException e){
						e.printStackTrace();
					}				
				}

				float alfaX2=(float) Math.acos((dx12)/(distance));
				float alfaY2=(float) Math.acos((dy12)/(distance));
				float alfaZ2=(float) Math.acos((dz12)/(distance));

				// Velocity component on collision's direction
				float modVa2= (float) (Math.abs(modVelocity2)*Math.cos(beta));
				float vax2=(float) (Math.abs(modVa2)*Math.cos(alfaX2));
				float vay2=(float) (Math.abs(modVa2)*Math.cos(alfaY2));
				float vaz2=(float) (Math.abs(modVa2)*Math.cos(alfaZ2));

				float vbx2=velocity2[0]-vax2;
				float vby2=velocity2[1]-vay2;
				float vbz2=velocity2[2]-vaz2;

				float va1t=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az);
				float va2t=(velocity2[0]*(ax)+velocity2[1]*(ay)+velocity2[2]*(az));
				
				dt =((r1+r2-distance)/Math.abs(va1t-va2t)); 			

				if (dt>1.1) {
					dt =1; 						
				}				

				center1[0]-=velocity1[0]*dt;
				center1[1]-=velocity1[1]*dt;
				center1[2]-=velocity1[2]*dt;
				p1.setCenter(center1[0], center1[1], center1[2]);

				center2[0]-=velocity2[0]*dt;
				center2[1]-=velocity2[1]*dt;
				center2[2]-=velocity2[2]*dt;
				p2.setCenter(center2[0], center2[1], center2[2]);

				float[] vaP1=new float[3];
				vaP1[0]=(float) (vax1 + (1+ed)*(vax2-vax1)/(1+mass1/mass2));
				vaP1[1]=(float) (vay1 + (1+ed)*(vay2-vay1)/(1+mass1/mass2));
				vaP1[2]=(float) (vaz1 + (1+ed)*(vaz2-vaz1)/(1+mass1/mass2));

				float[] vaP2=new float[3];
				vaP2[0]=(float) (vax2 + (1+ed)*(vax1-vax2)/(1+mass2/mass1));
				vaP2[1]=(float) (vay2 + (1+ed)*(vay1-vay2)/(1+mass2/mass1));
				vaP2[2]=(float) (vaz2 + (1+ed)*(vaz1-vaz2)/(1+mass2/mass1));

				float vxRound = ((vaP1[0]+vbx1));
				float vyRound = ((vaP1[1]+vby1));
				float vzRound = ((vaP1[2]+vbz1));
				velocity1[0]= vxRound;
				velocity1[1]= vyRound;
				velocity1[2]= vzRound;

				//float fi1=p1.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[0];
				//float theta1=p1.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[1];
				
				float[] angles=p1.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2]);		
				float fi1=angles[0];
				float theta1=angles[1];
				
				float nextVelocityModule1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
				p1.setVelocityComponent(nextVelocityModule1, fi1, theta1);

				vxRound = ((vaP2[0]+vbx2));
				vyRound = ((vaP2[1]+vby2));
				vzRound = ((vaP2[2]+vbz2));
				velocity2[0]= vxRound;
				velocity2[1]= vyRound;
				velocity2[2]= vzRound;

				//float fi2=p2.getVelocityAngles(velocity2[0], velocity2[1], velocity2[2])[0];
				//float theta2=p2.getVelocityAngles(velocity2[0], velocity2[1], velocity2[2])[1];
				
				float[] angles2=p2.getVelocityAngles(velocity2[0], velocity2[1], velocity2[2]);		
				float fi2=angles2[0];
				float theta2=angles2[1];
				
				float nextVelocityModule2=(float) Math.sqrt(velocity2[0]*velocity2[0]+velocity2[1]*velocity2[1]+velocity2[2]*velocity2[2]);
				p2.setVelocityComponent(nextVelocityModule2, fi2, theta2);

				// Evaluates the Momentum!
				momentum1=nextVelocityModule1*mass1;				
				momentum2=nextVelocityModule2*mass2;				
				float momentumAfter=momentum1+momentum2;

				if((momentumBefore*1.1)<momentumAfter){
					float error=momentumBefore/momentumAfter;
					nextVelocityModule1=nextVelocityModule1*error;
					nextVelocityModule2=nextVelocityModule2*error;

					p1.setVelocityModule(nextVelocityModule1);
					p2.setVelocityModule(nextVelocityModule2);

					velocity1=p1.getVelocityComponent();
					velocity2=p2.getVelocityComponent();
				}

				center1[0]+=velocity1[0]*dt;
				center1[1]+=velocity1[1]*dt;
				center1[2]+=velocity1[2]*dt;
				p1.setCenter(center1[0], center1[1], center1[2]);

				center2[0]+=velocity2[0]*dt;
				center2[1]+=velocity2[1]*dt;
				center2[2]+=velocity2[2]*dt;
				p2.setCenter(center2[0], center2[1], center2[2]);

				modVelocity1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
				modVelocity2=(float) Math.sqrt(velocity2[0]*velocity2[0]+velocity2[1]*velocity2[1]+velocity2[2]*velocity2[2]);
			}
		}
	}
	
	private float[] createCenterX(int numCubi, int x, float halfSide){

		float[] centerX = new float[numCubi];
		int limit = numCubi/x;  // temp for the scanning...
		float j;		
		j = -(x-1)*halfSide;
		int i=0; // scan index
		while(i<numCubi){
			while(i<limit){
				centerX[i]=j;
				i++;
			}
			if(i<numCubi){
				j += 2*halfSide;
				limit += numCubi/x;
			}
		}		
		return centerX;		
	}

	private float[] createCenterZ(int numCubi, int x, int y, int z, float halfSide){
		float[] centerZ = new float[numCubi];		

		int limit1;
		int limit2 = numCubi/x;
		float j;
		int i=0;
		int numeroIterazioni=1;
		do{ 
			j = -(z-1)*halfSide;
			while(i<limit2){
				limit1 =0;
				while(limit1<y){
					centerZ[i]=j;
					limit1++;
					i++;
				}
				j += 2*halfSide;
			}
			limit2 += numCubi/x;
			numeroIterazioni++;
		}while(numeroIterazioni<x+1);

		return centerZ;
	}

	private float[] createCenterY(int gridSize, int y, float halfSide){
		float[] centerY = new float[gridSize];		

		int limit;
		float j;
		int i=0;
		while(i<gridSize){
			j= -(y-1)*halfSide;
			limit = 0;
			while(limit<y){
				centerY[i]=j;
				limit++; i++;
				j += 2*halfSide;				
			}
		}
		return centerY;
	}

	@Override
	public boolean checkCoords(float x, float y, float z) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public AbstractThreadingStrategy getThreadingStrategy(){
		return this.threadingStrategy;
	}
}
