package simulator;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import MainLibrary.MobilityModel;
import MainLibrary.MotionStrategy;
import MainLibrary.NanoObj;


public class EffectiveDiffusionMotion extends MotionStrategy {

	private static EffectiveDiffusionMotion instance=null;
	public static float k=(float) (1.381*Math.pow(10, -23));	
	
	private FileOutputStream file;
	private PrintStream Output;
	
	private EffectiveDiffusionMotion(){
		
	}
	
	public static EffectiveDiffusionMotion getInstance(){
		if (instance==null) instance=new EffectiveDiffusionMotion();
		return instance;
	}
	
	public float getBrownianDiffusionCoefficient(NanoObj p){
		float eta=p.getManager().configWrapper.viscosity;		
		float T=p.getManager().configWrapper.T;
		float D=(float) (k*T/(6*Math.PI*eta*p.getRadius()*Math.pow(10, (p.getManager().configWrapper.u))));		
		float sigma2=(float) (6*D/3*p.getManager().get_dt()*Math.pow(10, (p.getManager().configWrapper.u)));
		return sigma2= (float) (Math.sqrt(sigma2)*Math.pow(10, (-p.getManager().configWrapper.u)));
	}
	
	public float getEffectiveDiffusionCoefficient(NanoObj p){
		float eta = p.getManager().configWrapper.viscosity;		
		float T = p.getManager().configWrapper.T;
		CylindricDomain c = (CylindricDomain) p.getDomain();
		float pipeRadius = (float) ((c.getRadius())/Math.pow(10,-p.getManager().configWrapper.u));
		float meanFlowVel = (float) (2*(Math.pow(10, -4)));
		float D = (float) (k*T/(6*Math.PI*eta*p.getRadius()*Math.pow(10, (p.getManager().configWrapper.u))));
		float pecletNumber = (pipeRadius*meanFlowVel)/D;
		float Deff = D*(1+(pecletNumber*pecletNumber)/48);		
		float sigma2 = (float) (6*Deff/3*p.getManager().get_dt()*Math.pow(10, (p.getManager().configWrapper.u)));
		return sigma2 = (float) (Math.sqrt(sigma2)*Math.pow(10, (-p.getManager().configWrapper.u)));
	}
	
	@Override
	public void move(NanoObj p) {
		float[] center=p.getCenter();
		float newVelocityModule=p.getVelocityModule();
		MobilityModel model=p.getManager().getMobilityModel();		
		float sigma2 = getBrownianDiffusionCoefficient(p);
		float sigma22 = getEffectiveDiffusionCoefficient(p);
		
		float[] component=p.getVelocityComponentMS();	
		float dx=component[0];
		float dy=component[1];
		float dz=component[2];
		
		
		if(p.getManager().configWrapper.ddt<1){				
			dx = model.getGauss(sigma2);			
			dy = model.getGauss(sigma22);			
			dz = model.getGauss(sigma2);
			
			newVelocityModule=(float) Math.sqrt((dx*dx)+(dy*dy)+(dz*dz));
			float[] angles=p.getVelocityAngles(dx,dy,dz);
			p.setVelocityComponentMS(newVelocityModule,angles[0], angles[1]);	
		}

		center[0]=center[0]+dx;
		center[1]=center[1]+dy;
		center[2]=center[2]+dz;
		p.setCenter(center[0],center[1],center[2]);	
	}
	
	public void exportEffectiveDiffusion(float dx, float dy, float dz){		
		try{					
			file = new FileOutputStream("Gaussian.txt",true);
			Output = new PrintStream(file);
			Output.println(dx+" "+dy+" "+dz);			
			 
		} catch (IOException e) {
			System.out.println("Output Error: " + e);
			
		} finally {
		    // Close resource.
		    if (file!= null) try { 
		        file.close();
		    } catch (IOException logOrIgnore) {
		    	System.out.println("Output Error [closing file]: " + logOrIgnore);
		    }
		}
	}
}
