package simulator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import MainLibrary.NanoObj;
import MainLibrary.OutputStrategy;

public class FlowModelMoveThread implements Runnable{

	protected List<NanoObj> partialNanoObjList=new ArrayList<NanoObj>();
	protected BloodFlowModel model;
	protected int size=0;
	public CountDownLatch latch;

	
	// ORIGINAL IMPLEMENTATION - BEFORE THE 2 TIMESTEPS!! (_dt e ddt)
	public FlowModelMoveThread(List<NanoObj> nanoList, CountDownLatch latch){	
		partialNanoObjList=nanoList;				
		model=(BloodFlowModel)partialNanoObjList.get(0).getDomain().getMobilityModel();
		size=partialNanoObjList.size();
		this.latch=latch;
	}

	public void run(){
		try {
					
			model.laminarFlowMovement(partialNanoObjList);	// OK
						
		} catch (Exception e1) {			
			e1.printStackTrace();
		}

		NanoObj p=null;		
		for (int i=0;i<size;i++){			
			try {
				p=partialNanoObjList.get(i);
				if(p!=null) {
					
					p.getMotionStrategy().move(p);	

					
					/** Create the TOTAL VELOCITY VECTOR 
					 * as SUM of the MotionStrategy and MobilityModel velocity component**/		
					
					float[] vMM=p.getVelocityComponentMM();						
					float[] vMS=p.getVelocityComponentMS();
					float[] vTOT=new float[3];
					vTOT[0]=vMS[0]+vMM[0];
					vTOT[1]=vMS[1]+vMM[1];
					vTOT[2]=vMS[2]+vMM[2];
					float[] anglesTOT = p.getVelocityAngles(vTOT[0], vTOT[1], vTOT[2]);
					float vModuleTOT=(float) Math.sqrt(vTOT[0]*vTOT[0]+vTOT[1]*vTOT[1]+vTOT[2]*vTOT[2]);				
					p.setVelocityComponent(vModuleTOT, anglesTOT[0], anglesTOT[1]);		
					
					/*
					// FIXME DEBUG ONLY
					//System.out.println(""+vModuleTOT+" "+anglesTOT[0]+" "+anglesTOT[1]);
					HashMap<Object,Object> results=new HashMap<Object,Object>();
					//results.put("export", ""+p.getVelocityModuleMM()+" "+p.getVelocityModuleMS()+" "+vModuleTOT+" "+anglesTOT[0]+" "+anglesTOT[1]);
					//results.put("export", ""+vModuleTOT+" "+anglesTOT[0]+" "+anglesTOT[1]);
					results.put("export", ""+p.getCenter()[1]);
					
					OutputDebugTXT oo=OutputDebugTXT.getInstance();
					
					oo.export(results);
					// END DEBUG
					*/	
							
				}else{
					System.out.println("========================");
					System.out.println(partialNanoObjList.get(0).getDomain().ID);
					System.out.println("FlowModelMoveThread -> p is NULL!!! - size: "+size+" - Iteration: "+i);
					System.out.println("disposed: "+partialNanoObjList.get(0).getDomain().getAllDisposedObj().size()+" - undisposed: "+partialNanoObjList.get(0).getDomain().getAllUndisposedObj().size());
					System.out.println("========================");					
				}
			} catch (Exception e) {
				System.out.println("--Error on FlowModelMoveThread.run() - SIZE: "+size+" - p.size(): "+partialNanoObjList.size());
				e.printStackTrace();
				System.exit(0);
			}
		}
		this.latch.countDown();
	}		
}
