package simulator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CountDownLatch;

import MainLibrary.CarrierObj;
import MainLibrary.MobilityModel;
import MainLibrary.NanoObj;

public class FlowModelMoveThread2Timesteps implements Runnable{

	protected List<NanoObj> partialNanoObjList=new ArrayList<NanoObj>();
	protected BloodFlowModel model;
	protected int size=0;
	public CountDownLatch latch;
	
	public int ddt_step=1;
	

	public FlowModelMoveThread2Timesteps(List<NanoObj> nanoList, CountDownLatch latch, MobilityModel m){	
		partialNanoObjList=nanoList;				
		model=(BloodFlowModel)partialNanoObjList.get(0).getDomain().getMobilityModel();
		size=partialNanoObjList.size();
		this.latch=latch;
		
		this.ddt_step=(int) (m.getManager().configWrapper.timeUnit/m.getManager().configWrapper._dt);
	}

	public void run(){
		try {
					
			model.laminarFlowMovement(partialNanoObjList);	// OK
						
		} catch (Exception e1) {			
			e1.printStackTrace();
		}

		NanoObj p=null;		
		for (int i=0;i<size;i++){			
			try {
				p=partialNanoObjList.get(i);
				if(p!=null) {
					
					p.getMotionStrategy().move(p);	

					float[] vTOT=new float[3];
					
					boolean isCarrier=(p instanceof CarrierObj);
					
					int divider=1;
					
					
					
					if( isCarrier || (p.getManager().configWrapper.ddt<1)){	

						/** Create the TOTAL VELOCITY VECTOR 
						 * as SUM of the MotionStrategy and MobilityModel velocity component**/		
						
						float[] vMM=p.getVelocityComponentMM();						
						float[] vMS=p.getVelocityComponentMS();						
						
						vTOT[0]=vMS[0]+vMM[0];
						vTOT[1]=vMS[1]+vMM[1];
						vTOT[2]=vMS[2]+vMM[2];
						float[] anglesTOT = p.getVelocityAngles(vTOT[0], vTOT[1], vTOT[2]);
						float vModuleTOT=(float) Math.sqrt(vTOT[0]*vTOT[0]+vTOT[1]*vTOT[1]+vTOT[2]*vTOT[2]);				
						p.setVelocityComponent(vModuleTOT, anglesTOT[0], anglesTOT[1]);	
					}
					
					if(!isCarrier){
						vTOT=p.getVelocityComponent();		
						divider=ddt_step;						
					}
					
					
					// PARTIAL MOVE - 31/07/2015
					float[] nano_center = p.getCenter();
					float[] center = {nano_center[0],nano_center[1],nano_center[2]};							
					
					center[0]=center[0]+(vTOT[0]/divider); 
					center[1]=center[1]+(vTOT[1]/divider);  
					center[2]=center[2]+(vTOT[2]/divider);
					p.setCenter(center[0],center[1],center[2]);				
					
					
					
					/*
					// FIXME DEBUG ONLY					
					float tt=p.getTheta();
					float ff=p.getFi();
					float mm=p.getVelocityModule();
					
					//System.out.println(""+vModuleTOT+" "+anglesTOT[0]+" "+anglesTOT[1]);
					HashMap<Object,Object> results=new HashMap<Object,Object>();
					//results.put("export", ""+p.getVelocityModuleMM()+" "+p.getVelocityModuleMS()+" "+vModuleTOT+" "+anglesTOT[0]+" "+anglesTOT[1]);
					//results.put("export", ""+mm+" "+tt+" "+ff);
					results.put("export", ""+center[1]);
					OutputDebugTXT oo=OutputDebugTXT.getInstance();
					
					oo.export(results);
					// END DEBUG
					*/
					
					
				}else{
					System.out.println("========================");
					System.out.println(partialNanoObjList.get(0).getDomain().ID);
					System.out.println("FlowModelMoveThread -> p is NULL!!! - size: "+size+" - Iteration: "+i);
					System.out.println("disposed: "+partialNanoObjList.get(0).getDomain().getAllDisposedObj().size()+" - undisposed: "+partialNanoObjList.get(0).getDomain().getAllUndisposedObj().size());
					System.out.println("========================");					
				}
			} catch (Exception e) {
				System.out.println("--Error on FlowModelMoveThread.run() - SIZE: "+size+" - p.size(): "+partialNanoObjList.size());
				e.printStackTrace();
				System.exit(0);
			}
		}
		this.latch.countDown();
	}		
}