package simulator;


import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import com.threed.jpct.SimpleVector;
//import scala.Math;
import engine.Engine3D;
import MainLibrary.AbstractDomain;
import MainLibrary.LigandObserver;
import MainLibrary.LinearMotion;
import MainLibrary.NanoObj;
import MainLibrary.RootDomain;
import Octree.OctreeNode;
import Octree.OctreeRoot;


public class FreeDiffusionModel extends DiffusionModel{

	public int simDistance=10000;	// micrometers between the nodes center
	public int xDisplacement=0;	// micrometers between the nodes center
	public int yDisplacement=0;	// micrometers between the nodes center
	public int zDisplacement=0;	// micrometers between the nodes center
	public int burst=0;	// emission burst
	public int txEmissionThreshold=0;	// emission Threshold upper bound



	// CONFIG FOR -RX NODE-
	public boolean receptorStats=false;
	public boolean receptorRandomTime=false;
	public boolean absorbentReceiver=false;
	public boolean rejectingReceiver=false;

	
	// CONFIG FOR -RX/TX NODE-
	public boolean transceiverNode=false;
	public float displacementL=0;
	public float displacementH=0;
	
	
	public String type="";


	public FreeDiffusionModel(){
		//threadingStrategy = new CalciumDiffusionThreadingStrategy(this);
	}

	public FreeDiffusionModel(String type){
		//threadingStrategy = new CalciumDiffusionThreadingStrategy(this);
		this.type=type;
	}

	public FreeDiffusionModel(RootDomain d){
		super(d);
		//threadingStrategy = new CalciumDiffusionThreadingStrategy(this);
	}


	public void networkCreation(){				
		if(type.contains("FreeDiffusion")){			
			networkCreation_v1();	
		}
	}



	public void networkCreation_v1(){


		System.out.println("-------------------------------------");				
		System.out.println("-------------------------------------");	


		// ENLARGED DOMAIN FOR DEBUG
		float[] cdCenter=new float[3];
		cdCenter[0]=1000000;
		cdCenter[1]=1000000;
		cdCenter[2]=1000000;			

		float halfSide=(float) (1000000/2);  // cube's half side 

		boolean useOctree1=true;

		TCPLike_CubicStrategy ss=new TCPLike_CubicStrategy(0);
		FreeDiffusionModel innerDiffusion=new FreeDiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube=new CubicDomain(getManager().rootDomain, innerDiffusion, ss, true, false, halfSide, cdCenter, useOctree1);
		AbstractDomain domainForRXNode=boundaryCube;


		// NODES

		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer.add(TCPLike_CarrierA_Observer.getInstance());
		observer.add(TCPLike_CarrierB_Observer.getInstance());

		if(useOctree1){
			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			boundaryCube.rootTree.cleanRootLeafs();
			boundaryCube.rootTree=null;		
			boundaryCube.rootTree=new OctreeRoot(cdCenter[0], cdCenter[1], cdCenter[2], 2*halfSide, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
			boundaryCube.rootTree.setDomain(boundaryCube);
		}

		// RX NODE ABS COORDINATES!
		float xCenterRX=xDisplacement;
		float yCenterRX=yDisplacement;
		float zCenterRX=zDisplacement;

		
		int TXNumberType=1;
		float radiusTX=domainForRXNode.getManager().configWrapper.nodeConfig.get(TXNumberType+"-radius").floatValue();
		
		int RXNumberType=2;   // era 29
		float radiusRX=domainForRXNode.getManager().configWrapper.nodeConfig.get(RXNumberType+"-radius").floatValue();
		
		int RXTXNumberType=3;
		float radiusRXTX=domainForRXNode.getManager().configWrapper.nodeConfig.get(RXTXNumberType+"-radius").floatValue();

		
		
		// TX NODE ABS COORDINATES
		float distance=(float) (simDistance+radiusRX+radiusTX);
		float xCenterTX=xCenterRX+distance;
		float yCenterTX=yCenterRX;
		float zCenterTX=zCenterRX;

		// CREATE TX NODE!
		SimpleTX nodeTX=new SimpleTX(domainForRXNode,xCenterTX,yCenterTX,zCenterTX,TXNumberType,observer); 	
		nodeTX.burstEmission=burst;	
		
		

		// CREATE RX NODE!
		SimpleRX nodeRX=new SimpleRX(domainForRXNode,xCenterRX,yCenterRX,zCenterRX,RXNumberType,observer); 		
		
		nodeRX.burstEmission=0;		
		nodeRX.receptorStats=receptorStats;
		nodeRX.receptorRandomTime=receptorRandomTime;		
		nodeRX.absorbentReceiver=absorbentReceiver;
		nodeRX.rejectingReceiver=rejectingReceiver;		

		nodeRX.setColor(Color.lightGray);

		
		
		/*
		if(transceiverNode){
			
			// CHECK The positioning of the RXTX Node
			float longitudinalMargins=distanceradiusRXTX
			
			
			// CREATE RX/TX NODE!
			

			float xCenterRXTX=xCenterRX+displacementL;
			float yCenterRXTX=yCenterRX+displacementH;
			float zCenterRXTX=zCenterRX;
			
			
			SimpleTX nodeRXTX=new SimpleTX(domainForRXNode,xCenterTX,yCenterTX,zCenterTX,RXTXNumberType,observer); 		
			nodeRXTX.burstEmission=burst;
			
			
			
			nodeRXTX.receptorRandomTime=receptorRandomTime;		
			nodeRXTX.absorbentReceiver=absorbentReceiver;
			nodeRXTX.rejectingReceiver=rejectingReceiver;		

			nodeRXTX.setColor(Color.white);
			
		}*/

		
		//========================================================================================================


		System.out.println(">>> simulation distance: "+simDistance);		
		System.out.println(">>> TX node - Released Burst: "+burst);	
		System.out.println(">>> TX Node  -  x: "+xCenterTX+", y: "+yCenterTX+", z: "+zCenterTX);
		System.out.println(">>> RX Node  -  x: "+xCenterRX+", y: "+yCenterRX+", z: "+zCenterRX);
		System.out.println();


		if(rejectingReceiver && absorbentReceiver){
			System.out.println();
			System.out.println();
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println(">>>  ||  ** WARNING **  You have defined both absorbentReceiver AND rejectingReceiver!  ||");
			System.out.println(">>>  ||   ++++++++++++++   REJECTING RECEIVER RULE WILL BE APPLIED!   ++++++++++++++    ||");
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println();
			System.out.println();
		}


		System.out.println(">>> SIMULATION ELEMENTS DISPOSED! ");			
		System.out.println();	


	}


	public float[] getPositionForCarrier(float vx,float vy,float vz){
		float[] angles=new float[2];		
		float module=(float) Math.sqrt(vx*vx+vy*vy+vz*vz);			
		angles[0]=0;	//fi
		angles[1]=0;	//theta	
		if (module!=0){
			float thetaArg=vz/module;
			if(thetaArg>1) {
				thetaArg=1;
			}else if(thetaArg<-1){
				thetaArg=-1;
			}

			angles[1]=(float) Math.acos(thetaArg);
		}
		if (Double.isNaN(vy)||Double.isNaN(vx)){
			angles[0]=0;
		}else{
			angles[0]=(float) Math.atan2(vy, vx);
		}		

		return angles;		
	}

	public void move(List<NanoObj> nanoList){

		if (nanoList.size()>0){			

			try{
				threadingStrategy.move(nanoList);				
			}catch (Exception e){
				e.printStackTrace();
			}

		}
	}
}