package simulator;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import MainLibrary.AbstractDomain;
import MainLibrary.LigandObserver;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.nIdle;
import MainLibrary.NoMotion;


public class HUVEC extends NodeObj{

	public boolean hasTransmit=false;
	public boolean hasSwitch=false;
	private long firstContact=-1;		

	public HUVEC(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);

		// You can set the internal state here!
		setState(new nIdle(this));
		getState().setExecutionTime(0);	
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
	}

	@Override
	public void idle() {
		processInfo();
	}

	@Override
	public void decode() {
		setState(new nIdle(this));
	}

	@Override
	public void encode() {				
		setState(new nIdle(this)); 
		getState().setExecutionTime(0);	
	}

	@Override
	public void searchNeighbor() {

	}

	@Override
	public void transmit() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(0);		
	}



	public Map<Object, Object> specializedExport() {
		Map<Object, Object> m=new HashMap<Object,Object>();
		m.put("Type", "Node");		
		m.put("firstContact", firstContact);
		m.putAll(getState().export());
		return m;
	}


	public void createReceptor(int n){
		Receptor r = null;

		float theta=0;
		float fi=0;

		for (int i=0;i<n;i++){
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];

			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));				

			r=new CD154Receptor(this, false, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			receptorSetup(r); 				
		}		
	}

	@Override
	public void receptorSetup(Receptor r) {		
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	}	

	@Override
	public void setMotionStrategy() {
		this.motionStrategy=NoMotion.getInstance();
	}

	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}

	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	public void processInfo() {
		if(getCollisionCount()>0 && firstContact<0){
			firstContact=getManager().getCurrentTime();
		}		
	}	

	public String assignName(){		
		return "HUVEC";		
	}

	@Override
	public void createSingleReceptor(float[] center, float radius, short receptorRadius, int receptorType) {
		// TODO Auto-generated method stub
	}
}
