package simulator;

import java.util.List;

import MainLibrary.AbstractDomain;
import MainLibrary.LivingObjObserver;
import MainLibrary.NanoObj;
import MainLibrary.NodeObj;

public class LivingBloodObjObserver extends LivingObjObserver{
	
	private boolean initialized=false;
	
	public long initialization_red=0;
	public long initialization_mac=0;
	public long initialization_plat=0;
	
	public float ratio_red=0;
	public float ratio_mac=0;
	public float ratio_plat=0;
	
	public long red=0;
	public long mac=0;
	public long plat=0;
	
	
	public LivingBloodObjObserver(AbstractDomain d) {
		super(d);
	}	
	
	public long[] refreshBloodCellsCounter(){
		
		red=0;
		mac=0;
		plat=0;
		
		List<NanoObj> nodeList=getDomain().getAllDisposedObj();		
		int sss=nodeList.size();
		
		for(int i=0;i<sss;i++){
			if(isRedBloodCell(nodeList.get(i))){
				red++;
			}else if(isWhiteBloodCell(nodeList.get(i))){
				mac++;
			}else if(isPlateletCell(nodeList.get(i))){
				plat++;
			}
		}		

		if(!initialized){
			
			initialization_red=red;
			initialization_mac=mac;
			initialization_plat=plat;
			
			initialized=true;
		}
		
		long[] results={red,mac,plat};		
		return results;
	}
	
	public float[] getOriginalRatio(){		
		float[] results={ratio_red,ratio_mac,ratio_plat};			
		return results;		
	}
	
	public long[] getOriginalCount(){		
		long[] results={initialization_red,initialization_mac,initialization_plat};			
		return results;		
	}
	
	public long[] getCellCount(){		
		long[] results={red,mac,plat};		
		return results;		
	}
	
	public synchronized void addNode(NodeObj n){
		if(isRedBloodCell(n)){
			red++;
		}else if(isWhiteBloodCell(n)){
			mac++;
		}else if(isPlateletCell(n)){
			plat++;
		}
		
		super.addNode(n);
	}
	
	public synchronized void removeNode(NodeObj n){
		
		if(isRedBloodCell(n)){
			red--;
		}else if(isWhiteBloodCell(n)){
			mac--;
		}else if(isPlateletCell(n)){
			plat--;
		}
		super.removeNode(n);
	}

	

	public boolean isRedBloodCell(NanoObj n){
		boolean isMatch=false;
		
		if(n instanceof RedBloodCell){
			isMatch=true;
		}		
		return isMatch;
	}

	public boolean isWhiteBloodCell(NanoObj n){
		boolean isMatch=false;

		if(n instanceof Macrophage){
			isMatch=true;
		}		
		return isMatch;
	}

	public boolean isPlateletCell(NanoObj n){
		boolean isMatch=false;

		if(n instanceof Platelet || n instanceof PlateletVesselTX || n instanceof PlateletVessel){
			isMatch=true;
		}		
		return isMatch;
	}
}
