package simulator;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;

import MainLibrary.AbstractDomain;
import engine.Engine3D;
import MainLibrary.CarrierObj;
import MainLibrary.LigandObserver;
import MainLibrary.LinearMotion;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.UniversalReceptor;
import MainLibrary.nIdle;


public class Macrophage extends NodeObj{
	
	private long firstContact=-1;
	public String debugName="";
	
	public Macrophage(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);
				
		// You can set the internal state here!
		setState(new nIdle(this));
		getState().setExecutionTime(1000);	
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
		setColor(Color.white);			
		
	}
	
	@Override
	public void idle() {
		processInfo();
	}
	
	@Override
	public void decode() {
		setState(new nIdle(this));
	}
	
	@Override
	public void encode() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(1000);	
	}

	@Override
	public void searchNeighbor() {
		
	}

	@Override
	public void transmit() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(1000);		
	}
	
	public void createCarriers(int n){
		while (n>0){				
			CD40LCarrier cc=new CD40LCarrier(getInnerDomain(), this, getCenter()[0], getCenter()[1], getCenter()[2],1);			
			cc.setMotionStrategy(BrownianMotion.getInstance());
			n--;
		}			
	}
	
	// FIXME - 21/02/2014 - If it works, it should be moved on the NodeObj, because could be useful for all Nanoobjs 
	// Verifies the reception of the ligand near receptors
	public void checkIncomingCarrier(CarrierObj c){		
		boolean catched=false;
		float x,  y,  z,  r;
	
		float[] coords=null;	

		
		// FIXME DEBUG - Inserted to relocate the carrier on the surface of the node
		float dt=(float) getCollisionTime(c);
		float[] velocity=c.getVelocityComponent();
		// Carrier Coords on the Node External Surface
		x=c.getCenter()[0]-velocity[0]*dt;
		y=c.getCenter()[1]-velocity[1]*dt;
		z=c.getCenter()[2]-velocity[2]*dt;	
		r=(float) c.getRadius();
		// END of FIXME


		// Gets the receptorList for this impact point!		
		ArrayList<UniversalReceptor> receptorList=receptorService.getQuadrantList(receptorService.getQuadrantForCarrier(this,c));
		//---------------------------------------------
		

		if (c.getDomain()==getInnerDomain()){		
			// Internal Domain!			
			for (int i=0;i<receptorList.size() && !catched;i++){				
				if((receptorList.get(i).isFree() && !(receptorList.get(i).canDownload()))){
					coords=receptorList.get(i).getAbsoluteCoords();
					float dim=receptorList.get(i).radius;							
					float dx = (coords[0]-x), dy = (coords[1]-y), dz= (coords[2]-z);
					float distance =(float) Math.sqrt(dx*dx+dy*dy+dz*dz);	
					
					if (distance<=((dim+r)) && (receptorList.get(i).checkCarrierCompatibility(c))){			
						catched=true;						
						receptorList.get(i).configureForUpload(c);	
						System.out.println("NodeObj.checkIncomingCarrier() - CARRIER DOWNLOAD!");						
					}
				}
			}
			
			if(!catched){	
				getInnerDomain().getCollisionStrategy().relocateInnerObjs(c,getInnerDomain());					
			}
			
		} else {		
			// External Domain!
			for (int i=0;i<receptorList.size() && !catched;i++){
				if((receptorList.get(i).isFree() && receptorList.get(i).canDownload())){					
					coords=receptorList.get(i).getAbsoluteCoords();
					float dim=receptorList.get(i).radius;								
					float dx = (coords[0]-x), dy = (coords[1]-y), dz= (coords[2]-z);
					float distance =(float) Math.sqrt(dx*dx+dy*dy+dz*dz);									
					if (distance<((dim+r)) && (receptorList.get(i).checkCarrierCompatibility(c))){							
						catched=true;
						receptorList.get(i).configureForDownload(c);							
					}
				}
			}
			
			if(!catched){	
				// NO available receptors. The carrier will be relocated!
				getDomain().getMobilityModel().relocateObjs(c,this);
			}
		} 
	}
	
		
	@Override
	public Map<Object, Object> specializedExport() {

		Map<Object, Object> m=new HashMap<Object,Object>();
		if(informationCount>0 ){
			m.put("Typev", "MacrophageVessel");

			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		

			m.put("IDv", "Macrophage-"+ID);	
			m.put("Xv", center[0]-domainCenter[0]);
			m.put("Yv", center[1]-domainCenter[1]);
			m.put("Zv", center[2]-domainCenter[2]);
			m.put("TimeStepv", ""+getManager().getCurrentTime()+"");	
			m.put("Assimilationsv", informationCount);	
			m.put("Actionv", "assimilation");		
			m.putAll(getState().export());
		}		
		
		return m;
	}	
	
	@Override
	public void createSingleReceptor(float[] cdCenter, float radius, short receptorRadius, int receptorType) {
		float[] angles=convertSphericalAngles(cdCenter[0],cdCenter[1],cdCenter[2], radius);
		float theta=angles[1];
		float fi=angles[0];

		
		Receptor r=new CD40Receptor(this, true, cdCenter, theta, fi, receptorRadius, receptorType, this.eventHandler);
		receptorList.add(r);
		receptorSetup(r); 

		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		*/		
	}
	
	public void createReceptor(int n){		
		/* TO BE REACTIVATED!!! */
		
		Receptor r = null;
		
		float theta=0;
		float fi=0;				
		
		for (int i=0;i<n;i++){		
			
			theta= (getDomain().getMobilityModel().getRandomAngles()[0]);
			fi= (getDomain().getMobilityModel().getRandomAngles()[1]);					
			
			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new CD40Receptor(this, true, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			receptorSetup(r); 				
			
			receptorService.addReceptor(r);
			
			//drawSmallPoint(getDomain(),r,fi,theta);
		}
	}	
	
	public void createReceptorORIGINAL(int n){
		Receptor r = null;
		
		float theta=0;
		float fi=0;				
		
		for (int i=0;i<n;i++){		
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];		
			
			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new CD40Receptor(this, true, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			receptorSetup(r); 		
		}
	}	
	
	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	}
	
	@Override
	public void setMotionStrategy() {		
		this.motionStrategy=BrownianMotionBloodStream.getInstance();
		//this.motionStrategy=LinearMotion.getInstance();	
	}
	
	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}
		
	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	@Override
	public void processInfo() {
		if(getCollisionCount()>0 && firstContact<0){
			firstContact=getManager().getCurrentTime();				
		}		
	}
	
	public String assignName(){		
		return "Macrophage";		
	}

	
	// Draw the receptor in the 3D window
	public void drawSmallPoint(AbstractDomain parent, UniversalReceptor r, float fi, float theta){

		if(getManager().is3DEnable==true){
			float[] rcenter = new float[3];
			
			rcenter[0]=r.cx+getCenter()[0];
			rcenter[1]=r.cy+getCenter()[1];
			rcenter[2]=r.cz+getCenter()[2];
			
			
			SimpleVector v0=new SimpleVector(rcenter);
			Object3D obj3D=null;
			obj3D = Primitives.getBox(100, 1);	
			//obj3D = Primitives.getBox(getRadius(), 1);	
			obj3D.setCulling(false);		
			obj3D.setAdditionalColor(Color.WHITE);		
			obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);		
			obj3D.build();
			obj3D.translate(v0);
			Engine3D engine3d=(Engine3D)parent.getManager().engine;
			engine3d.world.addObject(obj3D);
			obj3D.compileAndStrip();	
			
			short quad=receptorService.getQuadrant(fi, theta);
			
			switch (quad){
				case 0:
					obj3D.setAdditionalColor(Color.WHITE);
				break;
				case 1:
					obj3D.setAdditionalColor(Color.black);
				break;
				case 2:
					obj3D.setAdditionalColor(Color.cyan);
				break;
				case 3:
					obj3D.setAdditionalColor(Color.DARK_GRAY);
				break;
				case 4:
					obj3D.setAdditionalColor(Color.green);
				break;
				case 5:
					obj3D.setAdditionalColor(Color.magenta);
				break;
				case 6:
					obj3D.setAdditionalColor(Color.orange);
				break;
				case 7:
					obj3D.setAdditionalColor(Color.yellow);
				break;				
				
			}
		}
	}
	
	/*
	public void setCenter(float x, float y, float z){
		
		float[] cc=getCenter();
		
		if(cc[0]!=x || cc[1]!=y || cc[2] != z){
			System.out.println("CAMBIATO!");
		}		
		super.setCenter(x, y, z);	
		
	}*/
}