package simulator;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import MainLibrary.AbstractDomain;
import MainLibrary.AbstractThreadingStrategy;
import MainLibrary.CarrierObj;
import MainLibrary.CheckNanoObjCollisionThreadLatch;
import MainLibrary.HeapSorter;
import MainLibrary.MobilityModel;
import MainLibrary.NanoObj;


public class NewFlowThreadingStrategy extends AbstractThreadingStrategy {
	
	private final int maxThreads = Runtime.getRuntime().availableProcessors();
	private ExecutorService executorThreads = Executors.newFixedThreadPool(maxThreads);	 
	
	public NewFlowThreadingStrategy(MobilityModel m) {
		super(m);		
	}

	/* (non-Javadoc)
	 * @see MainLibrary.AbstractThreadingMoveStrategy#move(java.util.List)
	 */
	@Override
	public void move(List<NanoObj> nanoList) {
		
		
		//long start = System.nanoTime();		
		
		
		int div=maxThreads*4;
		int size=nanoList.size();		
		CountDownLatch latch=new CountDownLatch(1);

		if(size>div){
			int intPart=size/div;	

			if (intPart>0) {
				latch=new CountDownLatch(div);
			}				

			for(int i=0;i<div-1;i++){
				//executorThreads.execute(new FlowModelMoveThread(nanoList.subList(intPart*i, intPart*(i+1)),latch));				// original 1 timestep
				executorThreads.execute(new FlowModelMoveThread2Timesteps(nanoList.subList(intPart*i, intPart*(i+1)),latch,m));		// 2 timesteps
			}
			//executorThreads.execute(new FlowModelMoveThread(nanoList.subList(intPart*(div-1), size),latch));			// original 1 timestep
			executorThreads.execute(new FlowModelMoveThread2Timesteps(nanoList.subList(intPart*(div-1), size),latch,m));	// 2 timesteps

		}else if(size>0){
			//executorThreads.execute(new FlowModelMoveThread(nanoList,latch));				// original 1 timestep
			executorThreads.execute(new FlowModelMoveThread2Timesteps(nanoList,latch,m));		// 2 timesteps
			
		}		

		try {
			latch.await();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		
		/*
		long stop = System.nanoTime();			
		System.out.println(""+(stop-start));
		*/
		
		/*   // OLD
		int mThreads=maxThreads*4;			
		int size=nanoList.size();		
		CountDownLatch latch=new CountDownLatch(1);		
		
		if(size>0){
			if(size>mThreads){
				int intPart=size/mThreads;	
				
				if (intPart>0) {
					latch=new CountDownLatch(mThreads);
				}				
				
				for(int i=0;i<mThreads-1;i++){
					executorThreads.execute(new FlowModelMoveThread(nanoList.subList(intPart*i, intPart*(i+1)),latch));			
				}
				executorThreads.execute(new FlowModelMoveThread(nanoList.subList(intPart*(mThreads-1), size),latch));
				
			}else {
				executorThreads.execute(new FlowModelMoveThread(nanoList,latch));				
			}
		}
		
		try {
			latch.await();
		} catch (InterruptedException e) {			
			e.printStackTrace();
		}
		*/			
	}

	@Override
	public void checkCollision(List<NanoObj> objsToCheck) {
		AbstractDomain activeDomain=null;
		
		int objToCheck_size=objsToCheck.size();
		
		if (objToCheck_size>0){
			activeDomain=objsToCheck.get(0).getDomain();
			
			// Reorder first!!
			HeapSorter h=new HeapSorter();
			h.sort(objsToCheck);
		
			//Check Collision!!
			try {				
				
				int threadCount=maxThreads*4;				
				int subList=objToCheck_size/threadCount;
				int residue=objToCheck_size%threadCount;
				
				CountDownLatch latch=new CountDownLatch(1);
				if (subList>0) {
					latch=new CountDownLatch(threadCount);
								
					for (int i=0;i<threadCount-1;i++){	
						executorThreads.execute(new CheckNanoObjCollisionThreadLatch(i*subList, (i+1)*subList,objsToCheck,m,activeDomain, latch));								
					}					
				}
				
				if (residue==0) {	
					executorThreads.execute(new CheckNanoObjCollisionThreadLatch((threadCount-1)*subList, (threadCount)*subList,objsToCheck,m,activeDomain, latch));			
					
				} else {
					executorThreads.execute(new CheckNanoObjCollisionThreadLatch((threadCount-1)*subList, objToCheck_size,objsToCheck,m,activeDomain, latch));
				}				
				latch.await();	
				activeDomain.joinAllCollisionsLists();
				
			} catch (Exception e1) {				
				e1.printStackTrace();
			}	
			
			
			try {
				// Check the collision with other domains!
				if (objToCheck_size>0){	
										
					List<AbstractDomain> domainToCheck=m.reorderDomain_byHeap(activeDomain);
					//List<AbstractDomain> domainToCheck=m.reorderDomain(activeDomain);
				
					/*
					// ==============================================
					// PROVO IL MULTITHREAD - (ok ma da ottimizzare sulle chiamate quando non ci sono domini da controllare)
					// ==============================================
					try {				
						
						int threadCount=maxThreads*4;				
						int subList=domainToCheck.size()/threadCount;
						int residue=domainToCheck.size()%threadCount;
						
						CountDownLatch latch=new CountDownLatch(1);
						if (subList>0) {
							latch=new CountDownLatch(threadCount);
						}
									
						if (subList>0) {					
							for (int i=0;i<threadCount-1;i++){	
								executorThreads.execute(new CheckDomainCollisionThread(i*subList, (i+1)*subList,domainToCheck,m,activeDomain,objsToCheck, latch));								
							}					
						}
						
						if (residue==0) {	
							executorThreads.execute(new CheckDomainCollisionThread((threadCount-1)*subList, (threadCount)*subList,domainToCheck,m,activeDomain,objsToCheck, latch));			
							
						} else {
							executorThreads.execute(new CheckDomainCollisionThread((threadCount-1)*subList, domainToCheck.size(),domainToCheck,m,activeDomain,objsToCheck, latch));
						}				
						latch.await();	
						//activeDomain.joinAllCollisionsLists();
						
					} catch (Exception e1) {				
						e1.printStackTrace();
					}*/
					
					
					//long start = System.nanoTime();
					// ==============================================
					// PROVO IL MULTITHREAD - (come sopra, ma OTTIMIZZO QUANDO NON CI SONO DOMINI DA CONTROLLARE)
					// ==============================================
					try {				
						
						int d_size=domainToCheck.size();
						
						//FIXME PROVA A CONTROLLARE SOLO LE CARRIER
						List<CarrierObj> carrierList=activeDomain.getLivingObjObserver().activeCarrierList;
						
						
						//if(d_size>0){
						if(d_size>0 && carrierList.size()>0){
														
							int threadCount=maxThreads*4;				
							int subList=d_size/threadCount;
							int residue=d_size%threadCount;
							
							CountDownLatch latch=new CountDownLatch(1);
							
							if (subList>0) {
								latch=new CountDownLatch(threadCount);
												
								for (int i=0;i<threadCount-1;i++){	
									//executorThreads.execute(new CheckDomainCollisionThread(i*subList, (i+1)*subList,domainToCheck,m,activeDomain,objsToCheck, latch));
									executorThreads.execute(new CheckDomainCollisionThread(i*subList, (i+1)*subList,domainToCheck,m,activeDomain,carrierList, latch));								
								}					
							}
							
							if (residue==0) {	
								//executorThreads.execute(new CheckDomainCollisionThread((threadCount-1)*subList, (threadCount)*subList,domainToCheck,m,activeDomain,objsToCheck, latch));
								executorThreads.execute(new CheckDomainCollisionThread((threadCount-1)*subList, (threadCount)*subList,domainToCheck,m,activeDomain,carrierList, latch));			
								
							} else {
								//executorThreads.execute(new CheckDomainCollisionThread((threadCount-1)*subList, d_size,domainToCheck,m,activeDomain,objsToCheck, latch));
								executorThreads.execute(new CheckDomainCollisionThread((threadCount-1)*subList, d_size,domainToCheck,m,activeDomain,carrierList, latch));
							}				
							latch.await();	
							//activeDomain.joinAllCollisionsLists();
						}
						
						
						
					} catch (Exception e1) {				
						e1.printStackTrace();
					}
					
					
					/*
					long stop = System.nanoTime();			
					System.out.println(""+(stop-start));
					*/
					
					
					
					//=======================================
					//Check Collision!!		// ORIGINAL NO MULTITHREAD
					/*
					boolean isFinish=false;					
					for (int i=0;i<domainToCheck.size();i++){
						isFinish=false;
						try{
							for (int j=0;j<objSize && !isFinish;j++){
								if(objsToCheck.get(j).getDomain()==activeDomain){									
									float Dplus=domainToCheck.get(i).getDistance()+domainToCheck.get(i).getBoundingRadius();
									float Dminus=domainToCheck.get(i).getDistance()-domainToCheck.get(i).getBoundingRadius();
									float Nplus=objsToCheck.get(j).getAbsDistancePlusRadius();
									float Nminus=objsToCheck.get(j).getAbsDistanceMinusRadius();																		

									if(Dminus>Nplus) { 
										// They are not overlapped, neither the following objects!	
										isFinish=true;
										
									} else if((Dminus<=Nminus && Nminus<=Dplus) || (Dminus>=Nminus && Nplus>=Dminus) || (Dplus<=Nplus && Nminus<=Dplus)) { 
							
										// probable collision!!!							
										try{
											domainToCheck.get(i).getCollisionStrategy().checkExternalCollisionBound(domainToCheck.get(i), objsToCheck.get(j));

										}catch (Exception e){											
											e.printStackTrace();						
										}
									} 
								}
							}
						}catch (Exception e){
							e.printStackTrace();
						}
					}*/
					
				}
				//stop = System.currentTimeMillis();
				//System.out.println("COLLISION - DOMAIN CHECK in: "+(stop-start));
				//int oi=0;
				
			} catch (Exception e1) {			
				e1.printStackTrace();
			}
			
			
			
			//long start = System.nanoTime();
			
			/*
			// OK WITHOUT MULTITHREAD!
			try {
				// Check the collisions with their domain bounds!	
				//int objCount=objsToCheck.size();
				//if (objCount>0){	
				if(objToCheck_size>0){
					AbstractDomain domain=activeDomain;
					if ((domain!=null)){						
						try{
							domain.getCollisionStrategy().checkInternalCollisionBound(objsToCheck);
						}catch (Exception e){
							e.printStackTrace();						
						}
					}	
				}
			} catch (Exception e) {				
				e.printStackTrace();
			}
			*/
			
			/**/
			// OK WITH MULTITHREAD!!
			try {
				// Check the collisions with their domain bounds!	
				//int objCount=objsToCheck.size();
				//if (objCount>0){	
				if(objToCheck_size>0){
					AbstractDomain domain=activeDomain;
					if ((domain!=null)){

						int threadCount=maxThreads*4;				
						int subList=objToCheck_size/threadCount;
						int residue=objToCheck_size%threadCount;

						CountDownLatch latch=new CountDownLatch(1);

						if (subList>0) {
							latch=new CountDownLatch(threadCount);

							for (int i=0;i<threadCount-1;i++){	
								executorThreads.execute(new CheckInnerDomainCollisionThread(i*subList, (i+1)*subList,activeDomain,objsToCheck, latch));								
							}					
						}

						if (residue==0) {	
							executorThreads.execute(new CheckInnerDomainCollisionThread((threadCount-1)*subList, (threadCount)*subList,activeDomain,objsToCheck, latch));			

						} else {
							executorThreads.execute(new CheckInnerDomainCollisionThread((threadCount-1)*subList, objToCheck_size,activeDomain,objsToCheck, latch));
						}				
						latch.await();	
						//activeDomain.joinAllCollisionsLists();

					}	
				}
			} catch (Exception e) {				
				e.printStackTrace();
			}
			
			
			/*
			long stop = System.nanoTime();			
			System.out.println(""+(stop-start));
			*/
		}
	}
}
