package simulator;

import java.io.IOException;
import java.io.PrintStream;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.Executors;

//import Wrappers.OutputWrapperBloodVessel;

import com.mysql.jdbc.Connection;
import com.mysql.jdbc.Statement;

import MainLibrary.Configurator2;
import MainLibrary.Manager;
import MainLibrary.OutputStrategy;


/** This class stores all the relevant objects into a database file!   */


public class OutputDBMySQL extends OutputStrategy {

	//public List<OutputWrapperResults> wrapperList = new ArrayList<OutputWrapperResults>();	
	CopyOnWriteArrayList<OutputWrapperResults> wrapperList=new CopyOnWriteArrayList<OutputWrapperResults>(new ArrayList<OutputWrapperResults>());

	
	public List<OutputWrapperResults> wrapperListForRecovery = new ArrayList<OutputWrapperResults>();	
	
	private static OutputDBMySQL instance=null;
	private static String simulationID=null;
	private String address="localhost";
	private String port="3306";
	private String user="";
	private String password="";
	private String db="";
	private String disaster_recovery_db="";
	private String url="";
	private String description="N/A";	
	private int storingFactor=1;
	//public static String extraInfo="";
	private long nextKeepAlive=0;
	private String hostname="N/A";
	long keepAlive=12*3600*1000;
	



	public static OutputDBMySQL getInstance(){
		if (instance==null) instance=new OutputDBMySQL();		
		return instance;
	}

	private OutputDBMySQL(){
		//setClassType("OutputDBMySQL");
		simulationID=getSimulationID();
		address=Configurator2.readCustomStringConfigurations("config.xml", "Database", "address");		
		port=Configurator2.readCustomStringConfigurations("config.xml", "Database", "port");
		user=Configurator2.readCustomStringConfigurations("config.xml", "Database", "user");		
		password=Configurator2.readCustomStringConfigurations("config.xml", "Database", "password");

		
		db=Configurator2.readCustomStringConfigurations("config.xml", "Database", "db");	
		description=Configurator2.readCustomStringConfigurations("config.xml", "System", "description");	
		storingFactor=Integer.parseInt(Configurator2.readCustomStringConfigurations("config.xml", "Database", "storingFactorTime"));	
		
		try{
			disaster_recovery_db=Configurator2.readCustomStringConfigurations("config.xml", "Database", "disaster-recovery-db");	// url of php script
			keepAlive=Long.parseLong(Configurator2.readCustomStringConfigurations("config.xml", "Database", "keep-alive-db"));		// minutes
			keepAlive=keepAlive*60*1000;			
			refreshKeepAlive();		
			
			System.out.println();
			System.out.println("[.] THE BACKUP DATABASE FOR DISASTER RECOVERY IS DEFINED ON: "+disaster_recovery_db);
			System.out.println();
			
			hostname=InetAddress.getLocalHost().getHostName();
			
		}catch(Exception e){
			System.out.println();
			System.out.println("[!] THE BACKUP DATABASE FOR DISASTER RECOVERY IS NOT DEFINED!!");
			System.out.println();
		}
		//extraInfo=Configurator2.additionalInfo;
		
		if(storingFactor<1) storingFactor=1;

		url="jdbc:mysql://"+address+":"+port+"/"+db;


		System.out.println("========================================");
		System.out.println("OUTPUT DATABASE: "+url);					
		System.out.println("========================================");
	}	

	public void refreshKeepAlive(){		
		nextKeepAlive=System.currentTimeMillis()+keepAlive;
	}
	
	
	
	
	@Override
	public void export(Map<Object, Object> results) {	
		
		if(results.get("Type")=="TCPLike_BER-TX Node"){		

			String ID=(String) results.get("ID");
			String x=(String) results.get("X").toString();
			String y=(String) results.get("Y").toString();
			String z=(String) results.get("Z").toString();
			
			
			
			int total=(Integer) results.get("ASSIMILATION_TOTAL");
			for(int i=0;i<total;i++){	
				
				String timeStep=(String) results.get("ASSIMILATION_TimeStep_"+i).toString();
				String value=(String) results.get("ASSIMILATION_Value_"+i).toString();
				String action=(String) results.get("ASSIMILATION_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			total=(Integer) results.get("sendACK_TOTAL");
			for(int i=0;i<total;i++){	
				
				String timeStep=(String) results.get("sendACK_TimeStep_"+i).toString();
				String value=(String) results.get("sendACK_Value_"+i).toString();
				String action=(String) results.get("sendACK_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
						
			

		} else if(results.get("Type")=="Carrier"){		

			String ID=(String) results.get("ID");
			String x=(String) results.get("X").toString();
			String y=(String) results.get("Y").toString();
			String z=(String) results.get("Z").toString();


			String timeStep=(String) results.get("timestep").toString();
			String action="angles "+(String) results.get("angles").toString();
			String value="0";

			OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
			wrapperList.add(o);			

		} else if(results.get("Type")=="CarrierCollision"){				
			
			String ID=(String) results.get("ID");
			String value=(String) results.get("cCollision").toString();
			String timeStep=(String) results.get("timestep").toString();			
			String action="cCollision";

			OutputWrapperResults o=new OutputWrapperResults(ID, "0","0","0", timeStep, action, value);				
			wrapperList.add(o);			

		}

		else if(results.get("Type")=="TCPLike_BER-RX Node"){		

			String ID=(String) results.get("ID");
			String x=(String) results.get("X").toString();
			String y=(String) results.get("Y").toString();
			String z=(String) results.get("Z").toString();
			
			
			
			int total=(Integer) results.get("START_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("START_TimeStep_"+i).toString();
				String value=(String) results.get("START_Value_"+i).toString();
				String action=(String) results.get("START_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			
			total=(Integer) results.get("BER_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("BER_TimeStep_"+i).toString();
				String value=(String) results.get("BER_Value_"+i).toString();
				String action=(String) results.get("BER_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			
			total=(Integer) results.get("RTT_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("RTT_TimeStep_"+i).toString();
				String value=(String) results.get("RTT_Value_"+i).toString();
				String action=(String) results.get("RTT_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}	
						
			

		} else if(results.get("Type")=="TCPLike-TX-Node"){		

			String ID=(String) results.get("ID");
			String x=(String) results.get("X").toString();
			String y=(String) results.get("Y").toString();
			String z=(String) results.get("Z").toString();
			
			if (results.get("CRON_TimeStep")!=null){				
				
				String timeStep=(String) results.get("CRON_TimeStep").toString();
				String value=(String) results.get("CRON_Value").toString();
				String action=(String) results.get("CRON_Action").toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			int total=(Integer) results.get("START_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("START_TimeStep_"+i).toString();
				String value=(String) results.get("START_Value_"+i).toString();
				String action=(String) results.get("START_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			total=(Integer) results.get("CUT_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("CUT_TimeStep_"+i).toString();
				String value=(String) results.get("CUT_Value_"+i).toString();
				String action=(String) results.get("CUT_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			total=(Integer) results.get("STOP_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("STOP_TimeStep_"+i).toString();
				String value=(String) results.get("STOP_Value_"+i).toString();
				String action=(String) results.get("STOP_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			total=(Integer) results.get("START_Release_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("START_Release_TimeStep_"+i).toString();
				String value=(String) results.get("START_Release_Value_"+i).toString();
				String action=(String) results.get("START_Release_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			
			total=(Integer) results.get("STOP_Release_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("STOP_Release_TimeStep_"+i).toString();
				String value=(String) results.get("STOP_Release_Value_"+i).toString();
				String action=(String) results.get("STOP_Release_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			
			total=(Integer) results.get("CUT_Release_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("CUT_Release_TimeStep_"+i).toString();
				String value=(String) results.get("CUT_Release_Value_"+i).toString();
				String action=(String) results.get("CUT_Release_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			
			total=(Integer) results.get("STOP_TotalCarrier_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("STOP_TotalCarrier_TimeStep_"+i).toString();
				String value=(String) results.get("STOP_TotalCarrier_Value_"+i).toString();
				String action=(String) results.get("STOP_TotalCarrier_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
						
			
		}else if(results.get("Type")=="Simple-RX-Node"){		

			String ID=(String) results.get("ID");
			String x=(String) results.get("X").toString();
			String y=(String) results.get("Y").toString();
			String z=(String) results.get("Z").toString();
			
			
			if (results.get("packetTypes_TimeStep")!=null){				
				
				String timeStep=(String) results.get("packetTypes_TimeStep").toString();
				String value=(String) results.get("packetTypes_Value").toString();
				String action=(String) results.get("packetTypes_Action").toString();

				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			

			}

			
			
			if (results.get("CRON_TimeStep")!=null){				
				
				String timeStep=(String) results.get("CRON_TimeStep").toString();
				String value=(String) results.get("CRON_Value").toString();
				String action=(String) results.get("CRON_Action").toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			
			if (results.get("SERVED_TimeStep")!=null){				
				
				String timeStep=(String) results.get("SERVED_TimeStep").toString();
				String value=(String) results.get("SERVED_Value").toString();
				String action=(String) results.get("SERVED_Action").toString();

				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			

			}

			if (results.get("BUSY_TimeStep")!=null){				

				String timeStep=(String) results.get("BUSY_TimeStep").toString();
				String value=(String) results.get("BUSY_Value").toString();
				String action=(String) results.get("BUSY_Action").toString();

				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			

			}

			if (results.get("MISSED_TimeStep")!=null){				

				String timeStep=(String) results.get("MISSED_TimeStep").toString();
				String value=(String) results.get("MISSED_Value").toString();
				String action=(String) results.get("MISSED_Action").toString();

				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			

			}		
		}else if(results.get("Type")=="TCPLike-RX-Node"){		

			String ID=(String) results.get("ID");
			String x=(String) results.get("X").toString();
			String y=(String) results.get("Y").toString();
			String z=(String) results.get("Z").toString();
			
			
			if (results.get("packetTypes_TimeStep")!=null){				
				
				String timeStep=(String) results.get("packetTypes_TimeStep").toString();
				String value=(String) results.get("packetTypes_Value").toString();
				String action=(String) results.get("packetTypes_Action").toString();

				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			

			}

			
			
			if (results.get("CRON_TimeStep")!=null){				
				
				String timeStep=(String) results.get("CRON_TimeStep").toString();
				String value=(String) results.get("CRON_Value").toString();
				String action=(String) results.get("CRON_Action").toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			
			if (results.get("SERVED_TimeStep")!=null){				
				
				String timeStep=(String) results.get("SERVED_TimeStep").toString();
				String value=(String) results.get("SERVED_Value").toString();
				String action=(String) results.get("SERVED_Action").toString();

				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			

			}

			if (results.get("BUSY_TimeStep")!=null){				

				String timeStep=(String) results.get("BUSY_TimeStep").toString();
				String value=(String) results.get("BUSY_Value").toString();
				String action=(String) results.get("BUSY_Action").toString();

				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			

			}

			if (results.get("MISSED_TimeStep")!=null){				

				String timeStep=(String) results.get("MISSED_TimeStep").toString();
				String value=(String) results.get("MISSED_Value").toString();
				String action=(String) results.get("MISSED_Action").toString();

				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			

			}			 
			
			
			int total=(Integer) results.get("START_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("START_TimeStep_"+i).toString();
				String value=(String) results.get("START_Value_"+i).toString();
				String action=(String) results.get("START_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			total=(Integer) results.get("CUT_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("CUT_TimeStep_"+i).toString();
				String value=(String) results.get("CUT_Value_"+i).toString();
				String action=(String) results.get("CUT_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			total=(Integer) results.get("STOP_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("STOP_TimeStep_"+i).toString();
				String value=(String) results.get("STOP_Value_"+i).toString();
				String action=(String) results.get("STOP_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			
			
			total=(Integer) results.get("STOP_Assimilations_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("STOP_Assimilations_TimeStep_"+i).toString();
				String value=(String) results.get("STOP_Assimilations_Value_"+i).toString();
				String action=(String) results.get("STOP_Assimilations_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			
			total=(Integer) results.get("CUT_Assimilations_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("CUT_Assimilations_TimeStep_"+i).toString();
				String value=(String) results.get("CUT_Assimilations_Value_"+i).toString();
				String action=(String) results.get("CUT_Assimilations_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}	
			
			total=(Integer) results.get("RTT_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("RTT_TimeStep_"+i).toString();
				String value=(String) results.get("RTT_Value_"+i).toString();
				String action=(String) results.get("RTT_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}		
			
			total=(Integer) results.get("T_HALVE_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("T_HALVE_TimeStep_"+i).toString();
				String value=(String) results.get("T_HALVE_Value_"+i).toString();
				String action=(String) results.get("T_HALVE_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}			
			
			total=(Integer) results.get("N_HALVE_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("N_HALVE_TimeStep_"+i).toString();
				String value=(String) results.get("N_HALVE_Value_"+i).toString();
				String action=(String) results.get("N_HALVE_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			total=(Integer) results.get("T_PREV_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("T_PREV_TimeStep_"+i).toString();
				String value=(String) results.get("T_PREV_Value_"+i).toString();
				String action=(String) results.get("T_PREV_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			
			total=(Integer) results.get("N_EXP_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("N_EXP_TimeStep_"+i).toString();
				String value=(String) results.get("N_EXP_Value_"+i).toString();
				String action=(String) results.get("N_EXP_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			
			total=(Integer) results.get("a_TOTAL");
			for(int i=0;i<total;i++){				
							
				String timeStep=(String) results.get("a_TimeStep_"+i).toString();
				String value=(String) results.get("a_Value_"+i).toString();
				String action=(String) results.get("a_Action_"+i).toString();
				
				OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
				wrapperList.add(o);			
				 
			}
			
			

		}else if(results.get("Type")=="CYLINDER"){		

			String ID=(String) results.get("ID");
			String x=(String) results.get("X").toString();
			String y=(String) results.get("Y").toString();
			String z=(String) results.get("Z").toString();
			String timeStep=(String) results.get("TimeStep").toString();
			String value=(String) results.get("Value").toString();
			String action=(String) results.get("Action").toString();
			OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
			wrapperList.add(o);			
			
			
			/*
			String value2=(String) results.get("dCollision").toString();					
			String action2="dCollision";
			OutputWrapperResults o2=new OutputWrapperResults(ID, x,y,z, timeStep, action2, value2);				
			wrapperList.add(o2);	
			*/
				
			//---------------------------------------------
			
			int count=Integer.parseInt((String) results.get("carrierList"));

			for(int i=0;i<count;i++){
				String ID1=(String) results.get("Type_"+i);
				String x1=(String) results.get("X_"+i).toString();
				String y1=(String) results.get("Y_"+i).toString();
				String z1=(String) results.get("Z_"+i).toString();				
				
				OutputWrapperResults o1=new OutputWrapperResults(ID1, x1,y1,z1, timeStep, "", "");				
				wrapperList.add(o1);		
			}




		}else if(results.get("Type")=="CubeReceptor"){		

			String ID=(String) results.get("ID");
			String x=(String) results.get("X").toString();
			String y=(String) results.get("Y").toString();
			String z=(String) results.get("Z").toString();
			String timeStep=(String) results.get("TimeStep").toString();
			String value=(String) results.get("Assimilations").toString();
			String action=(String) results.get("Action").toString();
			OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
			wrapperList.add(o);			


		}else if(results.get("Typev")=="PlateletVessel"){		

			// TRACKING POSITION
			String ID=(String) results.get("ID_tracking");
			String x=(String) results.get("x").toString();
			String y=(String) results.get("y").toString();
			String z=(String) results.get("z").toString();
			String timeStep=(String) results.get("TimeStep_tracking").toString();
			String value="0";
			String action="tracking";				
			OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
			wrapperList.add(o);		



			// EMISSION PATTERN
			int count=(Integer) results.get("Results");

			for(int i=0;i<count;i++){
				String ID1=(String) results.get(i+"-IDv");
				String x1=(String) results.get(i+"-Xv").toString();
				String y1=(String) results.get(i+"-Yv").toString();
				String z1=(String) results.get(i+"-Zv").toString();
				String timeStep1=(String) results.get(i+"-TimeStepv").toString();
				String value1=(String) results.get(i+"-Emissionv").toString();
				String action1=(String) results.get(i+"-Actionv").toString();				
				OutputWrapperResults o1=new OutputWrapperResults(ID1, x1,y1,z1, timeStep1, action1, value1);				
				wrapperList.add(o1);		
			}



		}else if(results.get("Typev")=="MacrophageVessel"){

			String ID=(String) results.get("IDv").toString();
			String x=(String) results.get("Xv").toString();
			String y=(String) results.get("Yv").toString();
			String z=(String) results.get("Zv").toString();
			String timeStep=(String) results.get("TimeStepv").toString();
			String value=(String) results.get("Assimilationsv").toString();		
			String action=(String) results.get("Actionv").toString();	
			OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
			wrapperList.add(o);					
		}
		
		else if(results.get("Typev")=="RBC"){

			String ID=(String) results.get("IDv").toString();
			String x=(String) results.get("Xv").toString();
			String y=(String) results.get("Yv").toString();
			String z=(String) results.get("Zv").toString();
			String timeStep=(String) results.get("TimeStepv").toString();
			String value="0";		
			String action=(String) results.get("Actionv").toString();	
			OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
			wrapperList.add(o);					
		}
		
		else if(results.get("Typev")=="Platelet"){

			String ID=(String) results.get("IDv").toString();
			String x=(String) results.get("Xv").toString();
			String y=(String) results.get("Yv").toString();
			String z=(String) results.get("Zv").toString();
			String timeStep=(String) results.get("TimeStepv").toString();
			String value="0";	
			String action=(String) results.get("Actionv").toString();	
			OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
			wrapperList.add(o);					
		}

		else if(results.get("Typev")=="CalciumCellRX"){

			String ID=(String) results.get("IDv").toString();
			String x=(String) results.get("Xv").toString();
			String y=(String) results.get("Yv").toString();
			String z=(String) results.get("Zv").toString();
			String timeStep=(String) results.get("TimeStepv").toString();
			String value=(String) results.get("Assimilationsv").toString();		
			String action=(String) results.get("Actionv").toString();	
			OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
			wrapperList.add(o);	

			String ID2=(String) results.get("IDv2").toString();
			String x2=(String) results.get("Xv2").toString();
			String y2=(String) results.get("Yv2").toString();
			String z2=(String) results.get("Zv2").toString();
			String timeStep2=(String) results.get("TimeStepv2").toString();
			String value2=(String) results.get("Assimilationsv2").toString();		
			String action2=(String) results.get("Actionv2").toString();	
			OutputWrapperResults o2=new OutputWrapperResults(ID2, x2,y2,z2, timeStep2, action2, value2);				
			wrapperList.add(o2);					
		}	
		else if(results.get("Typev")=="CalciumCellTX"){

			String ID=(String) results.get("IDv").toString();
			String x=(String) results.get("Xv").toString();
			String y=(String) results.get("Yv").toString();
			String z=(String) results.get("Zv").toString();
			String timeStep=(String) results.get("TimeStepv").toString();
			String value=(String) results.get("Emissionv").toString();		
			String action=(String) results.get("Actionv").toString();	
			OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
			wrapperList.add(o);	


		}
		else if(results.get("Typev")=="TCPLike-TEST-RX"){

			String ID=(String) results.get("IDv").toString();
			String x=(String) results.get("Xv").toString();
			String y=(String) results.get("Yv").toString();
			String z=(String) results.get("Zv").toString();
			String timeStep=(String) results.get("TimeStepv").toString();
			String value=(String) results.get("Assimilationsv").toString();		
			String action=(String) results.get("Actionv").toString();	
			OutputWrapperResults o=new OutputWrapperResults(ID, x,y,z, timeStep, action, value);				
			wrapperList.add(o);	
		}
		
		else if(results.get("Typev")=="TCPLike-TEST-TX"){	
			String ID1=(String) results.get("IDv");
			String x1=(String) results.get("Xv").toString();
			String y1=(String) results.get("Yv").toString();
			String z1=(String) results.get("Zv").toString();
			String timeStep1=(String) results.get("TimeStepv").toString();
			String value1=(String) results.get("Emissionv").toString();
			String action1=(String) results.get("Actionv").toString();				
			OutputWrapperResults o1=new OutputWrapperResults(ID1, x1,y1,z1, timeStep1, action1, value1);				
			wrapperList.add(o1);	
		
		}else if(results.get("Typev")=="DeathCounter"){

			String timeStep=(String) results.get("DeathTime").toString();
			String action="DEATH";	
			String value=(String) results.get("DeathValue").toString();
				
			OutputWrapperResults o=new OutputWrapperResults("DeathCounter", "0","0","0", timeStep, action, value);				
			wrapperList.add(o);					
		
		}
		
		
		if(results.get("Typew")=="LivingCounter"){

			String timeStep=(String) results.get("LiveTime").toString();
			String action="TOTAL OBJS";	
			String value=(String) results.get("LiveValue").toString();
				
			
			String red=(String) results.get("RED-LIVE").toString();			
			String white=(String) results.get("WHITE-LIVE").toString();			
			String plat=(String) results.get("PLAT-LIVE").toString();
			
			String nano=(String) results.get("nano-collision").toString();			
			String domain=(String) results.get("domain-collision").toString();
			
			
			OutputWrapperResults o=new OutputWrapperResults("LivingCounter", "0","0","0", timeStep, action, value);				
			wrapperList.add(o);	
			
			OutputWrapperResults o1=new OutputWrapperResults("RED-LIVE", "0","0","0", timeStep, "RED-LIVE", red);				
			wrapperList.add(o1);	
			
			OutputWrapperResults o2=new OutputWrapperResults("WHITE-LIVE", "0","0","0", timeStep, "WHITE-LIVE", white);				
			wrapperList.add(o2);	
			
			OutputWrapperResults o3=new OutputWrapperResults("PLAT-LIVE", "0","0","0", timeStep, "PLAT-LIVE", plat);				
			wrapperList.add(o3);	
			
			OutputWrapperResults o4=new OutputWrapperResults("nano-collision", "0","0","0", timeStep, "nano-collision", nano);				
			wrapperList.add(o4);
			
			OutputWrapperResults o5=new OutputWrapperResults("domain-collision", "0","0","0", timeStep, "domain-collision", domain);				
			wrapperList.add(o5);
			
			
		} 
	
	}

	@Override
	public void close() {
		// TODO Auto-generated method stub
	}	

	private Connection connect() {		
		Connection con=null;		
		try {				
			con = (Connection) DriverManager.getConnection(url, user, password);				
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			System.out.println("  ");
			System.out.println("[!]  [Unable to create a connection to DB...] ");
			System.out.println("  ");
			//e.printStackTrace();
		}
		return con;		
	}	

	@Override
	public void initialize(Manager m) {			
		try {						
			Connection con=connect();

			Statement stmt = (Statement) con.createStatement();
			stmt.executeUpdate("INSERT INTO simulation (ID, description) VALUES ('"+simulationID+"', '"+description+"') ");				
			con.close();			

		} catch (SQLException e) {
			// TODO Auto-generated catch block
			
			e.printStackTrace();
		}		
	}	
	
	
	public void update(String extraInfo) {			
		try {						
			Connection con=connect();

			description=description+" "+extraInfo;
			
			Statement stmt = (Statement) con.createStatement();
			//int res=stmt.executeUpdate("UPDATE simulation SET `description` = CONCAT(description,' "+extraInfo+" ','') WHERE ID='"+simulationID+"'");			
			//stmt.executeUpdate("UPDATE simulation SET `description` = '"+description+" "+extraInfo+"'  WHERE ID='"+simulationID+"'");
			stmt.executeUpdate("UPDATE simulation SET `description` = '"+description+"'  WHERE ID='"+simulationID+"'");
			
			con.close();			

		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}	

	public void store(){		
		Connection con=connect();	
		String finalSQL="";
		String sql="";	

		long now=System.currentTimeMillis();
		
		if(now>=nextKeepAlive){
			keepAliveBackupDB();
			refreshKeepAlive();
		}
		
			try {	

				if(wrapperList.size()>0){
					/*
					for(OutputWrapperResults oo : wrapperList){

						String value=(oo.value);

						//int value=Integer.parseInt(oo.value);
						String nanoobj=oo.id;
						long timestep=Long.parseLong(oo.timestep);
						String action=oo.action;
						float x=Float.parseFloat(oo.x);
						float y=Float.parseFloat(oo.y);
						float z=Float.parseFloat(oo.z);					

						sql=sql+" "+"('"+timestep+"','"+simulationID+"','"+nanoobj+"','"+x+"','"+y+"','"+z+"','"+action+"','"+value+"'), ";							
					}*/			

					sql=getSQLDataFromWrapper(wrapperList);
					
					sql=sql.substring(0, sql.lastIndexOf(","));
					finalSQL="INSERT IGNORE INTO results (timestep,simulation,nanoobj,x,y,z,type,value) VALUES "+sql;	

					if(con!=null){
						if(wrapperListForRecovery.size()>0){
							String sql0=getSQLDataFromWrapper(wrapperListForRecovery);							
							String sql1=sql0+" "+sql+",";
							sql1=sql1.substring(0, sql1.lastIndexOf(","));
							finalSQL="INSERT IGNORE INTO results (timestep,simulation,nanoobj,x,y,z,type,value) VALUES "+sql1;
						}
						
						Statement stmt = (Statement) con.createStatement();
						stmt.executeUpdate(finalSQL);
					}else{
						System.out.println("[!]  [Unable to STORE data on PRIMARY DB --> redirected to the BACKUP DB] ");
						phpStore(finalSQL);	
						
					}

				}else{
					System.out.println("-- Nothing to store... ");
				}
			} catch (Exception e) {
				System.out.println("Exception - [Unable to STORE data on PRIMARY DB --> redirected to the BACKUP DB] ");
				e.printStackTrace();

				phpStore(finalSQL);				
				

			} finally{
				try {
					con.close();
					wrapperList.clear();
					wrapperListForRecovery.clear();
				} catch (Exception e) {		
					System.out.println("  ");
					System.out.println("[!]  [Unable to close the connection to DB...] ");
					System.out.println("  ");
					//e.printStackTrace();
				}
			}
		
	}
	
	public void phpStore(final String data){
		Executors.newSingleThreadExecutor().execute(new Runnable() {
			@Override 
			public void run() {
				// code in here

				try {
					// open a connection to the site					
					URL url = new URL(disaster_recovery_db);
					URLConnection con = url.openConnection();
					// activate the output
					con.setDoOutput(true);
					PrintStream ps = new PrintStream(con.getOutputStream());
					// send your parameters to your site
					ps.print("check=nano1234567890");
					ps.print("&filename="+simulationID);
					ps.print("&data="+data);


					// we have to get the input stream in order to actually send the request
					con.getInputStream();

					// close the print stream
					ps.close();
					
					wrapperListForRecovery.addAll(wrapperList);
					wrapperList.clear();
					
					
				} catch (MalformedURLException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		});
	}
	
	public void keepAliveBackupDB(){

		if(disaster_recovery_db!=null){
			
			Executors.newSingleThreadExecutor().execute(new Runnable() {
				@Override 
				public void run() {
					// code in here

					try {					

						// open a connection to the site					
						URL url = new URL(disaster_recovery_db);
						URLConnection con = url.openConnection();
						// activate the output
						con.setDoOutput(true);
						PrintStream ps = new PrintStream(con.getOutputStream());
						ps.print("host="+hostname);					
						
						// we have to get the input stream in order to actually send the request
						con.getInputStream();

						// close the print stream
						ps.close();						

					} catch (MalformedURLException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			});
		}
	}

	
	public String getSQLDataFromWrapper(List<OutputWrapperResults> wrapperList){
		
		String sql="";	
		
		for(OutputWrapperResults oo : wrapperList){

			String value=(oo.value);

			//int value=Integer.parseInt(oo.value);
			String nanoobj=oo.id;
			long timestep=Long.parseLong(oo.timestep);
			String action=oo.action;
			float x=Float.parseFloat(oo.x);
			float y=Float.parseFloat(oo.y);
			float z=Float.parseFloat(oo.z);					

			sql=sql+" "+"('"+timestep+"','"+simulationID+"','"+nanoobj+"','"+x+"','"+y+"','"+z+"','"+action+"','"+value+"'), ";							
		}
		return sql;
	}
	
	
	private String getSimulationID(){
		Random r=new Random();
		if (simulationID==null) {					
			String name=Configurator2.readCustomStringConfigurations("config.xml", "System", "simulationName");			
			int day, month, year;
			GregorianCalendar gc = new GregorianCalendar();
			day = gc.get(Calendar.DAY_OF_MONTH);
			month = (gc.get(Calendar.MONTH))+1;
			year = gc.get(Calendar.YEAR);		    
			int hour,minute,second;
			hour=gc.get(Calendar.HOUR_OF_DAY);
			minute=gc.get(Calendar.MINUTE);
			second=gc.get(Calendar.SECOND);
			simulationID="sim_"+name+"_"+r.nextInt(9999)+"_"+year+month+day+hour+minute+second;
		}
		return simulationID;
	}

	public int getStoringFactor(){
		return storingFactor;
	}
}
