package simulator;

import java.io.*;
import java.util.*;
import MainLibrary.LivingObjObserver;
import MainLibrary.Manager;
import MainLibrary.NodeObj;
import MainLibrary.OutputStrategy;


public class OutputTXTCollisions extends OutputStrategy{

	private static OutputTXTCollisions instance=null;
	private static String fileName=null;
	private static FileOutputStream file;
	private static PrintStream Output;
	private OutputTXTCollisions(){
		
	}
	
	public static OutputTXTCollisions getInstance(){
		if (instance==null) instance=new OutputTXTCollisions();

		try {
			file = new FileOutputStream(instance.getFileName(),true);
			Output = new PrintStream(file);
			 
		} catch (IOException e) {
			System.out.println("Output Error: " + e);
		}
		return instance;
	}
	
	public void initialize(Manager m){
		try {
			List<NodeObj> nodeList=new ArrayList<NodeObj>();			
			List<NodeObj> allNodeList= new ArrayList<NodeObj>();
			List<LivingObjObserver> obsList=m.getAllLivingObjObserver();
			int size=obsList.size();
			for (int i=0;i<size;i++){
				allNodeList.addAll(obsList.get(i).getLivingNodes());
			}		
			
			file = new FileOutputStream(getFileName(),true);
			Output = new PrintStream(file);		
			
			for(int i=0;i<allNodeList.size();i++){
				if (allNodeList.get(i).getTimeToLive()>0){
					nodeList.add(allNodeList.get(i));					
				}				
			}
			
			Output.print(nodeList.size()+" ");
			
			int txCount=0;
			for(int i=0;i<nodeList.size();i++){				
				if (nodeList.get(i) instanceof Platelet) {
					txCount++;
				}											
			}
			Output.print(txCount);	
			
		} catch (IOException e) {
			System.out.println("Initialize Output Error: " + e);
			
		} finally {
		    // Close resource.
		    if (file!= null) try { 
		        file.close();
		    } catch (IOException logOrIgnore) {
		    	System.out.println("Output Error [closing file]: " + logOrIgnore);
		    }
		}
	}
	
	@Override
	public void export(Map<Object, Object> results) {
					  
		try {
			file = new FileOutputStream(getFileName(),true);
			Output = new PrintStream(file);
			
			if (results.containsKey("CurrentTime")){
				Output.println();
				Output.print(results.get("CurrentTime")+" ");		
				
			}else{				
				Output.print(results.get("Collision Count")+" ");
				Output.print(results.get("Number of Information object")+" ");
				
			}			
		} catch (IOException e) {
			System.out.println("Output Error: " + e);
			
		} finally {
		    // Close resource.
		    if (file!= null) try { 
		        file.close();
		    } catch (IOException logOrIgnore) {
		    	System.out.println("Output Error [closing file]: " + logOrIgnore);
		    }
		}
	} 	

	public void close(){
		
	}
	
	public String getFileName(){
		Random r=new Random();
		if (fileName==null) {
			int day, month, year;
		    GregorianCalendar gc = new GregorianCalendar();
		    day = gc.get(Calendar.DAY_OF_MONTH);
		    month = (gc.get(Calendar.MONTH))+1;
		    year = gc.get(Calendar.YEAR);
		    
	    	int hour,minute,second;
	    	hour=gc.get(Calendar.HOUR_OF_DAY);
	    	minute=gc.get(Calendar.MINUTE);
	    	second=gc.get(Calendar.SECOND);
	    	fileName="Collisions_exportID"+r.nextInt(1000)+"["+year+"-"+month+"-"+day+"]@"+hour+"-"+minute+"-"+second+".txt";
		}
		return fileName;
	}

	@Override
	public void update(String text) {
		// TODO Auto-generated method stub
		
	}	      
}