package simulator;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import MainLibrary.AbstractDomain;
import MainLibrary.LigandObserver;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.nEncode;
import MainLibrary.nIdle;
import MainLibrary.nTransmit;


public class PlateletVessel extends NodeObj{

	public int carrierCreated=0;	
	private long firstRelease=-1;
	private boolean hasExported=false;
	

	public PlateletVessel(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);

		// You can set the internal state here!
		setState(new nIdle(this));
		getState().setExecutionTime(1);	
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
	
	}	
	
	@Override
	public void idle() {
		if (carrierCreated==0 && burstEmission>0){
			activateCell();
		}
	}

	@Override
	public void decode() {
		setState(new nIdle(this));

	}

	@Override
	public void encode() {
		createCarriers(burstEmission);
		setState(new nTransmit(this));
	}

	@Override
	public void searchNeighbor() {

	}

	@Override
	public void transmit() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(0);		
	}

	public void createCarriers(int n){
		while (n>0){				
			CD40LCarrier cc=new CD40LCarrier(getInnerDomain(), this, getCenter()[0], getCenter()[1], getCenter()[2],1);			
			cc.setMotionStrategy(BrownianMotionBloodStream.getInstance());
			n--;
			carrierCreated++;
		}			
	}

	@Override
	public Map<Object, Object> specializedExport() {

		/* ORIGINAL
		Map<Object, Object> m=new HashMap<Object,Object>();
		if(carrierCreated>0 && !hasExported){
			m.put("Typev", "PlateletVessel");

			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		
			
			if(Float.isNaN(center[0]) || Float.isNaN(center[1]) || Float.isNaN(center[2])){
				System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> NaN NaN NaN");
			}
			
			m.put("IDv", "Platelet-"+ID);	
			m.put("Xv", center[0]-domainCenter[0]);
			m.put("Yv", center[1]-domainCenter[1]);
			m.put("Zv", center[2]-domainCenter[2]);
			m.put("TimeStepv", ""+firstRelease+"");	
			m.put("Emissionv", carrierCreated);
			m.put("Actionv", "emission");
			m.putAll(getState().export());

			hasExported=true;
		}
		*/
		
		
		// FIXME For DEBUG
		Map<Object, Object> m=new HashMap<Object,Object>();	
		if(exportThis){	

			m.put("Typev", "Platelet");

			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		

			m.put("IDv", "Platelet-"+ID);	
			m.put("Xv", center[0]-domainCenter[0]);
			m.put("Yv", center[1]-domainCenter[1]);
			m.put("Zv", center[2]-domainCenter[2]);
			m.put("TimeStepv", ""+getManager().getCurrentTime()+"");	

			m.put("Actionv", "Track");		
			m.putAll(getState().export());
		}

		return m;
	}	
	
	public void createReceptor(int n){		

		Receptor r = null;		
		float theta=0;
		float fi=0;			

		for (int i=0;i<n;i++){
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];		

			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			

			r=new CD154Receptor(this, false, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			receptorSetup(r);					
		}
	}	

	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	}

	@Override
	public void setMotionStrategy() {		
		this.motionStrategy=BrownianMotionBloodStream.getInstance();
	}

	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}

	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	public void activateCell() {
		/*
		float check=(float) (((((CylindricDomain) getDomain()).getHeight()-101000-500000)/2)-(((CylindricDomain) getDomain()).getHeight()-101000-250000)*0.1);
		check=-775000+100000-7500+400000;
		*/
		float check=-900000;		

		if ((getCenter()[1])>=check){ 
			firstRelease=getManager().getCurrentTime();
			setState(new nEncode(this));
			getState().setExecutionTime(1);	
		}

	}

	public String assignName(){		
		return "Platelet";		
	}

	@Override
	public void processInfo() {
		// TODO Auto-generated method stub		
	}

	@Override
	public void createSingleReceptor(float[] center, float radius, short receptorRadius, int receptorType) {
		// TODO Auto-generated method stub
		
	}
}