
package simulator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.LigandObserver;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.nEncode;
import MainLibrary.nIdle;
import MainLibrary.nTransmit;


public class PlateletVesselTX extends NodeObj{

	public int carrierCreated=0;	
	private long[] releaseTimeLog=new long[10];
	private int[] releaseCarrierLog=new int[10];
	private int indexReleaseLog=0;
	private int indexExportedLog=0;
	private ArrayList<float[]> releasePositions=new ArrayList<float[]>();
	
	//private boolean hasExported=false;
	private int timeForEmission=0;
	private int indexPattern=0;
	private String emissionPattern="";
	
	private int countdownForEmission=0;
	
	boolean hasTransmitted=false;
	


	public PlateletVesselTX(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);

		// You can set the internal state here!
		/*
		setState(new nIdle(this));
		getState().setExecutionTime(1);
		*/	
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
		
		setState(new nTransmit(this));
		getState().setExecutionTime(0);
		
		
		
		// Setup for the emission pattern
		try {	
			if(domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-timeForEmission") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-emissionPattern")){
				timeForEmission=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-timeForEmission").intValue();
				emissionPattern=Integer.toString((domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-emissionPattern").intValue()));		
				//countdownForEmission=timeForEmission;  // emits after this time
				countdownForEmission=0;		// emitis immediately!!
				
			}else{
				throw new Exception();
			}
		} catch (Exception e) {
			System.out.println();
			System.out.println(">>> WARNING! The transmitter node has not been configured for the EMISSION PATTERN!");
			System.out.println();
			System.out.println("    The simulation ends now");
			System.out.println();
			e.printStackTrace();
			System.exit(0);
		}
		
		// END setup for emission pattern
		
	}
	
	public void enlargeReleaseTimeLog(){
		long[] temp=releaseTimeLog;
		int[] tempCarrier=releaseCarrierLog;
		int size=temp.length;
		size=size+size/2;
		
		releaseTimeLog=new long[size];
		releaseCarrierLog=new int[size];
		
		for(int i=0;i<temp.length;i++){
			releaseTimeLog[i]=temp[i];
			releaseCarrierLog[i]=tempCarrier[i];
		}		
	}
	
	@Override
	public boolean mandatoryCreation(){
		return true;
	}

	public void standardBehavior() {		
		//super.standardBehavior();		// FIXME You have to activate this line if you want to emit through RECEPTORS!
		
		
		/*
		if (indexPattern<emissionPattern.length()){
			countdownForEmission--;
			if(countdownForEmission<=0){


				if (firstRelease<0) firstRelease=getManager().getCurrentTime();
				if(emissionPattern.charAt(indexPattern)=='1'){
					System.out.println(">>> Releasing CARRIERS!!! (Platelet)");
					createCarriers(burstEmission);
				}


				
				indexPattern++;
				countdownForEmission=timeForEmission;
			}
		}
		*/
		emitsWithoutReceptors();		// FIXME You have to activate this line if you want to emit WITHOUT ANY RECEPTORS!

	}
	
	@Override
	public void idle() {	
		
		/* OLD BEFORE PATTERN - SEE standardBehaviour()!!!
		if (carrierCreated==0 && burstEmission>0){
			activateCell();
		} */
		getState().setExecutionTime(10);	
		
	}

	@Override
	public void decode() {
		setState(new nIdle(this));

	}

	@Override
	public void encode() {
		/* OLD BEFORE PATTERN - SEE standardBehaviour()!!!
		createCarriers(burstEmission);
		setState(new nTransmit(this));
		*/
		setState(new nIdle(this));
	}

	@Override
	public void searchNeighbor() {

	}

	@Override
	public void transmit() {
		
		if (indexPattern<emissionPattern.length()){
			countdownForEmission--;
			if(countdownForEmission<=0){


				int created=0;
				if(emissionPattern.charAt(indexPattern)=='1'){
					System.out.println(">>> Releasing CARRIERS!!! (Platelet)");
					created=createCarriers(burstEmission);
				}
				if(releaseTimeLog.length<=indexReleaseLog){
					enlargeReleaseTimeLog();
				}
				releaseTimeLog[indexReleaseLog]=getManager().getCurrentTime();
				releaseCarrierLog[indexReleaseLog]=created;

				float[] center=getCenter();
				float[] domainCenter=getDomain().getAbsCenter();					

				float[] currentRelativePosition={center[0]-domainCenter[0],center[1]-domainCenter[1],center[2]-domainCenter[2]};
				releasePositions.add(currentRelativePosition);
				indexReleaseLog++;

				//}
				indexPattern++;
				countdownForEmission=timeForEmission;
			}
		}else{
			hasTransmitted=true;
			setState(new nIdle(this));
			getState().setExecutionTime(10);	
		}
		
		
		/* OLD BEFORE PATTERN - SEE standardBehaviour()!!!
		setState(new nIdle(this)); 
		getState().setExecutionTime(0);	
		*/	
	}

	public int createCarriers(int n){
		int createdNow=0;
		while (n>0){				
			CD40LCarrier cc=new CD40LCarrier(getInnerDomain(), this, getCenter()[0], getCenter()[1], getCenter()[2],1);			
			cc.setMotionStrategy(BrownianMotionBloodStream.getInstance());
			n--;
			createdNow++;			
		}
		carrierCreated=carrierCreated+createdNow;
		return createdNow;
	}
	
	
	public void emitsWithoutReceptors(){
		
		float theta=0;
		float fi=0;			
		
		int undisposedCarriers=getInnerDomain().undisposedCarrierList.size();

		for(int i=undisposedCarriers-1;i>=0;i--){		
		
		//for (int i=0;i<undisposedCarriers;i++){				
			
			if(getDomain()==null){
				System.out.println();
				System.out.println("*** PlateletVesselTX: emitsWithoutReceptors() -> DOMINIOP DEL NODO = NULL!");
				System.out.println();
			}
		
			CarrierObj carrier=getInnerDomain().undisposedCarrierList.get(i);	
			carrier.removeFromActualDomain();
			//carrier.setDomain(getDomain());  
			
			
			fi=(float) (2*Math.PI*(rand.nextFloat())); //fi
			theta=(float) ((Math.acos((2*rand.nextFloat())-1))); //theta

			double radius=getRadius();	
			radius=radius+2*carrier.getRadius();
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));		
			

			float[] nodeCenter=getCenter();			
			float x=coords[0]+nodeCenter[0];
			float y=coords[1]+nodeCenter[1];
			float z=coords[2]+nodeCenter[2];		

		
			getDomain().attachNanoObj(carrier);
			carrier.disposeOnMedium();
			
			carrier.setCenter(x, y, z);
			carrier.setVelocityComponent(carrier.getVelocityModule(), fi, theta);	
		}
		getInnerDomain().undisposedCarrierList.clear();		
	}
	
	

	@Override
	public Map<Object, Object> specializedExport() {

		Map<Object, Object> m=new HashMap<Object,Object>();

		m.put("Typev", "PlateletVessel");
		m.put("ID_tracking", "Platelet-"+ID+"-track");
		m.put("TimeStep_tracking", getManager().getCurrentTime());

		
		int diff=(indexReleaseLog-indexExportedLog);
		int k=0;
		m.put("Results", diff);
		
		for (int i=indexExportedLog;i<indexReleaseLog;i++){
			
			m.put(k+"-IDv", "Platelet-"+ID);	
			m.put(k+"-Xv", releasePositions.get(i)[0]);
			m.put(k+"-Yv", releasePositions.get(i)[1]);
			m.put(k+"-Zv", releasePositions.get(i)[2]);
			m.put(k+"-TimeStepv", ""+releaseTimeLog[i]+"");	
			m.put(k+"-Emissionv", ""+releaseCarrierLog[i]+"");
			m.put(k+"-Actionv", "emission");
			
			indexExportedLog++;
			k++;
		}
		
		m.putAll(getState().export());

		return m;
	}

	/*
	public Map<Object, Object> specializedExport_BAK() {

		Map<Object, Object> m=new HashMap<Object,Object>();
		if(carrierCreated>0 && !hasExported){
			m.put("Typev", "PlateletVessel");

			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		
			
			if(Float.isNaN(center[0]) || Float.isNaN(center[1]) || Float.isNaN(center[2])){
				System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> NaN NaN NaN");
			}
			
			m.put("IDv", "Platelet-"+ID);	
			m.put("Xv", center[0]-domainCenter[0]);
			m.put("Yv", center[1]-domainCenter[1]);
			m.put("Zv", center[2]-domainCenter[2]);
			//m.put("TimeStepv", ""+releaseTime+"");	
			m.put("Emissionv", carrierCreated);
			m.put("Actionv", "emission");
			m.putAll(getState().export());

			hasExported=true;
		}
		return m;
	}
	*/
	
	@Override
	public void createSingleReceptor(float[] cdCenter, float radius, short receptorRadius, int receptorType) {
		float[] angles=convertSphericalAngles(cdCenter[0],cdCenter[1],cdCenter[2], radius);
		float theta=angles[1];
		float fi=angles[0];

		
		Receptor r=new CD154Receptor(this, true, cdCenter, theta, fi, receptorRadius, receptorType, this.eventHandler);
		receptorList.add(r);
		receptorSetup(r); 

		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		*/		
	}
	
	public void createReceptor(int n){		

		Receptor r = null;		
		float theta=0;
		float fi=0;			

		for (int i=0;i<n;i++){
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];		

			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			

			r=new CD154Receptor(this, false, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			receptorSetup(r);					
		}
	}	

	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	}

	@Override
	public void setMotionStrategy() {		
		this.motionStrategy=BrownianMotionBloodStream.getInstance();
	}

	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}

	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	public void activateCell() {
		/*
		float check=(float) (((((CylindricDomain) getDomain()).getHeight()-101000-500000)/2)-(((CylindricDomain) getDomain()).getHeight()-101000-250000)*0.1);
		check=-775000+100000-7500+400000;
		*/
		float check=-900000;		

		if ((getCenter()[1])>=check){ 
			//releaseTime=getManager().getCurrentTime();
			setState(new nEncode(this));
			getState().setExecutionTime(1);	
		}

	}

	public String assignName(){		
		return "Platelet";		
	}

	@Override
	public void processInfo() {
		// TODO Auto-generated method stub		
	}
}