package simulator;

import MainLibrary.AbstractDomain;
import MainLibrary.LinearMotion;
import MainLibrary.Manager;
import MainLibrary.NodeObj;

public class PunctiformEmitter {
	
	
	private AbstractDomain domain;	
	private Manager manager;
	
	private float[] centerRX=new float[3];
	
		
	private double totalCarriers=0;
	private double coeff=0.33333333;
	private int size= 0;
	private float step=0; // Carrier reciprocal distance
	
	private float distance=0;	

	public float xMin=Float.MAX_VALUE;	
	
	private int symbolTime=1;	// timesteps
	private char[] emissionPattern;	
	private int currentSymbolIndex=0;
	
	private long nextEmissionTime=0;
	
	private float radiusRXNode;
	
	private short alignment=0;		// angle emission point for cylindric  // 0=0, 45=1, 90=2
	private float[] alignCorrection=new float[3];
	
	
	public PunctiformEmitter(AbstractDomain domain, float x, float y, float z, float radiusRX, short alignment, double totalCarriers, float cStep, float simDistance, int symbolTime, String pattern){
		
		this.domain=domain;
		this.manager=domain.getManager();
		
		this.radiusRXNode=radiusRX;
		
		this.centerRX[0]=x;
		this.centerRX[1]=y;
		this.centerRX[2]=z;		
		
		this.totalCarriers=totalCarriers;
		this.size= (int) Math.ceil(Math.pow(totalCarriers, coeff)); //47   -> totalCarriers^(1/3)	
		
		this.step=cStep; // Carrier reciprocal distance
		this.distance=simDistance;
		
		
		this.symbolTime=symbolTime;			
		this.nextEmissionTime=manager.getCurrentTime();	// EMITS NOW!		
		
		
		char[] emissionPattern = new char[pattern.length()];
		
		for(int icc=0; icc<pattern.length();icc++){
			if(pattern.charAt(icc)=='1'){
				emissionPattern[icc]='1';	
			}else{
				emissionPattern[icc]='0';
			}
		}
		
		this.emissionPattern=emissionPattern;
		
		this.alignment=alignment;
		
		// 0=0,   45=1,   90=2
		if(alignment==0){
			alignCorrection[0]=distance;
			alignCorrection[1]=0;
			alignCorrection[2]=0;
			
		}else if(alignment==1){
			alignCorrection[0]=distance/1.414214f;
			alignCorrection[1]=distance/1.414214f;
			alignCorrection[2]=0;
			
		}else if(alignment==2){
			alignCorrection[0]=0;
			alignCorrection[1]=distance;
			alignCorrection[2]=0;
		}
		
		
	}

	
	public void exec(){

		if(currentSymbolIndex<emissionPattern.length){

			if(nextEmissionTime<=manager.getCurrentTime()){

				if(emissionPattern[currentSymbolIndex]=='1'){

					// EMISSION!
					disposeCarrier();
					
				

				}else{
					// NO-EMISSION: symbol==0
					System.out.println();
					System.out.println(">>> =========================================================================================");
					System.out.println(">>> PUNCTIFORM EMISSION!  [0]  Carrier Released: 0");
					System.out.println(">>> =========================================================================================");
					System.out.println();

				}

				currentSymbolIndex=currentSymbolIndex+1;
				nextEmissionTime=manager.getCurrentTime()+symbolTime;

			}
		}
	}

	private void disposeCarrier(){		
			
		int createdCarrier=0;

		float dist1=0;		

		
		
		for(int i=0;i<size;i=i+1){
			for(int j=0;j<size;j=j+1){
				for(int k=0;k<size;k=k+1){			

					if (createdCarrier<totalCarriers){

						/**
						 * 0  = choose only x
						 * 90 = choose only y or only z
						 * 45 = choose both x/rad(2) and y/rad(2) or both x/rad(2) and z/rad(2) 
						 */
						
						/*
						float rx=centerRX[0]+distance+i*step-(size*step/2);
						float ry=centerRX[1]+j*step-(size*step/2);
						float rz=centerRX[2]+k*step-(size*step/2);						
						*/
						
						
						float rx=centerRX[0]+alignCorrection[0]+i*step-(size*step/2);
						float ry=centerRX[1]+alignCorrection[1]+j*step-(size*step/2);
						float rz=centerRX[2]+alignCorrection[2]+k*step-(size*step/2);		

						
						
						TCPLike_Carrier_A cc=new TCPLike_Carrier_A(domain, rx, ry, rz, 5);			
						cc.setMotionStrategy(BrownianMotion.getInstance());						// OK
						//cc.setMotionStrategy(LinearMotion.getInstance());						// DEBUG
						//cc.setVelocityComponent(30, 0, -(float)Math.PI/2);
						


						if(xMin>rx) xMin=rx;

						createdCarrier++;						

						dist1=dist1+(float) Math.sqrt((rx-centerRX[0])*(rx-centerRX[0])+(ry-centerRX[1])*(ry-centerRX[1])+(rz-centerRX[2])*(rz-centerRX[2]));

					}
				}	
			}
		}
		
		System.out.println();
		System.out.println(">>> =========================================================================================");
		System.out.println(">>> PUNCTIFORM EMISSION!  [1]  Carrier Released: "+createdCarrier+"  -  Mean Distance: "+(dist1/createdCarrier-radiusRXNode)+" [nm]");
		System.out.println(">>> xMin: "+xMin);
		System.out.println(">>> =========================================================================================");
		System.out.println();
		
	}

}
