package simulator;

import java.awt.Color;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import MainLibrary.AbstractDomain;
import MainLibrary.LigandObserver;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.nIdle;


public class RedBloodCell extends NodeObj{

	public int carrierCreated=0;
	private long firstContact=-1;
	
	
	public RedBloodCell(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);
						
		// You can set the internal state here!
		setState(new nIdle(this)); 
		getState().setExecutionTime(1000);		
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));	
		setColor(Color.red.brighter());		
	}
	
	@Override
	public void idle() {
		getState().setExecutionTime(1000);	
	}
	
	@Override
	public void decode() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(1000);	
	}
	
	@Override
	public void encode() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(1000);	
	}

	@Override
	public void searchNeighbor() {
		
	}

	@Override
	public void transmit() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(1000);		
	}
	
	public void createCarriers(int n){
			
	}

	@Override
	public Map<Object, Object> specializedExport() {
		/*  ORIGINAL
		Map<Object, Object> m=new HashMap<Object,Object>();
		m.put("Type", "Node");		
		m.put("firstContact", firstContact);
		m.putAll(getState().export());
		*/	
		

		// FIXME For DEBUG
		Map<Object, Object> m=new HashMap<Object,Object>();	
		if(exportThis){	
			m.put("Typev", "RBC");

			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		

			m.put("IDv", "RBC-"+ID);	
			m.put("Xv", center[0]-domainCenter[0]);
			m.put("Yv", center[1]-domainCenter[1]);
			m.put("Zv", center[2]-domainCenter[2]);
			m.put("TimeStepv", ""+getManager().getCurrentTime()+"");	

			m.put("Actionv", "Track");		
			m.putAll(getState().export());

		}
		
		return m;
	}
	
	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	}
	
	@Override
	public void setMotionStrategy() {		
		this.motionStrategy=BrownianMotionBloodStream.getInstance();
	}
	
	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}
		
	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	@Override
	public void processInfo() {
		// TODO Auto-generated method stub	
	}
	
	public String assignName(){		
		return "Red Blood Cell";		
	}
	
	public void exec(){
		
	}

	@Override
	public void createReceptor(int n) {
		// TODO Auto-generated method stub
		// No Receptors for this Cell!		
	}

	@Override
	public void createSingleReceptor(float[] center, float radius, short receptorRadius, int receptorType) {
		// TODO Auto-generated method stub
		
	}
}