package simulator;

import java.util.HashMap;
import java.util.Map;

import MainLibrary.AbstractCollisionDomainStrategy;
import MainLibrary.AbstractDomain;
import MainLibrary.MobilityModel;

import com.threed.jpct.SimpleVector;

public class ShellCylindricDomain extends CylindricDomain{
	
	private SimpleVector focusVersor;
	private SimpleVector ortogonalVersor;

	public ShellCylindricDomain(AbstractDomain parentDomain,
			MobilityModel model, AbstractCollisionDomainStrategy strategy, boolean visible, boolean export, float h, float r, float[] dCenter, SimpleVector orientation, SimpleVector cylinderOrtogonalOrientation, boolean useOctree) {
		super(parentDomain, model, strategy, visible, export, h, r, dCenter, useOctree);
		// TODO Auto-generated constructor stub

		
		ID="SHELL CYLINDRIC DOMAIN";

		this.focusVersor=orientation.normalize();		// Points on the center of the transmitter node!		
		this.ortogonalVersor=cylinderOrtogonalOrientation.normalize();
		
		setDirectionP(focusVersor);
		setDirectionN(ortogonalVersor);		

		if(useOctree) initializeOctree(getManager().configurator);

		if (isVisible()) getManager().engine.addWrapper3DDomain(this);	
		
		localExportTime=localExportTime*500;

	}
	
	public void setParabolicVersor(SimpleVector v){		
		this.focusVersor=new SimpleVector(v.normalize());
	}
	
	public SimpleVector getParabolicVersor(){
		return focusVersor;
	}	
	
	public String getType(){
		return "SHELL CYLINDRIC DOMAIN";
	}	


	@Override
	public Map<Object, Object> export() {
		exportString=new HashMap<Object, Object>();		
		exportString.put("ID", ID);	
		
		
		long objCount=0;
		
		boolean hasToExport=false;
		long time=getManager().getCurrentTime();
		
		//if(time%10000==0){
		if(time%localExportTime==0 || time==1000 ){
			hasToExport=true;
			
			objCount=getActiveNanoObjListSize();
			
		}
		
		if(hasToExport  ){
			exportString.put("Type", "CYLINDER");
   
			float[] center=new float[3];
			float[] domainCenter=getRelativeCenter();			

			
			exportString.put("X", center[0]-domainCenter[0]);
			exportString.put("Y", center[1]-domainCenter[1]);
			exportString.put("Z", center[2]-domainCenter[2]);		
			
			
			exportString.put("TimeStep", time+"");	
			exportString.put("Value", objCount+"");	
			exportString.put("Action", "Objs count");		
			
			/*
			ShellCylindricStrategy stra=(ShellCylindricStrategy) getCollisionStrategy();
			exportString.put("dCollision", ""+stra.dCollision);
			*/
			
			
			int ssize=carrierList.size();
			exportString.put("carrierList", ""+ssize);			
			
			for(int i=0; i<ssize;i++){
				
				float[] ccenter=carrierList.get(i).getCenter();
				
				exportString.put("Type_"+i, "CarrierInsideShell_"+carrierList.get(i).ID);
				exportString.put("TimeStep_"+i, time+"");
				exportString.put("X_"+i, ccenter[0]);
				exportString.put("Y_"+i, ccenter[1]);
				exportString.put("Z_"+i, ccenter[2]);	
				
			}
			
			
		}
		
		
		return exportString;
	}

}
