package simulator;

import java.awt.Color;

import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.NanoObj;

import com.threed.jpct.SimpleVector;

public class ShellCylindricStrategy extends CylindricStrategy {
	
	boolean bottomless=false;

	public ShellCylindricStrategy(int type) {
		super(type);
		// TODO Auto-generated constructor stub
	}	
	
	
	public ShellCylindricStrategy(int type, boolean bottomless) {
		super(type);
		this.bottomless=bottomless;
		// TODO Auto-generated constructor stub
	}



	public void absorb(NanoObj toRelocate, AbstractDomain domain){

		if(toRelocate instanceof CarrierObj){
			toRelocate.switchDomain(domain);
			domain.activatePropagationObj((CarrierObj)toRelocate);

		}else{

			toRelocate.switchDomain(domain);
			domain.activatePropagationObj(toRelocate);
		}
	}


	@Override
	public void relocateOuterObjs(NanoObj toRelocate, AbstractDomain domainToRelocate) {
		if(toRelocate.getDomain()!=domain){
			
			/*
			// FIXME DEBUG
			synchronized(this){
				dCollision++;
			}*/

			ShellCylindricDomain domain=(ShellCylindricDomain)domainToRelocate;

			float[] velocity1=toRelocate.getVelocityComponent();
			float modVelocity1=toRelocate.getVelocityModule();
			float modVelocity2=0;
			double dt=0;	

			if (modVelocity1==0 && modVelocity2==0){				

			} else{

				float[] distances=getSignedDistanceOnAxis(domain,toRelocate);	// SIGNED!!!	
				float[] domainCenter=domain.getAbsCenter();		
				float[] nanoCenter=toRelocate.getCenter();	
				float distanceP= Math.abs(distances[0])-toRelocate.getRadius();
				//float distanceN= distances[1]-toRelocate.getRadius();	
				float distanceN= (float) ((Math.sqrt(distances[1]*distances[1]+distances[2]*distances[2]))-toRelocate.getRadius());	

				if (((distanceP)<=domain.getHeight()/2) && (distanceN<=domain.getRadius())){						

					float dsP=distanceP-(domain.getHeight()/2);
					float dsN=distanceN-(domain.getRadius());

					if (dsP>=dsN){

						// CHECK IF ABSORB OR BOUNCE
						if(bottomless || distances[0]>0){		// ok
					

							// ENTER INTO THE DOMAIN!!

							// ABSORBENT!
							absorb(toRelocate,domainToRelocate);

						}else{

							// BOUNCE!!


							SimpleVector velocityVector=new SimpleVector(velocity1);
							float angleV=velocityVector.calcAngle(domain.getDirectionP());
							float dsPV=(float) ((dsP)*Math.cos(angleV));
							dt =(float) Math.abs((dsPV))/Math.abs(toRelocate.getVelocityModule()); // Warning! catch the div-by-zero!!!

							if (dt>1.1) {
								dt=1;					
							}

							nanoCenter[0]+=-velocity1[0]*dt;
							nanoCenter[1]+=-velocity1[1]*dt;
							nanoCenter[2]+=-velocity1[2]*dt;
							toRelocate.setCenter(nanoCenter[0], nanoCenter[1], nanoCenter[2]);

							SimpleVector v1=new SimpleVector(toRelocate.getVelocityComponent());
							SimpleVector d=domain.getDirectionP();
							double dModule=Math.sqrt(d.x*d.x+d.y*d.y+d.z*d.z);
							float[] dVersorComponent=new float[3];
							dVersorComponent[0]=(float) (d.x/dModule);
							dVersorComponent[1]=(float) (d.y/dModule);
							dVersorComponent[2]=(float) (d.z/dModule);

							SimpleVector dVersor=new SimpleVector(dVersorComponent);
							domainCenter=domain.getAbsCenter();					
							nanoCenter=toRelocate.getCenter();

							float x=nanoCenter[0]-domainCenter[0];
							float y=nanoCenter[1]-domainCenter[1];
							float z=nanoCenter[2]-domainCenter[2];

							float relativeCenterModule=(float) Math.sqrt(x*x+y*y+z*z);
							SimpleVector nanoCenterVersor=new SimpleVector(x/relativeCenterModule,y/relativeCenterModule,z/relativeCenterModule);
							SimpleVector oTempVersor=nanoCenterVersor.calcCross(dVersor);
							if(oTempVersor.length()!=0){
								oTempVersor=new SimpleVector(oTempVersor.x/oTempVersor.length(),oTempVersor.y/oTempVersor.length(),oTempVersor.z/oTempVersor.length());
							}else{
								oTempVersor=new SimpleVector(0,0,0);
							}

							SimpleVector nTempVersor=oTempVersor.calcCross(dVersor);
							float alfa=nTempVersor.calcAngle(v1);				

							double n1Module=toRelocate.getVelocityModule()*Math.cos(alfa);
							float[] n1components=new float[3];
							n1components[0]=(float) (n1Module*nTempVersor.x);
							n1components[1]=(float) (n1Module*nTempVersor.y);
							n1components[2]=(float) (n1Module*nTempVersor.z);				
							SimpleVector n1=new SimpleVector(n1components);

							float gamma=dVersor.calcAngle(v1);				
							double d1Module=toRelocate.getVelocityModule()*Math.cos(gamma);
							float[] d1components=new float[3];
							d1components[0]=(float) (d1Module*dVersor.x);
							d1components[1]=(float) (d1Module*dVersor.y);
							d1components[2]=(float) (d1Module*dVersor.z);				
							SimpleVector d1=new SimpleVector(d1components);

							float ro=oTempVersor.calcAngle(v1);				
							double o1Module=toRelocate.getVelocityModule()*Math.cos(ro);
							float[] o1components=new float[3];
							o1components[0]=(float) (o1Module*oTempVersor.x);
							o1components[1]=(float) (o1Module*oTempVersor.y);
							o1components[2]=(float) (o1Module*oTempVersor.z);				
							SimpleVector o1=new SimpleVector(o1components);

							//InelasticCollision
							d1.scalarMul(inelasticCoefficient);						
							// END InelasticCollision

							n1.add(o1);
							SimpleVector v2=n1.calcSub(d1);

							//float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
							//float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];							
							float[] angles=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z);		
							float fi1=angles[0];
							float theta1=angles[1];
							
							float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
							toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);	



							velocity1=toRelocate.getVelocityComponent();
							nanoCenter[0]+=velocity1[0]*dt;
							nanoCenter[1]+=velocity1[1]*dt;
							nanoCenter[2]+=velocity1[2]*dt;
							toRelocate.setCenter(nanoCenter[0], nanoCenter[1], nanoCenter[2]);						


						}



					}else{
						SimpleVector velocityVector=new SimpleVector(velocity1);
						float angleV=velocityVector.calcAngle(domain.getDirectionP());
						float dsNV=(float) ((dsN)*Math.sin(angleV));
						dt =(float) Math.abs((dsNV))/Math.abs(toRelocate.getVelocityModule()); 

						if (dt>1.1) {
							dt=1;					
						}

						nanoCenter[0]+=-velocity1[0]*dt;
						nanoCenter[1]+=-velocity1[1]*dt;
						nanoCenter[2]+=-velocity1[2]*dt;
						toRelocate.setCenter(nanoCenter[0], nanoCenter[1], nanoCenter[2]);

						SimpleVector v1=new SimpleVector(toRelocate.getVelocityComponent());
						SimpleVector d=domain.getDirectionP();
						double dModule=Math.sqrt(d.x*d.x+d.y*d.y+d.z*d.z);
						float[] dVersorComponent=new float[3];
						dVersorComponent[0]=(float) (d.x/dModule);
						dVersorComponent[1]=(float) (d.y/dModule);
						dVersorComponent[2]=(float) (d.z/dModule);

						SimpleVector dVersor=new SimpleVector(dVersorComponent);
						domainCenter=domain.getAbsCenter();
						nanoCenter=toRelocate.getCenter();

						float x=nanoCenter[0]-domainCenter[0];
						float y=nanoCenter[1]-domainCenter[1];
						float z=nanoCenter[2]-domainCenter[2];

						float relativeCenterModule=(float) Math.sqrt(x*x+y*y+z*z);
						SimpleVector nanoCenterVersor=new SimpleVector(x/relativeCenterModule,y/relativeCenterModule,z/relativeCenterModule);
						SimpleVector oTempVersor=nanoCenterVersor.calcCross(dVersor);
						oTempVersor=new SimpleVector(oTempVersor.x/oTempVersor.length(),oTempVersor.y/oTempVersor.length(),oTempVersor.z/oTempVersor.length());
						SimpleVector nTempVersor=oTempVersor.calcCross(dVersor);
						nTempVersor=nTempVersor.normalize();

						float alfa=nTempVersor.calcAngle(v1);		
						double n1Module=toRelocate.getVelocityModule()*Math.cos(alfa);
						float[] n1components=new float[3];
						n1components[0]=(float) (n1Module*nTempVersor.x);
						n1components[1]=(float) (n1Module*nTempVersor.y);
						n1components[2]=(float) (n1Module*nTempVersor.z);				
						SimpleVector n1=new SimpleVector(n1components);

						float gamma=dVersor.calcAngle(v1);				
						double d1Module=toRelocate.getVelocityModule()*Math.cos(gamma);
						float[] d1components=new float[3];
						d1components[0]=(float) (d1Module*dVersor.x);
						d1components[1]=(float) (d1Module*dVersor.y);
						d1components[2]=(float) (d1Module*dVersor.z);				
						SimpleVector d1=new SimpleVector(d1components);

						float ro=oTempVersor.calcAngle(v1);				
						double o1Module=toRelocate.getVelocityModule()*Math.cos(ro);
						float[] o1components=new float[3];
						o1components[0]=(float) (o1Module*oTempVersor.x);
						o1components[1]=(float) (o1Module*oTempVersor.y);
						o1components[2]=(float) (o1Module*oTempVersor.z);				
						SimpleVector o1=new SimpleVector(o1components);

						//InelasticCollision
						n1.scalarMul(inelasticCoefficient);						
						// END InelasticCollision

						d1.add(o1);
						SimpleVector v2=d1.calcSub(n1);	

						//float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
						//float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];
						float[] angles=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z);		
						float fi1=angles[0];
						float theta1=angles[1];
						
						float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
						toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);	


						velocity1=toRelocate.getVelocityComponent();
						nanoCenter[0]+=velocity1[0]*dt;
						nanoCenter[1]+=velocity1[1]*dt;
						nanoCenter[2]+=velocity1[2]*dt;
						toRelocate.setCenter(nanoCenter[0], nanoCenter[1], nanoCenter[2]);
					}				
				} 				
			}				
		}
	}	



	public void relocateInnerObjs(NanoObj toRelocate, AbstractDomain domain) {
		if(toRelocate.getDomain()==domain){		

			ShellCylindricDomain owner=(ShellCylindricDomain)domain;

			float[] center1=toRelocate.getCenter();
			float[] center2=owner.getAbsCenter();			
			float modVelocity1=toRelocate.getVelocityModule();
			//float modVelocity2=0;
			double dt=0;			

			if (modVelocity1==0){
				// FIXME Even if both velocity are null, it should handles the collision in some way!
			} else{
				float[] distances=getSignedDistanceOnAxis(domain,toRelocate);			// SIGNED!!	
				float[] domainCenter=domain.getAbsCenter();				
				float distanceP= (Math.abs(distances[0])+toRelocate.getRadius());
				//float distanceN= Math.abs(distances[1]+toRelocate.getRadius());	
				float distanceN= (float) ((Math.sqrt(distances[1]*distances[1]+distances[2]*distances[2]))+toRelocate.getRadius());	

				float dx21 = (center2[0]-center1[0]);
				float dy21 = (center2[1]-center1[1]);
				float dz21 = (center2[2]-center1[2]);		

				SimpleVector nanoObjRelativeCenter=new SimpleVector(dx21,dy21,dz21);
				//double angleP=getAngle(domain.getDirectionP(),nanoObjRelativeCenter);
				//double angleN=(Math.PI/2)-angleP;				
				double angleN=owner.getAngleN(nanoObjRelativeCenter, Math.abs(distances[1]), Math.abs(distances[2]));


				if (distanceP>=owner.getHeight()/2){


					// CHECK IF ABSORB OR BOUNCE
					if(bottomless || distances[0]>0){	// ok
									
						
						// ABSORB
						AbstractDomain parent=domain.getParentDomain();
						absorb(toRelocate,parent);			

					}else{

						// BOUNCE

						float[] velocity1=toRelocate.getVelocityComponent();
						SimpleVector velocityVector=new SimpleVector(velocity1);
						float ds=(distanceP-(owner.getHeight()/2));	

						// get the velocity component along the longitudinal axis!
						float angleVP=velocityVector.calcAngle(domain.getDirectionP());
						double vP=Math.abs(modVelocity1*Math.cos(angleVP));					
						// end of the velocity component elaboration...

						dt = Math.abs(ds/vP); // Warning! catch the div-by-zero!!!  //NEW APPROACH

						if (dt>1.1) {
							dt=1;						
						}

						center1[0]+=-velocity1[0]*dt;
						center1[1]+=-velocity1[1]*dt;
						center1[2]+=-velocity1[2]*dt;
						toRelocate.setCenter(center1[0], center1[1], center1[2]);

						SimpleVector v1=new SimpleVector(toRelocate.getVelocityComponent());
						SimpleVector d=domain.getDirectionP();
						double dModule=Math.sqrt(d.x*d.x+d.y*d.y+d.z*d.z);
						float[] dVersorComponent=new float[3];
						dVersorComponent[0]=(float) (d.x/dModule);
						dVersorComponent[1]=(float) (d.y/dModule);
						dVersorComponent[2]=(float) (d.z/dModule);

						SimpleVector dVersor=new SimpleVector(dVersorComponent);

						float[] nanoCenter=toRelocate.getCenter();
						float x=nanoCenter[0]-domainCenter[0];
						float y=nanoCenter[1]-domainCenter[1];
						float z=nanoCenter[2]-domainCenter[2];

						float relativeCenterModule=(float) Math.sqrt(x*x+y*y+z*z);
						SimpleVector nanoCenterVersor=new SimpleVector(x/relativeCenterModule,y/relativeCenterModule,z/relativeCenterModule);
						SimpleVector oTempVersor=nanoCenterVersor.calcCross(dVersor);

						if(oTempVersor.length()!=0){
							oTempVersor=new SimpleVector(oTempVersor.x/oTempVersor.length(),oTempVersor.y/oTempVersor.length(),oTempVersor.z/oTempVersor.length());
						}else{
							oTempVersor=new SimpleVector(0,0,0);
						}

						SimpleVector nTempVersor=oTempVersor.calcCross(dVersor);

						float alfa=nTempVersor.calcAngle(v1);			
						double n1Module=toRelocate.getVelocityModule()*Math.cos(alfa);
						float[] n1components=new float[3];
						n1components[0]=(float) (n1Module*nTempVersor.x);
						n1components[1]=(float) (n1Module*nTempVersor.y);
						n1components[2]=(float) (n1Module*nTempVersor.z);				
						SimpleVector n1=new SimpleVector(n1components);

						float gamma=dVersor.calcAngle(v1);				
						double d1Module=(toRelocate.getVelocityModule()*Math.cos(gamma)); 
						float[] d1components=new float[3];
						d1components[0]=(float) (d1Module*dVersor.x);
						d1components[1]=(float) (d1Module*dVersor.y);
						d1components[2]=(float) (d1Module*dVersor.z);				
						SimpleVector d1=new SimpleVector(d1components);

						float ro=oTempVersor.calcAngle(v1);				
						double o1Module=toRelocate.getVelocityModule()*Math.cos(ro);
						float[] o1components=new float[3];
						o1components[0]=(float) (o1Module*oTempVersor.x);
						o1components[1]=(float) (o1Module*oTempVersor.y);
						o1components[2]=(float) (o1Module*oTempVersor.z);				
						SimpleVector o1=new SimpleVector(o1components);

						SimpleVector dsPP=new SimpleVector(dVersor);
						dsPP.scalarMul((float) ds);	

						//InelasticCollision
						d1.scalarMul(inelasticCoefficient);			
						// END InelasticCollision

						n1.add(o1);
						SimpleVector v2=n1.calcSub(d1);

						//float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
						//float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];
						float[] angles=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z);		
						float fi1=angles[0];
						float theta1=angles[1];
						float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
						toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);	

						velocity1=toRelocate.getVelocityComponent();

						center1[0]+=velocity1[0]*dt;
						center1[1]+=velocity1[1]*dt;
						center1[2]+=velocity1[2]*dt;
						toRelocate.setCenter(center1[0], center1[1], center1[2]);	

						// Avoids that the nano-object crosses the edge				
						float[] distances2=getDistanceOnAxis(domain,toRelocate);					
						float dP= Math.abs(distances2[0]+toRelocate.getRadius());							
						ds=dP-owner.getHeight()/2;					

						SimpleVector correction=new SimpleVector(dVersor);

						if(ds>0){									

							SimpleVector xyz=new SimpleVector(x,y,z);
							float angleCorr=correction.calcAngle(xyz);

							correction.scalarMul((float) -Math.abs((ds)));

							float x1=(float) (correction.x/Math.cos(angleCorr));
							float y1=(float) (correction.y/Math.cos(angleCorr));
							float z1=(float) (correction.z/Math.cos(angleCorr));

							center1[0]+=x1;
							center1[1]+=y1;
							center1[2]+=z1;
							toRelocate.setCenter(center1[0], center1[1], center1[2]);	

						}else{
							correction.scalarMul(0);
						}					

						// Checks that each nano object is still inside this domain
						distances=getDistanceOnAxis(domain,toRelocate);							
						distanceP= Math.abs(distances[0]+toRelocate.getRadius());
						distanceN= Math.abs(distances[1]+toRelocate.getRadius());	

						if (distanceP-2*toRelocate.getRadius()>owner.getHeight()/2 ){
							toRelocate.setColor(Color.green);
							System.out.println("P - The relocateInner has not compute very well the new position of the nano objects!!! "+distanceP+"  -  "+distanceN+"  -  V: "+toRelocate.getVelocityModule());
						}	
					}

				} else if (distanceN>=owner.getRadius()){		

					/** =======================
					 *  GET the AXIS d, n and o! 
					 * ========================= */
					SimpleVector v1=new SimpleVector(toRelocate.getVelocityComponent());
					SimpleVector dVersor=domain.getDirectionP();

					float[] nanoCenter=toRelocate.getCenter();
					float x=nanoCenter[0]-domainCenter[0];
					float y=nanoCenter[1]-domainCenter[1];
					float z=nanoCenter[2]-domainCenter[2];

					float relativeCenterModule=(float) Math.sqrt(x*x+y*y+z*z);
					SimpleVector nanoCenterVersor=new SimpleVector(x/relativeCenterModule,y/relativeCenterModule,z/relativeCenterModule);
					SimpleVector oTempVersor=nanoCenterVersor.calcCross(dVersor);
					oTempVersor=new SimpleVector(oTempVersor.x/oTempVersor.length(),oTempVersor.y/oTempVersor.length(),oTempVersor.z/oTempVersor.length());

					// The N versor points towards inside of domain, such as towards its center (over the secant plane and the d versor)
					SimpleVector nTempVersor=oTempVersor.calcCross(dVersor);
					nTempVersor=nTempVersor.normalize();


					double ds=(distanceN-(owner.getRadius()));	
					float[] velocity1=toRelocate.getVelocityComponent();

					dt =(float) Math.abs((ds/Math.cos(angleN)))/Math.abs(toRelocate.getVelocityModule()); // Warning! catch the div-by-zero!!!

					if (dt>1.1) {
						dt=1;						
					}

					center1=toRelocate.getCenter();	
					center1[0]+=-velocity1[0]*dt;
					center1[1]+=-velocity1[1]*dt;
					center1[2]+=-velocity1[2]*dt;
					toRelocate.setCenter(center1[0], center1[1], center1[2]);


					nanoCenter=toRelocate.getCenter();

					x=nanoCenter[0]-domainCenter[0];
					y=nanoCenter[1]-domainCenter[1];
					z=nanoCenter[2]-domainCenter[2];

					relativeCenterModule=(float) Math.sqrt(x*x+y*y+z*z);
					nanoCenterVersor=new SimpleVector(x/relativeCenterModule,y/relativeCenterModule,z/relativeCenterModule);
					oTempVersor=nanoCenterVersor.calcCross(dVersor);
					oTempVersor=new SimpleVector(oTempVersor.x/oTempVersor.length(),oTempVersor.y/oTempVersor.length(),oTempVersor.z/oTempVersor.length());

					// The N versor points towards inside of domain, such as towards its center (over the secant plane and the d versor)
					nTempVersor=oTempVersor.calcCross(dVersor);
					nTempVersor=nTempVersor.normalize();



					float alfa=nTempVersor.calcAngle(v1);		
					double n1Module=Math.abs(toRelocate.getVelocityModule()*Math.cos(alfa));  

					float[] n1components=new float[3];
					n1components[0]=(float) (n1Module*nTempVersor.x);	
					n1components[1]=(float) (n1Module*nTempVersor.y);
					n1components[2]=(float) (n1Module*nTempVersor.z);				
					SimpleVector n1=new SimpleVector(n1components);
					SimpleVector dsNN=new SimpleVector(nTempVersor);

					float angleN1=nanoCenterVersor.calcAngle(n1);
					float ds_On_N1_Module=(float) (ds*Math.cos(angleN1));
					dsNN.scalarMul(Math.abs(ds_On_N1_Module));		

					float gamma=dVersor.calcAngle(v1);	
					double d1Module=toRelocate.getVelocityModule()*Math.cos(gamma);
					float[] d1components=new float[3];
					d1components[0]=(float) (d1Module*dVersor.x);
					d1components[1]=(float) (d1Module*dVersor.y);
					d1components[2]=(float) (d1Module*dVersor.z);				
					SimpleVector d1=new SimpleVector(d1components);

					float ro=oTempVersor.calcAngle(v1);				
					double o1Module=toRelocate.getVelocityModule()*Math.cos(ro);
					float[] o1components=new float[3];
					o1components[0]=(float) (o1Module*oTempVersor.x);
					o1components[1]=(float) (o1Module*oTempVersor.y);
					o1components[2]=(float) (o1Module*oTempVersor.z);				
					SimpleVector o1=new SimpleVector(o1components);

					n1.scalarMul(inelasticCoefficient);
					d1.add(o1);

					SimpleVector v2=new SimpleVector(d1);
					v2.add(n1);	

					//float fi1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[0];
					//float theta1=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z)[1];
					float[] angles=toRelocate.getVelocityAngles(v2.x,v2.y, v2.z);		
					float fi1=angles[0];
					float theta1=angles[1];
					float nextVelocityModule1=(float) Math.sqrt(v2.x*v2.x+v2.y*v2.y+v2.z*v2.z);
					toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);	

					velocity1=toRelocate.getVelocityComponent();

					center1[0]+=velocity1[0]*dt;
					center1[1]+=velocity1[1]*dt;
					center1[2]+=velocity1[2]*dt;
					toRelocate.setCenter(center1[0], center1[1], center1[2]);


					x=center1[0]-domainCenter[0];
					y=center1[1]-domainCenter[1];
					z=center1[2]-domainCenter[2];

					relativeCenterModule=(float) Math.sqrt(x*x+y*y+z*z);
					nanoCenterVersor=new SimpleVector(x/relativeCenterModule,y/relativeCenterModule,z/relativeCenterModule);
					oTempVersor=nanoCenterVersor.calcCross(dVersor);
					oTempVersor=new SimpleVector(oTempVersor.x/oTempVersor.length(),oTempVersor.y/oTempVersor.length(),oTempVersor.z/oTempVersor.length());

					// The N versor points towards inside of domain, such as towards its center (over the secant plane and the d versor)
					nTempVersor=oTempVersor.calcCross(dVersor);
					nTempVersor=nTempVersor.normalize();


					// Avoids that the nano-object crosses the edge						
					float[] distances2=getDistanceOnAxis(domain,toRelocate);					
					//float dN= Math.abs(distances2[1]+toRelocate.getRadius());
					float dN= (float) ((Math.sqrt(distances2[1]*distances2[1]+distances2[2]*distances2[2]))+toRelocate.getRadius());	
					ds=dN-owner.getRadius();					

					SimpleVector correction=new SimpleVector(nTempVersor);					

					if(ds>0){		
						// N					

						SimpleVector xyz=new SimpleVector(x,y,z);
						float angleCorr=correction.calcAngle(xyz);

						correction.scalarMul((float) -Math.abs((ds)));

						float x1=(float) (correction.x/Math.cos(angleCorr));
						float y1=(float) (correction.y/Math.cos(angleCorr));
						float z1=(float) (correction.z/Math.cos(angleCorr));

						center1[0]+=x1;
						center1[1]+=y1;
						center1[2]+=z1;
						toRelocate.setCenter(center1[0], center1[1], center1[2]);						

					}else{
						correction.scalarMul(0);
					}

					// Checks that the nano objects is still inside this domain
					distances=getDistanceOnAxis(domain,toRelocate);		
					distanceP= Math.abs(distances[0]+toRelocate.getRadius());
					//distanceN= Math.abs(distances[1]+toRelocate.getRadius());	
					distanceN= (float) ((Math.sqrt(distances[1]*distances[1]+distances[2]*distances[2]))+toRelocate.getRadius());


					if (distanceP-2*toRelocate.getRadius()>owner.getHeight()/2 || distanceN-2*toRelocate.getRadius()>owner.getRadius()){
						toRelocate.setColor(Color.green);
						System.out.println("N - The relocateInner has not compute very well the new position of the nano objects!!! "+distanceP+"  -  "+distanceN+"  -  V: "+toRelocate.getVelocityModule());
					}	
				}
			}
		}		
	}

}
