package simulator;

import java.util.List;
import java.util.Map;

import com.threed.jpct.SimpleVector;

import MainLibrary.AbstractCollisionDomainStrategy;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.Manager;
import MainLibrary.MobilityModel;
import MainLibrary.NanoObj;
import MainLibrary.NodeObj;


public class ShellSphericalDomain extends AbstractDomain{	

	private float radius=0;		
	private SimpleVector parabolicFocusVersor;
	private float angleCalotta;	// [0; 1] it is expressed in radiants! e.g. [0; pi]
	private float lengthCalotta;
	
	
	public ShellSphericalDomain(AbstractDomain parentDomain, MobilityModel model, AbstractCollisionDomainStrategy strategy, boolean visible, boolean export, float radius, float[] dCenter, SimpleVector parabolicFocusVersor, float angleCalotta, boolean useOctree) {
		super(parentDomain, model, strategy, visible, export, dCenter, useOctree);			
		setRadius(radius);
		setBoundingRadius();		
		ID="SHELL SPHERICAL DOMAIN";
		
		this.parabolicFocusVersor=parabolicFocusVersor.normalize();		// Points on the center of the transmitter node!
		this.angleCalotta=(float) (angleCalotta*Math.PI);
		//lengthCalotta=(float) (radius*Math.cos(this.angleCalotta));
		lengthCalotta=(float) (radius*(1+Math.cos(this.angleCalotta)));
		
		
		if(useOctree) initializeOctree(getManager().configurator);

		if (isVisible()) getManager().engine.addWrapper3DDomain(this);	
	}


	public void setRadius(float radius) {
		this.radius = radius;
	}	

	public void setParabolicVersor(SimpleVector v){		
		this.parabolicFocusVersor=new SimpleVector(v.normalize());
	}
	
	public SimpleVector getParabolicVersor(){
		return parabolicFocusVersor;
	}
	
	public float getAngleCalotta(){
		return angleCalotta;
	}
	
	public float getCalotta(){
		return lengthCalotta;
	}

	public void handleBoundCollision(NanoObj n){		
		if(n.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(n,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(n,this);
		}
	}

	public void handleBoundCollision(NodeObj n){
		if(n.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(n,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(n,this);
		}
	}

	public void handleBoundCollision(CarrierObj c){			
		if(c.getDomain()==this){
			getCollisionStrategy().relocateInnerObjs(c,this);
		}else{
			getCollisionStrategy().relocateOuterObjs(c,this);
		}
	}

	public String getType(){
		return "SHELL SPHERICAL DOMAIN";
	}	

	public float getRadius() {
		return radius;
	}

	@Override
	public void setBoundingRadius() {
		boundingRadius=getRadius();		
	}	

	@Override
	public boolean isInside(NanoObj objToCheck) {
		//FIXME To be deeper tested...


		// If the Obj's center is inside, then the whole object is considered inside!		
		boolean isInside=false;
		float[] distances=getCollisionStrategy().getDistanceOnAxis(this,objToCheck);			
		float distanceP= distances[0]-objToCheck.getRadius();		
		float distanceN= distances[1]-objToCheck.getRadius();			
		float distanceO= distances[2]-objToCheck.getRadius();	

		if (((Math.abs(distanceP))<=getRadius()) && (Math.abs(distanceN)<=getRadius()) && (Math.abs(distanceO)<=getRadius())){	
			isInside=true;
		} 
		return isInside;
	}

	@Override
	public void initializeWrapper3D(Manager m){
		if(m.is3DEnable) getManager().engine.addWrapper3DDomain(this);	
	}


	@Override
	public void initializeDirectionAxes() {
		// Sets the three direction axes!

		float[] dCenter1=new float[3];
		dCenter1[0]=0;
		dCenter1[1]=1;
		dCenter1[2]=0;		
		setDirectionP(new SimpleVector(dCenter1));				

		float[] dCenter2=new float[3];
		dCenter2[0]=1;
		dCenter2[1]=0;
		dCenter2[2]=0;
		setDirectionN(new SimpleVector(dCenter2));		
	}
	/*
	public void detachNanoObj(CarrierObj c){
		undisposedCarrierList.remove(c);
		carrierList.remove(c);
		getLivingObjObserver().removeCarrier(c);		
		//c.setDomain(null);

		try {
			// Octree
			//getRootTree().remove(c);
			removeFromOctree(c);

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("Eccezione AbstractDomain.detachCarrier -> OCTREE - "+ID);
		}
	}*/	


	public boolean isFreeSpace(NanoObj obj){
		float[] coords=obj.getCenter();
		float radius=obj.getRadius();
		boolean isFree=true;
		float[] tempCoords=new float[3];
		double tempRadius=0;
		boolean endSearch=false;

		List<AbstractDomain> childDomainList=getAllChildDomains();		
		for(int i=0;i<childDomainList.size() && isFree;i++){
			if(childDomainList.get(i).isInside(obj)) isFree=false;
		}			

		if(isFree){
			List<NanoObj> allNanoObjList=getAllDisposedObj();			

			for (int i=0;i<allNanoObjList.size()& (!endSearch);i++){
				tempCoords=allNanoObjList.get(i).getCenter();
				tempRadius=allNanoObjList.get(i).getRadius();
				//check=i;
				float distance=getDistance(coords,tempCoords);

				if (distance<(radius+tempRadius)){					
					isFree=false;
					endSearch=true;
				}
			}
		}	
		return isFree;
	}
	
	
	public void activatePropagationObj(NanoObj n){
		if(undisposedGenericObjList.contains(n)) undisposedGenericObjList.remove(n);
		if (!genericObjList.contains(n)) {
			genericObjList.add(n);				
		}
		getMobilityModel().getRootDomain().attachOnGrid(n);
	}


	@Override
	public void trigger() {
		// TODO Auto-generated method stub
		
	}


	@Override
	public Map<Object, Object> export() {
		// TODO Auto-generated method stub
		return null;
	}	
}