package simulator;

import java.awt.Color;
import java.math.BigDecimal;
import java.util.List;
import com.threed.jpct.Matrix;
import com.threed.jpct.SimpleVector;

import MainLibrary.*;

public class ShellSphericalStrategy extends AbstractCollisionDomainStrategy{
	
	//private ShellSphericalDomain domainToCheck;
	
	public ShellSphericalStrategy(int type){
		super(type);		
	}

	public void checkInternalCollisionBound(List<NanoObj> objsToCheck) {		
		boolean isFinish=false;
		for (int i=0;i<objsToCheck.size() && !isFinish;i++){			
			try{

				/** FIXME You have to control also the collisions with the other domains that share the same parentDomain! 
				 * So it is needed also the collision control on the external shells. 
				 * In what follows it considers only the collisions with inner shell. */  
				//ShellSphericalDomain domainToCheck=(ShellSphericalDomain)domain;

				// TIP: You can evaluate the bounding sphere to perform a preliminary check collision
				if(objsToCheck.get(i).getTimeToLive()>0){

					if((objsToCheck.get(i).getAbsDistancePlusRadius()>=domain.getBoundingRadius())){													
						checkRealDomainBounds(objsToCheck.get(i));						
					} else {						
						isFinish=true;
					}					
				}
			}catch (Exception e){
				e.printStackTrace();
			}
		}		
	}	

	
	// Here you have to evaluate the relative position of any carrierObj with the spherical domain walls! Half sphere is transparent, whereas the other half is full reflective!
	public void checkRealDomainBounds(NanoObj n) {
		
		ShellSphericalDomain domainToCheck=(ShellSphericalDomain)domain;
		
		float[] domainCenter=domain.getAbsCenter();		
		float domainRadius=domainToCheck.getRadius();
		float[] nanoCenter=n.getCenter();
		float nanoRadius=n.getRadius();		
		float[] velocity1=n.getVelocityComponent();
		
		

		float dx = (domainCenter[0]-nanoCenter[0]), dy = (domainCenter[1]-nanoCenter[1]), dz= (domainCenter[2]-nanoCenter[2]);
		float distance = (float) Math.sqrt(dx*dx+dy*dy+dz*dz);

		if (distance>=((domainRadius-nanoRadius))){

			// Unit vector in the direction of the collision
			float ax=(dx/distance), ay=(dy/distance), az=(dz/distance);

			// Projection of the velocities in these axes
			float va1=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az); 
			//float va2=(0*ax+0*ay+0*az);

			// the collision should have occurred at t-dt (Actually this is also an approximation).
			float dt=1;

			try {
				//dt =(float) ((distance-(domainRadius-nanoRadius))/Math.abs(va1-va2)); // Warning! catch the div-by-zero!!!
				dt =(float) ((distance-(domainRadius-nanoRadius))/Math.abs(va1)); // Warning! catch the div-by-zero!!!
			} catch (Exception e) {
				System.out.println(e.getMessage());	
			}	
			domain.addToCollisionsListInternal(domain,n,dt);	
			//addToCollisionsList(n,dt);
		}
	}		
	

	public boolean isUnderTheDomeBAK(float c1, float c2, float c3){
		// CHECK: REFLECTIVE SCREEN OR ABSORBENT SCREEN?
		boolean result=false;
		ShellSphericalDomain domainToCheck=(ShellSphericalDomain)domain;
		float ld=domainToCheck.getCalotta();

		SimpleVector cVersor=new SimpleVector(c1, c2, c3);
		SimpleVector versor=domainToCheck.getParabolicVersor();
		float projection=cVersor.calcDot(versor);
		if(projection<=ld){
			result=true;			
		}
		return result;
	}
	
	
	public boolean isUnderTheDome(float c1, float c2, float c3, float rr){
		// CHECK: REFLECTIVE SCREEN OR ABSORBENT SCREEN?
		boolean result=false;
		ShellSphericalDomain domainToCheck=(ShellSphericalDomain)domain;
		float ld=domainToCheck.getCalotta();

		SimpleVector cVersor=new SimpleVector(c1, c2, c3);
		SimpleVector versor=domainToCheck.getParabolicVersor();
		float scalar=cVersor.calcDot(versor);
		float projection=domainToCheck.getRadius()+scalar-rr;
		if(projection<=ld){
			result=true;			
		}
		return result;
	}

	
	public void relocateInnerObjs(NanoObj toRelocate, AbstractDomain domain) {

		ShellSphericalDomain owner=(ShellSphericalDomain)domain;

		float[] center1=toRelocate.getCenter();
		float[] center2=owner.getAbsCenter();
		float[] velocity1=toRelocate.getVelocityComponent();
		float r1=toRelocate.getRadius();
		float r2=owner.getRadius();
		float modVelocity1=toRelocate.getVelocityModule();		
		double dt=0;		
		

		if (modVelocity1==0){

			// If both velocities are zero, you may want to handle the relocation in some way...

		} else{

			
			float dx21 = (center2[0]-center1[0]);
			float dy21 = (center2[1]-center1[1]);
			float dz21 = (center2[2]-center1[2]);		

			float distance = (float) Math.sqrt(dx21*dx21+dy21*dy21+dz21*dz21);
							
			
			// CHECK: REFLECTIVE SCREEN OR ABSORBENT SCREEN?			
			
			if(isUnderTheDome(-dx21,-dy21,-dz21,r1)){
				
				// BOUNCE!
				
				if (distance>((r2-r1))){	
					float modDiffVelocity=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
					dt = ((distance-(r2-r1))/modDiffVelocity);			
					if (dt>1.1) {
						dt=1;					
					}					
					
					center1[0]+=-velocity1[0]*dt;
					center1[1]+=-velocity1[1]*dt;
					center1[2]+=-velocity1[2]*dt;
					toRelocate.setCenter(center1[0], center1[1], center1[2]);

					
					//================ NEW =================
					SimpleVector versorRadius=new SimpleVector(-dx21,-dy21,-dz21);
					versorRadius=versorRadius.normalize();	
					
					SimpleVector velocity=new SimpleVector(velocity1[0],velocity1[1],velocity1[2]);						
					float angleBounce=versorRadius.calcAngle(velocity);
					
					//float module1=velocity.length();
					
					
					/*
					//-----		ORIGINAL -------------------------------------------				
					SimpleVector orto=(versorRadius).calcCross(velocity);			
					Matrix rotation = new Matrix();
					rotation.rotateAxis(orto, (float) (-(Math.PI/2)*angleBounce));
					velocity.rotate(rotation);					
					//---------------------------------------------------------------
					*/
					
					
					/*
					//--------	DEBUG	WTF???	--------------------------				
					int cross_factor=1;
					boolean warning=false;				
					
					if(angleBounce==Math.PI/2 || angleBounce==0){
						warning=true;
					}else					
					if(angleBounce<(Math.PI/2) && angleBounce>0){
						cross_factor=1;
					}else					
					if(angleBounce>(Math.PI/2) && angleBounce<(Math.PI)){
						cross_factor=-1;
					}					
					
					SimpleVector orto=(versorRadius).calcCross(velocity);			
					Matrix rotation = new Matrix();
					//rotation.rotateAxis(orto, (float) (-(Math.PI/2)*angleBounce));  // original???
					//rotation.rotateAxis(orto, (float) ((-2*angleBounce)));		// debug??
					rotation.rotateAxis(orto, (float) ((-2*cross_factor*angleBounce)));		// debug??
					velocity.rotate(rotation);					
					//----- END DEBUG WTF ??? ---------------------------
					*/					
					
					// ============== END NEW ==================
			
					
					
					
					
					// ================ SECOND APPROACH ==========
					SimpleVector orto_da=(versorRadius).calcCross(velocity);	
					orto_da=orto_da.normalize();
					
					SimpleVector orto_nn=(orto_da).calcCross(versorRadius);
					orto_nn=orto_nn.normalize();			
										
					SimpleVector v_new=new SimpleVector(0,0,0);
					double comp_d=velocity.length()*Math.cos(angleBounce);
					double comp_n=velocity.length()*Math.sin(angleBounce);
					
					if(angleBounce<=Math.PI/2){
						orto_nn.scalarMul((float) comp_n);
						versorRadius.scalarMul((float) -comp_d);
					}else{
						orto_nn.scalarMul((float) -comp_n);
						versorRadius.scalarMul((float) comp_d);
					}					
					//orto_nn.scalarMul((float) comp_n);
					//versorRadius.scalarMul((float) -comp_d);
					
					v_new.add(versorRadius);
					v_new.add(orto_nn);
					
					//float angle_new=versorRadius.calcAngle(v_new);
					velocity=v_new;					
					// =============== END SECOND ================
					
					
					
					float[] angles=toRelocate.getVelocityAngles(velocity.x, velocity.y, velocity.z);
					toRelocate.setVelocityComponent(velocity.length(), angles[0], angles[1]);			

					velocity1=toRelocate.getVelocityComponent();

					center1[0]+=velocity1[0]*dt;
					center1[1]+=velocity1[1]*dt;
					center1[2]+=velocity1[2]*dt;
					toRelocate.setCenter(center1[0], center1[1], center1[2]);	
					
					
					

					//FIXME DEBUG!!!
					/*			
					float dx211 = (center2[0]-center1[0]);
					float dy211 = (center2[1]-center1[1]);
					float dz211 = (center2[2]-center1[2]);		

					float distance2 = (float) Math.sqrt(dx211*dx211+dy211*dy211+dz211*dz211);
					
					if(distance2>(r2-r1)*1.001){
						toRelocate.setColor(Color.green);
						System.out.println();
						System.out.println(">>> SHELL SPHERICAL STRATEGY - Relocate INNER - Increased Distance! d2: "+distance2+" > d1: "+distance);
						System.out.println(">>> ["+velocity.x+", "+velocity.y+", "+velocity.z+"]  ["+velocity1[0]+", "+velocity1[1]+", "+velocity1[2]+"]");
						System.out.println("  -  angle: "+(angleBounce*180/Math.PI)+"  -  "+toRelocate.ID);
						System.out.println();
											
					}*/
					// END FIXME					
									
				}
				
				
			}else{
				
				// ABSORBENT!				
				
				AbstractDomain parent=domain.getParentDomain();
				absorb(toRelocate,parent);
			}
		}
	}
	
	
	
	
	public void relocateInnerObjs_SILLY(NanoObj toRelocate, AbstractDomain domain) {

		ShellSphericalDomain owner=(ShellSphericalDomain)domain;

		float[] center1=toRelocate.getCenter();
		float[] center2=owner.getAbsCenter();
		float[] velocity1=toRelocate.getVelocityComponent();
		float r1=toRelocate.getRadius();
		float r2=owner.getRadius();
		float modVelocity1=toRelocate.getVelocityModule();
		float modVelocity2=0;
		double dt=0;		
		

		if (modVelocity1==0 && modVelocity2==0){

			// If both velocities are zero, you may want to handle the relocation in some way...

		} else{

			float dx21 = (center2[0]-center1[0]);
			float dy21 = (center2[1]-center1[1]);
			float dz21 = (center2[2]-center1[2]);		

			float distance = (float) Math.sqrt(dx21*dx21+dy21*dy21+dz21*dz21);
			
			
			// CHECK: REFLECTIVE SCREEN OR ABSORBENT SCREEN?			
			
			if(isUnderTheDome(-dx21,-dy21,-dz21,r1)){
				
				// BOUNCE!
				
				if (distance>((r2-r1))){	
					float modDiffVelocity=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
					dt = ((distance-(r2-r1))/modDiffVelocity);			
					if (dt>1.1) {
						dt=1;					
					}

					center1[0]+=-velocity1[0]*dt;
					center1[1]+=-velocity1[1]*dt;
					center1[2]+=-velocity1[2]*dt;
					toRelocate.setCenter(center1[0], center1[1], center1[2]);

					float fi1=toRelocate.getVelocityAngles(-velocity1[0],-velocity1[1], -velocity1[2])[0];
					float theta1=toRelocate.getVelocityAngles(-velocity1[0], -velocity1[1], -velocity1[2])[1];
					float nextVelocityModule1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
					toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);			

					velocity1=toRelocate.getVelocityComponent();

					center1[0]+=velocity1[0]*dt;
					center1[1]+=velocity1[1]*dt;
					center1[2]+=velocity1[2]*dt;
					toRelocate.setCenter(center1[0], center1[1], center1[2]);	
				}
				
				
			}else{
				
				// ABSORBENT!
				
				// # OLD
				/*
				AbstractDomain d1=toRelocate.getDomain();
				AbstractDomain d2=domain.getParentDomain();
				
				try {
					toRelocate.setColor(Color.green);
					AbstractDomain parent=domain.getParentDomain();
					toRelocate.switchDomain(parent);
					parent.activatePropagationObj(toRelocate);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					System.out.println("  --- DEBUG --- parent: "+d2.ID+"  ||   nanoobj: "+d1.ID);
					e.printStackTrace();
				}
				*/
				
				AbstractDomain parent=domain.getParentDomain();
				absorb(toRelocate,parent);				
				
				// # NEW
				/*
				AbstractDomain parent=domain.getParentDomain();
				switchDomain(toRelocate,parent);	
				parent.activatePropagationObj(toRelocate);
				*/
			}
		}
	}
	/*
	public synchronized void addToCollisionsList(NanoObj p1,float dt){
		AbstractCollision c=new DomainCollisionInternal(p1,p1.getDomain(),dt);

		CalciumSphericalDomain sDomain=(CalciumSphericalDomain)p1.getDomain();		

		domain.getCollisionsList().add(c);

		if (domain.getManager().is3DEnable) {
			p1.setColor(Color.red);			
			if (sDomain.getOwner()!=null) sDomain.setColor(Color.red);
		}
		p1.increaseCollisionCount();
		if (sDomain.getOwner()!=null) sDomain.getOwner().increaseCollisionCount();		
	}*/

	
	public void relocateOuterObjs(NanoObj toRelocate, AbstractDomain domain) {
		// TODO Auto-generated method stub

		ShellSphericalDomain domainToCheck=(ShellSphericalDomain)domain;

		float ed=inelasticCoefficient;

		float[] center1=toRelocate.getCenter();
		float[] center2=domain.getAbsCenter();
		float[] velocity1=toRelocate.getVelocityComponent();
		float r1=toRelocate.getRadius();
		float r2=domainToCheck.getRadius();
		float modVelocity1=toRelocate.getVelocityModule();
		//float modVelocity2=0;
		float mass1=toRelocate.getMass();
		double dt=0;

		// The momentum has to be <= before and after the collision!
		//float momentum1=modVelocity1*mass1;
		float momentumBefore=modVelocity1*mass1;		
		//float momentumBefore=momentum1;


		if (modVelocity1==0){

		} else{

			float dx21 = (center2[0]-center1[0]);
			float dy21 = (center2[1]-center1[1]);
			float dz21 = (center2[2]-center1[2]);			

			float distance = (float) Math.sqrt(dx21*dx21+dy21*dy21+dz21*dz21);

			if (distance<((r1+r2))){

				if(isUnderTheDome(-dx21,-dy21,-dz21,r1)){

					// BOUNCE!

					// Unit vector in the direction of the collision
					float ax=((dx21/distance)), ay=((dy21/distance)), az=((dz21/distance));

					float alfa=0;
					if (modVelocity1*distance!=0){
						try{
							BigDecimal bg = new BigDecimal((velocity1[0]*dx21+velocity1[1]*dy21+velocity1[2]*dz21)/(modVelocity1*distance)); 
							bg = bg.setScale(3, BigDecimal.ROUND_HALF_UP); 
							alfa=(float) Math.acos(bg.doubleValue()); // angle between v and d
						} catch (NumberFormatException e){
							e.printStackTrace();
						}
					} 

					if(ax>1 || ax<-1 || ay>1 || ay<-1 || az>1 || az<-1){
						System.out.println("==========================================================================");
						System.out.println(" RelocateNANOOBJ - gli ax ay e az eccedono i limiti dell'arccos() "+distance);
						System.out.println("==========================================================================");
					}

					float alfaX1=(float) Math.acos((dx21)/(distance));
					float alfaY1=(float) Math.acos((dy21)/(distance));
					float alfaZ1=(float) Math.acos((dz21)/(distance));

					float modVa1=(float) (Math.abs(modVelocity1)*Math.cos(alfa));
					float vax1=(float) (Math.abs(modVa1)*Math.cos(alfaX1));
					float vay1=(float) (Math.abs(modVa1)*Math.cos(alfaY1));
					float vaz1=(float) (Math.abs(modVa1)*Math.cos(alfaZ1));

					float vbx1=velocity1[0]-vax1;
					float vby1=velocity1[1]-vay1;
					float vbz1=velocity1[2]-vaz1;


					float va1t=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az);


					dt =((r1+r2-distance)/Math.abs(va1t)); 			

					if (dt>1.1) {
						dt =1; 						
					}				

					center1[0]-=velocity1[0]*dt;
					center1[1]-=velocity1[1]*dt;
					center1[2]-=velocity1[2]*dt;
					toRelocate.setCenter(center1[0], center1[1], center1[2]);



					float[] vaP1=new float[3];
					vaP1[0]=(float) (vax1 + (1+ed)*(-vax1));
					vaP1[1]=(float) (vay1 + (1+ed)*(-vay1));
					vaP1[2]=(float) (vaz1 + (1+ed)*(-vaz1));



					float vxRound = ((vaP1[0]+vbx1));
					float vyRound = ((vaP1[1]+vby1));
					float vzRound = ((vaP1[2]+vbz1));
					velocity1[0]= vxRound;
					velocity1[1]= vyRound;
					velocity1[2]= vzRound;

					//float fi1=toRelocate.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[0];
					//float theta1=toRelocate.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2])[1];
					float[] angles=toRelocate.getVelocityAngles(velocity1[0], velocity1[1], velocity1[2]);		
					float fi1=angles[0];
					float theta1=angles[1];
					
					float nextVelocityModule1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
					toRelocate.setVelocityComponent(nextVelocityModule1, fi1, theta1);


					// Evaluates the Momentum!
					//momentum1=nextVelocityModule1*mass1;							
					//float momentumAfter=momentum1;
					float momentumAfter=nextVelocityModule1*mass1;
					

					if((momentumBefore*1.1)<momentumAfter){
						float error=momentumBefore/momentumAfter;
						nextVelocityModule1=nextVelocityModule1*error;					

						toRelocate.setVelocityModule(nextVelocityModule1);					
						velocity1=toRelocate.getVelocityComponent();					
					}

					center1[0]+=velocity1[0]*dt;
					center1[1]+=velocity1[1]*dt;
					center1[2]+=velocity1[2]*dt;
					toRelocate.setCenter(center1[0], center1[1], center1[2]);

					modVelocity1=(float) Math.sqrt(velocity1[0]*velocity1[0]+velocity1[1]*velocity1[1]+velocity1[2]*velocity1[2]);
				}else{					
					
					// ABSORBENT!		
					
					// # OLD
					/*
					toRelocate.switchDomain(domain);
					domain.activatePropagationObj(toRelocate);
					 * */
					
					absorb(toRelocate,domain);
					
				}
			}
		}
	}

	
	
	public void absorb(NanoObj toRelocate, AbstractDomain domain){

		if(toRelocate instanceof CarrierObj){
			toRelocate.switchDomain(domain);
			domain.activatePropagationObj((CarrierObj)toRelocate);

		}else{

			toRelocate.switchDomain(domain);
			domain.activatePropagationObj(toRelocate);
		}
	}

	@Override
	public double getExternalCollisionTime(NanoObj collisionObj, AbstractDomain d) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public double getInternalCollisionTime(NanoObj collisionObj, AbstractDomain d) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public boolean isAnInternalCollisionBound(NanoObj n) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isCollidingFromOutside(AbstractDomain domainToCheck,
			NanoObj objToCheck) {
		// TODO Auto-generated method stub
			
		return true;
	}

	@Override
	public void checkInternalCollisionBound(NanoObj objToCheck) {
		// TODO Auto-generated method stub
		//System.out.println("Spherical Strategy - check internal collision - SINGLE!");

		if(objToCheck.getTimeToLive()>0){			
			//ShellSphericalDomain domain= (ShellSphericalDomain) objToCheck.getDomain();
			AbstractDomain domain= objToCheck.getDomain();
			
			// TIP: You can evaluate the bounding sphere to perform a preliminary check collision


			if((objToCheck.getAbsDistancePlusRadius()>=domain.getBoundingRadius())){													
				checkRealDomainBounds(objToCheck);						
			} 				
		}
	}




	@Override
	public void checkExternalCollisionBound(AbstractDomain domainToCheck, NanoObj p1) {
		

		CarrierObj objToCheck=(CarrierObj)p1;

		if(isCollidingFromOutside(domainToCheck, objToCheck)){
			ShellSphericalDomain domain=(ShellSphericalDomain)domainToCheck;

			float[] domainCenter=domain.getAbsCenter();		
			float domainRadius=((ShellSphericalDomain)domain).getRadius();
			float[] nanoCenter=objToCheck.getCenter();
			float nanoRadius=objToCheck.getRadius();		
			float[] velocity1=objToCheck.getVelocityComponent();

			float dx = (domainCenter[0]-nanoCenter[0]), dy = (domainCenter[1]-nanoCenter[1]), dz= (domainCenter[2]-nanoCenter[2]);
			float distance = (float) Math.sqrt(dx*dx+dy*dy+dz*dz);
			//---

			if (distance<((domainRadius+nanoRadius))){


				// Unit vector in the direction of the collision
				float ax=(dx/distance), ay=(dy/distance), az=(dz/distance);

				// Projection of the velocities in these axes
				float va1=(velocity1[0]*ax+velocity1[1]*ay+velocity1[2]*az); 
				//float va2=0;

				// the collision should have occurred at t-dt (Actually this is also an approximation).
				float dt=1;
				try {
					//dt =(float) ((domainRadius+nanoRadius-distance)/Math.abs(va1-va2)); // Warning! catch the div-by-zero!!!
					dt =(float) ((domainRadius+nanoRadius-distance)/Math.abs(va1)); // Warning! catch the div-by-zero!!!
				} catch (Exception e) {
					System.out.println(e.getMessage());	
				}		
				
				domain.getParentDomain().addToCollisionsListExternal(domain,objToCheck,dt);				
			}
		}		

	}


}
