package simulator;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.LigandObserver;
import MainLibrary.NoMotion;
import MainLibrary.NodeObj;
import MainLibrary.OutputStrategy;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.nIdle;
import MainLibrary.nTransmit;


public class SimpleRX extends NodeObj{
	
	public boolean receptorStats=false;
	public boolean receptorRandomTime=false;
	
	public long served=0;
	public long busy=0;
	public long missed=0;	
		
	public boolean absorbentReceiver=false;
	public boolean rejectingReceiver=false;
	
	private long localExportTime=1000;	
	
	private OutputStrategy outputStrategy_Receptor=OutputTXTReceptors.getInstance();

	
	public SimpleRX(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);
				
		// You can set the internal state here!
		//setState(new nIdle(this));
		//getState().setExecutionTime(1000);	
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
		setColor(Color.white);	
		
		informationCount=0;	
		
		
		// EXPORT MySQL
		//localExportTime=domain.getManager().configWrapper.exportTimeLength*OutputDBMySQL.getInstance().getStoringFactor();
		localExportTime=domain.getManager().configWrapper.exportTimeLength;
		
		/*
		setState(new nTransmit(this));
		getState().setExecutionTime(0);
		*/
		
		
	}
	
	@Override
	public void idle() {
		processInfo();
	}
	
	@Override
	public void decode() {
		setState(new nIdle(this));
	}
	
	@Override
	public void encode() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(10);	
	}

	@Override
	public void searchNeighbor() {
		
	}	
	
	@Override
	public void transmit() {			
		
	}
	
	@Override
	public Map<Object, Object> specializedExport() {

		Map<Object, Object> m=new HashMap<Object,Object>();
		
		boolean hasToExport=false;
		long time=getManager().getCurrentTime();
		
		//if(time%10000==0){
		if(time%localExportTime==0){
			hasToExport=true;
		}
		
		if(hasToExport){
			m.put("Type", "Simple-RX-Node");
   
			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		

			m.put("ID", "Simple-RX-Node-"+ID);	
			m.put("X", center[0]-domainCenter[0]);
			m.put("Y", center[1]-domainCenter[1]);
			m.put("Z", center[2]-domainCenter[2]);
			
			
			
				m.put("CRON_TimeStep", time+"");	
				m.put("CRON_Value", informationCount+"");	
				m.put("CRON_Action", "RX assimilation");		
			
			
			if(receptorStats){

			    m.put("SERVED_TimeStep", ""+time+"");	
			    m.put("SERVED_Value", served);	
			    m.put("SERVED_Action", "lambda_a");
			    
			    m.put("BUSY_TimeStep", ""+time+"");	
			    m.put("BUSY_Value", busy);	
			    m.put("BUSY_Action", "lambda_r");
			    
			    m.put("MISSED_TimeStep", ""+time+"");	
			    m.put("MISSED_Value", missed);	
			    m.put("MISSED_Action", "lambda_m");
			    
			    
			    if(time%100000==0){
			    	Thread t = new Thread(new Runnable() { public void run() { 
			    		Map<Object, Object> receptorMap=new HashMap<Object,Object>();

				    	for(Receptor rr : receptorList){
				    		receptorMap.putAll(rr.export());
				    	}		

				    	outputStrategy_Receptor.export(receptorMap);
			    		}});
			    	
			    	t.start();			    	
			    }
			}
			
			m.putAll(getState().export());
		}
		return m;
	}	
	
	
	@Override
	public void createSingleReceptor(float[] cdCenter, float radius, short receptorRadius, int receptorType) {
		float[] angles=convertSphericalAngles(cdCenter[0],cdCenter[1],cdCenter[2], radius);
		float theta=angles[1];
		float fi=angles[0];

		
		TCPLike_ReceptorB r=new TCPLike_ReceptorB(this, true, cdCenter, theta, fi, receptorRadius, receptorType, this.eventHandler);
		receptorList.add(r);
		receptorSetup(r); 

		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		*/		
	}
	
	public void createReceptor(int n){
		Receptor r = null;
		
		float theta=0;
		float fi=0;				
		
		for (int i=0;i<n;i++){	
			
			/*
			// Creates the receptors for UPLOAD
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];		
			
			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new TCPLike_ReceptorB(this, false, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			//receptorSetup(r); 
			r.attachObs(r.getMyObserver());
			*/
			
			// Creates the receptors for DOWNLOAD
			
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];				
			
				
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new TCPLike_ReceptorA(this, true, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler, i);
			receptorList.add(r);
			//receptorSetup(r); 			
			r.attachObs(r.getMyObserver());
		}
	}	
	
	
	public void checkIncomingCarrier(CarrierObj c){		

		// Check if the receiver node is able to assimilate carriers
		// WARNING! If the rejectingReceiver is TRUE, ALL incoming carriers will be rejected!
		if(rejectingReceiver){
			missed++; 	// uses the lambda_m parameter to monitor the total amount of collisions!
			
			getDomain().getMobilityModel().relocateObjs(c,this);
			
		}
		// Check if the surface receptors are used for the assimilation
		// WARNING! If the absorbent is TRUE, no check on the type of carrier assimilated, it will absorb everything!
		else if(!absorbentReceiver){

			boolean catched=false;
			float x,  y,  z,  r;

			float[] coords=null;	

			// FIXME Relocates the carrierObj on the surface of the nanoObj
			float dt=(float) getCollisionTime(c);
			float[] velocity=c.getVelocityComponent();
			// Carrier Coords on the Node External Surface
			x=c.getCenter()[0]-velocity[0]*dt;
			y=c.getCenter()[1]-velocity[1]*dt;
			z=c.getCenter()[2]-velocity[2]*dt;	
			r=(float) c.getRadius();
			// END FIXME


			
			
			if (c.getDomain()==getInnerDomain()){		
				// Internal Domain!			
				for (int i=0;i<receptorList.size() && !catched;i++){				
					if((receptorList.get(i).isFree() && !(receptorList.get(i).canDownload()))){
						coords=receptorList.get(i).getAbsoluteCoords();
						float dim=receptorList.get(i).radius;							
						float dx = (coords[0]-x), dy = (coords[1]-y), dz= (coords[2]-z);
						float distance =(float) Math.sqrt(dx*dx+dy*dy+dz*dz);	

						if (distance<=((dim+r)) && (receptorList.get(i).checkCarrierCompatibility(c))){			
							catched=true;						
							receptorList.get(i).configureForUpload(c);	
							//System.out.println("NodeObj.checkIncomingCarrier() - CARRIER DOWNLOAD!");
						}
					}
				}

				if(!catched){	
					getInnerDomain().getCollisionStrategy().relocateInnerObjs(c,getInnerDomain());					
				}

			} else {			


				// External Domain!
				// ===  NEW FOR SISTEMA A CODA  ===
				boolean isMissed=true;
				for (int i=0;i<receptorList.size() && !catched;i++){
					//if((receptorList.get(i).isFree() && receptorList.get(i).canDownload())){					
					coords=receptorList.get(i).getAbsoluteCoords();
					float dim=receptorList.get(i).radius;								
					float dx = (coords[0]-x), dy = (coords[1]-y), dz= (coords[2]-z);
					float distance =(float) Math.sqrt(dx*dx+dy*dy+dz*dz);									
					if (distance<((dim+r)) && (receptorList.get(i).checkCarrierCompatibility(c))){
						isMissed=false;
						if(receptorList.get(i).isFree()){
							catched=true;
							receptorList.get(i).configureForDownload(c, receptorRandomTime);
							served++;
						}else{
							busy++;
						}
					} 
					//}
				}

				if(!catched){											
					getDomain().getMobilityModel().relocateObjs(c,this);
				}
				if(isMissed){
					if(receptorList.get(0).checkCarrierCompatibility(c)){
						missed++;
					}
				}
				// =======================

			} 
		}else{

			// ABSORBENT RECEIVER!!! NO SURFACE RECEPTORS ARE USED!!!
			downloadInfoFrom(c);
		}
	}
	
	
	
	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	
	}
	
	@Override
	public void setMotionStrategy() {		
		this.motionStrategy=BrownianMotion.getInstance();
		//this.motionStrategy=NoMotion.getInstance();
	}
	
	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}
		
	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	@Override
	public void processInfo() {
		/*
		if(getCollisionCount()>0 && firstContact<0){
			firstContact=getManager().getCurrentTime();				
		}*/	
	}
	
	public String assignName(){		
		return "SimpleRX";		
	}

	public void exec(){
		super.exec();
	}


	public void downloadInfoFrom(CarrierObj c, Receptor r){		
		informationCount++;			
		c.destroy();
	}
	
	public void downloadInfoFrom(CarrierObj c){		
		informationCount++;			
		c.destroy();
	}	
}