package simulator;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.LigandObserver;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.nEncode;
import MainLibrary.nIdle;
import MainLibrary.nTransmit;


public class SimpleTX extends NodeObj{

	public int carrierCreated=0;
	private long firstContact=-1;	
	
	
	public SimpleTX(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);
				
		// You can set the internal state here!
		setState(new nEncode(this));
		getState().setExecutionTime(1);	
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
	}
	
	@Override
	public void idle() {
		if (carrierCreated==0 && burstEmission>0){
			activateCell();
		}
	}
	
	@Override
	public void decode() {
		setState(new nIdle(this));
		
	}
	
	@Override
	public void encode() {
		createCarriers(burstEmission);
		setState(new nTransmit(this));
	}

	@Override
	public void searchNeighbor() {
		
	}

	@Override
	public void transmit() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(0);		
	}
	
	public void createCarriers(int n){
		while (n>0){	
			
			TCPLike_Carrier_A cc=new TCPLike_Carrier_A(getInnerDomain(), this, getCenter()[0], getCenter()[1], getCenter()[2], 5);			
			cc.setMotionStrategy(BrownianMotion.getInstance());			
			
			n--;
			carrierCreated++;
		}
	}
	
	@Override
	public Map<Object, Object> specializedExport() {

		Map<Object, Object> m=new HashMap<Object,Object>();
		m.put("Type", "Simple-TX-Node");
		
		m.put("firstContact", firstContact);
		m.putAll(getState().export());
		return m;
	}

	public void createReceptor(int n){
		
		Receptor r = null;		
		float theta=0;
		float fi=0;		
		
		for (int i=0;i<n;i++){			
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];		
			
			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new CD154Receptor(this, false, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			receptorSetup(r); 			
		}		
	}	
	
	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	}
	
	@Override
	public void setMotionStrategy() {		
		this.motionStrategy=BrownianMotion.getInstance();
		//this.motionStrategy=BrownianMotion.getInstance();		
	}
	
	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}
		
	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	@Override	
	public void standardBehavior() {
		emitsWithoutReceptors();
	}


	public void emitsWithoutReceptors(){

		float theta=0;
		float fi=0;			

		int undisposedCarriers=getInnerDomain().undisposedCarrierList.size();

		for(int i=undisposedCarriers-1;i>=0;i--){		

			//for (int i=0;i<undisposedCarriers;i++){				

			if(getDomain()==null){
				System.out.println();
				System.out.println("*** SIMPLE_TX: emitsWithoutReceptors() -> DOMINIOP DEL NODO = NULL!");
				System.out.println();
			}

			CarrierObj carrier=getInnerDomain().undisposedCarrierList.get(i);	
			carrier.removeFromActualDomain();
			//carrier.setDomain(getDomain());  			

			fi=(float) (2*Math.PI*(rand.nextFloat())); //fi
			theta=(float) ((Math.acos((2*rand.nextFloat())-1))); //theta

			double radius=getRadius();	
			radius=radius+2*carrier.getRadius();
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));					

			float[] nodeCenter=getCenter();			
			float x=coords[0]+nodeCenter[0];
			float y=coords[1]+nodeCenter[1];
			float z=coords[2]+nodeCenter[2];

			//if (getDomain().isFreeSpace(carrier)){			
			getDomain().attachNanoObj(carrier);
			carrier.disposeOnMedium();
			//getInnerDomain().undisposedCarrierList.remove(carrier);	

			//}else{			
			//	System.out.println(">>> Delayed Upload: "+getProcessingTime());
			//}	

			carrier.setCenter(x, y, z);
			carrier.setVelocityComponent(carrier.getVelocityModule(), fi, theta);	
			
			
		}
		getInnerDomain().undisposedCarrierList.clear();		
	}
	

	public void activateCell() {

		setState(new nEncode(this));
		getState().setExecutionTime(1);	

	}

	public String assignName(){		
		return "SimpleTX";		
	}

	@Override
	public void processInfo() {
		
	}

	@Override
	public void createSingleReceptor(float[] center, float radius, short receptorRadius, int receptorType) {
		// TODO Auto-generated method stub
		
	}	
}