package simulator;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.URLDecoder;
import java.net.UnknownHostException;

import MainLibrary.ConfigWrapper;
import MainLibrary.MobilityModel;
import MainLibrary.NoOutputStrategy;
import engine.Engine3D;


public class Start {


	// Supported simulation types
	private final static String simBlood="Blood";
	private final static String simTCP="TCPLike";
	private final static String simPunctiform="Punctiform";
	private final static String simPunctiformShell="PunctiformShell";
	private final static String simPunctiformCylindricShell="PunctiformCylindricShell";
	private final static String simFreeDiffusion="FreeDiffusion";		// IEEE Summer School TEST LAB

	public static String programArguments="";	

	public static void main(String[] args) {

		int n = Runtime.getRuntime().availableProcessors();
		System.out.println(">>> CPU CORE: "+n);


		Engine3D engine = null;

		for (int i=0;i<args.length;i++){
			if(args[i].contains("3d")){
				engine = new Engine3D();
			}
		}	

		/*
		//  =====================================================================================
		//  ====  This is the internal mobility model for all NodeObjs and for the Manager!  ====
		//  =====================================================================================

		//DiffusionModel model=new DiffusionModel();
		BloodFlowModel model=new BloodFlowModel();
		//CalciumDiffusionModel model=new CalciumDiffusionModel();
		//TCPLike_DiffusionModel model=new TCPLike_DiffusionModel();
		 */

		MobilityModel model=null;


		//  =====================================================================================
		//  ====  Sets the OutputStrategy  ====
		//  =====================================================================================

		//OutputStrategyDBCubicReceptorDomain outputObj=OutputStrategyDBCubicReceptorDomain.getInstance();
		OutputDBMySQL outputObj=OutputDBMySQL.getInstance();
		//NoOutputStrategy outputObj=NoOutputStrategy.getInstance();






		//  =====================================================================================
		//  ====  Read CUSTOM simulation arguments (from command line)  ====
		//  =====================================================================================

		String sim="";

		for (int i=0;i<args.length;i++){
			if(args[i].contains("-sim")){	
				sim=args[i].substring(4);				
			}	
		}

		if(sim.isEmpty()){
			try {
				throw new Exception();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				System.out.println("\nSimulation type not set! Please, specify the -sim argument in the command line. e.g.: -simBlood or -simTCPLike");
				System.exit(0);
			}
		}else{

			if(sim.contentEquals(simBlood)){



				//  =====================================================================================
				//  ====  This is the internal mobility model for all NodeObjs and for the Manager!  ====
				//  =====================================================================================

				//DiffusionModel model=new DiffusionModel();
				//BloodFlowModel model=new BloodFlowModel();					// OK
				//CalciumDiffusionModel model=new CalciumDiffusionModel();
				//TCPLike_DiffusionModel model=new TCPLike_DiffusionModel();
				model=new BloodFlowModel();

				// CUSTOM: For BloodVessel Simulations!			
				for (int i=0;i<args.length;i++){

					//if(args[i].contains("-viscosity")){
					//	float value=Integer.parseInt(args[i].substring(10));
					//	((BloodFlowModel)model).plasmaViscosity=value;
					//	programArguments=programArguments+" [Plasma Viscosity: "+value+"]";
					//}

					if(args[i].contains("-l")){	
						int value=Integer.parseInt(args[i].substring(2));
						((BloodFlowModel)model).plateletLevel=value;
						programArguments=programArguments+" [Platelet Level: "+value+"]";

						//m.plateletLevel=Integer.parseInt(args[i].substring(2));
					}			
					if(args[i].contains("-domainStrategy")){
						int value=Integer.parseInt(args[i].substring(15));
						((BloodFlowModel)model).domainStrategyType=value;
						programArguments=programArguments+" [Domain Strategy: "+value+"]";
					}
					if(args[i].contains("-vmean")){
						float value=Float.parseFloat(args[i].substring(6));
						((BloodFlowModel)model).meanFlowVel=value;
						programArguments=programArguments+" [Mean Velocity: "+value+"]";
					}
					if(args[i].contains("-ECReceptor")){
						int value=Integer.parseInt(args[i].substring(11));
						((BloodFlowModel)model).ECReceptor=value;
						programArguments=programArguments+" [EC Receptors: "+value+"]";
					}
				}	
				// END BloodVessel custom config


			}else if(sim.contentEquals(simTCP)){


				//  =====================================================================================
				//  ====  This is the internal mobility model for all NodeObjs and for the Manager!  ====
				//  =====================================================================================

				//DiffusionModel model=new DiffusionModel();
				//BloodFlowModel model=new BloodFlowModel();					
				//CalciumDiffusionModel model=new CalciumDiffusionModel();
				//TCPLike_DiffusionModel model=new TCPLike_DiffusionModel();	// OK
				model=new TCPLike_DiffusionModel();


				// CUSTOM: For TCPLike Simulations!			
				for (int i=0;i<args.length;i++){
					if(args[i].contains("-d")){
						int value=Integer.parseInt(args[i].substring(2));
						((TCPLike_DiffusionModel)model).simDistance=value;
						programArguments=programArguments+" [Dist: "+value+"]";
					}
					if(args[i].contains("-xd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).xDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-yd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).yDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-zd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).zDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-burst")){
						int value=Integer.parseInt(args[i].substring(6));
						((TCPLike_DiffusionModel)model).burst=value;
						programArguments=programArguments+" [Burst: "+value+"]";
					}
					if(args[i].contains("-txth")){
						int value=Integer.parseInt(args[i].substring(5));
						((TCPLike_DiffusionModel)model).txEmissionThreshold=value;
						programArguments=programArguments+" [TX_Th: "+value+"]";
					}
					if(args[i].contains("-rttth")){
						int value=Integer.parseInt(args[i].substring(6));
						((TCPLike_DiffusionModel)model).RTTAssimilationThreshold=value;
						programArguments=programArguments+" [RTT_Th: "+value+"]";
					}

					if(args[i].contains("-alfa")){
						int value=Integer.parseInt(args[i].substring(5));
						((TCPLike_DiffusionModel)model).alfa=value;
						programArguments=programArguments+" [alfa: "+value+"]";
					}
					if(args[i].contains("-beta")){
						float value=Float.parseFloat(args[i].substring(5));
						((TCPLike_DiffusionModel)model).beta=value;
						programArguments=programArguments+" [beta: "+value+"]";
					}
					if(args[i].contains("-scheduled")){
						int value=Integer.parseInt(args[i].substring(10));
						((TCPLike_DiffusionModel)model).scheduledWindow=value;
						programArguments=programArguments+" [Scheduled: "+value+"]";
					}

					if(args[i].contains("-estimator")){
						String value=args[i].substring(10);
						((TCPLike_DiffusionModel)model).estimator=value;
						programArguments=programArguments+" [Estimator: "+value+"]";
					}
					if(args[i].contains("-sd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).shutDown=value;
						programArguments=programArguments+" [Shut Down Factor: "+value+"]";
					}
					if(args[i].contains("-c4e")){
						String value=args[i].substring(4);
						((TCPLike_DiffusionModel)model).c4e=value;
						programArguments=programArguments+" [Carrier For Emission: "+value+"]";
					}
					if(args[i].contains("-t4e")){
						String value=args[i].substring(4);
						((TCPLike_DiffusionModel)model).t4e=value;
						programArguments=programArguments+" [Time For Emission: "+value+"]";
					}
					if(args[i].contains("-1debug")){
						//String value=args[i].substring(6);
						((TCPLike_DiffusionModel)model).simDebug=true;
						programArguments=programArguments+" [DEBUG MODE]";
					}
					if(args[i].contains("-receptorStats")){
						//String value=args[i].substring(6);
						((TCPLike_DiffusionModel)model).receptorStats=true;
						programArguments=programArguments+" [RECEPTOR STATS]";
					}
					if(args[i].contains("-paramRTT")){
						float value=Float.parseFloat(args[i].substring(9));
						((TCPLike_DiffusionModel)model).paramRTT=value;
						programArguments=programArguments+" [PARAM RTT: "+value+"]";
					}
					if(args[i].contains("-enableHALVE")){
						float value=Float.parseFloat(args[i].substring(12));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).cutEnabled=enabled;
						programArguments=programArguments+" [SEND HALVE: "+enabled+"]";
					}
					if(args[i].contains("-enableSTOP")){
						float value=Float.parseFloat(args[i].substring(11));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).stopEnabled=enabled;
						programArguments=programArguments+" [SEND STOP: "+enabled+"]";
					}
					if(args[i].contains("-receptorRandomTime")){
						((TCPLike_DiffusionModel)model).receptorRandomTime=true;
						programArguments=programArguments+" [RECEPTOR VARIABLE TIME]";
					}
					if(args[i].contains("-packetSize")){
						int val=Integer.parseInt(args[i].substring(11));
						((TCPLike_DiffusionModel)model).packetSize=val;
						programArguments=programArguments+" [PACKET SIZE: "+val+"]";
					}
					if(args[i].contains("-absorbentRX")){
						float value=Float.parseFloat(args[i].substring(12));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).absorbentReceiver=enabled;
						programArguments=programArguments+" [ABSORBENT RX: "+enabled+"]";
					}
					if(args[i].contains("-rejectingRX")){
						float value=Float.parseFloat(args[i].substring(12));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).rejectingReceiver=enabled;
						programArguments=programArguments+" [REJECTING RX: "+enabled+"]";
					}
					if(args[i].contains("-RXIdle")){
						float value=Float.parseFloat(args[i].substring(7));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).RXisIdleOnStart=enabled;
						programArguments=programArguments+" [RX IDLE ON START: "+enabled+"]";
					}	

					if(args[i].contains("-RXShellRadius")){
						float value=Float.parseFloat(args[i].substring(14));						
						((TCPLike_DiffusionModel)model).RXShellRadius=value;
						programArguments=programArguments+" [RX SHELL: REFLECTING SCREEN DISPOSED! Radius: "+value+" nm]";
					}	

					if(args[i].contains("-RXShellAngle")){
						float value=Float.parseFloat(args[i].substring(13));						
						((TCPLike_DiffusionModel)model).RXShellAngle=value;
						programArguments=programArguments+" [RX SHELL: REFLECTING SCREEN DISPOSED! Angle: "+value+" * 3.1415 rad]";
					}	
				}	
				// END TCPLike custom config

			}else if(sim.contentEquals(simPunctiform)){


				//  =====================================================================================
				//  ====  This is the internal mobility model for all NodeObjs and for the Manager!  ====
				//  =====================================================================================


				model=new TCPLike_DiffusionModel("Punctiform");


				// CUSTOM: For TCPLike Simulations!			
				for (int i=0;i<args.length;i++){
					if(args[i].contains("-d")){
						int value=Integer.parseInt(args[i].substring(2));
						((TCPLike_DiffusionModel)model).simDistance=value;
						programArguments=programArguments+" [Dist: "+value+"]";
					}
					if(args[i].contains("-xd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).xDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-yd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).yDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-zd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).zDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-burst")){
						int value=Integer.parseInt(args[i].substring(6));
						((TCPLike_DiffusionModel)model).burst=value;
						programArguments=programArguments+" [Burst: "+value+"]";
					}					

					if(args[i].contains("-receptorStats")){
						//String value=args[i].substring(6);
						((TCPLike_DiffusionModel)model).receptorStats=true;
						programArguments=programArguments+" [RECEPTOR STATS]";
					}

					if(args[i].contains("-absorbentRX")){
						float value=Float.parseFloat(args[i].substring(12));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).absorbentReceiver=enabled;
						programArguments=programArguments+" [ABSORBENT RX: "+enabled+"]";
					}
					if(args[i].contains("-rejectingRX")){
						float value=Float.parseFloat(args[i].substring(12));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).rejectingReceiver=enabled;
						programArguments=programArguments+" [REJECTING RX: "+enabled+"]";
					}
					if(args[i].contains("-RXIdle")){
						float value=Float.parseFloat(args[i].substring(7));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).RXisIdleOnStart=enabled;
						programArguments=programArguments+" [RX IDLE ON START: "+enabled+"]";
					}	
					if(args[i].contains("-cStep")){
						float value=Float.parseFloat(args[i].substring(6));						
						((TCPLike_DiffusionModel)model).cStep=value;
						programArguments=programArguments+" [Carrier Step (punctiform positioning): "+value+"]";
					}	
					if(args[i].contains("-carrierCollision")){
						float value=Float.parseFloat(args[i].substring(17));						
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).cCollisioncontrol=enabled;
						programArguments=programArguments+" [Carrier Collision Control: "+enabled+"]";
					}


				}	
				// END Punctiform custom config



			}else if(sim.contentEquals(simPunctiformShell)){


				//  =====================================================================================
				//  ====  This is the internal mobility model for all NodeObjs and for the Manager!  ====
				//  =====================================================================================

				model=new TCPLike_DiffusionModel("PunctiformShell");

				for (int i=0;i<args.length;i++){
					if(args[i].contains("-d")){
						int value=Integer.parseInt(args[i].substring(2));
						((TCPLike_DiffusionModel)model).simDistance=value;
						programArguments=programArguments+" [Dist: "+value+"]";
					}
					if(args[i].contains("-xd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).xDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-yd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).yDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-zd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).zDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-burst")){
						int value=Integer.parseInt(args[i].substring(6));
						((TCPLike_DiffusionModel)model).burst=value;
						programArguments=programArguments+" [Burst: "+value+"]";
					}
					if(args[i].contains("-receptorStats")){
						//String value=args[i].substring(6);
						((TCPLike_DiffusionModel)model).receptorStats=true;
						programArguments=programArguments+" [RECEPTOR STATS]";
					}
					if(args[i].contains("-absorbentRX")){
						float value=Float.parseFloat(args[i].substring(12));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).absorbentReceiver=enabled;
						programArguments=programArguments+" [ABSORBENT RX: "+enabled+"]";
					}
					if(args[i].contains("-rejectingRX")){
						float value=Float.parseFloat(args[i].substring(12));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).rejectingReceiver=enabled;
						programArguments=programArguments+" [REJECTING RX: "+enabled+"]";
					}
					if(args[i].contains("-RXIdle")){
						float value=Float.parseFloat(args[i].substring(7));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).RXisIdleOnStart=enabled;
						programArguments=programArguments+" [RX IDLE ON START: "+enabled+"]";
					}	
					if(args[i].contains("-cStep")){
						float value=Float.parseFloat(args[i].substring(6));						
						((TCPLike_DiffusionModel)model).cStep=value;
						programArguments=programArguments+" [Carrier Step (punctiform positioning): "+value+"]";
					}	
					if(args[i].contains("-carrierCollision")){
						float value=Float.parseFloat(args[i].substring(17));						
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).cCollisioncontrol=enabled;
						programArguments=programArguments+" [Carrier Collision Control: "+enabled+"]";
					}

					// SHELL!
					if(args[i].contains("-RXShellRadius")){
						float value=Float.parseFloat(args[i].substring(14));						
						((TCPLike_DiffusionModel)model).RXShellRadius=value;
						programArguments=programArguments+" [RX SHELL: REFLECTING SCREEN DISPOSED! Radius: "+value+" nm]";
					}	

					if(args[i].contains("-RXShellAngle")){
						float value=Float.parseFloat(args[i].substring(13));						
						((TCPLike_DiffusionModel)model).RXShellAngle=value;
						programArguments=programArguments+" [RX SHELL: REFLECTING SCREEN DISPOSED! Angle: "+value+" * 3.1415 rad]";
					}	

				}		
				// END Punctiform SHELL custom config



			}else if(sim.contentEquals(simPunctiformCylindricShell)){


				//  =====================================================================================
				//  ====  This is the internal mobility model for all NodeObjs and for the Manager!  ====
				//  =====================================================================================

				model=new TCPLike_DiffusionModel("PunctiformCylindricShell");

				for (int i=0;i<args.length;i++){
					if(args[i].contains("-d")){
						int value=Integer.parseInt(args[i].substring(2));
						((TCPLike_DiffusionModel)model).simDistance=value;
						programArguments=programArguments+" [Dist: "+value+"]";
					}
					if(args[i].contains("-xd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).xDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-yd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).yDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-zd")){
						int value=Integer.parseInt(args[i].substring(3));
						((TCPLike_DiffusionModel)model).zDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-burst")){
						int value=Integer.parseInt(args[i].substring(6));
						((TCPLike_DiffusionModel)model).burst=value;
						programArguments=programArguments+" [Burst: "+value+"]";
					}
					if(args[i].contains("-receptorStats")){
						//String value=args[i].substring(6);
						((TCPLike_DiffusionModel)model).receptorStats=true;
						programArguments=programArguments+" [RECEPTOR STATS]";
					}
					if(args[i].contains("-absorbentRX")){
						float value=Float.parseFloat(args[i].substring(12));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).absorbentReceiver=enabled;
						programArguments=programArguments+" [ABSORBENT RX: "+enabled+"]";
					}
					if(args[i].contains("-rejectingRX")){
						float value=Float.parseFloat(args[i].substring(12));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).rejectingReceiver=enabled;
						programArguments=programArguments+" [REJECTING RX: "+enabled+"]";
					}
					if(args[i].contains("-RXIdle")){
						float value=Float.parseFloat(args[i].substring(7));
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).RXisIdleOnStart=enabled;
						programArguments=programArguments+" [RX IDLE ON START: "+enabled+"]";
					}	
					if(args[i].contains("-cStep")){
						float value=Float.parseFloat(args[i].substring(6));						
						((TCPLike_DiffusionModel)model).cStep=value;
						programArguments=programArguments+" [Carrier Step (punctiform positioning): "+value+"]";
					}	
					if(args[i].contains("-carrierCollision")){
						float value=Float.parseFloat(args[i].substring(17));						
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).cCollisioncontrol=enabled;
						programArguments=programArguments+" [Carrier Collision Control: "+enabled+"]";
					}

					// SHELL!
					if(args[i].contains("-RXShellRadius")){
						float value=Float.parseFloat(args[i].substring(14));						
						((TCPLike_DiffusionModel)model).RXShellRadius=value;
						programArguments=programArguments+" [RX SHELL: REFLECTING SCREEN DISPOSED! Radius: "+value+" nm]";
					}
					
					// SHELL!
					if(args[i].contains("-RXShellHeight")){
						float value=Float.parseFloat(args[i].substring(14));						
						((TCPLike_DiffusionModel)model).RXShellHeight=value;
						programArguments=programArguments+" [RX SHELL: REFLECTING SCREEN DISPOSED! Height: "+value+" nm]";
					}

					if(args[i].contains("-RXShellAngle")){
						float value=Float.parseFloat(args[i].substring(13));						
						((TCPLike_DiffusionModel)model).RXShellAngle=value;
						programArguments=programArguments+" [RX SHELL: REFLECTING SCREEN DISPOSED! Angle: "+value+" * 3.1415 rad]";
					}	
					
					if(args[i].contains("-pattern")){
						String value=(args[i].substring(8));						
						((TCPLike_DiffusionModel)model).punctiformPattern=value;
						programArguments=programArguments+" [TX EMISSION PATTERN: "+value+" ]";
					}
					
					if(args[i].contains("-symboltime")){
						float value=Float.parseFloat(args[i].substring(11));						
						((TCPLike_DiffusionModel)model).symbolTime=value;
						programArguments=programArguments+" [SYMBOL TIME: "+value+" ]";
					}
					
					if(args[i].contains("-align")){
						short value=Short.parseShort(args[i].substring(6));						
						((TCPLike_DiffusionModel)model).alignment=value;
						programArguments=programArguments+" [EMISSION ALIGNMENT: "+value+"]";
					}
					
					if(args[i].contains("-bottomless")){
						float value=Float.parseFloat(args[i].substring(11));						
						boolean enabled=false;
						if(value==1) enabled=true;
						((TCPLike_DiffusionModel)model).isBottomless=enabled;
						programArguments=programArguments+" [Cylinder Domain Bottomless: "+enabled+"]";
					}
					
					if(args[i].contains("-RXBottomDisplacement")){
						float value=Float.parseFloat(args[i].substring(21));						
						((TCPLike_DiffusionModel)model).d=value;
						programArguments=programArguments+" [NODE RX distance from CYLINDER BOTTOM: "+value+" ]";
					}	
					
					if(args[i].contains("-cylDistance")){
						float value=Float.parseFloat(args[i].substring(12));						
						((TCPLike_DiffusionModel)model).cylDistance=value;
						programArguments=programArguments+" [CYLINDER FRONT FACE distance from TX: "+value+" ]";
					}

				}		
				// END Punctiform CYLINDRIC SHELL custom config
			}
			
			
			
			else if(sim.contentEquals(simFreeDiffusion)){


				//  =====================================================================================
				//  ====  This is the internal mobility model for all NodeObjs and for the Manager!  ====
				//  =====================================================================================

				model=new FreeDiffusionModel("FreeDiffusion");

				for (int i=0;i<args.length;i++){
					if(args[i].contains("-d")){
						int value=Integer.parseInt(args[i].substring(2));
						((FreeDiffusionModel)model).simDistance=value;
						programArguments=programArguments+" [Dist: "+value+"]";
					}
					if(args[i].contains("-xd")){
						int value=Integer.parseInt(args[i].substring(3));
						((FreeDiffusionModel)model).xDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-yd")){
						int value=Integer.parseInt(args[i].substring(3));
						((FreeDiffusionModel)model).yDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-zd")){
						int value=Integer.parseInt(args[i].substring(3));
						((FreeDiffusionModel)model).zDisplacement=value;
						//programArguments=programArguments+" - Dist: "+value;
					}
					if(args[i].contains("-burst")){
						int value=Integer.parseInt(args[i].substring(6));
						((FreeDiffusionModel)model).burst=value;
						programArguments=programArguments+" [Burst: "+value+"]";
					}
					if(args[i].contains("-receptorStats")){
						//String value=args[i].substring(6);
						((FreeDiffusionModel)model).receptorStats=true;
						programArguments=programArguments+" [RECEPTOR STATS]";
					}
					if(args[i].contains("-absorbentRX")){
						float value=Float.parseFloat(args[i].substring(12));
						boolean enabled=false;
						if(value==1) enabled=true;
						((FreeDiffusionModel)model).absorbentReceiver=enabled;
						programArguments=programArguments+" [ABSORBENT RX: "+enabled+"]";
					}
					if(args[i].contains("-rejectingRX")){
						float value=Float.parseFloat(args[i].substring(12));
						boolean enabled=false;
						if(value==1) enabled=true;
						((FreeDiffusionModel)model).rejectingReceiver=enabled;
						programArguments=programArguments+" [REJECTING RX: "+enabled+"]";
					}
					if(args[i].contains("-transceiverNode")){
						float value=Float.parseFloat(args[i].substring(16));
						boolean enabled=false;
						if(value==1) enabled=true;
						((FreeDiffusionModel)model).transceiverNode=enabled;
						programArguments=programArguments+" [TRANSCEIVER NODE: "+enabled+"]";
					}
					if(args[i].contains("-transceiverDist_H:")){
						int value=Integer.parseInt(args[i].substring(6));
						((FreeDiffusionModel)model).displacementH=value;
						programArguments=programArguments+" [TRANSCEIVER NODE Displacement: "+value+"]";
					}
					if(args[i].contains("-transceiverDist_L:")){
						int value=Integer.parseInt(args[i].substring(6));
						((FreeDiffusionModel)model).displacementL=value;
						programArguments=programArguments+" [TRANSCEIVER NODE Longitudinal distance: "+value+"]";
					}
					
					
					
					
					

				}		
				// END Free Diffusion TEST LAB
			}


		}


		/*
		// ===  Collecting ALL args[] parameters!  ===			
		for (int i=0;i<args.length;i++){
			programArguments=programArguments+""+args[i]+" ";
		}				
		// ===  END Collecting  ===
		 */


		//  =====================================================================================
		//  ====  Create the Manager (at the end, after the collection of all custom args)  ====
		//  =====================================================================================

		iManager m=new iManager(model, outputObj, engine, true);
		//m.configurator.additionalInfo=programArguments;	


		programArguments=programArguments+"  -  "+model.extraInfo;
		
		// EXTRACT CURRENT PATH -------------------------
		String decodedPath="";
		String path = Start.class.getProtectionDomain().getCodeSource().getLocation().getPath();		

		try {
			decodedPath = URLDecoder.decode(path, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if(decodedPath.length()>0) programArguments=programArguments+" [JAR Path: "+decodedPath+"]";

		// -----------------------------------------------

		try {
			String hostname=InetAddress.getLocalHost().getHostName();
			programArguments=programArguments+"  - "+hostname;
		} catch (UnknownHostException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		/** Update the description of the simulation  */
		ConfigWrapper cc=m.configurator.m;
		programArguments=" [dt: "+cc.timeUnit+" ns]"+programArguments;			
		outputObj.update(programArguments);        // OK

		
		
		System.out.println(programArguments);



		// Start the Normal version (NOT GRID)!
		System.out.println("dt: "+m.get_dt());
		Thread simulator=new Thread(m);
		simulator.start();		
	}	
}
