package simulator;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import com.threed.jpct.SimpleVector;
//import scala.Math;
import engine.Engine3D;
import MainLibrary.AbstractDomain;
import MainLibrary.LigandObserver;
import MainLibrary.LinearMotion;
import MainLibrary.NanoObj;
import MainLibrary.RootDomain;
import Octree.OctreeNode;
import Octree.OctreeRoot;

public class TCPLike_DiffusionModel extends DiffusionModel{

	public int simDistance=10000;	// micrometers between the nodes center
	public int xDisplacement=0;	// micrometers between the nodes center
	public int yDisplacement=0;	// micrometers between the nodes center
	public int zDisplacement=0;	// micrometers between the nodes center
	public int burst=0;	// emission burst
	public int txEmissionThreshold=0;	// emission Threshold upper bound
	public int RTTAssimilationThreshold=1;	// assimilation Threshold for RX node (for RTT estimation)

	// CONFIG FOR -RX NODE-
	public int alfa=1;						// coeff for the exponential laws
	public float beta=1;						// coeff for CUT
	public int scheduledWindow=10000;		// waiting window (scheduled) for evaluate STOP or CUT
	public String estimator="";
	public boolean receptorStats=false;
	public boolean receptorRandomTime=false;
	public float paramRTT=0.5f;
	public boolean cutEnabled=true;
	public boolean stopEnabled=true;
	public boolean absorbentReceiver=false;
	public boolean rejectingReceiver=false;
	public boolean RXisIdleOnStart=false;
	public float RXShellRadius=0;		// SPHERICAL
	public float RXShellAngle=0;		// BOTH SPHERICAL AND CYLINDRIC
	public float RXShellHeight=0;		// CYLINDRIC

	public int packetSize=1;


	// CONFIG FOR -TX NODE-
	public int shutDown=10;

	public String c4e="HALVE";
	public String t4e="NOCHANGE";
	public boolean TXisIdleOnStart=true;

	public boolean simDebug=false;


	public PunctiformEmitter emitter=null;
	public String punctiformPattern="1";	// default: 1 burst
	public float symbolTime=0;				// seconds
	public short alignment=0;				// angle emission point for cylindric  // 0=0, 45=1, 90=2



	// CONFIG FOR -PUNCTIFORM EMISSION-
	float cStep=1;
	boolean cCollisioncontrol=true;	

	boolean isBottomless=false;			// Is CylinderDomain bottomless?

	float d=Float.NaN;					// distance of RX node from the bottom of the cylinder

	float cylDistance=Float.NaN;		// distance of the cylinder front surface from the TX emitting point


	public String type="";


	public TCPLike_DiffusionModel(){
		//threadingStrategy = new CalciumDiffusionThreadingStrategy(this);
	}

	public TCPLike_DiffusionModel(String type){
		//threadingStrategy = new CalciumDiffusionThreadingStrategy(this);
		this.type=type;
	}

	public TCPLike_DiffusionModel(RootDomain d){
		super(d);
		//threadingStrategy = new CalciumDiffusionThreadingStrategy(this);
	}


	public void networkCreation(){		
		//networkCreationTCPLike();							// OK  	-  exact positionse (d = 20, 40 , 50, ecc...)	
		//networkCreationTCPLike_BitErrorRate_Evaluation();
		//networkCreation_DEBUG_OCTREE();					// ONLY FOR DEBUG		


		/*
		if(type.contains("PunctiformCylindricShellMobileRX")){
			networkCreationTCPLike_PunctiformCylindricShell_mobileNodeRX_v2();		// OK - TX PUNCTIFORM SIMULATIONS WITH SHELL AND MOBILE RX INSIDE THE CYLINDER
		}else*/ 
		if(type.contains("PunctiformCylindricShell")){
			//networkCreationTCPLike_PunctiformCylindricShell();			// OK - TX PUNCTIFORM SIMULATIONS WITH SHELL
			//networkCreationTCPLike_PunctiformCylindricShell_mobileNodeRX();	// NEW IMPLEMENTATION, WORKS ON BOTH CylindricShell Cimulations (the oldest and the newest)
			networkCreationTCPLike_PunctiformCylindricShell_mobileNodeRX_v2();	// NEW IMPLEMENTATION, WORKS ON BOTH CylindricShell Cimulations (the oldest and the newest)
		}else if(type.contains("PunctiformShell")){
			networkCreationTCPLike_PunctiformShell();		// OK - TX PUNCTIFORM SIMULATIONS WITH SHELL
		}else if(type.contains("Punctiform")){
			networkCreationTCPLike_Punctiform();			// OK - TX PUNCTIFORM SIMULATIONS
		}else {
			networkCreationTCPLike_OctreeCentered();		// OK  - Node Objs are centered on the center of smaller octree cubes (includes also the SHELL case)
		}
	}




	public void networkCreationTCPLike(){	

		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [TCP-like]"); 
		System.out.println("-------------------------------------");			


		// DOMAIN

		float[] cdCenter=new float[3];
		cdCenter[0]=100000;
		cdCenter[1]=100000;
		cdCenter[2]=100000;	

		//float halfSide=(float) (63508.5/2);  // cube's half side 
		float halfSide=(float) (100000/2);  // cube's half side 


		TCPLike_CubicStrategy ss=new TCPLike_CubicStrategy(0);
		TCPLike_DiffusionModel innerDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube=new CubicDomain(getManager().rootDomain, innerDiffusion, ss, true, false, halfSide, cdCenter, true);


		// NODES

		List<LigandObserver> observer=new ArrayList<LigandObserver>();

		/*
		float xCenterRX=cdCenter[0]-2*halfSide/2+2*halfSide/4;
		float yCenterRX=cdCenter[1]-2*halfSide/2+2*halfSide/4;
		float zCenterRX=cdCenter[2]-2*halfSide/2+2*halfSide/4;	
		 */
		//float halfOctreeSide=boundaryCube.boundingRadius/2;

		boundaryCube.rootTree.cleanRootLeafs();
		boundaryCube.rootTree=null;		
		boundaryCube.rootTree=new OctreeRoot(cdCenter[0], cdCenter[1], cdCenter[2], 2*halfSide, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
		boundaryCube.rootTree.setDomain(boundaryCube);

		/*
		float xCenterRX=cdCenter[0]-halfSide+xDisplacement;
		float yCenterRX=cdCenter[1]-halfSide+yDisplacement;
		float zCenterRX=cdCenter[2]+zDisplacement;
		 */

		// ABS COORDINATES!
		float xCenterRX=xDisplacement;
		float yCenterRX=yDisplacement;
		float zCenterRX=zDisplacement;

		observer.add(TCPLike_CarrierA_Observer.getInstance());
		observer.add(TCPLike_CarrierB_Observer.getInstance());
		TCPLike_Node_RX nodeRX=new TCPLike_Node_RX(boundaryCube,xCenterRX,yCenterRX,zCenterRX,27,observer); 		// ORIGINAL SIMULATION 
		//TCPLike_TEST_RX nodeRX=new TCPLike_TEST_RX(boundaryCube,xCenterRX,yCenterRX,zCenterRX,27,observer); 		// TEST THRESHOLD
		nodeRX.burstEmission=burst;										// ORIGINAL SIMULATION 


		nodeRX.setColor(Color.lightGray);

		/*	
		float xCenter=cdCenter[0]+2*halfSide/2-2*halfSide/4;
		float yCenter=cdCenter[1]+2*halfSide/2-2*halfSide/4;
		float zCenter=cdCenter[2]+2*halfSide/2-2*halfSide/4;	

		float xCenter=xCenterRX-000;
		float yCenter=yCenterRX-000;
		float zCenter=zCenterRX-7500;	
		 */

		//double ttt=Math.sqrt(Math.pow(simDistance,2)/2);
		//double ttt1=Math.pow(5E9,0.5);
		float xCenter=(float) (xCenterRX+Math.sqrt(Math.pow(simDistance,2)/2));
		float yCenter=(float) (yCenterRX+Math.sqrt(Math.pow(simDistance,2)/2));
		float zCenter=zCenterRX;	

		observer=new ArrayList<LigandObserver>();
		observer.add(TCPLike_CarrierB_Observer.getInstance());
		TCPLike_Node_TX nodeTX=new TCPLike_Node_TX(boundaryCube,xCenter,yCenter,zCenter,26,txEmissionThreshold,observer);		// ORIGINAL SIMULATION  
		//TCPLike_TEST_TX nodeTX=new TCPLike_TEST_TX(boundaryCube,xCenter,yCenter,zCenter,26,observer);  		// TEST THRESHOLD
		//nodeTX.burstEmission=burst;			// TEST THRESHOLD
		nodeTX.setColor(Color.darkGray);	


		float r0=nodeRX.getRadius();
		float r1=nodeTX.getRadius();

		float dx=xCenterRX-xCenter;
		float dy=yCenterRX-yCenter;
		float dz=zCenterRX-zCenter;

		float dist=(float) Math.sqrt(dx*dx+dy*dy+dz*dz)-r0-r1;

		System.out.println();
		System.out.println(">>> RX Node  -  x: "+xCenterRX+", y: "+yCenterRX+", z: "+zCenterRX);
		System.out.println(">>> TX Node  -  x: "+xCenter+", y: "+yCenter+", z: "+zCenter);
		System.out.println(">>> TX-RX Distance: "+dist);


		System.out.println(">>> TX-RX Distance: "+dist);
		System.out.println();

		System.out.println(">>> SIMULATION ELEMENTS DISPOSED! ");			
		System.out.println();	
	}

	public void networkCreationTCPLike_OctreeCentered(){	

		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [TCP-like Octree Centered]"); 
		System.out.println("-------------------------------------");			


		// DOMAIN
		/*  ORIGINAL COORDINATES 
		float[] cdCenter=new float[3];
		cdCenter[0]=100000;
		cdCenter[1]=100000;
		cdCenter[2]=100000;	

		float halfSide=(float) (100000/2);  // cube's half side 
		 */

		/**/
		// ENLARGED DOMAIN FOR DEBUG
		float[] cdCenter=new float[3];
		cdCenter[0]=1000000;
		cdCenter[1]=1000000;
		cdCenter[2]=1000000;			

		float halfSide=(float) (500000/2);  // cube's half side 5x

		System.out.println("===================================");
		System.out.println(">>>  CUBE FULL SIDE: "+(halfSide*2)+" nm");
		System.out.println("===================================");

		boolean useOctree1=true;

		TCPLike_CubicStrategy ss=new TCPLike_CubicStrategy(0);
		TCPLike_DiffusionModel innerDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube=new CubicDomain(getManager().rootDomain, innerDiffusion, ss, true, false, halfSide, cdCenter, useOctree1);
		AbstractDomain domainForRXNode=boundaryCube;


		/*  DEBUG CUBE
		float[] cdCenter3=new float[3];
		cdCenter3[0]=-100000;
		cdCenter3[1]=-100000;
		cdCenter3[2]=-100000;			

		float halfSide3=(float) (10000/2);  // cube's half side 


		TCPLike_CubicStrategy ss3=new TCPLike_CubicStrategy(0);
		TCPLike_DiffusionModel innerDiffusion3=new TCPLike_DiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube3=new CubicDomain(boundaryCube, innerDiffusion3, ss3, true, false, halfSide3, cdCenter3, true);
		 */


		// NODES

		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer.add(TCPLike_CarrierA_Observer.getInstance());
		observer.add(TCPLike_CarrierB_Observer.getInstance());



		if(useOctree1){
			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			boundaryCube.rootTree.cleanRootLeafs();
			boundaryCube.rootTree=null;		
			boundaryCube.rootTree=new OctreeRoot(cdCenter[0], cdCenter[1], cdCenter[2], 2*halfSide, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
			boundaryCube.rootTree.setDomain(boundaryCube);
		}

		// RX NODE ABS COORDINATES!
		float xCenterRX=xDisplacement;
		float yCenterRX=yDisplacement;
		float zCenterRX=zDisplacement;


		// -------------------------------
		// WARNING 
		// WARNING
		// WARNING
		// -------------------------------

		// TX NODE ABS COORDINATES			
		float minOctreeLeaf=getManager().configWrapper.nodeSideLimit;		// FIXME ORIGINALE!!! RIABILITARE QUESTA RIGA E COMMENTARE QUELLA SOTTO! 	
		//float minOctreeLeaf=6250;		// SOLO PER ASSORBENTE E RICEVENTE CON OCTREE!
		float disp2=(float) Math.ceil(Math.sqrt(Math.pow(simDistance,2)/2)/minOctreeLeaf);

		float xCenterTX=xCenterRX+minOctreeLeaf*disp2;
		float yCenterTX=yCenterRX+minOctreeLeaf*disp2;
		float zCenterTX=zCenterRX;



		if(RXShellRadius>0){

			// SHELL DOMAIN
			float[] cdCenter1=new float[3];
			cdCenter1[0]=xDisplacement-cdCenter[0];
			cdCenter1[1]=yDisplacement-cdCenter[1];
			cdCenter1[2]=zDisplacement-cdCenter[2];		


			SimpleVector sphereOrientation=new SimpleVector(xCenterTX-xCenterRX,yCenterTX-yCenterRX,zCenterTX-zCenterRX);			
			float calotta=(float) RXShellAngle;
			boolean useOctree=false;

			ShellSphericalStrategy ss1=new ShellSphericalStrategy(0);			

			TCPLike_DiffusionModel innerShellDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);	
			ShellSphericalDomain shell=new ShellSphericalDomain(boundaryCube, innerShellDiffusion, ss1, true, false, RXShellRadius, cdCenter1, sphereOrientation, calotta, useOctree);


			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			if(useOctree){
				shell.rootTree.cleanRootLeafs();
				shell.rootTree=null;		
				shell.rootTree=new OctreeRoot(cdCenter1[0], cdCenter1[1], cdCenter1[2], 2*RXShellRadius, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
				shell.rootTree.setDomain(boundaryCube);
			}


			domainForRXNode=shell;
		}





		//TCPLike_Node_RX nodeRX=new TCPLike_Node_RX(domainForRXNode,xCenterRX,yCenterRX,zCenterRX,27,observer); 		// SHELL SIMULATION 
		TCPLike_Node_RX nodeRX=new TCPLike_Node_RX(boundaryCube,xCenterRX,yCenterRX,zCenterRX,27,observer); 		// ORIGINAL SIMULATION 
		//TCPLike_TEST_RX nodeRX=new TCPLike_TEST_RX(boundaryCube,xCenterRX,yCenterRX,zCenterRX,27,observer); 		// TEST THRESHOLD

		nodeRX.burstEmission=burst;										// ORIGINAL SIMULATION 	
		nodeRX.RTTAssimilationThreshold=RTTAssimilationThreshold;		// ORIGINAL SIMULATION
		nodeRX.alfa=alfa;												// ORIGINAL SIMULATION 	
		nodeRX.beta=beta;												// ORIGINAL SIMULATION 	
		nodeRX.scheduledWindow=scheduledWindow;							// ORIGINAL SIMULATION - Tw	
		nodeRX.estimator=estimator;										// ORIGINAL SIMULATION 	

		nodeRX.currentEmissionStrategy=c4e;		// ORIGINAL SIMULATION  
		nodeRX.timeForEmissionStrategy=t4e;		// ORIGINAL SIMULATION  
		nodeRX.debug=simDebug;
		nodeRX.receptorStats=receptorStats;
		nodeRX.receptorRandomTime=receptorRandomTime;
		nodeRX.param_RTT=paramRTT;
		nodeRX.cutEnabled=cutEnabled;
		nodeRX.stopEnabled=stopEnabled;
		nodeRX.absorbentReceiver=absorbentReceiver;
		nodeRX.rejectingReceiver=rejectingReceiver;

		nodeRX.receivedPacketType=new int[packetSize];

		if(RXisIdleOnStart){
			nodeRX.isFirstStart=false;	
			TXisIdleOnStart=false;
		}

		nodeRX.setColor(Color.lightGray);



		System.out.println();
		System.out.println("===========  RX NODE PARAMETERS  ==========");
		System.out.println(">>>  BURST: "+burst);
		System.out.println(">>>  RTT Threshold: "+RTTAssimilationThreshold);
		System.out.println(">>>  alfa: "+alfa);
		System.out.println(">>>  beta: "+beta);
		System.out.println(">>>  Scheduled Window: "+scheduledWindow);
		System.out.println(">>>  Estimator: "+estimator);
		System.out.println("===========================================");
		System.out.println();



		//===== TEST FOR CUT =====
		//TCPLike_Node_RX_OnlyCUT nodeRX=new TCPLike_Node_RX_OnlyCUT(boundaryCube,xCenterRX,yCenterRX,zCenterRX,27,observer);  		// TEST CUT		
		//========================




		// ============================================================
		// Non raddoppia pi, ma rimanda i segnali incrementando di 1000
		// =============================================================

		/*
		float xCenter=(float) (xCenterRX+Math.sqrt(Math.pow(simDistance,2)/2));
		float yCenter=(float) (yCenterRX+Math.sqrt(Math.pow(simDistance,2)/2));
		float zCenter=zCenterRX;
		 */

		/* SPOSTATO SOPRA!
		float minOctreeLeaf=getManager().configWrapper.nodeSideLimit;		
		float disp2=(float) Math.ceil(Math.sqrt(Math.pow(simDistance,2)/2)/minOctreeLeaf);

		float xCenterTX=xCenterRX+minOctreeLeaf*disp2;
		float yCenterTX=yCenterRX+minOctreeLeaf*disp2;
		float zCenterTX=zCenterRX;
		 */

		observer=new ArrayList<LigandObserver>();
		observer.add(TCPLike_CarrierB_Observer.getInstance());
		TCPLike_Node_TX nodeTX=new TCPLike_Node_TX(boundaryCube,xCenterTX,yCenterTX,zCenterTX,26,txEmissionThreshold,observer);		// ORIGINAL SIMULATION  
		nodeTX.shutDown=shutDown;				// ORIGINAL SIMULATION  
		nodeTX.currentEmissionStrategy=c4e;		// ORIGINAL SIMULATION  
		nodeTX.timeForEmissionStrategy=t4e;		// ORIGINAL SIMULATION  

		nodeTX.transmittedPacketType=packetSize;	

		if(!TXisIdleOnStart){
			nodeTX.transmitNow();
		}


		//TCPLike_TEST_TX nodeTX=new TCPLike_TEST_TX(boundaryCube,xCenter,yCenter,zCenter,26,observer);  		// TEST THRESHOLD
		//nodeTX.burstEmission=burst;			// TEST THRESHOLD

		//===== TEST FOR CUT =====
		//TCPLike_Node_TX_OnlyCUT nodeTX=new TCPLike_Node_TX_OnlyCUT(boundaryCube,xCenter,yCenter,zCenter,26,txEmissionThreshold,observer);  		// TEST CUT		
		//========================

		nodeTX.setColor(Color.darkGray);	

		System.out.println();
		System.out.println("===========  TX NODE PARAMETERS  ==========");
		System.out.println(">>>  # of waiting windows for Shut Down : "+shutDown);
		System.out.println(">>>  Current Emission Strategy: "+c4e);
		System.out.println(">>>  Time For Emission Strategy: "+t4e);
		System.out.println(">>>  Carriers reservoir: "+txEmissionThreshold);	
		System.out.println("===========================================");
		System.out.println();


		float r0=nodeRX.getRadius();		
		float r1=nodeTX.getRadius();

		float dx=xCenterRX-xCenterTX;
		float dy=yCenterRX-yCenterTX;
		float dz=zCenterRX-zCenterTX;

		float dist=(float) Math.sqrt(dx*dx+dy*dy+dz*dz)-r0-r1;

		System.out.println();
		System.out.println(">>> RX Node  -  x: "+xCenterRX+", y: "+yCenterRX+", z: "+zCenterRX);
		System.out.println(">>> TX Node  -  x: "+xCenterTX+", y: "+yCenterTX+", z: "+zCenterTX);
		System.out.println(">>> TX-RX Distance: "+dist);


		if(rejectingReceiver && absorbentReceiver){
			System.out.println();
			System.out.println();
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println(">>>  ||  ** WARNING **  You have defined both absorbentReceiver AND rejectingReceiver!  ||");
			System.out.println(">>>  ||   ++++++++++++++   REJECTING RECEIVER RULE WILL BE APPLIED!   ++++++++++++++    ||");
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println();
			System.out.println();
		}


		System.out.println(">>> SIMULATION ELEMENTS DISPOSED! ");			
		System.out.println();	
	}



	public void networkCreationTCPLike_Punctiform(){	

		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [TCP-like Octree Centered - TX node PUNCTIFORM]"); 
		System.out.println("-------------------------------------");			


		// ENLARGED DOMAIN FOR DEBUG
		float[] cdCenter=new float[3];
		cdCenter[0]=1000000;
		cdCenter[1]=1000000;
		cdCenter[2]=1000000;			

		float halfSide=(float) (1000000/2);  // cube's half side 


		boolean useOctree1=false;

		TCPLike_CubicStrategy ss=new TCPLike_CubicStrategy(0);
		TCPLike_DiffusionModel innerDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube=new CubicDomain(getManager().rootDomain, innerDiffusion, ss, true, false, halfSide, cdCenter, useOctree1);
		AbstractDomain domainForRXNode=boundaryCube;


		// NODES

		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer.add(TCPLike_CarrierA_Observer.getInstance());
		observer.add(TCPLike_CarrierB_Observer.getInstance());

		if(useOctree1){
			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			boundaryCube.rootTree.cleanRootLeafs();
			boundaryCube.rootTree=null;		
			boundaryCube.rootTree=new OctreeRoot(cdCenter[0], cdCenter[1], cdCenter[2], 2*halfSide, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
			boundaryCube.rootTree.setDomain(boundaryCube);
		}

		// RX NODE ABS COORDINATES!
		float xCenterRX=xDisplacement;
		float yCenterRX=yDisplacement;
		float zCenterRX=zDisplacement;



		TCPLike_Node_RX nodeRX=new TCPLike_Node_RX(domainForRXNode,xCenterRX,yCenterRX,zCenterRX,29,observer); 		

		nodeRX.burstEmission=burst;										// ORIGINAL SIMULATION 	
		nodeRX.RTTAssimilationThreshold=RTTAssimilationThreshold;		// ORIGINAL SIMULATION
		nodeRX.alfa=alfa;												// ORIGINAL SIMULATION 	
		nodeRX.beta=beta;												// ORIGINAL SIMULATION 	
		nodeRX.scheduledWindow=scheduledWindow;							// ORIGINAL SIMULATION - Tw	
		nodeRX.estimator=estimator;										// ORIGINAL SIMULATION 	

		nodeRX.currentEmissionStrategy=c4e;		// ORIGINAL SIMULATION  
		nodeRX.timeForEmissionStrategy=t4e;		// ORIGINAL SIMULATION  
		nodeRX.debug=simDebug;
		nodeRX.receptorStats=receptorStats;
		nodeRX.receptorRandomTime=receptorRandomTime;
		nodeRX.param_RTT=paramRTT;
		nodeRX.cutEnabled=cutEnabled;
		nodeRX.stopEnabled=stopEnabled;
		nodeRX.absorbentReceiver=absorbentReceiver;
		nodeRX.rejectingReceiver=rejectingReceiver;

		nodeRX.receivedPacketType=new int[packetSize];

		if(RXisIdleOnStart){
			nodeRX.isFirstStart=false;	
			TXisIdleOnStart=false;
		}

		nodeRX.setColor(Color.lightGray);




		// ======================================
		// DISABILITA CHECK COLLISIONI SU CARRIER
		// ======================================
		if(cCollisioncontrol){
			domainForRXNode.ObjFor1Check=0;		// ENABLED!
		}else{
			domainForRXNode.ObjFor1Check=10;	// DISABLED!
		}




		//-------------------------------------------------
		// CARRIERS DISPOSED ON THE FREE SPACE, WITHOUT ANY NODE OBJ (on a CUBIC VOLUME)
		//-------------------------------------------------
		double totalCarriers=burst;
		int createdCarrier=0;
		double coeff=0.33333333;
		int size= (int) Math.ceil(Math.pow(totalCarriers, coeff)); //47   -> totalCarriers^(1/3)	
		float step=cStep; // Carrier reciprocal distance
		int dd=simDistance;

		float distance=(float) ((dd+nodeRX.getRadius()));

		float xMin=Float.MAX_VALUE;	
		float xMax=Float.MIN_VALUE;	
		
		float yMin=Float.MAX_VALUE;	
		float yMax=Float.MIN_VALUE;	
		
		float zMin=Float.MAX_VALUE;	
		float zMax=Float.MIN_VALUE;	
		
		
		
		
		
		float dist1=0;		

		for(int i=0;i<size;i=i+1){
			for(int j=0;j<size;j=j+1){
				for(int k=0;k<size;k=k+1){			

					if (createdCarrier<totalCarriers){

						float rx=xCenterRX+distance+i*step-(size*step/2);
						float ry=yCenterRX+j*step-(size*step/2);
						float rz=zCenterRX+k*step-(size*step/2);						

						TCPLike_Carrier_A cc=new TCPLike_Carrier_A(boundaryCube, rx, ry, rz, 5);			
						cc.setMotionStrategy(BrownianMotion.getInstance());				

						if(xMin>rx) xMin=rx;		// original						
						if(xMax<rx) xMax=rx;
						
						if(yMin>ry) yMin=ry;						
						if(yMax<ry) yMax=ry;
						
						if(zMin>rz) zMin=rz;						
						if(zMax<rz) zMax=rz;
						
						

						createdCarrier++;						

						dist1=dist1+(float) Math.sqrt((rx-xCenterRX)*(rx-xCenterRX)+(ry-yCenterRX)*(ry-yCenterRX)+(rz-zCenterRX)*(rz-zCenterRX));

					}
				}	
			}
		}


		float r0=nodeRX.getRadius();		

		System.out.println(">>> simulation distance: "+simDistance);
		System.out.println(">>> carrier distance (step): "+cStep+" [nm]");
		System.out.println(">>> carrier burst (punctiform positioning): "+burst);
		System.out.println(">>> TOTAL DISPOSED CARRIERS: "+createdCarrier);
		System.out.println();
		System.out.println(">>> RX Node  -  x: "+xCenterRX+", y: "+yCenterRX+", z: "+zCenterRX);
		System.out.println(">>> xMin: "+xMin+"  -  xMax: "+xMax);
		System.out.println(">>> yMin: "+yMin+"  -  yMax: "+yMax);
		System.out.println(">>> zMin: "+zMin+"  -  zMax: "+zMax);
		System.out.println(">>> Carriers-RX Mean Distance: "+(dist1/createdCarrier-r0));



		if(rejectingReceiver && absorbentReceiver){
			System.out.println();
			System.out.println();
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println(">>>  ||  ** WARNING **  You have defined both absorbentReceiver AND rejectingReceiver!  ||");
			System.out.println(">>>  ||   ++++++++++++++   REJECTING RECEIVER RULE WILL BE APPLIED!   ++++++++++++++    ||");
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println();
			System.out.println();
		}


		System.out.println(">>> SIMULATION ELEMENTS DISPOSED! ");			
		System.out.println();	
	}


	// PUNCTIFORM SHELL SIMULATION!!
	public void networkCreationTCPLike_PunctiformShell(){


		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [TCP-like Octree Centered - TX node PUNCTIFORM - RX node with SHELL]"); 
		System.out.println("-------------------------------------");			


		// ENLARGED DOMAIN FOR DEBUG
		float[] cdCenter=new float[3];
		cdCenter[0]=1000000;
		cdCenter[1]=1000000;
		cdCenter[2]=1000000;			

		float halfSide=(float) (1000000/2);  // cube's half side 


		boolean useOctree1=true;

		TCPLike_CubicStrategy ss=new TCPLike_CubicStrategy(0);
		TCPLike_DiffusionModel innerDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube=new CubicDomain(getManager().rootDomain, innerDiffusion, ss, true, false, halfSide, cdCenter, useOctree1);
		AbstractDomain domainForRXNode=boundaryCube;


		// NODES

		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer.add(TCPLike_CarrierA_Observer.getInstance());
		observer.add(TCPLike_CarrierB_Observer.getInstance());

		if(useOctree1){
			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			boundaryCube.rootTree.cleanRootLeafs();
			boundaryCube.rootTree=null;		
			boundaryCube.rootTree=new OctreeRoot(cdCenter[0], cdCenter[1], cdCenter[2], 2*halfSide, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
			boundaryCube.rootTree.setDomain(boundaryCube);
		}

		// RX NODE ABS COORDINATES!
		float xCenterRX=xDisplacement;
		float yCenterRX=yDisplacement;
		float zCenterRX=zDisplacement;

		int nodeNumberType=29;
		float radiusRX=domainForRXNode.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-radius").floatValue();



		//-------------------------------------------------
		// CARRIERS DISPOSED ON THE FREE SPACE, WITHOUT ANY NODE OBJ (on a CUBIC VOLUME)
		//-------------------------------------------------
		double totalCarriers=burst;
		int createdCarrier=0;
		double coeff=0.33333333;
		int size= (int) Math.ceil(Math.pow(totalCarriers, coeff)); //47   -> totalCarriers^(1/3)	
		float step=cStep; // Carrier reciprocal distance
		int dd=simDistance;

		//float distance=(float) ((dd+nodeRX.getRadius()));
		float distance=(float) (dd+radiusRX);


		float xMin=Float.MAX_VALUE;			
		float dist1=0;		

		for(int i=0;i<size;i=i+1){
			for(int j=0;j<size;j=j+1){
				for(int k=0;k<size;k=k+1){			

					if (createdCarrier<totalCarriers){

						float rx=xCenterRX+distance+i*step-(size*step/2);
						float ry=yCenterRX+j*step-(size*step/2);
						float rz=zCenterRX+k*step-(size*step/2);						

						TCPLike_Carrier_A cc=new TCPLike_Carrier_A(boundaryCube, rx, ry, rz, 5);			
						cc.setMotionStrategy(BrownianMotion.getInstance());						// OK
						//cc.setMotionStrategy(LinearMotion.getInstance());						// debug
						//cc.setVelocityComponent(2, 0, (float) -Math.PI/2);					// debug

						//SimpleVector qqq=new SimpleVector(distance,0,0);



						if(xMin>rx) xMin=rx;

						createdCarrier++;						

						dist1=dist1+(float) Math.sqrt((rx-xCenterRX)*(rx-xCenterRX)+(ry-yCenterRX)*(ry-yCenterRX)+(rz-zCenterRX)*(rz-zCenterRX));

					}
				}	
			}
		}



		//================================
		//    SHELL DOMAIN FOR RX NODE
		//================================		

		// TX NODE ABS COORDINATES
		float xCenterTX=xCenterRX+distance;
		float yCenterTX=yCenterRX;
		float zCenterTX=zCenterRX;


		if(RXShellRadius>0){

			// SHELL DOMAIN
			float[] cdCenter1=new float[3];
			cdCenter1[0]=xDisplacement-cdCenter[0];
			cdCenter1[1]=yDisplacement-cdCenter[1];
			cdCenter1[2]=zDisplacement-cdCenter[2];			

			SimpleVector sphereOrientation=new SimpleVector(xCenterTX-xCenterRX,yCenterTX-yCenterRX,zCenterTX-zCenterRX);			
			float calotta=(float) RXShellAngle;
			boolean useOctree=false;

			ShellSphericalStrategy ss1=new ShellSphericalStrategy(0);

			TCPLike_DiffusionModel innerShellDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);	
			ShellSphericalDomain shell=new ShellSphericalDomain(boundaryCube, innerShellDiffusion, ss1, true, false, RXShellRadius, cdCenter1, sphereOrientation, calotta, useOctree);


			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			if(useOctree){
				shell.rootTree.cleanRootLeafs();
				shell.rootTree=null;		
				shell.rootTree=new OctreeRoot(cdCenter1[0], cdCenter1[1], cdCenter1[2], 2*RXShellRadius, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
				shell.rootTree.setDomain(boundaryCube);
			}

			domainForRXNode=shell;
		}
		//================================
		//       END SHELL SPHERE
		//================================	





		// CREATE RX NODE!
		TCPLike_Node_RX nodeRX=new TCPLike_Node_RX(domainForRXNode,xCenterRX,yCenterRX,zCenterRX,nodeNumberType,observer); 		

		nodeRX.burstEmission=burst;										// ORIGINAL SIMULATION 	
		nodeRX.RTTAssimilationThreshold=RTTAssimilationThreshold;		// ORIGINAL SIMULATION
		nodeRX.alfa=alfa;												// ORIGINAL SIMULATION 	
		nodeRX.beta=beta;												// ORIGINAL SIMULATION 	
		nodeRX.scheduledWindow=scheduledWindow;							// ORIGINAL SIMULATION - Tw	
		nodeRX.estimator=estimator;										// ORIGINAL SIMULATION 	

		nodeRX.currentEmissionStrategy=c4e;		// ORIGINAL SIMULATION  
		nodeRX.timeForEmissionStrategy=t4e;		// ORIGINAL SIMULATION  
		nodeRX.debug=simDebug;
		nodeRX.receptorStats=receptorStats;
		nodeRX.receptorRandomTime=receptorRandomTime;
		nodeRX.param_RTT=paramRTT;
		nodeRX.cutEnabled=cutEnabled;
		nodeRX.stopEnabled=stopEnabled;
		nodeRX.absorbentReceiver=absorbentReceiver;
		nodeRX.rejectingReceiver=rejectingReceiver;

		nodeRX.receivedPacketType=new int[packetSize];

		if(RXisIdleOnStart){
			nodeRX.isFirstStart=false;	
			TXisIdleOnStart=false;
		}

		nodeRX.setColor(Color.lightGray);


		// ======================================
		// DISABLE CARRIER CHECK COLLISION
		// ======================================
		if(cCollisioncontrol){
			domainForRXNode.ObjFor1Check=0;		// ENABLED!
		}else{
			domainForRXNode.ObjFor1Check=10;	// DISABLED!
		}








		float r0=nodeRX.getRadius();		

		System.out.println(">>> simulation distance: "+simDistance);
		System.out.println(">>> carrier distance (step): "+cStep+" [nm]");
		System.out.println(">>> carrier burst (punctiform positioning): "+burst);
		System.out.println(">>> TOTAL DISPOSED CARRIERS: "+createdCarrier);
		System.out.println();
		System.out.println(">>> RX Node  -  x: "+xCenterRX+", y: "+yCenterRX+", z: "+zCenterRX);
		System.out.println(">>> xMin: "+xMin);
		System.out.println(">>> Carriers-RX Mean Distance: "+(dist1/createdCarrier-r0));



		if(rejectingReceiver && absorbentReceiver){
			System.out.println();
			System.out.println();
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println(">>>  ||  ** WARNING **  You have defined both absorbentReceiver AND rejectingReceiver!  ||");
			System.out.println(">>>  ||   ++++++++++++++   REJECTING RECEIVER RULE WILL BE APPLIED!   ++++++++++++++    ||");
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println();
			System.out.println();
		}


		System.out.println(">>> SIMULATION ELEMENTS DISPOSED! ");			
		System.out.println();	


	}



	// PUNCTIFORM CYLINDRIC SHELL SIMULATION!!
	public void networkCreationTCPLike_PunctiformCylindricShell(){


		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [TCP-like Octree Centered - TX node PUNCTIFORM EMITTER HANDLER - RX node with CYLINDRIC SHELL]"); 
		System.out.println("-------------------------------------");			


		// ENLARGED DOMAIN FOR DEBUG
		float[] cdCenter=new float[3];
		cdCenter[0]=1000000;
		cdCenter[1]=1000000;
		cdCenter[2]=1000000;			

		float halfSide=(float) (1000000/2);  // cube's half side 


		boolean useOctree1=true;

		TCPLike_CubicStrategy ss=new TCPLike_CubicStrategy(0);
		TCPLike_DiffusionModel innerDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube=new CubicDomain(getManager().rootDomain, innerDiffusion, ss, true, false, halfSide, cdCenter, useOctree1);
		AbstractDomain domainForRXNode=boundaryCube;


		// NODES

		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer.add(TCPLike_CarrierA_Observer.getInstance());
		observer.add(TCPLike_CarrierB_Observer.getInstance());

		if(useOctree1){
			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			boundaryCube.rootTree.cleanRootLeafs();
			boundaryCube.rootTree=null;		
			boundaryCube.rootTree=new OctreeRoot(cdCenter[0], cdCenter[1], cdCenter[2], 2*halfSide, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
			boundaryCube.rootTree.setDomain(boundaryCube);
		}

		// RX NODE ABS COORDINATES!
		float xCenterRX=xDisplacement;
		float yCenterRX=yDisplacement;
		float zCenterRX=zDisplacement;

		int nodeNumberType=29;
		float radiusRX=domainForRXNode.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-radius").floatValue();




		/*
			// ORIGINAL BEFORE PunctiformEmitter implementation!  -  commented on 03/09/2015
			//-------------------------------------------------
			// CARRIERS DISPOSED ON THE FREE SPACE, WITHOUT ANY NODE OBJ (on a CUBIC VOLUME)
			//-------------------------------------------------
			double totalCarriers=burst;
			int createdCarrier=0;
			double coeff=0.33333333;
			int size= (int) Math.ceil(Math.pow(totalCarriers, coeff)); //47   -> totalCarriers^(1/3)	
			float step=cStep; // Carrier reciprocal distance
			int dd=simDistance;

			//float distance=(float) ((dd+nodeRX.getRadius()));
			float distance=(float) (dd+radiusRX);


			float xMin=Float.MAX_VALUE;			
			float dist1=0;		

			for(int i=0;i<size;i=i+1){
				for(int j=0;j<size;j=j+1){
					for(int k=0;k<size;k=k+1){			

						if (createdCarrier<totalCarriers){

							float rx=xCenterRX+distance+i*step-(size*step/2);
							float ry=yCenterRX+j*step-(size*step/2);
							float rz=zCenterRX+k*step-(size*step/2);						

							TCPLike_Carrier_A cc=new TCPLike_Carrier_A(boundaryCube, rx, ry, rz, 5);			
							cc.setMotionStrategy(BrownianMotion.getInstance());						// OK
							//cc.setMotionStrategy(LinearMotion.getInstance());						// debug
							//cc.setVelocityComponent(2, 0, (float) -Math.PI/2);					// debug

							//SimpleVector qqq=new SimpleVector(distance,0,0);



							if(xMin>rx) xMin=rx;

							createdCarrier++;						

							dist1=dist1+(float) Math.sqrt((rx-xCenterRX)*(rx-xCenterRX)+(ry-yCenterRX)*(ry-yCenterRX)+(rz-zCenterRX)*(rz-zCenterRX));

						}
					}	
				}
			}
			//--------------------- END ORIGINAL CARRIER CREATION --------------------
		 */



		// NEW IMPLEMENTATION FOR PUNCTIFORM CARRIER CREATION WITH PATTERN!!! ----   03/09/2015
		short alignmentType=0;
		if(alignment==45) alignmentType=1;
		if(alignment==90) alignmentType=2;

		float timestepValue=(float) (getManager().configurator.m._dt*Math.pow(10,-9));			
		int symbolTimestep=(int) (symbolTime/timestepValue);						
		float distance=(float) (simDistance+radiusRX);			
		innerDiffusion.emitter = new PunctiformEmitter(boundaryCube, xCenterRX, yCenterRX, zCenterRX, radiusRX, alignmentType, burst, cStep, distance, symbolTimestep, punctiformPattern);
		innerDiffusion.emitter.exec();
		//----------------- END NEW CARRIER CREATION WITH PATTERN ----------------




		/*
			//------------------------------------------------------------------------
			// DEBUG ONLY!!!
			//------------------------------------------------------------------------
			//-------------------------------------------------
			// CARRIERS DISPOSED ON THE FREE SPACE, WITHOUT ANY NODE OBJ (on a CUBIC VOLUME)
			//-------------------------------------------------
			double totalCarriers2=burst;
			int createdCarrier2=0;
			double coeff2=0.33333333;
			int size2= (int) Math.ceil(Math.pow(totalCarriers2, coeff2)); //47   -> totalCarriers^(1/3)	
			float step2=cStep; // Carrier reciprocal distance
			int dd2=-simDistance;

			//float distance=(float) ((dd+nodeRX.getRadius()));
			float distance2=(float) (dd2+radiusRX);


			float xMin2=Float.MAX_VALUE;			
			float dist12=0;		

			for(int i=0;i<size2;i=i+1){
				for(int j=0;j<size2;j=j+1){
					for(int k=0;k<size2;k=k+1){			

						if (createdCarrier2<totalCarriers2){

							float rx=xCenterRX+distance2+i*step2-(size2*step2/2);
							float ry=yCenterRX+j*step2-(size2*step2/2);
							float rz=zCenterRX+k*step2-(size2*step2/2);						

							TCPLike_Carrier_A cc=new TCPLike_Carrier_A(boundaryCube, rx, ry, rz, 5);			
							//cc.setMotionStrategy(BrownianMotion.getInstance());						// OK
							cc.setMotionStrategy(LinearMotion.getInstance());						// debug
							cc.setVelocityComponent(-2, 0, (float) -Math.PI/2);					// debug
							cc.setColor(Color.white);
							//SimpleVector qqq=new SimpleVector(distance,0,0);



							if(xMin2>rx) xMin2=rx;

							createdCarrier2++;						

							dist12=dist12+(float) Math.sqrt((rx-xCenterRX)*(rx-xCenterRX)+(ry-yCenterRX)*(ry-yCenterRX)+(rz-zCenterRX)*(rz-zCenterRX));

						}
					}	
				}
			}

			// END DEBUG! ------------------------------------------------------------------------------
		 */



		//=======================================
		//    CYLINDRIC SHELL DOMAIN FOR RX NODE
		//=======================================		

		// TX NODE ABS COORDINATES
		float xCenterTX=xCenterRX+distance;
		float yCenterTX=yCenterRX;
		float zCenterTX=zCenterRX;


		if(RXShellRadius>0 && RXShellHeight>0){

			// SHELL DOMAIN
			float[] cdCenter1=new float[3];
			cdCenter1[0]=xDisplacement-cdCenter[0];
			cdCenter1[1]=yDisplacement-cdCenter[1];
			cdCenter1[2]=zDisplacement-cdCenter[2];			

			SimpleVector cylinderOrientation=new SimpleVector(xCenterTX-xCenterRX,yCenterTX-yCenterRX,zCenterTX-zCenterRX);	

			//------------------------------------
			//cylinderOrientation.scalarMul(-1);	// Opposite direction with respect to the emitting point!
			//------------------------------------

			SimpleVector cylinderOrtogonalOrientation=new SimpleVector(0,0,1);	

			boolean useOctree=false;


			ShellCylindricStrategy ss1=new ShellCylindricStrategy(0, isBottomless);				

			TCPLike_DiffusionModel innerShellDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);	

			ShellCylindricDomain shell=new ShellCylindricDomain(boundaryCube, innerShellDiffusion, ss1, true, false, RXShellHeight, RXShellRadius, cdCenter1, cylinderOrientation, cylinderOrtogonalOrientation, useOctree);


			// EXPORT CYLINDRIC DOMAIN!!!
			getManager().addDomainForExport(shell);


			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			if(useOctree){
				shell.rootTree.cleanRootLeafs();
				shell.rootTree=null;		
				shell.rootTree=new OctreeRoot(cdCenter1[0], cdCenter1[1], cdCenter1[2], 2*RXShellRadius, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
				shell.rootTree.setDomain(boundaryCube);
			}

			domainForRXNode=shell;
		}
		//================================
		//       END SHELL CYLINDRIC
		//================================	





		// CREATE RX NODE!
		TCPLike_Node_RX nodeRX=new TCPLike_Node_RX(domainForRXNode,xCenterRX,yCenterRX,zCenterRX,nodeNumberType,observer); 		

		nodeRX.burstEmission=burst;										// ORIGINAL SIMULATION 	
		nodeRX.RTTAssimilationThreshold=RTTAssimilationThreshold;		// ORIGINAL SIMULATION
		nodeRX.alfa=alfa;												// ORIGINAL SIMULATION 	
		nodeRX.beta=beta;												// ORIGINAL SIMULATION 	
		nodeRX.scheduledWindow=scheduledWindow;							// ORIGINAL SIMULATION - Tw	
		nodeRX.estimator=estimator;										// ORIGINAL SIMULATION 	

		nodeRX.currentEmissionStrategy=c4e;		// ORIGINAL SIMULATION  
		nodeRX.timeForEmissionStrategy=t4e;		// ORIGINAL SIMULATION  
		nodeRX.debug=simDebug;
		nodeRX.receptorStats=receptorStats;
		nodeRX.receptorRandomTime=receptorRandomTime;
		nodeRX.param_RTT=paramRTT;
		nodeRX.cutEnabled=cutEnabled;
		nodeRX.stopEnabled=stopEnabled;
		nodeRX.absorbentReceiver=absorbentReceiver;
		nodeRX.rejectingReceiver=rejectingReceiver;

		nodeRX.receivedPacketType=new int[packetSize];

		if(RXisIdleOnStart){
			nodeRX.isFirstStart=false;	
			TXisIdleOnStart=false;
		}

		nodeRX.setColor(Color.lightGray);


		// ======================================
		// DISABLE CARRIER CHECK COLLISION
		// ======================================
		if(cCollisioncontrol){
			domainForRXNode.ObjFor1Check=0;		// ENABLED!
		}else{
			domainForRXNode.ObjFor1Check=10;	// DISABLED!
		}



		float r0=nodeRX.getRadius();		

		System.out.println(">>> simulation distance: "+simDistance);
		System.out.println(">>> carrier distance (step): "+cStep+" [nm]");
		System.out.println(">>> carrier burst (NEW punctiform positioning): "+burst);
		//System.out.println(">>> TOTAL DISPOSED CARRIERS: "+createdCarrier);
		//System.out.println();
		System.out.println(">>> RX Node  -  x: "+xCenterRX+", y: "+yCenterRX+", z: "+zCenterRX);
		//System.out.println(">>> xMin: "+xMin);
		//System.out.println(">>> Carriers-RX Mean Distance: "+(dist1/createdCarrier-r0));
		//System.out.println();
		System.out.println(">>> EMISSION PATTERN: "+punctiformPattern);
		System.out.println(">>> Symbol Time: "+symbolTime+" [s]");
		System.out.println();
		/**/


		if(rejectingReceiver && absorbentReceiver){
			System.out.println();
			System.out.println();
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println(">>>  ||  ** WARNING **  You have defined both absorbentReceiver AND rejectingReceiver!  ||");
			System.out.println(">>>  ||   ++++++++++++++   REJECTING RECEIVER RULE WILL BE APPLIED!   ++++++++++++++    ||");
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println();
			System.out.println();
		}


		System.out.println(">>> SIMULATION ELEMENTS DISPOSED! ");			
		System.out.println();	


	}



	// PUNCTIFORM CYLINDRIC SHELL SIMULATION!!
	public void networkCreationTCPLike_PunctiformCylindricShell_mobileNodeRX(){


		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [TCP-like Octree Centered - TX node PUNCTIFORM EMITTER HANDLER - RX node with CYLINDRIC SHELL]"); 
		System.out.println("-------------------------------------");			


		// ENLARGED DOMAIN FOR DEBUG
		float[] cdCenter=new float[3];
		cdCenter[0]=1000000;
		cdCenter[1]=1000000;
		cdCenter[2]=1000000;			

		float halfSide=(float) (1000000/2);  // cube's half side 


		boolean useOctree1=true;

		TCPLike_CubicStrategy ss=new TCPLike_CubicStrategy(0);
		TCPLike_DiffusionModel innerDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube=new CubicDomain(getManager().rootDomain, innerDiffusion, ss, true, false, halfSide, cdCenter, useOctree1);
		AbstractDomain domainForRXNode=boundaryCube;


		// NODES

		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer.add(TCPLike_CarrierA_Observer.getInstance());
		observer.add(TCPLike_CarrierB_Observer.getInstance());

		if(useOctree1){
			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			boundaryCube.rootTree.cleanRootLeafs();
			boundaryCube.rootTree=null;		
			boundaryCube.rootTree=new OctreeRoot(cdCenter[0], cdCenter[1], cdCenter[2], 2*halfSide, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
			boundaryCube.rootTree.setDomain(boundaryCube);
		}

		// RX NODE ABS COORDINATES!
		float xCenterRX=xDisplacement;
		float yCenterRX=yDisplacement;
		float zCenterRX=zDisplacement;

		int nodeNumberType=29;
		float radiusRX=domainForRXNode.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-radius").floatValue();




		// NEW IMPLEMENTATION FOR PUNCTIFORM CARRIER CREATION WITH PATTERN!!! ----   03/09/2015
		short alignmentType=0;
		if(alignment==45) alignmentType=1;
		if(alignment==90) alignmentType=2;

		float timestepValue=(float) (getManager().configurator.m._dt*Math.pow(10,-9));			
		int symbolTimestep=(int) (symbolTime/timestepValue);						
		float distance=(float) (simDistance+radiusRX);			
		innerDiffusion.emitter = new PunctiformEmitter(boundaryCube, xCenterRX, yCenterRX, zCenterRX, radiusRX, alignmentType, burst, cStep, distance, symbolTimestep, punctiformPattern);
		innerDiffusion.emitter.exec();
		//----------------- END NEW CARRIER CREATION WITH PATTERN ----------------




		//=======================================
		//    CYLINDRIC SHELL DOMAIN FOR RX NODE
		//=======================================		

		// TX NODE ABS COORDINATES
		float xCenterTX=xCenterRX+distance;
		float yCenterTX=yCenterRX;
		float zCenterTX=zCenterRX;


		if(RXShellRadius>0 && RXShellHeight>0){

			// SHELL DOMAIN		---		05/10/2015  mobile coordinates!
			float[] cdCenter1=new float[3];
			cdCenter1[0]=xDisplacement-cdCenter[0];
			cdCenter1[1]=yDisplacement-cdCenter[1];
			cdCenter1[2]=zDisplacement-cdCenter[2];			

			SimpleVector cylinderOrientation=new SimpleVector(xCenterTX-xCenterRX,yCenterTX-yCenterRX,zCenterTX-zCenterRX);	


			//  ----  Move the cylinder center respect to node RX  ----

			//float d=1000;	// DEBUG ONLY


			if (Float.isNaN(d)){
				d=RXShellHeight/2-radiusRX;  // In the case that the d parameter was not set! Then dispose node RX on the center of cylinder domain
			}

			if(d>RXShellHeight-2*radiusRX || d<0){



				System.out.println();
				System.out.println();
				System.out.println("     +-----------------------------------------------------------------------------------+");
				System.out.println(">>>  ||  ** WARNING **  You have defined a WRONG CYLINDER DISPLACEMENT !                ||");
				System.out.println(">>>  ||   +++++++++++   It's impossible to dispose the NODE RX INSIDE!!   ++++++++++    ||");
				System.out.println("     +-----------------------------------------------------------------------------------+");
				System.out.println();
				System.out.println();

				System.exit(0);
			}

			float D=RXShellHeight/2-radiusRX-d;

			cdCenter1[0]=cdCenter1[0]+D;


			float[] absCylCoords=new float[3];
			absCylCoords[0]=cdCenter1[0]+cdCenter[0];
			absCylCoords[1]=cdCenter1[1]+cdCenter[1];
			absCylCoords[2]=cdCenter1[2]+cdCenter[2];


			System.out.println(">>>  =======================================================================================");
			System.out.println(">>>  CYLINDER ABS COORDS: "+absCylCoords[0]+", "+absCylCoords[1]+", "+absCylCoords[2]+"   -   CYLINDER FULL LENGTH:  "+RXShellHeight);
			System.out.println(">>>  NODE RX  ABS COORDS: "+xCenterRX+", "+yCenterRX+", "+zCenterRX+"    -    radius RX: "+radiusRX);
			System.out.println();
			System.out.println(">>>  CYLINDER-NODE_RX DISPLACEMENT (centers) D:  "+D);
			System.out.println(">>>  NODE_RX BOTTOM DISTANCE d:  "+d);		
			System.out.println(">>>  CYLINDER:  [SIDE A: "+(absCylCoords[0]-RXShellHeight/2)+"],  [SIDE B: "+(absCylCoords[0]+RXShellHeight/2)+"]");
			System.out.println(">>>  =======================================================================================");
			System.out.println();
			System.out.println();
			//  ----  END MOVE  ----



			//------------------------------------
			//cylinderOrientation.scalarMul(-1);	// Opposite direction with respect to the emitting point!
			//------------------------------------

			SimpleVector cylinderOrtogonalOrientation=new SimpleVector(0,0,1);	

			boolean useOctree=false;


			ShellCylindricStrategy ss1=new ShellCylindricStrategy(0, isBottomless);				

			TCPLike_DiffusionModel innerShellDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);	

			ShellCylindricDomain shell=new ShellCylindricDomain(boundaryCube, innerShellDiffusion, ss1, true, false, RXShellHeight, RXShellRadius, cdCenter1, cylinderOrientation, cylinderOrtogonalOrientation, useOctree);


			// EXPORT CYLINDRIC DOMAIN!!!
			getManager().addDomainForExport(shell);


			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			if(useOctree){
				shell.rootTree.cleanRootLeafs();
				shell.rootTree=null;		
				shell.rootTree=new OctreeRoot(cdCenter1[0], cdCenter1[1], cdCenter1[2], 2*RXShellRadius, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
				shell.rootTree.setDomain(boundaryCube);
			}

			domainForRXNode=shell;
		}
		//================================
		//       END SHELL CYLINDRIC
		//================================	





		// CREATE RX NODE!
		TCPLike_Node_RX nodeRX=new TCPLike_Node_RX(domainForRXNode,xCenterRX,yCenterRX,zCenterRX,nodeNumberType,observer); 		

		nodeRX.burstEmission=burst;										// ORIGINAL SIMULATION 	
		nodeRX.RTTAssimilationThreshold=RTTAssimilationThreshold;		// ORIGINAL SIMULATION
		nodeRX.alfa=alfa;												// ORIGINAL SIMULATION 	
		nodeRX.beta=beta;												// ORIGINAL SIMULATION 	
		nodeRX.scheduledWindow=scheduledWindow;							// ORIGINAL SIMULATION - Tw	
		nodeRX.estimator=estimator;										// ORIGINAL SIMULATION 	

		nodeRX.currentEmissionStrategy=c4e;		// ORIGINAL SIMULATION  
		nodeRX.timeForEmissionStrategy=t4e;		// ORIGINAL SIMULATION  
		nodeRX.debug=simDebug;
		nodeRX.receptorStats=receptorStats;
		nodeRX.receptorRandomTime=receptorRandomTime;
		nodeRX.param_RTT=paramRTT;
		nodeRX.cutEnabled=cutEnabled;
		nodeRX.stopEnabled=stopEnabled;
		nodeRX.absorbentReceiver=absorbentReceiver;
		nodeRX.rejectingReceiver=rejectingReceiver;

		nodeRX.receivedPacketType=new int[packetSize];

		if(RXisIdleOnStart){
			nodeRX.isFirstStart=false;	
			TXisIdleOnStart=false;
		}

		nodeRX.setColor(Color.lightGray);


		// ======================================
		// DISABLE CARRIER CHECK COLLISION
		// ======================================
		if(cCollisioncontrol){
			domainForRXNode.ObjFor1Check=0;		// ENABLED!
		}else{
			domainForRXNode.ObjFor1Check=10;	// DISABLED!
		}








		float r0=nodeRX.getRadius();		

		System.out.println(">>> simulation distance: "+simDistance);
		System.out.println(">>> carrier distance (step): "+cStep+" [nm]");
		System.out.println(">>> carrier burst (NEW punctiform positioning): "+burst);
		//System.out.println(">>> TOTAL DISPOSED CARRIERS: "+createdCarrier);
		//System.out.println();
		System.out.println(">>> RX Node  -  x: "+xCenterRX+", y: "+yCenterRX+", z: "+zCenterRX);
		//System.out.println(">>> xMin: "+xMin);
		//System.out.println(">>> Carriers-RX Mean Distance: "+(dist1/createdCarrier-r0));
		//System.out.println();
		System.out.println(">>> EMISSION PATTERN: "+punctiformPattern);
		System.out.println(">>> Symbol Time: "+symbolTime+" [s]");
		System.out.println();
		/**/


		if(rejectingReceiver && absorbentReceiver){
			System.out.println();
			System.out.println();
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println(">>>  ||  ** WARNING **  You have defined both absorbentReceiver AND rejectingReceiver!  ||");
			System.out.println(">>>  ||   ++++++++++++++   REJECTING RECEIVER RULE WILL BE APPLIED!   ++++++++++++++    ||");
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println();
			System.out.println();
		}


		System.out.println(">>> SIMULATION ELEMENTS DISPOSED! ");			
		System.out.println();	


	}


	public void drawCylinder(){
		System.out.println();
		System.out.println();
		System.out.println();
		System.out.println("                 +-----------------------------------+");	
		System.out.println("                 |                                   |");
		System.out.println("                 |                          _        |"); 
		System.out.println("    *            |                  .      (_)       |"); 
		System.out.println("    TX           |                 Cd       RX       |"); 
		System.out.println("                 |                                   |"); 
		System.out.println("                 +-----------------------------------+");	
		System.out.println();
		System.out.println();

	}


	// PUNCTIFORM CYLINDRIC SHELL SIMULATION!!
	public void networkCreationTCPLike_PunctiformCylindricShell_mobileNodeRX_v2(){


		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [TCP-like Octree Centered - TX node PUNCTIFORM EMITTER HANDLER - RX node with CYLINDRIC SHELL]"); 
		System.out.println("                       [AUTOMATIC CYLINDER LENGTH FOR CUSTOM NODE RX POSITION                                       ]");	
		System.out.println("-------------------------------------");	


		// ENLARGED DOMAIN FOR DEBUG
		float[] cdCenter=new float[3];
		cdCenter[0]=1000000;
		cdCenter[1]=1000000;
		cdCenter[2]=1000000;			

		float halfSide=(float) (1000000/2);  // cube's half side 


		// FIXME Ho disabilitato temporaneamente l'Octree, ma  da RIABILITARE!!!
		boolean useOctree1=true;

		TCPLike_CubicStrategy ss=new TCPLike_CubicStrategy(0);
		TCPLike_DiffusionModel innerDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube=new CubicDomain(getManager().rootDomain, innerDiffusion, ss, true, false, halfSide, cdCenter, useOctree1);
		AbstractDomain domainForRXNode=boundaryCube;


		// NODES

		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer.add(TCPLike_CarrierA_Observer.getInstance());
		observer.add(TCPLike_CarrierB_Observer.getInstance());

		if(useOctree1){
			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			boundaryCube.rootTree.cleanRootLeafs();
			boundaryCube.rootTree=null;		
			boundaryCube.rootTree=new OctreeRoot(cdCenter[0], cdCenter[1], cdCenter[2], 2*halfSide, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
			boundaryCube.rootTree.setDomain(boundaryCube);
		}

		// RX NODE ABS COORDINATES!
		float xCenterRX=xDisplacement;
		float yCenterRX=yDisplacement;
		float zCenterRX=zDisplacement;

		int nodeNumberType=29;
		float radiusRX=domainForRXNode.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-radius").floatValue();




		// NEW IMPLEMENTATION FOR PUNCTIFORM CARRIER CREATION WITH PATTERN!!! ----   03/09/2015
		short alignmentType=0;
		if(alignment==45) alignmentType=1;
		if(alignment==90) alignmentType=2;

		float timestepValue=(float) (getManager().configurator.m._dt*Math.pow(10,-9));			
		int symbolTimestep=(int) (symbolTime/timestepValue);						
		float distance=(float) (simDistance+radiusRX);			
		innerDiffusion.emitter = new PunctiformEmitter(boundaryCube, xCenterRX, yCenterRX, zCenterRX, radiusRX, alignmentType, burst, cStep, distance, symbolTimestep, punctiformPattern);
		innerDiffusion.emitter.exec();
		//----------------- END NEW CARRIER CREATION WITH PATTERN ----------------



		/**/
		//=======================================
		//    CYLINDRIC SHELL DOMAIN FOR RX NODE
		//=======================================		

		// TX NODE ABS COORDINATES
		float xCenterTX=xCenterRX+distance;
		float yCenterTX=yCenterRX;
		float zCenterTX=zCenterRX;


		//if(RXShellRadius>0 && RXShellHeight>0){

		// SHELL DOMAIN		---		05/10/2015  mobile coordinates!
		float[] cdCenter1=new float[3];
		cdCenter1[0]=xDisplacement-cdCenter[0];
		cdCenter1[1]=yDisplacement-cdCenter[1];
		cdCenter1[2]=zDisplacement-cdCenter[2];			

		SimpleVector cylinderOrientation=new SimpleVector(xCenterTX-xCenterRX,yCenterTX-yCenterRX,zCenterTX-zCenterRX);	



		//  ----  Move the cylinder center respect to node RX  ----				

		boolean exitNow=false;

		float D=0;
		boolean isBottomDisplaced=false;
		float sd=simDistance+radiusRX;

		if (!Float.isNaN(cylDistance) && !Float.isNaN(d)){
			RXShellHeight=(sd-cylDistance+d+radiusRX);
			D=sd-cylDistance-RXShellHeight/2;
			cdCenter1[0]=cdCenter1[0]+D;
			isBottomDisplaced=true;

			drawCylinder();

		}else if(Float.isNaN(cylDistance) && Float.isNaN(d) && (RXShellRadius>0 && RXShellHeight>0)){
			System.out.println(">>>  DEFAULT SIMULATION:  Cylinder center and Node RX center are overlapped!");
		}else{
			exitNow=true;
		}

		if (RXShellRadius<=0){
			exitNow=true;
		}

		if (Float.isNaN(d)){
			d=RXShellHeight/2-radiusRX;  // In the case that the d parameter was not set! Then dispose node RX on the center of cylinder domain
		}

		if(d>RXShellHeight-2*radiusRX || d<0  || exitNow){

			System.out.println();
			System.out.println();
			System.out.println(">>>  +------------------------------------------------------------------------+");
			System.out.println(">>>  ||  ** WARNING **  You have defined a WRONG CYLINDER CONFIGURATION!     ||");
			System.out.println(">>>  ||   +++++++++++   CHECK your configuration parameters    ++++++++++    ||");					
			System.out.println(">>>  +------------------------------------------------------------------------+");
			System.out.println(">>>  Cylinder Radius: "+RXShellRadius+", Height: "+RXShellHeight+", cylDistance: "+cylDistance+", RX Bottom d: "+d+"");
			System.out.println();
			System.out.println();
			System.out.println(">>>  ABORT SIMULATION...");

			System.exit(0);
		}


		float[] absCylCoords=new float[3];
		absCylCoords[0]=cdCenter1[0]+cdCenter[0];
		absCylCoords[1]=cdCenter1[1]+cdCenter[1];
		absCylCoords[2]=cdCenter1[2]+cdCenter[2];



		float d_a=(xCenterRX-radiusRX)-(absCylCoords[0]-RXShellHeight/2);
		float d_b=(absCylCoords[0]+RXShellHeight/2)-(xCenterRX+radiusRX);

		System.out.println(">>>  =======================================================================================");
		System.out.println(">>>  CYL DISTANCE (configuration): "+cylDistance);
		System.out.println(">>>  CYL DISTANCE (calculated):    "+(sd-RXShellHeight/2-D));
		System.out.println(">>>  CYLINDER ABS COORDS: "+absCylCoords[0]+", "+absCylCoords[1]+", "+absCylCoords[2]+"   -   CYLINDER FULL LENGTH:  "+RXShellHeight);
		System.out.println(">>>  NODE RX  ABS COORDS: "+xCenterRX+", "+yCenterRX+", "+zCenterRX+"   -    radius RX: "+radiusRX);
		System.out.println();
		System.out.println(">>>  CYLINDER-NODE_RX DISPLACEMENT (centers) D:  "+D);
		System.out.println(">>>  NODE_RX BOTTOM DISTANCE d (configuration):  "+d);	
		System.out.println(">>>  NODE_RX BOTTOM DISTANCE d (calculated):  [SIDE A: "+d_a+"],  [SIDE B: "+d_b+"]");		
		System.out.println(">>>  CYLINDER:  [SIDE A: "+(absCylCoords[0]-RXShellHeight/2)+"],  [SIDE B: "+(absCylCoords[0]+RXShellHeight/2)+"]");
		System.out.println(">>>  =======================================================================================");


		if(isBottomDisplaced){
			OutputDBMySQL.getInstance().update("[RUNTIME INFO: (cylDistance: "+(sd-RXShellHeight/2-D)+") (RX NODE BOTTOM distance: "+d_a+")]");
		}else{
			System.out.println(">>>  Cylinder and RX NODE centers ALIGNED (configured bottom distance ignored)!");
			OutputDBMySQL.getInstance().update("[RUNTIME INFO: Cylinder and RX NODE centers ALIGNED (configured bottom distance ignored)!]");
		}


		System.out.println();
		System.out.println();				


		//  ----  END MOVE  ----



		//------------------------------------
		//cylinderOrientation.scalarMul(-1);	// Opposite direction with respect to the emitting point!
		//------------------------------------

		SimpleVector cylinderOrtogonalOrientation=new SimpleVector(0,0,1);	

		boolean useOctree=false;

		//========================================================================================================
		//========================================================================================================
		//========================================================================================================
		//========================================================================================================

		ShellCylindricStrategy ss1=new ShellCylindricStrategy(0, isBottomless);				

		TCPLike_DiffusionModel innerShellDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);	

		ShellCylindricDomain shell=new ShellCylindricDomain(boundaryCube, innerShellDiffusion, ss1, true, true, RXShellHeight, RXShellRadius, cdCenter1, cylinderOrientation, cylinderOrtogonalOrientation, useOctree);


		// EXPORT CYLINDRIC DOMAIN!!!
		getManager().addDomainForExport(shell);


		//  WARNING!! You have to override the rootTree creation rules! 
		//  Because in the AbstractDomain it is defined a DISPLACEMENT! 
		//  (used for the blood flow case...) 

		if(useOctree){
			shell.rootTree.cleanRootLeafs();
			shell.rootTree=null;		
			shell.rootTree=new OctreeRoot(cdCenter1[0], cdCenter1[1], cdCenter1[2], 2*RXShellRadius, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
			shell.rootTree.setDomain(boundaryCube);
		}

		domainForRXNode=shell;
		//}
		//================================
		//       END SHELL CYLINDRIC
		//================================	



		// DISABILITATO TEMPORANEAMENTE!
		/**/
		// CREATE RX NODE!
		TCPLike_Node_RX nodeRX=new TCPLike_Node_RX(domainForRXNode,xCenterRX,yCenterRX,zCenterRX,nodeNumberType,observer); 		

		nodeRX.burstEmission=burst;										// ORIGINAL SIMULATION 	
		nodeRX.RTTAssimilationThreshold=RTTAssimilationThreshold;		// ORIGINAL SIMULATION
		nodeRX.alfa=alfa;												// ORIGINAL SIMULATION 	
		nodeRX.beta=beta;												// ORIGINAL SIMULATION 	
		nodeRX.scheduledWindow=scheduledWindow;							// ORIGINAL SIMULATION - Tw	
		nodeRX.estimator=estimator;										// ORIGINAL SIMULATION 	

		nodeRX.currentEmissionStrategy=c4e;		// ORIGINAL SIMULATION  
		nodeRX.timeForEmissionStrategy=t4e;		// ORIGINAL SIMULATION  
		nodeRX.debug=simDebug;
		nodeRX.receptorStats=receptorStats;
		nodeRX.receptorRandomTime=receptorRandomTime;
		nodeRX.param_RTT=paramRTT;
		nodeRX.cutEnabled=cutEnabled;
		nodeRX.stopEnabled=stopEnabled;
		nodeRX.absorbentReceiver=absorbentReceiver;
		nodeRX.rejectingReceiver=rejectingReceiver;

		nodeRX.receivedPacketType=new int[packetSize];

		if(RXisIdleOnStart){
			nodeRX.isFirstStart=false;	
			TXisIdleOnStart=false;
		}

		nodeRX.setColor(Color.lightGray);


		// IF you have to remove the RX NODE, decomment the following line!
		//nodeRX.kill();


		// ======================================
		// DISABLE CARRIER CHECK COLLISION
		// ======================================
		if(cCollisioncontrol){
			domainForRXNode.ObjFor1Check=0;		// ENABLED!
		}else{
			domainForRXNode.ObjFor1Check=10;	// DISABLED!
		}


		//float r0=nodeRX.getRadius();


		//========================================================================================================
		//========================================================================================================
		//========================================================================================================
		//========================================================================================================


		System.out.println(">>> simulation distance: "+simDistance);
		System.out.println(">>> carrier distance (step): "+cStep+" [nm]");
		System.out.println(">>> carrier burst (NEW punctiform positioning): "+burst);
		//System.out.println(">>> TOTAL DISPOSED CARRIERS: "+createdCarrier);
		//System.out.println();
		System.out.println(">>> RX Node  -  x: "+xCenterRX+", y: "+yCenterRX+", z: "+zCenterRX);
		//System.out.println(">>> xMin: "+xMin);
		//System.out.println(">>> Carriers-RX Mean Distance: "+(dist1/createdCarrier-r0));
		//System.out.println();
		System.out.println(">>> EMISSION PATTERN: "+punctiformPattern);
		System.out.println(">>> Symbol Time: "+symbolTime+" [s]");
		System.out.println();
		/**/


		if(rejectingReceiver && absorbentReceiver){
			System.out.println();
			System.out.println();
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println(">>>  ||  ** WARNING **  You have defined both absorbentReceiver AND rejectingReceiver!  ||");
			System.out.println(">>>  ||   ++++++++++++++   REJECTING RECEIVER RULE WILL BE APPLIED!   ++++++++++++++    ||");
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println();
			System.out.println();
		}


		System.out.println(">>> SIMULATION ELEMENTS DISPOSED! ");			
		System.out.println();	


	}



	//===============================================
	// PUNCTIFORM NO SHELL SIMULATION!! For TEST LAB
	//===============================================
	public void networkCreationTCPLike_PunctiformEmitter_NoSHELL_mobileNodeRX_v2(){


		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [TCP-like Octree Centered - TX node PUNCTIFORM EMITTER HANDLER - RX node with CYLINDRIC SHELL]"); 
		System.out.println("                       [AUTOMATIC CYLINDER LENGTH FOR CUSTOM NODE RX POSITION                                       ]");	
		System.out.println("-------------------------------------");	


		// ENLARGED DOMAIN FOR DEBUG
		float[] cdCenter=new float[3];
		cdCenter[0]=1000000;
		cdCenter[1]=1000000;
		cdCenter[2]=1000000;			

		float halfSide=(float) (1000000/2);  // cube's half side 


		// FIXME Ho disabilitato temporaneamente l'Octree, ma  da RIABILITARE!!!
		boolean useOctree1=true;

		TCPLike_CubicStrategy ss=new TCPLike_CubicStrategy(0);
		TCPLike_DiffusionModel innerDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube=new CubicDomain(getManager().rootDomain, innerDiffusion, ss, true, false, halfSide, cdCenter, useOctree1);
		AbstractDomain domainForRXNode=boundaryCube;


		// NODES

		List<LigandObserver> observer=new ArrayList<LigandObserver>();
		observer.add(TCPLike_CarrierA_Observer.getInstance());
		observer.add(TCPLike_CarrierB_Observer.getInstance());

		if(useOctree1){
			/** WARNING!! You have to override the rootTree creation rules! 
			 *  Because in the AbstractDomain it is defined a DISPLACEMENT! 
			 *  (used for the blood flow case...) 
			 *  */
			boundaryCube.rootTree.cleanRootLeafs();
			boundaryCube.rootTree=null;		
			boundaryCube.rootTree=new OctreeRoot(cdCenter[0], cdCenter[1], cdCenter[2], 2*halfSide, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
			boundaryCube.rootTree.setDomain(boundaryCube);
		}

		// RX NODE ABS COORDINATES!
		float xCenterRX=xDisplacement;
		float yCenterRX=yDisplacement;
		float zCenterRX=zDisplacement;

		int nodeNumberType=29;
		float radiusRX=domainForRXNode.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-radius").floatValue();




		// NEW IMPLEMENTATION FOR PUNCTIFORM CARRIER CREATION WITH PATTERN!!! ----   03/09/2015
		short alignmentType=0;
		if(alignment==45) alignmentType=1;
		if(alignment==90) alignmentType=2;

		float timestepValue=(float) (getManager().configurator.m._dt*Math.pow(10,-9));			
		int symbolTimestep=(int) (symbolTime/timestepValue);						
		float distance=(float) (simDistance+radiusRX);			
		innerDiffusion.emitter = new PunctiformEmitter(boundaryCube, xCenterRX, yCenterRX, zCenterRX, radiusRX, alignmentType, burst, cStep, distance, symbolTimestep, punctiformPattern);
		innerDiffusion.emitter.exec();
		//----------------- END NEW CARRIER CREATION WITH PATTERN ----------------



		/**/
		//=======================================
		//    CYLINDRIC SHELL DOMAIN FOR RX NODE
		//=======================================		

		// TX NODE ABS COORDINATES
		float xCenterTX=xCenterRX+distance;
		float yCenterTX=yCenterRX;
		float zCenterTX=zCenterRX;


		//if(RXShellRadius>0 && RXShellHeight>0){

		// SHELL DOMAIN		---		05/10/2015  mobile coordinates!
		float[] cdCenter1=new float[3];
		cdCenter1[0]=xDisplacement-cdCenter[0];
		cdCenter1[1]=yDisplacement-cdCenter[1];
		cdCenter1[2]=zDisplacement-cdCenter[2];			

		SimpleVector cylinderOrientation=new SimpleVector(xCenterTX-xCenterRX,yCenterTX-yCenterRX,zCenterTX-zCenterRX);	



		//  ----  Move the cylinder center respect to node RX  ----				

		boolean exitNow=false;

		float D=0;
		boolean isBottomDisplaced=false;
		float sd=simDistance+radiusRX;

		if (!Float.isNaN(cylDistance) && !Float.isNaN(d)){
			RXShellHeight=(sd-cylDistance+d+radiusRX);
			D=sd-cylDistance-RXShellHeight/2;
			cdCenter1[0]=cdCenter1[0]+D;
			isBottomDisplaced=true;

			drawCylinder();

		}else if(Float.isNaN(cylDistance) && Float.isNaN(d) && (RXShellRadius>0 && RXShellHeight>0)){
			System.out.println(">>>  DEFAULT SIMULATION:  Cylinder center and Node RX center are overlapped!");
		}else{
			exitNow=true;
		}

		if (RXShellRadius<=0){
			exitNow=true;
		}

		if (Float.isNaN(d)){
			d=RXShellHeight/2-radiusRX;  // In the case that the d parameter was not set! Then dispose node RX on the center of cylinder domain
		}

		if(d>RXShellHeight-2*radiusRX || d<0  || exitNow){

			System.out.println();
			System.out.println();
			System.out.println(">>>  +------------------------------------------------------------------------+");
			System.out.println(">>>  ||  ** WARNING **  You have defined a WRONG CYLINDER CONFIGURATION!     ||");
			System.out.println(">>>  ||   +++++++++++   CHECK your configuration parameters    ++++++++++    ||");					
			System.out.println(">>>  +------------------------------------------------------------------------+");
			System.out.println(">>>  Cylinder Radius: "+RXShellRadius+", Height: "+RXShellHeight+", cylDistance: "+cylDistance+", RX Bottom d: "+d+"");
			System.out.println();
			System.out.println();
			System.out.println(">>>  ABORT SIMULATION...");

			System.exit(0);
		}


		float[] absCylCoords=new float[3];
		absCylCoords[0]=cdCenter1[0]+cdCenter[0];
		absCylCoords[1]=cdCenter1[1]+cdCenter[1];
		absCylCoords[2]=cdCenter1[2]+cdCenter[2];



		float d_a=(xCenterRX-radiusRX)-(absCylCoords[0]-RXShellHeight/2);
		float d_b=(absCylCoords[0]+RXShellHeight/2)-(xCenterRX+radiusRX);

		System.out.println(">>>  =======================================================================================");
		System.out.println(">>>  CYL DISTANCE (configuration): "+cylDistance);
		System.out.println(">>>  CYL DISTANCE (calculated):    "+(sd-RXShellHeight/2-D));
		System.out.println(">>>  CYLINDER ABS COORDS: "+absCylCoords[0]+", "+absCylCoords[1]+", "+absCylCoords[2]+"   -   CYLINDER FULL LENGTH:  "+RXShellHeight);
		System.out.println(">>>  NODE RX  ABS COORDS: "+xCenterRX+", "+yCenterRX+", "+zCenterRX+"   -    radius RX: "+radiusRX);
		System.out.println();
		System.out.println(">>>  CYLINDER-NODE_RX DISPLACEMENT (centers) D:  "+D);
		System.out.println(">>>  NODE_RX BOTTOM DISTANCE d (configuration):  "+d);	
		System.out.println(">>>  NODE_RX BOTTOM DISTANCE d (calculated):  [SIDE A: "+d_a+"],  [SIDE B: "+d_b+"]");		
		System.out.println(">>>  CYLINDER:  [SIDE A: "+(absCylCoords[0]-RXShellHeight/2)+"],  [SIDE B: "+(absCylCoords[0]+RXShellHeight/2)+"]");
		System.out.println(">>>  =======================================================================================");


		if(isBottomDisplaced){
			OutputDBMySQL.getInstance().update("[RUNTIME INFO: (cylDistance: "+(sd-RXShellHeight/2-D)+") (RX NODE BOTTOM distance: "+d_a+")]");
		}else{
			System.out.println(">>>  Cylinder and RX NODE centers ALIGNED (configured bottom distance ignored)!");
			OutputDBMySQL.getInstance().update("[RUNTIME INFO: Cylinder and RX NODE centers ALIGNED (configured bottom distance ignored)!]");
		}


		System.out.println();
		System.out.println();				


		//  ----  END MOVE  ----



		//------------------------------------
		//cylinderOrientation.scalarMul(-1);	// Opposite direction with respect to the emitting point!
		//------------------------------------

		SimpleVector cylinderOrtogonalOrientation=new SimpleVector(0,0,1);	

		boolean useOctree=false;

		//========================================================================================================
		//========================================================================================================
		//========================================================================================================
		//========================================================================================================

		ShellCylindricStrategy ss1=new ShellCylindricStrategy(0, isBottomless);				

		TCPLike_DiffusionModel innerShellDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);	

		ShellCylindricDomain shell=new ShellCylindricDomain(boundaryCube, innerShellDiffusion, ss1, true, true, RXShellHeight, RXShellRadius, cdCenter1, cylinderOrientation, cylinderOrtogonalOrientation, useOctree);


		// EXPORT CYLINDRIC DOMAIN!!!
		getManager().addDomainForExport(shell);


		//  WARNING!! You have to override the rootTree creation rules! 
		//  Because in the AbstractDomain it is defined a DISPLACEMENT! 
		//  (used for the blood flow case...) 

		if(useOctree){
			shell.rootTree.cleanRootLeafs();
			shell.rootTree=null;		
			shell.rootTree=new OctreeRoot(cdCenter1[0], cdCenter1[1], cdCenter1[2], 2*RXShellRadius, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
			shell.rootTree.setDomain(boundaryCube);
		}

		domainForRXNode=shell;
		//}
		//================================
		//       END SHELL CYLINDRIC
		//================================	



		// DISABILITATO TEMPORANEAMENTE!
		/**/
		// CREATE RX NODE!
		TCPLike_Node_RX nodeRX=new TCPLike_Node_RX(domainForRXNode,xCenterRX,yCenterRX,zCenterRX,nodeNumberType,observer); 		

		nodeRX.burstEmission=burst;										// ORIGINAL SIMULATION 	
		nodeRX.RTTAssimilationThreshold=RTTAssimilationThreshold;		// ORIGINAL SIMULATION
		nodeRX.alfa=alfa;												// ORIGINAL SIMULATION 	
		nodeRX.beta=beta;												// ORIGINAL SIMULATION 	
		nodeRX.scheduledWindow=scheduledWindow;							// ORIGINAL SIMULATION - Tw	
		nodeRX.estimator=estimator;										// ORIGINAL SIMULATION 	

		nodeRX.currentEmissionStrategy=c4e;		// ORIGINAL SIMULATION  
		nodeRX.timeForEmissionStrategy=t4e;		// ORIGINAL SIMULATION  
		nodeRX.debug=simDebug;
		nodeRX.receptorStats=receptorStats;
		nodeRX.receptorRandomTime=receptorRandomTime;
		nodeRX.param_RTT=paramRTT;
		nodeRX.cutEnabled=cutEnabled;
		nodeRX.stopEnabled=stopEnabled;
		nodeRX.absorbentReceiver=absorbentReceiver;
		nodeRX.rejectingReceiver=rejectingReceiver;

		nodeRX.receivedPacketType=new int[packetSize];

		if(RXisIdleOnStart){
			nodeRX.isFirstStart=false;	
			TXisIdleOnStart=false;
		}

		nodeRX.setColor(Color.lightGray);


		// IF you have to remove the RX NODE, decomment the following line!
		//nodeRX.kill();


		// ======================================
		// DISABLE CARRIER CHECK COLLISION
		// ======================================
		if(cCollisioncontrol){
			domainForRXNode.ObjFor1Check=0;		// ENABLED!
		}else{
			domainForRXNode.ObjFor1Check=10;	// DISABLED!
		}


		//float r0=nodeRX.getRadius();


		//========================================================================================================
		//========================================================================================================
		//========================================================================================================
		//========================================================================================================


		System.out.println(">>> simulation distance: "+simDistance);
		System.out.println(">>> carrier distance (step): "+cStep+" [nm]");
		System.out.println(">>> carrier burst (NEW punctiform positioning): "+burst);
		//System.out.println(">>> TOTAL DISPOSED CARRIERS: "+createdCarrier);
		//System.out.println();
		System.out.println(">>> RX Node  -  x: "+xCenterRX+", y: "+yCenterRX+", z: "+zCenterRX);
		//System.out.println(">>> xMin: "+xMin);
		//System.out.println(">>> Carriers-RX Mean Distance: "+(dist1/createdCarrier-r0));
		//System.out.println();
		System.out.println(">>> EMISSION PATTERN: "+punctiformPattern);
		System.out.println(">>> Symbol Time: "+symbolTime+" [s]");
		System.out.println();
		/**/


		if(rejectingReceiver && absorbentReceiver){
			System.out.println();
			System.out.println();
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println(">>>  ||  ** WARNING **  You have defined both absorbentReceiver AND rejectingReceiver!  ||");
			System.out.println(">>>  ||   ++++++++++++++   REJECTING RECEIVER RULE WILL BE APPLIED!   ++++++++++++++    ||");
			System.out.println("     +-----------------------------------------------------------------------------------+");
			System.out.println();
			System.out.println();
		}


		System.out.println(">>> SIMULATION ELEMENTS DISPOSED! ");			
		System.out.println();	


	}


	public float[] getPositionForCarrier(float vx,float vy,float vz){
		float[] angles=new float[2];		
		float module=(float) Math.sqrt(vx*vx+vy*vy+vz*vz);			
		angles[0]=0;	//fi
		angles[1]=0;	//theta	
		if (module!=0){
			float thetaArg=vz/module;
			if(thetaArg>1) {
				thetaArg=1;
			}else if(thetaArg<-1){
				thetaArg=-1;
			}

			angles[1]=(float) Math.acos(thetaArg);
		}
		if (Double.isNaN(vy)||Double.isNaN(vx)){
			angles[0]=0;
		}else{
			angles[0]=(float) Math.atan2(vy, vx);
		}		

		return angles;		
	}


	public void networkCreationTCPLike_BitErrorRate_Evaluation(){	

		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [TCP-like Bit Error Rate]"); 
		System.out.println("-------------------------------------");			


		// DOMAIN

		float[] cdCenter=new float[3];
		cdCenter[0]=100000;
		cdCenter[1]=100000;
		cdCenter[2]=100000;	


		float halfSide=(float) (100000/2);  // cube's half side 


		TCPLike_CubicStrategy ss=new TCPLike_CubicStrategy(0);
		TCPLike_DiffusionModel innerDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube=new CubicDomain(getManager().rootDomain, innerDiffusion, ss, true, false, halfSide, cdCenter, true);


		// NODES

		List<LigandObserver> observer=new ArrayList<LigandObserver>();


		boundaryCube.rootTree.cleanRootLeafs();
		boundaryCube.rootTree=null;		
		boundaryCube.rootTree=new OctreeRoot(cdCenter[0], cdCenter[1], cdCenter[2], 2*halfSide, getManager().configurator.m.nodeTreeLimit, getManager().configurator.m.nodeSideLimit);	// OK
		boundaryCube.rootTree.setDomain(boundaryCube);



		// ABS COORDINATES!
		float xCenterRX=xDisplacement;
		float yCenterRX=yDisplacement;
		float zCenterRX=zDisplacement;

		observer.add(TCPLike_CarrierA_Observer.getInstance());
		observer.add(TCPLike_CarrierB_Observer.getInstance());
		TCPLike_Node_RX_BER nodeRX=new TCPLike_Node_RX_BER(boundaryCube,xCenterRX,yCenterRX,zCenterRX,29,observer); 		// ORIGINAL SIMULATION 
		//TCPLike_TEST_RX nodeRX=new TCPLike_TEST_RX(boundaryCube,xCenterRX,yCenterRX,zCenterRX,27,observer); 		// TEST THRESHOLD
		nodeRX.burstEmission=burst;										// ORIGINAL SIMULATION 	
		nodeRX.RTTAssimilationThreshold=RTTAssimilationThreshold;		// ORIGINAL SIMULATION
		//nodeRX.alfa=alfa;												// ORIGINAL SIMULATION 	
		//nodeRX.beta=beta;												// ORIGINAL SIMULATION 	
		nodeRX.scheduledWindow=scheduledWindow;							// ORIGINAL SIMULATION 	
		//nodeRX.estimator=estimator;									// ORIGINAL SIMULATION 	

		System.out.println();
		System.out.println("===========  RX NODE PARAMETERS  ==========");
		System.out.println(">>>  RX BURST: "+burst);
		System.out.println(">>>  RTT Threshold: "+RTTAssimilationThreshold);
		System.out.println(">>>  BER Pattern (0 = RAND): "+nodeRX.patternBER);
		System.out.println("===========================================");
		System.out.println();

		nodeRX.setColor(Color.lightGray);



		float minOctreeLeaf=getManager().configWrapper.nodeSideLimit;		
		float disp2=(float) Math.ceil(Math.sqrt(Math.pow(simDistance,2)/2)/minOctreeLeaf);

		float xCenter=xCenterRX+minOctreeLeaf*disp2;
		float yCenter=yCenterRX+minOctreeLeaf*disp2;
		float zCenter=zCenterRX;


		observer=new ArrayList<LigandObserver>();
		observer.add(TCPLike_CarrierB_Observer.getInstance());
		TCPLike_Node_TX_BER nodeTX=new TCPLike_Node_TX_BER(boundaryCube,xCenter,yCenter,zCenter,28,txEmissionThreshold,observer);		// ORIGINAL SIMULATION  
		nodeTX.shutDown=shutDown;		// ORIGINAL SIMULATION 	

		nodeTX.setColor(Color.darkGray);	


		float r0=nodeRX.getRadius();
		float r1=nodeTX.getRadius();

		float dx=xCenterRX-xCenter;
		float dy=yCenterRX-yCenter;
		float dz=zCenterRX-zCenter;

		float dist=(float) Math.sqrt(dx*dx+dy*dy+dz*dz)-r0-r1;

		System.out.println();
		System.out.println(">>> RX Node  -  x: "+xCenterRX+", y: "+yCenterRX+", z: "+zCenterRX);
		System.out.println(">>> TX Node  -  x: "+xCenter+", y: "+yCenter+", z: "+zCenter);
		System.out.println(">>> TX-RX Distance: "+dist);		
		System.out.println(">>> SIMULATION ELEMENTS DISPOSED! ");			
		System.out.println();	
	}



	public void networkCreation_DEBUG_OCTREE(){	

		System.out.println("-------------------------------------");
		System.out.println("Auto-Create Network... [DEBUG OCTREE]"); 
		System.out.println("-------------------------------------");			


		// DOMAIN

		float[] cdCenter=new float[3];
		cdCenter[0]=100000;
		cdCenter[1]=100000;
		cdCenter[2]=100000;	

		//float halfSide=(float) (63508.5/2);  // cube's half side 
		float halfSide=(float) (100000/2);  // cube's half side 


		TCPLike_CubicStrategy ss=new TCPLike_CubicStrategy(0);
		TCPLike_DiffusionModel innerDiffusion=new TCPLike_DiffusionModel(getManager().rootDomain);		
		CubicDomain boundaryCube=new CubicDomain(getManager().rootDomain, innerDiffusion, ss, true, false, halfSide, cdCenter, true);


		Engine3D e3d=(Engine3D) getManager().engine;

		float rr=(float) (Math.PI/4);
		float side=2*boundaryCube.boundingRadius;
		float sside=(float) (side/(Math.pow(2, 4)));

		float cc=(float) (side/(Math.pow(2, 5)));

		for (int i=1;i<=4;i++){
			for (int j=1;j<=4;j++){				
				for (int k=1;k<=4;k++){

					float x=cdCenter[0]-cc*i;
					float y=cdCenter[1]-cc*j;
					float z=cdCenter[2]-cc*k;	

					e3d.createBox(x, y, z, 0, rr, 0, Color.pink, sside);
				}
			}
		}

		System.out.println(">>> SIMULATION ELEMENTS DISPOSED! ");			
		System.out.println();	
	}






	public void move(List<NanoObj> nanoList){

		//------------------------------------------------------------------
		// NEW IMPLEMENTATION OF PUNCTIFORM EMISSION!    03/09/2015
		//------------------------------------------------------------------
		if(emitter!=null){
			emitter.exec();
		}

		//------------------------------------------------------------------



		if (nanoList.size()>0){

			/*
			// FIXME ONLY FOR DEBUG!!!			
			long time= getManager().getCurrentTime();
			if(time%1000==0){

				ArrayList<OctreeNode> lista=nanoList.get(0).getDomain().getRootTree().getLeafList();

				for(int i=0;i<lista.size();i++){
					System.out.println(lista.get(i).getInfoLight());
					//System.out.println(lista.get(i).getInfoMatlab());
					
				}
			}*/
			
			
			

			//long start=System.nanoTime();

			try{
				threadingStrategy.move(nanoList);				
			}catch (Exception e){
				e.printStackTrace();
			}
			/*
			long stop=System.nanoTime();

			System.out.println(""+(stop-start)/1000);
			 */
		}
	}
}
