package simulator;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.LigandObserver;
import MainLibrary.NoMotion;
import MainLibrary.NodeObj;
import MainLibrary.OutputStrategy;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.nIdle;
import MainLibrary.nTransmit;


public class TCPLike_Node_RX extends NodeObj{
	
	//private long firstContact=-1;	
	
	public int carrierCreated=0;	
	protected long[] releaseTimeLog=new long[10];
	protected int[] releaseCarrierLog=new int[10];
	protected int indexReleaseLog=0;
	//private int indexExportedLog=0;
	protected ArrayList<float[]> releasePositions=new ArrayList<float[]>();
	
	//private boolean hasExported=false;
	protected int timeForEmission=0;
	protected int indexPattern=0;
	protected String emissionPattern="";
	
	protected String patternSTART="";
	protected String patternSTOP="";
	protected String patternCUT="";
	
	protected int countdownForEmission=0;
	
	
	/**  === Estimation of Round Trip Time ===  */
	protected int waitingTimeRTT=50000;		// 200000  timesteps. Couldn't be smaller than the duration of the message! 
	protected int timerRTT=-1;				// timesteps
	protected long emissionLastSimbol_RTT=-1;
	protected int estimatedRTT=-1;			// timesteps
	//private int startForRTT=-1;			// timesteps
	protected boolean isWaitingForRTT=true;
	protected boolean isStillEmitting=false;
	protected boolean hasJustStopped=false;
	
	protected int RTTAssimilationThreshold=1;

	protected long timeFirstAssimilation=-1;
	
	//private int waitingWindow=0;	
	long timestep=0;
	int windowSize=100;
	int pos=0;
	protected int currentAssimilation=0;		// for the circular buffer
	//int lastCount=0;
	protected int[] circularBufferAssimilations= new int[windowSize]; 
	protected int assimilationThreshold=1;
	protected int totalAssimilationThreshold=1000;
	
	protected boolean transientPhase=true;
	protected boolean isFirstStart=true;
	
	
	/**  ==== estimations for CUT or STOP in standardBehaviour ====  */	
	protected int N_prev=5;	// to be parametric (from config.xml or by command line)
	protected long t_prev=0;
	protected long ta=0;
	protected int N_cut=0;
	protected long t_cut=ta;
	protected long t_now=0;
	protected long t_new=0;
	protected double a=0;
	protected double a_prev=0;
	protected float alfa=1;
	protected float beta=(float) 0.95;
	protected boolean estimationInitialized=false;
	protected int timeStop=0;
	protected int timeCut=0;
	protected boolean isScheduled_N_cut=false;
	protected boolean isScheduledNextCheck=false;
	protected long wait=0;
	protected long scheduledWindow=10000;
	protected String estimator="Q";
	protected boolean wait2RTTAfterSTOP=false;
	protected long waitAfterSTOP=0;
	
	protected boolean hasToRefresh_a=true;
	protected boolean hasToRefresh_t_prev=false;
	
	protected double[] a_estimator_list=new double[128];
	protected int a_index=0;
	
	
	protected long t_RTT=0;
	
	protected float param_RTT=(float) 0.5;		
	
	protected double minSquares_x=0;
	protected double minSquares_y=0;
	
	/**  ==== END estimations for CUT or STOP in standardBehaviour ====  */	
	
	
	public String currentEmissionStrategy="HALVE";
	public String timeForEmissionStrategy="NOCHANGE";
	
	public boolean debug=false;
	private boolean hasToWait=false;	
	private long next_estimation=-1;
	
	public boolean receptorStats=false;
	public boolean receptorRandomTime=false;
	
	public long served=0;
	public long busy=0;
	public long missed=0;
	
	private HashMap<String,String> evaluateRTT=new HashMap<String,String>();
	private HashMap<String,String> evaluateCUT=new HashMap<String,String>();
	private HashMap<String,String> evaluateSTOP=new HashMap<String,String>();
	
	private HashMap<String,String> sendSTART=new HashMap<String,String>();
	private HashMap<String,String> sendCUT=new HashMap<String,String>();
	private HashMap<String,String> sendSTOP=new HashMap<String,String>();
	
	private HashMap<String,String> evaluateT_HALVE=new HashMap<String,String>();
	private HashMap<String,String> evaluateN_HALVE=new HashMap<String,String>();
	private HashMap<String,String> evaluateT_PREV=new HashMap<String,String>();
	
	private HashMap<String,String> evaluateN_EXP=new HashMap<String,String>();
	private HashMap<String,String> evaluate_a=new HashMap<String,String>();
	
	private HashMap<String,String> evaluate_packetTypes=new HashMap<String,String>();
	
	
	public int[] receivedPacketType=new int[100]; 	// Keeps trace of each paylod received (its type!!)
	
	public int differentReceivedPacketType=0;		// How many different packets have been received?
	
	public boolean absorbentReceiver=false;
	public boolean rejectingReceiver=false;
	
	private long localExportTime=1000;
	
	
	
	private OutputStrategy outputStrategy_Receptor=OutputTXTReceptors.getInstance();

	public boolean cutEnabled=true;
	public boolean stopEnabled=true;
	
	
	
	public TCPLike_Node_RX(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);
				
		// You can set the internal state here!
		//setState(new nIdle(this));
		//getState().setExecutionTime(1000);	
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
		setColor(Color.white);	
		
		informationCount=0;
		
		// RTT
		isWaitingForRTT=true;
		timerRTT=-1;
		
		// EXPORT MySQL
		//localExportTime=domain.getManager().configWrapper.exportTimeLength*OutputDBMySQL.getInstance().getStoringFactor();
		localExportTime=domain.getManager().configWrapper.exportTimeLength;
		
		/*
		setState(new nTransmit(this));
		getState().setExecutionTime(0);
		*/
		
		try {	
			if(domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-globalAssimilationThreshold") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-patternSTART") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-patternSTOP") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-patternCUT") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-assimilationThreshold") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-timeForAssimilation") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-timeForEmission")){
				timeForEmission=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-timeForEmission").intValue();
				//emissionPattern=Integer.toString((domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-emissionPattern").intValue()));	
				windowSize=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-timeForAssimilation").intValue();
				assimilationThreshold=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-assimilationThreshold").intValue();
				//countdownForEmission=timeForEmission;  // emits after this time
				emissionPattern=Integer.toString((domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-emissionPattern").intValue()));
				patternSTART=Integer.toString(domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-patternSTART").intValue());	
				patternSTOP=Integer.toString(domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-patternSTOP").intValue());	
				patternCUT=Integer.toString(domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-patternCUT").intValue());	
				totalAssimilationThreshold=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-globalAssimilationThreshold").intValue();	
				circularBufferAssimilations= new int[windowSize]; 
				waitingTimeRTT=(domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-timeout").intValue());	
				
				emissionPattern=patternSTART;
				
				
				countdownForEmission=0;		// emitis immediately!!
				
			}else{
				throw new Exception();
			}
		} catch (Exception e) {
			System.out.println();
			System.out.println(">>> WARNING! The transmitter node has not been configured for the EMISSION PATTERN!");
			System.out.println();
			System.out.println("    The simulation ends now");
			System.out.println();
			e.printStackTrace();
			System.exit(0);
		}
	}
	
	@Override
	public void idle() {
		processInfo();
	}
	
	@Override
	public void decode() {
		setState(new nIdle(this));
	}
	
	@Override
	public void encode() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(10);	
	}

	@Override
	public void searchNeighbor() {
		
	}

	public void sendStart(){
		setState(new nTransmit(this));
		getState().setExecutionTime(0);
		System.out.println("======   SEND START (RX)  ======");
		
		isStillEmitting=true;
		indexPattern=0;
		countdownForEmission=0;		
		emissionPattern=patternSTART;
		hasJustStopped=false;
		
		sendSTART.put(Long.toString(getManager().getCurrentTime()), Long.toString(burstEmission));
	}
	
	public void sendStop(){
		setState(new nTransmit(this));
		getState().setExecutionTime(0);
		System.out.println("======   SEND STOP (RX)  ======");
		
		timerRTT=0;
		
		/*
		if(hasJustStopped){
			//waitingTimeRTT=waitingTimeRTT*2;
			burstEmission=burstEmission*2;
		}*/
		
		isStillEmitting=true;
		indexPattern=0;
		countdownForEmission=0;		
		emissionPattern=patternSTOP;
		//waitingWindow=0;
		//waitingWindow=estimatedRTT+windowSize;
		
		hasJustStopped=true;
		
		sendSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(burstEmission));
	}
	
	public void sendCut(){
		setState(new nTransmit(this));
		getState().setExecutionTime(0);
		System.out.println("======   SEND CUT (RX)   ======");
		
		timerRTT=0;
		
		isStillEmitting=true;
		indexPattern=0;
		countdownForEmission=0;		
		emissionPattern=patternCUT;
		//waitingWindow=0;
		//waitingWindow=estimatedRTT+windowSize;
		
		sendCUT.put(Long.toString(getManager().getCurrentTime()), Long.toString(burstEmission));
	}
	
	@Override
	public void transmit() {		
		
		/*
		if(informationCount>0 && emissionPattern!="10") {
			emissionPattern="10";
			indexPattern=0;
			countdownForEmission=timeForEmission;
		}*/
		
		if (indexPattern<emissionPattern.length()){
			countdownForEmission--;
			if(countdownForEmission<=0){

				int created=0;
				if(emissionPattern.charAt(indexPattern)=='1'){
					
					created=createCarriers(burstEmission);				
					System.out.println(">>> RX Releasing CARRIERS!!! (Control Node) "+created);
					
					/*
					if(indexPattern==0 && isWaitingForRTT && timerRTT<0){
						// The pattern has to start with a 1, otherwise it NEVER monitor the RTT!
						timerRTT=1;
					}
					*/					
				}
				
				//FIXME New RTT Estimation
				if(indexPattern==emissionPattern.length()-1 && isWaitingForRTT && timerRTT<0){
					// The pattern has to start with a 1, otherwise it NEVER monitor the RTT!
					timerRTT=1;
					emissionLastSimbol_RTT=getManager().getCurrentTime();
				}
				
				
				if(releaseTimeLog.length<=indexReleaseLog){
					enlargeReleaseTimeLog();
				}
				releaseTimeLog[indexReleaseLog]=getManager().getCurrentTime();
				releaseCarrierLog[indexReleaseLog]=created;

				float[] center=getCenter();
				float[] domainCenter=getDomain().getAbsCenter();					

				float[] currentRelativePosition={center[0]-domainCenter[0],center[1]-domainCenter[1],center[2]-domainCenter[2]};
				releasePositions.add(currentRelativePosition);
				indexReleaseLog++;

				indexPattern++;
				countdownForEmission=timeForEmission;
			}
		}else{
			countdownForEmission--;
			if(countdownForEmission<=0){
				isStillEmitting=false;
				setState(new nIdle(this));
				getState().setExecutionTime(10);
				countdownForEmission=timeForEmission;
			}			
		}		
	}
	
	
	
	public void enlargeReleaseTimeLog(){
		long[] temp=releaseTimeLog;
		int[] tempCarrier=releaseCarrierLog;
		int size=temp.length;
		size=size+size/2;
		
		releaseTimeLog=new long[size];
		releaseCarrierLog=new int[size];
		
		for(int i=0;i<temp.length;i++){
			releaseTimeLog[i]=temp[i];
			releaseCarrierLog[i]=tempCarrier[i];
		}		
	}
	

	public int createCarriers(int n){
		int createdNow=0;
		try {
			
			while (n>0){				
				TCPLike_Carrier_B cc=new TCPLike_Carrier_B(getInnerDomain(), this, getCenter()[0], getCenter()[1], getCenter()[2],4);			
				cc.setMotionStrategy(BrownianMotion.getInstance());
				n--;
				createdNow++;			
			}
			carrierCreated=carrierCreated+createdNow;
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return createdNow;
	}
		
	@Override
	public Map<Object, Object> specializedExport() {

		Map<Object, Object> m=new HashMap<Object,Object>();
		
		boolean hasToExport=false;
		long time=getManager().getCurrentTime();
		
		//if(time%10000==0){
		if(time%localExportTime==0){
			hasToExport=true;
		}
		
		if(hasToExport || sendSTART.size()>0 || sendCUT.size()>0 || sendSTOP.size()>0 || evaluateCUT.size()>0 || evaluateSTOP.size()>0 || evaluateRTT.size()>0 
				|| evaluateT_HALVE.size()>0 || evaluateN_HALVE.size()>0 || evaluateT_PREV.size()>0 || evaluateN_EXP.size()>0 || evaluate_a.size()>0){
			m.put("Type", "TCPLike-RX-Node");
   
			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		

			m.put("ID", "TCPLike-RX-Node-"+ID);	
			m.put("X", center[0]-domainCenter[0]);
			m.put("Y", center[1]-domainCenter[1]);
			m.put("Z", center[2]-domainCenter[2]);
			
			
			if(hasToExport){
				m.put("CRON_TimeStep", time+"");	
				m.put("CRON_Value", informationCount+"");	
				m.put("CRON_Action", "RX assimilation");		
			}
			
			if(receptorStats){

			    m.put("SERVED_TimeStep", ""+time+"");	
			    m.put("SERVED_Value", served);	
			    m.put("SERVED_Action", "lambda_a");
			    
			    m.put("BUSY_TimeStep", ""+time+"");	
			    m.put("BUSY_Value", busy);	
			    m.put("BUSY_Action", "lambda_r");
			    
			    m.put("MISSED_TimeStep", ""+time+"");	
			    m.put("MISSED_Value", missed);	
			    m.put("MISSED_Action", "lambda_m");
			    
			    
			    if(time%100000==0){
			    	Thread t = new Thread(new Runnable() { public void run() { 
			    		Map<Object, Object> receptorMap=new HashMap<Object,Object>();

				    	for(Receptor rr : receptorList){
				    		receptorMap.putAll(rr.export());
				    	}		

				    	outputStrategy_Receptor.export(receptorMap);
			    		}});
			    	
			    	t.start();			    	
			    }
			}
					
			
			
			int i=0;
			
			Iterator it = sendSTART.entrySet().iterator();
		    while (it.hasNext()) {
		        Map.Entry pairs = (Map.Entry)it.next();		        
		        
		        m.put("START_TimeStep_"+i, ""+pairs.getKey()+"");	
				m.put("START_Value_"+i, pairs.getValue());	
				m.put("START_Action_"+i, "send START");
				i++;
				
				//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("START_TOTAL", i);
		    sendSTART.clear();
		    
		    i=0;
			
			Iterator itCUT = sendCUT.entrySet().iterator();
		    while (itCUT.hasNext()) {
		        Map.Entry pairs = (Map.Entry)itCUT.next();		        
		        
		        m.put("CUT_TimeStep_"+i, ""+pairs.getKey()+"");	
				m.put("CUT_Value_"+i, pairs.getValue());	
				m.put("CUT_Action_"+i, "send CUT");
				i++;
				
				//itCUT.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("CUT_TOTAL", i);
		    sendCUT.clear();
		    
		    i=0;
			
			Iterator itSTOP = sendSTOP.entrySet().iterator();
		    while (itSTOP.hasNext()) {
		        Map.Entry pairs = (Map.Entry)itSTOP.next();		        
		        
		        m.put("STOP_TimeStep_"+i, ""+pairs.getKey()+"");	
				m.put("STOP_Value_"+i, pairs.getValue());	
				m.put("STOP_Action_"+i, "send STOP");
				i++;
				
				//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("STOP_TOTAL", i);
		    sendSTOP.clear();
			

		   	    
		    i=0;

		    Iterator itCUT2 = evaluateCUT.entrySet().iterator();
		    while (itCUT2.hasNext()) {
		    	Map.Entry pairs = (Map.Entry)itCUT2.next();		        

		    	m.put("CUT_Assimilations_TimeStep_"+i, ""+pairs.getKey()+"");	
		    	m.put("CUT_Assimilations_Value_"+i, pairs.getValue());	
		    	m.put("CUT_Assimilations_Action_"+i, "current assimilations (CUT)");
		    	i++;

		    	//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("CUT_Assimilations_TOTAL", i);
		    evaluateCUT.clear();
		    
		    
		    i=0;

		    Iterator itSTOP2 = evaluateSTOP.entrySet().iterator();
		    while (itSTOP2.hasNext()) {
		    	Map.Entry pairs = (Map.Entry)itSTOP2.next();		        

		    	m.put("STOP_Assimilations_TimeStep_"+i, ""+pairs.getKey()+"");	
		    	m.put("STOP_Assimilations_Value_"+i, pairs.getValue());	
		    	m.put("STOP_Assimilations_Action_"+i, "global assimilations (STOP)");
		    	i++;

		    	//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("STOP_Assimilations_TOTAL", i);
		    evaluateSTOP.clear();
		    
		    
		    i=0;

		    Iterator itRTT = evaluateRTT.entrySet().iterator();
		    while (itRTT.hasNext()) {
		    	Map.Entry pairs = (Map.Entry)itRTT.next();		        

		    	m.put("RTT_TimeStep_"+i, ""+pairs.getKey()+"");	
		    	m.put("RTT_Value_"+i, pairs.getValue());	
		    	m.put("RTT_Action_"+i, "RTT");
		    	i++;

		    	//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("RTT_TOTAL", i);
		    evaluateRTT.clear();
		    
		    
		    i=0;

		    Iterator itT_HALVE = evaluateT_HALVE.entrySet().iterator();
		    while (itT_HALVE.hasNext()) {
		    	Map.Entry pairs = (Map.Entry)itT_HALVE.next();		        

		    	m.put("T_HALVE_TimeStep_"+i, ""+pairs.getKey()+"");	
		    	m.put("T_HALVE_Value_"+i, pairs.getValue());	
		    	m.put("T_HALVE_Action_"+i, "t_halve");
		    	i++;

		    	//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("T_HALVE_TOTAL", i);
		    evaluateT_HALVE.clear();
		    
		    
		    i=0;

		    Iterator itN_HALVE = evaluateN_HALVE.entrySet().iterator();
		    while (itN_HALVE.hasNext()) {
		    	Map.Entry pairs = (Map.Entry)itN_HALVE.next();		        

		    	m.put("N_HALVE_TimeStep_"+i, ""+pairs.getKey()+"");	
		    	m.put("N_HALVE_Value_"+i, pairs.getValue());	
		    	m.put("N_HALVE_Action_"+i, "n_halve");
		    	i++;

		    	//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("N_HALVE_TOTAL", i);
		    evaluateN_HALVE.clear();
		    
		    
		    i=0;

		    Iterator itT_PREV = evaluateT_PREV.entrySet().iterator();
		    while (itT_PREV.hasNext()) {
		    	Map.Entry pairs = (Map.Entry)itT_PREV.next();		        

		    	m.put("T_PREV_TimeStep_"+i, ""+pairs.getKey()+"");	
		    	m.put("T_PREV_Value_"+i, pairs.getValue());	
		    	m.put("T_PREV_Action_"+i, "t_prev");
		    	i++;

		    	//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("T_PREV_TOTAL", i);
		    evaluateT_PREV.clear();
		    
		    
		    
		    
		    i=0;

		    Iterator itN_EXP = evaluateN_EXP.entrySet().iterator();
		    while (itN_EXP.hasNext()) {
		    	Map.Entry pairs = (Map.Entry)itN_EXP.next();		        

		    	m.put("N_EXP_TimeStep_"+i, ""+pairs.getKey()+"");	
		    	m.put("N_EXP_Value_"+i, pairs.getValue());	
		    	m.put("N_EXP_Action_"+i, "n_exp");
		    	i++;

		    	//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("N_EXP_TOTAL", i);
		    evaluateN_EXP.clear();
		    
		    
		    i=0;

		    Iterator ita = evaluate_a.entrySet().iterator();
		    while (ita.hasNext()) {
		    	Map.Entry pairs = (Map.Entry)ita.next();		        

		    	m.put("a_TimeStep_"+i, ""+pairs.getKey()+"");	
		    	m.put("a_Value_"+i, pairs.getValue());	
		    	m.put("a_Action_"+i, "a");
		    	i++;

		    	//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("a_TOTAL", i);
		    evaluate_a.clear();
		    
		    
		    
		    m.put("packetTypes_TimeStep", ""+time+""); 
		    m.put("packetTypes_Value", differentReceivedPacketType);		    
		    m.put("packetTypes_Action", "packetTypes");
		    
		    
		    
		    
			
			m.putAll(getState().export());
		}
		return m;
	}	
	
	
	@Override
	public void createSingleReceptor(float[] cdCenter, float radius, short receptorRadius, int receptorType) {
		float[] angles=convertSphericalAngles(cdCenter[0],cdCenter[1],cdCenter[2], radius);
		float theta=angles[1];
		float fi=angles[0];

		
		TCPLike_ReceptorB r=new TCPLike_ReceptorB(this, true, cdCenter, theta, fi, receptorRadius, receptorType, this.eventHandler);
		receptorList.add(r);
		receptorSetup(r); 

		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		*/		
	}
	
	public void createReceptor(int n){
		Receptor r = null;
		
		float theta=0;
		float fi=0;				
		
		for (int i=0;i<n;i++){	
			
			/*
			// Creates the receptors for UPLOAD
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];		
			
			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new TCPLike_ReceptorB(this, false, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			//receptorSetup(r); 
			r.attachObs(r.getMyObserver());
			*/
			
			// Creates the receptors for DOWNLOAD
			
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];				
			
				
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new TCPLike_ReceptorA(this, true, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler, i);
			receptorList.add(r);
			//receptorSetup(r); 			
			r.attachObs(r.getMyObserver());
		}
	}	
	
	
	
	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	
	}
	
	@Override
	public void setMotionStrategy() {		
		//this.motionStrategy=BrownianMotion.getInstance();
		this.motionStrategy=NoMotion.getInstance();
	}
	
	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}
		
	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	@Override
	public void processInfo() {
		/*
		if(getCollisionCount()>0 && firstContact<0){
			firstContact=getManager().getCurrentTime();				
		}*/	
	}
	
	public String assignName(){		
		return "TCPLike-RX Node";		
	}
	
	public void exec(){
		
		try {
			timestep=getManager().getCurrentTime();
			//int bufferSize=circularBufferAssimilations.length;
			pos=(int) (timestep%windowSize);
			
			currentAssimilation=0;
			circularBufferAssimilations[pos]=0;
			
			
			super.exec();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	public void downloadInfoFrom(CarrierObj c, Receptor r){		
		informationCount++;	
		
		receivedPacketType[c.packetType]=receivedPacketType[c.packetType] + 1;	
		
		
		currentAssimilation++;
		circularBufferAssimilations[pos]=currentAssimilation;
		
		if (timeFirstAssimilation<0) timeFirstAssimilation=getManager().getCurrentTime();
		
		if(isWaitingForRTT && informationCount>=RTTAssimilationThreshold && c instanceof TCPLike_Carrier_A){
			
			//FIXME OLD RTT
			//estimatedRTT=timerRTT+timeForEmission;		// Added timeForEmission. Otherwise the RTT starts from the end of the emitted signal and not from the start!	
			
			t_RTT=getManager().getCurrentTime();
			//FIXME New RTT
			estimatedRTT=(int) (t_RTT-emissionLastSimbol_RTT);
			
			
			transientPhase=true;	// required for quadraticEstimation()
			
			isWaitingForRTT=false;
			//windowSize=estimatedRTT;	// FIXME Have to be equal to the duration of the symbol emitted by the other node
			circularBufferAssimilations= new int[windowSize]; 
			timerRTT=0;
			//waitingWindow=estimatedRTT+windowSize;
			evaluateRTT.put(Long.toString(getManager().getCurrentTime()), Long.toString(estimatedRTT));
			
		}
		c.destroy();
	}
	
	/*
	public void downloadCarrier(CarrierObj c) {	
		
		if(c instanceof CalciumCarrier){
			assimilatedIons++;
		}else if(c instanceof CalciumCarrierBG){
			assimilatedBG++;
		}
		
		c.kill(); 		
	}*/
	
	public long getCircularBufferAssimilationSum(){
		long sum=0;
		int size=circularBufferAssimilations.length;
		
		for(int i=0;i<size;i++){
			sum=sum+circularBufferAssimilations[i];
		}
		
		return sum;
	}
	
	
	//  ========   NO CUT!   ========
	//@Override	
	public void standardBehavior_NO_CUT() {


		if(isFirstStart){
			isFirstStart=false;
			sendStart();	
			
			System.out.println();
			System.out.println(">>>>>>>>>>>>>    IL CUT E' DISABILITATO!!    <<<<<<<<<<<<<<<");
			System.out.println();
		}

		// ========================================
		// WAITS FOR THE RTT AFTER THE START SIGNAL
		// ========================================
		if(!isStillEmitting){
			if(isWaitingForRTT && timerRTT>=0){

				if(timerRTT<waitingTimeRTT-1){

					timerRTT++;
				}else{
					if(!(indexPattern<emissionPattern.length())){

						// Assumes that the node has finished to transmit the control signal					
						timerRTT=-1;
						//waitingTimeRTT=(waitingTimeRTT)*2;


						//burstEmission=burstEmission*2;
						burstEmission=burstEmission+1000;


						sendStart();	// Sets the internal state to TRANSMIT, so on the next timestep will start the emission!

					}else{					
						// Waits for the completion of the previous emission...
						timerRTT++;
					}				
				}	

			}else if (informationCount>0){

				// ====================================
				// MONITOR THE ASSIMILATIONS FOR THE RTT AFTER "START" SIGNAL
				// ====================================


				//FIXME Observe for each timestep. It will evaluates the buffer (which observe the previous Ts)

				long currAssimilation=getCircularBufferAssimilationSum();

				if (informationCount>totalAssimilationThreshold){
					//if (informationCount>1500 && timerRTT>=(estimatedRTT+windowSize)){
					// SEND STOP MESSAGE!
					evaluateSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
					sendStop();

					//}else if (currAssimilation>assimilationThreshold && timerRTT>=(estimatedRTT+windowSize)){
				}else {


					/** NEW IMPLEMENTATION WITHOUT CUT */
					long now=getManager().getCurrentTime();
					
					double a=currAssimilation/Math.pow((now-timeFirstAssimilation),2);
					int timeStop=patternSTOP.length()-1;
					
					
					if(Math.pow((now+estimatedRTT+timeStop*timeForEmission-timeFirstAssimilation),2)*a>=totalAssimilationThreshold){
						// SEND STOP MESSAGE!
						evaluateSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
						sendStop();
					}					
				}
			}
		}

		try {
			emitsWithoutReceptors();
			//emitsWithoutReceptorsForDEBUG();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}


	
	public void standardBehavior() {
				
		
		if(isFirstStart){
			isFirstStart=false;
			sendStart();			
		}
		
		t_now=getManager().getCurrentTime();
		
		if(isScheduled_N_cut && t_now>=t_cut){					
			N_cut=informationCount;
			
			evaluateN_HALVE.put(Long.toString(getManager().getCurrentTime()), Integer.toString(N_cut));
			
			isScheduled_N_cut=false;
		}
		
		// ========================================
		// WAITS FOR THE RTT AFTER THE START SIGNAL
		// ========================================
		if(!isStillEmitting){
			if(isWaitingForRTT && timerRTT>=0){

				if(timerRTT<waitingTimeRTT-1){
					
					timerRTT++;
				}else{
					if(!(indexPattern<emissionPattern.length())){

						// Assumes that the node has finished to transmit the control signal					
						timerRTT=-1;
						//waitingTimeRTT=(waitingTimeRTT)*2;
						
						
						//burstEmission=burstEmission*2;
						burstEmission=burstEmission+1000;
						

						sendStart();	// Sets the internal state to TRANSMIT, so on the next timestep will start the emission!

					}else{					
						// Waits for the completion of the previous emission...
						timerRTT++;
					}				
				}	

			}else if (informationCount>0){

				// ====================================
				// MONITOR THE ASSIMILATIONS FOR THE RTT AFTER "START" SIGNAL
				// ====================================

				if (!estimationInitialized){					

					ta=timeFirstAssimilation;		// FIXME BUG??
					
					N_prev=informationCount;
					//t_prev=ta;
					t_prev=getManager().getCurrentTime();
					N_cut=informationCount;
					//t_cut=ta;
					t_cut=getManager().getCurrentTime();
					timeStop=(patternSTOP.length()-1)*timeForEmission;
					timeCut=(patternCUT.length()-1)*timeForEmission;

					estimationInitialized=true;					
				}



				if(wait2RTTAfterSTOP){
					if(waitAfterSTOP>=2*estimatedRTT){
						wait2RTTAfterSTOP=false;
						waitAfterSTOP=0;
					}else{
						waitAfterSTOP++;
					}
				}else{

					if(isScheduledNextCheck){					
						if(wait>=scheduledWindow){
							isScheduledNextCheck=false;
							wait=0;
						}else{
							wait++;
						}
					}


					if(estimator.equals("Q")){
						quadraticEstimation();
						
					}else if(estimator.equals("P")){
						quadraticPacketEstimation();
						
					}else{
						linearEstimation();
					}				
				}
			}
		}


		try {
			emitsWithoutReceptors();
			//emitsWithoutReceptorsForDEBUG();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private enum Command {
	    RESTART, HALVE, NOCHANGE, DOUBLE;
	}
			
	public int get_estimation_for_t_prev(String value){

		Command cmd = Command.valueOf(value); // surround with try/catch

		int result=0;
		
		switch (cmd) {
		case RESTART:  //hasToRefresh_t_prev=false;
			result=1;
		break;
		
		case HALVE:  //hasToRefresh_t_prev=true;
			result=2;
		break;
		
		case NOCHANGE:  //hasToRefresh_t_prev=false;
			result=0;
		break;
		
		default: //hasToRefresh_t_prev=false;
			result=0;
		break;

		}
		//return hasToRefresh_t_prev;
		return result;
	}
	
	public boolean get_estimation_for_a(String value){
		Command cmd = Command.valueOf(value); // surround with try/catch

		boolean result=false;
		
		switch (cmd) {
		case DOUBLE: //hasToRefresh_a=true;
			result=true;
		break;
		
		case NOCHANGE:  //hasToRefresh_a=false;
			result=false;
		break;
		
		default: //hasToRefresh_a=false;
			result=false;
		break;
		}
		//return hasToRefresh_a;
		return result;
	}
	
	public double get_param_a_estimation(){
		
		
		int count=0;
		for(int i=0;i<a_estimator_list.length;i++){
			if(a_estimator_list[i]!=0){
				a=a+a_estimator_list[i];
				count++;
			}			
		}	
		a=a/count;
		return a;		
	}
	

	public void quadraticEstimation(){

		/**  QUADRATIC  */	

		//long currAssimilation=getCircularBufferAssimilationSum();	

		if(informationCount>=totalAssimilationThreshold && !isScheduledNextCheck && stopEnabled){

			// SEND STOP MESSAGE!
			evaluateSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
			sendStop();

			wait2RTTAfterSTOP=true;				
			//hasBreak=true;
			//System.out.println(">>>  SEND STOP 1  <<<");

		}else{

			//if (informationCount>=assimilationThreshold && t_now>t_cut && !isScheduledNextCheck){
			//if (t_now>t_cut && !isScheduledNextCheck){		// CHECK CONTROL on the 250 assimilations is DISABLED
			if(!isScheduledNextCheck){
				if (t_now>t_cut+scheduledWindow){				// CHECK CONTROL on the 250 assimilations is DISABLED

					if(transientPhase){

						/* ORIGINAL BEFORE ESTIMATION
					if(t_now>=(t_cut+estimatedRTT/2)){

						//a=(informationCount-N_cut)/Math.pow((t_now-t_cut),alfa);

						a=(informationCount-N_cut)/Math.pow((t_now-t_cut),alfa);						

						transientPhase=false;
						hasToWait=false;
					}else{
						hasToWait=true;
					}*/

						// ESTIMATION OF PARAM a
						/*
						if(t_now>=(t_cut+estimatedRTT/4)){

							long RTT_64=estimatedRTT/64;

							if(t_now%RTT_64==0){

								a_estimator_list[a_index]=(informationCount-N_cut)/Math.pow((t_now-t_cut),alfa);								
								evaluate_a.put(Long.toString(getManager().getCurrentTime()), Double.toString(a_estimator_list[a_index]));
								a_index++;
							}

							if(t_now>(t_cut+estimatedRTT/2)){

								a=get_param_a_estimation();
								transientPhase=false;
								hasToWait=false;
								evaluate_a.put(Long.toString(getManager().getCurrentTime()), Double.toString(a));
							}else{
								hasToWait=true;
							}
						}else{
							hasToWait=true;
						}*/
						
						
						
						
						/*
						// NEW ESTIMATION OF PARAM a
						if(t_now>=(t_cut+estimatedRTT/4)){

							int RTT_64=estimatedRTT/64;							

							if(next_estimation<0){
								next_estimation=(t_cut+estimatedRTT/4)+RTT_64;
							}							
							
							if(t_now>=next_estimation){

								a_estimator_list[a_index]=(informationCount-N_cut)/Math.pow((t_now-t_cut),alfa);								
								evaluate_a.put(Long.toString(getManager().getCurrentTime()), Double.toString(a_estimator_list[a_index]));
								
								System.out.println("[a: "+a_estimator_list[a_index]+" ]");
								
								a_index++;
								next_estimation=t_now+RTT_64;								
							}

							if(t_now>(t_cut+estimatedRTT/2)){

								a=get_param_a_estimation();
								transientPhase=false;
								hasToWait=false;
								evaluate_a.put(Long.toString(getManager().getCurrentTime()), Double.toString(a));
							}else{
								hasToWait=true;
							}
						}else{
							hasToWait=true;
						}*/
						
						minSquaresEstimation();
					}
						
						

					if(!hasToWait){
						boolean hasBreak=false;

						/*
				if(hasToRefresh_a){
					a=(informationCount-N_cut)/Math.pow((t_now-t_cut),alfa);
					hasToRefresh_a=false;
				}*/

						t_new=t_now+estimatedRTT+timeStop;

						
						if(informationCount+2*a*(t_now-t_prev)*(t_new-t_now)>=totalAssimilationThreshold && stopEnabled){
						// Correzione del 23/07/2014 perch quello sotto forse  buggato...
						//if((beta*(a*Math.pow((t_new-t_prev),alfa)-a*Math.pow((t_cut-t_prev),alfa))+N_cut)>=totalAssimilationThreshold){
							// Commentato il 23/07/2014 perch forse buggato...
							//if((beta*(a*Math.pow((t_new-t_cut),alfa)-a*Math.pow((t_cut-t_prev),alfa))+N_cut)>=totalAssimilationThreshold){
							
							//if((beta*a*Math.pow((t_new-t_cut),alfa)+N_cut)>=totalAssimilationThreshold){

							// SEND STOP MESSAGE!
							evaluateSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
							sendStop();

							wait2RTTAfterSTOP=true;				
							hasBreak=true;
							//System.out.println(">>>  SEND STOP 2  <<<");

						}else if(cutEnabled){
							//if(t_prev>t_cut){

							double N_exp=beta*(a*Math.pow((t_now-t_prev),alfa)-a*Math.pow((t_cut-t_prev),alfa))+N_cut;						
							//double N_exp=beta*(a_prev*Math.pow((t_now-t_cut),alfa))+N_cut;

							System.out.println("=====   N_exp: "+N_exp+"   -   a: "+a+"   =====");
							evaluateN_EXP.put(Long.toString(getManager().getCurrentTime()), Double.toString(N_exp));
							

							if(N_exp>informationCount){

								t_cut=t_now+timeCut+estimatedRTT;		// At this time you have to store the assimilations to N_cut
								isScheduled_N_cut=true;

								// =============================
								if(get_estimation_for_a(timeForEmissionStrategy)){
									// DOUBLE
									a=a/4;
									scheduledWindow=scheduledWindow*2;

									if(get_estimation_for_t_prev(currentEmissionStrategy)==2){
										// HALVE
										t_prev=t_prev;			// new strategy... 23/05/2014
									}else if(get_estimation_for_t_prev(currentEmissionStrategy)==1){
										// RESTART
										t_prev=t_cut;			// new strategy... 23/05/2014
									}
									evaluateT_PREV.put(Long.toString(getManager().getCurrentTime()), Long.toString(t_prev));
									evaluate_a.put(Long.toString(getManager().getCurrentTime()), Double.toString(a));

								}else{
									// NOCHANGE
									// param a has not changed!!!
									if(get_estimation_for_t_prev(currentEmissionStrategy)==2){
										// HALVE
										t_prev=(t_cut+t_prev)/2;			// new strategy... 23/05/2014
									}else if(get_estimation_for_t_prev(currentEmissionStrategy)==1){
										// RESTART
										t_prev=t_cut;			// new strategy... 23/05/2014
									}
									evaluateT_PREV.put(Long.toString(getManager().getCurrentTime()), Long.toString(t_prev));
								}
								// =============================

								evaluateT_HALVE.put(Long.toString(getManager().getCurrentTime()), Long.toString(t_cut));


								// SEND CUT MESSAGE!
								//evaluateCUT.put(Long.toString(getManager().getCurrentTime()), Long.toString(currAssimilation));
								evaluateCUT.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
								sendCut();							

								// ===============================================================================
								// FIXME  DECOMMENT WHAT FOLLOWS IF YOU WANT TO RECALC THE a COEFF. AFTER EACH CUT
								// hasToRefresh_a=true;
								// ===============================================================================

								hasBreak=true;
							}
							//}
						}
						if(!hasBreak){
							N_prev=informationCount;		// FIXME Non serve piu'??
							//a_prev=a;
							//t_prev=t_now;
							//t_prev=t_now;	// FIXME Provo a riabilitarlo...
							setScheduledNextCheck();
						}
					}	
				}else{
					setScheduledNextCheck();
				}
			}
		}
	}
	
	
	
	public boolean hasReceivedAllPackets(){
		
		boolean hasFinished=true;
		
		int types=0;
				
		int size=receivedPacketType.length;
		
		for (int i=0;i<size;i++){
			if(receivedPacketType[i]<1) {
				hasFinished=false;
				
			}else{
				types=types+1;
			}
		}
		
		differentReceivedPacketType=types;
		
		return hasFinished;
		
	}
	
	/*
	public boolean hasReceivedAllPackets(){
		
		boolean hasFinished=true;
		boolean done=false;
				
		int size=receivedPacketType.length;
		
		for (int i=0;i<size && !done;i++){
			if(receivedPacketType[i]<1) {
				hasFinished=false;
				done=true;
			}
		}		
		
		return hasFinished;		
	}
	*/
	
	public void quadraticPacketEstimation(){

		/**  QUADRATIC PACKET ESTIMATION  -  19/12/2014  */	

		//long currAssimilation=getCircularBufferAssimilationSum();	

		//if(informationCount>=totalAssimilationThreshold && !isScheduledNextCheck){
		if(hasReceivedAllPackets()){
			// SEND STOP MESSAGE!
			evaluateSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
			sendStop();

			wait2RTTAfterSTOP=true;				
			//hasBreak=true;
			//System.out.println(">>>  SEND STOP 1  <<<");

		}else{

			//if (informationCount>=assimilationThreshold && t_now>t_cut && !isScheduledNextCheck){
			//if (t_now>t_cut && !isScheduledNextCheck){		// CHECK CONTROL on the 250 assimilations is DISABLED
			if(!isScheduledNextCheck){
				if (t_now>t_cut+scheduledWindow){				// CHECK CONTROL on the 250 assimilations is DISABLED

					if(transientPhase){

						/* ORIGINAL BEFORE ESTIMATION
					if(t_now>=(t_cut+estimatedRTT/2)){

						//a=(informationCount-N_cut)/Math.pow((t_now-t_cut),alfa);

						a=(informationCount-N_cut)/Math.pow((t_now-t_cut),alfa);						

						transientPhase=false;
						hasToWait=false;
					}else{
						hasToWait=true;
					}*/

						// ESTIMATION OF PARAM a
						/*
						if(t_now>=(t_cut+estimatedRTT/4)){

							long RTT_64=estimatedRTT/64;

							if(t_now%RTT_64==0){

								a_estimator_list[a_index]=(informationCount-N_cut)/Math.pow((t_now-t_cut),alfa);								
								evaluate_a.put(Long.toString(getManager().getCurrentTime()), Double.toString(a_estimator_list[a_index]));
								a_index++;
							}

							if(t_now>(t_cut+estimatedRTT/2)){

								a=get_param_a_estimation();
								transientPhase=false;
								hasToWait=false;
								evaluate_a.put(Long.toString(getManager().getCurrentTime()), Double.toString(a));
							}else{
								hasToWait=true;
							}
						}else{
							hasToWait=true;
						}*/
						
						
						
						
						/*
						// NEW ESTIMATION OF PARAM a
						if(t_now>=(t_cut+estimatedRTT/4)){

							int RTT_64=estimatedRTT/64;							

							if(next_estimation<0){
								next_estimation=(t_cut+estimatedRTT/4)+RTT_64;
							}							
							
							if(t_now>=next_estimation){

								a_estimator_list[a_index]=(informationCount-N_cut)/Math.pow((t_now-t_cut),alfa);								
								evaluate_a.put(Long.toString(getManager().getCurrentTime()), Double.toString(a_estimator_list[a_index]));
								
								System.out.println("[a: "+a_estimator_list[a_index]+" ]");
								
								a_index++;
								next_estimation=t_now+RTT_64;								
							}

							if(t_now>(t_cut+estimatedRTT/2)){

								a=get_param_a_estimation();
								transientPhase=false;
								hasToWait=false;
								evaluate_a.put(Long.toString(getManager().getCurrentTime()), Double.toString(a));
							}else{
								hasToWait=true;
							}
						}else{
							hasToWait=true;
						}*/
						
						minSquaresEstimation();
					}
						
						

					if(!hasToWait){
						boolean hasBreak=false;

						/*
				if(hasToRefresh_a){
					a=(informationCount-N_cut)/Math.pow((t_now-t_cut),alfa);
					hasToRefresh_a=false;
				}*/

						t_new=t_now+estimatedRTT+timeStop;

						// ORIGINAL QUADRATIC
						//if(informationCount+2*a*(t_now-t_prev)*(t_new-t_now)>=totalAssimilationThreshold){
						
						//NEW PACKET
						if(hasReceivedAllPackets()){
							
						// Correzione del 23/07/2014 perch quello sotto forse  buggato...
						//if((beta*(a*Math.pow((t_new-t_prev),alfa)-a*Math.pow((t_cut-t_prev),alfa))+N_cut)>=totalAssimilationThreshold){
							// Commentato il 23/07/2014 perch forse buggato...
							//if((beta*(a*Math.pow((t_new-t_cut),alfa)-a*Math.pow((t_cut-t_prev),alfa))+N_cut)>=totalAssimilationThreshold){
							
							//if((beta*a*Math.pow((t_new-t_cut),alfa)+N_cut)>=totalAssimilationThreshold){

							// SEND STOP MESSAGE!
							evaluateSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
							sendStop();

							wait2RTTAfterSTOP=true;				
							hasBreak=true;
							//System.out.println(">>>  SEND STOP 2  <<<");

						}else if(cutEnabled){
							//if(t_prev>t_cut){

							double N_exp=beta*(a*Math.pow((t_now-t_prev),alfa)-a*Math.pow((t_cut-t_prev),alfa))+N_cut;						
							//double N_exp=beta*(a_prev*Math.pow((t_now-t_cut),alfa))+N_cut;

							System.out.println("=====   N_exp: "+N_exp+"   -   a: "+a+"   =====");
							evaluateN_EXP.put(Long.toString(getManager().getCurrentTime()), Double.toString(N_exp));
							

							if(N_exp>informationCount){

								t_cut=t_now+timeCut+estimatedRTT;		// At this time you have to store the assimilations to N_cut
								isScheduled_N_cut=true;

								// =============================
								if(get_estimation_for_a(timeForEmissionStrategy)){
									// DOUBLE
									a=a/4;
									scheduledWindow=scheduledWindow*2;

									if(get_estimation_for_t_prev(currentEmissionStrategy)==2){
										// HALVE
										t_prev=t_prev;			// new strategy... 23/05/2014
									}else if(get_estimation_for_t_prev(currentEmissionStrategy)==1){
										// RESTART
										t_prev=t_cut;			// new strategy... 23/05/2014
									}
									evaluateT_PREV.put(Long.toString(getManager().getCurrentTime()), Long.toString(t_prev));
									evaluate_a.put(Long.toString(getManager().getCurrentTime()), Double.toString(a));

								}else{
									// NOCHANGE
									// param a has not changed!!!
									if(get_estimation_for_t_prev(currentEmissionStrategy)==2){
										// HALVE
										t_prev=(t_cut+t_prev)/2;			// new strategy... 23/05/2014
									}else if(get_estimation_for_t_prev(currentEmissionStrategy)==1){
										// RESTART
										t_prev=t_cut;			// new strategy... 23/05/2014
									}
									evaluateT_PREV.put(Long.toString(getManager().getCurrentTime()), Long.toString(t_prev));
								}
								// =============================

								evaluateT_HALVE.put(Long.toString(getManager().getCurrentTime()), Long.toString(t_cut));


								// SEND CUT MESSAGE!
								//evaluateCUT.put(Long.toString(getManager().getCurrentTime()), Long.toString(currAssimilation));
								evaluateCUT.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
								sendCut();							

								// ===============================================================================
								// FIXME  DECOMMENT WHAT FOLLOWS IF YOU WANT TO RECALC THE a COEFF. AFTER EACH CUT
								// hasToRefresh_a=true;
								// ===============================================================================

								hasBreak=true;
							}
							//}
						}
						if(!hasBreak){
							N_prev=informationCount;		
							//a_prev=a;
							//t_prev=t_now;
							setScheduledNextCheck();
						}
					}	
				}else{
					setScheduledNextCheck();
				}
			}
		}
	}

	
	public void minSquaresEstimation(){
		
		// NEW ESTIMATION OF PARAM a - min squares!!!
		if(t_now>=t_RTT){

			double time=Math.pow((t_now-t_RTT),2);
			
			minSquares_y=minSquares_y+informationCount*time;
			minSquares_x=minSquares_x+time*time;			

		
			//if(t_now>=(t_RTT+estimatedRTT/2)){
			if(t_now>=(t_RTT+estimatedRTT*param_RTT)){

				a=minSquares_y/minSquares_x;
				transientPhase=false;
				hasToWait=false;
				evaluate_a.put(Long.toString(getManager().getCurrentTime()), Double.toString(a));
			}else{
				hasToWait=true;
			}
		}else{
			hasToWait=true;
		}
	}
	
	public void setScheduledNextCheck(){
		isScheduledNextCheck=true;
		wait=0;
	}
	
	public void linearEstimation(){

		/**  LINEAR  */				

		long currAssimilation=getCircularBufferAssimilationSum();		

		if(informationCount>=totalAssimilationThreshold && !isScheduledNextCheck){
			// SEND STOP MESSAGE!
			evaluateSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
			sendStop();

			wait2RTTAfterSTOP=true;		
			//hasBreak=true;
		}else{

			if (informationCount>=assimilationThreshold && t_now>t_cut && !isScheduledNextCheck){

				boolean hasBreak=false;

				if(t_now>(t_cut+timeForEmission)){
					a=currAssimilation/timeForEmission;
				}else{
					a=(informationCount-N_cut)/(t_now-t_cut);
				}			

				t_new=t_now+estimatedRTT+timeStop;

				if((informationCount+beta*a*(t_new-t_now))>=totalAssimilationThreshold){

					// SEND STOP MESSAGE!
					evaluateSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
					sendStop();

					wait2RTTAfterSTOP=true;		
					hasBreak=true;

				}else{
					if(t_prev>t_cut){
						double N_exp=N_prev+beta*a_prev*(t_now-t_prev);

						if(N_exp>informationCount){

							t_cut=t_now+timeCut+estimatedRTT;		// At this time you have to store the assimilations to N_cut
							isScheduled_N_cut=true;

							// SEND CUT MESSAGE!
							evaluateCUT.put(Long.toString(getManager().getCurrentTime()), Long.toString(currAssimilation));
							sendCut();

							hasBreak=true;
						}
					}
				}

				if(!hasBreak){
					N_prev=informationCount;
					a_prev=a;
					t_prev=t_now;

					isScheduledNextCheck=true;
					wait=0;

				}				
			}
		}
	}



	public void emitsWithoutReceptorsForDEBUG(){

		float theta=0;
		float fi=0;			

		int undisposedCarriers=getInnerDomain().undisposedCarrierList.size();


		for(int i=undisposedCarriers-1;i>=0;i--){		


			if(getDomain()==null){
				System.out.println();
				System.out.println("*** TCP_NODE_RX: emitsWithoutReceptors() -> DOMINIOP DEL NODO = NULL!");
				System.out.println();
			}

			CarrierObj carrier=getInnerDomain().undisposedCarrierList.get(i);			
			carrier.removeFromActualDomain();
			//carrier.setDomain(getDomain());  


			float ffi=0;
			float ttheta=0;

			fi=(float) (2*Math.PI*(ffi)); //fi
			theta=(float) ((Math.acos((2*ttheta)-1))); //theta

			double radius=getRadius();	
			radius=radius+2*carrier.getRadius();
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));		


			float[] nodeCenter=getCenter();			
			float x=coords[0]+nodeCenter[0];
			float y=coords[1]+nodeCenter[1];
			float z=coords[2]+nodeCenter[2];		



			//if (getDomain().isFreeSpace(carrier)){			
			getDomain().attachNanoObj(carrier);
			carrier.disposeOnMedium();
			//getInnerDomain().undisposedCarrierList.remove(carrier);	

			//}else{			
			//	System.out.println(">>> Delayed Upload: "+getProcessingTime());
			//}	

			carrier.setCenter(x, y, z);
			carrier.setVelocityComponent(carrier.getVelocityModule(), fi, theta);	
	
		}
		
		/*
		for (int i=0;i<undisposedCarriers;i++){				
			

		}*/
		getInnerDomain().undisposedCarrierList.clear();		
	}
	
	
	public void standardBehavior_ORIGINAL_LINEAR() {


		if(isFirstStart){
			isFirstStart=false;
			sendStart();			
		}

		// ========================================
		// WAITS FOR THE RTT AFTER THE START SIGNAL
		// ========================================
		if(!isStillEmitting){
			if(isWaitingForRTT && timerRTT>=0){

				if(timerRTT<waitingTimeRTT-1){

					timerRTT++;
				}else{
					if(!(indexPattern<emissionPattern.length())){

						// Assumes that the node has finished to transmit the control signal					
						timerRTT=-1;
						//waitingTimeRTT=(waitingTimeRTT)*2;


						//burstEmission=burstEmission*2;
						burstEmission=burstEmission+1000;


						sendStart();	// Sets the internal state to TRANSMIT, so on the next timestep will start the emission!

					}else{					
						// Waits for the completion of the previous emission...
						timerRTT++;
					}				
				}	

			}else if (informationCount>0){

				// ====================================
				// MONITOR THE ASSIMILATIONS FOR THE RTT AFTER "START" SIGNAL
				// ====================================


				//FIXME HAVE TO OBSERVE FOR EACH TIMESTEP - EVALUATES THE BUFFER (WHICH OBSERVE THE PREVIOUS Ts)

				/**if (timerRTT<waitingWindow){
				//if (timerRTT<(estimatedRTT+windowSize)){
				//if(waitingWindow<windowSize-1){
					//if (waitingWindow==0) System.out.println(" >>>  (RX) WINDOW STARTS: "+getManager().getCurrentTime());
					//waitingWindow++;
					timerRTT++;

				}else{*/

				long currAssimilation=getCircularBufferAssimilationSum();

				if (informationCount>totalAssimilationThreshold){
					//if (informationCount>1500 && timerRTT>=(estimatedRTT+windowSize)){
					// SEND STOP MESSAGE!
					evaluateSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
					sendStop();

					//}else if (currAssimilation>assimilationThreshold && timerRTT>=(estimatedRTT+windowSize)){
				}else if (currAssimilation>assimilationThreshold){


					/** NEW IMPLEMENTATION */
					long now=getManager().getCurrentTime();
					long Tp=0;
					if(estimatedRTT>0 && timeFirstAssimilation>0 && (now-timeFirstAssimilation>timeForEmission)){
						Tp=windowSize;
					}else{
						Tp=(now-timeFirstAssimilation);
					}
					int timeStop=patternSTOP.length()-1;
					long k=currAssimilation*(estimatedRTT+timeStop*timeForEmission)/Tp;

					if (informationCount+k>totalAssimilationThreshold){
						// SEND STOP MESSAGE!
						evaluateSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(informationCount));
						sendStop();
					}else{

						/** END NEW IMPLEMENTATION */


						// SEND CUT MESSAGE!
						evaluateCUT.put(Long.toString(getManager().getCurrentTime()), Long.toString(currAssimilation));
						sendCut();
					}

				}/**
					else{
						//indexPattern=0;
						//countdownForEmission=0;						


						//waitingWindow=0;
						//timerRTT++;
						timerRTT=estimatedRTT;


						//if (timerRTT<(estimatedRTT+windowSize)){
						//	timerRTT++;
						//}else{
						//	waitingWindow=0;
						//}
					}*/

			}
		}
		/**}*/


		try {
			emitsWithoutReceptors();
			//emitsWithoutReceptorsForDEBUG();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void emitsWithoutReceptors(){
		
		float theta=0;
		float fi=0;			
		
		int undisposedCarriers=getInnerDomain().undisposedCarrierList.size();
		
		for(int i=undisposedCarriers-1;i>=0;i--){	
			
			if(getDomain()==null){
				System.out.println();
				System.out.println("*** TCP_NODE_RX: emitsWithoutReceptors() -> DOMINIOP DEL NODO = NULL!");
				System.out.println();
			}
		
			CarrierObj carrier=getInnerDomain().undisposedCarrierList.get(i);			
			carrier.removeFromActualDomain();
			//carrier.setDomain(getDomain()); 

			fi=(float) (2*Math.PI*(rand.nextFloat())); //fi
			theta=(float) ((Math.acos((2*rand.nextFloat())-1))); //theta

			double radius=getRadius();	
			radius=radius+2*carrier.getRadius();
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));	

			float[] nodeCenter=getCenter();			
			float x=coords[0]+nodeCenter[0];
			float y=coords[1]+nodeCenter[1];
			float z=coords[2]+nodeCenter[2];	

			//if (getDomain().isFreeSpace(carrier)){			
			getDomain().attachNanoObj(carrier);
			carrier.disposeOnMedium();
			//getInnerDomain().undisposedCarrierList.remove(carrier);	

			//}else{			
			//	System.out.println(">>> Delayed Upload: "+getProcessingTime());
			//}	

			carrier.setCenter(x, y, z);
			carrier.setVelocityComponent(carrier.getVelocityModule(), fi, theta);	
		}		
		/*
		for (int i=0;i<undisposedCarriers;i++){	
		}*/
		getInnerDomain().undisposedCarrierList.clear();		
	}
	
	
	// Verifies the reception of the ligand near receptors
	public void checkIncomingCarrier(CarrierObj c){		

		// Check if the receiver node is able to assimilate carriers
		// WARNING! If the rejectingReceiver is TRUE, ALL incoming carriers will be rejected!
		if(rejectingReceiver){
			missed++; 	// uses the lambda_m parameter to monitor the total amount of collisions!
			
			getDomain().getMobilityModel().relocateObjs(c,this);
			
		}
		// Check if the surface receptors are used for the assimilation
		// WARNING! If the absorbent is TRUE, no check on the type of carrier assimilated, it will absorb everything!
		else if(!absorbentReceiver){

			boolean catched=false;
			float x,  y,  z,  r;

			float[] coords=null;	

			// FIXME Relocates the carrierObj on the surface of the nanoObj
			float dt=(float) getCollisionTime(c);
			float[] velocity=c.getVelocityComponent();
			// Carrier Coords on the Node External Surface
			x=c.getCenter()[0]-velocity[0]*dt;
			y=c.getCenter()[1]-velocity[1]*dt;
			z=c.getCenter()[2]-velocity[2]*dt;	
			r=(float) c.getRadius();
			// END FIXME


			if (c.getDomain()==getInnerDomain()){		
				// Internal Domain!			
				for (int i=0;i<receptorList.size() && !catched;i++){				
					if((receptorList.get(i).isFree() && !(receptorList.get(i).canDownload()))){
						coords=receptorList.get(i).getAbsoluteCoords();
						float dim=receptorList.get(i).radius;							
						float dx = (coords[0]-x), dy = (coords[1]-y), dz= (coords[2]-z);
						float distance =(float) Math.sqrt(dx*dx+dy*dy+dz*dz);	

						if (distance<=((dim+r)) && (receptorList.get(i).checkCarrierCompatibility(c))){			
							catched=true;						
							receptorList.get(i).configureForUpload(c);	
							//System.out.println("NodeObj.checkIncomingCarrier() - CARRIER DOWNLOAD!");
						}
					}
				}

				if(!catched){	
					getInnerDomain().getCollisionStrategy().relocateInnerObjs(c,getInnerDomain());					
				}

			} else {		
				// External Domain!
				/*
			for (int i=0;i<receptorList.size() && !catched;i++){
				if((receptorList.get(i).isFree() && receptorList.get(i).canDownload())){					
					coords=receptorList.get(i).getAbsoluteCoords();
					float dim=receptorList.get(i).radius;								
					float dx = (coords[0]-x), dy = (coords[1]-y), dz= (coords[2]-z);
					float distance =(float) Math.sqrt(dx*dx+dy*dy+dz*dz);									
					if (distance<((dim+r)) && (receptorList.get(i).checkCarrierCompatibility(c))){							
						catched=true;
						receptorList.get(i).configureForDownload(c);											
					}
				}
			}

			if(!catched){											
				getDomain().getMobilityModel().relocateObjs(c,this);
			}*/


				// External Domain!
				// ===  NEW FOR SISTEMA A CODA  ===
				boolean isMissed=true;
				for (int i=0;i<receptorList.size() && !catched;i++){
					//if((receptorList.get(i).isFree() && receptorList.get(i).canDownload())){					
					coords=receptorList.get(i).getAbsoluteCoords();
					float dim=receptorList.get(i).radius;								
					float dx = (coords[0]-x), dy = (coords[1]-y), dz= (coords[2]-z);
					float distance =(float) Math.sqrt(dx*dx+dy*dy+dz*dz);									
					if (distance<((dim+r)) && (receptorList.get(i).checkCarrierCompatibility(c))){
						isMissed=false;
						if(receptorList.get(i).isFree()){
							catched=true;
							receptorList.get(i).configureForDownload(c, receptorRandomTime);
							served++;
						}else{
							busy++;
						}
					} 
					//}
				}

				if(!catched){											
					getDomain().getMobilityModel().relocateObjs(c,this);
				}
				if(isMissed){
					if(receptorList.get(0).checkCarrierCompatibility(c)){
						missed++;
					}
				}
				// =======================

			} 
		}else{

			// ABSORBENT RECEIVER!!! NO SURFACE RECEPTORS ARE USED!!!
			downloadInfoFrom(c);
		}
	}
	
	public void downloadInfoFrom(CarrierObj c){		
		informationCount++;			
		c.destroy();
	}	
}