package simulator;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.LigandObserver;
import MainLibrary.NoMotion;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.nIdle;
import MainLibrary.nTransmit;


public class TCPLike_Node_RX_OnlyCUT extends NodeObj{
	
	//private long firstContact=-1;	
	
	public int carrierCreated=0;	
	private long[] releaseTimeLog=new long[10];
	private int[] releaseCarrierLog=new int[10];
	private int indexReleaseLog=0;
	//private int indexExportedLog=0;
	private ArrayList<float[]> releasePositions=new ArrayList<float[]>();
	
	//private boolean hasExported=false;
	private int timeForEmission=0;
	private int indexPattern=0;
	private String emissionPattern="";
	
	private String patternSTART="";
	private String patternSTOP="";
	private String patternCUT="";
	
	private int countdownForEmission=0;
	
	
	/**  === Estimation of Round Trip Time ===  */
	private int waitingTimeRTT=50000;	// 200000  timesteps. It couldn't be smaller than the message duration 
	private int timerRTT=-1;			// timesteps
	private int estimatedRTT=-1;		// timesteps
	//private int startForRTT=-1;		// timesteps
	private boolean isWaitingForRTT=true;
	private boolean isStillEmitting=false;
	private boolean hasJustStopped=false;

	
	private int waitingWindow=0;	
	long timestep=0;
	int windowSize=100;
	int pos=0;
	private int currentAssimilation=0;		// for the circular buffer
	//int lastCount=0;
	private int[] circularBufferAssimilations= new int[windowSize]; 
	private int assimilationThreshold=1;
	private int totalAssimilationThreshold=1000;
	
	
	private boolean isFirstStart=true;

	
	private HashMap<String,String> evaluateRTT=new HashMap<String,String>();
	private HashMap<String,String> evaluateCUT=new HashMap<String,String>();
	private HashMap<String,String> evaluateSTOP=new HashMap<String,String>();
	
	private HashMap<String,String> sendSTART=new HashMap<String,String>();
	private HashMap<String,String> sendCUT=new HashMap<String,String>();
	private HashMap<String,String> sendSTOP=new HashMap<String,String>();
	

	private int counter=0;
	
	public TCPLike_Node_RX_OnlyCUT(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);
				
		// You can set the internal state here!
		setState(new nIdle(this));
		getState().setExecutionTime(1000);	
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
		setColor(Color.white);	
		
		informationCount=0;
		
		// RTT
		isWaitingForRTT=true;
		timerRTT=-1;
		
		/*
		setState(new nTransmit(this));
		getState().setExecutionTime(0);
		*/
		
		try {	
			if(domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-globalAssimilationThreshold") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-patternSTART") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-patternSTOP") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-patternCUT") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-assimilationThreshold") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-timeForAssimilation") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-timeForEmission")){
				timeForEmission=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-timeForEmission").intValue();
				//emissionPattern=Integer.toString((domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-emissionPattern").intValue()));	
				windowSize=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-timeForAssimilation").intValue();
				assimilationThreshold=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-assimilationThreshold").intValue();
				//countdownForEmission=timeForEmission;  // emits after this time
				emissionPattern=Integer.toString((domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-emissionPattern").intValue()));
				patternSTART=Integer.toString(domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-patternSTART").intValue());	
				patternSTOP=Integer.toString(domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-patternSTOP").intValue());	
				patternCUT=Integer.toString(domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-patternCUT").intValue());	
				totalAssimilationThreshold=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-globalAssimilationThreshold").intValue();	
				circularBufferAssimilations= new int[windowSize]; 
				waitingTimeRTT=(domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-timeout").intValue());	
				
				emissionPattern=patternSTART;
				
				
				countdownForEmission=0;		// emitis immediately!!
				
			}else{
				throw new Exception();
			}
		} catch (Exception e) {
			System.out.println();
			System.out.println(">>> WARNING! The transmitter node has not been configured for the EMISSION PATTERN!");
			System.out.println();
			System.out.println("    The simulation ends now");
			System.out.println();
			e.printStackTrace();
			System.exit(0);
		}
	}
	
	@Override
	public void idle() {
		processInfo();
	}
	
	@Override
	public void decode() {
		setState(new nIdle(this));
	}
	
	@Override
	public void encode() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(10);	
	}

	@Override
	public void searchNeighbor() {
		
	}

	public void sendStart(){
		setState(new nTransmit(this));
		getState().setExecutionTime(0);
		System.out.println("======   START (RX)  ======");
		
		isStillEmitting=true;
		indexPattern=0;
		countdownForEmission=0;		
		emissionPattern=patternSTART;
		hasJustStopped=false;
		
		sendSTART.put(Long.toString(getManager().getCurrentTime()), Long.toString(burstEmission));
	}
	
	public void sendStop(){
		setState(new nTransmit(this));
		getState().setExecutionTime(0);
		System.out.println("======   STOP (RX)  ======");
		
		timerRTT=0;
			
		if(hasJustStopped){
			//waitingTimeRTT=waitingTimeRTT*2;
			burstEmission=burstEmission*2;
		}
		
		isStillEmitting=true;
		indexPattern=0;
		countdownForEmission=0;		
		emissionPattern=patternSTOP;
		//waitingWindow=0;
		waitingWindow=estimatedRTT+windowSize;
		
		hasJustStopped=true;
		
		sendSTOP.put(Long.toString(getManager().getCurrentTime()), Long.toString(burstEmission));
	}
	
	public void sendCut(){
		setState(new nTransmit(this));
		getState().setExecutionTime(0);
		System.out.println("======   CUT (RX)   ======");
		
		timerRTT=0;
		
		isStillEmitting=true;
		indexPattern=0;
		countdownForEmission=0;		
		emissionPattern=patternCUT;
		waitingWindow=0;
		waitingWindow=estimatedRTT+windowSize;
		
		sendCUT.put(Long.toString(getManager().getCurrentTime()), Long.toString(burstEmission));
	}
	
	@Override
	public void transmit() {		
		
		/*
		if(informationCount>0 && emissionPattern!="10") {
			emissionPattern="10";
			indexPattern=0;
			countdownForEmission=timeForEmission;
		}*/
		
		if (indexPattern<emissionPattern.length()){
			countdownForEmission--;
			if(countdownForEmission<=0){

				int created=0;
				if(emissionPattern.charAt(indexPattern)=='1'){
					
					created=createCarriers(burstEmission);		
					System.out.println(">>> RX Releasing CARRIERS!!! (Control Node) "+created);
					
					if(indexPattern==0 && isWaitingForRTT && timerRTT<0){
						// The pattern has to start with a 1, otherwise it NEVER monitor the RTT!
						timerRTT=1;
					}
					
				}
				
				if(releaseTimeLog.length<=indexReleaseLog){
					enlargeReleaseTimeLog();
				}
				releaseTimeLog[indexReleaseLog]=getManager().getCurrentTime();
				releaseCarrierLog[indexReleaseLog]=created;

				float[] center=getCenter();
				float[] domainCenter=getDomain().getAbsCenter();					

				float[] currentRelativePosition={center[0]-domainCenter[0],center[1]-domainCenter[1],center[2]-domainCenter[2]};
				releasePositions.add(currentRelativePosition);
				indexReleaseLog++;

				indexPattern++;
				countdownForEmission=timeForEmission;
			}
		}else{
			countdownForEmission--;
			if(countdownForEmission<=0){
				isStillEmitting=false;
				setState(new nIdle(this));
				getState().setExecutionTime(10);
				countdownForEmission=timeForEmission;
			}
			
		}		
	}
	
	
	
	public void enlargeReleaseTimeLog(){
		long[] temp=releaseTimeLog;
		int[] tempCarrier=releaseCarrierLog;
		int size=temp.length;
		size=size+size/2;
		
		releaseTimeLog=new long[size];
		releaseCarrierLog=new int[size];
		
		for(int i=0;i<temp.length;i++){
			releaseTimeLog[i]=temp[i];
			releaseCarrierLog[i]=tempCarrier[i];
		}		
	}
	

	public int createCarriers(int n){
		int createdNow=0;
		try {
			
			while (n>0){				
				TCPLike_Carrier_B cc=new TCPLike_Carrier_B(getInnerDomain(), this, getCenter()[0], getCenter()[1], getCenter()[2],4);			
				cc.setMotionStrategy(BrownianMotion.getInstance());
				n--;
				createdNow++;			
			}
			carrierCreated=carrierCreated+createdNow;
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return createdNow;
	}
		
	@Override
	public Map<Object, Object> specializedExport() {

		Map<Object, Object> m=new HashMap<Object,Object>();
		
		boolean hasToExport=false;
		long time=getManager().getCurrentTime();
		if(time%10000==0){
			hasToExport=true;
		}
		
		if(hasToExport || sendSTART.size()>0 || sendCUT.size()>0 || sendSTOP.size()>0 || evaluateCUT.size()>0 || evaluateSTOP.size()>0 || evaluateRTT.size()>0){
			m.put("Type", "TCPLike-RX-Node");

			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		

			m.put("ID", "TCPLike-RX-Node-"+ID);	
			m.put("X", center[0]-domainCenter[0]);
			m.put("Y", center[1]-domainCenter[1]);
			m.put("Z", center[2]-domainCenter[2]);
			
			
			if(hasToExport){
				m.put("CRON_TimeStep", time+"");	
				m.put("CRON_Value", informationCount+"");	
				m.put("CRON_Action", "RX assimilation");		
			}
			
			
			
			
			int i=0;
			
			Iterator it = sendSTART.entrySet().iterator();
		    while (it.hasNext()) {
		        Map.Entry pairs = (Map.Entry)it.next();		        
		        
		        m.put("START_TimeStep_"+i, ""+pairs.getKey()+"");	
				m.put("START_Value_"+i, pairs.getValue());	
				m.put("START_Action_"+i, "send START");
				i++;
				
				//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("START_TOTAL", i);
		    sendSTART.clear();
		    
		    i=0;
			
			Iterator itCUT = sendCUT.entrySet().iterator();
		    while (itCUT.hasNext()) {
		        Map.Entry pairs = (Map.Entry)itCUT.next();		        
		        
		        m.put("CUT_TimeStep_"+i, ""+pairs.getKey()+"");	
				m.put("CUT_Value_"+i, pairs.getValue());	
				m.put("CUT_Action_"+i, "send CUT");
				i++;
				
				//itCUT.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("CUT_TOTAL", i);
		    sendCUT.clear();
		    
		    i=0;
			
			Iterator itSTOP = sendSTOP.entrySet().iterator();
		    while (itSTOP.hasNext()) {
		        Map.Entry pairs = (Map.Entry)itSTOP.next();		        
		        
		        m.put("STOP_TimeStep_"+i, ""+pairs.getKey()+"");	
				m.put("STOP_Value_"+i, pairs.getValue());	
				m.put("STOP_Action_"+i, "send STOP");
				i++;
				
				//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("STOP_TOTAL", i);
		    sendSTOP.clear();
			

		   	    
		    i=0;

		    Iterator itCUT2 = evaluateCUT.entrySet().iterator();
		    while (itCUT2.hasNext()) {
		    	Map.Entry pairs = (Map.Entry)itCUT2.next();		        

		    	m.put("CUT_Assimilations_TimeStep_"+i, ""+pairs.getKey()+"");	
		    	m.put("CUT_Assimilations_Value_"+i, pairs.getValue());	
		    	m.put("CUT_Assimilations_Action_"+i, "current assimilations (CUT)");
		    	i++;

		    	//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("CUT_Assimilations_TOTAL", i);
		    evaluateCUT.clear();
		    
		    
		    i=0;

		    Iterator itSTOP2 = evaluateSTOP.entrySet().iterator();
		    while (itSTOP2.hasNext()) {
		    	Map.Entry pairs = (Map.Entry)itSTOP2.next();		        

		    	m.put("STOP_Assimilations_TimeStep_"+i, ""+pairs.getKey()+"");	
		    	m.put("STOP_Assimilations_Value_"+i, pairs.getValue());	
		    	m.put("STOP_Assimilations_Action_"+i, "global assimilations (STOP)");
		    	i++;

		    	//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("STOP_Assimilations_TOTAL", i);
		    evaluateSTOP.clear();
		    
		    
		    i=0;

		    Iterator itRTT = evaluateRTT.entrySet().iterator();
		    while (itRTT.hasNext()) {
		    	Map.Entry pairs = (Map.Entry)itRTT.next();		        

		    	m.put("RTT_TimeStep_"+i, ""+pairs.getKey()+"");	
		    	m.put("RTT_Value_"+i, pairs.getValue());	
		    	m.put("RTT_Action_"+i, "RTT");
		    	i++;

		    	//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("RTT_TOTAL", i);
		    evaluateRTT.clear();
		    
			
			m.putAll(getState().export());
		}
		return m;
	}	
	
	
	@Override
	public void createSingleReceptor(float[] cdCenter, float radius, short receptorRadius, int receptorType) {
		float[] angles=convertSphericalAngles(cdCenter[0],cdCenter[1],cdCenter[2], radius);
		float theta=angles[1];
		float fi=angles[0];

		
		TCPLike_ReceptorB r=new TCPLike_ReceptorB(this, true, cdCenter, theta, fi, receptorRadius, receptorType, this.eventHandler);
		receptorList.add(r);
		receptorSetup(r); 

		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		*/		
	}
	
	public void createReceptor(int n){
		Receptor r = null;
		
		float theta=0;
		float fi=0;				
		
		for (int i=0;i<n;i++){	
			
			// Creates the receptors for UPLOAD
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];		
			
			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new TCPLike_ReceptorB(this, false, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			//receptorSetup(r); 
			r.attachObs(r.getMyObserver());
			
			
			// Creates the receptors for DOWNLOAD
			
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];				
				
			coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new TCPLike_ReceptorA(this, true, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			//receptorSetup(r); 			
			r.attachObs(r.getMyObserver());
		}
	}	
	
	
	
	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	
	}
	
	@Override
	public void setMotionStrategy() {		
		//this.motionStrategy=BrownianMotion.getInstance();
		this.motionStrategy=NoMotion.getInstance();
	}
	
	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}
		
	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	@Override
	public void processInfo() {
		/*
		if(getCollisionCount()>0 && firstContact<0){
			firstContact=getManager().getCurrentTime();				
		}*/	
		
		
	}
	
	public String assignName(){		
		return "TCPLike-RX Node";		
	}
	
	public void exec(){
		
		try {
			timestep=getManager().getCurrentTime();
			//int bufferSize=circularBufferAssimilations.length;
			pos=(int) (timestep%windowSize);
			
			currentAssimilation=0;
			circularBufferAssimilations[pos]=0;
			
			
			super.exec();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
	public void downloadInfoFrom(CarrierObj c, Receptor r){		
		informationCount++;	
		
		currentAssimilation++;
		circularBufferAssimilations[pos]=currentAssimilation;
		
		
		if(isWaitingForRTT && c instanceof TCPLike_Carrier_A){
			
			estimatedRTT=timerRTT;			
			isWaitingForRTT=false;
			//windowSize=estimatedRTT;	// FIXME Have to be equal to the duration of the symbol emitted by the other node?
			circularBufferAssimilations= new int[windowSize]; 
			timerRTT=0;
			waitingWindow=estimatedRTT+windowSize;
			evaluateRTT.put(Long.toString(getManager().getCurrentTime()), Long.toString(estimatedRTT));
		}
		c.destroy();
	}
	
	/*
	public void downloadCarrier(CarrierObj c) {	
		
		if(c instanceof CalciumCarrier){
			assimilatedIons++;
		}else if(c instanceof CalciumCarrierBG){
			assimilatedBG++;
		}
		
		c.kill(); 		
	}*/
	
	public long getCircularBufferAssimilationSum(){
		long sum=0;
		int size=circularBufferAssimilations.length;
		
		for(int i=0;i<size;i++){
			sum=sum+circularBufferAssimilations[i];
		}
		
		return sum;
	}
	
	@Override	
	public void standardBehavior() {
				
		// ====================================
		// WAITS FOR THE RTT AFTER START SIGNAL
		// ====================================
		if(!isStillEmitting){
			counter++;
			if (informationCount>0 && counter>=windowSize){

				// ====================================
				// MONITOR THE ASSIMILATIONS FOR THE RTT AFTER START SIGNAL
				// ====================================

				long currAssimilation=getCircularBufferAssimilationSum();

				if (currAssimilation>assimilationThreshold){
					// SEND CUT MESSAGE!
					evaluateCUT.put(Long.toString(getManager().getCurrentTime()), Long.toString(currAssimilation));
					sendCut();	
					counter=0;
				}
			}
		}
		try {
			emitsWithoutReceptors();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	
	public void emitsWithoutReceptors(){
		
		float theta=0;
		float fi=0;			
		
		int undisposedCarriers=getInnerDomain().undisposedCarrierList.size();
		
		for(int i=undisposedCarriers-1;i>=0;i--){					
			
			if(getDomain()==null){
				System.out.println();
				System.out.println("*** TCP_NODE_RX: emitsWithoutReceptors() -> DOMINIOP DEL NODO = NULL!");
				System.out.println();
			}
		
			CarrierObj carrier=getInnerDomain().undisposedCarrierList.get(i);			
			carrier.removeFromActualDomain();
			//carrier.setDomain(getDomain());  


			fi=(float) (2*Math.PI*(rand.nextFloat())); 				//fi
			theta=(float) ((Math.acos((2*rand.nextFloat())-1))); 	//theta

			double radius=getRadius();	
			radius=radius+2*carrier.getRadius();
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));		


			float[] nodeCenter=getCenter();			
			float x=coords[0]+nodeCenter[0];
			float y=coords[1]+nodeCenter[1];
			float z=coords[2]+nodeCenter[2];	


			//if (getDomain().isFreeSpace(carrier)){			
			getDomain().attachNanoObj(carrier);
			carrier.disposeOnMedium();
			//getInnerDomain().undisposedCarrierList.remove(carrier);	

			//}else{			
			//	System.out.println(">>> Delayed Upload: "+getProcessingTime());
			//}	

			carrier.setCenter(x, y, z);
			carrier.setVelocityComponent(carrier.getVelocityModule(), fi, theta);	
	
		}
		getInnerDomain().undisposedCarrierList.clear();		
	}
}