
package simulator;


import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
//import scala.Char;
import MainLibrary.AbstractDomain;
import MainLibrary.LigandObserver;
import MainLibrary.NoMotion;


public class TCPLike_Node_TX_BER extends TCPLike_Node_TX{
	
	private HashMap<String,String> assimilatedPattern=new HashMap<String,String>();
	private HashMap<String,String> sendACK=new HashMap<String,String>();
	
	
		
	public TCPLike_Node_TX_BER(AbstractDomain domain, float x, float y, float z, int nodeNumberType, long emissionThreshold, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,emissionThreshold, observer);		
	}	
		
	@Override	
	public void standardBehavior() {

		checkAssimilationPattern();		
		
		emitsWithoutReceptors();	
	}
		
	public void checkAssimilationPattern(){
	
		
		if (!sync && !waitAfterReceivedCommand){

			evaluateBuffer();

		}else if(waitingWindow<windowSize-1){		
			
			waitingWindow++;				
		}else{
			
			waitAfterReceivedCommand=false;
			
			evaluateBuffer();
		}
	}
	
	public void evaluateBuffer(){

		char currentAssimilation=getAssimilatedBit();
		
		
		if(sync){
			long now=getManager().getCurrentTime();
						
			System.out.println("[TX NODE] assimilated: "+currentAssimilation+"   @ "+now);
			assimilatedPattern.put(Long.toString(now), String.valueOf(currentAssimilation));
			

		}else{ 			

			// FIXME New implementation for sync
			int kk=20;
			if(!syncTemp && currentAssimilation=='1'){

				syncTemp=true;
				waitCondition=true;
				waitTemp=0;

				ii=0;
				Tk=windowSize/kk;		// windowSize is Ts!
				Tstar=0;
				Tsync=Tstar;
				dati_max=getCircularBufferAssimilationSum();
				tk_done=false;
			}

			if(waitCondition){

				if(waitTemp<Tk){
					waitTemp++;
				}else{

					waitTemp=0;

					if(ii<kk && !tk_done){
						long currentWindowAssimilation=getCircularBufferAssimilationSum();
						if (currentWindowAssimilation>=dati_max){
							Tsync=Tstar+ii*Tk;
							dati_max=currentWindowAssimilation;
							ii++;
						}else{
							tk_done=true;
						}
					}else{
						tk_done=true;
					}
					//} // end while

					if(tk_done){
						sync=true;
						syncTemp=false;
						waitCondition=false;
						waitTemp=0;
						waitingWindow=Tsync;

						//evaluatingMessage="1";
						currentAssimilation='1';
						long now=getManager().getCurrentTime();
						
						//System.out.println("synk ok - "+evaluatingMessage+"   @ "+now);						
						
						System.out.println("[TX NODE] assimilated: "+currentAssimilation+"   @ "+now);
						assimilatedPattern.put(Long.toString(now), String.valueOf(currentAssimilation));
						
						sendACK();
						
					}
				}
			}
		}	


		if(!isEmitting){
			waitingWindow=0;	// WAITS FOR THE NEXT BIT (THE BUFFER WILL BE EMPTY)
		}

	}	

	public void sendACK(){		
		createCarriers(burstEmission);	
		System.out.println("[TX NODE] send ACK");
		sendACK.put(Long.toString(getManager().getCurrentTime()), String.valueOf(burstEmission));
	}

	@Override
	public void transmit() {		
		//createCarriers(burstEmission);	
	}
	
		
	@Override
	public Map<Object, Object> specializedExport() {

		Map<Object, Object> m=new HashMap<Object,Object>();
		
		
		if(assimilatedPattern.size()>0 || sendACK.size()>0){
			m.put("Type", assignName());

			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		

			m.put("ID", assignName()+"-"+ID);	
			m.put("X", center[0]-domainCenter[0]);
			m.put("Y", center[1]-domainCenter[1]);
			m.put("Z", center[2]-domainCenter[2]);
			
			int i=0;
			
			Iterator it = assimilatedPattern.entrySet().iterator();
		    while (it.hasNext()) {
		        Map.Entry pairs = (Map.Entry)it.next();		        
		        
		        m.put("ASSIMILATION_TimeStep_"+i, ""+pairs.getKey()+"");	
				m.put("ASSIMILATION_Value_"+i, pairs.getValue());	
				m.put("ASSIMILATION_Action_"+i, "receive BIT");
				i++;
				
				//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("ASSIMILATION_TOTAL", i);
		    assimilatedPattern.clear();
		    
		    
		    i=0;
			
			it = sendACK.entrySet().iterator();
		    while (it.hasNext()) {
		        Map.Entry pairs = (Map.Entry)it.next();		        
		        
		        m.put("sendACK_TimeStep_"+i, ""+pairs.getKey()+"");	
				m.put("sendACK_Value_"+i, pairs.getValue());	
				m.put("sendACK_Action_"+i, "send ACK");
				i++;
				
				//it.remove(); // avoids a ConcurrentModificationException
		    }
		    m.put("sendACK_TOTAL", i);
		    sendACK.clear();
		    
		    
			m.putAll(getState().export());
		}
		return m;
	}	
	

	@Override
	public void setMotionStrategy() {		
		this.motionStrategy=NoMotion.getInstance();
	}	

	public String assignName(){		
		return "TCPLike_BER-TX Node";		
	}	
}