package simulator;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;

import engine.Engine3D;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.LigandObserver;
import MainLibrary.NanoObj;
import MainLibrary.NoMotion;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.UniversalReceptor;
import MainLibrary.nIdle;
import MainLibrary.nTransmit;


public class TCPLike_TEST_RX extends NodeObj{
	

	
	public int carrierCreated=0;	
	
	long timestep=0;
	int windowSize=100;
	int pos=0;
	
	private long lastCount=0;

	
	
	public TCPLike_TEST_RX(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);
				
		// You can set the internal state here!
		setState(new nIdle(this));
		getState().setExecutionTime(10);	
		
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
		setColor(Color.white);	
		
		informationCount=0;
		
		
		
	}
	
	@Override
	public void idle() {
		processInfo();
	}
	
	@Override
	public void decode() {
		setState(new nIdle(this));
	}
	
	@Override
	public void encode() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(10);	
	}

	@Override
	public void searchNeighbor() {
		
	}

	public void checkIncomingCarrier(CarrierObj c){
		
		if(getManager().is3DEnable){
		float[] carrierCenter=c.getCenter();
		
		drawSmallPoint(carrierCenter);
		}
		super.checkIncomingCarrier(c);
	}
	
	@Override
	public void transmit() {		
		
	
	}
	
	
	
	@Override
	public void createSingleReceptor(float[] cdCenter, float radius, short receptorRadius, int receptorType) {
		float[] angles=convertSphericalAngles(cdCenter[0],cdCenter[1],cdCenter[2], radius);
		float theta=angles[1];
		float fi=angles[0];

		
		TCPLike_ReceptorB r=new TCPLike_ReceptorB(this, true, cdCenter, theta, fi, receptorRadius, receptorType, this.eventHandler);
		receptorList.add(r);
		receptorSetup(r); 

		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		*/		
	}
	
	public void createReceptor(int n){
		Receptor r = null;
		
		float theta=0;
		float fi=0;				
		
		for (int i=0;i<n;i++){	
			
			// Creates the receptors for UPLOAD
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];		
			
			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new TCPLike_ReceptorB(this, false, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			//receptorSetup(r); 
			r.attachObs(r.getMyObserver());
			
			
			// Creates the receptors for DOWNLOAD
			
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];				
				
			coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			
			r=new TCPLike_ReceptorA(this, true, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			//receptorSetup(r); 			
			r.attachObs(r.getMyObserver());
		}
	}	
	
	
	
	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	
	}
	
	@Override
	public void setMotionStrategy() {		
		//this.motionStrategy=BrownianMotion.getInstance();
		this.motionStrategy=NoMotion.getInstance();
	}
	
	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}
		
	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	@Override
	public void processInfo() {
		
		
	}
	
	public String assignName(){		
		return "TCPLike-TEST-RX Node";		
	}
	
	
	
	
	public void downloadInfoFrom(CarrierObj c, Receptor r){				
		informationCount++;	
		
		
		//c.kill();		//FIXME  Provo a SOSTITUIRLO con c.destroy(), perch la carrier al momento appartiene al dominio interno!
		c.destroy();
		/*
		System.out.println();
		System.out.println("RX DOWNLOAD!  "+informationCount);
		System.out.println();
		*/
	}
	
	
	

	
	public void emitsWithoutReceptors(){
		
		float theta=0;
		float fi=0;			
		
		int undisposedCarriers=getInnerDomain().undisposedCarrierList.size();

		
		for(int i=undisposedCarriers-1;i>=0;i--){		
			
			
			if(getDomain()==null){
				System.out.println();
				System.out.println("*** TCP_NODE_RX: emitsWithoutReceptors() -> DOMINIOP DEL NODO = NULL!");
				System.out.println();
			}
		
			CarrierObj carrier=getInnerDomain().undisposedCarrierList.get(i);			
			carrier.removeFromActualDomain();
			//carrier.setDomain(getDomain());  


			fi=(float) (2*Math.PI*(rand.nextFloat())); //fi
			theta=(float) ((Math.acos((2*rand.nextFloat())-1))); //theta

			double radius=getRadius();	
			radius=radius+2*carrier.getRadius();
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));		


			float[] nodeCenter=getCenter();			
			float x=coords[0]+nodeCenter[0];
			float y=coords[1]+nodeCenter[1];
			float z=coords[2]+nodeCenter[2];		



			//if (getDomain().isFreeSpace(carrier)){			
			getDomain().attachNanoObj(carrier);
			carrier.disposeOnMedium();
			//getInnerDomain().undisposedCarrierList.remove(carrier);	

			//}else{			
			//	System.out.println(">>> Delayed Upload: "+getProcessingTime());
			//}	

			carrier.setCenter(x, y, z);
			carrier.setVelocityComponent(carrier.getVelocityModule(), fi, theta);	
	
		}
		
		/*
		for (int i=0;i<undisposedCarriers;i++){				
			

		}*/
		getInnerDomain().undisposedCarrierList.clear();		
	}
	
	@Override
	public Map<Object, Object> specializedExport() {

	
		
		Map<Object, Object> m=new HashMap<Object,Object>();
		if(lastCount<informationCount ){
			m.put("Typev", "TCPLike-TEST-RX");

			lastCount=informationCount;
			
			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		

			m.put("IDv", "TCPLike-TEST-RX-"+ID);	
			/*
			m.put("Xv", center[0]-domainCenter[0]);
			m.put("Yv", center[1]-domainCenter[1]);
			m.put("Zv", center[2]-domainCenter[2]);
			*/
			m.put("Xv", center[0]);
			m.put("Yv", center[1]);
			m.put("Zv", center[2]);
			m.put("TimeStepv", ""+getManager().getCurrentTime()+"");	
			m.put("Assimilationsv", informationCount);	
			m.put("Actionv", "assimilation");		
			
			m.putAll(getState().export());
		}
		return m;
	}	
	
	// Draw the receptor in the 3D window
		public void drawSmallPoint(float[] coords){
			SimpleVector v0=new SimpleVector(coords);
			Object3D obj3D=null;
			obj3D = Primitives.getBox(10, 1);	
			//obj3D = Primitives.getBox(getRadius(), 1);	
			obj3D.setCulling(false);		
			obj3D.setAdditionalColor(Color.green);		
			obj3D.setEnvmapped(Object3D.ENVMAP_ENABLED);		
			obj3D.build();
			obj3D.translate(v0);
			Engine3D engine3d=(Engine3D)getManager().engine;
			engine3d.world.addObject(obj3D);
			obj3D.compileAndStrip();		
		} 	
}