package simulator;


import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import MainLibrary.AbstractDomain;
import MainLibrary.CarrierObj;
import MainLibrary.LigandObserver;
import MainLibrary.NoMotion;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.nEncode;
import MainLibrary.nIdle;
import MainLibrary.nTransmit;



public class TCPLike_TEST_TX extends NodeObj{

	public int carrierCreated=0;		
	
	// === TCP parameters ===
	//private int currentEmission=0;

	
	private boolean hasExported=false;
		
	long timestep=0;
	int windowSize=100;
	int pos=0;
	
	int lastCount=0;

	long releaseTime=0;

	int globalEmission=0;
	
	private int timeForEmission=0;
	int countdownForEmission=0;
	
	long currentWindowAssimilation=0;
	

	public int currentEmission=0;
	
	public TCPLike_TEST_TX(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x,y,z,nodeNumberType,observer);

		// You can set the internal state here!		
		
		setState(new nTransmit(this));
		getState().setExecutionTime(1);
		
		currentEmission=burstEmission;
		
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
		
		
		// Load last configuration values...
		try{
			if(domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-timeForEmission") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-patternSTART") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-patternSTOP") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-patternCUT") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-assimilationThreshold") && domain.getManager().configWrapper.nodeConfig.containsKey(nodeNumberType+"-timeForAssimilation")){
				timeForEmission=domain.getManager().configWrapper.nodeConfig.get(nodeNumberType+"-timeForEmission").intValue();						
				//currentEmission=burstEmission;
				
			}else{
				throw new Exception();
			}
		} catch (Exception e) {
			System.out.println();
			System.out.println(">>> WARNING! The NODES have not well configured! (Check config.xml)");
			System.out.println();
			System.out.println("    The simulation ends now");
			System.out.println();
			e.printStackTrace();
			System.exit(0);
		}				
	}	
	
	@Override
	public boolean mandatoryCreation(){
		return true;
	}

	@Override	
	public void standardBehavior() {
		if(informationCount>lastCount){
			//System.out.println("TX NODE ASSIMILATIONS: "+informationCount);
			lastCount=informationCount;
		}		
		
		emitsWithoutReceptors();		
	}
	
	
	
	
	public void emitsWithoutReceptors(){
			
		float theta=0;
		float fi=0;			
		
		int undisposedCarriers=getInnerDomain().undisposedCarrierList.size();

		for(int i=undisposedCarriers-1;i>=0;i--){		
		
		//for (int i=0;i<undisposedCarriers;i++){				
			
			if(getDomain()==null){
				System.out.println();
				System.out.println("*** TCP_NODE_TX: emitsWithoutReceptors() -> DOMINIOP DEL NODO = NULL!");
				System.out.println();
			}
		
			CarrierObj carrier=getInnerDomain().undisposedCarrierList.get(i);	
			carrier.removeFromActualDomain();
			//carrier.setDomain(getDomain());  
			
			
			fi=(float) (2*Math.PI*(rand.nextFloat())); //fi
			theta=(float) ((Math.acos((2*rand.nextFloat())-1))); //theta

			double radius=getRadius();	
			radius=radius+2*carrier.getRadius();
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));		
			

			float[] nodeCenter=getCenter();			
			float x=coords[0]+nodeCenter[0];
			float y=coords[1]+nodeCenter[1];
			float z=coords[2]+nodeCenter[2];		

		
			getDomain().attachNanoObj(carrier);
			carrier.disposeOnMedium();
			
			carrier.setCenter(x, y, z);
			carrier.setVelocityComponent(carrier.getVelocityModule(), fi, theta);	
		}
		getInnerDomain().undisposedCarrierList.clear();		
	}
	
	@Override
	public void idle() {	
		
		/* OLD BEFORE PATTERN - SEE standardBehaviour()!!!
		if (carrierCreated==0 && burstEmission>0){
			activateCell();
		} */
		getState().setExecutionTime(10);	
		
	}

	@Override
	public void decode() {
		setState(new nIdle(this));
	}

	@Override
	public void encode() {
		/* OLD BEFORE PATTERN - SEE standardBehaviour()!!!
		createCarriers(burstEmission);
		setState(new nTransmit(this));
		*/
		setState(new nIdle(this));
	}

	@Override
	public void searchNeighbor() {

	}

	/**/
	@Override
	public void transmit() {		
		//BURST
		countdownForEmission--;
		if(countdownForEmission<=0){		
			System.out.println(">>> TX Releasing CARRIERS!!!  "+burstEmission);			
			createCarriers(burstEmission);				
			//countdownForEmission=timeForEmission;
			
			releaseTime=getManager().getCurrentTime();
			
			setState(new nIdle(this));
			getState().setExecutionTime(10);	
			
		}			
	}
	
	
	/*
	@Override
	public void transmit() {		
		//CONTINUOUS
		countdownForEmission--;
		if(countdownForEmission<=0){		
			//System.out.println(">>> TX Releasing CARRIERS!!!  "+currentEmission);			
			createCarriers(currentEmission);				
			countdownForEmission=timeForEmission;
			currentEmission++;
			releaseTime=getManager().getCurrentTime();
			
			
			//setState(new nIdle(this));
			//getState().setExecutionTime(10);	
			
		}			
	}*/

	public int createCarriers(int n){
		int createdNow=0;
		while (n>0){				
			TCPLike_Carrier_A cc=new TCPLike_Carrier_A(getInnerDomain(), this, getCenter()[0], getCenter()[1], getCenter()[2],3);			
			cc.setMotionStrategy(BrownianMotion.getInstance());
			n--;
			createdNow++;			
		}
		carrierCreated=carrierCreated+createdNow;	
		
		return createdNow;
	}
		
	
	public void checkIncomingCarrier(CarrierObj c){	

		boolean catched=false;
		float x,  y,  z,  r;
		r=(float) c.getRadius();
		float[] coords=null;	

		if (!(c instanceof TCPLike_Carrier_A)){


			// Verifies the reception of the ligand near receptors

			// FIXME DEBUG AGGIUNTO PER RIPOSIZIONARE LA CARRIER SUL GUSCIO DEL NODO
			float dt=(float) getCollisionTime(c);
			float[] velocity=c.getVelocityComponent();
			// Carrier Coords on the Node External Surface
			x=c.getCenter()[0]-velocity[0]*dt;
			y=c.getCenter()[1]-velocity[1]*dt;
			z=c.getCenter()[2]-velocity[2]*dt;	
			// FINE FIXME


			// External Domain!
			for (int i=0;i<receptorList.size() && !catched;i++){
				if((receptorList.get(i).isFree() && receptorList.get(i).canDownload())){					
					coords=receptorList.get(i).getAbsoluteCoords();
					float dim=receptorList.get(i).radius;								
					float dx = (coords[0]-x), dy = (coords[1]-y), dz= (coords[2]-z);
					float distance =(float) Math.sqrt(dx*dx+dy*dy+dz*dz);									
					if (distance<((dim+r)) && (receptorList.get(i).checkCarrierCompatibility(c))){							
						catched=true;
						receptorList.get(i).configureForDownload(c);							
					}
				}
			}

			if(!catched){	
				//STO QUI! NON HA TROVATO RECETTORI COSI CHIAMA IL RELOCATE, NON  colpa del check Collision (forse...)
				//FIXME DA RIMUOVERE IL SET COLOR!
				//c.setColor(Color.red);				
				getDomain().getMobilityModel().relocateObjs(c,this);
			}

		}else{			

			getDomain().getMobilityModel().relocateObjs(c,this);
		}
	}



	
	public Map<Object, Object> specializedExport() {

		Map<Object, Object> m=new HashMap<Object,Object>();
		if(carrierCreated>0 && !hasExported){
			m.put("Typev", "TCPLike-TEST-TX");

			float[] center=new float[3];
			float[] domainCenter=getDomain().getAbsCenter();

			center=getCenter();		
			
			if(Float.isNaN(center[0]) || Float.isNaN(center[1]) || Float.isNaN(center[2])){
				System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> NaN NaN NaN");
			}
			
			m.put("IDv", "TCPLike-TEST-TX-"+ID);	
			/*
			m.put("Xv", center[0]-domainCenter[0]);
			m.put("Yv", center[1]-domainCenter[1]);
			m.put("Zv", center[2]-domainCenter[2]);
			*/
			m.put("Xv", center[0]);
			m.put("Yv", center[1]);
			m.put("Zv", center[2]);
			
			m.put("TimeStepv", ""+releaseTime+"");	
			m.put("Emissionv", carrierCreated);
			m.put("Actionv", "emission");
			m.putAll(getState().export());

			hasExported=true;
		}
		return m;
	}

	
	@Override
	public void createSingleReceptor(float[] cdCenter, float radius, short receptorRadius, int receptorType) {
		float[] angles=convertSphericalAngles(cdCenter[0],cdCenter[1],cdCenter[2], radius);
		float theta=angles[1];
		float fi=angles[0];

		
		Receptor r=new TCPLike_ReceptorB(this, true, cdCenter, theta, fi, receptorRadius, receptorType, this.eventHandler);
		receptorList.add(r);
		receptorSetup(r); 

		/** Draw each Receptor */
		/*
		for(int i=0; i<receptorList.size(); i++){
			((CD40UniversalReceptor) receptorList.get(i)).drawSmallPoint(receptorList.get(i).getParent());
		}
		*/		
	}
	
	public void createReceptor(int n){		

		Receptor r = null;		
		float theta=0;
		float fi=0;			

		for (int i=0;i<n;i++){
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];		

			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			

			r=new TCPLike_ReceptorB(this, true, coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType, this.eventHandler);
			receptorList.add(r);
			//receptorSetup(r);
			r.attachObs(r.getMyObserver());
		}
	}	

	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	}

	@Override
	public void setMotionStrategy() {		
		this.motionStrategy=NoMotion.getInstance();
	}

	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}

	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	public void activateCell() {
		//releaseTime=getManager().getCurrentTime();
		setState(new nEncode(this));
		getState().setExecutionTime(0);	
	}

	public String assignName(){		
		return "TCPLike-TEST-TX Node";		
	}

	@Override
	public void processInfo() {
		// TODO Auto-generated method stub
		
	}
	

	
	public void downloadInfoFrom(CarrierObj c, Receptor r){		
		informationCount++;	
		
		
		//c.kill();		//FIXME  Provo a SOSTITUIRLO con c.destroy(), perch la carrier al momento appartiene al dominio interno!
		c.destroy();
		/*
		System.out.println();
		System.out.println("TX DOWNLOAD!  "+informationCount);
		System.out.println();
		*/
	}
	
	public void saveAllAndExit(){
		getManager().hasStopped=true;
		//getManager().specializedExport();
	}
}