package simulator;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import MainLibrary.AbstractDomain;
import MainLibrary.LigandObserver;
import MainLibrary.NodeObj;
import MainLibrary.Receptor;
import MainLibrary.InnerSphericalStrategy;
import MainLibrary.nEncode;
import MainLibrary.nIdle;
import MainLibrary.nTransmit;
import MainLibrary.NoMotion;


public class THCell extends NodeObj{

	public boolean hasTransmit=false;
	private long firstContact=-1;


	public THCell(AbstractDomain domain, float x, float y, float z, int nodeNumberType, List<LigandObserver> observer){
		super(domain,x, y,z,nodeNumberType,observer);

		// You can set the internal state here!
		setState(new nIdle(this));
		getState().setExecutionTime(0);	
		getInnerDomain().setCollisionStrategy(new InnerSphericalStrategy(0));
	}

	@Override
	public void idle() {
		processInfo();
	}

	@Override
	public void decode() {
		setState(new nIdle(this));		
	}

	@Override
	public void encode() {
		createCarriers(burstEmission);
		setState(new nTransmit(this));
	}

	@Override
	public void searchNeighbor() {

	}

	@Override
	public void transmit() {
		setState(new nIdle(this)); 
		getState().setExecutionTime(0);		
	}

	public void createCarriers(int n){
		while (n>0){
			new CD40LCarrier(getInnerDomain(), this, getCenter()[0], getCenter()[1], getCenter()[2],0);
			n--;
		}	
	}

	@Override
	public Map<Object, Object> specializedExport() {
		Map<Object, Object> m=new HashMap<Object,Object>();
		m.put("Type", "Node");

		m.put("firstContact", firstContact);
		m.putAll(getState().export());
		return m;
	}

	public void createReceptor(int n){
		Receptor r;		
		Random rand=new Random();

		float theta=0;
		float fi=0;

		for (int i=0;i<n;i++){		
			theta=getDomain().getMobilityModel().getRandomAngles()[0];
			fi=getDomain().getMobilityModel().getRandomAngles()[1];

			double radius=getRadius();			
			float[] coords=new float[3];
			coords[0]=(float) (radius*Math.sin((double)theta)*Math.cos((double)fi));
			coords[1]=(float) (radius*Math.sin((double)theta)*Math.sin((double)fi));
			coords[2]=(float) (radius*Math.cos((double)theta));			

			r=new CD154Receptor(this, rand.nextBoolean(), coords, theta, fi, getManager().configWrapper.receptorStateConfig.get(receptorNumberType+"-radius").shortValue(),receptorNumberType,this.eventHandler);
			receptorList.add(r);
			receptorSetup(r); 		
		}
	}

	@Override
	public void receptorSetup(Receptor r) {
		List<LigandObserver> obsList=getObsList();
		int index=rand.nextInt(obsList.size());
		r.attachObs(obsList.get(index));
	}

	@Override
	public void setMotionStrategy() {
		this.motionStrategy=NoMotion.getInstance();
	}

	public float getAbsDistancePlusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()+getRadius()+receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()+getRadius();
		}
		return distance;
	}

	public float getAbsDistanceMinusRadius(){
		float distance=0;
		if(receptorList.size()>0){
			distance=getAbsDistance()-getRadius()-receptorList.get(0).radius/2;
		}else{
			distance=getAbsDistance()-getRadius();
		}
		return distance;
	}

	@Override
	public void processInfo() {

		if(getCollisionCount()>0 && firstContact<0){
			firstContact=getManager().getCurrentTime();
		}

		if (!hasTransmit){
			int size=informationCount;
			if (size>0){
				if (getInnerDomain().undisposedCarrierList.size()<=0){
					setState(new nEncode(this));
					getState().setExecutionTime(2);						
					hasTransmit=true;					
				}
			}
		}
	}

	public String assignName(){		
		return "THCell";		
	}
	
	@Override
	public void createSingleReceptor(float[] center, float radius, short receptorRadius,int receptorType) {
		// TODO Auto-generated method stub
		
	}
}