package simulator;

import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import MainLibrary.AbstractDomain;
import MainLibrary.AbstractEngine3D;
import MainLibrary.LivingObjObserver;
import MainLibrary.Manager;
import MainLibrary.MobilityModel;
import MainLibrary.NanoObj;
import MainLibrary.OutputStrategy;

public class iManager extends Manager {
	
	
	private final int maxThreads = Runtime.getRuntime().availableProcessors();
	private ExecutorService executorThreads = Executors.newFixedThreadPool(maxThreads);	 
	
	
	private Map<Object, Object> exportString=new HashMap<Object, Object>();
	private List<NanoObj> allObjList=new ArrayList<NanoObj>();
	private int allDomainsObj=0;
	
	public String DBAddress="localhost";  
	public int DBPort=6132;
	//public int plateletLevel=0;
	private int printTimeOnScreen=configWrapper.printTimeOnScreen;
	//public MySQLBackup_BloodVessel backupExecutor=MySQLBackup_BloodVessel.getInstance();	
	//private boolean hasInitializedBackup=false;
	private double calcTime= 1;
	
	public CalciumSphericalDomain spherical=null;
	
	public iManager(MobilityModel m, OutputStrategy o, AbstractEngine3D engine,  boolean useOctree) {
		super(m, o, engine, useOctree);	
		//backupExecutor.setManager(this);
		calcTime= (Math.pow(10,configWrapper.u)*get_dt());		
	}	
	
	@Override
	public void nextSimulationStep(){		
		super.nextSimulationStep();	
	}
		
	
	
	// NEW 16/10/2015 - MULTITHREADING!! 
	@Override
	public void specializedExport() {	
		
		allObjList.clear();
		exportString.put("CurrentTime", getCurrentTime());
		exportString.put("CPUTime", (stop-start));
		/*
		if(getCurrentTime()>=100)
			backupExecutor.collectData();
		*/
		
		//long start=System.nanoTime();
		
		//---------------------------
		// STEP 1
		//---------------------------	
		
		try {
			for(OutputStrategy o : outputObjList){
				o.export(exportString);
			}
		} catch (Exception e1) {
			System.out.println("-3.1- Error on iManager.specializedExport()");
			e1.printStackTrace();
		}
		/*
		long stop=System.nanoTime();
		System.out.print(""+(stop-start));
		
		
		start=System.nanoTime();
		*/
		//---------------------------
		// STEP 2
		//---------------------------	
		
		try {
			ArrayList<LivingObjObserver> list=getAllLivingObjObserver();
			int size=list.size();		
			for (int k=0;k<size;k++){
				allObjList.addAll(list.get(k).getLivingNodes());
				//allObjList.addAll(list.get(k).getLivingCarriers()); 	// FIXME ONLY FOR DEBUG!!
			}
		} catch (Exception e1) {
			System.out.println("-3.2- Error on iManager.specializedExport()");
			e1.printStackTrace();
		}	
		/*
		stop=System.nanoTime();
		System.out.print(" "+(stop-start));
		
		
		start=System.nanoTime();
		*/
		//---------------------------
		// STEP 3
		//---------------------------	
		// ORIGINALE NON OTTIMIZZATO
		/*
		try {
			for(NanoObj n : allObjList){
				for(OutputStrategy o : outputObjList){
					o.export(n.export());
				}
			}
		} catch (Exception e1) {
			System.out.println("-3.3- Error on iManager.specializedExport()");
			e1.printStackTrace();
		}*/	
				

		//---------------------------		
		// VERSIONE OTTIMIZZATA DEL BLOCCO SOPRA
		//---------------------------
		int div=maxThreads;
		int size=allObjList.size();		
		CountDownLatch latch=new CountDownLatch(1);

		if(size>div){
			int intPart=size/div;	

			if (intPart>0) {
				latch=new CountDownLatch(div);
			}				

			for(int i=0;i<div-1;i++){				
				executorThreads.execute(new NanoObjExporterThread(allObjList.subList(intPart*i, intPart*(i+1)),outputObjList,latch));		
			}

			executorThreads.execute(new NanoObjExporterThread(allObjList.subList(intPart*(div-1), size),outputObjList,latch));	

		}else if(size>0){			
			executorThreads.execute(new NanoObjExporterThread(allObjList,outputObjList,latch));		
		}else{
			// The ObjList is empty!
			latch.countDown();
		}

		try {
			latch.await();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}			

		//---------------------------
		
		/*
		stop=System.nanoTime();
		System.out.print(" "+(stop-start));
		
		
		
		
		
		start=System.nanoTime();	
		*/	
		//---------------------------
		// STEP 4
		//---------------------------	
		
		/*
		// OTTIMIZZARE!!!
		for(AbstractDomain d : exportDomain){
			for(OutputStrategy o : outputObjList){
				try {					
					o.export(d.export());
				} catch (Exception e) {
					System.out.println("-3.4- Error on iManager.specializedExport() - exportDomain NULL - size() "+exportDomain.size());
					e.printStackTrace();
				}	
			}
		}
		*/		
		
		//---------------------------		
		// VERSIONE OTTIMIZZATA DEL BLOCCO SOPRA
		//---------------------------
		//int div=maxThreads;
		size=exportDomain.size();		
		latch=new CountDownLatch(1);

		if(size>0){

			if(size>div){
				int intPart=size/div;	

				if (intPart>0) {
					latch=new CountDownLatch(div);
				}				

				for(int i=0;i<div-1;i++){				
					executorThreads.execute(new DomainExporterThread(exportDomain.subList(intPart*i, intPart*(i+1)),outputObjList,latch));		
				}

				executorThreads.execute(new DomainExporterThread(exportDomain.subList(intPart*(div-1), size),outputObjList,latch));	

			//}else if(size>0){	
			}else {			
				executorThreads.execute(new DomainExporterThread(exportDomain,outputObjList,latch));		
			}

			try {
				latch.await();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}			
		}
		//---------------------------
				
		/*
		stop=System.nanoTime();
		System.out.print(" "+(stop-start));
		
		
		
		start=System.nanoTime();
		*/
		//---------------------------
		// STEP 5  -  SAVE ALL!
		//---------------------------	
		
		try {
			if((outputObjList.contains(OutputDBMySQL.getInstance()) && getCurrentTime()%(configWrapper.exportTimeLength*OutputDBMySQL.getInstance().getStoringFactor())==0) || hasStopped){
				OutputDBMySQL.getInstance().store();				
			}
		} catch (Exception e) {
			System.out.println("-3.5- Error on iManager.specializedExport() - OutputDBMySQL.getInstance().store() ");			
			e.printStackTrace();
		}
		
		/*
		stop=System.nanoTime();
		System.out.println(" "+(stop-start));
		*/
	}	
	
	
	
	// ORIGINAL - WITHOUT MULTITHREADING!
	public void specializedExport_ORIGINAL() {		
		allObjList.clear();
		exportString.put("CurrentTime", getCurrentTime());
		exportString.put("CPUTime", (stop-start));
		/*
		if(getCurrentTime()>=100)
			backupExecutor.collectData();
		*/
		
		//long start=System.nanoTime();
		
		
		try {
			for(OutputStrategy o : outputObjList){
				o.export(exportString);
			}
		} catch (Exception e1) {
			System.out.println("-3.1- Error on iManager.specializedExport()");
			e1.printStackTrace();
		}
		/*
		long stop=System.nanoTime();
		System.out.print(""+(stop-start));
		*/
		
		//start=System.nanoTime();
		
		try {
			ArrayList<LivingObjObserver> list=getAllLivingObjObserver();
			int size=list.size();		
			for (int k=0;k<size;k++){
				allObjList.addAll(list.get(k).getLivingNodes());
			}
		} catch (Exception e1) {
			System.out.println("-3.2- Error on iManager.specializedExport()");
			e1.printStackTrace();
		}	
		/*
		stop=System.nanoTime();
		System.out.print(" "+(stop-start));
		*/
		
		//start=System.nanoTime();
		
		try {
			for(NanoObj n : allObjList){
				for(OutputStrategy o : outputObjList){
					o.export(n.export());
				}
			}
		} catch (Exception e1) {
			System.out.println("-3.3- Error on iManager.specializedExport()");
			e1.printStackTrace();
		}	
		/*
		stop=System.nanoTime();
		System.out.print(" "+(stop-start));
		*/
		
		//start=System.nanoTime();
		
		// OTTIMIZZARE!!!
		for(AbstractDomain d : exportDomain){
			for(OutputStrategy o : outputObjList){
				try {					
					o.export(d.export());
				} catch (Exception e) {
					System.out.println("-3.4- Error on iManager.specializedExport() - exportDomain NULL - size() "+exportDomain.size());
					e.printStackTrace();
				}	
			}
		}
		
		/*
		stop=System.nanoTime();
		System.out.print(" "+(stop-start));
		*/
		
		//start=System.nanoTime();
		
		try {
			if((outputObjList.contains(OutputDBMySQL.getInstance()) && getCurrentTime()%(configWrapper.exportTimeLength*OutputDBMySQL.getInstance().getStoringFactor())==0) || hasStopped){
				OutputDBMySQL.getInstance().store();				
			}
		} catch (Exception e) {
			System.out.println("-3.5- Error on iManager.specializedExport() - OutputDBMySQL.getInstance().store() ");			
			e.printStackTrace();
		}
		
		/*
		stop=System.nanoTime();
		System.out.println(" "+(stop-start));
		*/
	}	

	public String getTitleForCurrentStep(){
	
		long currentTime=getCurrentTime();
		 stop=System.currentTimeMillis();
		 totalObj=rootDomain.getActiveNanoObjListSize();

		 /*
		 for (int i=0;i<rootDomain.getAllChildDomains().size();i++){
			 totalObj=totalObj+rootDomain.getAllChildDomains().get(i).getActiveNanoObjListSize();
		 }
		 */
		 
		 /*
		 //FIXME Only for CALCIUM Scenario		 
		 if (spherical!=null) {
			 totalObj=totalObj+spherical.getActiveNanoObjListSize();			 
		 }*/
		 
	     String title="";
	     //elapsedTime= (Math.pow(10,configWrapper.u)*currentTime*get_dt());
	     elapsedTime= calcTime*currentTime;
	     int createdObj=LivingObjObserver.getObjCount();
	     title="[TimeStep: "+currentTime+"] - [ElapsedTime: "+elapsedTime+" s] - Living Obj: "+createdObj+" - (CPU Time: "+(stop-start)+" ms)";
	     
	     return title;	
	}
	
	public void printCurrentStep(){		
	     
		if (is3DEnable) {			
			String title=getTitleForCurrentStep();
			engine.title=title;	
		}			
			
	    if (currentTime%printTimeOnScreen==0) {  
	    	String title=getTitleForCurrentStep();
	    	System.out.println(title);	    	    
		    allDomainsObj=0;
		    
		    /*
		    // FIXME ONLY FOR DEBUG
		    long tot=getMobilityModel().moveTime+getMobilityModel().checkTime+getMobilityModel().avoidTime;
		    double ratio=tot/(stop-start);
		    System.out.println("m: "+getMobilityModel().moveTime+" - c: "+getMobilityModel().checkTime+" - a: "+getMobilityModel().avoidTime);	 
		    System.out.println("Ratio: "+ratio);	
		    System.out.println();	
		    
		    getMobilityModel().moveTime=0;
		    getMobilityModel().checkTime=0;
		    getMobilityModel().avoidTime=0;
		    */
	    }
	}	
	
	public void printObjAllDomains(AbstractDomain domain){
		
		List<AbstractDomain> childDomainList=domain.getAllChildDomains();
		for(int i=0;i<childDomainList.size();i++){
			printObjAllDomains(childDomainList.get(i));
		}
		System.out.println("[DOMAIN DISPOSED] "+domain.getAllDisposedObj().size()+"   [DOMAIN UNDISPOSED] "+domain.getAllUndisposedObj().size());	
		allDomainsObj=allDomainsObj+domain.getAllDisposedObj().size();
		allDomainsObj=allDomainsObj+domain.getAllUndisposedObj().size();
	}
}


//============================================
//EXPORTER PRIVATE CLASS - (MultiThread)
//============================================

class NanoObjExporterThread implements Runnable{

List<NanoObj> list=new ArrayList<NanoObj>();
List<OutputStrategy> olist=new ArrayList<OutputStrategy>();
CountDownLatch latch;

NanoObjExporterThread(List<NanoObj> list, List<OutputStrategy> olist, CountDownLatch latch){
	this.list=list;
	this.olist=olist;
	this.latch=latch;
}


@Override
public void run() {
	
	for(NanoObj n : list){
		for(OutputStrategy o : olist){
			try {					
				o.export(n.export());
			} catch (Exception e) {
				System.out.println("-3.4- Error on iManager.specializedExport() -> DomainExporterThread - exportDomain NULL - size() ");
				e.printStackTrace();
			}	
		}
	}
	
	this.latch.countDown();
}
}


// ============================================
//  EXPORTER PRIVATE CLASS - (MultiThread)
// ============================================

class DomainExporterThread implements Runnable{

	List<AbstractDomain> list=new ArrayList<AbstractDomain>();
	List<OutputStrategy> olist=new ArrayList<OutputStrategy>();
	CountDownLatch latch;
	
	DomainExporterThread(List<AbstractDomain> list, List<OutputStrategy> olist, CountDownLatch latch){
		this.list=list;
		this.olist=olist;
		this.latch=latch;
	}
	

	@Override
	public void run() {

		for(AbstractDomain d : list){
			for(OutputStrategy o : olist){
				try {					
					o.export(d.export());
				} catch (Exception e) {
					System.out.println("-3.4- Error on iManager.specializedExport() -> DomainExporterThread - exportDomain NULL - size() ");
					e.printStackTrace();
				}	
			}
		}
		
		this.latch.countDown();
	}
}
